@@ -1,5 +1,5 @@
 /*
-    SPDX-FileCopyrightText: 2004-2018 Thomas Baumgart <tbaumgart@kde.org>
+    SPDX-FileCopyrightText: 2004-2021 Thomas Baumgart <tbaumgart@kde.org>
     SPDX-FileCopyrightText: 2017 Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
@@ -7,6 +7,8 @@
 #ifndef KCURRENCYCALCULATOR_H
 #define KCURRENCYCALCULATOR_H
 
+#include "kmm_base_dialogs_export.h"
+
 // ----------------------------------------------------------------------------
 // QT Includes
 
@@ -24,6 +26,7 @@ class MyMoneyMoney;
 class MyMoneySplit;
 class MyMoneyTransaction;
 class MyMoneySecurity;
+class MultiCurrencyEdit;
 
 namespace Ui {
 class KCurrencyCalculator;
@@ -36,12 +39,14 @@ typedef qint64 signed64;
   */
 
 class KCurrencyCalculatorPrivate;
-class KCurrencyCalculator : public QDialog
+class KMM_BASE_DIALOGS_EXPORT KCurrencyCalculator : public QDialog
 {
     Q_OBJECT
     Q_DISABLE_COPY(KCurrencyCalculator)
 
 public:
+    explicit KCurrencyCalculator(QWidget* parent = nullptr);
+
     /**
       * @param from the @p from currency
       * @param to   the @p to currency
@@ -53,13 +58,14 @@ public:
       *
       * @note @p value must not be 0!
       */
-    explicit KCurrencyCalculator(const MyMoneySecurity& from,
-                                 const MyMoneySecurity& to,
-                                 const MyMoneyMoney& value,
-                                 const MyMoneyMoney& shares,
-                                 const QDate& date,
-                                 const signed64 resultFraction = 100,
-                                 QWidget *parent = nullptr);
+    KCurrencyCalculator(const MyMoneySecurity& from,
+                        const MyMoneySecurity& to,
+                        const MyMoneyMoney& value,
+                        const MyMoneyMoney& shares,
+                        const QDate& date,
+                        const signed64 resultFraction = 100,
+                        QWidget* parent = nullptr);
+
     ~KCurrencyCalculator();
 
     /**
@@ -84,6 +90,29 @@ public:
                                 MyMoneyMoney>& priceInfo,
                                 QWidget* parentWidget);
 
+    void setFromCurrency(const MyMoneySecurity& sec);
+    void setToCurrency(const MyMoneySecurity& sec);
+    void setFromAmount(const MyMoneyMoney& amount);
+    void setToAmount(const MyMoneyMoney& amount);
+    void setDate(const QDate& date);
+    void setResultFraction(signed64 fraction);
+
+    /**
+     * This method asks the user for exchange rate information
+     * according to the data found in the @a amountEdit widget.
+     * In case the two commodities in @a amountEdit are identical
+     * the method returns immediately.
+     */
+    static void updateConversion(MultiCurrencyEdit* amountEdit, const QDate date);
+
+    /**
+     * Passing @c true as argument allows to suppress sending out
+     * update signals from the MyMoneyFile object during price
+     * update. Sending out signals causes a problem when performed
+     * in a nested dialog (e.g. investment split fees)
+     */
+    void blockSignalsDuringUpdate(bool block);
+
 protected Q_SLOTS:
     void slotSetToAmount();
     void slotSetExchangeRate();
