@@ -12,7 +12,6 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
-#include <QTimer>
 #include <QPixmap>
 #include <QBitmap>
 #include <QList>
@@ -27,43 +26,30 @@
 // ----------------------------------------------------------------------------
 // KDE Includes
 
-#include <KTreeWidgetSearchLineWidget>
-#include <KMessageBox>
 #include <KLocalizedString>
+#include <KMessageBox>
+#include <KTreeWidgetSearchLine>
+#include <KTreeWidgetSearchLineWidget>
 
 // ----------------------------------------------------------------------------
 // Project Includes
 
 #include "ui_kcurrencyeditdlg.h"
 
-#include "mymoneyexception.h"
-#include "mymoneysecurity.h"
-#include "mymoneyfile.h"
-#include "mymoneyprice.h"
+#include "icons/icons.h"
 #include "kavailablecurrencydlg.h"
 #include "kcurrencyeditordlg.h"
 #include "kmymoneyutils.h"
-#include "icons/icons.h"
-#include "storageenums.h"
+#include "menuenums.h"
 #include "mymoneyenums.h"
+#include "mymoneyexception.h"
+#include "mymoneyfile.h"
+#include "mymoneyprice.h"
+#include "mymoneysecurity.h"
+#include "storageenums.h"
 
 using namespace Icons;
 
-// duplicated eMenu namespace from menuenums.h for consistency
-// there shouldn't be any clash, because we don't need menuenums.h here
-namespace eMenu {
-enum class Action {
-    // *************
-    // The currency menu
-    // *************
-    NewCurrency, RenameCurrency, DeleteCurrency,
-    SetBaseCurrency,
-};
-inline uint qHash(const Action key, uint seed) {
-    return ::qHash(static_cast<uint>(key), seed);
-}
-}
-
 // this delegate is needed to disable editing the currency id (column 1)
 // since QTreeWidgetItem has only one set of flags for the whole row
 // the column editable property couldn't be set in an easier way
@@ -128,8 +114,8 @@ public:
                 ++it;
             }
         } else if (mode == RemoveSelected) {
-            QList<QTreeWidgetItem*> currencyRows = ui->m_currencyList->selectedItems();
-            foreach(auto currencyRow, currencyRows) {
+            const QList<QTreeWidgetItem*> currencyRows = ui->m_currencyList->selectedItems();
+            for (const auto& currencyRow : currencyRows) {
                 MyMoneySecurity currency = currencyRow->data(0, Qt::UserRole).value<MyMoneySecurity>();
                 if (file->baseCurrency() != currency && !file->isReferenced(currency, skip))
                     KMyMoneyUtils::deleteSecurity(currency, q);
@@ -149,7 +135,7 @@ public:
                     MyMoneyFile::instance()->setBaseCurrency(cur);
                     ft.commit();
                 } catch (const MyMoneyException &e) {
-                    KMessageBox::sorry(q, i18n("Cannot set %1 as base currency: %2", cur.name(), QString::fromLatin1(e.what())), i18n("Set base currency"));
+                    KMessageBox::error(q, i18n("Cannot set %1 as base currency: %2", cur.name(), QString::fromLatin1(e.what())), i18n("Set base currency"));
                 }
             }
         }
@@ -170,11 +156,11 @@ public:
                     m_currentCurrency = currency;
                     ft.commit();
                 } catch (const MyMoneyException &e) {
-                    KMessageBox::sorry(q, i18n("Cannot update currency. %1", QString::fromLatin1(e.what())), i18n("Update currency"));
+                    KMessageBox::error(q, i18n("Cannot update currency. %1", QString::fromLatin1(e.what())), i18n("Update currency"));
                 }
             }
         } catch (const MyMoneyException &e) {
-            KMessageBox::sorry(q, i18n("Cannot update currency. %1", QString::fromLatin1(e.what())), i18n("Update currency"));
+            KMessageBox::error(q, i18n("Cannot update currency. %1", QString::fromLatin1(e.what())), i18n("Update currency"));
         }
     }
 
@@ -207,9 +193,9 @@ public:
 
                 } catch (const MyMoneyException &e) {
                     if (currency.id().isEmpty()) {
-                        KMessageBox::sorry(q_ptr, i18n("Cannot create new currency. %1", QString::fromLatin1(e.what())), i18n("New currency"));
+                        KMessageBox::error(q_ptr, i18n("Cannot create new currency. %1", QString::fromLatin1(e.what())), i18n("New currency"));
                     } else {
-                        KMessageBox::sorry(q_ptr, i18n("Cannot modify currency. %1", QString::fromLatin1(e.what())), i18n("Edit currency"));
+                        KMessageBox::error(q_ptr, i18n("Cannot modify currency. %1", QString::fromLatin1(e.what())), i18n("Edit currency"));
                     }
                 }
             } else {
@@ -261,10 +247,10 @@ KCurrencyEditDlg::KCurrencyEditDlg(QWidget *parent) :
     connect(d->ui->m_editCurrencyButton, &QAbstractButton::clicked, this, &KCurrencyEditDlg::slotEditCurrency);
     connect(d->ui->m_removeUnusedCurrencyButton, &QAbstractButton::clicked, this, &KCurrencyEditDlg::slotRemoveUnusedCurrency);
 
-    QTimer::singleShot(10, this, SLOT(timerDone()));
+    QMetaObject::invokeMethod(this, "finishCtor", Qt::QueuedConnection);
 }
 
-void KCurrencyEditDlg::timerDone()
+void KCurrencyEditDlg::finishCtor()
 {
     Q_D(KCurrencyEditDlg);
     slotLoadCurrencies();
@@ -294,7 +280,7 @@ KCurrencyEditDlg::~KCurrencyEditDlg()
 
 void KCurrencyEditDlg::slotLoadCurrencies()
 {
-    const QSet<QString> metalSymbols { "XAU", "XPD", "XPT", "XAG" };
+    const KMMStringSet metalSymbols{"XAU", "XPD", "XPT", "XAG"};
 
     Q_D(KCurrencyEditDlg);
 
@@ -307,13 +293,13 @@ void KCurrencyEditDlg::slotLoadCurrencies()
     disconnect(d->ui->m_currencyList, &QTreeWidget::itemChanged, this, static_cast<void (KCurrencyEditDlg::*)(QTreeWidgetItem *, int)>(&KCurrencyEditDlg::slotUpdateCurrency));
 
     QList<MyMoneySecurity> list = MyMoneyFile::instance()->currencyList();
-    QList<MyMoneySecurity>::ConstIterator it;
+    QList<MyMoneySecurity>::const_iterator it;
     QTreeWidgetItem *first = 0;
 
     // sort the currencies ...
-    // ... and make sure a few precious metals are at the ned
-    qSort(list.begin(), list.end(),
-          [=] (const MyMoneySecurity& c1, const MyMoneySecurity& c2)
+    // ... and make sure a few precious metals are at the end
+    std::sort(list.begin(), list.end(),
+              [=] (const MyMoneySecurity& c1, const MyMoneySecurity& c2)
     {
         const bool c1Metal = c1.tradingSymbol().startsWith(QLatin1Char('X')) && metalSymbols.contains(c1.tradingSymbol());
         const bool c2Metal = c2.tradingSymbol().startsWith(QLatin1Char('X')) && metalSymbols.contains(c2.tradingSymbol());
@@ -341,7 +327,7 @@ void KCurrencyEditDlg::slotLoadCurrencies()
 
     d->ui->m_currencyList->setSortingEnabled(false);
     d->ui->m_currencyList->clear();
-    for (it = list.constBegin(); it != list.constEnd(); ++it) {
+    for (it = list.cbegin(); it != list.cend(); ++it) {
         QTreeWidgetItem *p = new QTreeWidgetItem(d->ui->m_currencyList);
         p->setText(0, (*it).name());
         p->setData(0, Qt::UserRole, QVariant::fromValue(*it));
@@ -419,11 +405,10 @@ void KCurrencyEditDlg::slotSelectCurrency(const QString& id)
 
     while (*it) {
         if ((*it)->text(1) == id) {
-            d->ui->m_currencyList->blockSignals(true);
+            QSignalBlocker blocked(d->ui->m_currencyList);
             slotSelectCurrency(*it);
             d->ui->m_currencyList->setCurrentItem(*it);
             d->ui->m_currencyList->scrollToItem(*it);
-            d->ui->m_currencyList->blockSignals(false);
             break;
         }
         ++it;
@@ -465,7 +450,7 @@ void KCurrencyEditDlg::slotSelectCurrency(QTreeWidgetItem *item)
         d->ui->m_selectBaseCurrencyButton->setDisabled(rc1 || count != 1);
         d->ui->m_editCurrencyButton->setDisabled(count != 1);
         d->ui->m_removeCurrencyButton->setDisabled((rc1 || rc2) && count <= 1);
-//    emit selectObject(d->m_currentCurrency);
+        // Q_EMIT selectObject(d->m_currentCurrency);
     }
 }
 
@@ -505,7 +490,7 @@ void KCurrencyEditDlg::slotShowCurrencyMenu(const QPoint& p)
         const QVector<actionInfo> actionInfos {
             {eMenu::Action::NewCurrency,      &KCurrencyEditDlg::slotNewCurrency,     i18n("New currency"),            Icon::DocumentNew, true},
             {eMenu::Action::RenameCurrency,   &KCurrencyEditDlg::slotRenameCurrency,  i18n("Rename currency"),         Icon::EditRename,  cond1},
-            {eMenu::Action::DeleteCurrency,   &KCurrencyEditDlg::slotDeleteCurrency,  i18n("Delete currency"),         Icon::EditDelete,  cond2},
+            {eMenu::Action::DeleteCurrency,   &KCurrencyEditDlg::slotDeleteCurrency,  i18n("Delete currency"),         Icon::EditRemove,  cond2},
             {eMenu::Action::SetBaseCurrency,  &KCurrencyEditDlg::slotSetBaseCurrency, i18n("Select as base currency"), Icon::KMyMoney,    cond3},
         };
 
@@ -516,7 +501,7 @@ void KCurrencyEditDlg::slotShowCurrencyMenu(const QPoint& p)
             connect(a, &QAction::triggered, this, info.callback);
             LUTActions.append(a);
         }
-        menu->addSection(i18nc("Menu header","Currency options"));
+        menu->addSection(i18nc("@title:menu", "Currency options"));
         menu->addActions(LUTActions);
         menu->exec(QCursor::pos());
     }
@@ -526,7 +511,7 @@ void KCurrencyEditDlg::slotSelectBaseCurrency()
 {
     Q_D(KCurrencyEditDlg);
     if (!d->m_currentCurrency.id().isEmpty()) {
-        QTreeWidgetItem* p = d->ui->m_currencyList->currentItem();
+        QTreeWidgetItem* const p = d->ui->m_currencyList->currentItem();
         d->setBaseCurrency(d->m_currentCurrency);
         // in case the dataChanged() signal was not sent out (nested FileTransaction)
         // we update the list manually
@@ -552,8 +537,8 @@ void KCurrencyEditDlg::slotAddCurrency()
         auto file = MyMoneyFile::instance();
         QMap<MyMoneySecurity, MyMoneyPrice> ancientCurrencies = file->ancientCurrencies();
         MyMoneyFileTransaction ft;
-        QList<QTreeWidgetItem *> currencyRows = d->m_availableCurrencyDlg->selectedItems(); // get selected currencies from new dialog
-        foreach (auto currencyRow, currencyRows) {
+        const QList<QTreeWidgetItem*> currencyRows = d->m_availableCurrencyDlg->selectedItems(); // get selected currencies from new dialog
+        for (const auto& currencyRow : currencyRows) {
             MyMoneySecurity currency = currencyRow->data(0, Qt::UserRole).value<MyMoneySecurity>();
             file->addCurrency(currency);
             if (ancientCurrencies.value(currency, MyMoneyPrice()) != MyMoneyPrice()) { // if ancient currency is added...
@@ -576,7 +561,7 @@ void KCurrencyEditDlg::slotAddCurrency()
                             if (!ancientCurrency.id().isEmpty()) {
                                 file->addPrice(ancientCurrencies[ancientCurrency]);
                             }
-                        } catch (MyMoneyException& e) {
+                        } catch (MyMoneyException&) {
                             // nothing to do because ancient currency is not on file
                         }
                         break;
@@ -587,6 +572,7 @@ void KCurrencyEditDlg::slotAddCurrency()
         ft.commit();
         d->ui->m_removeUnusedCurrencyButton->setDisabled(file->currencyList().count() <= 1);
     }
+    d->m_searchWidget->searchLine()->updateSearch();
     delete d->m_availableCurrencyDlg;
 }
 
@@ -647,7 +633,7 @@ void KCurrencyEditDlg::slotDeleteCurrency()
             MyMoneyFile::instance()->removeCurrency(d->m_currentCurrency);
             ft.commit();
         } catch (const MyMoneyException &e) {
-            KMessageBox::sorry(this, i18n("Cannot delete currency %1. %2", d->m_currentCurrency.name(), QString::fromLatin1(e.what())), i18n("Delete currency"));
+            KMessageBox::error(this, i18n("Cannot delete currency %1. %2", d->m_currentCurrency.name(), QString::fromLatin1(e.what())), i18n("Delete currency"));
         }
     }
 }
@@ -662,7 +648,7 @@ void KCurrencyEditDlg::slotSetBaseCurrency()
                 MyMoneyFile::instance()->setBaseCurrency(d->m_currentCurrency);
                 ft.commit();
             } catch (const MyMoneyException &e) {
-                KMessageBox::sorry(this, i18n("Cannot set %1 as base currency: %2", d->m_currentCurrency.name(), QString::fromLatin1(e.what())), i18n("Set base currency"));
+                KMessageBox::error(this, i18n("Cannot set %1 as base currency: %2", d->m_currentCurrency.name(), QString::fromLatin1(e.what())), i18n("Set base currency"));
             }
         }
     }
