@@ -7,6 +7,8 @@
 #ifndef KMYMONEYPRICEDLG_H
 #define KMYMONEYPRICEDLG_H
 
+#include "kmm_extended_dialogs_export.h"
+
 // ----------------------------------------------------------------------------
 // QT Includes
 
@@ -18,11 +20,8 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
-class MyMoneyPrice;
-class QTreeWidgetItem;
-
 class KMyMoneyPriceDlgPrivate;
-class KMyMoneyPriceDlg : public QDialog
+class KMM_EXTENDED_DIALOGS_EXPORT KMyMoneyPriceDlg : public QDialog
 {
     Q_OBJECT
     Q_DISABLE_COPY(KMyMoneyPriceDlg)
@@ -31,15 +30,10 @@ public:
     explicit KMyMoneyPriceDlg(QWidget* parent);
     ~KMyMoneyPriceDlg();
 
-private:
-    QTreeWidgetItem* loadPriceItem(const MyMoneyPrice& basePrice);
-
 protected Q_SLOTS:
-    void slotSelectPrice();
     void slotNewPrice();
     void slotDeletePrice();
     void slotEditPrice();
-    void slotLoadWidgets();
     void slotOnlinePriceUpdate();
     void slotShowPriceMenu(const QPoint& p);
 
