@@ -73,6 +73,8 @@ KSettingsHome::KSettingsHome(QWidget* parent) :
     connect(d->ui->m_upButton, &QAbstractButton::clicked, this, &KSettingsHome::slotMoveUp);
     connect(d->ui->m_downButton, &QAbstractButton::clicked, this, &KSettingsHome::slotMoveDown);
 
+    connect(d->ui->kcfg_HideZeroBalanceAccountsHome, &QCheckBox::stateChanged, this, &KSettingsHome::hideZeroBalanceAccountsHomeChanged);
+
     // Don't show it to the user, we only need it to load and save the settings
     d->ui->kcfg_ItemList->hide();
 }
@@ -83,6 +85,14 @@ KSettingsHome::~KSettingsHome()
     delete d;
 }
 
+void KSettingsHome::setHideZeroBalanceAccountsHome(bool state)
+{
+    Q_D(KSettingsHome);
+    if (d->ui->kcfg_HideZeroBalanceAccountsHome->isChecked() != state) {
+        d->ui->kcfg_HideZeroBalanceAccountsHome->setChecked(state);
+    }
+}
+
 void KSettingsHome::slotLoadItems()
 {
     Q_D(KSettingsHome);
@@ -90,11 +100,11 @@ void KSettingsHome::slotLoadItems()
         return;
 
     QStringList list = KMyMoneySettings::listOfItems();
-    QStringList::ConstIterator it;
+    QStringList::const_iterator it;
     d->ui->m_homePageList->clear();
     QListWidgetItem *sel = 0;
 
-    for (it = list.constBegin(); it != list.constEnd(); ++it) {
+    for (it = list.cbegin(); it != list.cend(); ++it) {
         int idx = (*it).toInt();
         // skip over unknown item entries
         if (idx == 0)
