@@ -1,6 +1,7 @@
 /*
     SPDX-FileCopyrightText: 2000-2001 Michael Edwardes <mte@users.sourceforge.net>
     SPDX-FileCopyrightText: 2017, 2018 Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
+    SPDX-FileCopyrightText: 2021 Dawid Wróbel <me@dawidwrobel.com>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 #ifndef KMYMONEY_H
@@ -31,7 +32,6 @@
 #include "onlinejobtyped.h"
 #include "mymoneykeyvaluecontainer.h"
 #include "mymoneymoney.h"
-#include "selectedtransactions.h"
 #include "mymoneysplit.h"
 #include "mymoneytransaction.h"
 #include "mymoneyenums.h"
@@ -50,6 +50,7 @@ class MyMoneyTransaction;
 class WebConnect;
 class creditTransfer;
 class IMyMoneyOperationsFormat;
+class SelectedObjects;
 
 template <class T> class onlineJobTyped;
 typedef  void (*KMyMoneyAppCallback)(int, int, const QString &);
@@ -102,6 +103,10 @@ private Q_SLOTS:
       */
     void slotShowContextMenuForConsistencyCheck(const QPoint &);
 
+    void slotSelectionChanged(const SelectedObjects& selection);
+
+    void slotAddSharedAction(eMenu::Action action, QAction* defaultAction);
+
 protected Q_SLOTS:
     /**
       * This slot is intended to be used as part of auto saving. This is used when the
@@ -119,6 +124,21 @@ protected Q_SLOTS:
       */
     void slotEnableMessages();
 
+    /**
+     * This slot navigates to KMyMoney online community.
+     */
+    void slotGetOnlineHelp();
+
+    /**
+     * This slot navigates to KMyMoney release news page.
+     */
+    void slotWhatsNew();
+
+    /**
+     * This slot navigates to KMyMoney website.
+     */
+    void slotVisitWebsite();
+
     /**
       * Called to run performance test.
       */
@@ -196,19 +216,6 @@ protected Q_SLOTS:
       */
     void slotBackupHandleEvents();
 
-    void slotShowTipOfTheDay();
-
-    void slotShowPreviousView();
-
-    void slotShowNextView();
-
-    void slotViewSelected(View view);
-
-    /**
-      * Calls the print logic for the current view
-      */
-    void slotPrintView();
-
     /**
       * Call this slot, if any configuration parameter has changed
       */
@@ -219,6 +226,12 @@ protected Q_SLOTS:
       */
     void slotNewFeature();
 
+    /**
+     * This slot starts the tab order editor on the current widget
+     * if it is supported.
+     */
+    void slotEditTabOrder();
+
     /**
       * This slot triggers an update of all views and restarts
       * a single shot timer to call itself again at beginning of
@@ -242,6 +255,28 @@ protected Q_SLOTS:
 
     void slotCheckSchedules();
 
+    void slotDeleteTransactions();
+    void slotDuplicateTransactions();
+    void slotDisplayTransactionDetails();
+    void slotCopySplits();
+    void slotMarkTransactions();
+    void slotMoveTransactionTo();
+    void slotTransactionOpenURL();
+    bool openUrl(const QUrl& url);
+    void slotMoveToToday();
+    void slotMatchTransaction();
+    void slotAcceptTransaction();
+    void slotCreateScheduledTransaction();
+    void slotStartReconciliation();
+    void slotReportReconciliation();
+    void slotEnterOverdueSchedules();
+    void slotFindTransaction();
+    void slotCloseAccount();
+    void slotReopenAccount();
+
+    void slotExecuteAction();
+    void slotExecuteActionWithData();
+
 public:
     /**
       * This method checks if there is at least one asset or liability account
@@ -297,12 +332,6 @@ public:
     void dumpActions() const;
 #endif
 
-    /**
-      * Popup the context menu with the respective @p containerName.
-      * Valid container names are defined in kmymoneyui.rc
-      */
-    void showContextMenu(const QString& containerName);
-
     void createAccount(MyMoneyAccount& newAccount, MyMoneyAccount& parentAccount, MyMoneyAccount& brokerageAccount, MyMoneyMoney openingBal);
 
     QString filename() const;
@@ -400,13 +429,13 @@ protected:
     void resizeEvent(QResizeEvent*) final override;
 #endif
 
-    void createSchedule(MyMoneySchedule newSchedule, MyMoneyAccount& newAccount);
-
     /**
       * This method preloads the holidays for the duration of the default forecast period
       */
     void preloadHolidays();
 
+    bool event(QEvent * event) override;
+
 public Q_SLOTS:
 
     void slotFileInfoDialog();
@@ -435,11 +464,6 @@ public Q_SLOTS:
       */
     void slotHideReconciledTransactions();
 
-    /**
-      * Toggles the show reconciled balances setting
-      */
-    void slotShowReconciledBalances();
-
     /**
       * Toggles the hide unused categories setting
       */
@@ -480,22 +504,6 @@ public Q_SLOTS:
       */
     void slotEquityPriceUpdate();
 
-    /**
-      * This slot reparents account @p src to be a child of account @p dest
-      *
-      * @param src account to be reparented
-      * @param dest new parent
-      */
-    void slotReparentAccount(const MyMoneyAccount& src, const MyMoneyAccount& dest);
-
-    /**
-      * This slot reparents account @p src to be a held at institution @p dest
-      *
-      * @param src account to be reparented
-      * @param dest new parent institution
-      */
-    void slotReparentAccount(const MyMoneyAccount& src, const MyMoneyInstitution& dest);
-
     /**
       * Create a new investment in a given @p parent investment account
       */
@@ -564,14 +572,6 @@ Q_SIGNALS:
       */
     void fileLoaded(const QUrl &url);
 
-    /**
-      * This signal is emitted when a transaction/list of transactions has been selected by
-      * the GUI. If no transaction is selected or the selection is removed,
-      * @p transactions is identical to an empty QList. This signal is used
-      * by plugins to get information about changes.
-      */
-    void transactionsSelected(const KMyMoneyRegister::SelectedTransactions& transactions);
-
     /**
      * This signal is emitted if a specific transaction with @a transactionId in account
      * @a accountId is selected
@@ -628,6 +628,7 @@ public Q_SLOTS:
     bool slotFileSave();
     bool slotFileSaveAs();
     bool slotFileClose();
+    bool slotCloseViewOrFile();
     /**
       * closes all open windows by calling close() on each memberList item
       * until the list is empty, then quits the application.
