@@ -14,20 +14,23 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
-#include <QWidget>
-#include <QApplication>
-#include <QList>
-#include <QPixmap>
-#include <QWizard>
 #include <QAbstractButton>
-#include <QPixmapCache>
-#include <QIcon>
-#include <QPainter>
+#include <QApplication>
 #include <QBitArray>
+#include <QFileInfo>
+#include <QGroupBox>
+#include <QIcon>
+#include <QList>
+#include <QPainter>
+#include <QPixmap>
+#include <QPixmapCache>
 #include <QRegularExpression>
 #include <QRegularExpressionMatch>
 #include <QTemporaryFile>
-#include <QFileInfo>
+#include <QWidget>
+#include <QWizard>
+#include <amountedit.h>
+#include <creditdebitedit.h>
 
 // ----------------------------------------------------------------------------
 // KDE Headers
@@ -36,6 +39,7 @@
 #include <KGuiItem>
 #include <KIO/StatJob>
 #include <KIO/StoredTransferJob>
+#include <KLazyLocalizedString>
 #include <KLocalizedString>
 #include <KMessageBox>
 #include <KStandardGuiItem>
@@ -64,25 +68,18 @@
 #include "storageenums.h"
 #include "mymoneyenums.h"
 #include "kmymoneyplugin.h"
+#include "statusmodel.h"
+#include "journalmodel.h"
+#include "splitmodel.h"
+#include "accountsmodel.h"
+
+#include "kmmyesno.h"
 
 using namespace Icons;
 
-KMyMoneyUtils::KMyMoneyUtils()
-{
-}
-
-KMyMoneyUtils::~KMyMoneyUtils()
-{
-}
-
-const QString KMyMoneyUtils::occurrenceToString(const eMyMoney::Schedule::Occurrence occurrence)
-{
-    return i18nc("Frequency of schedule", MyMoneySchedule::occurrenceToString(occurrence).toLatin1());
-}
-
 const QString KMyMoneyUtils::paymentMethodToString(eMyMoney::Schedule::PaymentType paymentType)
 {
-    return i18nc("Scheduled Transaction payment type", MyMoneySchedule::paymentMethodToString(paymentType).toLatin1());
+    return i18n(MyMoneySchedule::paymentMethodToString(paymentType));
 }
 
 const QString KMyMoneyUtils::weekendOptionToString(eMyMoney::Schedule::WeekendOption weekendOption)
@@ -116,16 +113,16 @@ KGuiItem KMyMoneyUtils::accountsFilterGuiItem()
 }
 
 const char* homePageItems[] = {
-    I18N_NOOP("Payments"),
-    I18N_NOOP("Preferred accounts"),
-    I18N_NOOP("Payment accounts"),
-    I18N_NOOP("Favorite reports"),
-    I18N_NOOP("Forecast (schedule)"),
-    I18N_NOOP("Net worth forecast"),
-    I18N_NOOP("Forecast (history)"),        // unused, s.a. KSettingsHome::slotLoadItems()
-    I18N_NOOP("Assets and Liabilities"),
-    I18N_NOOP("Budget"),
-    I18N_NOOP("CashFlow"),
+    kli18n("Scheduled payments").untranslatedText(),
+    kli18n("Preferred accounts").untranslatedText(),
+    kli18n("Payment accounts").untranslatedText(),
+    kli18n("Favorite reports").untranslatedText(),
+    kli18n("Forecast (schedule)").untranslatedText(),
+    kli18n("Net worth forecast").untranslatedText(),
+    kli18n("Forecast (history)").untranslatedText(), // unused, s.a. KSettingsHome::slotLoadItems()
+    kli18n("Assets and Liabilities").untranslatedText(),
+    kli18n("Budget").untranslatedText(),
+    kli18n("Cash Flow").untranslatedText(),
     // insert new items above this comment
     0,
 };
@@ -190,73 +187,39 @@ void KMyMoneyUtils::checkConstants()
 #endif
 }
 
-QString KMyMoneyUtils::variableCSS()
+QString KMyMoneyUtils::getStylesheet(QString baseStylesheet)
 {
+    if (baseStylesheet.isEmpty())
+        baseStylesheet = QStandardPaths::locate(QStandardPaths::AppConfigLocation, "html/kmymoney.css");
+
     QColor tcolor = KColorScheme(QPalette::Active).foreground(KColorScheme::NormalText).color();
     QColor link = KColorScheme(QPalette::Active).foreground(KColorScheme::LinkText).color();
 
     QString css;
-    css += "<style type=\"text/css\">\n<!--\n";
+    css += QString("@media screen {\n");
     css += QString(".row-even, .item0 { background-color: %1; color: %2 }\n")
-           .arg(KMyMoneySettings::schemeColor(SchemeColor::ListBackground1).name()).arg(tcolor.name());
+               .arg(KMyMoneySettings::schemeColor(SchemeColor::ListBackground1).name(), tcolor.name());
     css += QString(".row-odd, .item1  { background-color: %1; color: %2 }\n")
-           .arg(KMyMoneySettings::schemeColor(SchemeColor::ListBackground2).name()).arg(tcolor.name());
-    css += QString("a { color: %1 }\n").arg(link.name());
-    css += "-->\n</style>\n";
+               .arg(KMyMoneySettings::schemeColor(SchemeColor::ListBackground2).name(), tcolor.name());
+    css += QString(".negativetext  { color: %1; }\n").arg(KMyMoneySettings::schemeColor(SchemeColor::Negative).name());
+    css += QString("a { color: %1; }\n").arg(link.name());
+    css += QString("}\n");
+
+    QFile cssFile(baseStylesheet);
+    if (cssFile.open(QIODevice::ReadOnly)) {
+        QTextStream cssStream(&cssFile);
+        css += cssStream.readAll();
+        cssFile.close();
+    }
+
     return css;
 }
 
-QString KMyMoneyUtils::findResource(QStandardPaths::StandardLocation type, const QString& filename)
-{
-    QLocale locale;
-    QString country;
-    QString localeName = locale.bcp47Name();
-    QString language = localeName;
-
-    // extract language and country from the bcp47name
-    QRegularExpression regExp(QLatin1String("(\\w+)_(\\w+)"));
-    QRegularExpressionMatch match = regExp.match(localeName);
-    if (match.hasMatch()) {
-        language = match.captured(1);
-        country = match.captured(2);
-    }
-
-    QString rc;
-
-    // check that the placeholder is present and set things up
-    if (filename.indexOf("%1") != -1) {
-        /// @fixme somehow I have the impression, that language and country
-        ///    mappings to the filename are not correct. This certainly must
-        ///    be overhauled at some point in time (ipwizard, 2017-10-22)
-        QString mask = filename.arg("_%1.%2");
-        rc = QStandardPaths::locate(type, mask.arg(country, language));
-
-        // search the given resource
-        if (rc.isEmpty()) {
-            mask = filename.arg("_%1");
-            rc = QStandardPaths::locate(type, mask.arg(language));
-        }
-        if (rc.isEmpty()) {
-            // qDebug(QString("html/home_%1.html not found").arg(country).toLatin1());
-            rc = QStandardPaths::locate(type, mask.arg(country));
-        }
-        if (rc.isEmpty()) {
-            rc = QStandardPaths::locate(type, filename.arg(""));
-        }
-    } else {
-        rc = QStandardPaths::locate(type, filename);
-    }
-
-    if (rc.isEmpty()) {
-        qWarning("No resource found for (%s,%s)", qPrintable(QStandardPaths::displayName(type)), qPrintable(filename));
-    }
-    return rc;
-}
-
 const MyMoneySplit KMyMoneyUtils::stockSplit(const MyMoneyTransaction& t)
 {
     MyMoneySplit investmentAccountSplit;
-    foreach (const auto split, t.splits()) {
+    const auto splits = t.splits();
+    for (const auto& split : splits) {
         if (!split.accountId().isEmpty()) {
             auto acc = MyMoneyFile::instance()->account(split.accountId());
             if (acc.isInvest()) {
@@ -358,73 +321,21 @@ QString KMyMoneyUtils::getAdjacentNumber(const QString& number, int offset)
     // make sure the offset is either -1 or 1
     offset = (offset >= 0) ? 1 : -1;
 
-    QString num = number;
-    //                   +-#1--+ +#2++-#3-++-#4--+
-    QRegExp exp(QString("(.*\\D)?(0*)(\\d+)(\\D.*)?"));
-    if (exp.indexIn(num) != -1) {
-        QString arg1 = exp.cap(1);
-        QString arg2 = exp.cap(2);
-        QString arg3 = QString::number(exp.cap(3).toULong() + offset);
-        QString arg4 = exp.cap(4);
-        num = QString("%1%2%3%4").arg(arg1, arg2, arg3, arg4);
-    } else {
-        num = QStringLiteral("1");
+    //                              +-#1--+ +#2++-#3-++-#4--+
+    static const QRegularExpression exp(QString("(.*\\D)?(0*)(\\d+)(\\D.*)?"));
+    QRegularExpressionMatch match = exp.match(number);
+    if (match.hasMatch()) {
+        return QStringLiteral("%1%2%3%4").arg(match.captured(1), match.captured(2), QString::number(match.captured(3).toULong() + offset), match.captured(4));
     }
-    return num;
-}
-
-quint64 KMyMoneyUtils::numericPart(const QString & num)
-{
-    quint64 num64 = 0;
-    QRegExp exp(QString("(.*\\D)?(0*)(\\d+)(\\D.*)?"));
-    if (exp.indexIn(num) != -1) {
-        // QString arg1 = exp.cap(1);
-        QString arg2 = exp.cap(2);
-        QString arg3 = QString::number(exp.cap(3).toULongLong());
-        // QString arg4 = exp.cap(4);
-        num64 = QString("%2%3").arg(arg2, arg3).toULongLong();
-    }
-    return num64;
+    return QStringLiteral("1");
 }
 
 QString KMyMoneyUtils::reconcileStateToString(eMyMoney::Split::State flag, bool text)
 {
     QString txt;
-    if (text) {
-        switch (flag) {
-        case eMyMoney::Split::State::NotReconciled:
-            txt = i18nc("Reconciliation state 'Not reconciled'", "Not reconciled");
-            break;
-        case eMyMoney::Split::State::Cleared:
-            txt = i18nc("Reconciliation state 'Cleared'", "Cleared");
-            break;
-        case eMyMoney::Split::State::Reconciled:
-            txt = i18nc("Reconciliation state 'Reconciled'", "Reconciled");
-            break;
-        case eMyMoney::Split::State::Frozen:
-            txt = i18nc("Reconciliation state 'Frozen'", "Frozen");
-            break;
-        default:
-            txt = i18nc("Unknown reconciliation state", "Unknown");
-            break;
-        }
-    } else {
-        switch (flag) {
-        case eMyMoney::Split::State::NotReconciled:
-            break;
-        case eMyMoney::Split::State::Cleared:
-            txt = i18nc("Reconciliation flag C", "C");
-            break;
-        case eMyMoney::Split::State::Reconciled:
-            txt = i18nc("Reconciliation flag R", "R");
-            break;
-        case eMyMoney::Split::State::Frozen:
-            txt = i18nc("Reconciliation flag F", "F");
-            break;
-        default:
-            txt = i18nc("Flag for unknown reconciliation state", "?");
-            break;
-        }
+    const QModelIndex idx = MyMoneyFile::instance()->statusModel()->index(static_cast<int>(flag), 0);
+    if (idx.isValid()) {
+        txt = idx.data(text ? eMyMoney::Model::SplitReconcileStatusRole : eMyMoney::Model::SplitReconcileFlagRole).toString();
     }
     return txt;
 }
@@ -448,7 +359,8 @@ MyMoneyTransaction KMyMoneyUtils::scheduledTransaction(const MyMoneySchedule& sc
 
 KXmlGuiWindow* KMyMoneyUtils::mainWindow()
 {
-    foreach (QWidget *widget, QApplication::topLevelWidgets()) {
+    const auto widgetList = QApplication::topLevelWidgets();
+    for (QWidget* widget : qAsConst(widgetList)) {
         KXmlGuiWindow* result = dynamic_cast<KXmlGuiWindow*>(widget);
         if (result)
             return result;
@@ -469,55 +381,50 @@ void KMyMoneyUtils::updateWizardButtons(QWizard* wizard)
     wizard->button(QWizard::BackButton)->setIcon(KStandardGuiItem::back(KStandardGuiItem::UseRTL).icon());
 }
 
-void KMyMoneyUtils::dissectTransaction(const MyMoneyTransaction& transaction, const MyMoneySplit& split, MyMoneySplit& assetAccountSplit, QList<MyMoneySplit>& feeSplits, QList<MyMoneySplit>& interestSplits, MyMoneySecurity& security, MyMoneySecurity& currency, eMyMoney::Split::InvestmentTransactionType& transactionType)
+void KMyMoneyUtils::dissectInvestmentTransaction(const QModelIndex &investSplitIdx, QModelIndex &assetAccountSplitIdx, SplitModel* feeSplitModel, SplitModel* interestSplitModel, MyMoneySecurity &security, MyMoneySecurity &currency, eMyMoney::Split::InvestmentTransactionType &transactionType)
 {
+    // clear split models
+    feeSplitModel->unload();
+    interestSplitModel->unload();
+
+    assetAccountSplitIdx = QModelIndex(); // set to none to check later if it was assigned
+    const auto file = MyMoneyFile::instance();
+
     // collect the splits. split references the stock account and should already
     // be set up. assetAccountSplit references the corresponding asset account (maybe
     // empty), feeSplits is the list of all expenses and interestSplits
     // the list of all incomes
-    assetAccountSplit = MyMoneySplit(); // set to none to check later if it was assigned
-    auto file = MyMoneyFile::instance();
-    foreach (const auto tsplit, transaction.splits()) {
-        auto acc = file->account(tsplit.accountId());
-        if (tsplit.id() == split.id()) {
-            security = file->security(acc.currencyId());
-        } else if (acc.accountGroup() == eMyMoney::Account::Type::Expense) {
-            feeSplits.append(tsplit);
-            // feeAmount += tsplit.value();
-        } else if (acc.accountGroup() == eMyMoney::Account::Type::Income) {
-            interestSplits.append(tsplit);
-            // interestAmount += tsplit.value();
+    auto idx = MyMoneyFile::baseModel()->mapToBaseSource(investSplitIdx);
+    const auto list = idx.model()->match(idx.model()->index(0, 0), eMyMoney::Model::JournalTransactionIdRole,
+                                         idx.data(eMyMoney::Model::JournalTransactionIdRole),
+                                         -1,                         // all splits
+                                         Qt::MatchFlags(Qt::MatchExactly | Qt::MatchCaseSensitive | Qt::MatchRecursive));
+    for (const auto& splitIdx : list) {
+        auto accIdx = file->accountsModel()->indexById(splitIdx.data(eMyMoney::Model::SplitAccountIdRole).toString());
+        const auto accountGroup = accIdx.data(eMyMoney::Model::AccountGroupRole).value<eMyMoney::Account::Type>();
+        if (splitIdx.row() == idx.row()) {
+            security = file->security(accIdx.data(eMyMoney::Model::AccountCurrencyIdRole).toString());
+        } else if (accountGroup == eMyMoney::Account::Type::Expense) {
+            feeSplitModel->appendSplit(file->journalModel()->itemByIndex(splitIdx).split());
+        } else if (accountGroup == eMyMoney::Account::Type::Income) {
+            interestSplitModel->appendSplit(file->journalModel()->itemByIndex(splitIdx).split());
         } else {
-            if (assetAccountSplit == MyMoneySplit()) // first asset Account should be our requested brokerage account
-                assetAccountSplit = tsplit;
-            else if (tsplit.value().isNegative())  // the rest (if present) is handled as fee or interest
-                feeSplits.append(tsplit);              // and shouldn't be allowed to override assetAccountSplit
-            else if (tsplit.value().isPositive())
-                interestSplits.append(tsplit);
+            if (!assetAccountSplitIdx.isValid()) { // first asset Account should be our requested brokerage account
+                assetAccountSplitIdx = splitIdx;
+            } else if (idx.data(eMyMoney::Model::SplitValueRole).value<MyMoneyMoney>().isNegative()) { // the rest (if present) is handled as fee or interest
+                feeSplitModel->appendSplit(file->journalModel()->itemByIndex(splitIdx).split());
+            } else if (idx.data(eMyMoney::Model::SplitValueRole).value<MyMoneyMoney>().isPositive()) {
+                interestSplitModel->appendSplit(file->journalModel()->itemByIndex(splitIdx).split());
+            }
         }
     }
 
     // determine transaction type
-    if (split.action() == MyMoneySplit::actionName(eMyMoney::Split::Action::AddShares)) {
-        transactionType = (!split.shares().isNegative()) ? eMyMoney::Split::InvestmentTransactionType::AddShares : eMyMoney::Split::InvestmentTransactionType::RemoveShares;
-    } else if (split.action() == MyMoneySplit::actionName(eMyMoney::Split::Action::BuyShares)) {
-        transactionType = (!split.value().isNegative()) ? eMyMoney::Split::InvestmentTransactionType::BuyShares : eMyMoney::Split::InvestmentTransactionType::SellShares;
-    } else if (split.action() == MyMoneySplit::actionName(eMyMoney::Split::Action::Dividend)) {
-        transactionType = eMyMoney::Split::InvestmentTransactionType::Dividend;
-    } else if (split.action() == MyMoneySplit::actionName(eMyMoney::Split::Action::ReinvestDividend)) {
-        transactionType = eMyMoney::Split::InvestmentTransactionType::ReinvestDividend;
-    } else if (split.action() == MyMoneySplit::actionName(eMyMoney::Split::Action::Yield)) {
-        transactionType = eMyMoney::Split::InvestmentTransactionType::Yield;
-    } else if (split.action() == MyMoneySplit::actionName(eMyMoney::Split::Action::SplitShares)) {
-        transactionType = eMyMoney::Split::InvestmentTransactionType::SplitShares;
-    } else if (split.action() == MyMoneySplit::actionName(eMyMoney::Split::Action::InterestIncome)) {
-        transactionType = eMyMoney::Split::InvestmentTransactionType::InterestIncome;
-    } else
-        transactionType = eMyMoney::Split::InvestmentTransactionType::BuyShares;
+    transactionType = idx.data(eMyMoney::Model::TransactionInvestementType).value<eMyMoney::Split::InvestmentTransactionType>();
 
     currency.setTradingSymbol("???");
     try {
-        currency = file->security(transaction.commodity());
+        currency = file->security(file->journalModel()->itemByIndex(idx).transaction().commodity());
     } catch (const MyMoneyException &) {
     }
 }
@@ -573,7 +480,7 @@ void KMyMoneyUtils::deleteSecurity(const MyMoneySecurity& security, QWidget* par
         dontAsk = "DeleteSecurity";
         dontAsk2 = "DeleteSecurityPrices";
     }
-    if (KMessageBox::questionYesNo(parent, msg, i18n("Delete security"), KStandardGuiItem::yes(), KStandardGuiItem::no(), dontAsk) == KMessageBox::Yes) {
+    if (KMessageBox::questionTwoActions(parent, msg, i18n("Delete security"), KMMYesNo::yes(), KMMYesNo::no(), dontAsk) == KMessageBox::PrimaryAction) {
         MyMoneyFileTransaction ft;
         auto file = MyMoneyFile::instance();
 
@@ -581,10 +488,11 @@ void KMyMoneyUtils::deleteSecurity(const MyMoneySecurity& security, QWidget* par
         skip.fill(true);
         skip.clearBit((int)eStorage::Reference::Price);
         if (file->isReferenced(security, skip)) {
-            if (KMessageBox::questionYesNo(parent, msg2, i18n("Delete prices"), KStandardGuiItem::yes(), KStandardGuiItem::no(), dontAsk2) == KMessageBox::Yes) {
+            if (KMessageBox::questionTwoActions(parent, msg2, i18n("Delete prices"), KMMYesNo::yes(), KMMYesNo::no(), dontAsk2) == KMessageBox::PrimaryAction) {
                 try {
                     QString secID = security.id();
-                    foreach (auto priceEntry, file->priceList()) {
+                    const auto priceList = file->priceList();
+                    for (const auto& priceEntry : priceList) {
                         const MyMoneyPrice& price = priceEntry.first();
                         if (price.from() == secID || price.to() == secID)
                             file->removePrice(price);
@@ -618,9 +526,8 @@ bool KMyMoneyUtils::fileExists(const QUrl &url)
             fileExists = check_file.exists() && check_file.isFile();
 
         } else {
-#if KIO_VERSION < QT_VERSION_CHECK(5, 70, 0)
-            short int detailLevel = 0; // Lowest level: file/dir/symlink/none
-            KIO::StatJob* statjob = KIO::stat(url, KIO::StatJob::SourceSide, detailLevel);
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+            auto statjob = KIO::stat(url, KIO::StatJob::SourceSide, KIO::StatNoDetails);
 #else
             auto statjob = KIO::statDetails(url, KIO::StatJob::SourceSide, KIO::StatNoDetails);
 #endif
@@ -657,15 +564,17 @@ QString KMyMoneyUtils::downloadFile(const QUrl &url)
     return filename;
 }
 
-bool KMyMoneyUtils::newPayee(const QString& newnameBase, QString& id)
+std::tuple<bool, QString> KMyMoneyUtils::newPayee(const QString& newnameBase)
 {
     bool doit = true;
+    QString id;
 
     if (newnameBase != i18n("New Payee")) {
         // Ask the user if that is what he intended to do?
-        const auto msg = i18n("<qt>Do you want to add <b>%1</b> as payer/receiver?</qt>", newnameBase);
+        const auto msg = i18n("<qt>Do you want to add <b>%1</b> as payee/receiver?</qt>", newnameBase);
 
-        if (KMessageBox::questionYesNo(nullptr, msg, i18n("New payee/receiver"), KStandardGuiItem::yes(), KStandardGuiItem::no(), "NewPayee") == KMessageBox::No) {
+        if (KMessageBox::questionTwoActions(nullptr, msg, i18n("New payee/receiver"), KMMYesNo::yes(), KMMYesNo::no(), "NewPayee")
+            == KMessageBox::SecondaryAction) {
             doit = false;
             // we should not keep the 'no' setting because that can confuse people like
             // I have seen in some usability tests. So we just delete it right away.
@@ -682,9 +591,12 @@ bool KMyMoneyUtils::newPayee(const QString& newnameBase, QString& id)
             QString newname(newnameBase);
             // adjust name until a unique name has been created
             int count = 0;
+
             for (;;) {
                 try {
-                    MyMoneyFile::instance()->payeeByName(newname);
+                    const auto payee = MyMoneyFile::instance()->payeeByName(newname);
+                    if (payee.id().isEmpty())
+                        break;
                     newname = QString::fromLatin1("%1 [%2]").arg(newnameBase).arg(++count);
                 } catch (const MyMoneyException &) {
                     break;
@@ -698,22 +610,23 @@ bool KMyMoneyUtils::newPayee(const QString& newnameBase, QString& id)
             id = p.id();
             ft.commit();
         } catch (const MyMoneyException &e) {
-            KMessageBox::detailedSorry(nullptr, i18n("Unable to add payee"), QString::fromLatin1(e.what()));
+            KMessageBox::detailedError(nullptr, i18n("Unable to add payee"), QString::fromLatin1(e.what()));
             doit = false;
         }
     }
-    return doit;
+    return std::make_tuple(doit, id);
 }
 
-void KMyMoneyUtils::newTag(const QString& newnameBase, QString& id)
+std::tuple<bool, QString> KMyMoneyUtils::newTag(const QString& newnameBase)
 {
     bool doit = true;
+    QString id;
 
     if (newnameBase != i18n("New Tag")) {
         // Ask the user if that is what he intended to do?
         const auto msg = i18n("<qt>Do you want to add <b>%1</b> as tag?</qt>", newnameBase);
 
-        if (KMessageBox::questionYesNo(nullptr, msg, i18n("New tag"), KStandardGuiItem::yes(), KStandardGuiItem::no(), "NewTag") == KMessageBox::No) {
+        if (KMessageBox::questionTwoActions(nullptr, msg, i18n("New tag"), KMMYesNo::yes(), KMMYesNo::no(), "NewTag") == KMessageBox::SecondaryAction) {
             doit = false;
             // we should not keep the 'no' setting because that can confuse people like
             // I have seen in some usability tests. So we just delete it right away.
@@ -732,7 +645,9 @@ void KMyMoneyUtils::newTag(const QString& newnameBase, QString& id)
             int count = 0;
             for (;;) {
                 try {
-                    MyMoneyFile::instance()->tagByName(newname);
+                    if (MyMoneyFile::instance()->tagByName(newname).id().isEmpty()) {
+                        break;
+                    }
                     newname = QString::fromLatin1("%1 [%2]").arg(newnameBase).arg(++count);
                 } catch (const MyMoneyException &) {
                     break;
@@ -745,9 +660,10 @@ void KMyMoneyUtils::newTag(const QString& newnameBase, QString& id)
             id = ta.id();
             ft.commit();
         } catch (const MyMoneyException &e) {
-            KMessageBox::detailedSorry(nullptr, i18n("Unable to add tag"), QString::fromLatin1(e.what()));
+            KMessageBox::detailedError(nullptr, i18n("Unable to add tag"), QString::fromLatin1(e.what()));
         }
     }
+    return std::make_tuple(doit, id);
 }
 
 void KMyMoneyUtils::newInstitution(MyMoneyInstitution& institution)
@@ -793,18 +709,16 @@ bool KMyMoneyUtils::canUpdateAllAccounts()
 {
     const auto file = MyMoneyFile::instance();
     auto rc = false;
-    if (!file->storageAttached())
-        return rc;
 
     QList<MyMoneyAccount> accList;
     file->accountList(accList);
     QList<MyMoneyAccount>::const_iterator it_a;
-    auto it_p = pPlugins.online.constEnd();
-    for (it_a = accList.constBegin(); (it_p == pPlugins.online.constEnd()) && (it_a != accList.constEnd()); ++it_a) {
+    auto it_p = pPlugins.online.cend();
+    for (it_a = accList.cbegin(); (it_p == pPlugins.online.cend()) && (it_a != accList.cend()); ++it_a) {
         if ((*it_a).hasOnlineMapping()) {
             // check if provider is available
             it_p = pPlugins.online.constFind((*it_a).onlineBankingSettings().value("provider").toLower());
-            if (it_p != pPlugins.online.constEnd()) {
+            if (it_p != pPlugins.online.cend()) {
                 QStringList protocols;
                 (*it_p)->protocols(protocols);
                 if (!protocols.isEmpty()) {
@@ -831,8 +745,154 @@ void KMyMoneyUtils::showStatementImportResult(const QStringList& resultMessages,
 
 QString KMyMoneyUtils::normalizeNumericString(const qreal& val, const QLocale& loc, const char f, const int prec)
 {
-    return loc.toString(val, f, prec)
-           .remove(loc.groupSeparator())
-           .remove(QRegularExpression("0+$"))
-           .remove(QRegularExpression("\\" + loc.decimalPoint() + "$"));
+    static const QRegularExpression trailingZeroesRegex("0+$");
+    return loc.toString(val, f, prec).remove(loc.groupSeparator()).remove(trailingZeroesRegex).remove(QRegularExpression("\\" + loc.decimalPoint() + "$"));
+}
+
+QStringList KMyMoneyUtils::tabOrder(const QString& name, const QStringList& defaultTabOrder)
+{
+    KSharedConfigPtr config = KSharedConfig::openConfig();
+    KConfigGroup grp = config->group(QLatin1String("TabOrder"));
+    return grp.readEntry(name, defaultTabOrder);
+}
+
+#if 0
+static QString widgetName(QWidget* w)
+{
+    return w->objectName().isEmpty() ? w->metaObject()->className() : w->objectName();
+}
+
+static void dumpFocusChain(QWidget* w, QWidget* end, int additionalTabstops = 0, bool forward = true)
+{
+    QString txt;
+    int loopValid = 80;
+    int trailing = -1;
+    do {
+        const auto policy = w->focusPolicy();
+        if (policy == Qt::TabFocus || policy == Qt::StrongFocus ) {
+            if (!txt.isEmpty()) {
+                txt += forward ? QLatin1String(" -> ") : QLatin1String(" <- ");
+            }
+            txt += widgetName(w);
+            --loopValid;
+            w = forward ? w->nextInFocusChain() : w->previousInFocusChain();
+            if (w == end && (trailing < 0)) {
+                trailing = additionalTabstops+1;
+                loopValid = trailing;
+            }
+        }
+        --trailing;
+    } while ((loopValid > 0) && (trailing != 0));
+
+    if (end) {
+        txt += forward ? QLatin1String(" -> ") : QLatin1String(" <- ");
+        txt += widgetName(w);
+    }
+
+    qDebug() << txt;
+}
+#endif
+
+void KMyMoneyUtils::setupTabOrder(QWidget* parent, const QStringList& tabOrder)
+{
+    const auto widgetCount = tabOrder.count();
+    if (widgetCount > 0) {
+        auto prev = parent->findChild<QWidget*>(tabOrder.at(0));
+        for (int i = 1; (prev != nullptr) && (i < widgetCount); ++i) {
+            const auto next = parent->findChild<QWidget*>(tabOrder.at(i));
+            if (next) {
+                parent->setTabOrder(prev, next);
+                prev = next;
+            } else {
+                qDebug() << tabOrder.at(i) << "not found :(";
+            }
+        }
+    }
+}
+
+void KMyMoneyUtils::storeTabOrder(const QString& name, const QStringList& tabOrder)
+{
+    KSharedConfigPtr config = KSharedConfig::openConfig();
+    KConfigGroup grp = config->group(QLatin1String("TabOrder"));
+    grp.writeEntry(name, tabOrder);
+}
+
+bool KMyMoneyUtils::tabFocusHelper(QWidget* topLevelWidget, bool next)
+{
+    const auto reason = next ? Qt::TabFocusReason : Qt::BacktabFocusReason;
+    const auto tabOrder = topLevelWidget->property("kmm_currenttaborder").toStringList();
+
+    if (tabOrder.isEmpty())
+        return false;
+
+    auto focusWidget = topLevelWidget->focusWidget();
+
+    enum firstOrLastVisible {
+        FirstVisible,
+        LastVisible,
+    };
+
+    auto findFirstOrLastVisible = [&](firstOrLastVisible type) {
+        const int ofs = (type == FirstVisible) ? 1 : -1;
+        int idx = (type == FirstVisible) ? 0 : tabOrder.count() - 1;
+        for (; idx >= 0 && idx < tabOrder.count(); idx += ofs) {
+            auto w = topLevelWidget->findChild<QWidget*>(tabOrder.at(idx));
+            // in case of embedded transaction editors, we may search
+            // for a widget that is known to the parent
+            if (!w) {
+                auto parent = topLevelWidget->parentWidget();
+                while (true) {
+                    w = parent->findChild<QWidget*>(tabOrder.at(idx));
+                    if (!w && qobject_cast<QGroupBox*>(parent)) {
+                        parent = parent->parentWidget();
+                        continue;
+                    }
+                    break;
+                }
+            }
+            if (w && w->isVisible() && w->isEnabled()) {
+                return w;
+            }
+        }
+        return static_cast<QWidget*>(nullptr);
+    };
+
+    auto selectWidget = [&](QWidget* w) {
+        if (w) {
+            // if we point to a constructed widget (e.g. ButtonBox) we
+            // need to select the last widget if going backward
+            if (reason == Qt::BacktabFocusReason && !w->findChildren<QWidget*>().isEmpty()) {
+                auto parent = w;
+                while (w->nextInFocusChain()->parentWidget() == parent) {
+                    w = w->nextInFocusChain();
+                }
+            }
+            w->setFocus(reason);
+        }
+    };
+
+    auto adjustToContainer = [&](const char* containerClass, const char* widgetClass) {
+        if (focusWidget->qt_metacast(widgetClass) && focusWidget->parentWidget()->qt_metacast(containerClass) && (reason == Qt::BacktabFocusReason)) {
+            if (focusWidget->previousInFocusChain() == focusWidget->parentWidget()) {
+                focusWidget = focusWidget->parentWidget();
+            }
+        }
+    };
+
+    // In case of a CreditDebitEdit widget and we leave from the left backwards,
+    // we need to adjust the widget to point to the container widget.
+    adjustToContainer("CreditDebitEdit", "AmountEdit");
+    // In case of a QDialogButtonBox widget and we leave from the left backwards,
+    // we need to adjust the widget to point to the container widget.
+    // adjustToContainer("QDialogButtonBox", "QPushButton");
+
+    if ((reason == Qt::BacktabFocusReason) && (findFirstOrLastVisible(FirstVisible) == focusWidget)) {
+        selectWidget(findFirstOrLastVisible(LastVisible));
+        return true;
+
+    } else if ((reason == Qt::TabFocusReason) && (findFirstOrLastVisible(LastVisible) == focusWidget)) {
+        selectWidget(findFirstOrLastVisible(FirstVisible));
+        return true;
+    }
+    return false;
 }
