@@ -15,6 +15,8 @@ Q_LOGGING_CATEGORY(WebConnectLog, "WebConnect")
 
 class WebConnect::Private
 {
+    Q_DISABLE_COPY_MOVE(Private)
+
 public:
     explicit Private(WebConnect* parent)
         : q(parent)
@@ -149,7 +151,7 @@ void WebConnect::dataAvailable()
     QUrl url;
     in >> url;
     qCDebug(WebConnectLog) << "Processing" << url;
-    emit gotUrl(url);
+    Q_EMIT gotUrl(url);
 }
 
 void WebConnect::loadFile(const QUrl& url)
