@@ -1,15 +1,17 @@
 /*
     SPDX-FileCopyrightText: 2017-2018 Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
+    SPDX-FileCopyrightText: 2019 Thomas Baumgart <tbaumgart@kde.org>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
 #include "equitiesmodel.h"
+#include "accountsmodel.h"
+#include "mymoneyfile.h"
+#include "securitiesmodel.h"
 
 // ----------------------------------------------------------------------------
 // QT Includes
-
-#include <QMenu>
-#include <QDebug>
+#include <QDate>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
@@ -19,26 +21,25 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include "mymoneymoney.h"
-#include "mymoneyfile.h"
-#include "mymoneyaccount.h"
-#include "mymoneysecurity.h"
 #include "mymoneyprice.h"
+#include "mymoneymoney.h"
 #include "mymoneyenums.h"
+#include "mymoneyutils.h"
 
-class EquitiesModel::Private
+class EquitiesModelPrivate
 {
+    Q_DECLARE_PUBLIC(EquitiesModel);
+    EquitiesModel* q_ptr;
+
 public:
-    Private() : m_file(MyMoneyFile::instance())
+    EquitiesModelPrivate(EquitiesModel* qq)
+        : q_ptr(qq)
     {
-        QVector<Column> columns {Column::Equity, Column::Symbol, Column::Value,
-                                 Column::Quantity, Column::Price};
-        foreach (auto const column, columns)
-            m_columns.append(column);
     }
 
-    ~Private() {}
+    ~EquitiesModelPrivate() {}
 
+#if 0
     void loadInvestmentAccount(QStandardItem *node, const MyMoneyAccount &invAcc)
     {
         auto itInvAcc = new QStandardItem(invAcc.name());
@@ -47,7 +48,7 @@ public:
         itInvAcc->setColumnCount(m_columns.count());
         setAccountData(node, itInvAcc->row(), invAcc, m_columns);
 
-        foreach (const auto strStkAcc, invAcc.accountList()) { // only stock or bond accounts are expected here
+        for (const auto strStkAcc : invAcc.accountList()) { // only stock or bond accounts are expected here
             auto stkAcc = m_file->account(strStkAcc);
             auto itStkAcc = new QStandardItem(strStkAcc);
             itStkAcc->setEditable(false);
@@ -123,7 +124,7 @@ public:
 
         auto balance = m_file->balance(account.id());
         auto security = m_file->security(account.currencyId());
-        auto tradingCurrency = m_file->security(security.tradingCurrency());
+        auto tradingCurrency = m_file->currency(security.tradingCurrency());
         auto price = m_file->price(account.currencyId(), tradingCurrency.id());
 
         // Value
@@ -182,41 +183,124 @@ public:
 
     MyMoneyFile *m_file;
     QList<EquitiesModel::Column> m_columns;
+#endif
 };
 
 EquitiesModel::EquitiesModel(QObject *parent)
-    : QStandardItemModel(parent), d(new Private)
+    : KExtraColumnsProxyModel(parent)
+    , d_ptr(new EquitiesModelPrivate(this))
 {
-    init();
+    appendColumn(i18nc("@title stock symbol column", "Symbol"));
+    appendColumn(i18n("Quantity"));
+    appendColumn(i18n("Price"));
+    appendColumn(i18n("Value"));
+    appendColumn(i18n("Last Price Update"));
 }
 
 EquitiesModel::~EquitiesModel()
 {
+    Q_D(EquitiesModel);
     delete d;
 }
 
-void EquitiesModel::init()
+QVariant EquitiesModel::extraColumnData(const QModelIndex& parent, int row, int extraColumn, int role) const
 {
-    QStringList headerLabels;
-    foreach (const auto column, d->m_columns)
-        headerLabels.append(getHeaderName(column));
-    setHorizontalHeaderLabels(headerLabels);
+    const auto file = MyMoneyFile::instance();
+
+    const auto idx = index(row, 0, parent);
+    const auto baseIdx = MyMoneyFile::baseModel()->mapToBaseSource(idx);
+    auto model = qobject_cast<const AccountsModel*>(baseIdx.model());
+
+    const auto acc = model->itemByIndex(baseIdx);
+
+    if (role == Qt::DisplayRole || role == Qt::EditRole) {
+        const auto securityIdx = file->securitiesModel()->indexById(acc.currencyId());
+        const auto tradingCurrencyIdx = securityIdx.data(eMyMoney::Model::SecurityTradingCurrencyIndexRole).value<QModelIndex>();
+
+        switch (extraColumn) {
+        case Symbol:
+            return securityIdx.data(eMyMoney::Model::SecuritySymbolRole);
+
+        case Value: {
+            const auto balance = baseIdx.data(eMyMoney::Model::AccountBalanceRole).value<MyMoneyMoney>();
+            const auto tradingCurrencyId = tradingCurrencyIdx.data(eMyMoney::Model::IdRole).toString();
+            const auto prec = MyMoneyMoney::denomToPrec(tradingCurrencyIdx.data(eMyMoney::Model::SecuritySmallestAccountFractionRole).toInt());
+            const auto tradingCurrencySymbol = tradingCurrencyIdx.data(eMyMoney::Model::SecuritySymbolRole).toString();
+
+            const auto value = (file->price(acc.currencyId(), tradingCurrencyId).rate(tradingCurrencyId) * balance);
+
+            if(role == Qt::EditRole) {
+                return value.toDouble();
+            } else {
+                return value.formatMoney(tradingCurrencySymbol, prec);
+            }
+        }
+
+        case Quantity: {
+            const auto balance = baseIdx.data(eMyMoney::Model::AccountBalanceRole).value<MyMoneyMoney>();
+            const auto prec = MyMoneyMoney::denomToPrec(securityIdx.data(eMyMoney::Model::SecuritySmallestAccountFractionRole).toInt());
+
+            if(role == Qt::EditRole) {
+                return balance.toDouble();
+            } else {
+                return balance.formatMoney(QString(), prec);
+            }
+        }
+
+        case Price: {
+            const auto tradingCurrencyId = tradingCurrencyIdx.data(eMyMoney::Model::IdRole).toString();
+            const auto prec = securityIdx.data(eMyMoney::Model::SecurityPricePrecisionRole).toInt();
+            const auto tradingCurrencySymbol = tradingCurrencyIdx.data(eMyMoney::Model::SecuritySymbolRole).toString();
+
+            const auto value = file->price(acc.currencyId(), tradingCurrencyId).rate(tradingCurrencyId);
+
+            if(role == Qt::EditRole) {
+                return value.toDouble();
+            } else {
+                return value.formatMoney(tradingCurrencySymbol, prec);
+            }
+        }
+
+        case LastPriceUpdate: {
+            const auto tradingCurrencyId = tradingCurrencyIdx.data(eMyMoney::Model::IdRole).toString();
+            const auto priceDate = MyMoneyUtils::formatDate(file->price(acc.currencyId(), tradingCurrencyId).date());
+            return priceDate;
+        }
+
+        default:
+            break;
+        }
+    } else {
+        switch(role) {
+        case Qt::DecorationRole:
+            return QVariant();
+
+        case Qt::TextAlignmentRole:
+            switch(extraColumn) {
+            case Symbol:
+                return QVariant(Qt::AlignLeft | Qt::AlignVCenter);
+            default:
+                return QVariant(Qt::AlignRight | Qt::AlignVCenter);
+            }
+            break;
+
+        default:
+            return idx.data(role);
+        }
+    }
+    return QVariant();
 }
 
-void EquitiesModel::load()
+QVariant EquitiesModel::headerData(int section, Qt::Orientation orientation, int role) const
 {
-    this->blockSignals(true);
-
-    auto rootItem = invisibleRootItem();
-    QList<MyMoneyAccount> accList;
-    d->m_file->accountList(accList);                        // get all available accounts
-    foreach (const auto acc, accList)
-        if (acc.accountType() == eMyMoney::Account::Type::Investment)  // but add only investment accounts (and its children) to the model
-            d->loadInvestmentAccount(rootItem, acc);
-
-    this->blockSignals(false);
+    // KExtraColumnsProxyModel only supports Qt::DisplayRole
+    if (role == eMyMoney::Model::LongDisplayRole) {
+        role = Qt::DisplayRole;
+    }
+    return KExtraColumnsProxyModel::headerData(section, orientation, role);
 }
 
+#if 0
 /**
   * Notify the model that an object has been added. An action is performed only if the object is an account.
   */
@@ -310,7 +394,7 @@ void EquitiesModel::slotObjectRemoved(eMyMoney::File::Object objType, const QStr
         return;
 
     const auto indexList = match(index(0, 0), Role::EquityID, id, -1, Qt::MatchFlags(Qt::MatchExactly | Qt::MatchRecursive));
-    foreach (const auto index, indexList)
+    for (const auto& index : indexList)
         removeRow(index.row(), index.parent());
 }
 
@@ -346,6 +430,8 @@ QString EquitiesModel::getHeaderName(const Column column)
         return i18n("Quantity");
     case Price:
         return i18n("Price");
+    case LastPriceUpdate:
+        return i18n("Last Price Update");
     default:
         return QString();
     }
@@ -453,7 +539,7 @@ void EquitiesFilterProxyModel::slotColumnsMenu(const QPoint)
     // create menu
     QMenu menu(i18n("Displayed columns"));
     QList<QAction *> actions;
-    foreach (const auto idColumn, idColumns) {
+    for (const auto idColumn : idColumns) {
         auto a = new QAction(nullptr);
         a->setObjectName(QString::number(idColumn));
         a->setText(EquitiesModel::getHeaderName(idColumn));
@@ -471,12 +557,13 @@ void EquitiesFilterProxyModel::slotColumnsMenu(const QPoint)
         const auto contains = d->m_visColumns.contains(idColumn);
         if (isChecked && !contains) {           // column has just been enabled
             d->m_visColumns.append(idColumn);     // change filtering variable
-            emit columnToggled(idColumn, true);   // emit signal for method to add column to model
+            Q_EMIT columnToggled(idColumn, true);   // Q_EMIT signal for method to add column to model
             invalidate();                         // refresh model to reflect recent changes
         } else if (!isChecked && contains) {    // column has just been disabled
             d->m_visColumns.removeOne(idColumn);
-            emit columnToggled(idColumn, false);
+            Q_EMIT columnToggled(idColumn, false);
             invalidate();
         }
     }
 }
+#endif
