@@ -2,7 +2,7 @@
     SPDX-FileCopyrightText: 2000-2002 Michael Edwardes <mte@users.sourceforge.net>
     SPDX-FileCopyrightText: 2001 Felix Rodriguez <frodriguez@users.sourceforge.net>
     SPDX-FileCopyrightText: 2002-2003 Kevin Tambascio <ktambascio@users.sourceforge.net>
-    SPDX-FileCopyrightText: 2006-2017 Thomas Baumgart <tbaumgart@kde.org>
+    SPDX-FileCopyrightText: 2006-2019 Thomas Baumgart <tbaumgart@kde.org>
     SPDX-FileCopyrightText: 2006 Ace Jones <acejones@users.sourceforge.net>
     SPDX-FileCopyrightText: 2017-2018 Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
     SPDX-License-Identifier: GPL-2.0-or-later
@@ -14,12 +14,11 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
-#include <QRegExp>
+#include <QDebug>
+#include <QIcon>
+#include <QPainter>
 #include <QPixmap>
 #include <QPixmapCache>
-#include <QPainter>
-#include <QIcon>
-#include <QDebug>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
@@ -29,17 +28,17 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include "mymoneyutils.h"
+#include "icons.h"
 #include "mymoneyexception.h"
-#include "mymoneysplit.h"
 #include "mymoneyfile.h"
-#include "mymoneysecurity.h"
 #include "mymoneyinstitution.h"
 #include "mymoneypayee.h"
-#include "payeeidentifier/payeeidentifiertyped.h"
+#include "mymoneysecurity.h"
+#include "mymoneysplit.h"
+#include "mymoneyutils.h"
 #include "payeeidentifier/ibanbic/ibanbic.h"
 #include "payeeidentifier/nationalaccount/nationalaccount.h"
-#include "icons/icons.h"
+#include "payeeidentifier/payeeidentifiertyped.h"
 
 using namespace Icons;
 
@@ -55,6 +54,12 @@ MyMoneyAccount::MyMoneyAccount(const QString &id):
 {
 }
 
+MyMoneyAccount::MyMoneyAccount(MyMoneyAccountPrivate* dd, const MyMoneyAccount& other)
+    : MyMoneyObject(*dd, other.id())
+    , MyMoneyKeyValueContainer(other)
+{
+}
+
 MyMoneyAccount::MyMoneyAccount(const MyMoneyAccount& other) :
     MyMoneyObject(*new MyMoneyAccountPrivate(*other.d_func()), other.id()),
     MyMoneyKeyValueContainer(other)
@@ -98,6 +103,7 @@ void MyMoneyAccount::setInstitutionId(const QString& id)
 {
     Q_D(MyMoneyAccount);
     d->m_institution = id;
+    d->clearReferences();
 }
 
 QString MyMoneyAccount::name() const
@@ -182,6 +188,7 @@ void MyMoneyAccount::setParentAccountId(const QString& parent)
 {
     Q_D(MyMoneyAccount);
     d->m_parentAccount = parent;
+    d->clearReferences();
 }
 
 QStringList MyMoneyAccount::accountList() const
@@ -295,6 +302,7 @@ void MyMoneyAccount::setCurrencyId(const QString& id)
 {
     Q_D(MyMoneyAccount);
     d->m_currencyId = id;
+    d->clearReferences();
 }
 
 bool MyMoneyAccount::isAssetLiability() const
@@ -304,7 +312,12 @@ bool MyMoneyAccount::isAssetLiability() const
 
 bool MyMoneyAccount::isIncomeExpense() const
 {
-    return accountGroup() == Account::Type::Income || accountGroup() == Account::Type::Expense;
+    return isIncomeExpense(accountType());
+}
+
+bool MyMoneyAccount::isIncomeExpense(eMyMoney::Account::Type type)
+{
+    return (accountGroup(type) == Account::Type::Income) || (accountGroup(type) == Account::Type::Expense);
 }
 
 bool MyMoneyAccount::isLoan() const
@@ -331,22 +344,32 @@ bool MyMoneyAccount::isLiquidLiability() const
 
 bool MyMoneyAccount::isCostCenterRequired() const
 {
-    return value("CostCenter").toLower() == QLatin1String("yes");
+    return value(QLatin1String("CostCenter"), false);
 }
 
 void MyMoneyAccount::setCostCenterRequired(bool required)
 {
-    if(required) {
-        setValue("CostCenter", "yes");
-    } else {
-        deletePair("CostCenter");
-    }
+    setValue(QLatin1String("CostCenter"), required, false);
 }
 
-bool MyMoneyAccount::hasReferenceTo(const QString& id) const
+bool MyMoneyAccount::isInTaxReports() const
 {
-    Q_D(const MyMoneyAccount);
-    return (id == d->m_institution) || (id == d->m_parentAccount) || (id == d->m_currencyId);
+    return value(QLatin1String("Tax"), false);
+}
+
+void MyMoneyAccount::setIsInTaxReports(bool include)
+{
+    setValue(QLatin1String("Tax"), include, false);
+}
+
+eMyMoney::Account::PayeeCreation MyMoneyAccount::payeeCreation() const
+{
+    return value<eMyMoney::Account::PayeeCreation>(QLatin1String("PayeeCreation"), eMyMoney::Account::PayeeCreation::ApplicationDefault);
+}
+
+void MyMoneyAccount::setPayeeCreation(eMyMoney::Account::PayeeCreation creationMode)
+{
+    setValue(QLatin1String("PayeeCreation"), creationMode, eMyMoney::Account::PayeeCreation::ApplicationDefault);
 }
 
 void MyMoneyAccount::setOnlineBankingSettings(const MyMoneyKeyValueContainer& values)
@@ -363,15 +386,12 @@ MyMoneyKeyValueContainer MyMoneyAccount::onlineBankingSettings() const
 
 void MyMoneyAccount::setClosed(bool closed)
 {
-    if (closed)
-        setValue("mm-closed", "yes");
-    else
-        deletePair("mm-closed");
+    setValue(QLatin1String("mm-closed"), closed, false);
 }
 
 bool MyMoneyAccount::isClosed() const
 {
-    return !(value("mm-closed").isEmpty());
+    return value(QLatin1String("mm-closed"), false);
 }
 
 int MyMoneyAccount::fraction(const MyMoneySecurity& sec) const
@@ -421,54 +441,87 @@ MyMoneyMoney MyMoneyAccount::balance() const
     return d->m_balance;
 }
 
+MyMoneyMoney MyMoneyAccount::postedValue() const
+{
+    Q_D(const MyMoneyAccount);
+    return d->m_postedValue;
+}
+
+MyMoneyMoney MyMoneyAccount::totalPostedValue() const
+{
+    Q_D(const MyMoneyAccount);
+    return d->m_totalPostedValue;
+}
+
+void MyMoneyAccount::adjustBalance(const MyMoneySplit& s, bool reverse)
+{
+    Q_D(MyMoneyAccount);
+    const MyMoneyMoney oldBalance = d->m_balance;
+
+    if (s.action() == MyMoneySplit::actionName(eMyMoney::Split::Action::SplitShares)) {
+        if (reverse)
+            d->m_balance = d->m_balance / s.shares();
+        else
+            d->m_balance = d->m_balance * s.shares();
+    } else {
+        if (reverse)
+            d->m_balance -= s.shares();
+        else
+            d->m_balance += s.shares();
+    }
+}
+
 void MyMoneyAccount::setBalance(const MyMoneyMoney& val)
 {
     Q_D(MyMoneyAccount);
     d->m_balance = val;
 }
 
-QPixmap MyMoneyAccount::accountPixmap(const bool reconcileFlag, const int size) const
+void MyMoneyAccount::setPostedValue(const MyMoneyMoney& val)
 {
-    static const QHash<Account::Type, Icon> accToIco{{Account::Type::Asset, Icon::Asset},
-                                                     {Account::Type::Investment, Icon::Investment},
-                                                     {Account::Type::Stock, Icon::Stock},
-                                                     {Account::Type::MoneyMarket, Icon::Stock},
-                                                     {Account::Type::Checkings, Icon::Checking},
-                                                     {Account::Type::Savings, Icon::Savings},
-                                                     {Account::Type::AssetLoan, Icon::LoanAsset},
-                                                     {Account::Type::Loan, Icon::Loan},
-                                                     {Account::Type::CreditCard, Icon::CreditCard},
-                                                     {Account::Type::Asset, Icon::Asset},
-                                                     {Account::Type::Cash, Icon::Cash},
-                                                     {Account::Type::Income, Icon::Income},
-                                                     {Account::Type::Expense, Icon::Expense},
-                                                     {Account::Type::Equity, Icon::Equity}};
+    Q_D(MyMoneyAccount);
+    d->m_postedValue = val;
+}
 
-    Icon ixIcon = accToIco.value(accountType(), Icon::Liability);
+void MyMoneyAccount::setTotalPostedValue(const MyMoneyMoney& val)
+{
+    Q_D(MyMoneyAccount);
+    d->m_totalPostedValue = val;
+}
 
-    QString kyIcon = accountTypeToString(accountType()) + QString::number(size);
-    QPixmap pxIcon;
-
-    if (!QPixmapCache::find(kyIcon, &pxIcon)) {
-        pxIcon = Icons::get(ixIcon).pixmap(size); // Qt::AA_UseHighDpiPixmaps (in Qt 5.7) doesn't return highdpi pixmap
-        QPixmapCache::insert(kyIcon, pxIcon);
+QIcon MyMoneyAccount::accountIcon() const
+{
+    switch (accountType()) {
+    case Account::Type::Checkings:
+        return Icons::get(isClosed() ? Icons::Icon::CheckingClosed : Icons::Icon::Checking);
+    case Account::Type::Savings:
+        return Icons::get(isClosed() ? Icons::Icon::SavingsClosed : Icons::Icon::Savings);
+    case Account::Type::Cash:
+        return Icons::get(isClosed() ? Icons::Icon::CashClosed : Icons::Icon::Cash);
+    case Account::Type::CreditCard:
+        return Icons::get(isClosed() ? Icons::Icon::CreditCardClosed : Icons::Icon::CreditCard);
+    case Account::Type::Loan:
+        return Icons::get(isClosed() ? Icons::Icon::LoanClosed : Icons::Icon::Loan);
+    case Account::Type::Investment:
+    case Account::Type::MoneyMarket:
+        return Icons::get(isClosed()? Icons::Icon::InvestmentClosed : Icons::Icon::Investment);
+    case Account::Type::Asset:
+        return Icons::get(isClosed() ? Icons::Icon::AssetClosed : Icons::Icon::Asset);
+    case Account::Type::Liability:
+        return Icons::get(isClosed() ? Icons::Icon::LiabilityClosed : Icons::Icon::Liability);
+    case Account::Type::Income:
+        return Icons::get(Icons::Icon::Income);
+    case Account::Type::Expense:
+        return Icons::get(Icons::Icon::Expense);
+    case Account::Type::AssetLoan:
+        return Icons::get(isClosed() ? Icons::Icon::AssetLoanClosed : Icons::Icon::AssetLoan);
+    case Account::Type::Stock:
+        return Icons::get(isClosed() ? Icons::Icon::SecurityClosed : Icons::Icon::Security);
+    case Account::Type::Equity:
+        return Icons::get(Icons::Icon::Equity);
+    default:
+        return Icons::get(isClosed() ? Icons::Icon::BankAccountClosed : Icons::Icon::BankAccount);
     }
-
-    if (isClosed())
-        ixIcon = Icon::AccountClosed;
-    else if (reconcileFlag)
-        ixIcon = Icon::Reconciled;
-    else if (hasOnlineMapping())
-        ixIcon = Icon::Download;
-    else
-        return pxIcon;
-
-    QPixmap pxOverlay = Icons::get(ixIcon).pixmap(size);
-    QPainter pxPainter(&pxIcon);
-    const QSize szIcon = pxIcon.size();
-    pxPainter.drawPixmap(szIcon.width() / 2, szIcon.height() / 2,
-                         szIcon.width() / 2, szIcon.height() / 2, pxOverlay);
-    return pxIcon;
 }
 
 QString MyMoneyAccount::accountTypeToString(const Account::Type accountType)
@@ -518,32 +571,38 @@ bool MyMoneyAccount::addReconciliation(const QDate& date, const MyMoneyMoney& am
     reconciliationHistory();
 
     d->m_reconciliationHistory[date] = amount;
+
+    /// @todo remove old reconciliation history storage method
+    /// once we don't need to write the KVP entry anymore
+    /// this whole block can be removed
     QString history, sep;
     QMap<QDate, MyMoneyMoney>::const_iterator it;
-    for (it = d->m_reconciliationHistory.constBegin();
-            it != d->m_reconciliationHistory.constEnd();
-            ++it) {
-
+    for (it = d->m_reconciliationHistory.cbegin(); it != d->m_reconciliationHistory.cend(); ++it) {
         history += QString("%1%2:%3").arg(sep,
                                           it.key().toString(Qt::ISODate),
                                           (*it).toString());
         sep = QLatin1Char(';');
     }
-    setValue("reconciliationHistory", history);
+    setValue(QLatin1String("reconciliationHistory"), history);
+
     return true;
 }
 
+/// @todo remove old reconciliation history storage method
+/// once we don't need to read the history from the KVP entry anymore
+/// this whole method can be removed
 QMap<QDate, MyMoneyMoney> MyMoneyAccount::reconciliationHistory()
 {
     Q_D(MyMoneyAccount);
+
     // check if the internal history member is already loaded
     if (d->m_reconciliationHistory.count() == 0
             && !value("reconciliationHistory").isEmpty()) {
-        QStringList entries = value("reconciliationHistory").split(';');
-        foreach (const QString& entry, entries) {
-            QStringList parts = entry.split(':');
+        const QStringList entries = value("reconciliationHistory").split(';');
+        for (const auto& entry : qAsConst(entries)) {
+            const auto parts = entry.split(':');
             if (parts.count() == 2) {
-                QDate date = QDate::fromString(parts[0], Qt::ISODate);
+                const auto date = QDate::fromString(parts[0], Qt::ISODate);
                 MyMoneyMoney amount(parts[1]);
                 if (parts.count() == 2 && date.isValid()) {
                     d->m_reconciliationHistory[date] = amount;
@@ -553,7 +612,12 @@ QMap<QDate, MyMoneyMoney> MyMoneyAccount::reconciliationHistory()
             }
         }
     }
+    return d->m_reconciliationHistory;
+}
 
+QMap<QDate, MyMoneyMoney> MyMoneyAccount::reconciliationHistory() const
+{
+    Q_D(const MyMoneyAccount);
     return d->m_reconciliationHistory;
 }
 
@@ -581,7 +645,7 @@ QList< payeeIdentifier > MyMoneyAccount::payeeIdentifiers() const
     if (!number().isEmpty()) {
         payeeIdentifierTyped<payeeIdentifiers::nationalAccount> national(new payeeIdentifiers::nationalAccount);
         national->setAccountNumber(number());
-        national->setBankCode(file->institution(institutionId()).sortcode());
+        national->setBankCode(file->institution(institutionId()).bankcode());
         if (file->user().state().length() == 2)
             national->setCountry(file->user().state());
         national->setOwnerName(file->user().name());
@@ -605,6 +669,7 @@ QString MyMoneyAccount::stdAccName(eMyMoney::Account::Standard stdAccID)
         {eMyMoney::Account::Standard::Expense,   QStringLiteral("AStd::Expense")},
         {eMyMoney::Account::Standard::Income,    QStringLiteral("AStd::Income")},
         {eMyMoney::Account::Standard::Equity,    QStringLiteral("AStd::Equity")},
+        {eMyMoney::Account::Standard::Favorite,  QStringLiteral("AStd::Favorite")},
     };
     return stdAccNames.value(stdAccID);
 }
@@ -613,3 +678,36 @@ QString MyMoneyAccount::accountSeparator()
 {
     return QStringLiteral(":");
 }
+
+void MyMoneyAccount::setBudgetAccountType(eMyMoney::Account::Type type)
+{
+    if ((type != eMyMoney::Account::Type::Income) && (type != eMyMoney::Account::Type::Expense)) {
+        type = eMyMoney::Account::Type::Unknown;
+    }
+    setValue(QLatin1String("budgetAccountType"), type, eMyMoney::Account::Type::Unknown);
+}
+
+eMyMoney::Account::Type MyMoneyAccount::budgetAccountType() const
+{
+    return value<eMyMoney::Account::Type>(QLatin1String("budgetAccountType"), accountType());
+}
+
+MyMoneyMoney MyMoneyAccount::balanceFactor(eMyMoney::Account::Type accountType)
+{
+    switch (accountGroup(accountType)) {
+    case eMyMoney::Account::Type::Expense:
+    case eMyMoney::Account::Type::Equity:
+    case eMyMoney::Account::Type::Liability:
+        return MyMoneyMoney::MINUS_ONE;
+
+    default:
+        break;
+    }
+    return MyMoneyMoney::ONE;
+}
+
+MyMoneyMoney MyMoneyAccount::balanceFactor() const
+{
+    Q_D(const MyMoneyAccount);
+    return balanceFactor(d->m_accountType);
+}
