@@ -2,7 +2,7 @@
     SPDX-FileCopyrightText: 2000-2002 Michael Edwardes <mte@users.sourceforge.net>
     SPDX-FileCopyrightText: 2001 Felix Rodriguez <frodriguez@users.sourceforge.net>
     SPDX-FileCopyrightText: 2002-2003 Kevin Tambascio <ktambascio@users.sourceforge.net>
-    SPDX-FileCopyrightText: 2006-2017 Thomas Baumgart <tbaumgart@kde.org>
+    SPDX-FileCopyrightText: 2006-2019 Thomas Baumgart <tbaumgart@kde.org>
     SPDX-FileCopyrightText: 2006 Ace Jones <acejones@users.sourceforge.net>
     SPDX-FileCopyrightText: 2017-2018 Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
     SPDX-License-Identifier: GPL-2.0-or-later
@@ -19,24 +19,27 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
+#include "kmm_mymoney_export.h"
 #include "mymoneykeyvaluecontainer.h"
 #include "mymoneyobject.h"
-#include "kmm_mymoney_export.h"
 #include "mymoneyunittestable.h"
 
 class QString;
 class QDate;
-class QPixmap;
-class MyMoneySecurity;
 class MyMoneyMoney;
+class MyMoneySecurity;
 class MyMoneySplit;
 class payeeIdentifier;
 namespace eMyMoney {
 namespace Account {
 enum class Type;
 enum class Standard;
+enum class PayeeCreation;
 }
 }
+namespace Icons {
+enum class Icon;
+}
 template <class T> class payeeIdentifierTyped;
 
 /**
@@ -75,12 +78,17 @@ template <class T> class payeeIdentifierTyped;
 **/
 
 class MyMoneyAccountPrivate;
+class MyMoneyAccountLoan;
 class KMM_MYMONEY_EXPORT MyMoneyAccount : public MyMoneyObject, public MyMoneyKeyValueContainer /*, public MyMoneyPayeeIdentifierContainer */
 {
     Q_DECLARE_PRIVATE_D(MyMoneyObject::d_ptr, MyMoneyAccount)
+    friend MyMoneyAccountLoan;
 
     KMM_MYMONEY_UNIT_TESTABLE
 
+protected:
+    explicit MyMoneyAccount(MyMoneyAccountPrivate* dd, const MyMoneyAccount& other);
+
 public:
     /**
       * This is the constructor for a new empty account
@@ -115,8 +123,8 @@ public:
     bool operator == (const MyMoneyAccount &) const;
 
     /**
-      * This converts the account type into one of the four
-      * major account types liability, asset, expense or income.
+      * This converts the account type into one of the five
+      * major account types liability, asset, expense, income or equity.
       *
       * The current assignment is as follows:
       *
@@ -143,6 +151,9 @@ public:
       * - Expense
       *   - Expense
       *
+      * - Equity
+      *   - Equity
+      *
       * @return accountTypeE of major account type
       */
     eMyMoney::Account::Type accountGroup() const;
@@ -366,23 +377,39 @@ public:
       */
     void setCurrencyId(const QString& id);
 
-    /**
-      * This method checks if a reference to the given object exists. It returns,
-      * a @p true if the object is referencing the one requested by the
-      * parameter @p id. If it does not, this method returns @p false.
-      *
-      * @param id id of the object to be checked for references
-      * @retval true This object references object with id @p id.
-      * @retval false This object does not reference the object with id @p id.
-      */
-    bool hasReferenceTo(const QString& id) const override;
-
     /**
       * This member returns the balance of this account based on
       * all transactions stored in the journal.
       */
     MyMoneyMoney balance() const;
 
+    /**
+     * This member returns the value of this account based on
+     * all transactions stored in the journal and currently available
+     * price information.
+     */
+    MyMoneyMoney postedValue() const;
+
+    /**
+     * This member returns the value of this account and all its
+     * subaccounts based on all transactions stored in the journal
+     * and currently available price information.
+     */
+    MyMoneyMoney totalPostedValue() const;
+
+    /**
+      * This method adjusts the balance of this account
+      * according to the difference contained in the split @p s.
+      * If the s.action() is MyMoneySplit::actionName(eMyMoney::Split::Action::SplitShares) then
+      * the balance will be adjusted accordingly.
+      *
+      * @param s const reference to MyMoneySplit object containing the
+      *             value to be added/subtracted to/from the balance
+      * @param reverse add (false) or subtract (true) the shares contained in the split.
+      *          It also affects the balance for share splits in the opposite direction.
+      */
+    void adjustBalance(const MyMoneySplit& s, bool reverse = false);
+
     /**
       * This method sets the balance of this account
       * according to the value provided by @p val.
@@ -392,6 +419,24 @@ public:
       */
     void setBalance(const MyMoneyMoney& val);
 
+    /**
+     * This method sets the total value of this account to @p val. This
+     * does not include the subaccounts
+     *
+     * @param val const reference to MyMoneyMoney object containing the
+     *             value to be assigned to the total posted value
+     */
+    void setPostedValue(const MyMoneyMoney& val);
+
+    /**
+     * This method sets the total value of this account and all its
+     * subaccounts to @p val.
+     *
+     * @param val const reference to MyMoneyMoney object containing the
+     *             value to be assigned to the total posted value
+     */
+    void setTotalPostedValue(const MyMoneyMoney& val);
+
     /**
       * This method sets the kvp's for online banking with this account
       *
@@ -471,6 +516,12 @@ public:
       */
     bool isIncomeExpense() const;
 
+    /**
+     * Returns @c true if the @a type is either Income or Expense.
+     * In all other cases it returns @c false.
+     */
+    static bool isIncomeExpense(eMyMoney::Account::Type type);
+
     /**
       * This method returns @a true if the account type is
       * either Asset or Liability
@@ -524,6 +575,27 @@ public:
      */
     void setCostCenterRequired(bool required = true);
 
+    /**
+     * This method returns true if a category is to be included in tax reports
+     */
+    bool isInTaxReports() const;
+
+    /**
+     * This method allows to control if this account (category) is to be included
+     * in tax reports. It is if @a include is @c true (the default).
+     */
+    void setIsInTaxReports(bool include = true);
+
+    /**
+     * set the mode payee creation should be handled for this account during import
+     */
+    void setPayeeCreation(eMyMoney::Account::PayeeCreation creationMode);
+
+    /**
+     * return how payee creation should be handled for this account during import
+     */
+    eMyMoney::Account::PayeeCreation payeeCreation() const;
+
     /**
      * This method returns a name that has a brokerage suffix of
      * the current name. It only works on investment accounts and
@@ -532,12 +604,9 @@ public:
     QString brokerageName() const;
 
     /**
-     * @param reconcileFlag if set to @a true a reconcile overlay will be
-     *                      added to the pixmap returned
-     * @param size is a hint for the size of the icon
-     * @return a pixmap using DesktopIcon for the account type
+     * @return an QIcon for the account.
      */
-    QPixmap accountPixmap(const bool reconcileFlag = false, const int size = 64) const;
+    QIcon accountIcon() const;
 
     /**
      * This method is used to convert the internal representation of
@@ -560,20 +629,45 @@ public:
     bool addReconciliation(const QDate& date, const MyMoneyMoney& amount);
 
     /**
-      * @return QMap with the reconciliation history for the account
-      *
-      * @sa addReconciliation()
-      */
+     * Return the information about stored reconciliations of the
+     * account.
+     *
+     * @return QMap with the reconciliation history for the account
+     *
+     * @sa addReconciliation()
+     *
+     * @note If the internal map is empty, the values will be
+     * loaded from the KVP with key "reconciliationHistory".
+     *
+     * @deprecated Use the const version instead.
+     */
     QMap<QDate, MyMoneyMoney> reconciliationHistory();
 
+    /**
+     * Return the information about stored reconciliations of the
+     * account.
+     *
+     * @return QMap with the reconciliation history for the account
+     *
+     * @sa addReconciliation()
+     */
+    QMap<QDate, MyMoneyMoney> reconciliationHistory() const;
+
+    MyMoneyMoney balanceFactor() const;
+
     /**
       * @return @c true if account has an online mapping, @c false otherwise
       */
     bool hasOnlineMapping() const;
 
+    eMyMoney::Account::Type budgetAccountType() const;
+    void setBudgetAccountType(eMyMoney::Account::Type type);
+
     static QString stdAccName(eMyMoney::Account::Standard stdAccID);
     static QString accountSeparator();
 
+    static MyMoneyMoney balanceFactor(eMyMoney::Account::Type accountType);
+
     QDataStream &operator<<(const MyMoneyAccount &);
     QDataStream &operator>>(MyMoneyAccount &);
 };
@@ -602,5 +696,6 @@ inline MyMoneyAccount & MyMoneyAccount::operator=(MyMoneyAccount other) // krazy
  */
 Q_DECLARE_METATYPE(MyMoneyAccount)
 Q_DECLARE_METATYPE(eMyMoney::Account::Type)
+Q_DECLARE_METATYPE(eMyMoney::Account::PayeeCreation)
 
 #endif
