@@ -3,7 +3,7 @@
     SPDX-FileCopyrightText: 2001-2002 Felix Rodriguez <frodriguez@users.sourceforge.net>
     SPDX-FileCopyrightText: 2002-2004 Kevin Tambascio <ktambascio@users.sourceforge.net>
     SPDX-FileCopyrightText: 2004-2005 Ace Jones <acejones@users.sourceforge.net>
-    SPDX-FileCopyrightText: 2006-2018 Thomas Baumgart <tbaumgart@kde.org>
+    SPDX-FileCopyrightText: 2006-2020 Thomas Baumgart <tbaumgart@kde.org>
     SPDX-FileCopyrightText: 2006 Darren Gould <darren_gould@gmx.de>
     SPDX-FileCopyrightText: 2017-2018 Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
     SPDX-License-Identifier: GPL-2.0-or-later
@@ -15,11 +15,13 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
+#include "qcontainerfwd.h"
 #include <QObject>
 
 // ----------------------------------------------------------------------------
 // Project Includes
 #include "kmm_mymoney_export.h"
+#include "kmmset.h"
 #include "mymoneyunittestable.h"
 
 /**
@@ -113,8 +115,8 @@
   */
 template <class Key, class T> class QMap;
 class QString;
-class QStringList;
 class QBitArray;
+class QUndoStack;
 class MyMoneyStorageMgr;
 class MyMoneyCostCenter;
 class MyMoneyAccount;
@@ -136,6 +138,30 @@ class MyMoneyTransaction;
 class MyMoneyTransactionFilter;
 class onlineJob;
 
+// the models
+class PayeesModel;
+class CostCenterModel;
+class SchedulesModel;
+class TagsModel;
+class SecuritiesModel;
+class BudgetsModel;
+class AccountsModel;
+class InstitutionsModel;
+class JournalModel;
+class PriceModel;
+class ParametersModel;
+class OnlineJobsModel;
+class ReportsModel;
+class MyMoneyModelBase;
+
+// non permanent models
+class ReconciliationModel;
+class SpecialDatesModel;
+class SchedulesJournalModel;
+class StatusModel;
+class KDescendantsProxyModel;
+/// @note add new models here
+
 namespace eMyMoney {
 namespace Account {
 enum class Type;
@@ -153,6 +179,7 @@ enum class State;
 }
 }
 
+
 class KMM_MYMONEY_EXPORT MyMoneyFile : public QObject
 {
     Q_OBJECT
@@ -161,6 +188,13 @@ class KMM_MYMONEY_EXPORT MyMoneyFile : public QObject
 public:
     friend class MyMoneyNotifier;
 
+    typedef enum {
+        CreationDate,
+        LastModificationDate,
+        FileFixVersion,
+        UserID,
+    } FixedKey;
+
     /**
       * This is the function to access the MyMoneyFile object.
       * It returns a pointer to the single instance of the object.
@@ -172,14 +206,28 @@ public:
       */
     ~MyMoneyFile();
 
+    static MyMoneyModelBase* baseModel();
+
     /**
-      * @deprecated This is a convenience constructor. Do not use it anymore.
-      * It will be deprecated in a future version of the engine.
-      *
-      * @param storage pointer to object that implements the MyMoneyStorageMgr
-      *                interface.
-      */
-    explicit MyMoneyFile(MyMoneyStorageMgr *storage);
+     * This returns the currently implemented highest fix version
+     */
+    constexpr static int availableFixVersion() {
+        return 5;
+    }
+
+    /**
+     * returns the current file fix version of the loaded data
+     *
+     * @sa setFileFixVersion(), availableFixVersion()
+     */
+    int fileFixVersion() const;
+
+    /**
+     * Sets the file fix level to @a version
+     *
+     * @sa fileFixVersion(), availableFixVersion()
+     */
+    void setFileFixVersion(int version);
 
     // general get functions
     MyMoneyPayee user() const;
@@ -234,17 +282,30 @@ public:
       * @return const pointer to the current attached storage object.
       *         If no object is attached, returns 0.
       */
-    MyMoneyStorageMgr* storage() const;
+    // MyMoneyStorageMgr* storage() const;
 
     /**
-      * This method must be called before any single change or a series of changes
-      * in the underlying storage area is performed.
-      * Once all changes are complete (i.e. the transaction is completed),
-      * commitTransaction() must be called to finalize all changes. If an error occurs
-      * during the processing of the changes call rollbackTransaction() to undo the
-      * changes done so far.
-      */
-    void startTransaction();
+     * This method clears all data in all storage models
+     */
+    void unload();
+
+    /**
+     * This method must be called before any single change or a series of changes
+     * in the underlying storage area is performed.
+     * Once all changes are complete (i.e. the transaction is completed),
+     * commitTransaction() must be called to finalize all changes. If an error occurs
+     * during the processing of the changes call rollbackTransaction() to undo the
+     * changes done so far.
+     *
+     * The @a undoActionText will be attached to the transaction
+     *
+     * If @a journalBlocking is true (the default), signals sent from the concatenated
+     * journal are suppressed and once the transaction is finished, a complete reset
+     * of the model will be performed. Where this is not required, you may set
+     * @a journalBlocking to false to maintain signal emission and preventing the
+     * complete model reset.
+     */
+    void startTransaction(const QString& undoActionText = QString(), bool journalBlocking = true);
 
     /**
       * This method returns whether a transaction has been started (@a true)
@@ -362,7 +423,7 @@ public:
       * flag after a failed upload to a server when the save operation
       * to a local temp file was OK.
       */
-    void setDirty() const;
+    void setDirty(bool dirty = true) const;
 
     /**
       * Adds an institution to the file-global institution pool. A
@@ -480,15 +541,12 @@ public:
     bool isTransfer(const MyMoneyTransaction& t) const;
 
     /**
-      * This method is used to set the name for the specified standard account
-      * within the storage area. An exception will be thrown, if an error
-      * occurs
-      *
-      * @param id QString reference to one of the standard accounts.
-      * @param name QString reference to the name to be set
-      *
-      */
-    void setAccountName(const QString& id, const QString& name) const;
+     * Return @a true if transaction @a t is an investment transaction.
+     * A transaction is called an investment transaction if at least
+     * one split references an investment account and one split
+     * references an account denoted in an investment security.
+     */
+    bool isInvestmentTransaction(const MyMoneyTransaction& t) const;
 
     /**
       * Deletes an existing account from the file global account pool
@@ -579,21 +637,20 @@ public:
       */
     MyMoneyTransaction transaction(const QString& account, const int idx) const;
 
-    /**
-      * This method is used to pull a list of transactions from the file
-      * global transaction pool. It returns all those transactions
-      * that match the filter passed as argument. If the filter is empty,
-      * the whole journal will be returned.
-      * The list returned is sorted according to the transactions posting date.
-      * If more than one transaction exists for the same date, the order among
-      * them is undefined.
-      *
-      * @param filter MyMoneyTransactionFilter object with the match criteria
-      *
-      * @return set of transactions in form of a QList<MyMoneyTransaction>
-      */
-    QList<MyMoneyTransaction> transactionList(MyMoneyTransactionFilter& filter) const;
+    QStringList journalEntryIds(MyMoneyTransactionFilter filter) const;
 
+    /**
+     * This method is used to pull a list of transactions from the file
+     * global transaction pool. It returns all those transactions
+     * that match the filter passed as argument. If the filter is empty,
+     * the whole journal will be returned.
+     * The list returned is sorted according to the transactions posting date.
+     * If more than one transaction exists for the same date, the order among
+     * them is undefined.
+     *
+     * @param list Reference to QList receiving the matching transactions
+     * @param filter MyMoneyTransactionFilter object with the match criteria
+     */
     void transactionList(QList<MyMoneyTransaction>& list, MyMoneyTransactionFilter& filter) const;
 
     void transactionList(QList<QPair<MyMoneyTransaction, MyMoneySplit> >& list, MyMoneyTransactionFilter& filter) const;
@@ -651,26 +708,14 @@ public:
       * This method returns the number of transactions currently known to file
       * in the range 0..MAXUINT
       *
-      * @param account QString reference to account id. If account is empty
-      +                all transactions (the journal) will be counted. If account
-      *                is not empty it returns the number of transactions
-      *                that have splits in this account.
+      * @param accountId QString reference to account id. If account is empty
+      +                  all transactions (the journal) will be counted. If account
+      *                  is not empty it returns the number of transactions
+      *                  that have splits in this account.
       *
       * @return number of transactions in journal/account
       */
-    unsigned int transactionCount(const QString& account) const;
-    unsigned int transactionCount() const;
-
-    /**
-      * This method returns a QMap filled with the number of transactions
-      * per account. The account id serves as index into the map. If one
-      * needs to have all transactionCounts() for many accounts, this method
-      * is faster than calling transactionCount(const QString& account) many
-      * times.
-      *
-      * @return QMap with numbers of transactions per account
-      */
-    QMap<QString, unsigned long> transactionCountMap() const;
+    unsigned int transactionCount(const QString& accountId = QString()) const;
 
     /**
       * This method returns the number of institutions currently known to file
@@ -858,6 +903,108 @@ public:
       */
     QList<MyMoneyPayee> payeeList() const;
 
+    /**
+     * The payees model instance
+     */
+    PayeesModel* payeesModel() const;
+
+    /**
+     * The costcenter model instance
+     */
+    CostCenterModel* costCenterModel() const;
+
+    /**
+     * The schedules model instance
+     */
+    SchedulesModel* schedulesModel() const;
+
+    /**
+     * The tags model instance
+     */
+    TagsModel* tagsModel() const;
+
+    /**
+     * The securities model instance
+     */
+    SecuritiesModel* securitiesModel() const;
+
+    /**
+     * The currencies model instance
+     */
+    SecuritiesModel* currenciesModel() const;
+
+    /**
+     * The budgets model instance
+     */
+    BudgetsModel* budgetsModel() const;
+
+    /**
+     * The accounts model instance
+     */
+    AccountsModel* accountsModel() const;
+
+    /**
+     * The accountsModel as a flat model
+     */
+    KDescendantsProxyModel* flatAccountsModel() const;
+
+    /**
+     * The institutions model instance
+     */
+    InstitutionsModel* institutionsModel() const;
+
+    /**
+     * The journal model instance
+     */
+    JournalModel* journalModel() const;
+
+    /**
+     * The price model instance
+     */
+    PriceModel* priceModel() const;
+
+    /**
+     * The parameters model instance
+     */
+    ParametersModel* parametersModel() const;
+
+    /**
+     * The online jobs model instance
+     */
+    OnlineJobsModel* onlineJobsModel() const;
+
+    /**
+     * The reports model instance
+     */
+    ReportsModel* reportsModel() const;
+
+    /**
+     * The user model instance
+     */
+    PayeesModel* userModel() const;
+
+    /**
+     * The user model instance
+     */
+    SpecialDatesModel* specialDatesModel() const;
+
+    /**
+     * The scheduled transactions as journal entries
+     */
+    SchedulesJournalModel* schedulesJournalModel() const;
+
+    /*
+     * The transaction status model
+     */
+    StatusModel* statusModel() const;
+
+    /*
+     * The reconciliation entry model
+     */
+    ReconciliationModel* reconciliationModel() const;
+
+    /// @note add new models here
+
     /**
       * This method is used to create a new tag
       *
@@ -952,11 +1099,11 @@ public:
      * In case the storage object does not have an id yet, a new one will be
      * assigned.
      *
-     * @return QString containing the value
+     * @return QUuid containing the value
      *
      * An exception is thrown if no storage object is attached.
      */
-    QString storageId();
+    QUuid storageId();
 
     /**
       * This method is used to delete a key-value-pair from the
@@ -981,7 +1128,7 @@ public:
 
     /**
       * This method is used to modify an existing MyMoneySchedule
-      * object. Therefor, the id attribute of the object must be set.
+      * object. Therefore, the id attribute of the object must be set.
       *
       * An exception will be thrown upon erroneous situations.
       *
@@ -1046,6 +1193,16 @@ public:
     QList<MyMoneySchedule> scheduleList(const QString& accountId) const;
     QList<MyMoneySchedule> scheduleList() const;
 
+    /**
+     * Returns the transaction for @a schedule. In case of a loan payment the
+     * transaction will be modified by calculateAutoLoan().
+     * The ID of the transaction as well as the entryDate will be reset.
+     *
+     * @returns adjusted transaction
+     */
+    MyMoneyTransaction scheduledTransaction(const MyMoneySchedule& schedule);
+
+
     QStringList consistencyCheck();
 
     /**
@@ -1474,6 +1631,17 @@ public:
     bool isReferenced(const MyMoneyObject& obj, const QBitArray& skipCheck) const;
     bool isReferenced(const MyMoneyObject& obj) const;
 
+    bool isReferenced(const QString& id, const QBitArray& skipCheck) const;
+    bool isReferenced(const QString& id) const;
+
+    /**
+     * This method returns the set of ids that are referenced by
+     * other objects in all of the data objects.
+     *
+     * @returns KMMStringSet of referenced objects
+     */
+    KMMStringSet referencedObjects() const;
+
     /**
       * Returns true if any of the accounts referenced by the splits
       * of transaction @a t is closed.
@@ -1590,13 +1758,42 @@ public:
      */
     void removeOnlineJob(const QStringList onlineJobIds);
 
+    /**
+     * mark all models as clean
+     */
+    void fileSaved();
+
+    /**
+     * This returns the string for specific parameters
+     */
+    const QString& fixedKey(FixedKey key) const;
+
+    QUndoStack* undoStack() const;
+
+    bool hasValidId (const MyMoneyAccount& acc) const;
+    bool hasValidId (const MyMoneyPayee& payee) const;
+
 protected:
     /**
       * This is the constructor for a new empty file description
       */
     MyMoneyFile();
 
+    /**
+     * Used by accountsModel to enclose a reparent operation
+     * inside a MyMoneyFileTransaction.
+     */
+    void reparentAccountByIds(const QString& accountId, const QString& newParentId);
+
+protected Q_SLOTS:
+    void finalizeFileOpen();
+
+    void reloadSpecialDates();
+
 Q_SIGNALS:
+    void storageTransactionStarted(bool journalBlocking);
+    void storageTransactionEnded(bool journalBlocking);
+
     /**
      * This signal is emitted when a transaction has been committed and
      * the notifications are about to be sent out.
@@ -1654,9 +1851,20 @@ Q_SIGNALS:
       */
     void valueChanged(const MyMoneyAccount& acc);
 
-private:
-    static MyMoneyFile file;
+    /**
+     * This signal is emitted once all data of a new backend is loaded.
+     */
+    void modelsLoaded();
 
+    /**
+     * This signal is emitted once all data of a new backend is ready to be used.
+     * The difference to modelsLoaded() is that some internal fixes can be applied
+     * in the meantime. Right before this signal is sent out, the dirty flag of all
+     * models will be reset.
+     */
+    void modelsReadyToUse();
+
+private:
     MyMoneyFile& operator=(MyMoneyFile&); // not allowed for singleton
     MyMoneyFile(const MyMoneyFile&);      // not allowed for singleton
 
@@ -1697,6 +1905,7 @@ class KMM_MYMONEY_EXPORT MyMoneyFileTransaction
 
 public:
     MyMoneyFileTransaction();
+    MyMoneyFileTransaction(const QString& undoActionText, bool journalBlocking);
     ~MyMoneyFileTransaction();
 
     /**
