@@ -35,8 +35,6 @@
 #include "mymoneyexception.h"
 #include "mymoneyenums.h"
 
-enum class eForecastMethod {Scheduled = 0, Historic = 1 };
-
 /**
  * daily balances of an account
  */
@@ -52,23 +50,23 @@ class MyMoneyForecastPrivate
     Q_DECLARE_PUBLIC(MyMoneyForecast)
 
 public:
-    explicit MyMoneyForecastPrivate(MyMoneyForecast *qq) :
-        q_ptr(qq),
-        m_accountsCycle(30),
-        m_forecastCycles(3),
-        m_forecastDays(90),
-        m_beginForecastDay(0),
-        m_forecastMethod(eForecastMethod::Scheduled),
-        m_historyMethod(1),
-        m_skipOpeningDate(true),
-        m_includeUnusedAccounts(false),
-        m_forecastDone(false),
-        m_includeFutureTransactions(true),
-        m_includeScheduledTransactions(true)
+    explicit MyMoneyForecastPrivate(MyMoneyForecast* qq)
+        : q_ptr(qq)
+        , m_accountsCycle(30)
+        , m_forecastCycles(3)
+        , m_forecastDays(90)
+        , m_beginForecastDay(0)
+        , m_forecastMethod(MyMoneyForecast::eForecastMethod::Scheduled)
+        , m_historyMethod(1)
+        , m_skipOpeningDate(true)
+        , m_includeUnusedAccounts(false)
+        , m_forecastDone(false)
+        , m_includeFutureTransactions(true)
+        , m_includeScheduledTransactions(true)
     {
     }
 
-    eForecastMethod forecastMethod() const
+    MyMoneyForecast::eForecastMethod forecastMethod() const
     {
         return m_forecastMethod;
     }
@@ -109,8 +107,8 @@ public:
         calculateAccountTrendList();
 
         //Calculate account daily balances
-        QSet<QString>::ConstIterator it_n;
-        for (it_n = m_forecastAccounts.constBegin(); it_n != m_forecastAccounts.constEnd(); ++it_n) {
+        KMMStringSet::const_iterator it_n;
+        for (it_n = m_forecastAccounts.cbegin(); it_n != m_forecastAccounts.cend(); ++it_n) {
             auto acc = file->account(*it_n);
 
             //set the starting balance of the account
@@ -162,8 +160,8 @@ public:
         auto file = MyMoneyFile::instance();
 
         //Calculate account monthly balances
-        QSet<QString>::ConstIterator it_n;
-        for (it_n = m_forecastAccounts.constBegin(); it_n != m_forecastAccounts.constEnd(); ++it_n) {
+        KMMStringSet::const_iterator it_n;
+        for (it_n = m_forecastAccounts.cbegin(); it_n != m_forecastAccounts.cend(); ++it_n) {
             auto acc = file->account(*it_n);
 
             for (QDate f_date = q->forecastStartDate(); f_date <= q->forecastEndDate();) {
@@ -188,8 +186,8 @@ public:
         auto file = MyMoneyFile::instance();
 
         //Calculate account monthly balances
-        QSet<QString>::ConstIterator it_n;
-        for (it_n = m_forecastAccounts.constBegin(); it_n != m_forecastAccounts.constEnd(); ++it_n) {
+        KMMStringSet::const_iterator it_n;
+        for (it_n = m_forecastAccounts.cbegin(); it_n != m_forecastAccounts.cend(); ++it_n) {
             auto acc = file->account(*it_n);
 
             for (QDate f_date = q->forecastStartDate(); f_date <= q->forecastEndDate(); f_date = f_date.addDays(1)) {
@@ -223,8 +221,8 @@ public:
             purgeForecastAccountsList(m_accountList);
 
         //adjust value of investments to deep currency
-        QSet<QString>::ConstIterator it_n;
-        for (it_n = m_forecastAccounts.constBegin(); it_n != m_forecastAccounts.constEnd(); ++it_n) {
+        KMMStringSet::const_iterator it_n;
+        for (it_n = m_forecastAccounts.cbegin(); it_n != m_forecastAccounts.cend(); ++it_n) {
             auto acc = file->account(*it_n);
 
             if (acc.isInvest()) {
@@ -265,8 +263,11 @@ public:
         filter.setDateFilter(q->forecastStartDate(), q->forecastEndDate());
         filter.setReportAllSplits(false);
 
-        foreach (const auto transaction, file->transactionList(filter)) {
-            foreach (const auto split, transaction.splits()) {
+        QList<MyMoneyTransaction> list;
+        file->transactionList(list, filter);
+        for (const auto& transaction : qAsConst(list)) {
+            const auto splits = transaction.splits();
+            for (const auto& split : qAsConst(splits)) {
                 if (!split.shares().isZero()) {
                     auto acc = file->account(split.accountId());
                     if (q->isForecastAccount(acc)) {
@@ -292,7 +293,7 @@ public:
         {
             s << "Already present transactions\n";
             QMap<QString, dailyBalances>::Iterator it_a;
-            QSet<QString>::ConstIterator it_n;
+            KMMStringSet::const_iterator it_n;
             for (it_n = m_nameIdx.begin(); it_n != m_nameIdx.end(); ++it_n) {
                 auto acc = file->account(*it_n);
                 it_a = m_accountList.find(*it_n);
@@ -323,7 +324,7 @@ public:
         if (schedule.count() > 0) {
             QList<MyMoneySchedule>::Iterator it;
             do {
-                qSort(schedule);
+                std::sort(schedule.begin(), schedule.end());
                 it = schedule.begin();
                 if (it == schedule.end())
                     break;
@@ -362,7 +363,7 @@ public:
                                 // make sure we have all 'starting balances' so that the autocalc works
                                 QMap<QString, MyMoneyMoney> balanceMap;
 
-                                foreach (const auto split, t.splits()) {
+                                for (const auto& split : qAsConst(t.splits())) {
                                     auto accountFromSplit = file->account(split.accountId());
                                     if (q->isForecastAccount(accountFromSplit)) {
                                         // collect all overdues on the first day
@@ -383,7 +384,7 @@ public:
                                 q->calculateAutoLoan(*it, t, balanceMap);
 
                                 // now add the splits to the balances
-                                foreach (const auto split, t.splits()) {
+                                for (const auto& split : qAsConst(t.splits())) {
                                     auto accountFromSplit = file->account(split.accountId());
                                     if (q->isForecastAccount(accountFromSplit)) {
                                         dailyBalances balance;
@@ -425,7 +426,7 @@ public:
         {
             s << "\n\nAdded scheduled transactions\n";
             QMap<QString, dailyBalances>::Iterator it_a;
-            QSet<QString>::ConstIterator it_n;
+            KMMStringSet::const_iterator it_n;
             for (it_n = m_nameIdx.begin(); it_n != m_nameIdx.end(); ++it_n) {
                 auto acc = file->account(*it_n);
                 it_a = m_accountList.find(*it_n);
@@ -448,8 +449,8 @@ public:
         auto file = MyMoneyFile::instance();
 
         //Calculate account daily balances
-        QSet<QString>::ConstIterator it_n;
-        for (it_n = m_forecastAccounts.constBegin(); it_n != m_forecastAccounts.constEnd(); ++it_n) {
+        KMMStringSet::const_iterator it_n;
+        for (it_n = m_forecastAccounts.cbegin(); it_n != m_forecastAccounts.cend(); ++it_n) {
             auto acc = file->account(*it_n);
 
             //set the starting balance of the account
@@ -492,7 +493,7 @@ public:
         }
 
         //if the method is linear regression, we have to add the opening balance to m_accountListPast
-        if (forecastMethod() == eForecastMethod::Historic && q->historyMethod() == 2) {
+        if (forecastMethod() == MyMoneyForecast::eForecastMethod::Historic && q->historyMethod() == 2) {
             //FIXME workaround for stock opening dates
             QDate openingDate;
             if (acc.accountType() == eMyMoney::Account::Type::Stock) {
@@ -630,8 +631,8 @@ public:
         qint64 totalWeight = 0;
 
         //Calculate account trends
-        QSet<QString>::ConstIterator it_n;
-        for (it_n = m_forecastAccounts.begin(); it_n != m_forecastAccounts.end(); ++it_n) {
+        KMMStringSet::const_iterator it_n;
+        for (it_n = m_forecastAccounts.cbegin(); it_n != m_forecastAccounts.cend(); ++it_n) {
             auto acc = file->account(*it_n);
             m_accountTrendList[acc.id()][0] = MyMoneyMoney(); // for today, the trend is 0
 
@@ -697,8 +698,8 @@ public:
         QList<MyMoneyAccount> accList;
         accList = q->forecastAccountList();
 
-        QList<MyMoneyAccount>::const_iterator accList_t = accList.constBegin();
-        for (; accList_t != accList.constEnd(); ++accList_t) {
+        QList<MyMoneyAccount>::const_iterator accList_t = accList.cbegin();
+        for (; accList_t != accList.cend(); ++accList_t) {
             m_forecastAccounts.insert((*accList_t).id());
         }
     }
@@ -712,8 +713,8 @@ public:
         QList<MyMoneyAccount> accList;
         accList = budgetAccountList();
 
-        QList<MyMoneyAccount>::const_iterator accList_t = accList.constBegin();
-        for (; accList_t != accList.constEnd(); ++accList_t) {
+        QList<MyMoneyAccount>::const_iterator accList_t = accList.cbegin();
+        for (; accList_t != accList.cend(); ++accList_t) {
             m_forecastAccounts.insert((*accList_t).id());
         }
     }
@@ -731,8 +732,11 @@ public:
         filter.setReportAllSplits(false);
 
         //Check past transactions
-        foreach (const auto transaction, file->transactionList(filter)) {
-            foreach (const auto split, transaction.splits()) {
+        QList<MyMoneyTransaction> list;
+        file->transactionList(list, filter);
+        for (const auto& transaction : qAsConst(list)) {
+            const auto splits = transaction.splits();
+            for (const auto& split : qAsConst(splits)) {
                 if (!split.shares().isZero()) {
                     auto acc = file->account(split.accountId());
 
@@ -768,8 +772,8 @@ public:
             purgeForecastAccountsList(m_accountListPast);
 
         //calculate running sum
-        QSet<QString>::ConstIterator it_n;
-        for (it_n = m_forecastAccounts.begin(); it_n != m_forecastAccounts.end(); ++it_n) {
+        KMMStringSet::const_iterator it_n;
+        for (it_n = m_forecastAccounts.cbegin(); it_n != m_forecastAccounts.cend(); ++it_n) {
             auto acc = file->account(*it_n);
             m_accountListPast[acc.id()][q->historyStartDate().addDays(-1)] = file->balance(acc.id(), q->historyStartDate().addDays(-1));
             for (QDate it_date = q->historyStartDate(); it_date <= q->historyEndDate();) {
@@ -779,7 +783,7 @@ public:
         }
 
         //adjust value of investments to deep currency
-        for (it_n = m_forecastAccounts.begin(); it_n != m_forecastAccounts.end(); ++it_n) {
+        for (it_n = m_forecastAccounts.cbegin(); it_n != m_forecastAccounts.cend(); ++it_n) {
             auto acc = file->account(*it_n);
 
             if (acc.isInvest()) {
@@ -865,7 +869,7 @@ public:
      */
     void purgeForecastAccountsList(QMap<QString, dailyBalances>& accountList)
     {
-        m_forecastAccounts.intersect(accountList.keys().toSet());
+        m_forecastAccounts &= KMMStringSet(accountList.keys());
     }
 
     MyMoneyForecast *q_ptr;
@@ -888,7 +892,7 @@ public:
     /**
      * list of forecast account ids.
      */
-    QSet<QString> m_forecastAccounts;
+    KMMStringSet m_forecastAccounts;
 
     /**
      * cycle of accounts in days
@@ -918,7 +922,7 @@ public:
     /**
      * forecast method
      */
-    eForecastMethod m_forecastMethod;
+    MyMoneyForecast::eForecastMethod m_forecastMethod;
 
     /**
      * history method
@@ -1118,8 +1122,11 @@ MyMoneyMoney MyMoneyForecast::calculateAccountTrend(const MyMoneyAccount& acc, q
     filter.setReportAllSplits(false);
 
     //add all transactions for that account
-    foreach (const auto transaction, file->transactionList(filter)) {
-        foreach (const auto split, transaction.splits()) {
+    QList<MyMoneyTransaction> list;
+    file->transactionList(list, filter);
+    for (const auto& transaction : qAsConst(list)) {
+        const auto splits = transaction.splits();
+        for (const auto& split : qAsConst(splits)) {
             if (!split.shares().isZero()) {
                 if (acc.id() == split.accountId()) netIncome += split.value();
             }
@@ -1377,7 +1384,7 @@ void MyMoneyForecast::createBudget(MyMoneyBudget& budget, QDate historyStart, QD
         budget.setBudgetStart(budgetStart);
 
         //go through all the accounts and add them to budget
-        for (auto it_nc = d->m_forecastAccounts.constBegin(); it_nc != d->m_forecastAccounts.constEnd(); ++it_nc) {
+        for (auto it_nc = d->m_forecastAccounts.cbegin(); it_nc != d->m_forecastAccounts.cend(); ++it_nc) {
             auto acc = file->account(*it_nc);
 
             MyMoneyBudget::AccountGroup budgetAcc;
@@ -1578,6 +1585,12 @@ int MyMoneyForecast::historyMethod() const
     return d->m_historyMethod;
 }
 
+MyMoneyForecast::eForecastMethod MyMoneyForecast::forecastMethod() const
+{
+    Q_D(const MyMoneyForecast);
+    return d->m_forecastMethod;
+}
+
 bool MyMoneyForecast::isIncludingUnusedAccounts() const
 {
     Q_D(const MyMoneyForecast);
