@@ -17,13 +17,17 @@
 
 #include "mymoneymoney.h"
 
-#include <stdint.h>
+#include <alkimia/alkversion.h>
 #include <gmpxx.h>
+#include <stdint.h>
 
 // ----------------------------------------------------------------------------
 // QT Includes
 
 #include <QDebug>
+#include <QLocale>
+#include <QRegularExpression>
+#include <QRegularExpressionMatch>
 #include <QString>
 
 // ----------------------------------------------------------------------------
@@ -43,14 +47,16 @@ enum fileVersionE : int {
     FILE_8_BYTE_VALUE,
 };
 
-QChar _thousandSeparator = QLatin1Char(',');
-QChar _decimalSeparator = QLatin1Char('.');
+QString _thousandSeparator = QLatin1String(",");
+QString _decimalSeparator = QLatin1String(".");
 eMyMoney::Money::signPosition _negativeMonetarySignPosition = PreceedQuantityAndSymbol;
 eMyMoney::Money::signPosition _positiveMonetarySignPosition = PreceedQuantityAndSymbol;
 bool _negativePrefixCurrencySymbol = false;
 bool _positivePrefixCurrencySymbol = false;
 bool _negativeSpaceSeparatesSymbol = true;
 bool _positiveSpaceSeparatesSymbol = true;
+bool _useQtInternalFormmater = false;
+QLocale _locale;
 eMyMoney::Money::fileVersionE _fileVersion = fileVersionE::FILE_4_BYTE_VALUE;
 }
 }
@@ -104,12 +110,20 @@ void MyMoneyMoney::setThousandSeparator(const QChar &separator)
     if (separator != QLatin1Char(' '))
         eMyMoney::Money::_thousandSeparator = separator;
     else
-        eMyMoney::Money::_thousandSeparator = 0;
+        eMyMoney::Money::_thousandSeparator = QLatin1Char(0);
+}
+
+void MyMoneyMoney::setThousandSeparator(const QString& separator)
+{
+    if (separator != QLatin1Char(' '))
+        eMyMoney::Money::_thousandSeparator = separator;
+    else
+        eMyMoney::Money::_thousandSeparator = QLatin1Char(0);
 }
 
 const QChar MyMoneyMoney::thousandSeparator()
 {
-    return eMyMoney::Money::_thousandSeparator;
+    return eMyMoney::Money::_thousandSeparator.isEmpty() ? QLatin1Char(0) : eMyMoney::Money::_thousandSeparator.at(0);
 }
 
 void MyMoneyMoney::setDecimalSeparator(const QChar &separator)
@@ -117,16 +131,24 @@ void MyMoneyMoney::setDecimalSeparator(const QChar &separator)
     if (separator != QLatin1Char(' '))
         eMyMoney::Money::_decimalSeparator = separator;
     else
-        eMyMoney::Money::_decimalSeparator = 0;
+        eMyMoney::Money::_decimalSeparator.clear();
+}
+
+void MyMoneyMoney::setDecimalSeparator(const QString& separator)
+{
+    if (separator != QLatin1Char(' '))
+        eMyMoney::Money::_decimalSeparator = separator;
+    else
+        eMyMoney::Money::_decimalSeparator = QLatin1Char(0);
 }
 
 const QChar MyMoneyMoney::decimalSeparator()
 {
-    return eMyMoney::Money::_decimalSeparator;
+    return eMyMoney::Money::_decimalSeparator.isEmpty() ? QLatin1Char(0) : eMyMoney::Money::_decimalSeparator.at(0);
 }
 
 MyMoneyMoney::MyMoneyMoney(const QString& pszAmount)
-    : AlkValue(pszAmount, eMyMoney::Money::_decimalSeparator)
+    : AlkValue(pszAmount, decimalSeparator())
 {
 }
 
@@ -144,7 +166,7 @@ MyMoneyMoney::MyMoneyMoney(qint64 Amount, const unsigned int denom)
     if (denom == 0)
         throw MYMONEYEXCEPTION_CSTRING("Denominator 0 not allowed!");
 
-    *this = AlkValue(QString::fromLatin1("%1/%2").arg(Amount).arg(denom), eMyMoney::Money::_decimalSeparator);
+    *this = AlkValue(QString::fromLatin1("%1/%2").arg(Amount).arg(denom), decimalSeparator());
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -176,7 +198,7 @@ MyMoneyMoney::MyMoneyMoney(const long int iAmount, const unsigned int denom)
 {
     if (denom == 0)
         throw MYMONEYEXCEPTION_CSTRING("Denominator 0 not allowed!");
-    *this = AlkValue(QString::fromLatin1("%1/%2").arg(iAmount).arg(denom), eMyMoney::Money::_decimalSeparator);
+    *this = AlkValue(QString::fromLatin1("%1/%2").arg(iAmount).arg(denom), decimalSeparator());
 }
 
 
@@ -196,6 +218,10 @@ QString MyMoneyMoney::formatMoney(int denom, bool showThousandSeparator) const
 
 QString MyMoneyMoney::formatMoney(const QString& currency, const int prec, bool showThousandSeparator) const
 {
+    if (eMyMoney::Money::_useQtInternalFormmater) {
+        return eMyMoney::Money::_locale.toCurrencyString(this->toDouble(), currency, prec);
+    }
+
     QString res;
     QString tmpCurrency = currency;
     int tmpPrec = prec;
@@ -365,7 +391,7 @@ bool MyMoneyMoney::isZero() const
 
 bool MyMoneyMoney::isAutoCalc() const
 {
-    return (*this == autoCalc);
+    return (*this == autoCalc) || (*this == -autoCalc);
 }
 
 MyMoneyMoney MyMoneyMoney::convert(const signed64 _denom, const AlkValue::RoundingMethod how) const
@@ -405,11 +431,11 @@ int MyMoneyMoney::denomToPrec(signed64 fract)
     return rc;
 }
 
-/**
- * @todo Eventually move this to AlkValue
- */
 MyMoneyMoney MyMoneyMoney::convertDenominator(mpz_class denom, const AlkValue::RoundingMethod how) const
 {
+#if ALK_VERSION >= ALK_VERSION_CHECK(8, 1, 71)
+    return static_cast<const MyMoneyMoney>(AlkValue::convertDenominator(denom, how));
+#else
     MyMoneyMoney in(*this);
     mpz_class in_num(mpq_numref(in.valueRef().get_mpq_t()));
 
@@ -530,4 +556,50 @@ MyMoneyMoney MyMoneyMoney::convertDenominator(mpz_class denom, const AlkValue::R
         }
     }
     return out;
+#endif
+}
+
+void MyMoneyMoney::detectCurrencyFormatting()
+{
+    const auto montaryLocale = qgetenv("LC_MONETARY");
+    if (!montaryLocale.isEmpty()) {
+        eMyMoney::Money::_locale = QLocale(montaryLocale);
+    }
+
+    qDebug() << "Monetary values will be formatted based on locale" << eMyMoney::Money::_locale.name()
+             << "Example: " << eMyMoney::Money::_locale.toCurrencyString(123.45);
+
+    const auto displayString = eMyMoney::Money::_locale.toCurrencyString(123.45);
+    const QRegularExpression digits("^([^01\\h]*)(\\h*)123[,\\.]45(\\h*)([^01]*)$");
+    const auto match = digits.match(displayString);
+    if (match.hasMatch()) {
+        setPositivePrefixCurrencySymbol(false);
+        setNegativePrefixCurrencySymbol(false);
+        setPositiveSpaceSeparatesSymbol(false);
+        setNegativeSpaceSeparatesSymbol(false);
+        setPositiveMonetarySignPosition(eMyMoney::Money::SucceedSymbol);
+        setNegativeMonetarySignPosition(eMyMoney::Money::PreceedQuantityAndSymbol);
+
+        if (match.capturedLength(1) > 0) {
+            setPositivePrefixCurrencySymbol(true);
+            setNegativePrefixCurrencySymbol(true);
+            setPositiveSpaceSeparatesSymbol(match.capturedLength(2) > 0);
+            setNegativeSpaceSeparatesSymbol(match.capturedLength(2) > 0);
+        } else if (match.capturedLength(3) > 0) {
+            setPositivePrefixCurrencySymbol(false);
+            setNegativePrefixCurrencySymbol(false);
+            setPositiveSpaceSeparatesSymbol(match.capturedLength(4) > 0);
+            setNegativeSpaceSeparatesSymbol(match.capturedLength(4) > 0);
+        }
+        const auto negativeDisplayString = eMyMoney::Money::_locale.toCurrencyString(-100);
+        if (negativeDisplayString == QStringLiteral("(%1)").arg(displayString)) {
+            setNegativeMonetarySignPosition(eMyMoney::Money::ParensAround);
+        }
+    } else {
+        qDebug() << "Using Qt internal formatter";
+        eMyMoney::Money::_useQtInternalFormmater = true;
+    }
+
+    setThousandSeparator(eMyMoney::Money::_locale.groupSeparator());
+    setDecimalSeparator(eMyMoney::Money::_locale.decimalPoint());
 }
