@@ -1,6 +1,6 @@
 /*
     SPDX-FileCopyrightText: 2000-2004 Michael Edwardes <mte@users.sourceforge.net>
-    SPDX-FileCopyrightText: 2002-2018 Thomas Baumgart <tbaumgart@kde.org>
+    SPDX-FileCopyrightText: 2002-2019 Thomas Baumgart <tbaumgart@kde.org>
     SPDX-FileCopyrightText: 2005 Ace Jones <acejones@users.sourceforge.net>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
@@ -17,6 +17,7 @@
 // ----------------------------------------------------------------------------
 // KDE Includes
 
+#include <KLazyLocalizedString>
 #include <KLocalizedString>
 
 // ----------------------------------------------------------------------------
@@ -33,13 +34,15 @@ using namespace eMyMoney;
 
 static IMyMoneyProcessingCalendar* processingCalendarPtr = 0;
 
-MyMoneySchedule::MyMoneySchedule() :
-    MyMoneyObject(*new MyMoneySchedulePrivate)
+MyMoneySchedule::MyMoneySchedule()
+    : MyMoneyObject(*new MyMoneySchedulePrivate)
+    , MyMoneyKeyValueContainer()
 {
 }
 
-MyMoneySchedule::MyMoneySchedule(const QString &id) :
-    MyMoneyObject(*new MyMoneySchedulePrivate, id)
+MyMoneySchedule::MyMoneySchedule(const QString& id)
+    : MyMoneyObject(*new MyMoneySchedulePrivate, id)
+    , MyMoneyKeyValueContainer()
 {
 }
 
@@ -51,8 +54,9 @@ MyMoneySchedule::MyMoneySchedule(const QString& name,
                                  const QDate& /* startDate */,
                                  const QDate& endDate,
                                  bool fixed,
-                                 bool autoEnter) :
-    MyMoneyObject(*new MyMoneySchedulePrivate)
+                                 bool autoEnter)
+    : MyMoneyObject(*new MyMoneySchedulePrivate)
+    , MyMoneyKeyValueContainer()
 {
     Q_D(MyMoneySchedule);
     // Set up the values possibly differing from defaults
@@ -67,13 +71,15 @@ MyMoneySchedule::MyMoneySchedule(const QString& name,
     d->m_endDate = endDate;
 }
 
-MyMoneySchedule::MyMoneySchedule(const MyMoneySchedule& other) :
-    MyMoneyObject(*new MyMoneySchedulePrivate(*other.d_func()), other.id())
+MyMoneySchedule::MyMoneySchedule(const MyMoneySchedule& other)
+    : MyMoneyObject(*new MyMoneySchedulePrivate(*other.d_func()), other.id())
+    , MyMoneyKeyValueContainer(other)
 {
 }
 
-MyMoneySchedule::MyMoneySchedule(const QString& id, const MyMoneySchedule& other) :
-    MyMoneyObject(*new MyMoneySchedulePrivate(*other.d_func()), id)
+MyMoneySchedule::MyMoneySchedule(const QString& id, const MyMoneySchedule& other)
+    : MyMoneyObject(*new MyMoneySchedulePrivate(*other.d_func()), id)
+    , MyMoneyKeyValueContainer(other)
 {
 }
 
@@ -152,7 +158,8 @@ void MyMoneySchedule::setTransaction(const MyMoneyTransaction& transaction, bool
     // we need to do this for the case that the transaction passed as argument
     // is a matched or imported transaction.
     auto firstSplit = true;
-    foreach (const auto split, t.splits()) {
+    const auto splits = t.splits();
+    for (const auto& split : splits) {
         MyMoneySplit s = split;
         // clear out the bankID
         if (!split.bankID().isEmpty()) {
@@ -173,7 +180,7 @@ void MyMoneySchedule::setTransaction(const MyMoneyTransaction& transaction, bool
             // simply skip the test
             // Don't check for accounts with an id of 'Phony-ID' which is used
             // internally for non-existing accounts (during creation of accounts)
-            if (file->storageAttached() && s.accountId() != QString("Phony-ID")) {
+            if (s.accountId() != QString("Phony-ID")) {
                 auto acc = file->account(s.accountId());
                 if (acc.isIncomeExpense()) {
                     s.setPayeeId(QString());
@@ -187,6 +194,7 @@ void MyMoneySchedule::setTransaction(const MyMoneyTransaction& transaction, bool
     // make sure that the transaction does not have an id so that we can enter
     // it into the engine
     d->m_transaction.clearId();
+    d->clearReferences();
 }
 
 void MyMoneySchedule::setEndDate(const QDate& date)
@@ -248,6 +256,12 @@ bool MyMoneySchedule::willEnd() const
 QDate MyMoneySchedule::nextDueDate() const
 {
     Q_D(const MyMoneySchedule);
+
+    if (lastDayInMonth()) {
+        const auto date = d->m_transaction.postDate();
+        return adjustedDate(QDate(date.year(), date.month(), date.daysInMonth()), weekendOption());
+    }
+
     return d->m_transaction.postDate();
 }
 
@@ -256,11 +270,6 @@ QDate MyMoneySchedule::adjustedNextDueDate() const
     if (isFinished())
         return QDate();
 
-    if (lastDayInMonth()) {
-        QDate date = nextDueDate();
-        return adjustedDate(QDate(date.year(), date.month(), date.daysInMonth()), weekendOption());
-    }
-
     return adjustedDate(nextDueDate(), weekendOption());
 }
 
@@ -699,12 +708,12 @@ MyMoneyAccount MyMoneySchedule::account(int cnt) const
 {
     Q_D(const MyMoneySchedule);
     QList<MyMoneySplit> splits = d->m_transaction.splits();
-    QList<MyMoneySplit>::ConstIterator it;
+    QList<MyMoneySplit>::const_iterator it;
     auto file = MyMoneyFile::instance();
     MyMoneyAccount acc;
 
     // search the first asset or liability account
-    for (it = splits.constBegin(); it != splits.constEnd() && (acc.id().isEmpty() || cnt); ++it) {
+    for (it = splits.cbegin(); it != splits.cend() && (acc.id().isEmpty() || cnt); ++it) {
         try {
             acc = file->account((*it).accountId());
             if (acc.isAssetLiability())
@@ -848,6 +857,11 @@ void MyMoneySchedule::fixDate(QDate& date) const
 {
     Q_D(const MyMoneySchedule);
     QDate fixDate(d->m_startDate);
+
+    if (d->m_lastDayInMonth) {
+        fixDate = QDate(fixDate.year(), fixDate.month(), fixDate.daysInMonth());
+    }
+
     if (fixDate.isValid()
             && date.day() != fixDate.day()
             && QDate::isValid(date.year(), date.month(), fixDate.day())) {
@@ -855,12 +869,6 @@ void MyMoneySchedule::fixDate(QDate& date) const
     }
 }
 
-bool MyMoneySchedule::hasReferenceTo(const QString& id) const
-{
-    Q_D(const MyMoneySchedule);
-    return d->m_transaction.hasReferenceTo(id);
-}
-
 QString MyMoneySchedule::occurrenceToString() const
 {
     return occurrenceToString(occurrenceMultiplier(), occurrence());
@@ -868,148 +876,131 @@ QString MyMoneySchedule::occurrenceToString() const
 
 QString MyMoneySchedule::occurrenceToString(Schedule::Occurrence occurrence)
 {
-    QString occurrenceString = I18N_NOOP2("Frequency of schedule", "Any");
-
     if (occurrence == Schedule::Occurrence::Once)
-        occurrenceString = I18N_NOOP2("Frequency of schedule", "Once");
+        return i18nc("Frequency of schedule", "Once");
     else if (occurrence == Schedule::Occurrence::Daily)
-        occurrenceString = I18N_NOOP2("Frequency of schedule", "Daily");
+        return i18nc("Frequency of schedule", "Daily");
     else if (occurrence == Schedule::Occurrence::Weekly)
-        occurrenceString = I18N_NOOP2("Frequency of schedule", "Weekly");
+        return i18nc("Frequency of schedule", "Weekly");
     else if (occurrence == Schedule::Occurrence::Fortnightly)
-        occurrenceString = I18N_NOOP2("Frequency of schedule", "Fortnightly");
+        return i18nc("Frequency of schedule", "Fortnightly");
     else if (occurrence == Schedule::Occurrence::EveryOtherWeek)
-        occurrenceString = I18N_NOOP2("Frequency of schedule", "Every other week");
+        return i18nc("Frequency of schedule", "Every other week");
     else if (occurrence == Schedule::Occurrence::EveryHalfMonth)
-        occurrenceString = I18N_NOOP2("Frequency of schedule", "Every half month");
+        return i18nc("Frequency of schedule", "Every half month");
     else if (occurrence == Schedule::Occurrence::EveryThreeWeeks)
-        occurrenceString = I18N_NOOP2("Frequency of schedule", "Every three weeks");
+        return i18nc("Frequency of schedule", "Every three weeks");
     else if (occurrence == Schedule::Occurrence::EveryFourWeeks)
-        occurrenceString = I18N_NOOP2("Frequency of schedule", "Every four weeks");
+        return i18nc("Frequency of schedule", "Every four weeks");
     else if (occurrence == Schedule::Occurrence::EveryThirtyDays)
-        occurrenceString = I18N_NOOP2("Frequency of schedule", "Every thirty days");
+        return i18nc("Frequency of schedule", "Every thirty days");
     else if (occurrence == Schedule::Occurrence::Monthly)
-        occurrenceString = I18N_NOOP2("Frequency of schedule", "Monthly");
+        return i18nc("Frequency of schedule", "Monthly");
     else if (occurrence == Schedule::Occurrence::EveryEightWeeks)
-        occurrenceString = I18N_NOOP2("Frequency of schedule", "Every eight weeks");
+        return i18nc("Frequency of schedule", "Every eight weeks");
     else if (occurrence == Schedule::Occurrence::EveryOtherMonth)
-        occurrenceString = I18N_NOOP2("Frequency of schedule", "Every two months");
+        return i18nc("Frequency of schedule", "Every two months");
     else if (occurrence == Schedule::Occurrence::EveryThreeMonths)
-        occurrenceString = I18N_NOOP2("Frequency of schedule", "Every three months");
+        return i18nc("Frequency of schedule", "Every three months");
     else if (occurrence == Schedule::Occurrence::Quarterly)
-        occurrenceString = I18N_NOOP2("Frequency of schedule", "Quarterly");
+        return i18nc("Frequency of schedule", "Quarterly");
     else if (occurrence == Schedule::Occurrence::EveryFourMonths)
-        occurrenceString = I18N_NOOP2("Frequency of schedule", "Every four months");
+        return i18nc("Frequency of schedule", "Every four months");
     else if (occurrence == Schedule::Occurrence::TwiceYearly)
-        occurrenceString = I18N_NOOP2("Frequency of schedule", "Twice yearly");
+        return i18nc("Frequency of schedule", "Twice yearly");
     else if (occurrence == Schedule::Occurrence::Yearly)
-        occurrenceString = I18N_NOOP2("Frequency of schedule", "Yearly");
+        return i18nc("Frequency of schedule", "Yearly");
     else if (occurrence == Schedule::Occurrence::EveryOtherYear)
-        occurrenceString = I18N_NOOP2("Frequency of schedule", "Every other year");
-    return occurrenceString;
+        return i18nc("Frequency of schedule", "Every other year");
+    return i18nc("Frequency of schedule", "Any");
 }
 
 QString MyMoneySchedule::occurrenceToString(int mult, Schedule::Occurrence type)
 {
-    QString occurrenceString = I18N_NOOP2("Frequency of schedule", "Any");
+    QString occurrenceString(occurrenceToString(type));
 
-    if (type == Schedule::Occurrence::Once)
-        switch (mult) {
-        case 1:
-            occurrenceString = I18N_NOOP2("Frequency of schedule", "Once");
-            break;
-        default:
-            occurrenceString = I18N_NOOP2("Frequency of schedule", QString("%1 times").arg(mult));
-        }
-    else if (type == Schedule::Occurrence::Daily)
-        switch (mult) {
-        case 1:
-            occurrenceString = I18N_NOOP2("Frequency of schedule", "Daily");
-            break;
-        case 30:
-            occurrenceString = I18N_NOOP2("Frequency of schedule", "Every thirty days");
-            break;
-        default:
-            occurrenceString = I18N_NOOP2("Frequency of schedule", QString("Every %1 days").arg(mult));
-        }
-    else if (type == Schedule::Occurrence::Weekly)
-        switch (mult) {
-        case 1:
-            occurrenceString = I18N_NOOP2("Frequency of schedule", "Weekly");
-            break;
-        case 2:
-            occurrenceString = I18N_NOOP2("Frequency of schedule", "Every other week");
-            break;
-        case 3:
-            occurrenceString = I18N_NOOP2("Frequency of schedule", "Every three weeks");
-            break;
-        case 4:
-            occurrenceString = I18N_NOOP2("Frequency of schedule", "Every four weeks");
-            break;
-        case 8:
-            occurrenceString = I18N_NOOP2("Frequency of schedule", "Every eight weeks");
-            break;
-        default:
-            occurrenceString = I18N_NOOP2("Frequency of schedule", QString("Every %1 weeks").arg(mult));
-        }
-    else if (type == Schedule::Occurrence::EveryHalfMonth)
-        switch (mult) {
-        case 1:
-            occurrenceString = I18N_NOOP2("Frequency of schedule", "Every half month");
-            break;
-        default:
-            occurrenceString = I18N_NOOP2("Frequency of schedule", QString("Every %1 half months").arg(mult));
-        }
-    else if (type == Schedule::Occurrence::Monthly)
-        switch (mult) {
-        case 1:
-            occurrenceString = I18N_NOOP2("Frequency of schedule", "Monthly");
-            break;
-        case 2:
-            occurrenceString = I18N_NOOP2("Frequency of schedule", "Every two months");
-            break;
-        case 3:
-            occurrenceString = I18N_NOOP2("Frequency of schedule", "Every three months");
-            break;
-        case 4:
-            occurrenceString = I18N_NOOP2("Frequency of schedule", "Every four months");
-            break;
-        case 6:
-            occurrenceString = I18N_NOOP2("Frequency of schedule", "Twice yearly");
-            break;
-        default:
-            occurrenceString = I18N_NOOP2("Frequency of schedule", QString("Every %1 months").arg(mult));
-        }
-    else if (type == Schedule::Occurrence::Yearly)
-        switch (mult) {
-        case 1:
-            occurrenceString = I18N_NOOP2("Frequency of schedule", "Yearly");
-            break;
-        case 2:
-            occurrenceString = I18N_NOOP2("Frequency of schedule", "Every other year");
-            break;
-        default:
-            occurrenceString = I18N_NOOP2("Frequency of schedule", QString("Every %1 years").arg(mult));
+    if (mult > 1) {
+        if (type == Schedule::Occurrence::Once) {
+            occurrenceString = i18nc("Frequency of schedule", "%1 times", mult);
+
+        } else if (type == Schedule::Occurrence::Daily) {
+            switch (mult) {
+            case 30:
+                occurrenceString = i18nc("Frequency of schedule", "Every thirty days");
+                break;
+            default:
+                occurrenceString = i18nc("Frequency of schedule", "Every %1 days", mult);
+            }
+
+        } else if (type == Schedule::Occurrence::Weekly) {
+            switch (mult) {
+            case 2:
+                occurrenceString = i18nc("Frequency of schedule", "Every other week");
+                break;
+            case 3:
+                occurrenceString = i18nc("Frequency of schedule", "Every three weeks");
+                break;
+            case 4:
+                occurrenceString = i18nc("Frequency of schedule", "Every four weeks");
+                break;
+            case 8:
+                occurrenceString = i18nc("Frequency of schedule", "Every eight weeks");
+                break;
+            default:
+                occurrenceString = i18nc("Frequency of schedule", "Every %1 weeks", mult);
+            }
+
+        } else if (type == Schedule::Occurrence::EveryHalfMonth) {
+            occurrenceString = QString(kli18nc("Frequency of schedule", "Every %1 half months").untranslatedText()).arg(mult);
+
+        } else if (type == Schedule::Occurrence::Monthly) {
+            switch (mult) {
+            case 2:
+                occurrenceString = i18nc("Frequency of schedule", "Every two months");
+                break;
+            case 3:
+                occurrenceString = i18nc("Frequency of schedule", "Every three months");
+                break;
+            case 4:
+                occurrenceString = i18nc("Frequency of schedule", "Every four months");
+                break;
+            case 6:
+                occurrenceString = i18nc("Frequency of schedule", "Twice yearly");
+                break;
+            default:
+                occurrenceString = i18nc("Frequency of schedule", "Every %1 months", mult);
+            }
+
+        } else if (type == Schedule::Occurrence::Yearly) {
+            switch (mult) {
+            case 2:
+                occurrenceString = i18nc("Frequency of schedule", "Every other year");
+                break;
+            default:
+                occurrenceString = i18nc("Frequency of schedule", "Every %1 years", mult);
+            }
         }
+    }
+
     return occurrenceString;
 }
 
 QString MyMoneySchedule::occurrencePeriodToString(Schedule::Occurrence type)
 {
-    QString occurrenceString = I18N_NOOP2("Schedule occurrence period", "Any");
+    QString occurrenceString = kli18nc("Schedule occurrence period", "Any").untranslatedText();
 
     if (type == Schedule::Occurrence::Once)
-        occurrenceString = I18N_NOOP2("Schedule occurrence period", "Once");
+        occurrenceString = kli18nc("Schedule occurrence period", "Once").untranslatedText();
     else if (type == Schedule::Occurrence::Daily)
-        occurrenceString = I18N_NOOP2("Schedule occurrence period", "Day");
+        occurrenceString = kli18nc("Schedule occurrence period", "Day").untranslatedText();
     else if (type == Schedule::Occurrence::Weekly)
-        occurrenceString = I18N_NOOP2("Schedule occurrence period", "Week");
+        occurrenceString = kli18nc("Schedule occurrence period", "Week").untranslatedText();
     else if (type == Schedule::Occurrence::EveryHalfMonth)
-        occurrenceString = I18N_NOOP2("Schedule occurrence period", "Half-month");
+        occurrenceString = kli18nc("Schedule occurrence period", "Half-month").untranslatedText();
     else if (type == Schedule::Occurrence::Monthly)
-        occurrenceString = I18N_NOOP2("Schedule occurrence period", "Month");
+        occurrenceString = kli18nc("Schedule occurrence period", "Month").untranslatedText();
     else if (type == Schedule::Occurrence::Yearly)
-        occurrenceString = I18N_NOOP2("Schedule occurrence period", "Year");
+        occurrenceString = kli18nc("Schedule occurrence period", "Year").untranslatedText();
     return occurrenceString;
 }
 
@@ -1019,56 +1010,53 @@ QString MyMoneySchedule::scheduleTypeToString(Schedule::Type type)
 
     switch (type) {
     case Schedule::Type::Bill:
-        text = I18N_NOOP2("Scheduled transaction type", "Bill");
+        text = kli18nc("Scheduled transaction type", "Bill").untranslatedText();
         break;
     case Schedule::Type::Deposit:
-        text = I18N_NOOP2("Scheduled transaction type", "Deposit");
+        text = kli18nc("Scheduled transaction type", "Deposit").untranslatedText();
         break;
     case Schedule::Type::Transfer:
-        text = I18N_NOOP2("Scheduled transaction type", "Transfer");
+        text = kli18nc("Scheduled transaction type", "Transfer").untranslatedText();
         break;
     case Schedule::Type::LoanPayment:
-        text = I18N_NOOP2("Scheduled transaction type", "Loan payment");
+        text = kli18nc("Scheduled transaction type", "Loan payment").untranslatedText();
         break;
     case Schedule::Type::Any:
     default:
-        text = I18N_NOOP2("Scheduled transaction type", "Unknown");
+        text = kli18nc("Scheduled transaction type", "Unknown").untranslatedText();
     }
     return text;
 }
 
-
-QString MyMoneySchedule::paymentMethodToString(Schedule::PaymentType paymentType)
+const char* MyMoneySchedule::paymentMethodToString(Schedule::PaymentType paymentType)
 {
-    QString text;
-
     switch (paymentType) {
     case Schedule::PaymentType::DirectDebit:
-        text = I18N_NOOP2("Scheduled Transaction payment type", "Direct debit");
+        return kli18nc("Scheduled Transaction payment type", "Direct debit").untranslatedText();
         break;
     case Schedule::PaymentType::DirectDeposit:
-        text = I18N_NOOP2("Scheduled Transaction payment type", "Direct deposit");
+        return kli18nc("Scheduled Transaction payment type", "Direct deposit").untranslatedText();
         break;
     case Schedule::PaymentType::ManualDeposit:
-        text = I18N_NOOP2("Scheduled Transaction payment type", "Manual deposit");
+        return kli18nc("Scheduled Transaction payment type", "Manual deposit").untranslatedText();
         break;
     case Schedule::PaymentType::Other:
-        text = I18N_NOOP2("Scheduled Transaction payment type", "Other");
+        return kli18nc("Scheduled Transaction payment type", "Other").untranslatedText();
         break;
     case Schedule::PaymentType::WriteChecque:
-        text = I18N_NOOP2("Scheduled Transaction payment type", "Write check");
+        return kli18nc("Scheduled Transaction payment type", "Write check").untranslatedText();
         break;
     case Schedule::PaymentType::StandingOrder:
-        text = I18N_NOOP2("Scheduled Transaction payment type", "Standing order");
+        return kli18nc("Scheduled Transaction payment type", "Standing order").untranslatedText();
         break;
     case Schedule::PaymentType::BankTransfer:
-        text = I18N_NOOP2("Scheduled Transaction payment type", "Bank transfer");
+        return kli18nc("Scheduled Transaction payment type", "Bank transfer").untranslatedText();
         break;
     case Schedule::PaymentType::Any:
-        text = I18N_NOOP2("Scheduled Transaction payment type", "Any (Error)");
+        return kli18nc("Scheduled Transaction payment type", "Any (Error)").untranslatedText();
         break;
     }
-    return text;
+    return {};
 }
 
 QString MyMoneySchedule::weekendOptionToString(Schedule::WeekendOption weekendOption)
@@ -1077,13 +1065,13 @@ QString MyMoneySchedule::weekendOptionToString(Schedule::WeekendOption weekendOp
 
     switch (weekendOption) {
     case Schedule::WeekendOption::MoveBefore:
-        text = I18N_NOOP("Change the date to the previous processing day");
+        text = kli18n("Change the date to the previous processing day").untranslatedText();
         break;
     case Schedule::WeekendOption::MoveAfter:
-        text = I18N_NOOP("Change the date to the next processing day");
+        text = kli18n("Change the date to the next processing day").untranslatedText();
         break;
     case Schedule::WeekendOption::MoveNothing:
-        text = I18N_NOOP("Do not change the date");
+        text = kli18n("Do not change the date").untranslatedText();
         break;
     }
     return text;
@@ -1264,7 +1252,6 @@ QDate MyMoneySchedule::addHalfMonths(QDate date, int mult) const
     } else if (mult < 0)  // Go backwards
         for (int i = 0; i > mult; i--) {
             d = newdate.day();
-            dm = newdate.daysInMonth();
             if (d > 15) {
                 dm = newdate.daysInMonth();
                 newdate = newdate.addDays((d == dm) ? 15 - dm : -15);
@@ -1435,3 +1422,13 @@ bool MyMoneySchedule::replaceId(const QString& newId, const QString& oldId)
     Q_D(MyMoneySchedule);
     return d->m_transaction.replaceId(newId, oldId);
 }
+
+void MyMoneySchedule::setKeepMultiCurrencyAmount(bool keepAmount)
+{
+    setValue("kmm-keepamount", keepAmount, false);
+}
+
+bool MyMoneySchedule::keepMultiCurrencyAmount() const
+{
+    return value("kmm-keepamount", false);
+}
