@@ -1,5 +1,5 @@
 /*
-    SPDX-FileCopyrightText: 2002-2017 Thomas Baumgart <tbaumgart@kde.org>
+    SPDX-FileCopyrightText: 2002-2019 Thomas Baumgart <tbaumgart@kde.org>
     SPDX-FileCopyrightText: 2004 Kevin Tambascio <ktambascio@users.sourceforge.net>
     SPDX-FileCopyrightText: 2005-2006 Ace Jones <acejones@users.sourceforge.net>
     SPDX-FileCopyrightText: 2017-2018 Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
@@ -91,6 +91,7 @@ void MyMoneySplit::setAccountId(const QString& account)
 {
     Q_D(MyMoneySplit);
     d->m_account = account;
+    d->clearReferences();
 }
 
 QString MyMoneySplit::costCenterId() const
@@ -103,6 +104,7 @@ void MyMoneySplit::setCostCenterId(const QString& costCenter)
 {
     Q_D(MyMoneySplit);
     d->m_costCenter = costCenter;
+    d->clearReferences();
 }
 
 QString MyMoneySplit::memo() const
@@ -164,9 +166,9 @@ QString MyMoneySplit::value(const QString& key) const
     return MyMoneyKeyValueContainer::value(key);
 }
 
-void MyMoneySplit::setValue(const QString& key, const QString& value)
+void MyMoneySplit::setValue(const QString& key, const QString& value, const QString& defaultValue)
 {
-    MyMoneyKeyValueContainer::setValue(key, value);
+    MyMoneyKeyValueContainer::setValue(key, value, defaultValue);
 }
 
 void MyMoneySplit::setValue(const MyMoneyMoney& value)
@@ -199,6 +201,7 @@ void MyMoneySplit::setPayeeId(const QString& payee)
 {
     Q_D(MyMoneySplit);
     d->m_payee = payee;
+    d->clearReferences();
 }
 
 QList<QString> MyMoneySplit::tagIdList() const
@@ -211,6 +214,7 @@ void MyMoneySplit::setTagIdList(const QList<QString>& tagList)
 {
     Q_D(MyMoneySplit);
     d->m_tagList = tagList;
+    d->clearReferences();
 }
 
 void MyMoneySplit::setAction(eMyMoney::Split::InvestmentTransactionType type)
@@ -244,6 +248,36 @@ void MyMoneySplit::setAction(eMyMoney::Split::InvestmentTransactionType type)
     }
 }
 
+eMyMoney::Split::InvestmentTransactionType MyMoneySplit::investmentTransactionType() const
+{
+    Q_D(const MyMoneySplit);
+    switch(actionStringToAction(d->m_action)) {
+    case Split::Action::BuyShares:
+        return (d->m_shares.isNegative()) ? Split::InvestmentTransactionType::SellShares : Split::InvestmentTransactionType::BuyShares;
+
+    case Split::Action::Dividend:
+        return Split::InvestmentTransactionType::Dividend;
+
+    case Split::Action::Yield:
+        return Split::InvestmentTransactionType::Yield;
+
+    case Split::Action::ReinvestDividend:
+        return Split::InvestmentTransactionType::ReinvestDividend;
+
+    case Split::Action::AddShares:
+        return (d->m_shares.isNegative()) ? Split::InvestmentTransactionType::RemoveShares : Split::InvestmentTransactionType::AddShares;
+
+    case Split::Action::SplitShares:
+        return Split::InvestmentTransactionType::SplitShares;
+
+    case Split::Action::InterestIncome:
+        return Split::InvestmentTransactionType::InterestIncome;
+
+    default:
+        return Split::InvestmentTransactionType::UnknownTransactionType;
+    }
+}
+
 QString MyMoneySplit::action() const
 {
     Q_D(const MyMoneySplit);
@@ -323,12 +357,6 @@ MyMoneyMoney MyMoneySplit::value(const QString& transactionCurrencyId, const QSt
     return (transactionCurrencyId == splitCurrencyId) ? d->m_value : d->m_shares;
 }
 
-MyMoneyMoney MyMoneySplit::actualPrice() const
-{
-    Q_D(const MyMoneySplit);
-    return d->m_price;
-}
-
 void MyMoneySplit::setPrice(const MyMoneyMoney& price)
 {
     Q_D(MyMoneySplit);
@@ -345,19 +373,6 @@ MyMoneyMoney MyMoneySplit::price() const
     return MyMoneyMoney::ONE;
 }
 
-bool MyMoneySplit::hasReferenceTo(const QString& id) const
-{
-    Q_D(const MyMoneySplit);
-    auto rc = false;
-    if (isMatched()) {
-        rc = matchedTransaction().hasReferenceTo(id);
-    }
-    for (int i = 0; i < d->m_tagList.size(); i++)
-        if (id == d->m_tagList[i])
-            return true;
-    return rc || (id == d->m_account) || (id == d->m_payee) || (id == d->m_costCenter);
-}
-
 bool MyMoneySplit::isMatched() const
 {
     Q_D(const MyMoneySplit);
@@ -417,7 +432,7 @@ bool MyMoneySplit::replaceId(const QString& newId, const QString& oldId)
     return changed;
 }
 
-QString MyMoneySplit::actionName(Split::Action action)
+QHash<Split::Action, QString> actionNamesLUT()
 {
     static const QHash<Split::Action, QString> actionNames {
         {Split::Action::Check,            QStringLiteral("Check")},
@@ -435,5 +450,15 @@ QString MyMoneySplit::actionName(Split::Action action)
         {Split::Action::SplitShares,      QStringLiteral("Split")},
         {Split::Action::InterestIncome,   QStringLiteral("IntIncome")},
     };
-    return actionNames[action];
+    return actionNames;
+}
+
+QString MyMoneySplit::actionName(Split::Action action)
+{
+    return actionNamesLUT().value(action);
+}
+
+Split::Action MyMoneySplit::actionStringToAction(const QString &text) const
+{
+    return actionNamesLUT().key(text, Split::Action::Unknown);
 }
