@@ -31,7 +31,8 @@ enum class Element {
     Price,
     Security,
 };
-uint qHash(const Element key, uint seed) {
+qHashSeedType qHash(const Element key, qHashSeedType seed)
+{
     return ::qHash(static_cast<uint>(key), seed);
 }
 
@@ -42,7 +43,7 @@ enum class Attribute {
     Version,
     AccountName,
     AccountNumber,
-    RoutingNumber,
+    BankCode,
     Currency,
     BeginDate,
     EndDate,
@@ -51,6 +52,7 @@ enum class Attribute {
     AccountID,
     SkipCategoryMatching,
     DatePosted,
+    DateProcessed,
     Payee,
     Memo,
     Number,
@@ -60,10 +62,13 @@ enum class Attribute {
     Action,
     Shares,
     Security,
+    SecurityId,
     BrokerageAccount,
     Category,
+    SmallestFraction,
 };
-uint qHash(const Attribute key, uint seed) {
+qHashSeedType qHash(const Attribute key, qHashSeedType seed)
+{
     return ::qHash(static_cast<uint>(key), seed);
 }
 }
@@ -71,7 +76,8 @@ uint qHash(const Attribute key, uint seed) {
 
 using namespace eMyMoney;
 
-const QHash<Statement::Type, QString> txAccountType {
+// clang-format off
+static const QHash<Statement::Type, QString> txAccountType{ // clazy:exclude=non-pod-global-static
     {Statement::Type::None,       QStringLiteral("none")},
     {Statement::Type::Checkings,  QStringLiteral("checkings")},
     {Statement::Type::Savings,    QStringLiteral("savings")},
@@ -79,8 +85,10 @@ const QHash<Statement::Type, QString> txAccountType {
     {Statement::Type::CreditCard, QStringLiteral("creditcard")},
     {Statement::Type::Invalid,    QStringLiteral("invalid")},
 };
+// clang-format on
 
-const QHash<Transaction::Action, QString> txAction {
+// clang-format off
+static const QHash<Transaction::Action, QString> txAction{ // clazy:exclude=non-pod-global-static
     {Transaction::Action::None,             QStringLiteral("none")},
     {Transaction::Action::Buy,              QStringLiteral("buy")},
     {Transaction::Action::Sell,             QStringLiteral("sell")},
@@ -93,30 +101,34 @@ const QHash<Transaction::Action, QString> txAction {
     {Transaction::Action::Interest,         QStringLiteral("interest")},
     {Transaction::Action::Invalid,          QStringLiteral("invalid")},
 };
+// clang-format on
 
 QString getElName(const Statement::Element el)
 {
-    static const QHash<Statement::Element, QString> elNames {
+    static const QHash<Statement::Element, QString> elNames{
+        // clang-format off
         {Statement::Element::KMMStatement, QStringLiteral("KMYMONEY-STATEMENT")},
         {Statement::Element::Statement,    QStringLiteral("STATEMENT")},
         {Statement::Element::Transaction,  QStringLiteral("TRANSACTION")},
         {Statement::Element::Split,        QStringLiteral("SPLIT")},
         {Statement::Element::Price,        QStringLiteral("PRICE")},
         {Statement::Element::Security,     QStringLiteral("SECURITY")},
+        // clang-format on
     };
     return elNames[el];
 }
 
 QString getAttrName(const Statement::Attribute attr)
 {
-    static const QHash<Statement::Attribute, QString> attrNames {
+    static const QHash<Statement::Attribute, QString> attrNames{
+        // clang-format off
         {Statement::Attribute::Name,                 QStringLiteral("name")},
         {Statement::Attribute::Symbol,               QStringLiteral("symbol")},
         {Statement::Attribute::ID,                   QStringLiteral("id")},
         {Statement::Attribute::Version,              QStringLiteral("version")},
         {Statement::Attribute::AccountName,          QStringLiteral("accountname")},
         {Statement::Attribute::AccountNumber,        QStringLiteral("accountnumber")},
-        {Statement::Attribute::RoutingNumber,        QStringLiteral("routingnumber")},
+        {Statement::Attribute::BankCode,             QStringLiteral("routingnumber")},
         {Statement::Attribute::Currency,             QStringLiteral("currency")},
         {Statement::Attribute::BeginDate,            QStringLiteral("begindate")},
         {Statement::Attribute::EndDate,              QStringLiteral("enddate")},
@@ -125,6 +137,7 @@ QString getAttrName(const Statement::Attribute attr)
         {Statement::Attribute::AccountID,            QStringLiteral("accountid")},
         {Statement::Attribute::SkipCategoryMatching, QStringLiteral("skipCategoryMatching")},
         {Statement::Attribute::DatePosted,           QStringLiteral("dateposted")},
+        {Statement::Attribute::DateProcessed,        QStringLiteral("entrydate")},
         {Statement::Attribute::Payee,                QStringLiteral("payee")},
         {Statement::Attribute::Memo,                 QStringLiteral("memo")},
         {Statement::Attribute::Number,               QStringLiteral("number")},
@@ -134,8 +147,11 @@ QString getAttrName(const Statement::Attribute attr)
         {Statement::Attribute::Action,               QStringLiteral("action")},
         {Statement::Attribute::Shares,               QStringLiteral("shares")},
         {Statement::Attribute::Security,             QStringLiteral("security")},
+        {Statement::Attribute::SecurityId,           QStringLiteral("securityId")},
         {Statement::Attribute::BrokerageAccount,     QStringLiteral("brokerageaccount")},
         {Statement::Attribute::Category,             QStringLiteral("version")},
+        {Statement::Attribute::SmallestFraction,     QStringLiteral("smallestFraction")},
+        // clang-format on
     };
     return attrNames[attr];
 }
@@ -148,7 +164,7 @@ void MyMoneyStatement::write(QDomElement& _root, QDomDocument* _doc) const
     e.setAttribute(getAttrName(Statement::Attribute::Version), QStringLiteral("1.1"));
     e.setAttribute(getAttrName(Statement::Attribute::AccountName), m_strAccountName);
     e.setAttribute(getAttrName(Statement::Attribute::AccountNumber), m_strAccountNumber);
-    e.setAttribute(getAttrName(Statement::Attribute::RoutingNumber), m_strRoutingNumber);
+    e.setAttribute(getAttrName(Statement::Attribute::BankCode), m_strBankCode);
     e.setAttribute(getAttrName(Statement::Attribute::Currency), m_strCurrency);
     e.setAttribute(getAttrName(Statement::Attribute::BeginDate), m_dateBegin.toString(Qt::ISODate));
     e.setAttribute(getAttrName(Statement::Attribute::EndDate), m_dateEnd.toString(Qt::ISODate));
@@ -158,25 +174,29 @@ void MyMoneyStatement::write(QDomElement& _root, QDomDocument* _doc) const
     e.setAttribute(getAttrName(Statement::Attribute::SkipCategoryMatching), m_skipCategoryMatching);
 
     // iterate over transactions, and add each one
-    foreach (const auto tansaction, m_listTransactions) {
+    for (const auto& transaction : qAsConst(m_listTransactions)) {
         auto p = _doc->createElement(getElName(Statement::Element::Transaction));
-        p.setAttribute(getAttrName(Statement::Attribute::DatePosted), tansaction.m_datePosted.toString(Qt::ISODate));
-        p.setAttribute(getAttrName(Statement::Attribute::Payee), tansaction.m_strPayee);
-        p.setAttribute(getAttrName(Statement::Attribute::Memo), tansaction.m_strMemo);
-        p.setAttribute(getAttrName(Statement::Attribute::Number), tansaction.m_strNumber);
-        p.setAttribute(getAttrName(Statement::Attribute::Amount), tansaction.m_amount.toString());
-        p.setAttribute(getAttrName(Statement::Attribute::BankID), tansaction.m_strBankID);
-        p.setAttribute(getAttrName(Statement::Attribute::Reconcile), (int)tansaction.m_reconcile);
-        p.setAttribute(getAttrName(Statement::Attribute::Action), txAction[tansaction.m_eAction]);
+        p.setAttribute(getAttrName(Statement::Attribute::DatePosted), transaction.m_datePosted.toString(Qt::ISODate));
+        if (transaction.m_dateProcessed.isValid()) {
+            p.setAttribute(getAttrName(Statement::Attribute::DateProcessed), transaction.m_dateProcessed.toString(Qt::ISODate));
+        }
+        p.setAttribute(getAttrName(Statement::Attribute::Payee), transaction.m_strPayee);
+        p.setAttribute(getAttrName(Statement::Attribute::Memo), transaction.m_strMemo);
+        p.setAttribute(getAttrName(Statement::Attribute::Number), transaction.m_strNumber);
+        p.setAttribute(getAttrName(Statement::Attribute::Amount), transaction.m_amount.toString());
+        p.setAttribute(getAttrName(Statement::Attribute::BankID), transaction.m_strBankID);
+        p.setAttribute(getAttrName(Statement::Attribute::Reconcile), (int)transaction.m_reconcile);
+        p.setAttribute(getAttrName(Statement::Attribute::Action), txAction[transaction.m_eAction]);
 
         if (m_eType == eMyMoney::Statement::Type::Investment) {
-            p.setAttribute(getAttrName(Statement::Attribute::Shares), tansaction.m_shares.toString());
-            p.setAttribute(getAttrName(Statement::Attribute::Security), tansaction.m_strSecurity);
-            p.setAttribute(getAttrName(Statement::Attribute::BrokerageAccount), tansaction.m_strBrokerageAccount);
+            p.setAttribute(getAttrName(Statement::Attribute::Shares), transaction.m_shares.toString());
+            p.setAttribute(getAttrName(Statement::Attribute::Security), transaction.m_strSecurity);
+            p.setAttribute(getAttrName(Statement::Attribute::SecurityId), transaction.m_strSecurityId);
+            p.setAttribute(getAttrName(Statement::Attribute::BrokerageAccount), transaction.m_strBrokerageAccount);
         }
 
         // add all the splits we know of (might be empty)
-        foreach (const auto split, tansaction.m_listSplits) {
+        for (const auto& split : qAsConst(transaction.m_listSplits)) {
             auto el = _doc->createElement(getElName(Statement::Element::Split));
             el.setAttribute(getAttrName(Statement::Attribute::AccountID), split.m_accountId);
             el.setAttribute(getAttrName(Statement::Attribute::Amount), split.m_amount.toString());
@@ -190,7 +210,7 @@ void MyMoneyStatement::write(QDomElement& _root, QDomDocument* _doc) const
     }
 
     // iterate over prices, and add each one
-    foreach (const auto price, m_listPrices) {
+    for (const auto& price : qAsConst(m_listPrices)) {
         auto p = _doc->createElement(getElName(Statement::Element::Price));
         p.setAttribute(getAttrName(Statement::Attribute::DatePosted), price.m_date.toString(Qt::ISODate));
         p.setAttribute(getAttrName(Statement::Attribute::Security), price.m_strSecurity);
@@ -200,11 +220,12 @@ void MyMoneyStatement::write(QDomElement& _root, QDomDocument* _doc) const
     }
 
     // iterate over securities, and add each one
-    foreach (const auto security, m_listSecurities) {
+    for (const auto& security : qAsConst(m_listSecurities)) {
         auto p = _doc->createElement(getElName(Statement::Element::Security));
         p.setAttribute(getAttrName(Statement::Attribute::Name), security.m_strName);
         p.setAttribute(getAttrName(Statement::Attribute::Symbol), security.m_strSymbol);
         p.setAttribute(getAttrName(Statement::Attribute::ID), security.m_strId);
+        p.setAttribute(getAttrName(Statement::Attribute::SmallestFraction), security.m_smallestFraction.toString());
 
         e.appendChild(p);
     }
@@ -219,7 +240,7 @@ bool MyMoneyStatement::read(const QDomElement& _e)
 
         m_strAccountName = _e.attribute(getAttrName(Statement::Attribute::AccountName));
         m_strAccountNumber = _e.attribute(getAttrName(Statement::Attribute::AccountNumber));
-        m_strRoutingNumber = _e.attribute(getAttrName(Statement::Attribute::RoutingNumber));
+        m_strBankCode = _e.attribute(getAttrName(Statement::Attribute::BankCode));
         m_strCurrency = _e.attribute(getAttrName(Statement::Attribute::Currency));
         m_dateBegin = QDate::fromString(_e.attribute(getAttrName(Statement::Attribute::BeginDate)), Qt::ISODate);
         m_dateEnd = QDate::fromString(_e.attribute(getAttrName(Statement::Attribute::EndDate)), Qt::ISODate);
@@ -238,6 +259,9 @@ bool MyMoneyStatement::read(const QDomElement& _e)
                 MyMoneyStatement::Transaction t;
 
                 t.m_datePosted = QDate::fromString(c.attribute(getAttrName(Statement::Attribute::DatePosted)), Qt::ISODate);
+                if (c.hasAttribute(getAttrName(Statement::Attribute::DateProcessed))) {
+                    t.m_dateProcessed = QDate::fromString(c.attribute(getAttrName(Statement::Attribute::DateProcessed)), Qt::ISODate);
+                }
                 t.m_amount = MyMoneyMoney(c.attribute(getAttrName(Statement::Attribute::Amount)));
                 t.m_strMemo = c.attribute(getAttrName(Statement::Attribute::Memo));
                 t.m_strNumber = c.attribute(getAttrName(Statement::Attribute::Number));
@@ -251,13 +275,14 @@ bool MyMoneyStatement::read(const QDomElement& _e)
                 if (m_eType == eMyMoney::Statement::Type::Investment) {
                     t.m_shares = MyMoneyMoney(c.attribute(getAttrName(Statement::Attribute::Shares)));
                     t.m_strSecurity = c.attribute(getAttrName(Statement::Attribute::Security));
+                    t.m_strSecurityId = c.attribute(getAttrName(Statement::Attribute::SecurityId));
                     t.m_strBrokerageAccount = c.attribute(getAttrName(Statement::Attribute::BrokerageAccount));
                 }
 
                 // process splits (if any)
-                child = c.firstChild();
-                while (!child.isNull() && child.isElement()) {
-                    c = child.toElement();
+                auto splitChild = c.firstChild();
+                while (!splitChild.isNull() && splitChild.isElement()) {
+                    c = splitChild.toElement();
                     if (c.tagName() == getElName(Statement::Element::Split)) {
                         MyMoneyStatement::Split s;
                         s.m_accountId = c.attribute(getAttrName(Statement::Attribute::AccountID));
@@ -267,7 +292,7 @@ bool MyMoneyStatement::read(const QDomElement& _e)
                         s.m_strMemo = c.attribute(getAttrName(Statement::Attribute::Memo));
                         t.m_listSplits += s;
                     }
-                    child = child.nextSibling();
+                    splitChild = splitChild.nextSibling();
                 }
                 m_listTransactions += t;
             } else if (c.tagName() == getElName(Statement::Element::Price)) {
@@ -284,6 +309,7 @@ bool MyMoneyStatement::read(const QDomElement& _e)
                 s.m_strName = c.attribute(getAttrName(Statement::Attribute::Name));
                 s.m_strSymbol = c.attribute(getAttrName(Statement::Attribute::Symbol));
                 s.m_strId = c.attribute(getAttrName(Statement::Attribute::ID));
+                s.m_smallestFraction = MyMoneyMoney(c.attribute(getAttrName(Statement::Attribute::SmallestFraction)));
 
                 m_listSecurities += s;
             }
@@ -297,7 +323,7 @@ bool MyMoneyStatement::read(const QDomElement& _e)
 bool MyMoneyStatement::isStatementFile(const QString& _filename)
 {
     // filename is considered a statement file if it contains
-    // the tag "<KMYMONEY2-STATEMENT>" in the first 20 lines.
+    // the tag "<KMYMONEY-STATEMENT>" in the first 20 lines.
     bool result = false;
 
     QFile f(_filename);
@@ -338,7 +364,9 @@ void MyMoneyStatement::writeXMLFile(const MyMoneyStatement& _s, const QString& _
     QFile g(filename);
     if (g.open(QIODevice::WriteOnly)) {
         QTextStream stream(&g);
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
         stream.setCodec("UTF-8");
+#endif
         stream << doc->toString();
         g.close();
     }
@@ -375,9 +403,13 @@ QDate MyMoneyStatement::statementEndDate() const
     if (m_dateEnd.isValid())
         return m_dateEnd;
 
+    // if we don't have an end date listed in the
+    // statement, we take the last postdate we
+    // find that is not in the future and use it
+    // as the end date of the statement.
     QDate postDate;
     for(auto& t : m_listTransactions) {
-        if (t.m_datePosted > postDate) {
+        if ((t.m_datePosted > postDate) && (t.m_datePosted <= QDate::currentDate())) {
             postDate = t.m_datePosted;
         }
     }
