@@ -12,8 +12,9 @@
 // QT Includes
 
 #include <QDate>
-#include <QFlags>
 #include <QDebug>
+#include <QFlags>
+#include <QRegularExpression>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
@@ -42,30 +43,35 @@ public:
         , m_treatTransfersAsIncomeExpense(false)
         , m_matchingSplitsCount(0)
         , m_invertText(false)
+        , m_filterIsRegExp(false)
     {
     }
 
     MyMoneyTransactionFilter::FilterSet m_filterSet;
-    bool                m_reportAllSplits;
-    bool                m_considerCategory;
-    bool                m_considerCategorySplits;
-    bool                m_matchOnly;
-    bool                m_treatTransfersAsIncomeExpense;
+    bool m_reportAllSplits;
+    bool m_considerCategory;
+    bool m_considerCategorySplits;
+    bool m_matchOnly;
+    bool m_treatTransfersAsIncomeExpense;
 
-    uint                m_matchingSplitsCount;
+    uint m_matchingSplitsCount;
 
-    QRegExp             m_text;
-    bool                m_invertText;
-    QHash<QString, QString>    m_accounts;
-    QHash<QString, QString>    m_payees;
-    QHash<QString, QString>    m_tags;
-    QHash<QString, QString>    m_categories;
-    QHash<int, QString>      m_states;
-    QHash<int, QString>      m_types;
-    QHash<int, QString>      m_validity;
-    QString             m_fromNr, m_toNr;
-    QDate               m_fromDate, m_toDate;
-    MyMoneyMoney        m_fromAmount, m_toAmount;
+    QRegularExpression m_text;
+    bool m_invertText;
+    bool m_filterIsRegExp;
+    QHash<QString, QString> m_accounts;
+    QHash<QString, QString> m_payees;
+    QHash<QString, QString> m_tags;
+    QHash<QString, QString> m_categories;
+    QHash<int, QString> m_states;
+    QHash<int, QString> m_types;
+    QHash<int, QString> m_validity;
+    QString m_fromNr;
+    QString m_toNr;
+    QDate m_fromDate;
+    QDate m_toDate;
+    MyMoneyMoney m_fromAmount;
+    MyMoneyMoney m_toAmount;
 };
 
 MyMoneyTransactionFilter::MyMoneyTransactionFilter() :
@@ -98,6 +104,7 @@ void MyMoneyTransactionFilter::clear()
     Q_D(MyMoneyTransactionFilter);
     d->m_filterSet = {};
     d->m_invertText = false;
+    d->m_filterIsRegExp = false;
     d->m_accounts.clear();
     d->m_categories.clear();
     d->m_payees.clear();
@@ -112,14 +119,22 @@ void MyMoneyTransactionFilter::clear()
 void MyMoneyTransactionFilter::clearAccountFilter()
 {
     Q_D(MyMoneyTransactionFilter);
-    d->m_filterSet.setFlag(accountFilterActive);
+    d->m_filterSet.setFlag(accountFilterActive, false);
     d->m_accounts.clear();
 }
 
-void MyMoneyTransactionFilter::setTextFilter(const QRegExp& text, bool invert)
+void MyMoneyTransactionFilter::clearCategoryFilter()
+{
+    Q_D(MyMoneyTransactionFilter);
+    d->m_filterSet.setFlag(categoryFilterActive, false);
+    d->m_categories.clear();
+}
+
+void MyMoneyTransactionFilter::setTextFilter(const QRegularExpression& text, bool isRegExp, bool invert)
 {
     Q_D(MyMoneyTransactionFilter);
     d->m_filterSet.setFlag(textFilterActive);
+    d->m_filterIsRegExp = isRegExp;
     d->m_invertText = invert;
     d->m_text = text;
 }
@@ -169,9 +184,15 @@ void MyMoneyTransactionFilter::addCategory(const QString& id)
 void MyMoneyTransactionFilter::setDateFilter(const QDate& from, const QDate& to)
 {
     Q_D(MyMoneyTransactionFilter);
-    d->m_filterSet.setFlag(dateFilterActive, from.isValid() | to.isValid());
+    d->m_filterSet.setFlag(dateFilterActive, from.isValid() || to.isValid());
     d->m_fromDate = from;
     d->m_toDate = to;
+    if (from.isValid() && to.isValid()) {
+        if (from > to) {
+            d->m_fromDate = to;
+            d->m_toDate = from;
+        }
+    }
 }
 
 void MyMoneyTransactionFilter::setAmountFilter(const MyMoneyMoney& from, const MyMoneyMoney& to)
@@ -702,6 +723,21 @@ bool MyMoneyTransactionFilter::payees(QStringList& list) const
     return result;
 }
 
+QStringList MyMoneyTransactionFilter::payees() const
+{
+    Q_D(const MyMoneyTransactionFilter);
+    QStringList list;
+
+    if (d->m_filterSet.testFlag(payeeFilterActive)) {
+        QHashIterator<QString, QString> it_payee(d->m_payees);
+        while (it_payee.hasNext()) {
+            it_payee.next();
+            list += it_payee.key();
+        }
+    }
+    return list;
+}
+
 bool MyMoneyTransactionFilter::tags(QStringList& list) const
 {
     Q_D(const MyMoneyTransactionFilter);
@@ -717,6 +753,13 @@ bool MyMoneyTransactionFilter::tags(QStringList& list) const
     return result;
 }
 
+QStringList MyMoneyTransactionFilter::tags() const
+{
+    QStringList tagIds;
+    tags(tagIds);
+    return tagIds;
+}
+
 bool MyMoneyTransactionFilter::accounts(QStringList& list) const
 {
     Q_D(const MyMoneyTransactionFilter);
@@ -733,6 +776,22 @@ bool MyMoneyTransactionFilter::accounts(QStringList& list) const
     return result;
 }
 
+QStringList MyMoneyTransactionFilter::accounts() const
+{
+    Q_D(const MyMoneyTransactionFilter);
+    QStringList list;
+
+    if (d->m_filterSet.testFlag(accountFilterActive)) {
+        QHashIterator<QString, QString> it_account(d->m_accounts);
+        while (it_account.hasNext()) {
+            it_account.next();
+            QString account = it_account.key();
+            list += account;
+        }
+    }
+    return list;
+}
+
 bool MyMoneyTransactionFilter::categories(QStringList& list) const
 {
     Q_D(const MyMoneyTransactionFilter);
@@ -838,10 +897,11 @@ bool MyMoneyTransactionFilter::firstValidity(int&i) const
     return result;
 }
 
-bool MyMoneyTransactionFilter::textFilter(QRegExp& exp) const
+bool MyMoneyTransactionFilter::textFilter(QRegularExpression& text, bool& isRegExp) const
 {
     Q_D(const MyMoneyTransactionFilter);
-    exp = d->m_text;
+    text = d->m_text;
+    isRegExp = d->m_filterIsRegExp;
     return d->m_filterSet.testFlag(textFilterActive);
 }
 
