@@ -15,6 +15,9 @@
 // QT Includes
 
 #include <QMetaType>
+class QString;
+class QDate;
+class QRegularExpression;
 
 // ----------------------------------------------------------------------------
 // KDE Includes
@@ -22,9 +25,6 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
-class QString;
-class QDate;
-
 template <typename T> class QList;
 
 class MyMoneyMoney;
@@ -64,7 +64,7 @@ public:
         amountFilterActive   = 0x0080,
         typeFilterActive     = 0x0100,
         stateFilterActive    = 0x0200,
-        validityFilterActive = 0x0400
+        validityFilterActive = 0x0400,
     };
     Q_DECLARE_FLAGS(FilterSet, FilterFlags)
 
@@ -112,22 +112,28 @@ public:
     void clearAccountFilter();
 
     /**
-      * This method is used to set the regular expression filter to the value specified
-      * as parameter @p exp. The following text based fields are searched:
-      *
-      * - Memo
-      * - Payee
-      * - Tag
-      * - Category
-      * - Shares / Value
-      * - Number
-      *
-      * @param exp The regular expression that must be found in a transaction
-      *            before it is included in the result set.
-      * @param invert If true, value must not be contained in any of the above mentioned fields
-      *
-      */
-    void setTextFilter(const QRegExp& exp, bool invert = false);
+     * This method is used to clear the category filter only.
+     */
+    void clearCategoryFilter();
+
+    /**
+     * This method is used to set the regular expression filter to the value specified
+     * as parameter @p exp. The following text based fields are searched:
+     *
+     * - Memo
+     * - Payee
+     * - Tag
+     * - Category
+     * - Shares / Value
+     * - Number
+     *
+     * @param exp The regular expression that must be found in a transaction
+     *            before it is included in the result set.
+     * @param isRegExp the user should see a regular expression if true or a wildcard if false
+     * @param invert If true, value must not be contained in any of the above mentioned fields
+     *
+     */
+    void setTextFilter(const QRegularExpression& exp, bool isRegExp, bool invert);
 
     /**
       * This method will add the account with id @p id to the list of matching accounts.
@@ -434,6 +440,13 @@ public:
       */
     bool payees(QStringList& list) const;
 
+    /**
+     * This method returns all payees set in the filter.
+     *
+     * @return QStringList of payeeIds to match;
+     */
+    QStringList payees() const;
+
     /**
       * This method returns whether a tag filter has been set,
       * and if so, it returns all the tags set in the filter.
@@ -443,6 +456,15 @@ public:
       */
     bool tags(QStringList& list) const;
 
+    /**
+     * This method returns all tag ids that have been
+     * added to the filter. If no tag filter has been set,
+     * an empty list is returned.
+     *
+     * @returns list of tag ids
+     */
+    QStringList tags() const;
+
     /**
       * This method returns whether an account filter has been set,
       * and if so, it returns all the accounts set in the filter.
@@ -452,6 +474,13 @@ public:
       */
     bool accounts(QStringList& list) const;
 
+    /**
+     * This method returns all accounts set in the filter.
+     *
+     * @return QStringList of payeeIds to match;
+     */
+    QStringList accounts() const;
+
     /**
       * This method returns whether a category filter has been set,
       * and if so, it returns all the categories set in the filter.
@@ -495,13 +524,15 @@ public:
     bool validities(QList<int>& list) const;
 
     /**
-      * This method returns whether a text filter has been set,
-      * and if so, it returns the text filter.
-      *
-      * @param text regexp to replace with text filter, or blank if none set
-      * @return return true if a text filter has been set
-      */
-    bool textFilter(QRegExp& text) const;
+     * This method returns whether a text filter has been set,
+     * and if so, it returns the text filter.
+     *
+     * @param text regexp to replace with text filter, or blank if none set
+     * @param isRegExp reference to bool which will return if @a text should
+     *                 be treated as regex (@c true) or wildcard (@c false)
+     * @return return true if a text filter has been set
+     */
+    bool textFilter(QRegularExpression& text, bool& isRegExp) const;
 
     /**
      * This method returns whether the text filter should return
