@@ -1,15 +1,19 @@
 /*
     SPDX-FileCopyrightText: 2013-2018 Christian Dávid <christian-david@web.de>
+    SPDX-FileCopyrightText: 2022      Thomas Baumgart <tbaumgart@kde.org>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
 #ifndef ONLINEJOBADMINISTRATION_H
 #define ONLINEJOBADMINISTRATION_H
 
+#include "kmm_mymoney_export.h"
+
 // ----------------------------------------------------------------------------
 // QT Includes
 
 #include <QMap>
+class QXmlStreamReader;
 
 // ----------------------------------------------------------------------------
 // KDE Includes
@@ -61,7 +65,6 @@ public:
 
     struct onlineJobEditOffer {
         QString fileName;
-        QString pluginKeyword;
         QString name;
     };
     using onlineJobEditOffers = QVector<onlineJobEditOffer>;
@@ -175,7 +178,7 @@ public:
     /**
      * @brief Are all preconditions set to edit the given job?
      */
-    bool canEditOnlineJob(const onlineJob& job);
+    bool canEditOnlineJob(const QString& jobId);
 
     /**
      * @brief See if a online task has a specified base
@@ -202,6 +205,7 @@ public:
      * @return pointer to task, caller gains ownership. Can be 0.
      */
     onlineTask* createOnlineTaskByXml(const QString& iid, const QDomElement& element) const;
+    onlineTask* createOnlineTaskByXml(QXmlStreamReader* reader, const QString& iid) const;
 
 Q_SIGNALS:
     /**
@@ -266,7 +270,7 @@ private:
      * @brief Get root instance of an onlineTask
      *
      * Returns a pointer from m_onlineTasks or tries to load/create
-     * a approiate root element.
+     * an appropriate root element.
      *
      * Only createOnlineTask and createOnlineTaskByXml use it.
      *
