@@ -5,9 +5,6 @@
 
 #include "payeeidentifier.h"
 
-#include <QDomDocument>
-#include <QDomElement>
-
 #include "payeeidentifierdata.h"
 
 payeeIdentifier::payeeIdentifier()
@@ -36,12 +33,12 @@ payeeIdentifier::payeeIdentifier(const payeeIdentifier::id_t& id, payeeIdentifie
 {
 }
 
-payeeIdentifier::payeeIdentifier(const QString& id, payeeIdentifierData*const identifierdata)
-    : m_id(id.mid(5).toUInt()),
-      m_payeeIdentifier(identifierdata)
+payeeIdentifier::payeeIdentifier(const QString& id, payeeIdentifierData* const identifierdata)
+    : m_id(QStringView{id}.mid(5).toUInt())
+    , m_payeeIdentifier(identifierdata)
 {
     bool ok = false; // hopefully the compiler optimizes this away if compiled in non-debug mode
-    Q_ASSERT(id.mid(5).toUInt(&ok) && ok);
+    Q_ASSERT(QStringView{id}.mid(5).toUInt(&ok) && ok);
 }
 
 payeeIdentifier::payeeIdentifier(const payeeIdentifier::id_t& id, const payeeIdentifier& other)
@@ -131,18 +128,3 @@ bool payeeIdentifier::operator==(const payeeIdentifier& other) const
     return (*data() == *(other.data()));
 }
 
-void payeeIdentifier::writeXML(QDomDocument& document, QDomElement& parent, const QString& elemenName) const
-{
-    // Important: type must be set before calling m_payeeIdentifier->writeXML()
-    // the plugin for unavailable plugins must be able to set type itself
-    QDomElement elem = document.createElement(elemenName);
-    if (m_id != 0)
-        elem.setAttribute("id", m_id);
-
-    if (!isNull()) {
-        elem.setAttribute("type", m_payeeIdentifier->payeeIdentifierId());
-        m_payeeIdentifier->writeXML(document, elem);
-    }
-    parent.appendChild(elem);
-}
-
