@@ -1,5 +1,5 @@
 /*
-    SPDX-FileCopyrightText: 2002-2018 Thomas Baumgart <tbaumgart@kde.org>
+    SPDX-FileCopyrightText: 2002-2019 Thomas Baumgart <tbaumgart@kde.org>
     SPDX-FileCopyrightText: 2004 Ace Jones <acejones@users.sourceforge.net>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
@@ -24,8 +24,7 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include "mymoneystoragemgr.h"
-#include "mymoneystoragemgr_p.h"
+#include "mymoneyfile.h"
 #include "mymoneyexception.h"
 #include "mymoneyinstitution.h"
 #include "mymoneyaccount.h"
@@ -40,6 +39,13 @@
 #include "mymoneysplit.h"
 #include "mymoneytransaction.h"
 #include "mymoneyenums.h"
+#include "parametersmodel.h"
+#include "accountsmodel.h"
+#include "journalmodel.h"
+#include "payeesmodel.h"
+#include "tagsmodel.h"
+#include "institutionsmodel.h"
+#include "schedulesmodel.h"
 
 MyMoneyStorageDump::MyMoneyStorageDump()
 {
@@ -49,16 +55,15 @@ MyMoneyStorageDump::~MyMoneyStorageDump()
 {
 }
 
-void MyMoneyStorageDump::readStream(QDataStream& /* s */, MyMoneyStorageMgr* /* storage */)
+void MyMoneyStorageDump::readStream(QDataStream& /* s */, MyMoneyFile* /* file */)
 {
     qDebug("Reading not supported by MyMoneyStorageDump!!");
 }
 
-void MyMoneyStorageDump::writeStream(QDataStream& _s, MyMoneyStorageMgr* _storage)
+void MyMoneyStorageDump::writeStream(QDataStream& _s, MyMoneyFile* file)
 {
     QTextStream s(_s.device());
-    MyMoneyStorageMgr* storage = _storage;
-    MyMoneyPayee user = storage->user();
+    MyMoneyPayee user = file->user();
 
     s << "File-Info\n";
     s << "---------\n";
@@ -69,27 +74,28 @@ void MyMoneyStorageDump::writeStream(QDataStream& _s, MyMoneyStorageMgr* _storag
     s << "user zip = " << user.postcode() << "\n";
     s << "user telephone = " << user.telephone() << "\n";
     s << "user e-mail = " << user.email() << "\n";
-    s << "creation date = " << storage->creationDate().toString(Qt::ISODate) << "\n";
-    s << "last modification date = " << storage->lastModificationDate().toString(Qt::ISODate) << "\n";
-    s << "base currency = " << storage->value("kmm-baseCurrency") << "\n";
+    s << "creation date = " << file->parametersModel()->itemById(file->fixedKey(MyMoneyFile::CreationDate)).value() << "\n";
+    s << "last modification date = " << file->parametersModel()->itemById(file->fixedKey(MyMoneyFile::LastModificationDate)).value() << "\n";
+    s << "base currency = " << file->value("kmm-baseCurrency") << "\n";
     s << "\n";
 
     s << "Internal-Info\n";
     s << "-------------\n";
     QList<MyMoneyAccount> list_a;
-    storage->accountList(list_a);
-    s << "accounts = " << list_a.count() << ", next id = " << _storage->d_func()->m_nextAccountID << "\n";
+    file->accountList(list_a);
+    s << "accounts = " << list_a.count() << ", next id = " << file->accountsModel()->peekNextId() << "\n";
     MyMoneyTransactionFilter filter;
     filter.setReportAllSplits(false);
     QList<MyMoneyTransaction> list_t;
-    storage->transactionList(list_t, filter);
-    QList<MyMoneyTransaction>::ConstIterator it_t;
-    s << "transactions = " << list_t.count() << ", next id = " << _storage->d_func()->m_nextTransactionID << "\n";
+    file->transactionList(list_t, filter);
+    QList<MyMoneyTransaction>::const_iterator it_t;
+    s << "transactions = " << list_t.count() << ", next id = " << file->journalModel()->peekNextId() << "\n";
     QMap<int, int> xferCount;
-    foreach (const auto transaction, list_t) {
+    for (const auto& transaction : qAsConst(list_t)) {
         auto accountCount = 0;
-        foreach (const auto split, transaction.splits()) {
-            auto acc = storage->account(split.accountId());
+        const auto splits = transaction.splits();
+        for (const auto& split : splits) {
+            auto acc = file->account(split.accountId());
             if (acc.accountGroup() != eMyMoney::Account::Type::Expense
                     && acc.accountGroup() != eMyMoney::Account::Type::Income)
                 accountCount++;
@@ -97,24 +103,24 @@ void MyMoneyStorageDump::writeStream(QDataStream& _s, MyMoneyStorageMgr* _storag
         if (accountCount > 1)
             xferCount[accountCount] = xferCount[accountCount] + 1;
     }
-    QMap<int, int>::ConstIterator it_cnt;
-    for (it_cnt = xferCount.constBegin(); it_cnt != xferCount.constEnd(); ++it_cnt) {
+    QMap<int, int>::const_iterator it_cnt;
+    for (it_cnt = xferCount.cbegin(); it_cnt != xferCount.cend(); ++it_cnt) {
         s << "               " << *it_cnt << " of them references " << it_cnt.key() << " accounts\n";
     }
 
-    s << "payees = " << _storage->payeeList().count() << ", next id = " << _storage->d_func()->m_nextPayeeID << "\n";
-    s << "tags = " << _storage->tagList().count() << ", next id = " << _storage->d_func()->m_nextTagID << "\n";
-    s << "institutions = " << _storage->institutionList().count() << ", next id = " << _storage->d_func()->m_nextInstitutionID << "\n";
-    s << "schedules = " << _storage->scheduleList(QString(), eMyMoney::Schedule::Type::Any, eMyMoney::Schedule::Occurrence::Any, eMyMoney::Schedule::PaymentType::Any,
-            QDate(), QDate(), false).count() << ", next id = " << _storage->d_func()->m_nextScheduleID << "\n";
+    s << "payees = " << file->payeeList().count() << ", next id = " << file->payeesModel()->peekNextId() << "\n";
+    s << "tags = " << file->tagList().count() << ", next id = " << file->tagsModel()->peekNextId() << "\n";
+    s << "institutions = " << file->institutionList().count() << ", next id = " << file->institutionsModel()->peekNextId() << "\n";
+    s << "schedules = " << file->scheduleList(QString(), eMyMoney::Schedule::Type::Any, eMyMoney::Schedule::Occurrence::Any, eMyMoney::Schedule::PaymentType::Any,
+            QDate(), QDate(), false).count() << ", next id = " << file->schedulesModel()->peekNextId() << "\n";
     s << "\n";
 
     s << "Institutions\n";
     s << "------------\n";
 
-    QList<MyMoneyInstitution> list_i = storage->institutionList();
-    QList<MyMoneyInstitution>::ConstIterator it_i;
-    for (it_i = list_i.constBegin(); it_i != list_i.constEnd(); ++it_i) {
+    QList<MyMoneyInstitution> list_i = file->institutionList();
+    QList<MyMoneyInstitution>::const_iterator it_i;
+    for (it_i = list_i.cbegin(); it_i != list_i.cend(); ++it_i) {
         s << "  ID = " << (*it_i).id() << "\n";
         s << "  Name = " << (*it_i).name() << "\n";
         s << "\n";
@@ -124,9 +130,9 @@ void MyMoneyStorageDump::writeStream(QDataStream& _s, MyMoneyStorageMgr* _storag
     s << "Payees" << "\n";
     s << "------" << "\n";
 
-    QList<MyMoneyPayee> list_p = storage->payeeList();
-    QList<MyMoneyPayee>::ConstIterator it_p;
-    for (it_p = list_p.constBegin(); it_p != list_p.constEnd(); ++it_p) {
+    QList<MyMoneyPayee> list_p = file->payeeList();
+    QList<MyMoneyPayee>::const_iterator it_p;
+    for (it_p = list_p.cbegin(); it_p != list_p.cend(); ++it_p) {
         s << "  ID = " << (*it_p).id() << "\n";
         s << "  Name = " << (*it_p).name() << "\n";
         s << "  Address = " << (*it_p).address() << "\n";
@@ -143,9 +149,9 @@ void MyMoneyStorageDump::writeStream(QDataStream& _s, MyMoneyStorageMgr* _storag
     s << "Tags" << "\n";
     s << "------" << "\n";
 
-    QList<MyMoneyTag> list_ta = storage->tagList();
-    QList<MyMoneyTag>::ConstIterator it_ta;
-    for (it_ta = list_ta.constBegin(); it_ta != list_ta.constEnd(); ++it_ta) {
+    QList<MyMoneyTag> list_ta = file->tagList();
+    QList<MyMoneyTag>::const_iterator it_ta;
+    for (it_ta = list_ta.cbegin(); it_ta != list_ta.cend(); ++it_ta) {
         s << "  ID = " << (*it_ta).id() << "\n";
         s << "  Name = " << (*it_ta).name() << "\n";
         s << "  Closed = " << (*it_ta).isClosed() << "\n";
@@ -158,13 +164,13 @@ void MyMoneyStorageDump::writeStream(QDataStream& _s, MyMoneyStorageMgr* _storag
     s << "Accounts" << "\n";
     s << "--------" << "\n";
 
-    list_a.push_front(storage->equity());
-    list_a.push_front(storage->expense());
-    list_a.push_front(storage->income());
-    list_a.push_front(storage->liability());
-    list_a.push_front(storage->asset());
-    QList<MyMoneyAccount>::ConstIterator it_a;
-    for (it_a = list_a.constBegin(); it_a != list_a.constEnd(); ++it_a) {
+    list_a.push_front(file->equity());
+    list_a.push_front(file->expense());
+    list_a.push_front(file->income());
+    list_a.push_front(file->liability());
+    list_a.push_front(file->asset());
+    QList<MyMoneyAccount>::const_iterator it_a;
+    for (it_a = list_a.cbegin(); it_a != list_a.cend(); ++it_a) {
         s << "  ID = " << (*it_a).id() << "\n";
         s << "  Name = " << (*it_a).name() << "\n";
         s << "  Number = " << (*it_a).number() << "\n";
@@ -174,14 +180,14 @@ void MyMoneyStorageDump::writeStream(QDataStream& _s, MyMoneyStorageMgr* _storag
             s << "  Currency = unknown\n";
         } else {
             if ((*it_a).isInvest()) {
-                s << "  Equity = " << storage->security((*it_a).currencyId()).name() << "\n";
+                s << "  Equity = " << file->security((*it_a).currencyId()).name() << "\n";
             } else {
-                s << "  Currency = " << storage->currency((*it_a).currencyId()).name() << "\n";
+                s << "  Currency = " << file->currency((*it_a).currencyId()).name() << "\n";
             }
         }
         s << "  Parent = " << (*it_a).parentAccountId();
         if (!(*it_a).parentAccountId().isEmpty()) {
-            MyMoneyAccount parent = storage->account((*it_a).parentAccountId());
+            MyMoneyAccount parent = file->account((*it_a).parentAccountId());
             s << " (" << parent.name() << ")";
         } else {
             s << "n/a";
@@ -190,7 +196,7 @@ void MyMoneyStorageDump::writeStream(QDataStream& _s, MyMoneyStorageMgr* _storag
 
         s << "  Institution = " << (*it_a).institutionId();
         if (!(*it_a).institutionId().isEmpty()) {
-            MyMoneyInstitution inst = storage->institution((*it_a).institutionId());
+            MyMoneyInstitution inst = file->institution((*it_a).institutionId());
             s << " (" << inst.name() << ")";
         } else {
             s << "n/a";
@@ -206,12 +212,12 @@ void MyMoneyStorageDump::writeStream(QDataStream& _s, MyMoneyStorageMgr* _storag
         dumpKVP("  OnlineBankingSettings: ", s, (*it_a).onlineBankingSettings());
 
         QStringList list_s = (*it_a).accountList();
-        QStringList::ConstIterator it_s;
+        QStringList::const_iterator it_s;
         if (list_s.count() > 0) {
             s << "  Children =" << "\n";
         }
-        for (it_s = list_s.constBegin(); it_s != list_s.constEnd(); ++it_s) {
-            MyMoneyAccount child = storage->account(*it_s);
+        for (it_s = list_s.cbegin(); it_s != list_s.cend(); ++it_s) {
+            MyMoneyAccount child = file->account(*it_s);
             s << "    " << *it_s << " (" << child.name() << ")\n";
         }
         s << "\n";
@@ -223,7 +229,7 @@ void MyMoneyStorageDump::writeStream(QDataStream& _s, MyMoneyStorageMgr* _storag
     s << "----------" << "\n";
 
     QList<MyMoneyCurrency> list_c = storage->currencyList();
-    QList<MyMoneyCurrency>::ConstIterator it_c;
+    QList<MyMoneyCurrency>::const_iterator it_c;
     for (it_c = list_c.begin(); it_c != list_c.end(); ++it_c) {
         s << "  Name = " << (*it_c).name() << "\n";
         s << "    ID = " << (*it_c).id() << "\n";
@@ -240,9 +246,9 @@ void MyMoneyStorageDump::writeStream(QDataStream& _s, MyMoneyStorageMgr* _storag
     s << "Securities" << "\n";
     s << "----------" << "\n";
 
-    QList<MyMoneySecurity> list_e = storage->securityList();
-    QList<MyMoneySecurity>::ConstIterator it_e;
-    for (it_e = list_e.constBegin(); it_e != list_e.constEnd(); ++it_e) {
+    QList<MyMoneySecurity> list_e = file->securityList();
+    QList<MyMoneySecurity>::const_iterator it_e;
+    for (it_e = list_e.cbegin(); it_e != list_e.cend(); ++it_e) {
         s << "  Name = " << (*it_e).name() << "\n";
         s << "    ID = " << (*it_e).id() << "\n";
         s << "    Market   = " << (*it_e).tradingMarket() << "\n";
@@ -251,7 +257,7 @@ void MyMoneyStorageDump::writeStream(QDataStream& _s, MyMoneyStorageMgr* _storag
         if ((*it_e).tradingCurrency().isEmpty()) {
             s << "unknown";
         } else {
-            MyMoneySecurity tradingCurrency = storage->currency((*it_e).tradingCurrency());
+            MyMoneySecurity tradingCurrency = file->currency((*it_e).tradingCurrency());
             if (!tradingCurrency.isCurrency()) {
                 s << "invalid currency: ";
             }
@@ -276,13 +282,13 @@ void MyMoneyStorageDump::writeStream(QDataStream& _s, MyMoneyStorageMgr* _storag
     s << "Prices" << "\n";
     s << "--------" << "\n";
 
-    MyMoneyPriceList list_pr = _storage->priceList();
-    MyMoneyPriceList::ConstIterator it_pr;
-    for (it_pr = list_pr.constBegin(); it_pr != list_pr.constEnd(); ++it_pr) {
+    MyMoneyPriceList list_pr = file->priceList();
+    MyMoneyPriceList::const_iterator it_pr;
+    for (it_pr = list_pr.cbegin(); it_pr != list_pr.cend(); ++it_pr) {
         s << "  From = " << it_pr.key().first << "\n";
         s << "    To = " << it_pr.key().second << "\n";
-        MyMoneyPriceEntries::ConstIterator it_pre;
-        for (it_pre = (*it_pr).constBegin(); it_pre != (*it_pr).constEnd(); ++it_pre) {
+        MyMoneyPriceEntries::const_iterator it_pre;
+        for (it_pre = (*it_pr).cbegin(); it_pre != (*it_pr).cend(); ++it_pre) {
             s << "      Date = " << (*it_pre).date().toString() << "\n";
             s << "        Price = " << (*it_pre).rate(QString()).formatMoney("", 8) << "\n";
             s << "        Source = " << (*it_pre).source() << "\n";
@@ -296,8 +302,8 @@ void MyMoneyStorageDump::writeStream(QDataStream& _s, MyMoneyStorageMgr* _storag
     s << "Transactions" << "\n";
     s << "------------" << "\n";
 
-    for (it_t = list_t.constBegin(); it_t != list_t.constEnd(); ++it_t) {
-        dumpTransaction(s, storage, *it_t);
+    for (it_t = list_t.cbegin(); it_t != list_t.cend(); ++it_t) {
+        dumpTransaction(s, file, *it_t);
     }
     s << "\n";
 
@@ -305,10 +311,10 @@ void MyMoneyStorageDump::writeStream(QDataStream& _s, MyMoneyStorageMgr* _storag
     s << "Schedules" << "\n";
     s << "---------" << "\n";
 
-    auto list_s = storage->scheduleList(QString(), eMyMoney::Schedule::Type::Any, eMyMoney::Schedule::Occurrence::Any, eMyMoney::Schedule::PaymentType::Any,
-                                        QDate(), QDate(), false);
-    QList<MyMoneySchedule>::ConstIterator it_s;
-    for (it_s = list_s.constBegin(); it_s != list_s.constEnd(); ++it_s) {
+    auto list_s = file->scheduleList(QString(), eMyMoney::Schedule::Type::Any, eMyMoney::Schedule::Occurrence::Any, eMyMoney::Schedule::PaymentType::Any,
+                                     QDate(), QDate(), false);
+    QList<MyMoneySchedule>::const_iterator it_s;
+    for (it_s = list_s.cbegin(); it_s != list_s.cend(); ++it_s) {
         s << "  ID = " << (*it_s).id() << "\n";
         s << "  Name = " << (*it_s).name() << "\n";
         s << "  Startdate = " << (*it_s).startDate().toString(Qt::ISODate) << "\n";
@@ -337,26 +343,26 @@ void MyMoneyStorageDump::writeStream(QDataStream& _s, MyMoneyStorageMgr* _storag
         }
 
         QList<QDate> list_d;
-        QList<QDate>::ConstIterator it_d;
+        QList<QDate>::const_iterator it_d;
 
         list_d = (*it_s).recordedPayments();
         if (list_d.count() > 0) {
             s << "  Recorded payments" << "\n";
-            for (it_d = list_d.constBegin(); it_d != list_d.constEnd(); ++it_d) {
+            for (it_d = list_d.cbegin(); it_d != list_d.cend(); ++it_d) {
                 s << "    " << (*it_d).toString(Qt::ISODate) << "\n";
             }
         }
         s << "  TRANSACTION\n";
-        dumpTransaction(s, storage, (*it_s).transaction());
+        dumpTransaction(s, file, (*it_s).transaction());
     }
     s << "\n";
 
     s << "Reports" << "\n";
     s << "-------" << "\n";
 
-    QList<MyMoneyReport> list_r = storage->reportList();
-    QList<MyMoneyReport>::ConstIterator it_r;
-    for (it_r = list_r.constBegin(); it_r != list_r.constEnd(); ++it_r) {
+    QList<MyMoneyReport> list_r = file->reportList();
+    QList<MyMoneyReport>::const_iterator it_r;
+    for (it_r = list_r.cbegin(); it_r != list_r.cend(); ++it_r) {
         s << "  ID = " << (*it_r).id() << "\n";
         s << "  Name = " << (*it_r).name() << "\n";
     }
@@ -364,9 +370,9 @@ void MyMoneyStorageDump::writeStream(QDataStream& _s, MyMoneyStorageMgr* _storag
     s << "Budgets" << "\n";
     s << "-------" << "\n";
 
-    QList<MyMoneyBudget> list_b = storage->budgetList();
-    QList<MyMoneyBudget>::ConstIterator it_b;
-    for (it_b = list_b.constBegin(); it_b != list_b.constEnd(); ++it_b) {
+    QList<MyMoneyBudget> list_b = file->budgetList();
+    QList<MyMoneyBudget>::const_iterator it_b;
+    for (it_b = list_b.cbegin(); it_b != list_b.cend(); ++it_b) {
         s << "  ID = " << (*it_b).id() << "\n";
         s << "  Name = " << (*it_b).name() << "\n";
     }
@@ -378,12 +384,13 @@ void MyMoneyStorageDump::dumpKVP(const QString& headline, QTextStream& s, const
     ind.fill(' ', indent);
     s << ind << headline << "\n";
     QMap<QString, QString>::const_iterator it;
-    for (it = kvp.pairs().constBegin(); it != kvp.pairs().constEnd(); ++it) {
+    const auto pairs = kvp.pairs();
+    for (it = pairs.cbegin(); it != pairs.cend(); ++it) {
         s << ind << "  '" << it.key() << "' = '" << it.value() << "'\n";
     }
 }
 
-void MyMoneyStorageDump::dumpTransaction(QTextStream& s, MyMoneyStorageMgr* storage, const MyMoneyTransaction& it_t)
+void MyMoneyStorageDump::dumpTransaction(QTextStream& s, MyMoneyFile* file, const MyMoneyTransaction& it_t)
 {
     s << "  ID = " << it_t.id() << "\n";
     s << "  Postdate  = " << it_t.postDate().toString(Qt::ISODate) << "\n";
@@ -395,19 +402,20 @@ void MyMoneyStorageDump::dumpTransaction(QTextStream& s, MyMoneyStorageMgr* stor
 
     s << "  Splits\n";
     s << "  ------\n";
-    foreach (const auto split, it_t.splits()) {
+    const auto splits = it_t.splits();
+    for (const auto& split : qAsConst(splits)) {
         s << "   ID = " << split.id() << "\n";
         s << "    Transaction = " << split.transactionId() << "\n";
         s << "    Payee = " << split.payeeId();
         if (!split.payeeId().isEmpty()) {
-            MyMoneyPayee p = storage->payee(split.payeeId());
+            MyMoneyPayee p = file->payee(split.payeeId());
             s << " (" << p.name() << ")" << "\n";
         } else
             s << " ()\n";
         for (int i = 0; i < split.tagIdList().size(); i++) {
-            s << "    Tag = " << split.tagIdList()[i];
-            if (!split.tagIdList()[i].isEmpty()) {
-                MyMoneyTag ta = storage->tag(split.tagIdList()[i]);
+            s << "    Tag = " << split.tagIdList().at(i);
+            if (!split.tagIdList().at(i).isEmpty()) {
+                MyMoneyTag ta = file->tag(split.tagIdList().at(i));
                 s << " (" << ta.name() << ")" << "\n";
             } else
                 s << " ()\n";
@@ -415,7 +423,7 @@ void MyMoneyStorageDump::dumpTransaction(QTextStream& s, MyMoneyStorageMgr* stor
         s << "    Account = " << split.accountId();
         MyMoneyAccount acc;
         try {
-            acc = storage->account(split.accountId());
+            acc = file->account(split.accountId());
             s << " (" << acc.name() << ") [" << acc.currencyId() << "]\n";
         } catch (const MyMoneyException &) {
             s << " (---) [---]\n";
@@ -441,8 +449,6 @@ void MyMoneyStorageDump::dumpTransaction(QTextStream& s, MyMoneyStorageMgr* stor
     s << "\n";
 }
 
-#define i18n QString
-
 const QString MyMoneyStorageDump::reconcileToString(eMyMoney::Split::State flag) const
 {
     QString rc;
