@@ -17,6 +17,7 @@
 // ----------------------------------------------------------------------------
 // KDE Includes
 
+#include <KLazyLocalizedString>
 #include <KLocalizedString>
 
 // ----------------------------------------------------------------------------
@@ -57,8 +58,8 @@ struct SchedulesModel::Private
     {
         ScheduleInfo rc;
         const auto transaction = schedule.transaction();
-        const auto s1 = (transaction.splits().size() < 1) ? MyMoneySplit() : transaction.splits()[0];
-        const auto s2 = (transaction.splits().size() < 2) ? MyMoneySplit() : transaction.splits()[1];
+        const auto s1 = (transaction.splits().size() < 1) ? MyMoneySplit() : transaction.splits().at(0);
+        const auto s2 = (transaction.splits().size() < 2) ? MyMoneySplit() : transaction.splits().at(1);
         MyMoneySplit split;
         MyMoneyAccount acc;
 
@@ -74,7 +75,8 @@ struct SchedulesModel::Private
         {
             auto found = false;
             QStringList list;
-            for (const auto& s : transaction.splits()) {
+            const auto splits = transaction.splits();
+            for (const auto& s : qAsConst(splits)) {
                 acc = MyMoneyFile::instance()->account(s.accountId());
                 list.append(acc.id());
                 if (acc.accountGroup() == eMyMoney::Account::Type::Asset
@@ -120,7 +122,7 @@ struct SchedulesModel::Private
     QString payee(const MyMoneySchedule& schedule) const
     {
         const auto transaction = schedule.transaction();
-        const auto s1 = (transaction.splits().size() < 1) ? MyMoneySplit() : transaction.splits()[0];
+        const auto s1 = (transaction.splits().size() < 1) ? MyMoneySplit() : transaction.splits().at(0);
         if (s1.payeeId().isEmpty()) {
             return {};
         }
@@ -158,7 +160,7 @@ void SchedulesModel::clearModelItems()
     // create the type entries
     insertRows(0, types.count());
     int row = 0;
-    foreach(auto type, types) {
+    for (const auto& type : types) {
         auto idx = index(row, 0);
         setData(idx, static_cast<int>(type.first), eMyMoney::Model::Roles::ScheduleTypeRole);
         setData(idx, type.second);
@@ -175,21 +177,21 @@ int SchedulesModel::columnCount(const QModelIndex& parent) const
 
 QVariant SchedulesModel::headerData(int section, Qt::Orientation orientation, int role) const
 {
-    const QVector<const char *> headers = {
-        I18N_NOOP2("Schedule header", "Type/Name"),
-        I18N_NOOP2("Schedule header", "Account"),
-        I18N_NOOP2("Schedule header", "Payee"),
-        I18N_NOOP2("Schedule header", "Amount"),
-        I18N_NOOP2("Schedule header", "Next Due Date"),
-        I18N_NOOP2("Schedule header", "Frequency"),
-        I18N_NOOP2("Schedule header", "Payment Method"),
+    const QVector<const char*> headers = {
+        kli18nc("Schedule header", "Type/Name").untranslatedText(),
+        kli18nc("Schedule header", "Account").untranslatedText(),
+        kli18nc("Schedule header", "Payee").untranslatedText(),
+        kli18nc("Schedule header", "Amount").untranslatedText(),
+        kli18nc("Schedule header", "Next Due Date").untranslatedText(),
+        kli18nc("Schedule header", "Frequency").untranslatedText(),
+        kli18nc("Schedule header", "Payment Method").untranslatedText(),
     };
 
     if(orientation == Qt::Horizontal && role == Qt::DisplayRole) {
         if (section < headers.count())
             return i18n(headers.at(section));
     }
-    return QAbstractItemModel::headerData(section, orientation, role);
+    return MyMoneyModelBase::headerData(section, orientation, role);
 }
 
 Qt::ItemFlags SchedulesModel::flags(const QModelIndex& index) const
@@ -197,7 +199,10 @@ Qt::ItemFlags SchedulesModel::flags(const QModelIndex& index) const
     if (index.parent().isValid()) {
         return (Qt::ItemIsEnabled | Qt::ItemIsSelectable);
     }
-    return Qt::ItemIsEnabled;
+    if (index.isValid())
+        return Qt::ItemIsEnabled;
+
+    return Qt::NoItemFlags;
 }
 
 QVariant SchedulesModel::data(const QModelIndex& idx, int role) const
@@ -236,14 +241,14 @@ QVariant SchedulesModel::data(const QModelIndex& idx, int role) const
 
         case Column::NextDueDate:
             if (!schedule.isFinished()) {
-                rc = MyMoneyUtils::formatDate(schedule.nextDueDate());
+                rc = MyMoneyUtils::formatDate(schedule.adjustedNextDueDate());
             } else {
                 rc = i18nc("Finished schedule", "Finished");
             }
             break;
 
         case Column::Frequency:
-            rc = i18n(schedule.occurrenceToString(schedule.occurrenceMultiplier(), schedule.occurrence()).toLatin1());
+            rc = schedule.occurrenceToString(schedule.occurrenceMultiplier(), schedule.occurrence());
             break;
 
         case Column::PaymentMethod:
@@ -311,7 +316,7 @@ QVariant SchedulesModel::data(const QModelIndex& idx, int role) const
         break;
 
     case eMyMoney::Model::ScheduleNextDueDateRole:
-        rc = schedule.nextDueDate();
+        rc = schedule.adjustedNextDueDate();
         break;
 
     case eMyMoney::Model::ScheduleFrequencyRole:
@@ -426,7 +431,7 @@ void SchedulesModel::doAddItem(const MyMoneySchedule& schedule, const QModelInde
     const QModelIndex idx = index(row, 0, group);
     static_cast<TreeItem<MyMoneySchedule>*>(idx.internalPointer())->dataRef() = sch
             setDirty();
-    emit dataChanged(idx, index(row, columnCount(group)-1, group));
+    Q_EMIT dataChanged(idx, index(row, columnCount(group)-1, group));
 #endif
 }
 
@@ -467,7 +472,8 @@ QList<MyMoneySchedule> SchedulesModel::scheduleList(const QString& accountId,
         if (!accountId.isEmpty()) {
             bool found = false;
             const MyMoneyTransaction& t = schedule.transaction();
-            foreach(const auto& split, t.splits()) {
+            const auto splits = t.splits();
+            for (const auto& split : qAsConst(splits)) {
                 if (split.accountId() == accountId) {
                     found = true;
                     break;
