@@ -10,12 +10,13 @@
 
 #define KMM_MYMONEY_UNIT_TESTABLE friend class MyMoneyAccountTest;
 
-#include "mymoneymoney.h"
 #include "mymoneyaccount.h"
 #include "mymoneyaccount_p.h"
-#include "mymoneysplit.h"
-#include "mymoneyexception.h"
+#include "mymoneyaccountloan.h"
 #include "mymoneyenums.h"
+#include "mymoneyexception.h"
+#include "mymoneymoney.h"
+#include "mymoneysplit.h"
 
 QTEST_GUILESS_MAIN(MyMoneyAccountTest)
 
@@ -55,7 +56,7 @@ void MyMoneyAccountTest::testConstructor()
     r.setParentAccountId(parent);
     r.setValue(QString("key"), "value");
     s.setShares(MyMoneyMoney::ONE);
-    r.setBalance(s.shares());
+    r.adjustBalance(s);
     QVERIFY(r.pairs().count() == 1);
     QVERIFY(r.value("key") == "value");
 
@@ -152,16 +153,53 @@ void MyMoneyAccountTest::testAssignmentConstructor()
 
     b = a;
 
-    QVERIFY(b.name() == "Account");
-    QVERIFY(b.institutionId() == "Inst1");
-    QVERIFY(b.accountType() == eMyMoney::Account::Type::Checkings);
-    QVERIFY(b.lastModified() == QDate());
-    QVERIFY(b.openingDate() == a.openingDate());
-    QVERIFY(b.description() == "Bla");
-    QVERIFY(b.number() == "assigned Number");
-    QVERIFY(b.value("Key") == "Value");
-    QVERIFY(b.accountList().count() == 1);
-    QVERIFY(b.accountList()[0] == "ChildAccount");
+    QCOMPARE(b.name(), "Account");
+    QCOMPARE(b.institutionId(), "Inst1");
+    QCOMPARE(b.accountType(), eMyMoney::Account::Type::Checkings);
+    QCOMPARE(b.lastModified(), QDate());
+    QCOMPARE(b.openingDate(), a.openingDate());
+    QCOMPARE(b.description(), "Bla");
+    QCOMPARE(b.number(), "assigned Number");
+    QCOMPARE(b.value("Key"), "Value");
+    QCOMPARE(b.accountList().count(), 1);
+    QCOMPARE(b.accountList().at(0), "ChildAccount");
+
+    MyMoneyAccountLoan c;
+
+    b.setLastModified(QDate::currentDate());
+
+    c = a;
+
+    QCOMPARE(c.name(), "Account");
+    QCOMPARE(c.institutionId(), "Inst1");
+    QCOMPARE(c.accountType(), eMyMoney::Account::Type::Checkings);
+    QCOMPARE(c.lastModified(), QDate());
+    QCOMPARE(c.openingDate(), a.openingDate());
+    QCOMPARE(c.description(), "Bla");
+    QCOMPARE(c.number(), "assigned Number");
+    QCOMPARE(c.value("Key"), "Value");
+    QCOMPARE(c.accountList().count(), 1);
+    QCOMPARE(c.accountList().at(0), "ChildAccount");
+}
+
+void MyMoneyAccountTest::testAdjustBalance()
+{
+    MyMoneyAccount a;
+    MyMoneySplit s;
+    s.setShares(MyMoneyMoney(3, 1));
+    a.adjustBalance(s);
+    QVERIFY(a.balance() == MyMoneyMoney(3, 1));
+    s.setShares(MyMoneyMoney(5, 1));
+    a.adjustBalance(s, true);
+    QVERIFY(a.balance() == MyMoneyMoney(-2, 1));
+    s.setShares(MyMoneyMoney(2, 1));
+    s.setAction(MyMoneySplit::actionName(eMyMoney::Split::Action::SplitShares));
+    a.adjustBalance(s);
+    QVERIFY(a.balance() == MyMoneyMoney(-4, 1));
+    s.setShares(MyMoneyMoney(4, 1));
+    s.setAction(QString());
+    a.adjustBalance(s);
+    QVERIFY(a.balance().isZero());
 }
 
 void MyMoneyAccountTest::testSubAccounts()
@@ -275,11 +313,11 @@ void MyMoneyAccountTest::testSetClosed()
 {
     MyMoneyAccount a;
 
-    QVERIFY(a.isClosed() == false);
+    QCOMPARE(a.isClosed(), false);
     a.setClosed(true);
-    QVERIFY(a.isClosed() == true);
+    QCOMPARE(a.isClosed(), true);
     a.setClosed(false);
-    QVERIFY(a.isClosed() == false);
+    QCOMPARE(a.isClosed(), false);
 }
 
 void MyMoneyAccountTest::specialAccountTypes_data()
@@ -342,15 +380,15 @@ void MyMoneyAccountTest::reconciliationHistory()
 
     QVERIFY(a.reconciliationHistory().isEmpty() == true);
     QVERIFY(a.addReconciliation(QDate(2011, 1, 2), MyMoneyMoney(123, 100)) == true);
-    QVERIFY(a.reconciliationHistory()[QDate(2011, 1, 2)] == MyMoneyMoney(123, 100));
-    QVERIFY(a.reconciliationHistory()[QDate(2011, 1, 1)] == MyMoneyMoney());
-    QVERIFY(a.reconciliationHistory()[QDate(2011, 1, 3)] == MyMoneyMoney());
+    QVERIFY(a.reconciliationHistory().value(QDate(2011, 1, 2)) == MyMoneyMoney(123, 100));
+    QVERIFY(a.reconciliationHistory().value(QDate(2011, 1, 1)) == MyMoneyMoney());
+    QVERIFY(a.reconciliationHistory().value(QDate(2011, 1, 3)) == MyMoneyMoney());
 
     QVERIFY(a.addReconciliation(QDate(2011, 2, 1), MyMoneyMoney(456, 100)) == true);
-    QVERIFY(a.reconciliationHistory()[QDate(2011, 1, 2)] == MyMoneyMoney(123, 100));
-    QVERIFY(a.reconciliationHistory()[QDate(2011, 1, 1)] == MyMoneyMoney());
-    QVERIFY(a.reconciliationHistory()[QDate(2011, 1, 3)] == MyMoneyMoney());
-    QVERIFY(a.reconciliationHistory()[QDate(2011, 2, 1)] == MyMoneyMoney(456, 100));
+    QVERIFY(a.reconciliationHistory().value(QDate(2011, 1, 2)) == MyMoneyMoney(123, 100));
+    QVERIFY(a.reconciliationHistory().value(QDate(2011, 1, 1)) == MyMoneyMoney());
+    QVERIFY(a.reconciliationHistory().value(QDate(2011, 1, 3)) == MyMoneyMoney());
+    QVERIFY(a.reconciliationHistory().value(QDate(2011, 2, 1)) == MyMoneyMoney(456, 100));
     QVERIFY(a.reconciliationHistory().count() == 2);
 }
 
@@ -363,3 +401,80 @@ void MyMoneyAccountTest::testHasOnlineMapping()
     a.setOnlineBankingSettings(kvp);
     QCOMPARE(a.hasOnlineMapping(), true);
 }
+
+void MyMoneyAccountTest::testBudgetOptions()
+{
+    MyMoneyAccount a;
+    a.setAccountType(eMyMoney::Account::Type::Asset);
+    QCOMPARE(a.accountType(), eMyMoney::Account::Type::Asset);
+    QCOMPARE(a.budgetAccountType(), eMyMoney::Account::Type::Asset);
+
+    a.setBudgetAccountType(eMyMoney::Account::Type::Income);
+    QCOMPARE(a.accountType(), eMyMoney::Account::Type::Asset);
+    QCOMPARE(a.budgetAccountType(), eMyMoney::Account::Type::Income);
+
+    a.setBudgetAccountType(eMyMoney::Account::Type::Expense);
+    QCOMPARE(a.accountType(), eMyMoney::Account::Type::Asset);
+    QCOMPARE(a.budgetAccountType(), eMyMoney::Account::Type::Expense);
+
+    a.setBudgetAccountType(eMyMoney::Account::Type::Investment);
+    QCOMPARE(a.accountType(), eMyMoney::Account::Type::Asset);
+    QCOMPARE(a.budgetAccountType(), eMyMoney::Account::Type::Asset);
+}
+
+void MyMoneyAccountTest::testBalanceFactor_data()
+{
+    QTest::addColumn<eMyMoney::Account::Type>("accountType");
+    QTest::addColumn<MyMoneyMoney>("factor");
+
+    // positive and null is debit
+    QTest::newRow("unknown") << eMyMoney::Account::Type::Unknown << MyMoneyMoney::ONE;
+    QTest::newRow("checking") << eMyMoney::Account::Type::Checkings << MyMoneyMoney::ONE;
+    QTest::newRow("savings") << eMyMoney::Account::Type::Savings << MyMoneyMoney::ONE;
+    QTest::newRow("cash") << eMyMoney::Account::Type::Cash << MyMoneyMoney::ONE;
+    QTest::newRow("investment") << eMyMoney::Account::Type::Investment << MyMoneyMoney::ONE;
+    QTest::newRow("asset") << eMyMoney::Account::Type::Asset << MyMoneyMoney::ONE;
+    QTest::newRow("currency") << eMyMoney::Account::Type::Currency << MyMoneyMoney::ONE;
+    QTest::newRow("moneymarket") << eMyMoney::Account::Type::MoneyMarket << MyMoneyMoney::ONE;
+    QTest::newRow("certificatedeposit") << eMyMoney::Account::Type::CertificateDep << MyMoneyMoney::ONE;
+    QTest::newRow("assetloan") << eMyMoney::Account::Type::AssetLoan << MyMoneyMoney::ONE;
+    QTest::newRow("stock") << eMyMoney::Account::Type::Stock << MyMoneyMoney::ONE;
+    QTest::newRow("income") << eMyMoney::Account::Type::Income << MyMoneyMoney::ONE;
+
+    QTest::newRow("expense") << eMyMoney::Account::Type::Expense << MyMoneyMoney::MINUS_ONE;
+    QTest::newRow("creditcard") << eMyMoney::Account::Type::CreditCard << MyMoneyMoney::MINUS_ONE;
+    QTest::newRow("liability") << eMyMoney::Account::Type::Liability << MyMoneyMoney::MINUS_ONE;
+    QTest::newRow("loan") << eMyMoney::Account::Type::Loan << MyMoneyMoney::MINUS_ONE;
+    QTest::newRow("equity") << eMyMoney::Account::Type::Equity << MyMoneyMoney::MINUS_ONE;
+}
+void MyMoneyAccountTest::testBalanceFactor()
+{
+    QFETCH(eMyMoney::Account::Type, accountType);
+    QFETCH(MyMoneyMoney, factor);
+
+    QCOMPARE(MyMoneyAccount::balanceFactor(accountType), factor);
+
+    MyMoneyAccount account;
+    account.setAccountType(accountType);
+    QCOMPARE(account.balanceFactor(), factor);
+}
+
+void MyMoneyAccountTest::testPayeeCreation_data()
+{
+    QTest::addColumn<eMyMoney::Account::PayeeCreation>("creationMode");
+
+    QTest::newRow("AskUser") << eMyMoney::Account::PayeeCreation::AskUser;
+    QTest::newRow("AutomaticCreaation") << eMyMoney::Account::PayeeCreation::AutomaticCreaation;
+    QTest::newRow("NoCreation") << eMyMoney::Account::PayeeCreation::NoCreation;
+    QTest::newRow("ApplicationDefault") << eMyMoney::Account::PayeeCreation::ApplicationDefault;
+}
+
+void MyMoneyAccountTest::testPayeeCreation()
+{
+    QFETCH(eMyMoney::Account::PayeeCreation, creationMode);
+
+    MyMoneyAccount account;
+    QCOMPARE(account.payeeCreation(), eMyMoney::Account::PayeeCreation::ApplicationDefault);
+    account.setPayeeCreation(creationMode);
+    QCOMPARE(account.payeeCreation(), creationMode);
+}
