@@ -29,12 +29,18 @@ private Q_SLOTS:
     void testSubAccounts();
     void testEquality();
     void testHasReferenceTo();
+    void testAdjustBalance();
     void testSetClosed();
     void specialAccountTypes();
     void specialAccountTypes_data();
     void addReconciliation();
     void reconciliationHistory();
     void testHasOnlineMapping();
+    void testBudgetOptions();
+    void testBalanceFactor();
+    void testBalanceFactor_data();
+    void testPayeeCreation();
+    void testPayeeCreation_data();
 };
 
 #endif
