@@ -14,7 +14,6 @@
 
 #include "mymoneyfile.h"
 #include "mymoneyaccount.h"
-#include "storage/mymoneystoragemgr.h"
 
 class MyMoneyFileTest : public QObject
 {
@@ -24,7 +23,6 @@ public:
 
 protected:
     MyMoneyFile *m;
-    MyMoneyStorageMgr* storage;
     MyMoneyAccount  m_inv;
 
 private Q_SLOTS:
@@ -55,16 +53,14 @@ private Q_SLOTS:
     void testModifyTransactionNewAccount();
     void testRemoveTransaction();
     void testBalanceTotal();
-    void testSetAccountName();
+    void testModifyStandardAccountNames();
     void testAddPayee();
     void testModifyPayee();
     void testRemovePayee();
     void testPayeeWithIdentifier();
     void testAddTransactionStd();
-    void testAttachStorage();
     void testAccount2Category();
     void testCategory2Account();
-    void testAttachedStorage();
     void testHasAccount();
     void testAddEquityAccount();
     void testReparentEquity();
@@ -111,7 +107,8 @@ private:
     void testRemoveStdAccount(const MyMoneyAccount& acc);
     void testReparentEquity(QList<eMyMoney::Account::Type>& list, MyMoneyAccount& parent);
     void clearObjectLists();
-    void AddOneAccount();
+    void addOneAccount();
+    void setupBaseCurrency();
 
 private:
     QStringList m_objectsAdded;
