@@ -44,6 +44,9 @@ private Q_SLOTS:
     void testReduce();
     void testZeroDenominator();
     void testMaxPrecision();
+    void testZeroIsMinusZero();
+    void testAutoCalcDetection();
+    void testBug491828();
 };
 
 #endif
