@@ -8,7 +8,6 @@
 #define MYMONEYTRANSACTIONFILTERTEST_H
 
 #include <QObject>
-class MyMoneyStorageMgr;
 class MyMoneyFile;
 
 class MyMoneyTransactionFilterTest : public QObject
@@ -17,7 +16,7 @@ class MyMoneyTransactionFilterTest : public QObject
 public:
     MyMoneyTransactionFilterTest();
 
-private slots:
+private Q_SLOTS:
     void init();
     void cleanup();
     void testMatchAmount();
@@ -42,7 +41,6 @@ private:
     QString acCheckingId;
     QString acExpenseId;
     QString acIncomeId;
-    MyMoneyStorageMgr* storage;
     MyMoneyFile* file;
 };
 
