@@ -10,7 +10,6 @@
 #include <QString>
 
 class MyMoneyFile;
-class MyMoneyStorageMgr;
 
 #define KMM_MYMONEY_UNIT_TESTABLE friend class onlineJobAdministrationTest;
 
@@ -18,7 +17,6 @@ class onlineJobAdministrationTest : public QObject
 {
     Q_OBJECT
 
-    MyMoneyStorageMgr* storage;
     MyMoneyFile* file;
     QString accountId;
 
@@ -31,6 +29,9 @@ private Q_SLOTS:
     void init();
     void getSettings();
     void registerOnlineTask();
+
+private:
+    void setupBaseCurrency();
 };
 
 #endif // ONLINEJOBADMINISTRATIONTEST_H
