@@ -1,5 +1,6 @@
 /*
     SPDX-FileCopyrightText: 2013-2016 Christian Dávid <christian-david@web.de>
+    SPDX-FileCopyrightText: 2019 Thomas Baumgart <tbaumgart@kde.org>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
@@ -21,14 +22,18 @@ class onlineTaskDummy3 : public onlineTask, public onlineTaskInterface
 public:
     ONLINETASK_META(dummyTask, "org.kmymoney.onlinetasks.dummytask");
 
-    bool isValid() const final override {
+    bool isValid() const final override
+    {
         return true;
     }
-    QString jobTypeName() const final override {
+    QString jobTypeName() const final override
+    {
         return QLatin1String("Dummy credit transfer");
     }
 
-    void writeXML(QDomDocument&, QDomElement&) const final override {}
+    void writeXML(QXmlStreamWriter*) const final override
+    {
+    }
 
 protected:
 
@@ -38,13 +43,21 @@ protected:
     bool hasReferenceTo(const QString&) const final override {
         return false;
     }
-
-    onlineTaskDummy3* createFromXml(const QDomElement &) const final override {
+    KMMStringSet referencedObjects() const final override
+    {
+        return {};
+    }
+    onlineTaskDummy3* createFromXml(QXmlStreamReader*) const final override
+    {
         return (new onlineTaskDummy3);
     }
     QString responsibleAccount() const final override {
         return QString();
     }
+    QString purpose() const final override
+    {
+        return QString();
+    }
 };
 
 void onlineJobTypedTest::initTestCase()
