@@ -1,6 +1,7 @@
 /*
     SPDX-FileCopyrightText: 2013-2014 Allan Anderson <agander93@gmail.com>
     SPDX-FileCopyrightText: 2019 Thomas Baumgart <tbaumgart@kde.org>
+    SPDX-FileCopyrightText: 2021 Dawid Wróbel <me@dawidwrobel.com>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
@@ -10,8 +11,6 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
-#include <QCoreApplication>
-#include <QStandardPaths>
 #include <QUrl>
 
 // ----------------------------------------------------------------------------
@@ -29,28 +28,19 @@
 
 #include "csvexportdlg.h"
 #include "csvwriter.h"
-#include "mymoneyutils.h"
+#include "kmymoneyplugin.h"
 #include "viewinterface.h"
 
-CSVExporter::CSVExporter(QObject *parent, const QVariantList &args)
-    : KMyMoneyPlugin::Plugin(parent, "csvexporter"/*must be the same as X-KDE-PluginInfo-Name*/)
+#include "kmmyesno.h"
+
+CSVExporter::CSVExporter(QObject *parent, const KPluginMetaData &metaData, const QVariantList &args)
+    : KMyMoneyPlugin::Plugin(parent, metaData, args)
     , m_action(nullptr)
 {
-    Q_UNUSED(args);
-    const auto componentName = QLatin1String("csvexporter");
+    Q_INIT_RESOURCE(csvexporter);
+
     const auto rcFileName = QLatin1String("csvexporter.rc");
-    setComponentName(componentName, i18n("CSV exporter"));
-
-    if (MyMoneyUtils::isRunningAsAppImage()) {
-        const QString rcFilePath = QString("%1/../share/kxmlgui5/%2/%3").arg(QCoreApplication::applicationDirPath(), componentName, rcFileName);
-        setXMLFile(rcFilePath);
-
-        const QString localRcFilePath =
-            QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation).first() + QLatin1Char('/') + componentName + QLatin1Char('/') + rcFileName;
-        setLocalXMLFile(localRcFilePath);
-    } else {
-        setXMLFile(rcFileName);
-    }
+    setXMLFile(rcFileName);
 
     createActions();
     // For information, announce that we have been loaded.
@@ -80,7 +70,6 @@ void CSVExporter::slotCsvExport()
             m_dlg->setWindowTitle(i18nc("CSV Exporter dialog title", "CSV Exporter"));
             CsvWriter* writer = new CsvWriter;
             writer->m_plugin = this;
-            connect(writer, &CsvWriter::signalProgress, m_dlg, &CsvExportDlg::slotStatusProgressBar);
 
             writer->write(m_dlg->filename(), m_dlg->accountId(),
                           m_dlg->accountSelected(), m_dlg->categorySelected(),
@@ -98,9 +87,8 @@ bool CSVExporter::okToWriteFile(const QUrl &url)
     bool fileExists = false;
 
     if (url.isValid()) {
-#if KIO_VERSION < QT_VERSION_CHECK(5, 70, 0)
-        short int detailLevel = 0; // Lowest level: file/dir/symlink/none
-        KIO::StatJob* statjob = KIO::stat(url, KIO::StatJob::SourceSide, detailLevel);
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+        auto statjob = KIO::stat(url, KIO::StatJob::SourceSide, KIO::StatNoDetails);
 #else
         auto statjob = KIO::statDetails(url, KIO::StatJob::SourceSide, KIO::StatNoDetails);
 #endif
@@ -112,12 +100,18 @@ bool CSVExporter::okToWriteFile(const QUrl &url)
     }
 
     if (fileExists) {
-        if (KMessageBox::warningYesNo(0, i18n("<qt>The file <b>%1</b> already exists. Do you really want to overwrite it?</qt>", url.toDisplayString(QUrl::PreferLocalFile)), i18n("File already exists")) != KMessageBox::Yes)
+        if (KMessageBox::warningTwoActions(
+                0,
+                i18n("<qt>The file <b>%1</b> already exists. Do you really want to overwrite it?</qt>", url.toDisplayString(QUrl::PreferLocalFile)),
+                i18n("File already exists"),
+                KMMYesNo::yes(),
+                KMMYesNo::no())
+            != KMessageBox::PrimaryAction)
             reallySaveFile = false;
     }
     return reallySaveFile;
 }
 
-K_PLUGIN_FACTORY_WITH_JSON(CSVExporterFactory, "csvexporter.json", registerPlugin<CSVExporter>();)
+K_PLUGIN_CLASS_WITH_JSON(CSVExporter, "csvexporter.json")
 
 #include "csvexporter.moc"
