@@ -22,17 +22,20 @@
 // ----------------------------------------------------------------------------
 // Project Headers
 
-#include "mymoneymoney.h"
-#include "mymoneyfile.h"
-#include "mymoneyaccount.h"
-#include "mymoneytransaction.h"
-#include "mymoneytransactionfilter.h"
-#include "mymoneysplit.h"
-#include "mymoneypayee.h"
-#include "mymoneyexception.h"
+#include "accountsmodel.h"
 #include "csvexportdlg.h"
 #include "csvexporter.h"
+#include "mymoneyaccount.h"
 #include "mymoneyenums.h"
+#include "mymoneyexception.h"
+#include "mymoneyfile.h"
+#include "mymoneymoney.h"
+#include "mymoneypayee.h"
+#include "mymoneysecurity.h"
+#include "mymoneysplit.h"
+#include "mymoneytransaction.h"
+#include "mymoneytransactionfilter.h"
+#include "securitiesmodel.h"
 
 CsvWriter::CsvWriter() :
     m_plugin(0),
@@ -56,7 +59,9 @@ void CsvWriter::write(const QString& filename,
     QFile csvFile(filename);
     if (csvFile.open(QIODevice::WriteOnly)) {
         QTextStream s(&csvFile);
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
         s.setCodec("UTF-8");
+#endif
 
         m_plugin->exporterDialog()->show();
         try {
@@ -67,7 +72,7 @@ void CsvWriter::write(const QString& filename,
             if (accountData) {
                 writeAccountEntry(s, accountId, startDate, endDate);
             }
-            emit signalProgress(-1, -1);
+            Q_EMIT signalProgress(-1, -1);
 
         } catch (const MyMoneyException &e) {
             KMessageBox::error(nullptr, i18n("Unexpected exception '%1'", QString::fromLatin1(e.what())));
@@ -77,7 +82,7 @@ void CsvWriter::write(const QString& filename,
         qDebug() << i18n("Export completed.\n");
         delete m_plugin->exporterDialog();  //  Can now delete as export finished
     } else {
-        KMessageBox::error(0, i18n("Unable to open file '%1' for writing", filename));
+        KMessageBox::error(0, i18n("Unable to open file '%1' for writing", filename).append(QString::fromLatin1(": ") + csvFile.errorString()));
     }
 }
 
@@ -91,34 +96,46 @@ void CsvWriter::writeAccountEntry(QTextStream& stream, const QString& accountId,
     MyMoneyTransactionFilter filter(accountId);
 
     QString type = account.accountTypeToString(account.accountType());
-    data = QString(i18n("Account Type:"));
+    data = i18n("Account Type:");
+    data += QString("%1").arg(type);
+    data += m_separator;
+
+    data += i18n("Account Name:");
+    data += QString("%1\n\n").arg(account.name());
 
     if (account.accountType() == eMyMoney::Account::Type::Investment) {
-        data += QString("%1\n\n").arg(type);
-        m_headerLine << QString(i18n("Date")) << QString(i18n("Security")) << QString(i18n("Action/Type")) << QString(i18n("Amount")) << QString(i18n("Quantity")) << QString(i18n("Price")) << QString(i18n("Interest")) << QString(i18n("Fees")) << QString(i18n("Account")) << QString(i18n("Memo")) << QString(i18n("Status"));
+        m_headerLine << i18nc("@title:column header in CSV export", "Date") << i18nc("@title:column header in CSV export", "Security")
+                     << i18nc("@title:column header in CSV export", "Symbol") << i18nc("@title:column header in CSV export", "Action/Type")
+                     << i18nc("@title:column header in CSV export", "Amount") << i18nc("@title:column header in CSV export", "Quantity")
+                     << i18nc("@title:column header in CSV export", "Price") << i18nc("@title:column header in CSV export", "Interest")
+                     << i18nc("@title:column header in CSV export", "Fees") << i18nc("@title:column header in CSV export", "Account")
+                     << i18nc("@title:column header in CSV export", "Memo") << i18nc("@title:column header in CSV export", "Status");
         data += m_headerLine.join(m_separator);
         extractInvestmentEntries(accountId, startDate, endDate);
     } else {
-        data += QString("%1\n\n").arg(type);
-        m_headerLine << QString(i18n("Date")) << QString(i18n("Payee")) << QString(i18n("Amount")) << QString(i18n("Account/Cat")) << QString(i18n("Memo")) << QString(i18n("Status")) << QString(i18n("Number"));
+        m_headerLine << i18nc("@title:column header in CSV export", "Date") << i18nc("@title:column header in CSV export", "Payee")
+                     << i18nc("@title:column header in CSV export", "Amount") << i18nc("@title:column header in CSV export", "Account/Cat")
+                     << i18nc("@title:column header in CSV export", "Memo") << i18nc("@title:column header in CSV export", "Status")
+                     << i18nc("@title:column header in CSV export", "Number");
         filter.setDateFilter(startDate, endDate);
 
-        QList<MyMoneyTransaction> trList = file->transactionList(filter);
-        QList<MyMoneyTransaction>::ConstIterator it;
-        signalProgress(0, trList.count());
+        QList<MyMoneyTransaction> trList;
+        file->transactionList(trList, filter);
+        QList<MyMoneyTransaction>::const_iterator it;
+        Q_EMIT signalProgress(0, trList.count());
         int count = 0;
         m_highestSplitCount = 0;
-        for (it = trList.constBegin(); it != trList.constEnd(); ++it) {
+        for (it = trList.cbegin(); it != trList.cend(); ++it) {
             writeTransactionEntry(*it, accountId, ++count);
             if (m_noError)
-                signalProgress(count, 0);
+                Q_EMIT signalProgress(count, 0);
         }
         data += m_headerLine.join(m_separator);
     }
 
     QString result;
-    QMap<QString, QString>::const_iterator it_map = m_map.constBegin();
-    while (it_map != m_map.constEnd()) {
+    auto it_map = m_map.cbegin();
+    while (it_map != m_map.cend()) {
         result += it_map.value();
         ++it_map;
     }
@@ -136,25 +153,26 @@ void CsvWriter::writeCategoryEntries(QTextStream &s)
     expense = file->expense();
 
     QStringList list = income.accountList() + expense.accountList();
-    emit signalProgress(0, list.count());
+    Q_EMIT signalProgress(0, list.count());
     QStringList::Iterator it_catList;
     int count = 0;
     for (it_catList = list.begin(); it_catList != list.end(); ++it_catList) {
         writeCategoryEntry(s, *it_catList, "");
-        emit signalProgress(++count, 0);
+        Q_EMIT signalProgress(++count, 0);
     }
 }
 
 void CsvWriter::writeCategoryEntry(QTextStream &s, const QString& accountId, const QString& leadIn)
 {
-    MyMoneyAccount acc = MyMoneyFile::instance()->account(accountId);
+    const MyMoneyAccount acc = MyMoneyFile::instance()->account(accountId);
     QString name = format(acc.name());
 
     s << leadIn << name;
     s << (acc.accountGroup() == eMyMoney::Account::Type::Expense ? QLatin1Char('E') : QLatin1Char('I'));
-    s << endl;
+    s << Qt::endl;
 
-    foreach (const auto sAccount, acc.accountList())
+    const auto accountList = acc.accountList();
+    for (const auto& sAccount : qAsConst(accountList))
         writeCategoryEntry(s, sAccount, name);
 }
 
@@ -167,7 +185,7 @@ void CsvWriter::writeTransactionEntry(const MyMoneyTransaction& t, const QString
     MyMoneySplit split = t.splitByAccount(accountId);
     QList<MyMoneySplit> splits = t.splits();
     if (splits.count() < 2) {
-        KMessageBox::sorry(0, i18n("Transaction number '%1' is missing an account assignment.\n"
+        KMessageBox::error(0, i18n("Transaction number '%1' is missing an account assignment.\n"
                                    "Date '%2', Payee '%3'.\nTransaction dropped.\n", count, t.postDate().toString(Qt::ISODate), file->payee(split.payeeId()).name()),
                            i18n("Invalid transaction"));
         m_noError = false;
@@ -181,7 +199,7 @@ void CsvWriter::writeTransactionEntry(const MyMoneyTransaction& t, const QString
     MyMoneyPayee payee = file->payee(split.payeeId());
     str += format(payee.name());
 
-    str += format(split.value());
+    str += format(split.value(), split.accountId());
 
     if (splits.count() > 1) {
         MyMoneySplit sp = t.splitByAccount(accountId, false);
@@ -208,15 +226,15 @@ void CsvWriter::writeTransactionEntry(const MyMoneyTransaction& t, const QString
     str += format(split.number(), false);
 
     if (splits.count() > 2) {
-        QList<MyMoneySplit>::ConstIterator it;
-        for (it = splits.constBegin(); it != splits.constEnd(); ++it) {
+        QList<MyMoneySplit>::const_iterator it;
+        for (it = splits.cbegin(); it != splits.cend(); ++it) {
             if (!((*it) == split)) {
-                writeSplitEntry(str, *it, splits.count() - 1, it+1 == splits.constEnd());
+                writeSplitEntry(str, *it, splits.count() - 1, it + 1 == splits.cend());
             }
         }
     }
     QString date = t.postDate().toString(Qt::ISODate);
-    m_map.insertMulti(date, str);
+    m_map.insert(date, str);
 }
 
 void CsvWriter::writeSplitEntry(QString &str, const MyMoneySplit& split, const int splitCount, const int lastEntry)
@@ -230,28 +248,31 @@ void CsvWriter::writeSplitEntry(QString &str, const MyMoneySplit& split, const i
 
     if (splitCount > m_highestSplitCount) {
         m_highestSplitCount++;
-        m_headerLine << QString(i18n("splitCategory")) << QString(i18n("splitMemo")) << QString(i18n("splitAmount"));
+        m_headerLine << i18nc("@title:column header in CSV export", "splitCategory") << i18nc("@title:column header in CSV export", "splitMemo")
+                     << i18nc("@title:column header in CSV export", "splitAmount");
         m_headerLine.join(m_separator);
     }
     str += format(split.memo());
 
-    str += format(split.value(), 2, !lastEntry);
+    str += format(split.value(), split.accountId(), !lastEntry);
 }
 
 void CsvWriter::extractInvestmentEntries(const QString& accountId, const QDate& startDate, const QDate& endDate)
 {
     MyMoneyFile* file = MyMoneyFile::instance();
 
-    foreach (const auto sAccount, file->account(accountId).accountList()) {
+    const auto accountList = file->account(accountId).accountList();
+    for (const auto& sAccount : qAsConst(accountList)) {
         MyMoneyTransactionFilter filter(sAccount);
         filter.setDateFilter(startDate, endDate);
-        QList<MyMoneyTransaction> list = file->transactionList(filter);
-        QList<MyMoneyTransaction>::ConstIterator itList;
-        signalProgress(0, list.count());
+        QList<MyMoneyTransaction> list;
+        file->transactionList(list, filter);
+        QList<MyMoneyTransaction>::const_iterator itList;
+        Q_EMIT signalProgress(0, list.count());
         int count = 0;
-        for (itList = list.constBegin(); itList != list.constEnd(); ++itList) {
+        for (itList = list.cbegin(); itList != list.cend(); ++itList) {
             writeInvestmentEntry(*itList, ++count);
-            signalProgress(count, 0);
+            Q_EMIT signalProgress(count, 0);
         }
     }
 }
@@ -262,6 +283,7 @@ void CsvWriter::writeInvestmentEntry(const MyMoneyTransaction& t, const int coun
     QString strAmount;
     QString strPrice;
     QString strAccName;
+    QString strAccSymbol;
     QString strCheckingAccountName;
     QString strMemo;
     QString strAction;
@@ -277,6 +299,7 @@ void CsvWriter::writeInvestmentEntry(const MyMoneyTransaction& t, const int coun
     MyMoneyMoney qty;
     MyMoneyMoney value;
     QMap<eMyMoney::Account::Type, QString> map;
+    MyMoneySecurity security;
 
     for (int i = 0; i < lst.count(); i++) {
         MyMoneyAccount acc = file->account(lst[i].accountId());
@@ -313,21 +336,21 @@ void CsvWriter::writeInvestmentEntry(const MyMoneyTransaction& t, const int coun
             chkAccntId = (*itSplit).accountId();
             chkAccnt = file->account(chkAccntId).name();
             strCheckingAccountName = format(file->accountToCategory(chkAccntId));
-            strAmount = format((*itSplit).value());
+            strAmount = format((*itSplit).value(), (*itSplit).accountId());
         } else if (acc.accountType() == eMyMoney::Account::Type::Income) {
             //
             //  eMyMoney::Account::Type::Income.
             //
             qty = (*itSplit).shares();
             value = (*itSplit).value();
-            strInterest = format(value);
+            strInterest = format(value, (*itSplit).accountId());
         } else if (acc.accountType() == eMyMoney::Account::Type::Expense) {
             //
             //  eMyMoney::Account::Type::Expense.
             //
             qty = (*itSplit).shares();
             value = (*itSplit).value();
-            strFees = format(value);
+            strFees = format(value, (*itSplit).accountId());
         }  else if (acc.accountType() == eMyMoney::Account::Type::Stock) {
             //
             //  eMyMoney::Account::Type::Stock.
@@ -343,7 +366,7 @@ void CsvWriter::writeInvestmentEntry(const MyMoneyTransaction& t, const int coun
             }
             if ((strAction == QLatin1String("DivX")) || (strAction == QLatin1String("IntIncX"))) {
                 if ((map.value(eMyMoney::Account::Type::Checkings).isEmpty()) && (map.value(eMyMoney::Account::Type::Cash).isEmpty())) {
-                    KMessageBox::sorry(0, i18n("Transaction number '%1' is missing an account assignment.\n"
+                    KMessageBox::error(0, i18n("Transaction number '%1' is missing an account assignment.\n"
                                                "Date '%2', Amount '%3'.\nTransaction dropped.\n", count, t.postDate().toString(Qt::ISODate), strAmount),
                                        i18n("Invalid transaction"));
                     return;
@@ -364,11 +387,15 @@ void CsvWriter::writeInvestmentEntry(const MyMoneyTransaction& t, const int coun
                 }
             } else if ((*itSplit).action() == QLatin1String("Reinvest")) {
                 qty = (*itSplit).shares();
-                strAmount = format((*itSplit).value());
+                strAmount = format((*itSplit).value(), (*itSplit).accountId());
                 strAction = QLatin1String("ReinvDiv");
             } else {
                 strAction = (*itSplit).action();
             }
+
+            strAccName = format(acc.name());
+            security = file->security(acc.currencyId());
+            strAccSymbol = format(security.tradingSymbol());
             //
             //  Add action.
             //
@@ -378,19 +405,17 @@ void CsvWriter::writeInvestmentEntry(const MyMoneyTransaction& t, const int coun
                 // TODO: value is not used below
                 if (strAction == QLatin1String("Sell")) {
                     value = -value;
+                    qty = -qty;
                 }
                 //
                 //  Add price.
                 //
-                strPrice = format((*itSplit).price(), 6);
+                strPrice = format((*itSplit).price(), security.pricePrecision());
                 if (!qty.isZero()) {
                     //
                     //  Add quantity.
                     //
-                    if (strAction == QLatin1String("Sell")) {
-                        qty = -qty;
-                    }
-                    strQuantity = format(qty);
+                    strQuantity = format(qty, MyMoneyMoney::denomToPrec(acc.fraction()));
                 }
             } else if ((strAction == QLatin1String("Shrsin")) || (strAction == QLatin1String("Shrsout"))) {
                 //
@@ -399,9 +424,8 @@ void CsvWriter::writeInvestmentEntry(const MyMoneyTransaction& t, const int coun
                 if (strAction == QLatin1String("Shrsout")) {
                     qty = -qty;
                 }
-                strQuantity = format(qty);
+                strQuantity = format(qty, MyMoneyMoney::denomToPrec(acc.fraction()));
             }
-            strAccName = format(acc.name());
             strAction += m_separator;
         }
 
@@ -424,9 +448,9 @@ void CsvWriter::writeInvestmentEntry(const MyMoneyTransaction& t, const int coun
             strFees = m_separator;
         }
     }  //  end of itSplit loop
-    str += strAccName + strAction + strAmount + strQuantity + strPrice + strInterest + strFees + strCheckingAccountName + strMemo + strStatus;
+    str += strAccName + strAccSymbol + strAction + strAmount + strQuantity + strPrice + strInterest + strFees + strCheckingAccountName + strMemo + strStatus;
     QString date = t.postDate().toString(Qt::ISODate);
-    m_map.insertMulti(date, str);
+    m_map.insert(date, str);
 }
 
 /**
@@ -435,17 +459,13 @@ void CsvWriter::writeInvestmentEntry(const MyMoneyTransaction& t, const int coun
  * @param withSeparator append field separator to string (default = true)
  * @return csv formatted string
  */
-QString CsvWriter::format(const QString &s, bool withSeparator)
+QString CsvWriter::format(const QString& s, bool withSeparator) const
 {
     if (s.isEmpty())
         return withSeparator ? m_separator : QString();
     QString m = s;
     m.remove('\'');
-#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
     m.replace(QLatin1Char('"'), QStringLiteral("\"\""));
-#else
-    m.replace(QLatin1Char('"'), QLatin1String("\"\""));
-#endif
     return QString("\"%1\"%2").arg(m, withSeparator ? m_separator : QString());
 }
 
@@ -456,7 +476,13 @@ QString CsvWriter::format(const QString &s, bool withSeparator)
  * @param withSeparator append field separator to string (default = true)
  * @return formatted value as string
  */
-QString CsvWriter::format(const MyMoneyMoney &value, int prec, bool withSeparator)
+QString CsvWriter::format(const MyMoneyMoney& value, int prec, bool withSeparator) const
 {
     return QString("\"%1\"%2").arg(value.formatMoney("", prec, false), withSeparator ? m_separator : QString());
 }
+
+QString CsvWriter::format(const MyMoneyMoney& value, const QString& accountId, bool withSeparator) const
+{
+    const auto account = MyMoneyFile::instance()->account(accountId);
+    return format(value, MyMoneyMoney::denomToPrec(account.fraction()), withSeparator);
+}
