@@ -2,6 +2,7 @@
     SPDX-FileCopyrightText: 2010 Allan Anderson <agander93@gmail.com>
     SPDX-FileCopyrightText: 2017-2018 Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
     SPDX-FileCopyrightText: 2020 Thomas Baumgart <tbaumgart@kde.org>
+    SPDX-FileCopyrightText: 2021 Dawid Wróbel <me@dawidwrobel.com>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
@@ -10,30 +11,30 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
-#include <QTextCodec>
-#include <QTextStream>
 #include <QFileDialog>
+#include <QPointer>
 #include <QRegularExpression>
 #include <QStandardItem>
-#include <QPointer>
+#include <QTextCodec>
+#include <QTextStream>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
 
+#include <KConfigGroup>
 #include <KLocalizedString>
 #include <KMessageBox>
-#include <KConfigGroup>
 
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include "mymoneyfile.h"
+#include "convdate.h"
+#include "csvutil.h"
 #include "mymoneyaccount.h"
+#include "mymoneyenums.h"
+#include "mymoneyfile.h"
 #include "mymoneysecurity.h"
 #include "mymoneytransaction.h"
-#include "csvutil.h"
-#include "convdate.h"
-#include "mymoneyenums.h"
 
 const QHash<Profile, QString> CSVImporterCore::m_profileConfPrefix {
     {Profile::Banking, QStringLiteral("Bank")},
@@ -98,13 +99,13 @@ const QString CSVImporterCore::m_confProfileNames = QStringLiteral("ProfileNames
 const QString CSVImporterCore::m_confPriorName = QStringLiteral("Prior");
 const QString CSVImporterCore::m_confMiscName = QStringLiteral("Misc");
 
-CSVImporterCore::CSVImporterCore() :
-    m_profile(0),
-    m_isActionTypeValidated(false)
+CSVImporterCore::CSVImporterCore()
+    : m_convertDate(new ConvertDate)
+    , m_file(new CSVFile)
+    , m_profile(nullptr)
+    , m_isActionTypeValidated(false)
+    , m_sortOrder(OrderUnknown)
 {
-    m_convertDate = new ConvertDate;
-    m_file = new CSVFile;
-
     m_priceFractions << MyMoneyMoney(0.01) << MyMoneyMoney(0.1) << MyMoneyMoney::ONE << MyMoneyMoney(10) << MyMoneyMoney(100);
 
     validateConfigFile();
@@ -424,7 +425,7 @@ bool CSVImporterCore::validateDateFormat(const int col)
 bool CSVImporterCore::validateDecimalSymbols(const QList<int> &columns)
 {
     bool isOK = true;
-    foreach (const auto column, columns) {
+    for (const auto& column : columns) {
         m_file->m_parse->setDecimalSymbol(m_decimalSymbolIndexMap.value(column));
 
         for (int row = m_profile->m_startLine; row <= m_profile->m_endLine; ++row) {
@@ -437,7 +438,6 @@ bool CSVImporterCore::validateDecimalSymbols(const QList<int> &columns)
                 break;
             }
         }
-
     }
     return isOK;
 }
@@ -468,8 +468,8 @@ bool CSVImporterCore::validateSecurity(const InvestmentProfile *profile)
 
 bool CSVImporterCore::validateSecurities()
 {
-    QSet<QString> onlySymbols;
-    QSet<QString> onlyNames;
+    KMMStringSet onlySymbols;
+    KMMStringSet onlyNames;
     sortSecurities(onlySymbols, onlyNames, m_mapSymbolName);
 
     if (!onlySymbols.isEmpty() || !onlyNames.isEmpty())
@@ -483,13 +483,16 @@ eMyMoney::Transaction::Action CSVImporterCore::processActionTypeField(const Inve
         return eMyMoney::Transaction::Action::None;
 
     QString type = m_file->m_model->item(row, col)->text();
-    QList<eMyMoney::Transaction::Action> actions;
-    actions << eMyMoney::Transaction::Action::Buy << eMyMoney::Transaction::Action::Sell <<                       // first and second most frequent action
-            eMyMoney::Transaction::Action::ReinvestDividend << eMyMoney::Transaction::Action::CashDividend <<  // we don't want "reinv-dividend" to be accidentally caught by "dividend"
-            eMyMoney::Transaction::Action::Interest <<
-            eMyMoney::Transaction::Action::Shrsin << eMyMoney::Transaction::Action::Shrsout;
+    const QList<eMyMoney::Transaction::Action> actions{
+        eMyMoney::Transaction::Action::Buy,
+        eMyMoney::Transaction::Action::Sell, // first and second most frequent action
+        eMyMoney::Transaction::Action::ReinvestDividend,
+        eMyMoney::Transaction::Action::CashDividend, // we don't want "reinv-dividend" to be accidentally caught by "dividend"
+        eMyMoney::Transaction::Action::Interest,
+        eMyMoney::Transaction::Action::Shrsin,
+        eMyMoney::Transaction::Action::Shrsout};
 
-    foreach (const auto action, actions) {
+    for (const auto& action : actions) {
         if (profile->m_transactionNames.value(action).contains(type, Qt::CaseInsensitive))
             return action;
     }
@@ -586,7 +589,7 @@ DecimalSymbol CSVImporterCore::detectDecimalSymbol(const int col, const QString
     DecimalSymbol detectedSymbol = DecimalSymbol::Auto;
     QString pattern;
 
-    QRegularExpression re("^[\\(+-]?\\d+[\\)]?$"); // matches '0' ; '+12' ; '-345' ; '(6789)'
+    static const QRegularExpression re("^[\\(+-]?\\d+[\\)]?$"); // matches '0' ; '+12' ; '-345' ; '(6789)'
 
     bool dotIsDecimalSeparator = false;
     bool commaIsDecimalSeparator = false;
@@ -667,7 +670,7 @@ int CSVImporterCore::detectDecimalSymbols(const QList<int> &columns)
     MyMoneyFile *file = MyMoneyFile::instance();
     file->accountList(accounts);
 
-    QList<eMyMoney::Account::Type> accountTypes{
+    const QList<eMyMoney::Account::Type> accountTypes{
         eMyMoney::Account::Type::Checkings,
         eMyMoney::Account::Type::Savings,
         eMyMoney::Account::Type::Liability,
@@ -680,17 +683,17 @@ int CSVImporterCore::detectDecimalSymbols(const QList<int> &columns)
         eMyMoney::Account::Type::Liability,
     };
 
-    QSet<QString> currencySymbols;
-    foreach (const auto account, accounts) {
+    KMMStringSet currencySymbols;
+    for (const auto& account : qAsConst(accounts)) {
         if (accountTypes.contains(account.accountType())) {                             // account must actually have currency property
             currencySymbols.insert(account.currencyId());                                 // add currency id
             currencySymbols.insert(file->currency(account.currencyId()).tradingSymbol()); // add currency symbol
         }
     }
-    QString filteredCurrencies = QStringList(currencySymbols.values()).join("");
-    QString pattern = QString::fromLatin1("%1%2").arg(QLocale().currencySymbol()).arg(filteredCurrencies);
+    QString filteredCurrencies = currencySymbols.values().join(QString());
+    QString pattern = QString::fromLatin1("%1%2").arg(QLocale().currencySymbol(), filteredCurrencies);
 
-    foreach (const auto column, columns) {
+    for (const auto& column : columns) {
         DecimalSymbol detectedSymbol = detectDecimalSymbol(column, pattern);
         if (detectedSymbol == DecimalSymbol::Auto) {
             ret = column;
@@ -703,88 +706,74 @@ int CSVImporterCore::detectDecimalSymbols(const QList<int> &columns)
 
 QList<MyMoneyAccount> CSVImporterCore::findAccounts(const QList<eMyMoney::Account::Type> &accountTypes, const QString &statementHeader)
 {
-    MyMoneyFile* file = MyMoneyFile::instance();
     QList<MyMoneyAccount> accountList;
-    file->accountList(accountList);
     QList<MyMoneyAccount> filteredTypes;
-    QList<MyMoneyAccount> filteredAccounts;
+    QMultiMap<int, MyMoneyAccount> accountsMatchingNumber;
+    QMultiMap<int, MyMoneyAccount> accountsMatchingName;
+    QMultiMap<int, MyMoneyAccount> accountsMatchingBoth;
+    QList<MyMoneyAccount> accountsMatching;
     QRegularExpression filterOutChars(QStringLiteral("[-., ]"));
 
-    foreach (const auto account, accountList) {
+    MyMoneyFile* file = MyMoneyFile::instance();
+    file->accountList(accountList);
+
+    for (const auto& account : qAsConst(accountList)) {
         if (accountTypes.contains(account.accountType()) && !(account).isClosed())
             filteredTypes.append(account);
     }
 
-    // filter out accounts whose names aren't in statements header
-    foreach (const auto account, filteredTypes) {
-        QString txt = account.name();
-        txt.remove(filterOutChars);
-        if (txt.isEmpty() || txt.length() < 3)
-            continue;
-        if (statementHeader.contains(txt, Qt::CaseInsensitive))
-            filteredAccounts.append(account);
+    // filter out accounts with matching numbers, names, and both
+    for (const auto& account : qAsConst(filteredTypes)) {
+        bool matchedNumber = false;
+        bool matchedName = false;
+
+        auto number = account.number();
+        number.remove(filterOutChars);
+        if (number.length() > 2 && statementHeader.contains(number, Qt::CaseInsensitive)) {
+            accountsMatchingNumber.insert(number.length(), account);
+            matchedNumber = true;
+        }
+
+        auto name = account.name();
+        name.remove(filterOutChars);
+        if (name.length() > 2 && statementHeader.contains(name, Qt::CaseInsensitive)) {
+            accountsMatchingName.insert(name.length(), account);
+            matchedName = true;
+        }
+
+        if (matchedNumber && matchedName)
+            accountsMatchingBoth.insert(number.length() + name.length(), account);
     }
 
-    // if filtering returned more results, filter out accounts whose numbers aren't in statements header
-    if (filteredAccounts.count() > 1) {
-        for (auto account = filteredAccounts.begin(); account != filteredAccounts.end();) {
-            QString txt = (*account).number();
-            txt.remove(filterOutChars);
-            if (txt.isEmpty() || txt.length() < 3) {
-                ++account;
-                continue;
-            }
-            if (statementHeader.contains(txt, Qt::CaseInsensitive))
-                ++account;
-            else
-                account = filteredAccounts.erase(account);
-        }
+    // the logic is:
+    // if a single account matching number is found, return that
+    if (accountsMatchingNumber.count() == 1)
+        return accountsMatchingNumber.values();
+    // if more than one account matching number is found
+    else if (accountsMatchingNumber.count() > 1) {
+        // see if there are any accounts that matched both number and name
+        // if one matching both was found, return it
+        if (accountsMatchingBoth.count() == 1)
+            return accountsMatchingBoth.values();
+        // if more than one was found still, return ones that have the longest sum(name, account)
+        else if (accountsMatchingBoth.count() > 1)
+            accountsMatching = accountsMatchingBoth.values(accountsMatchingBoth.lastKey());
+        else
+            // if neither account matched both name and number, return the ones that have the longest number
+            accountsMatching = accountsMatchingNumber.values(accountsMatchingNumber.lastKey());
+    }
+    // if none of the accounts matched a number, try with accounts that matched a name
+    else {
+        // if one account matched name, return it
+        if (accountsMatchingName.count() == 1)
+            return accountsMatchingName.values();
+        // if more than one account matched name, return the ones that have the longest name
+        else if (accountsMatchingName.count() > 1)
+            accountsMatching = accountsMatchingName.values(accountsMatchingName.lastKey());
     }
 
-    // if filtering returned more results, filter out accounts whose numbers are the shortest
-    if (filteredAccounts.count() > 1) {
-        for (auto i = 1; i < filteredAccounts.count();) {
-            auto firstAccNumber = filteredAccounts.at(0).number();
-            auto secondAccNumber = filteredAccounts.at(i).number();
-            if (firstAccNumber.length() > secondAccNumber.length()) {
-                filteredAccounts.removeAt(i);
-            } else if (firstAccNumber.length() < secondAccNumber.length()) {
-                filteredAccounts.removeAt(0);
-                --i;
-            } else {
-                ++i;
-            }
-        }
-    }
-
-    // if filtering returned more results, filter out accounts whose names are the shortest
-    if (filteredAccounts.count() > 1) {
-        for (auto i = 1; i < filteredAccounts.count();) {
-            auto firstAccName = filteredAccounts.at(0).name();
-            auto secondAccName = filteredAccounts.at(i).name();
-            if (firstAccName.length() > secondAccName.length()) {
-                filteredAccounts.removeAt(i);
-            } else if (firstAccName.length() < secondAccName.length()) {
-                filteredAccounts.removeAt(0);
-                --i;
-            } else {
-                ++i;
-            }
-        }
-    }
-
-    // if filtering by name and number didn't return nothing, then try filtering by number only
-    if (filteredAccounts.isEmpty()) {
-        foreach (const auto account, filteredTypes) {
-            QString txt = account.number();
-            txt.remove(filterOutChars);
-            if (txt.isEmpty() || txt.length() < 3)
-                continue;
-            if (statementHeader.contains(txt, Qt::CaseInsensitive))
-                filteredAccounts.append(account);
-        }
-    }
-    return filteredAccounts;
+    // otherwise, return accountsMatching, which may be empty if none of the accounts matched
+    return accountsMatching;
 }
 
 bool CSVImporterCore::detectAccount(MyMoneyStatement &st)
@@ -903,19 +892,7 @@ bool CSVImporterCore::processBankRow(MyMoneyStatement &st, const BankingProfile
             QRegularExpression exp;
             exp.setPatternOptions(QRegularExpression::CaseInsensitiveOption);
 
-#if QT_VERSION < QT_VERSION_CHECK(5, 12, 0)
-            auto pattern = profile->m_creditIndicator;
-            // simplified version of QRegularExpression::wildcardToRegularExpression()
-            // for older Qt versions where the method does not exist
-            pattern.replace(QLatin1String("\\"), QLatin1String("\\\\"));
-            pattern.replace(QLatin1String("."), QLatin1String("\\."));
-            pattern.replace(QLatin1String("*"), QLatin1String(".*"));
-            pattern.replace(QLatin1String("?"), QLatin1String("."));
-            pattern.prepend(QLatin1String("\\A"));
-            pattern.append(QLatin1String("\\z"));
-#else
             auto pattern = QRegularExpression::wildcardToRegularExpression(profile->m_creditIndicator);
-#endif
             exp.setPattern(pattern);
 
             if (exp.match(indicator).hasMatch()) {
@@ -962,16 +939,40 @@ bool CSVImporterCore::processBankRow(MyMoneyStatement &st, const BankingProfile
     }
 
     // process balance field
+    //
+    // The order in which transactions are included may be in ascending
+    // or descending order. That means, the current balance could be in
+    // the first record we process or the last. The following logic takes
+    // care of detecting the order by evaluating the order of the postdate
+    // entry. In case that all transactions are on one day, we cannot
+    // tell which entry to use and leave it at its initial value which
+    // informs the application that no online balance is available.
     col = profile->m_colTypeNum.value(Column::Balance, -1);
     if (col != -1) {
         // prior date than the one we have? Adjust it
         if (!st.m_dateBegin.isValid() || st.m_dateBegin > tr.m_datePosted) {
             st.m_dateBegin = tr.m_datePosted;
         }
-        // later or equal date, adjust it and the closing balance
-        if (!st.m_dateEnd.isValid() || st.m_dateEnd <= tr.m_datePosted) {
+
+        const auto balance = processAmountField(profile, row, col);
+
+        // in case we have no idea, we start with current values
+        if (!st.m_dateEnd.isValid()) {
             st.m_dateEnd = tr.m_datePosted;
-            st.m_closingBalance = processAmountField(profile, row, col);
+            m_firstBalance = balance;
+        }
+
+        // transaction on the same date?
+        if (st.m_dateEnd == tr.m_datePosted) {
+            if (m_sortOrder == OrderAscending) {
+                st.m_closingBalance = balance;
+            }
+        } else if (st.m_dateEnd < tr.m_datePosted) {
+            m_sortOrder = OrderAscending;
+            st.m_closingBalance = balance;
+        } else if (m_sortOrder == OrderUnknown) {
+            m_sortOrder = OrderDescending;
+            st.m_closingBalance = m_firstBalance;
         }
     }
 
@@ -983,10 +984,10 @@ bool CSVImporterCore::processBankRow(MyMoneyStatement &st, const BankingProfile
                        .arg(tr.m_datePosted.toString(Qt::ISODate))
                        .arg(MyMoneyTransaction::hash(txt));
     QString hash;
-    for (uchar idx = 0; idx < 0xFF; ++idx) {  // assuming threre will be no more than 256 transactions with the same hashBase
+    for (uchar idx = 0; idx < 0xFF; ++idx) {  // assuming there will be no more than 256 transactions with the same hashBase
         hash = QString::fromLatin1("%1-%2").arg(hashBase).arg(idx);
-        QSet<QString>::const_iterator it = m_hashSet.constFind(hash);
-        if (it == m_hashSet.constEnd())
+        KMMStringSet::const_iterator it = m_hashSet.find(hash);
+        if (it == m_hashSet.cend())
             break;
     }
     m_hashSet.insert(hash);
@@ -1179,9 +1180,9 @@ MyMoneyMoney CSVImporterCore::processCreditDebit(QString &credit, QString &debit
 
     if (!credit.isEmpty() && !debit.isEmpty()) {  // we do not expect both fields to be non-zero
         if (MyMoneyMoney(credit).isZero())
-            credit = QString();
+            credit.clear();
         if (MyMoneyMoney(debit).isZero())
-            debit = QString();
+            debit.clear();
     }
 
     if (!debit.startsWith(QLatin1Char('-')) && !debit.isEmpty()) // ensure debit field is negative
@@ -1291,10 +1292,9 @@ QList<eMyMoney::Transaction::Action> CSVImporterCore::createValidActionTypes(MyM
     return validActionTypes;
 }
 
-
-bool CSVImporterCore::sortSecurities(QSet<QString>& onlySymbols, QSet<QString>& onlyNames, QMap<QString, QString>& mapSymbolName)
+bool CSVImporterCore::sortSecurities(KMMStringSet& onlySymbols, KMMStringSet& onlyNames, QMap<QString, QString>& mapSymbolName)
 {
-    QList<MyMoneySecurity> securityList = MyMoneyFile::instance()->securityList();
+    const QList<MyMoneySecurity> securityList = MyMoneyFile::instance()->securityList();
     int symbolCol = m_profile->m_colTypeNum.value(Column::Symbol, -1);
     int nameCol = m_profile->m_colTypeNum.value(Column::Name, -1);
 
@@ -1318,9 +1318,9 @@ bool CSVImporterCore::sortSecurities(QSet<QString>& onlySymbols, QSet<QString>&
     }
 
     // try to find names for symbols
-    for (QSet<QString>::iterator symbol = onlySymbols.begin(); symbol != onlySymbols.end();) {
+    for (KMMStringSet::iterator symbol = onlySymbols.begin(); symbol != onlySymbols.end();) {
         QList<MyMoneySecurity> filteredSecurities;
-        foreach (const auto sec, securityList) {
+        for (const auto& sec : securityList) {
             if ((*symbol).compare(sec.tradingSymbol(), Qt::CaseInsensitive) == 0)
                 filteredSecurities.append(sec);      // gather all securities that by matched by symbol
         }
@@ -1337,9 +1337,9 @@ bool CSVImporterCore::sortSecurities(QSet<QString>& onlySymbols, QSet<QString>&
     }
 
     // try to find symbols for names
-    for (QSet<QString>::iterator name = onlyNames.begin(); name != onlyNames.end();) {
+    for (KMMStringSet::iterator name = onlyNames.begin(); name != onlyNames.end();) {
         QList<MyMoneySecurity> filteredSecurities;
-        foreach (const auto sec, securityList) {
+        for (const auto& sec : securityList) {
             if ((*name).compare(sec.name(), Qt::CaseInsensitive) == 0)
                 filteredSecurities.append(sec);      // gather all securities that by matched by name
         }
@@ -1372,6 +1372,9 @@ QList<int> CSVImporterCore::getNumericalColumns()
             columns << m_profile->m_colTypeNum.value(Column::Debit);
             columns << m_profile->m_colTypeNum.value(Column::Credit);
         }
+        if (m_profile->m_colTypeNum.value(Column::Balance, -1) != -1) {
+            columns << m_profile->m_colTypeNum.value(Column::Balance);
+        }
         break;
     case Profile::Investment:
         columns << m_profile->m_colTypeNum.value(Column::Amount);
@@ -1419,15 +1422,14 @@ bool CSVImporterCore::createStatement(MyMoneyStatement &st)
         st.m_eType = eMyMoney::Statement::Type::Investment;
 
         auto profile = dynamic_cast<InvestmentProfile *>(m_profile);
-        if (profile->m_autoAccountName)
-            detectAccount(st);
-
-        if ((m_profile->m_colTypeNum.value(Column::Fee, -1) == -1 ||
-                m_profile->m_colTypeNum.value(Column::Fee, -1) >= m_file->m_columnCount) &&
-                profile && !profile->m_feeRate.isEmpty()) // fee column has not been calculated so do it now
-            calculateFee();
-
         if (profile) {
+            if (profile->m_autoAccountName)
+                detectAccount(st);
+
+            if ((m_profile->m_colTypeNum.value(Column::Fee, -1) == -1 || m_profile->m_colTypeNum.value(Column::Fee, -1) >= m_file->m_columnCount) && profile
+                && !profile->m_feeRate.isEmpty()) // fee column has not been calculated so do it now
+                calculateFee();
+
             for (int row = m_profile->m_startLine; row <= m_profile->m_endLine; ++row)
                 if (!processInvestRow(st, profile, row)) { // parse fields
                     st = MyMoneyStatement();
@@ -1572,19 +1574,19 @@ bool InvestmentProfile::readSettings(const KSharedConfigPtr &config)
         exists = false;
 
     m_transactionNames[eMyMoney::Transaction::Action::Buy] = profilesGroup.readEntry(CSVImporterCore::m_transactionConfName.value(eMyMoney::Transaction::Action::Buy),
-            QString(i18nc("Type of operation as in financial statement", "buy")).split(',', QString::SkipEmptyParts));
+            QString(i18nc("Type of operation as in financial statement", "buy")).split(',', Qt::SkipEmptyParts));
     m_transactionNames[eMyMoney::Transaction::Action::Sell] = profilesGroup.readEntry(CSVImporterCore::m_transactionConfName.value(eMyMoney::Transaction::Action::Sell),
-            QString(i18nc("Type of operation as in financial statement", "sell,repurchase")).split(',', QString::SkipEmptyParts));
+            QString(i18nc("Type of operation as in financial statement", "sell,repurchase")).split(',', Qt::SkipEmptyParts));
     m_transactionNames[eMyMoney::Transaction::Action::ReinvestDividend] = profilesGroup.readEntry(CSVImporterCore::m_transactionConfName.value(eMyMoney::Transaction::Action::ReinvestDividend),
-            QString(i18nc("Type of operation as in financial statement", "reinvest,reinv,re-inv")).split(',', QString::SkipEmptyParts));
+            QString(i18nc("Type of operation as in financial statement", "reinvest,reinv,re-inv")).split(',', Qt::SkipEmptyParts));
     m_transactionNames[eMyMoney::Transaction::Action::CashDividend] = profilesGroup.readEntry(CSVImporterCore::m_transactionConfName.value(eMyMoney::Transaction::Action::CashDividend),
-            QString(i18nc("Type of operation as in financial statement", "dividend")).split(',', QString::SkipEmptyParts));
+            QString(i18nc("Type of operation as in financial statement", "dividend")).split(',', Qt::SkipEmptyParts));
     m_transactionNames[eMyMoney::Transaction::Action::Interest] = profilesGroup.readEntry(CSVImporterCore::m_transactionConfName.value(eMyMoney::Transaction::Action::Interest),
-            QString(i18nc("Type of operation as in financial statement", "interest,income")).split(',', QString::SkipEmptyParts));
+            QString(i18nc("Type of operation as in financial statement", "interest,income")).split(',', Qt::SkipEmptyParts));
     m_transactionNames[eMyMoney::Transaction::Action::Shrsin] = profilesGroup.readEntry(CSVImporterCore::m_transactionConfName.value(eMyMoney::Transaction::Action::Shrsin),
-            QString(i18nc("Type of operation as in financial statement", "add,stock dividend,divd reinv,transfer in,re-registration in,journal entry")).split(',', QString::SkipEmptyParts));
+            QString(i18nc("Type of operation as in financial statement", "add,stock dividend,divd reinv,transfer in,re-registration in,journal entry")).split(',', Qt::SkipEmptyParts));
     m_transactionNames[eMyMoney::Transaction::Action::Shrsout] = profilesGroup.readEntry(CSVImporterCore::m_transactionConfName.value(eMyMoney::Transaction::Action::Shrsout),
-            QString(i18nc("Type of operation as in financial statement", "remove")).split(',', QString::SkipEmptyParts));
+            QString(i18nc("Type of operation as in financial statement", "remove")).split(',', Qt::SkipEmptyParts));
 
     m_colTypeNum[Column::Date] = profilesGroup.readEntry(CSVImporterCore::m_colTypeConfName.value(Column::Date), -1);
     m_colTypeNum[Column::Type] = profilesGroup.readEntry(CSVImporterCore::m_colTypeConfName.value(Column::Type), -1);  //use for type col.
@@ -1719,6 +1721,12 @@ void CSVFile::getStartEndRow(CSVProfile *profile)
 
     if (profile->m_startLine > profile->m_endLine)   // Don't allow m_startLine > m_endLine
         profile->m_startLine = profile->m_endLine;
+
+    if (profile->m_startLine < 0) // Don't allow m_startLine negative
+        profile->m_startLine = 0;
+
+    if (profile->m_endLine < 0) // Don't allow m_endLine negative
+        profile->m_endLine = 0;
 }
 
 void CSVFile::getColumnCount(CSVProfile *profile, const QStringList &rows)
@@ -1738,8 +1746,8 @@ void CSVFile::getColumnCount(CSVProfile *profile, const QStringList &rows)
     FieldDelimiter possibleDelimiter = FieldDelimiter::Comma;
     m_columnCount = 0;
 
-    foreach (const auto row, rows) {
-        foreach(const auto delimiterIndex, delimiterIndexes) {
+    for (const auto& row : rows) {
+        for (const auto& delimiterIndex : qAsConst(delimiterIndexes)) {
             m_parse->setFieldDelimiter(delimiterIndex);
             colCount = m_parse->parseLine(row).count(); //  parse each line using each delimiter
 
@@ -1786,7 +1794,7 @@ bool CSVFile::getInFileName(QString inFileName)
 
     QPointer<QFileDialog> dialog = new QFileDialog(nullptr, QString(),
             fileInfo.absoluteFilePath());
-    dialog->setMimeTypeFilters({"text/csv", "text/plain", "application/octet-stream"});
+    dialog->setMimeTypeFilters({"text/csv", "text/tab-separated-values", "text/plain", "application/octet-stream"});
     dialog->setFileMode(QFileDialog::ExistingFile);
     QUrl url;
     if (dialog->exec() == QDialog::Accepted)
@@ -1816,25 +1824,34 @@ void CSVFile::readFile(CSVProfile *profile)
     if (!inFile.exists())
         return;
     inFile.open(QIODevice::ReadOnly);
-    QTextStream inStream(&inFile);
     QTextCodec* codec = QTextCodec::codecForMib(profile->m_encodingMIBEnum);
+#if QT_VERSION > QT_VERSION_CHECK(6, 0, 0)
+    QByteArray encodedString = inFile.readAll();
+    QString buf = codec->toUnicode(encodedString);
+#else
+    QTextStream inStream(&inFile);
     inStream.setCodec(codec);
 
     QString buf = inStream.readAll();
+#endif
     inFile.close();
+    m_model->clear();
     m_parse->setTextDelimiter(profile->m_textDelimiter);
     QStringList rows = m_parse->parseFile(buf);        // parse the buffer
     m_rowCount = m_parse->lastLine();                  // won't work without above line
     getColumnCount(profile, rows);
     getStartEndRow(profile);
 
-    // prepare model from rows having rowCount and columnCount
-    m_model->clear();
-    for (int i = 0; i < m_rowCount; ++i) {
-        QList<QStandardItem*> itemList;
-        QStringList columns = m_parse->parseLine(rows.takeFirst());  // take instead of read from rows to preserve memory
-        for (int j = 0; j < m_columnCount; ++j)
-            itemList.append(new QStandardItem(columns.value(j, QString())));
-        m_model->appendRow(itemList);
+    // prepare model from rows having rowCount and columnCount,
+    // but only if there is at least one row
+    if (m_rowCount > 0) {
+        for (int i = 0; i < m_rowCount; ++i) {
+            QList<QStandardItem*> itemList;
+            // use take instead of read from rows to preserve memory
+            QStringList columns = m_parse->parseLine(rows.takeFirst());
+            for (int j = 0; j < m_columnCount; ++j)
+                itemList.append(new QStandardItem(columns.value(j, QString())));
+            m_model->appendRow(itemList);
+        }
     }
 }
