@@ -13,19 +13,20 @@ class BankingProfile;
 class PricesProfile;
 class InvestmentProfile;
 class MyMoneyFile;
-class MyMoneyStorageMgr;
 
 class CSVImporterCoreTest : public QObject
 {
     Q_OBJECT
 
+public:
+    ~CSVImporterCoreTest() = default;
+
     CSVImporterCore     *csvImporter;
     BankingProfile      *debitCreditProfile;
     BankingProfile      *amountProfile;
     PricesProfile       *pricesProfile;
     InvestmentProfile   *investmentProfile;
     MyMoneyFile         *file;
-    MyMoneyStorageMgr *storage;
 private Q_SLOTS:
     void initTestCase();
     void init();
@@ -41,5 +42,8 @@ private Q_SLOTS:
     void testAutoDecimalSymbol();
     void testInvAccountAutodetection();
     void testCalculatedFeeColumn();
+
+private:
+    void setupBaseCurrency();
 };
 #endif
