@@ -1,5 +1,6 @@
 /*
     SPDX-FileCopyrightText: 2014 Christian Dávid <christian-david@web.de>
+    SPDX-FileCopyrightText: 2021 Dawid Wróbel <me@dawidwrobel.com>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
@@ -35,7 +36,7 @@ class ibanBicData : public KMyMoneyPlugin::Plugin, public KMyMoneyPlugin::DataPl
     KMM_MYMONEY_UNIT_TESTABLE
 
 public:
-    explicit ibanBicData(QObject *parent, const QVariantList &args);
+    explicit ibanBicData(QObject *parent, const KPluginMetaData &metaData, const QVariantList &args);
     ~ibanBicData() override;
 
     QVariant requestData(const QString &arg, uint type) override;
