@@ -11,17 +11,19 @@
 #include <QBoxLayout>
 
 // KDE includes
-#include <KPluginFactory>
-#include <KLocalizedString>
 #include <KAboutData>
+#include <KLocalizedString>
+#include <KPluginFactory>
 
-#include "pluginsettings.h"
-#include "ui_pluginsettingsdecl.h"
+#include "icalendarsettings.h"
+#include "ui_icalendarsettingsdecl.h"
 
-class PluginSettingsWidget : public QWidget, public Ui::PluginSettingsDecl
+class PluginSettingsWidget : public QWidget, public Ui::ICalendarSettingsDecl
 {
 public:
-    PluginSettingsWidget(QWidget* parent = 0) : QWidget(parent) {
+    explicit PluginSettingsWidget(QWidget* parent = 0)
+        : QWidget(parent)
+    {
         setupUi(this);
 
         kcfg_timeUnitInSeconds->addItem(i18n("Minutes"));
@@ -37,22 +39,13 @@ public:
     }
 };
 
-KCMiCalendarExporter::KCMiCalendarExporter(QWidget *parent, const QVariantList& args)
-    : KCModule(parent, args)
+KCMiCalendarExporter::KCMiCalendarExporter(QObject* parent, const QVariantList& args)
+    : KMMKCModule(parent, args)
 {
-    KAboutData *about = new KAboutData(QStringLiteral("kmm_printcheck"),
-                                       i18n("KMyMoney print check"),
-                                       QStringLiteral(VERSION), QString(),
-                                       KAboutLicense::GPL,
-                                       i18n("Copyright 2009" ) );
-    about->addAuthor( QString::fromUtf8("Cristian Oneț") );
-
-    setAboutData( about );
-
-    PluginSettingsWidget *w = new PluginSettingsWidget(this);
-    addConfig(PluginSettings::self(), w);
+    PluginSettingsWidget* w = new PluginSettingsWidget(widget());
+    addConfig(ICalendarSettings::self(), w);
     QVBoxLayout *layout = new QVBoxLayout;
-    setLayout(layout);
+    widget()->setLayout(layout);
     layout->addWidget(w);
     load();
 }
@@ -61,9 +54,6 @@ KCMiCalendarExporter::~KCMiCalendarExporter()
 {
 }
 
-K_PLUGIN_FACTORY_WITH_JSON(KCMiCalendarExporterFactory,
-                           "kcm_icalendarexporter.json",
-                           registerPlugin<KCMiCalendarExporter>();
-                          )
+K_PLUGIN_CLASS(KCMiCalendarExporter)
 
 #include "kcm_icalendarexporter.moc"
