@@ -1,5 +1,6 @@
 /*
     SPDX-FileCopyrightText: 2009 Cristian Onet onet.cristian @gmail.com
+    SPDX-FileCopyrightText: 2023 Thomas Baumgart <tbaumgart@kde.org>
     SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 */
 
@@ -18,19 +19,20 @@
 #include <libical/ical.h>
 
 // KMyMoney includes
-#include "mymoneymoney.h"
-#include "mymoneyfile.h"
-#include "mymoneysecurity.h"
-#include "mymoneyutils.h"
-#include "mymoneyschedule.h"
 #include "mymoneyaccount.h"
+#include "mymoneyenums.h"
+#include "mymoneyfile.h"
+#include "mymoneymoney.h"
+#include "mymoneypayee.h"
+#include "mymoneyschedule.h"
+#include "mymoneysecurity.h"
 #include "mymoneysplit.h"
 #include "mymoneytransaction.h"
-#include "mymoneypayee.h"
-#include "mymoneyenums.h"
+#include "mymoneyutils.h"
+#include "schedulesmodel.h"
 
 // plugin includes
-#include "pluginsettings.h"
+#include "icalendarsettings.h"
 
 using namespace eMyMoney;
 
@@ -166,7 +168,7 @@ struct icalrecurrencetype scheduleToRecurenceRule(const MyMoneySchedule& schedul
     case Schedule::Occurrence::Once:
     case Schedule::Occurrence::Any:
     default:
-        qWarning() << "Once, any or unknown recurrence returned recurrence is invalid" << endl;
+        qWarning() << "Once, any or unknown recurrence returned recurrence is invalid" << Qt::endl;
         recurrence.freq = ICAL_NO_RECURRENCE;
         break;
     }
@@ -186,12 +188,13 @@ QString scheduleToDescription(const MyMoneySchedule& schedule)
     QString category;
     bool isTransfer = false;
     bool isIncome = false;
-    foreach (const auto split, transaction.splits()) {
+    const auto splits = transaction.splits();
+    for (const auto& split : splits) {
         if (split.accountId() != account.id()) {
             if (!category.isEmpty())
                 category += ", "; // this is a split transaction
             const MyMoneyAccount& splitAccount = file->account(split.accountId());
-            category = splitAccount.name();
+            category += splitAccount.name();
 
             isTransfer = splitAccount.accountGroup() == Account::Type::Asset ||
                          splitAccount.accountGroup() == Account::Type::Liability;
@@ -218,7 +221,8 @@ struct KMMSchedulesToiCalendar::Private {
     QString m_icalendarAsString;
 };
 
-KMMSchedulesToiCalendar::KMMSchedulesToiCalendar() : d(new Private)
+KMMSchedulesToiCalendar::KMMSchedulesToiCalendar()
+    : d(new Private)
 {
 }
 
@@ -227,13 +231,11 @@ KMMSchedulesToiCalendar::~KMMSchedulesToiCalendar()
     delete d;
 }
 
-void KMMSchedulesToiCalendar::exportToFile(const QString& filePath, bool settingsChaged)
+void KMMSchedulesToiCalendar::exportToFile(const QString& filePath, bool writeEventVsTodo)
 {
-    if (!MyMoneyFile::instance()->storageAttached())
-        return;
-
     QFile icsFile(filePath);
 
+    const icalcomponent_kind newEntryKind = writeEventVsTodo ? ICAL_VEVENT_COMPONENT : ICAL_VTODO_COMPONENT;
     icsFile.open(QIODevice::ReadOnly);
     QTextStream stream(&icsFile);
     d->m_icalendarAsString = stream.readAll();
@@ -241,24 +243,35 @@ void KMMSchedulesToiCalendar::exportToFile(const QString& filePath, bool setting
 
     // create the calendar
     bool newCalendar = false;
-    icalcomponent* vCalendar = 0;
+    icalcomponent* vCalendar = nullptr;
+    struct icaltimetype atime = icaltime_from_timet_with_zone(time(0), 0, icaltimezone_get_utc_timezone());
+
     if (d->m_icalendarAsString.isEmpty()) {
         newCalendar = true;
         vCalendar = icalcomponent_new_vcalendar();
     } else {
         vCalendar = icalcomponent_new_from_string(d->m_icalendarAsString.toUtf8());
-        if (vCalendar == 0) {
-            qDebug() << "Error parsing the following string into an icalendar:" << endl;
-            qDebug() << d->m_icalendarAsString << endl;
-            qDebug() << "so we will overwrite this with a new calendar" << endl;
+        if (vCalendar == nullptr) {
+            qDebug() << "Error parsing the following string into an icalendar:" << Qt::endl;
+            qDebug() << d->m_icalendarAsString << Qt::endl;
+            qDebug() << "so we will overwrite this with a new calendar" << Qt::endl;
             newCalendar = true;
             vCalendar = icalcomponent_new_vcalendar();
+        } else {
+            // we have a calendar and must check if the entry kind was changed
+            icalcomponent* itTodo = icalcomponent_get_first_component(vCalendar, ICAL_VTODO_COMPONENT);
+            icalcomponent* itEvent = icalcomponent_get_first_component(vCalendar, ICAL_VEVENT_COMPONENT);
+            if (((itTodo != nullptr) && writeEventVsTodo) || (((itEvent != nullptr) && !writeEventVsTodo))) {
+                icalcomponent_free(vCalendar);
+                newCalendar = true;
+                vCalendar = icalcomponent_new_vcalendar();
+            }
         }
     }
 
-    if (vCalendar == 0) {
+    if (vCalendar == nullptr) {
         // one way or the other we must have a calendar by now
-        qDebug() << "Unable to create vcalendar component" << endl;
+        qDebug() << "Unable to create vcalendar component" << Qt::endl;
         return;
     }
 
@@ -271,31 +284,31 @@ void KMMSchedulesToiCalendar::exportToFile(const QString& filePath, bool setting
     // export schedules as TODOs
     auto file = MyMoneyFile::instance();
     QList<MyMoneySchedule> schedules = file->scheduleList();
-    for (QList<MyMoneySchedule>::const_iterator itSchedule = schedules.constBegin(); itSchedule != schedules.constEnd(); ++itSchedule) {
+    for (QList<MyMoneySchedule>::const_iterator itSchedule = schedules.cbegin(); itSchedule != schedules.cend(); ++itSchedule) {
         const MyMoneySchedule& myMoneySchedule = *itSchedule;
 
         if (myMoneySchedule.isFinished())
             continue;  // skip this schedule if it is already finished
 
-        icalcomponent* schedule = 0;
-        bool newTodo = false;
+        icalcomponent* schedule = nullptr;
+        bool newEntry = false;
         if (!newCalendar) {
             // try to find the schedule to update it if we do not use a new calendar
-            icalcomponent* itVTODO = icalcomponent_get_first_component(vCalendar, ICAL_VTODO_COMPONENT);
-            for (; itVTODO != 0; itVTODO = icalcomponent_get_next_component(vCalendar, ICAL_VTODO_COMPONENT)) {
-                if (icalcomponent_get_uid(itVTODO) == myMoneySchedule.id()) {
+            icalcomponent* itEntry = icalcomponent_get_first_component(vCalendar, newEntryKind);
+            for (; itEntry != nullptr; itEntry = icalcomponent_get_next_component(vCalendar, newEntryKind)) {
+                if (icalcomponent_get_uid(itEntry) == myMoneySchedule.id()) {
                     // we found our todo stop searching
-                    schedule = itVTODO;
+                    schedule = itEntry;
                     break;
                 }
             }
-            if (schedule == 0) {
-                schedule = icalcomponent_new_vtodo();
-                newTodo = true;
+            if (schedule == nullptr) {
+                schedule = writeEventVsTodo ? icalcomponent_new_vevent() : icalcomponent_new_vtodo();
+                newEntry = true;
             }
         } else {
-            schedule = icalcomponent_new_vtodo();
-            newTodo = true;
+            schedule = writeEventVsTodo ? icalcomponent_new_vevent() : icalcomponent_new_vtodo();
+            newEntry = true;
         }
 
         // description
@@ -306,15 +319,25 @@ void KMMSchedulesToiCalendar::exportToFile(const QString& filePath, bool setting
         icalcomponent_set_uid(schedule, myMoneySchedule.id().toUtf8());
         // dtstart
         icalcomponent_set_dtstart(schedule, qdateToIcalTimeType(myMoneySchedule.startDate()));
-        // due
-        icalcomponent_set_due(schedule, qdateToIcalTimeType(myMoneySchedule.nextDueDate()));
-        if (newTodo) {
+
+        // due (only supported for VTODO)
+        if (newEntryKind == ICAL_VTODO_COMPONENT) {
+            icalcomponent_set_due(schedule, qdateToIcalTimeType(myMoneySchedule.nextDueDate()));
+        }
+        // dtstamp
+        icalproperty* dtstamp = icalcomponent_get_first_property(schedule, ICAL_DTSTAMP_PROPERTY);
+        if (dtstamp != nullptr) {
+            icalcomponent_remove_property(schedule, dtstamp);
+        }
+        icalcomponent_add_property(schedule, icalproperty_new_dtstamp(atime));
+
+        if (newEntry) {
             // created
             icalcomponent_add_property(schedule, icalproperty_new_created(qdateTimeToIcalTimeType(QDateTime::currentDateTime())));
         } else {
             // last modified
             icalproperty* pLastMod = icalcomponent_get_first_property(schedule, ICAL_LASTMODIFIED_PROPERTY);
-            if (pLastMod != 0) {
+            if (pLastMod != nullptr) {
                 // set the current property
                 icalproperty_set_lastmodified(pLastMod, qdateTimeToIcalTimeType(QDateTime::currentDateTime()));
             } else {
@@ -324,48 +347,61 @@ void KMMSchedulesToiCalendar::exportToFile(const QString& filePath, bool setting
         }
         // recurrence
         icalproperty* pRRule = icalcomponent_get_first_property(schedule, ICAL_RRULE_PROPERTY);
-        if (pRRule != 0) {
+        if (pRRule != nullptr) {
             icalcomponent_remove_property(schedule, pRRule);
         }
         if (myMoneySchedule.occurrence() != Schedule::Occurrence::Once && myMoneySchedule.baseOccurrence() != Schedule::Occurrence::Any)
             icalcomponent_add_property(schedule, icalproperty_new_rrule(scheduleToRecurenceRule(myMoneySchedule)));
 
-        icalcomponent* oldAlarm = icalcomponent_get_first_component(schedule, ICAL_VALARM_COMPONENT);
-        if (oldAlarm && settingsChaged)
-            icalcomponent_remove_component(schedule, oldAlarm);
+        icalcomponent* alarm = icalcomponent_get_first_component(schedule, ICAL_VALARM_COMPONENT);
+        if (alarm)
+            icalcomponent_remove_component(schedule, alarm);
 
-        if (PluginSettings::createAlarm() && (!oldAlarm || settingsChaged)) {
+        if (ICalendarSettings::createAlarm()) {
             // alarm: beginning with one day before the todo is due every one hour
-            icalcomponent* alarm = icalcomponent_new_valarm();
+            alarm = icalcomponent_new_valarm();
             // alarm: action
             icalcomponent_add_property(alarm, icalproperty_new_action(ICAL_ACTION_DISPLAY));
             // alarm: description
             icalcomponent_set_description(alarm, scheduleToDescription(myMoneySchedule).toUtf8());
             // alarm: trigger
-            int triggerInterval = beforeAfterToInt(PluginSettings::beforeAfter()) * PluginSettings::timeUnits() * timeUnitsInSeconds(PluginSettings::timeUnitInSeconds());
+            int triggerInterval = beforeAfterToInt(ICalendarSettings::beforeAfter()) * ICalendarSettings::timeUnits() * timeUnitsInSeconds(ICalendarSettings::timeUnitInSeconds());
             icalcomponent_add_property(alarm, icalproperty_new_trigger(icaltriggertype_from_int(triggerInterval)));
             // alarm: duration
-            int intervalBetweenReminders = PluginSettings::intervalBetweenRemindersTimeUnits() * timeUnitsInSeconds(PluginSettings::intervalBetweenRemindersTimeUnitInSeconds());
+            int intervalBetweenReminders = ICalendarSettings::intervalBetweenRemindersTimeUnits() * timeUnitsInSeconds(ICalendarSettings::intervalBetweenRemindersTimeUnitInSeconds());
             icalcomponent_set_duration(alarm, icaldurationtype_from_int(intervalBetweenReminders));
-            if (PluginSettings::repeatingReminders()) {
+            if (ICalendarSettings::repeatingReminders()) {
                 // alarm: repeat
-                icalcomponent_add_property(alarm, icalproperty_new_repeat(PluginSettings::numberOfReminders()));
+                icalcomponent_add_property(alarm, icalproperty_new_repeat(ICalendarSettings::numberOfReminders()));
             }
             // add the alarm to the schedule
             icalcomponent_add_component(schedule, alarm);
         }
 
         // add the schedule to the calendar
-        if (newTodo)
+        if (newEntry)
             icalcomponent_add_component(vCalendar, schedule);
     }
 
-    icsFile.open(QIODevice::WriteOnly);
+    // now remove the ones that have been deleted by the user
 
+    for (icalcomponent* itEntry = icalcomponent_get_first_component(vCalendar, newEntryKind); itEntry != nullptr;
+         itEntry = icalcomponent_get_current_component(vCalendar)) {
+        const QString ical_uid = icalcomponent_get_uid(itEntry);
+        if (!file->schedulesModel()->indexById(ical_uid).isValid()) {
+            icalcomponent_remove_component(vCalendar, itEntry);
+        } else {
+            icalcomponent_get_next_component(vCalendar, newEntryKind);
+        }
+    }
+
+    // write out the ics file
+
+    icsFile.open(QIODevice::WriteOnly);
     d->m_icalendarAsString = QString::fromUtf8(icalcomponent_as_ical_string(vCalendar));
     // reclaim some memory :)
     icalcomponent_free(vCalendar);
     // write the calendar to the file
-    stream << d->m_icalendarAsString << endl;
+    stream << d->m_icalendarAsString << Qt::endl;
     icsFile.close();
 }
