@@ -1,7 +1,6 @@
 /*
     SPDX-FileCopyrightText: 2004 Martin Preuss <martin@libchipcard.de>
 
-
     SPDX-License-Identifier: GPL-2.0-or-later
 
 */
@@ -19,11 +18,7 @@ class KBMapAccount: public QDialog
 {
     Q_OBJECT
 public:
-    KBMapAccount(KBankingExt *kb,
-                 const char *bankCode,
-                 const char *accountId,
-                 QWidget* parent = 0,
-                 Qt::WindowFlags fl = 0);
+    KBMapAccount(KBankingExt* kb, const char* bankCode, const char* accountId, QWidget* parent = 0, Qt::WindowFlags fl = {});
     ~KBMapAccount();
 
     AB_ACCOUNT_SPEC *getAccount();
