@@ -20,11 +20,11 @@
 #include <QPointer>
 #include <QLineEdit>
 
+#include "passstore.h"
 #include "passwordtoggle.h"
 #include "widgets/chiptandialog.h"
 #include "widgets/phototandialog.h"
 
-
 #define KBANKING_TANMETHOD_TEXT          0x00000001
 #define KBANKING_TANMETHOD_CHIPTAN       0x00000002
 #define KBANKING_TANMETHOD_CHIPTAN_OPTIC 0x00000003
@@ -52,31 +52,46 @@ int gwenKdeGui::execDialog(GWEN_DIALOG *dlg, GWEN_UNUSED uint32_t guiid)
         return GWEN_ERROR_GENERIC;
     }
 
+    PassStore* passStore(nullptr);
     QDialog* dialog = dynamic_cast<QDialog*>(qt5Dlg.getMainWindow());
     const auto lineedits = dialog->findChildren<QLineEdit*>();
     for (const auto& edit : qAsConst(lineedits)) {
         if (edit->echoMode() == QLineEdit::Password) {
+            // check for available pass entry
+            passStore = new PassStore(edit, QLatin1String("KMyMoney/KBanking"), passwordId);
+            if (!passStore->isActionVisible()) {
+                qDebug() << "No password found for KMyMoney/KBanking/" << passStore->passwordId();
+            }
             new PasswordToggle(edit);
+            break;
         }
     }
-    return qt5Dlg.execute();
+    auto rc = qt5Dlg.execute();
+
+    return rc;
 }
 
 int gwenKdeGui::getPasswordText(uint32_t flags,
-                                const char *token,
-                                const char *title,
-                                const char *text,
-                                char *buffer,
+                                const char* token,
+                                const char* title,
+                                const char* text,
+                                char* buffer,
                                 int minLen,
                                 int maxLen,
                                 GWEN_GUI_PASSWORD_METHOD methodId,
-                                GWEN_DB_NODE *methodParams,
-                                uint32_t guiid) {
-    return QT5_Gui::getPassword(flags, token, title, text, buffer, minLen, maxLen, methodId, methodParams, guiid);
+                                GWEN_DB_NODE* methodParams,
+                                uint32_t guiid)
+{
+    // Keep the current password identifier
+    passwordId = QString(token);
+
+    // convert HTML linebreaks into regular ones
+    QString txt = QString::fromUtf8(text);
+    txt.replace(QLatin1String("<br>"), QLatin1String("\n"));
+
+    return QT5_Gui::getPassword(flags, token, title, txt.toUtf8(), buffer, minLen, maxLen, methodId, methodParams, guiid);
 }
 
-
-
 int gwenKdeGui::getPasswordHhd(uint32_t /*flags*/,
                                const char * /*token*/,
                                const char * /*title*/,
