@@ -3,6 +3,7 @@
     SPDX-FileCopyrightText: 2009 Cristian Onet onet.cristian @gmail.com
     SPDX-FileCopyrightText: 2010-2019 Thomas Baumgart tbaumgart @kde.org
     SPDX-FileCopyrightText: 2015 Christian David christian-david @web.de
+    SPDX-FileCopyrightText: 2021 Dawid Wróbel <me@dawidwrobel.com>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
@@ -20,15 +21,10 @@
 #include <QLabel>
 #include <QLayout>
 #include <QRadioButton>
-#include <QRegExp>
 #include <QRegularExpression>
 #include <QStringList>
 #include <QTimer>
-
-#ifdef IS_APPIMAGE
-#include <QCoreApplication>
-#include <QStandardPaths>
-#endif
+#include <QUuid>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
@@ -62,23 +58,24 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include "mymoney/onlinejob.h"
+#include <alkimia/alkenvironment.h>
 
+#include "aqbankingkmmoperators.h"
+#include "gwenhywfarqtoperators.h"
+#include "gwenkdegui.h"
 #include "kbaccountsettings.h"
 #include "kbmapaccount.h"
-#include "mymoneyfile.h"
-#include "onlinejobadministration.h"
-#include "kmymoneyview.h"
 #include "kbpickstartdate.h"
-#include "mymoneyinstitution.h"
-#include "mymoneytransactionfilter.h"
+#include "kmymoneyview.h"
+#include "mymoney/onlinejob.h"
 #include "mymoneyexception.h"
+#include "mymoneyfile.h"
+#include "mymoneyinstitution.h"
 #include "mymoneysecurity.h"
-
-#include "gwenkdegui.h"
-#include "gwenhywfarqtoperators.h"
-#include "aqbankingkmmoperators.h"
 #include "mymoneystatement.h"
+#include "mymoneytransactionfilter.h"
+#include "mymoneyutils.h"
+#include "onlinejobadministration.h"
 #include "statementinterface.h"
 #include "viewinterface.h"
 
@@ -89,18 +86,22 @@
 #include "phototan-demo.cpp"
 #endif
 
+#include "kmmyesno.h"
+
 class KBanking::Private
 {
 public:
-    Private() :
-        passwordCacheTimer(nullptr),
-        jobList(),
-        fileId()
+    Private()
+        : passwordCacheTimer(nullptr)
+        , gui(nullptr)
+        , jobList()
+        , fileId()
     {
         QString gwenProxy = QString::fromLocal8Bit(qgetenv("GWEN_PROXY"));
         if (gwenProxy.isEmpty()) {
             std::unique_ptr<KConfig> cfg = std::unique_ptr<KConfig>(new KConfig("kioslaverc"));
-            QRegExp exp("(\\w+://)?([^/]{2}.+:\\d+)");
+            const QRegularExpression exp(QLatin1String("(\\w+://)?([^/]{2}.+:\\d+)"));
+            QRegularExpressionMatch proxyMatch;
             QString proxy;
 
             KConfigGroup grp = cfg->group("Proxy Settings");
@@ -111,9 +112,10 @@ public:
 
             case 1: // manual specified
                 proxy = grp.readEntry("httpsProxy");
+                proxyMatch = exp.match(proxy);
                 qDebug("KDE https proxy setting is '%s'", qPrintable(proxy));
-                if (exp.exactMatch(proxy)) {
-                    proxy = exp.cap(2);
+                if (proxyMatch.hasMatch()) {
+                    proxy = proxyMatch.captured(2);
                     qDebug("Setting GWEN_PROXY to '%s'", qPrintable(proxy));
                     if (!qputenv("GWEN_PROXY", qPrintable(proxy))) {
                         qDebug("Unable to setup GWEN_PROXY");
@@ -151,14 +153,15 @@ public:
     }
 
     QTimer *passwordCacheTimer;
-    QMap<QString, QStringList>  jobList;
-    QString                     fileId;
-    QSet<QAction *>             actions;
+    gwenKdeGui* gui;
+    QMap<QString, QStringList> jobList;
+    QString fileId;
+    KMMSet<QAction*> actions;
 };
 
 
-KBanking::KBanking(QObject *parent, const QVariantList &args) :
-    OnlinePluginExtended(parent, "kbanking")
+KBanking::KBanking(QObject *parent, const KPluginMetaData &metaData, const QVariantList &args) :
+    OnlinePluginExtended(parent, metaData, args)
     , d(new Private)
     , m_configAction(nullptr)
     , m_importAction(nullptr)
@@ -167,6 +170,10 @@ KBanking::KBanking(QObject *parent, const QVariantList &args) :
     , m_statementCount(0)
 {
     Q_UNUSED(args)
+
+    Q_INIT_RESOURCE(kbanking);
+    Q_INIT_RESOURCE(chipTan);
+
     QString compileVersionSet = QLatin1String(GWENHYWFAR_VERSION_FULL_STRING "/" AQBANKING_VERSION_FULL_STRING);
     QString runtimeVersionSet = QString("%1/%2").arg(d->libVersion(&GWEN_Version), d->libVersion(&AB_Banking_GetVersion));
     qDebug() << QString("Plugins: kbanking loaded, build with (%1), run with (%2)").arg(compileVersionSet, runtimeVersionSet);
@@ -178,10 +185,31 @@ KBanking::~KBanking()
     qDebug("Plugins: kbanking unloaded");
 }
 
-void KBanking::plug()
+void KBanking::plug(KXMLGUIFactory* guiFactory)
 {
-    const auto componentName = QLatin1String("kbanking");
-    const auto rcFileName = QLatin1String("kbanking.rc");
+    Q_UNUSED(guiFactory)
+
+    if (qEnvironmentVariableIsEmpty("GWEN_LOGLEVEL")) {
+        if (AlkEnvironment::isRunningAsAppImage()) {
+            qDebug() << "Set loglevel for" << GWEN_LOGDOMAIN << "to verbose";
+            GWEN_Logger_SetLevel(GWEN_LOGDOMAIN, GWEN_LoggerLevel_Verbous);
+        } else {
+            GWEN_Logger_SetLevel(GWEN_LOGDOMAIN, GWEN_LoggerLevel_Warning);
+        }
+    }
+    if (qEnvironmentVariableIsEmpty("AQBANKING_LOGLEVEL")) {
+        if (AlkEnvironment::isRunningAsAppImage()) {
+            qDebug() << "Set loglevel for" << AQBANKING_LOGDOMAIN << "to verbose";
+            GWEN_Logger_SetLevel(AQBANKING_LOGDOMAIN, GWEN_LoggerLevel_Verbous);
+            qDebug() << "Set loglevel for"
+                     << "aqhbci"
+                     << "to verbose";
+            GWEN_Logger_SetLevel("aqhbci", GWEN_LoggerLevel_Info);
+        } else {
+            GWEN_Logger_SetLevel(AQBANKING_LOGDOMAIN, GWEN_LoggerLevel_Warning);
+        }
+    }
+
     m_kbanking = new KBankingExt(this, "KMyMoney");
 
     d->passwordCacheTimer = new QTimer(this);
@@ -190,44 +218,36 @@ void KBanking::plug()
     connect(d->passwordCacheTimer, &QTimer::timeout, this, &KBanking::slotClearPasswordCache);
 
     if (m_kbanking) {
-        //! @todo when is gwenKdeGui deleted?
-        gwenKdeGui *gui = new gwenKdeGui();
-        GWEN_Gui_SetGui(gui->getCInterface());
-        GWEN_Gui_SetLogHookFn(gui->getCInterface(), &KBanking::Private::gwenLogHook);
-        if (qEnvironmentVariableIsEmpty("GWEN_LOGLEVEL")) {
-            GWEN_Logger_SetLevel(GWEN_LOGDOMAIN, GWEN_LoggerLevel_Warning);
-        }
+        d->gui = new gwenKdeGui;
+        GWEN_Gui_SetGui(d->gui->getCInterface());
+
+        // Setup logging features
+        GWEN_Gui_SetLogHookFn(d->gui->getCInterface(), &KBanking::Private::gwenLogHook);
 
         if (m_kbanking->init() == 0) {
             // Tell the host application to load my GUI component
-            setComponentName(componentName, "KBanking");
-
-#ifdef IS_APPIMAGE
-            const QString rcFilePath = QString("%1/../share/kxmlgui5/%2/%3").arg(QCoreApplication::applicationDirPath(), componentName, rcFileName);
-            setXMLFile(rcFilePath);
-
-            const QString localRcFilePath = QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation).first() + QLatin1Char('/') + componentName + QLatin1Char('/') + rcFileName;
-            setLocalXMLFile(localRcFilePath);
-#else
+            const auto rcFileName = QLatin1String("kbanking.rc");
             setXMLFile(rcFileName);
-#endif
 
             // get certificate handling and dialog settings management
-            AB_Gui_Extend(gui->getCInterface(), m_kbanking->getCInterface());
+            AB_Gui_Extend(d->gui->getCInterface(), m_kbanking->getCInterface());
 
             // create actions
             createActions();
 
             // load protocol conversion list
             loadProtocolConversion();
-            if (qEnvironmentVariableIsEmpty("AQBANKING_LOGLEVEL"))
-                GWEN_Logger_SetLevel(AQBANKING_LOGDOMAIN, GWEN_LoggerLevel_Warning);
 
         } else {
             qWarning("Could not initialize KBanking online banking interface");
             delete m_kbanking;
             m_kbanking = 0;
         }
+
+        if (AlkEnvironment::isRunningAsAppImage()) {
+            GWEN_Logger_SetLevel(GWEN_LOGDOMAIN, GWEN_LoggerLevel_Warning);
+            GWEN_Logger_SetLevel(AQBANKING_LOGDOMAIN, GWEN_LoggerLevel_Warning);
+        }
     }
 }
 
@@ -238,8 +258,11 @@ void KBanking::unplug()
         m_kbanking->fini();
         delete m_kbanking;
     }
+    delete d->gui;
+    d->gui = nullptr;
+
     // remove and delete the actions for this plugin
-    for (const auto& action : d->actions) {
+    for (const auto& action : qAsConst(d->actions)) {
         actionCollection()->removeAction(action);
     }
     qDebug("Plugins: kbanking unplugged");
@@ -324,7 +347,7 @@ void KBanking::createActions()
 #ifdef KMM_DEBUG
     QAction *openChipTanDialog = actionCollection()->addAction("open_chiptan_dialog");
     openChipTanDialog->setText("Open ChipTan Dialog");
-    connect(openChipTanDialog, &QAction::triggered, [&]() {
+    connect(openChipTanDialog, &QAction::triggered, this, [&]() {
         auto dlg = new chipTanDialog();
         dlg->setHhdCode("0F04871100030333555414312C32331D");
         dlg->setInfoText("<html><h1>Test Graphic for debugging</h1><p>The encoded data is</p><p>Account Number: <b>335554</b><br/>Amount: <b>1,23</b></p></html>");
@@ -336,15 +359,15 @@ void KBanking::createActions()
 
     QAction *openPhotoTanDialog = actionCollection()->addAction("open_phototan_dialog");
     openPhotoTanDialog->setText("Open PhotoTan Dialog");
-    connect(openPhotoTanDialog, &QAction::triggered, [&]() {
+    connect(openPhotoTanDialog, &QAction::triggered, this, [&]() {
         auto dlg = new photoTanDialog();
         QImage img;
         img.loadFromData(photoTan, sizeof(photoTan), "PNG");
         img = img.scaled(300, 300, Qt::KeepAspectRatio);
         dlg->setPicture(QPixmap::fromImage(img));
         dlg->setInfoText("<html><h1>Test Graphic for debugging</h1><p>The encoded data is</p><p>unknown</p></html>");
-        connect(dlg, &QDialog::accepted, dlg, &chipTanDialog::deleteLater);
-        connect(dlg, &QDialog::rejected, dlg, &chipTanDialog::deleteLater);
+        connect(dlg, &QDialog::accepted, dlg, &photoTanDialog::deleteLater);
+        connect(dlg, &QDialog::rejected, dlg, &photoTanDialog::deleteLater);
         dlg->show();
     });
     d->actions.insert(openPhotoTanDialog);
@@ -428,9 +451,10 @@ AB_ACCOUNT_SPEC* KBanking::aqbAccount(const QString& accountId) const
 QString KBanking::stripLeadingZeroes(const QString& s) const
 {
     QString rc(s);
-    QRegExp exp("^(0*)([^0].*)");
-    if (exp.exactMatch(s)) {
-        rc = exp.cap(2);
+    const QRegularExpression stripZeroExp(QLatin1String("^(0*)([^0].*)"));
+    const auto number(stripZeroExp.match(s));
+    if (number.hasMatch()) {
+        rc = number.captured(2);
     }
     return rc;
 }
@@ -441,9 +465,9 @@ void KBanking::setupAccountReference(const MyMoneyAccount& acc, AB_ACCOUNT_SPEC*
 
     if (ab_acc) {
         QString accountNumber = stripLeadingZeroes(AB_AccountSpec_GetAccountNumber(ab_acc));
-        QString routingNumber = stripLeadingZeroes(AB_AccountSpec_GetBankCode(ab_acc));
+        QString bankCode = stripLeadingZeroes(AB_AccountSpec_GetBankCode(ab_acc));
 
-        QString val = QString("%1-%2-%3").arg(routingNumber, accountNumber).arg(AB_AccountSpec_GetType(ab_acc));
+        QString val = QString("%1-%2-%3").arg(bankCode, accountNumber).arg(AB_AccountSpec_GetType(ab_acc));
 
         if (val != acc.onlineBankingSettings().value("kbanking-acc-ref")) {
             kvp.clear();
@@ -605,7 +629,7 @@ bool KBanking::updateAccount(const MyMoneyAccount& acc, bool moreAccounts)
                                            i18n("Error"));
                     } else {
                         rc = true;
-                        emit queueChanged();
+                        Q_EMIT queueChanged();
                     }
                 }
             }
@@ -647,7 +671,7 @@ void KBanking::sendOnlineJob(QList<onlineJob>& jobs)
     QList<onlineJob> unhandledJobs;
 
     if (!jobs.isEmpty()) {
-        foreach (onlineJob job, jobs) {
+        for (onlineJob job : jobs) {
             if (sepaOnlineTransfer::name() == job.task()->taskName()) {
                 onlineJobTyped<sepaOnlineTransfer> typedJob(job);
                 enqueTransaction(typedJob);
@@ -661,12 +685,13 @@ void KBanking::sendOnlineJob(QList<onlineJob>& jobs)
 
         executeQueue();
     }
-    jobs = m_onlineJobQueue.values() + unhandledJobs;
+    jobs = m_onlineJobQueue.values();
+    jobs += unhandledJobs;
     m_onlineJobQueue.clear();
 }
 
 
-QStringList KBanking::availableJobs(QString accountId)
+QStringList KBanking::availableJobs(QString accountId) const
 {
     try {
         MyMoneyAccount acc = MyMoneyFile::instance()->account(accountId);
@@ -956,7 +981,7 @@ int KBankingExt::executeQueue(AB_IMEXPORTER_CONTEXT *ctx)
     _jobQueue = AB_Transaction_List2_new();
     AB_Transaction_List2_freeAll(oldQ);
 
-    emit m_parent->queueChanged();
+    Q_EMIT m_parent->queueChanged();
     m_parent->startPasswordTimer();
 
     return rv;
@@ -1009,7 +1034,7 @@ int KBankingExt::dequeueJob(AB_TRANSACTION *j)
     assert(_jobQueue);
     AB_Transaction_List2_Remove(_jobQueue, j);
     AB_Transaction_free(j);
-    emit m_parent->queueChanged();
+    Q_EMIT m_parent->queueChanged();
     return 0;
 }
 
@@ -1026,14 +1051,14 @@ bool KBankingExt::askMapAccount(const MyMoneyAccount& acc)
 
     QString bankId;
     QString accountId;
-    // extract some information about the bank. if we have a sortcode
+    // extract some information about the bank. if we have a bank code
     // (BLZ) we display it, otherwise the name is enough.
     try {
         const MyMoneyInstitution &bank = file->institution(acc.institutionId());
         bankId = bank.name();
-        if (!bank.sortcode().isEmpty())
-            bankId = bank.sortcode();
-    } catch (const MyMoneyException &e) {
+        if (!bank.bankcode().isEmpty())
+            bankId = bank.bankcode();
+    } catch (const MyMoneyException&) {
         // no bank assigned, we just leave the field empty
     }
 
@@ -1082,12 +1107,7 @@ bool KBankingExt::askMapAccount(const MyMoneyAccount& acc)
 
 QString KBankingExt::mappingId(const MyMoneyObject& object) const
 {
-    QString id = MyMoneyFile::instance()->storageId() + QLatin1Char('-') + object.id();
-
-    // AqBanking does not handle the enclosing parens, so we remove it
-    id.remove('{');
-    id.remove('}');
-    return id;
+    return MyMoneyFile::instance()->storageId().toString(QUuid::WithoutBraces) + QLatin1Char('-') + object.id();
 }
 
 
@@ -1134,12 +1154,18 @@ void KBankingExt::_xaToStatement(MyMoneyStatement &ks,
     QString s;
     QString memo;
     const char *p;
-    const AB_VALUE *val;
-    const GWEN_DATE *dt;
-    const GWEN_DATE *startDate = 0;
+    const AB_VALUE* val = nullptr;
+    const GWEN_DATE* dt = nullptr;
     MyMoneyStatement::Transaction kt;
     unsigned long h;
 
+    auto appendToMemo = [&](const QString& line) {
+        s += QStringLiteral(", %1").arg(line);
+        if (!memo.isEmpty())
+            memo.append('\n');
+        memo.append(line);
+    };
+
     kt.m_fees = MyMoneyMoney();
 
     // bank's transaction id
@@ -1158,54 +1184,52 @@ void KBankingExt::_xaToStatement(MyMoneyStatement &ks,
     if (p && *p) {
         QString tmpMemo;
 
-        s=QString::fromUtf8(p).trimmed();
-        tmpMemo=QString::fromUtf8(p).trimmed();
+        s = QString::fromUtf8(p).trimmed();
+        tmpMemo = QString::fromUtf8(p).trimmed();
         tmpMemo.replace('\n', ' ');
 
-        memo=tmpMemo;
+        memo = tmpMemo;
+    }
+
+    // This is to extract Paypal specific user supplied notes
+    // and append them to the memo field
+    p = AB_Transaction_GetMemo(t);
+    if (p) {
+        QString tmpMemo;
+
+        s += QString::fromUtf8(p).trimmed();
+        tmpMemo = QString::fromUtf8(p).trimmed();
+        tmpMemo.replace('\n', ' ');
+        memo.append(tmpMemo);
     }
 
     // in case we have some SEPA fields filled with information
     // we add them to the memo field
     p = AB_Transaction_GetEndToEndReference(t);
     if (p) {
-        s += QString(", EREF: %1").arg(p);
-        if(memo.length())
-            memo.append('\n');
-        memo.append(QString("EREF: %1").arg(p));
+        appendToMemo(QStringLiteral("EREF: %1").arg(QString::fromUtf8(p)));
     }
     p = AB_Transaction_GetCustomerReference(t);
     if (p) {
-        s += QString(", CREF: %1").arg(p);
-        if(memo.length())
-            memo.append('\n');
-        memo.append(QString("CREF: %1").arg(p));
+        appendToMemo(QStringLiteral("CREF: %1").arg(QString::fromUtf8(p)));
     }
     p = AB_Transaction_GetMandateId(t);
     if (p) {
-        s += QString(", MREF: %1").arg(p);
-        if(memo.length())
-            memo.append('\n');
-        memo.append(QString("MREF: %1").arg(p));
+        appendToMemo(QStringLiteral("MREF: %1").arg(QString::fromUtf8(p)));
     }
     p = AB_Transaction_GetCreditorSchemeId(t);
     if (p) {
-        s += QString(", CRED: %1").arg(p);
-        if(memo.length())
-            memo.append('\n');
-        memo.append(QString("CRED: %1").arg(p));
+        appendToMemo(QStringLiteral("CRED: %1").arg(QString::fromUtf8(p)));
     }
     p = AB_Transaction_GetOriginatorId(t);
     if (p) {
-        s += QString(", DEBT: %1").arg(p);
-        if(memo.length())
-            memo.append('\n');
-        memo.append(QString("DEBT: %1").arg(p));
+        appendToMemo(QStringLiteral("DEBT: %1").arg(QString::fromUtf8(p)));
     }
 
     const MyMoneyKeyValueContainer& kvp = acc.onlineBankingSettings();
     // check if we need the version with or without linebreaks
-    if (kvp.value("kbanking-memo-removelinebreaks").compare(QLatin1String("no"))) {
+    const auto removeLineBreaks = kvp.value("kbanking-memo-removelinebreaks").compare(QLatin1String("no")) != 0;
+    if (removeLineBreaks) {
         kt.m_strMemo = memo;
     } else {
         kt.m_strMemo = s;
@@ -1220,40 +1244,92 @@ void KBankingExt::_xaToStatement(MyMoneyStatement &ks,
     QString rePayee = kvp.value("kbanking-payee-regexp");
     if (!rePayee.isEmpty() && kt.m_strPayee.isEmpty()) {
         QString reMemo = kvp.value("kbanking-memo-regexp");
-        QStringList exceptions = kvp.value("kbanking-payee-exceptions").split(';', QString::SkipEmptyParts);
+        QStringList exceptions = kvp.value("kbanking-payee-exceptions").split(';', Qt::SkipEmptyParts);
 
         bool needExtract = true;
         QStringList::const_iterator it_s;
-        for (it_s = exceptions.constBegin(); needExtract && it_s != exceptions.constEnd(); ++it_s) {
-            QRegExp exp(*it_s, Qt::CaseInsensitive);
-            if (exp.indexIn(kt.m_strMemo) != -1) {
+        for (it_s = exceptions.cbegin(); needExtract && it_s != exceptions.cend(); ++it_s) {
+            const QRegularExpression exceptionExp(*it_s, QRegularExpression::CaseInsensitiveOption);
+            const auto memoRegExMatch(exceptionExp.match(kt.m_strMemo));
+            if (memoRegExMatch.hasMatch()) {
                 needExtract = false;
             }
         }
         if (needExtract) {
-            QRegExp expPayee(rePayee, Qt::CaseInsensitive);
-            QRegExp expMemo(reMemo, Qt::CaseInsensitive);
-            if (expPayee.indexIn(kt.m_strMemo) != -1) {
-                kt.m_strPayee = expPayee.cap(1);
-                if (expMemo.indexIn(kt.m_strMemo) != -1) {
-                    kt.m_strMemo = expMemo.cap(1);
+            const QRegularExpression payeeExp(rePayee, QRegularExpression::CaseInsensitiveOption);
+            const QRegularExpression memoExp(reMemo, QRegularExpression::CaseInsensitiveOption);
+            const auto payee(payeeExp.match(kt.m_strMemo));
+            const auto memoRegExMatch(memoExp.match(kt.m_strMemo));
+            if (payee.hasMatch()) {
+                kt.m_strPayee = payee.captured(1);
+                if (memoRegExMatch.hasMatch()) {
+                    kt.m_strMemo = memoRegExMatch.captured(1);
                 }
             }
         }
     }
 
+#if QT_VERSION_CHECK(AQBANKING_VERSION_MAJOR, AQBANKING_VERSION_MINOR, AQBANKING_VERSION_PATCHLEVEL) > QT_VERSION_CHECK(6, 2, 0)
+    // Append additional payee/payer information which
+    // can be found in additional fields (but only in
+    // versions > 6.2.0)
+    const auto includePayeeDetails = kvp.value("kbanking-memo-includepayeedetails").compare(QLatin1String("no")) != 0;
+    p = AB_Transaction_GetUltimateDebtor(t);
+    if (p) {
+        const auto delim = kt.m_strPayee.isEmpty() ? QChar() : QLatin1Char('/');
+        kt.m_strPayee.append(QStringLiteral("%1%2").arg(delim).arg(QString::fromUtf8(p)));
+        if (includePayeeDetails) {
+            kt.m_strMemo.append(QStringLiteral("\nABWA: %1").arg(QString::fromUtf8(p)));
+        }
+    }
+    p = AB_Transaction_GetUltimateCreditor(t);
+    if (p) {
+        const auto delim = kt.m_strPayee.isEmpty() ? QChar() : QLatin1Char('/');
+        kt.m_strPayee.append(QStringLiteral("%1%2").arg(delim).arg(QString::fromUtf8(p)));
+        if (includePayeeDetails) {
+            kt.m_strMemo.append(QStringLiteral("\nABWE: %1").arg(QString::fromUtf8(p)));
+        }
+    }
+#endif
+
     kt.m_strPayee = kt.m_strPayee.trimmed();
 
     // date
-    dt = AB_Transaction_GetDate(t);
-    if (!dt)
-        dt = AB_Transaction_GetValutaDate(t);
+    dt = AB_Transaction_GetValutaDate(t);
     if (dt) {
-        if (!startDate)
-            startDate = dt;
         kt.m_datePosted = QDate(GWEN_Date_GetYear(dt), GWEN_Date_GetMonth(dt), GWEN_Date_GetDay(dt));
+    }
+    dt = AB_Transaction_GetDate(t);
+    if (dt) {
+        kt.m_dateProcessed = QDate(GWEN_Date_GetYear(dt), GWEN_Date_GetMonth(dt), GWEN_Date_GetDay(dt));
+    }
+
+    // make sure, datePosted and dateProcessed are set for the transaction
+    if (!(kt.m_dateProcessed.isValid() || kt.m_datePosted.isValid())) {
+        DBG_WARN(0, "No date for transaction found, use today");
+        kt.m_dateProcessed = QDate::currentDate();
+        kt.m_datePosted = kt.m_dateProcessed;
+
     } else {
-        DBG_WARN(0, "No date for transaction");
+        if (!kt.m_dateProcessed.isValid()) {
+            kt.m_dateProcessed = kt.m_datePosted;
+        } else if (!kt.m_datePosted.isValid()) {
+            kt.m_datePosted = kt.m_dateProcessed;
+        }
+    }
+
+    // update the first and last date of the statement
+    // based on datePosted
+    if (!ks.m_dateBegin.isValid()) {
+        ks.m_dateBegin = kt.m_datePosted;
+    } else if (kt.m_datePosted < ks.m_dateBegin) {
+        ks.m_dateBegin = kt.m_datePosted;
+    }
+
+    if (!ks.m_dateEnd.isValid()) {
+        ks.m_dateEnd = kt.m_datePosted;
+    } else if (kt.m_datePosted > ks.m_dateEnd) {
+        ks.m_dateEnd = kt.m_datePosted;
     }
 
     // value
@@ -1279,29 +1355,13 @@ void KBankingExt::_xaToStatement(MyMoneyStatement &ks,
         // different nowadays, we make sure to use 100 for the
         // duplicate detection
         QString tmpVal = kt.m_amount.formatMoney(100, false);
-        tmpVal.remove(QRegExp("[,\\.]"));
+        tmpVal.remove(QRegularExpression(QLatin1String("[,\\.]")));
         tmpVal += QLatin1String("/100");
         h = MyMoneyTransaction::hash(tmpVal, h);
     } else {
         DBG_WARN(0, "No value for transaction");
     }
 
-    if (startDate) {
-        QDate d(QDate(GWEN_Date_GetYear(startDate), GWEN_Date_GetMonth(startDate), GWEN_Date_GetDay(startDate)));
-
-        if (!ks.m_dateBegin.isValid())
-            ks.m_dateBegin = d;
-        else if (d < ks.m_dateBegin)
-            ks.m_dateBegin = d;
-
-        if (!ks.m_dateEnd.isValid())
-            ks.m_dateEnd = d;
-        else if (d > ks.m_dateEnd)
-            ks.m_dateEnd = d;
-    } else {
-        DBG_WARN(0, "No date in current transaction");
-    }
-
     // add information about remote account to memo in case we have something
     const char *remoteAcc = AB_Transaction_GetRemoteAccountNumber(t);
     const char *remoteBankCode = AB_Transaction_GetRemoteBankCode(t);
@@ -1311,21 +1371,20 @@ void KBankingExt::_xaToStatement(MyMoneyStatement &ks,
 
     // make hash value unique in case we don't have one already
     if (kt.m_strBankID.isEmpty()) {
-        QString hashBase;
-        hashBase.sprintf("%s-%07lx", qPrintable(kt.m_datePosted.toString(Qt::ISODate)), h);
+        const auto hashBase(QStringLiteral("%1-%2").arg(kt.m_datePosted.toString(Qt::ISODate)).arg(h, 7, 16, QLatin1Char('0')));
         int idx = 1;
         QString hash;
         for (;;) {
             hash = QString("%1-%2").arg(hashBase).arg(idx);
             QMap<QString, bool>::const_iterator it;
             it = m_hashMap.constFind(hash);
-            if (it == m_hashMap.constEnd()) {
+            if (it == m_hashMap.cend()) {
                 m_hashMap[hash] = true;
                 break;
             }
             ++idx;
         }
-        kt.m_strBankID = QString("%1-%2").arg(acc.id()).arg(hash);
+        kt.m_strBankID = QString("%1-%2").arg(acc.id(), hash);
     }
 
     // store transaction
@@ -1338,8 +1397,6 @@ void KBankingExt::_slToStatement(MyMoneyStatement &ks,
                                  const AB_SECURITY *sy)
 {
     MyMoneyFile* file = MyMoneyFile::instance();
-    QString s;
-    QString memo;
     const char *p;
     const AB_VALUE *val;
     const GWEN_TIME *ti;
@@ -1378,7 +1435,8 @@ void KBankingExt::_slToStatement(MyMoneyStatement &ks,
     // generate dummy booking in case online balance does not match
     MyMoneySecurity security;
     MyMoneyAccount sacc;
-    foreach (const auto sAccount, file->account(acc.id()).accountList()) {
+    const auto accountList = file->account(acc.id()).accountList();
+    for (const auto& sAccount : accountList) {
         sacc=file->account(sAccount);
         security=file->security(sacc.currencyId());
         if ((!ksy.m_strSymbol.isEmpty() && QString::compare(ksy.m_strSymbol, security.tradingSymbol(), Qt::CaseInsensitive) == 0) ||
@@ -1432,12 +1490,12 @@ bool KBankingExt::importAccountInfo(AB_IMEXPORTER_CONTEXT *ctx,
 
     p = AB_ImExporterAccountInfo_GetBankCode(ai);
     if (p) {
-        ks.m_strRoutingNumber = m_parent->stripLeadingZeroes(p);
+        ks.m_strBankCode = m_parent->stripLeadingZeroes(p);
     }
 
     MyMoneyAccount kacc;
-    if (!ks.m_strAccountNumber.isEmpty() || !ks.m_strRoutingNumber.isEmpty()) {
-        kacc = m_parent->account("kbanking-acc-ref", QString("%1-%2-%3").arg(ks.m_strRoutingNumber, ks.m_strAccountNumber).arg(AB_ImExporterAccountInfo_GetAccountType(ai)));
+    if (!ks.m_strAccountNumber.isEmpty() || !ks.m_strBankCode.isEmpty()) {
+        kacc = m_parent->account("kbanking-acc-ref", QString("%1-%2-%3").arg(ks.m_strBankCode, ks.m_strAccountNumber).arg(AB_ImExporterAccountInfo_GetAccountType(ai)));
         ks.m_accountId = kacc.id();
     }
 
@@ -1511,9 +1569,12 @@ bool KBankingExt::importAccountInfo(AB_IMEXPORTER_CONTEXT *ctx,
 
     // import them
     if (!m_parent->importStatement(ks)) {
-        if (KMessageBox::warningYesNo(0,
-                                      i18n("Error importing statement. Do you want to continue?"),
-                                      i18n("Critical Error")) == KMessageBox::No) {
+        if (KMessageBox::warningTwoActions(0,
+                                           i18n("Error importing statement. Do you want to continue?"),
+                                           i18n("Critical Error"),
+                                           KMMYesNo::yes(),
+                                           KMMYesNo::no())
+            == KMessageBox::SecondaryAction) {
             DBG_ERROR(0, "User aborted");
             return false;
         }
@@ -1521,6 +1582,6 @@ bool KBankingExt::importAccountInfo(AB_IMEXPORTER_CONTEXT *ctx,
     return true;
 }
 
-K_PLUGIN_FACTORY_WITH_JSON(KBankingFactory, "kbanking.json", registerPlugin<KBanking>();)
+K_PLUGIN_CLASS_WITH_JSON(KBanking, "kbanking.json")
 
 #include "kbanking.moc"
