@@ -3,6 +3,7 @@
     SPDX-FileCopyrightText: 2009 Cristian Onet onet.cristian @gmail.com
     SPDX-FileCopyrightText: 2010-2019 Thomas Baumgart tbaumgart @kde.org
     SPDX-FileCopyrightText: 2016 Christian David christian-david @web.de
+    SPDX-FileCopyrightText: 2021 Dawid Wróbel <me@dawidwrobel.com>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 #ifndef KBANKING_H
@@ -15,8 +16,6 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
-#include <QSet>
-
 // ----------------------------------------------------------------------------
 // KDE & Library Includes
 
@@ -53,7 +52,7 @@ class KBanking : public KMyMoneyPlugin::OnlinePluginExtended
                  KMyMoneyPlugin::OnlinePlugin)
 
 public:
-    explicit KBanking(QObject *parent, const QVariantList &args);
+    explicit KBanking(QObject *parent, const KPluginMetaData &metaData, const QVariantList &args);
     ~KBanking() override;
 
     bool importStatement(const MyMoneyStatement& s);
@@ -64,12 +63,12 @@ public:
 
     void protocols(QStringList& protocolList) const override;
 
-    QStringList availableJobs(QString accountId) override;
+    QStringList availableJobs(QString accountId) const override;
     IonlineTaskSettings::ptr settings(QString accountId, QString taskName) override;
 
     void sendOnlineJob(QList<onlineJob>& jobs) override;
 
-    void plug() override;
+    void plug(KXMLGUIFactory* guiFactory) override;
     void unplug() override;
 
 private:
@@ -233,7 +232,7 @@ private:
     KBanking* m_parent;
     QMap<QString, bool> m_hashMap;
     AB_TRANSACTION_LIST2 *_jobQueue;
-    QSet<QString>   m_sepaKeywords;
+    KMMStringSet m_sepaKeywords;
 };
 
 #endif // KBANKING
