@@ -4,7 +4,6 @@
 */
 
 #include "kofxdirectconnectdlg.h"
-#include "kmymoneysettings.h"
 
 // ----------------------------------------------------------------------------
 // QT Includes
@@ -31,6 +30,8 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
+#include "config-kmymoney.h"
+#include "kmymoneysettings.h"
 #include "mymoneyofxconnector.h"
 
 class KOfxDirectConnectDlg::Private
@@ -124,8 +125,8 @@ bool KOfxDirectConnectDlg::init()
         m_job->addMetaData(QLatin1String("SendUserAgent"), QLatin1String("true"));
     }
 
-    connect(m_job, SIGNAL(result(KJob*)), this, SLOT(slotOfxFinished(KJob*)));
-    connect(m_job, SIGNAL(data(KIO::Job*,QByteArray)), this, SLOT(slotOfxData(KIO::Job*,QByteArray)));
+    connect(m_job, &KJob::result, this, &KOfxDirectConnectDlg::slotOfxFinished);
+    connect(m_job, &KIO::TransferJob::data, this, &KOfxDirectConnectDlg::slotOfxData);
 
     setStatus(QString("Contacting %1...").arg(m_connector.url()));
     kProgress1->setMaximum(3);
@@ -146,7 +147,7 @@ void KOfxDirectConnectDlg::setDetails(const QString& _details)
 
 void KOfxDirectConnectDlg::slotOfxData(KIO::Job*, const QByteArray& _ba)
 {
-    qDebug("Got %d bytes of data", _ba.size());
+    qDebug("Got " QLIST_COUNT_FORMAT " bytes of data", _ba.size());
     if (d->m_firstData) {
         setStatus("Connection established, retrieving data...");
         setDetails(QString("Downloading data to %1...").arg(m_tmpfile->fileName()));
@@ -185,6 +186,7 @@ void KOfxDirectConnectDlg::slotOfxFinished(KJob* /* e */)
     if (error) {
         qDebug("Show error message");
         m_job->uiDelegate()->showErrorMessage();
+#if QT_VERSION < QT_VERSION_CHECK(6, 3, 0)
     } else if (m_job->isErrorPage()) {
         qDebug("Process error page");
         QString details;
@@ -200,10 +202,11 @@ void KOfxDirectConnectDlg::slotOfxFinished(KJob* /* e */)
                 qDebug() << "The HTTP request failed: " << details;
             }
         }
-        KMessageBox::detailedSorry(this, i18n("The HTTP request failed."), details, i18nc("The HTTP request failed", "Failed"));
+        KMessageBox::detailedError(this, i18n("The HTTP request failed."), details, i18nc("The HTTP request failed", "Failed"));
+#endif
     } else if (m_tmpfile) {
         qDebug("Emit statementReady signal with '%s'", qPrintable(m_tmpfile->fileName()));
-        emit statementReady(m_tmpfile->fileName());
+        Q_EMIT statementReady(m_tmpfile->fileName());
         qDebug("Return from signal statementReady() processing");
     }
     delete m_tmpfile;
