@@ -1,5 +1,6 @@
 /*
     SPDX-FileCopyrightText: 2006 Ace Jones <acejones@users.sourceforge.net>
+    SPDX-FileCopyrightText: 2022 Dawid Wróbel <me@dawidwrobel.com>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
@@ -8,44 +9,45 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
-#include <QLabel>
-#include <QRegExp>
 #include <QCheckBox>
+#include <QLabel>
+#include <QProgressDialog>
+#include <QRegularExpression>
+#include <QStandardPaths>
 #include <QTabWidget>
+#include <QTextCodec>
 #include <QTextStream>
 #include <QTimer>
+#include <QUuid>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
 
+#include <KComboBox>
+#include <KListWidgetSearchLine>
 #include <KLocalizedString>
 #include <KMessageBox>
-
-#include <QProgressDialog>
-#include <KListWidgetSearchLine>
-#include <KComboBox>
 #include <KUrlRequester>
-#include <KWallet>
-#include <QStandardPaths>
 
 // ----------------------------------------------------------------------------
 // Project Includes
 
 #include "../ofxpartner.h"
+#include "kmmkeychain.h"
+#include "kmymoneysettings.h"
 #include "mymoneyofxconnector.h"
 #include "passwordtoggle.h"
 
-using KWallet::Wallet;
-
 class KOnlineBankingSetupWizard::Private
 {
 public:
-    Private() : m_prevPage(-1), m_wallet(0), m_walletIsOpen(false) {}
+    Private()
+        : m_prevPage(-1)
+    {
+    }
     QFile       m_fpTrace;
     QTextStream m_trace;
-    int         m_prevPage;
-    Wallet      *m_wallet;
-    bool        m_walletIsOpen;
+    int m_prevPage;
 };
 
 KOnlineBankingSetupWizard::KOnlineBankingSetupWizard(QWidget *parent):
@@ -59,10 +61,6 @@ KOnlineBankingSetupWizard::KOnlineBankingSetupWizard(QWidget *parent):
 
     m_applicationEdit->hide();
     m_headerVersionEdit->hide();
-#ifndef LIBOFX_HAVE_CLIENTUID
-    m_editClientUid->setEnabled(false);
-    m_clientUidLabel->setEnabled(false);
-#endif
 
     m_appId = new OfxAppVersion(m_applicationCombo, m_applicationEdit, "");
     m_headerVersion = new OfxHeaderVersion(m_headerVersionCombo, "");
@@ -79,7 +77,7 @@ KOnlineBankingSetupWizard::KOnlineBankingSetupWizard(QWidget *parent):
     QCoreApplication::processEvents();
 
     //set password field according to KDE preferences
-    m_editPassword->setPasswordMode(true);
+    m_editPassword->setEchoMode(QLineEdit::EchoMode::PasswordEchoOnEdit);
     new PasswordToggle(m_editPassword);
 
     // make sure to not exceed data fields
@@ -90,24 +88,25 @@ KOnlineBankingSetupWizard::KOnlineBankingSetupWizard(QWidget *parent):
     vboxLayout1->insertWidget(0, searchLine);
     QTimer::singleShot(20, searchLine, SLOT(setFocus()));
 
-    OfxPartner::setDirectory(QStandardPaths::writableLocation(QStandardPaths::DataLocation) + QLatin1Char('/') + "");
+    OfxPartner::setDirectory(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + QLatin1Char('/') + "");
     m_listFi->addItems(OfxPartner::BankNames());
     m_fInit = true;
     delete dlg;
 
     checkNextButton();
-    connect(this, SIGNAL(currentIdChanged(int)), this, SLOT(checkNextButton()));
-    connect(this, SIGNAL(currentIdChanged(int)), this, SLOT(newPage(int)));
-    connect(m_listFi, SIGNAL(itemSelectionChanged()), this, SLOT(checkNextButton()));
-    connect(m_listAccount, SIGNAL(itemSelectionChanged()), this, SLOT(checkNextButton()));
-    connect(m_selectionTab, SIGNAL(currentChanged(int)), this, SLOT(checkNextButton()));
-    connect(m_fid, SIGNAL(userTextChanged(QString)), this, SLOT(checkNextButton()));
-    connect(m_bankName, SIGNAL(userTextChanged(QString)), this, SLOT(checkNextButton()));
-    connect(m_url, SIGNAL(textChanged(QString)), this, SLOT(checkNextButton()));
-    connect(m_editUsername, SIGNAL(userTextChanged(QString)), this, SLOT(checkNextButton()));
-    connect(m_editPassword, SIGNAL(userTextChanged(QString)), this, SLOT(checkNextButton()));
-    connect(m_applicationEdit, SIGNAL(userTextChanged(QString)), this, SLOT(checkNextButton()));
-    connect(m_applicationCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(applicationSelectionChanged()));
+    connect(this, &KOnlineBankingSetupWizard::currentIdChanged, this, &KOnlineBankingSetupWizard::checkNextButton);
+    connect(this, &KOnlineBankingSetupWizard::currentIdChanged, this, &KOnlineBankingSetupWizard::newPage);
+
+    connect(m_listFi, &QListWidget::itemSelectionChanged, this, &KOnlineBankingSetupWizard::checkNextButton);
+    connect(m_listAccount, &QTreeWidget::itemSelectionChanged, this, &KOnlineBankingSetupWizard::checkNextButton);
+    connect(m_selectionTab, &QTabWidget::currentChanged, this, &KOnlineBankingSetupWizard::checkNextButton);
+    connect(m_fid, &QLineEdit::textEdited, this, &KOnlineBankingSetupWizard::checkNextButton);
+    connect(m_bankName, &QLineEdit::textEdited, this, &KOnlineBankingSetupWizard::checkNextButton);
+    connect(m_url, &KUrlRequester::textChanged, this, &KOnlineBankingSetupWizard::checkNextButton);
+    connect(m_editUsername, &QLineEdit::textEdited, this, &KOnlineBankingSetupWizard::checkNextButton);
+    connect(m_editPassword, &QLineEdit::textEdited, this, &KOnlineBankingSetupWizard::checkNextButton);
+    connect(m_applicationEdit, &QLineEdit::textEdited, this, &KOnlineBankingSetupWizard::checkNextButton);
+    connect(m_applicationCombo, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &KOnlineBankingSetupWizard::applicationSelectionChanged);
 
     // setup text on buttons
     setButtonText(QWizard::NextButton, i18nc("Go to next page of the wizard", "&Next"));
@@ -135,18 +134,6 @@ void KOnlineBankingSetupWizard::applicationSelectionChanged()
     checkNextButton();
 }
 
-void KOnlineBankingSetupWizard::walletOpened(bool ok)
-{
-    if (ok && (d->m_wallet->hasFolder(KWallet::Wallet::PasswordFolder()) ||
-               d->m_wallet->createFolder(KWallet::Wallet::PasswordFolder())) &&
-            d->m_wallet->setFolder(KWallet::Wallet::PasswordFolder())) {
-        d->m_walletIsOpen = true;
-    } else {
-        qDebug("Wallet was not opened");
-    }
-    m_storePassword->setEnabled(d->m_walletIsOpen);
-}
-
 void KOnlineBankingSetupWizard::checkNextButton()
 {
     bool enableButton = false;
@@ -185,11 +172,6 @@ void KOnlineBankingSetupWizard::newPage(int id)
         switch (d->m_prevPage) {
         case 0:
             ok = finishFiPage();
-            // open the KDE wallet if not already opened
-            if (ok && !d->m_wallet) {
-                d->m_wallet = Wallet::openWallet(Wallet::NetworkWallet(), winId(), Wallet::Asynchronous);
-                connect(d->m_wallet, SIGNAL(walletOpened(bool)), SLOT(walletOpened(bool)));
-            }
             focus = m_editUsername;
             break;
         case 1:
@@ -240,8 +222,8 @@ bool KOnlineBankingSetupWizard::finishFiPage()
             m_textDetails->clear();
             m_textDetails->append(QString("<p>Details for %1:</p>").arg(bank));
             QStringList fipids = OfxPartner::FipidForBank(bank);
-            QStringList::const_iterator it_fipid = fipids.constBegin();
-            while (it_fipid != fipids.constEnd()) {
+            QStringList::const_iterator it_fipid = fipids.cbegin();
+            while (it_fipid != fipids.cend()) {
                 // For each fipid, get the connection details
                 info = OfxPartner::ServiceInfo(*it_fipid);
 
@@ -283,13 +265,13 @@ bool KOnlineBankingSetupWizard::finishFiPage()
             result = true;
         } else
             // error!  No current item
-            KMessageBox::sorry(this, i18n("Please choose a bank."));
+            KMessageBox::error(this, i18n("Please choose a bank."));
 
     } else {  // manual entry of values
         if (m_fid->text().isEmpty()
                 || m_url->url().isEmpty()
                 || m_bankName->text().isEmpty()) {
-            KMessageBox::sorry(this, i18n("Please fill all fields with values."));
+            KMessageBox::error(this, i18n("Please fill all fields with values."));
         }
 
         m_textDetails->clear();
@@ -328,15 +310,16 @@ bool KOnlineBankingSetupWizard::finishLoginPage()
 {
     bool result = true;
 
-    QString username = m_editUsername->text();
-    QString password = m_editPassword->text();
-    QString clientUid = m_editClientUid->text();
+    const auto username = m_editUsername->text();
+    const auto password = m_editPassword->text();
+    const auto clientUid = m_editClientUid->text();
+    const auto userAgent = m_userAgent->text();
 
     m_listAccount->clear();
 
     // Process an account request for each fipid
-    m_it_info = m_bankInfo.constBegin();
-    while (m_it_info != m_bankInfo.constEnd()) {
+    m_it_info = m_bankInfo.cbegin();
+    while (m_it_info != m_bankInfo.cend()) {
         OfxFiLogin fi;
         memset(&fi, 0, sizeof(OfxFiLogin));
         Q_ASSERT(sizeof(fi.fid) == sizeof((*m_it_info).fid));
@@ -345,21 +328,20 @@ bool KOnlineBankingSetupWizard::finishLoginPage()
         memcpy(fi.org, (*m_it_info).org, OFX_ORG_LENGTH - 1);
         strncpy(fi.userid, username.toLatin1().left(OFX_USERID_LENGTH - 1), OFX_USERID_LENGTH - 1);
         strncpy(fi.userpass, password.toLatin1(), OFX_USERPASS_LENGTH - 1);
-#ifdef LIBOFX_HAVE_CLIENTUID
         strncpy(fi.clientuid, clientUid.toLatin1(), OFX_CLIENTUID_LENGTH - 1);
-#endif
 
         // pretend we're Quicken 2008
         // https://ofxblog.wordpress.com/2007/06/06/ofx-appid-and-appver-for-intuit-products/
         // https://ofxblog.wordpress.com/2007/06/06/ofx-appid-and-appver-for-microsoft-money/
         QString appId = m_appId->appId();
-        QRegExp exp("(.*):(.*)");
-        if (exp.indexIn(appId) != -1) {
-            strncpy(fi.appid, exp.cap(1).toLatin1(), OFX_APPID_LENGTH - 1);
-            if (exp.cap(2).isEmpty()) {
+        static const QRegularExpression appVersionExp(QLatin1String("(.*):(.*)"));
+        const auto appVersion(appVersionExp.match(appId));
+        if (appVersion.hasMatch()) {
+            strncpy(fi.appid, appVersion.captured(1).toLatin1(), OFX_APPID_LENGTH - 1);
+            if (appVersion.captured(2).isEmpty()) {
                 strncpy(fi.appver, m_applicationEdit->text().toLatin1(), OFX_APPVER_LENGTH - 1);
             } else {
-                strncpy(fi.appver, exp.cap(2).toLatin1(), OFX_APPVER_LENGTH - 1);
+                strncpy(fi.appver, appVersion.captured(2).toLatin1(), OFX_APPVER_LENGTH - 1);
             }
         } else {
             strncpy(fi.appid, "QWIN", OFX_APPID_LENGTH - 1);
@@ -369,14 +351,26 @@ bool KOnlineBankingSetupWizard::finishLoginPage()
         QString hver = m_headerVersion->headerVersion();
         strncpy(fi.header_version, hver.toLatin1(), OFX_HEADERVERSION_LENGTH - 1);
 
-        QUrl filename(QString("file://%1response.ofx").arg(QStandardPaths::writableLocation(QStandardPaths::DataLocation) + QLatin1Char('/')));
-        QByteArray req(libofx_request_accountinfo(&fi));
+        auto codec = QTextCodec::codecForName("Windows-1251");
+        auto request = codec->toUnicode(libofx_request_accountinfo(&fi));
+
+        // Check if we need to tweak the request for specific institutions
+        MyMoneyOfxConnector::institutionSpecificRequestAdjustment(request);
+
+        QUrl filename(QString("file://%1response.ofx").arg(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + QLatin1Char('/')));
+
         // because the event loop is running while the request is performed disable the back button
         // (this function is not reentrant so the application might crash when back/next are used)
         QAbstractButton *backButton = button(QWizard::BackButton);
         bool backButtonState = backButton->isEnabled();
         backButton->setEnabled(false);
-        OfxHttpRequest(QString("POST"), QUrl((*m_it_info).url), req, QMap<QString, QString>(), filename, false);
+
+        QMap<QString, QString> metaData;
+        if (!userAgent.isEmpty()) {
+            metaData.insert(QLatin1String("UserAgent"), userAgent);
+        }
+
+        OfxHttpRequest(QString("POST"), QUrl((*m_it_info).url), codec->fromUnicode(request), metaData, filename, false);
         backButton->setEnabled(backButtonState);
 
         LibofxContextPtr ctx = libofx_get_new_context();
@@ -392,7 +386,7 @@ bool KOnlineBankingSetupWizard::finishLoginPage()
     }
 
     if (! m_listAccount->topLevelItem(0)) {
-        KMessageBox::sorry(this, i18n("No suitable accounts were found at this bank."));
+        KMessageBox::error(this, i18n("No suitable accounts were found at this bank."));
         result = false;
     } else {
         m_listAccount->resizeColumnToContents(0);
@@ -408,7 +402,7 @@ bool KOnlineBankingSetupWizard::finishAccountPage()
     bool result = true;
 
     if (! m_listAccount->currentItem()) {
-        KMessageBox::sorry(this, i18n("Please choose an account"));
+        KMessageBox::error(this, i18n("Please choose an account"));
         result = false;
     }
 
@@ -453,29 +447,29 @@ int KOnlineBankingSetupWizard::ofxAccountCallback(struct OfxAccountData data, vo
     }
 
     if (data.bank_id_valid)
-        kvps.setValue("bankid", data.bank_id);
+        kvps.setValue("bankid", QLatin1String(data.bank_id));
 
     if (data.broker_id_valid)
-        kvps.setValue("bankid", data.broker_id);
+        kvps.setValue("bankid", QLatin1String(data.broker_id));
 
     if (data.branch_id_valid)
-        kvps.setValue("branchid", data.branch_id);
+        kvps.setValue("branchid", QLatin1String(data.branch_id));
 
     if (data.account_number_valid)
-        kvps.setValue("accountid", data.account_number);
+        kvps.setValue("accountid", QLatin1String(data.account_number));
 
     if (data.account_id_valid)
-        kvps.setValue("uniqueId", data.account_id);
+        kvps.setValue("uniqueId", QLatin1String(data.account_id));
 
     kvps.setValue("username", pthis->m_editUsername->text());
     kvps.setValue("password", pthis->m_editPassword->text());
-#ifdef LIBOFX_HAVE_CLIENTUID
     kvps.setValue("clientUid", pthis->m_editClientUid->text());
-#endif
-
-    kvps.setValue("url", (*(pthis->m_it_info)).url);
-    kvps.setValue("fid", (*(pthis->m_it_info)).fid);
-    kvps.setValue("org", (*(pthis->m_it_info)).org);
+    if (!pthis->m_userAgent->text().isEmpty()) {
+        kvps.setValue("kmmofx-useragent", pthis->m_userAgent->text());
+    }
+    kvps.setValue("url", QLatin1String((*(pthis->m_it_info)).url));
+    kvps.setValue("fid", QLatin1String((*(pthis->m_it_info)).fid));
+    kvps.setValue("org", QLatin1String((*(pthis->m_it_info)).org));
     kvps.setValue("fipid", "");
     QListWidgetItem* item = pthis->m_listFi->currentItem();
     if (item)
@@ -553,14 +547,12 @@ bool KOnlineBankingSetupWizard::chosenSettings(MyMoneyKeyValueContainer& setting
             if (!hVer.isEmpty())
                 settings.setValue("kmmofx-headerVersion", hVer);
             if (m_storePassword->isChecked()) {
-                if (d->m_walletIsOpen) {
-                    QString key = OFX_PASSWORD_KEY(settings.value("url"), settings.value("uniqueId"));
-                    d->m_wallet->writePassword(key, settings.value("password"));
-                    settings.deletePair("password");
-                }
-            } else {
-                settings.deletePair("password");
+                const QString key = OFX_PASSWORD_KEY(settings.value("url"), settings.value("uniqueId"));
+                auto keyChain = new KMMKeychain();
+                keyChain->writeKey(key, settings.value("password"));
             }
+            settings.deletePair("password");
+
             result = true;
         }
     }
