@@ -1,5 +1,6 @@
 /*
     SPDX-FileCopyrightText: 2002 Ace Jones <acejones@users.sourceforge.net>
+    SPDX-FileCopyrightText: 2022 Dawid Wróbel <me@dawidwrobel.com>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
@@ -18,7 +19,6 @@
 #include <QDateTime>
 #include <QDebug>
 #include <QList>
-#include <QRegExp>
 #include <QRegularExpression>
 #include <QRegularExpressionMatch>
 #include <QTextCodec>
@@ -27,21 +27,19 @@
 // ----------------------------------------------------------------------------
 // KDE Includes
 
-#include <KLocalizedString>
 #include <KComboBox>
-#include <KPasswordDialog>
-#include <KWallet>
-#include <KMainWindow>
 #include <KLineEdit>
+#include <KLocalizedString>
+#include <KMainWindow>
+#include <KPasswordDialog>
 
 // ----------------------------------------------------------------------------
 // Project Includes
 
+#include "kmmkeychain.h"
 #include "mymoneyaccount.h"
-#include "mymoneykeyvaluecontainer.h"
 #include "mymoneyenums.h"
-
-using KWallet::Wallet;
+#include "mymoneykeyvaluecontainer.h"
 
 OfxHeaderVersion::OfxHeaderVersion(KComboBox* combo, const QString& headerVersion) :
     m_combo(combo)
@@ -115,17 +113,17 @@ OfxAppVersion::OfxAppVersion(KComboBox* combo, KLineEdit* versionEdit, const QSt
 
     QMap<QString, QString>::const_iterator it_a;
     // check for an exact match
-    for (it_a = m_appMap.constBegin(); it_a != m_appMap.constEnd(); ++it_a) {
+    for (it_a = m_appMap.cbegin(); it_a != m_appMap.cend(); ++it_a) {
         if (*it_a == appId)
             break;
     }
 
     // not found, check if we have a manual version of this product
-    QRegularExpression appExp("(\\w+:)(\\d+|\\w+)");
+    static const QRegularExpression appExp("(\\w+:)(\\d+|\\w+)");
     auto matcher = appExp.match(appId);
-    if (it_a == m_appMap.constEnd()) {
+    if (it_a == m_appMap.cend()) {
         if (matcher.hasMatch()) {
-            for (it_a = m_appMap.constBegin(); it_a != m_appMap.constEnd(); ++it_a) {
+            for (it_a = m_appMap.cbegin(); it_a != m_appMap.cend(); ++it_a) {
                 if (*it_a == matcher.captured(1))
                     break;
             }
@@ -133,7 +131,7 @@ OfxAppVersion::OfxAppVersion(KComboBox* combo, KLineEdit* versionEdit, const QSt
     }
 
     // if we still haven't found it, we use a default as last resort
-    if (it_a != m_appMap.constEnd()) {
+    if (it_a != m_appMap.cend()) {
         combo->setCurrentItem(it_a.key());
         if ((*it_a).endsWith(':')) {
             if (versionEdit) {
@@ -168,7 +166,7 @@ const QString OfxAppVersion::appId() const
 
 bool OfxAppVersion::isValid() const
 {
-    QRegExp exp(".+:\\d+");
+    static const QRegularExpression validAppVersionExp(QRegularExpression::anchoredPattern(QLatin1String(".+:\\d+")));
     QString app = m_combo->currentText();
     if (m_appMap[app].endsWith(':')) {
         if (m_versionEdit) {
@@ -179,7 +177,7 @@ bool OfxAppVersion::isValid() const
     } else {
         app = m_appMap[app];
     }
-    return exp.exactMatch(app);
+    return validAppVersionExp.match(app).hasMatch();
 }
 
 MyMoneyOfxConnector::MyMoneyOfxConnector(const MyMoneyAccount& _account):
@@ -210,20 +208,18 @@ QString MyMoneyOfxConnector::username() const
 }
 QString MyMoneyOfxConnector::password() const
 {
-    // if we don't find a password in the wallet, we use the old method
-    // and retrieve it from the settings stored in the KMyMoney data storage.
-    // in case we don't have a password on file, we ask the user
-    QString key = OFX_PASSWORD_KEY(m_fiSettings.value("url"), m_fiSettings.value("uniqueId"));
-    QString pwd = m_fiSettings.value("password");
+    const QString key = OFX_PASSWORD_KEY(m_fiSettings.value("url"), m_fiSettings.value("uniqueId"));
 
-    // now check for the wallet
-    Wallet *wallet = openSynchronousWallet();
-    if (wallet
-            && !Wallet::keyDoesNotExist(Wallet::NetworkWallet(), Wallet::PasswordFolder(), key)) {
-        wallet->setFolder(Wallet::PasswordFolder());
-        wallet->readPassword(key, pwd);
+    auto keyChain = new KMMKeychain();
+    QString pwd = keyChain->readKeySynchronous(key);
+
+    // if we don't find a password in the Keychain, we use the old method
+    // and retrieve it from the settings stored in the KMyMoney data storage.
+    if (pwd.isEmpty()) {
+        pwd = m_fiSettings.value("password");
     }
 
+    // in case we don't have a password on file, we ask the user
     if (pwd.isEmpty()) {
         QPointer<KPasswordDialog> dlg = new KPasswordDialog(0);
         dlg->setPrompt(i18n("Enter your password for account <b>%1</b>", m_account.name()));
@@ -231,6 +227,7 @@ QString MyMoneyOfxConnector::password() const
             pwd = dlg->password();
         delete dlg;
     }
+
     return pwd;
 }
 QString MyMoneyOfxConnector::accountnum() const
@@ -303,19 +300,21 @@ OfxAccountData::AccountType MyMoneyOfxConnector::accounttype() const
     // This is a bit of a personalized hack.  Sometimes we may want to override the
     // ofx type for an account.  For now, I will stash it in the notes!
 
-    QRegExp rexp("OFXTYPE:([A-Z]*)");
-    if (rexp.indexIn(m_account.description()) != -1) {
-        QString override = rexp.cap(1);
-        qDebug() << "MyMoneyOfxConnector::accounttype() overriding to " << result;
+    static const QRegularExpression accountTypeExp(QLatin1String("OFXTYPE:([A-Z]*)"));
+    const auto accountType(accountTypeExp.match(m_account.description()));
+    if (accountType.hasMatch()) {
+        const auto override = accountType.captured(1);
 
-        if (override == "BANK")
+        if (override == QLatin1String("BANK"))
             result = OfxAccountData::OFX_CHECKING;
-        else if (override == "CC")
+        else if (override == QLatin1String("CC"))
             result = OfxAccountData::OFX_CREDITCARD;
-        else if (override == "INV")
+        else if (override == QLatin1String("INV"))
             result = OfxAccountData::OFX_INVESTMENT;
-        else if (override == "MONEYMARKET")
+        else if (override == QLatin1String("MONEYMARKET"))
             result = OfxAccountData::OFX_MONEYMRKT;
+
+        qDebug() << "MyMoneyOfxConnector::accounttype() overriding to " << result;
     }
 
     return result;
@@ -328,18 +327,17 @@ void MyMoneyOfxConnector::initRequest(OfxFiLogin* fi) const
     strncpy(fi->org, fiorg().toLatin1(), OFX_ORG_LENGTH - 1);
     strncpy(fi->userid, username().toLatin1(), OFX_USERID_LENGTH - 1);
     strncpy(fi->userpass, password().toLatin1(), OFX_USERPASS_LENGTH - 1);
-#ifdef LIBOFX_HAVE_CLIENTUID
     strncpy(fi->clientuid, clientUid().toLatin1(), OFX_CLIENTUID_LENGTH - 1);
-#endif
 
     // If we don't know better, we pretend to be Quicken 2008
     // https://ofxblog.wordpress.com/2007/06/06/ofx-appid-and-appver-for-intuit-products/
     // https://ofxblog.wordpress.com/2007/06/06/ofx-appid-and-appver-for-microsoft-money/
     QString appId = m_account.onlineBankingSettings().value("appId");
-    QRegExp exp("(.*):(.*)");
-    if (exp.indexIn(appId) != -1) {
-        strncpy(fi->appid, exp.cap(1).toLatin1(), OFX_APPID_LENGTH - 1);
-        strncpy(fi->appver, exp.cap(2).toLatin1(), OFX_APPVER_LENGTH - 1);
+    static const QRegularExpression exp("(.*):(.*)");
+    QRegularExpressionMatch match = exp.match(appId);
+    if (match.hasMatch()) {
+        strncpy(fi->appid, match.captured(1).toLatin1(), OFX_APPID_LENGTH - 1);
+        strncpy(fi->appver, match.captured(2).toLatin1(), OFX_APPVER_LENGTH - 1);
     } else {
         strncpy(fi->appid, "QWIN", OFX_APPID_LENGTH - 1);
         strncpy(fi->appver, "1700", OFX_APPVER_LENGTH - 1);
@@ -368,7 +366,7 @@ QString MyMoneyOfxConnector::statementRequest() const
 
     QString result;
     if (fi.userpass[0]) {
-        char *szrequest = libofx_request_statement(&fi, &account, QDateTime(statementStartDate()).toTime_t());
+        char* szrequest = libofx_request_statement(&fi, &account, statementStartDate().startOfDay().toSecsSinceEpoch());
         auto codec = QTextCodec::codecForName("Windows-1251");
         result = codec->toUnicode(szrequest);
         free(szrequest);
@@ -381,13 +379,16 @@ QString MyMoneyOfxConnector::statementRequest() const
 
 void MyMoneyOfxConnector::institutionSpecificRequestAdjustment(QString& request)
 {
+    static const QRegularExpression newFileUidRegex("NEWFILEUID:[\\d\\.]+");
+    static const QRegularExpression trnUidRegex("<TRNUID>[\\d\\.]+");
+    static const QRegularExpression dateRegex("<DTACCTUP>19700101");
+
     if (request.contains(QLatin1String("<FID>67811")) || request.contains(QLatin1String("<FID>00000"))) {
         // USAA requires some specific settings
         // and we do the same with our test account
-        request.replace(QRegularExpression("NEWFILEUID:[\\d\\.]+"), QLatin1String("NEWFILEUID:NONE"));
-        request.replace(QRegularExpression("<TRNUID>[\\d\\.]+"),
-                        QStringLiteral("<TRNUID>%1").arg(QUuid::createUuid().toString(QUuid::WithoutBraces).toUpper()));
-        request.replace(QRegularExpression("<DTACCTUP>19700101"), QLatin1String("<DTACCTUP>19900101"));
+        request.replace(newFileUidRegex, QLatin1String("NEWFILEUID:NONE"));
+        request.replace(trnUidRegex, QStringLiteral("<TRNUID>%1").arg(QUuid::createUuid().toString(QUuid::WithoutBraces).toUpper()));
+        request.replace(dateRegex, QLatin1String("<DTACCTUP>19900101"));
     }
 }
 
@@ -757,39 +758,3 @@ MyMoneyOfxConnector::Tag MyMoneyOfxConnector::investmentTransaction(const MyMone
     return Tag("ERROR").element("DETAILS", "This transaction contains an unsupported action type");
 }
 #endif
-
-KWallet::Wallet *openSynchronousWallet()
-{
-    using KWallet::Wallet;
-
-    // first handle the simple case in which we already use the wallet but need the object again in
-    // this case the wallet access permission dialog will no longer appear so we don't need to pass
-    // a valid window id or do anything special since the function call should return immediately
-    const bool alreadyUsingTheWallet = Wallet::users(Wallet::NetworkWallet()).contains("KMyMoney");
-    if (alreadyUsingTheWallet) {
-        return Wallet::openWallet(Wallet::NetworkWallet(), 0, Wallet::Synchronous);
-    }
-
-    // search for a suitable parent for the wallet that needs to be deactivated while the
-    // wallet access permission dialog is not dismissed with either accept or reject
-    KWallet::Wallet *wallet = 0;
-    QWidget *parentWidgetForWallet = 0;
-    if (qApp->activeModalWidget()) {
-        parentWidgetForWallet = qApp->activeModalWidget();
-    } else if (qApp->activeWindow()) {
-        parentWidgetForWallet = qApp->activeWindow();
-    } else {
-        QList<KMainWindow *> mainWindowList = KMainWindow::memberList();
-        if (!mainWindowList.isEmpty())
-            parentWidgetForWallet = mainWindowList.front();
-    }
-    // only open the wallet synchronously if we have a valid parent otherwise crashes could occur
-    if (parentWidgetForWallet) {
-        // while the wallet is being opened disable the widget to prevent input processing
-        const bool enabled = parentWidgetForWallet->isEnabled();
-        parentWidgetForWallet->setEnabled(false);
-        wallet = Wallet::openWallet(Wallet::NetworkWallet(), parentWidgetForWallet->winId(), Wallet::Synchronous);
-        parentWidgetForWallet->setEnabled(enabled);
-    }
-    return wallet;
-}
