@@ -1,6 +1,7 @@
 /*
     SPDX-FileCopyrightText: 2005 Ace Jones acejones @users.sourceforge.net
     SPDX-FileCopyrightText: 2010-2019 Thomas Baumgart tbaumgart @kde.org
+    SPDX-FileCopyrightText: 2021-2022 Dawid Wróbel <me@dawidwrobel.com>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
@@ -10,30 +11,29 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
+#include <QByteArray>
 #include <QFile>
-#include <QTextStream>
 #include <QRadioButton>
 #include <QSpinBox>
-#include <QByteArray>
-#include <QCoreApplication>
-#include <QStandardPaths>
+#include <QTextCodec>
+#include <QTextStream>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
 
-#include <KPluginFactory>
-#include <QUrl>
-#include <KMessageBox>
 #include <KActionCollection>
-#include <KLocalizedString>
-#include <KWallet>
-#include <KSharedConfig>
 #include <KConfigGroup>
+#include <KLocalizedString>
+#include <KMessageBox>
+#include <KPluginFactory>
+#include <KSharedConfig>
+#include <QUrl>
 
 // ----------------------------------------------------------------------------
 // Project Includes
 
 #include "importinterface.h"
+#include "kmmkeychain.h"
 #include "kmymoneyutils.h"
 #include "kofxdirectconnectdlg.h"
 #include "konlinebankingsetupwizard.h"
@@ -42,15 +42,12 @@
 #include "mymoneyexception.h"
 #include "mymoneystatement.h"
 #include "mymoneystatementreader.h"
-#include "mymoneyutils.h"
 #include "statementinterface.h"
 #include "ui_importoption.h"
 #include "viewinterface.h"
 #include <libofx/libofx.h>
 
-//#define DEBUG_LIBOFX
-
-using KWallet::Wallet;
+// #define DEBUG_LIBOFX
 
 typedef enum  {
     UniqueIdUnknown = -1,
@@ -61,9 +58,18 @@ typedef enum  {
 class OFXImporter::Private
 {
 public:
-    Private() : m_valid(false), m_preferName(PreferId), m_uniqueIdSource(UniqueIdUnknown), m_walletIsOpen(false), m_invertAmount(false), m_fixBuySellSignage(false),
-        m_statusDlg(nullptr), m_wallet(nullptr), m_action(nullptr),
-        m_updateStartDate(QDate(1900,1,1)), m_timestampOffset(0) {}
+    Private()
+        : m_valid(false)
+        , m_preferName(PreferId)
+        , m_uniqueIdSource(UniqueIdUnknown)
+        , m_invertAmount(false)
+        , m_fixBuySellSignage(false)
+        , m_statusDlg(nullptr)
+        , m_action(nullptr)
+        , m_updateStartDate(QDate(1900, 1, 1))
+        , m_timestampOffset(0)
+    {
+    }
 
     bool m_valid;
     enum NamePreference {
@@ -71,8 +77,7 @@ public:
         PreferName,
         PreferMemo,
     } m_preferName;
-    UniqueTransactionIdSource  m_uniqueIdSource;
-    bool m_walletIsOpen;
+    UniqueTransactionIdSource m_uniqueIdSource;
     bool m_invertAmount;
     bool m_fixBuySellSignage;
     QList<MyMoneyStatement> m_statementlist;
@@ -82,11 +87,10 @@ public:
     QStringList m_warnings;
     QStringList m_errors;
     KOnlineBankingStatus* m_statusDlg;
-    Wallet *m_wallet;
     QAction* m_action;
     QDate m_updateStartDate;
     int m_timestampOffset;
-    QSet<QString> m_hashes;
+    KMMStringSet m_hashes;
 
     int constructTimeOffset(const QTimeEdit* timestampOffset, const KComboBox* timestampOffsetSign) const
     {
@@ -98,6 +102,11 @@ public:
         return offset;
     }
 
+    QString sanitizedString(const char* str)
+    {
+        auto text = QString::fromUtf8(str);
+        return text.replace(QChar(0), QString());
+    }
 };
 
 
@@ -110,8 +119,8 @@ static UniqueTransactionIdSource defaultIdSource()
 }
 
 
-OFXImporter::OFXImporter(QObject *parent, const QVariantList &args) :
-    KMyMoneyPlugin::Plugin(parent, "ofximporter"),
+OFXImporter::OFXImporter(QObject *parent, const KPluginMetaData &metaData, const QVariantList &args) :
+    KMyMoneyPlugin::Plugin(parent, metaData, args),
     /*
      * the string in the line above must be the same as
      * X-KDE-PluginInfo-Name and the provider name assigned in
@@ -120,21 +129,11 @@ OFXImporter::OFXImporter(QObject *parent, const QVariantList &args) :
     KMyMoneyPlugin::ImporterPlugin(),
     d(new Private)
 {
-    Q_UNUSED(args)
-    const auto componentName = QLatin1String("ofximporter");
+    Q_INIT_RESOURCE(ofximporter);
+
     const auto rcFileName = QLatin1String("ofximporter.rc");
-    setComponentName(componentName, i18n("OFX Importer"));
 
-    if (MyMoneyUtils::isRunningAsAppImage()) {
-        const QString rcFilePath = QString("%1/../share/kxmlgui5/%2/%3").arg(QCoreApplication::applicationDirPath(), componentName, rcFileName);
-        setXMLFile(rcFilePath);
-
-        const QString localRcFilePath =
-            QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation).first() + QLatin1Char('/') + componentName + QLatin1Char('/') + rcFileName;
-        setLocalXMLFile(localRcFilePath);
-    } else {
-        setXMLFile(rcFileName);
-    }
+    setXMLFile(rcFileName);
 
     createActions();
 
@@ -166,10 +165,10 @@ void OFXImporter::slotImportFile()
     option->m_uniqueIdSource->setCurrentIndex(defaultIdSource());
 
     QUrl url = importInterface()->selectFile(i18n("OFX import file selection"),
-               QString(),
-               QStringLiteral("*.ofx *.qfx *.ofc|OFX files (*.ofx *.qfx *.ofc);;*|All files (*)"),
-               QFileDialog::ExistingFile,
-               widget);
+                                             QString(),
+                                             i18nc("Filter for QFileDialog", "OFX files (*.ofx *.qfx *.ofc);;All files (*)"),
+                                             QFileDialog::ExistingFile,
+                                             widget);
 
     d->m_preferName = static_cast<OFXImporter::Private::NamePreference>(option->m_preferName->currentIndex());
     d->m_uniqueIdSource = static_cast<UniqueTransactionIdSource>(option->m_uniqueIdSource->currentIndex());
@@ -192,14 +191,9 @@ void OFXImporter::slotImportFile()
     delete widget;
 }
 
-QString OFXImporter::formatName() const
+QStringList OFXImporter::formatMimeTypes() const
 {
-    return QStringLiteral("OFX");
-}
-
-QString OFXImporter::formatFilenameFilter() const
-{
-    return QStringLiteral("*.ofx *.qfx *.ofc");
+    return {"application/x-ofx", "application/vnd.intu.qfx", "application/x-ofc"};
 }
 
 
@@ -306,8 +300,15 @@ bool OFXImporter::import(const QString& filename)
     libofx_set_dtd_dir(ctx, dir.data());
 #endif
 
+#ifdef Q_OS_WIN
+    const auto dir = QFile::encodeName(QStringLiteral("%1/data/libofx/dtd/").arg(QCoreApplication::applicationDirPath()));
+    qDebug() << "Set DTD dir to" << dir;
+    libofx_set_dtd_dir(ctx, dir);
+#endif
+
+    qDebug() << "Start processing OFX data from" << filename_deep;
     libofx_proc_file(ctx, filename_deep, AUTODETECT);
-    qDebug("process data done");
+    qDebug() << "Processing OFX data done";
     libofx_free_context(ctx);
 
     if (d->m_valid) {
@@ -337,22 +338,23 @@ int OFXImporter::ofxTransactionCallback(struct OfxTransactionData data, void * p
 //   kDebug(2) << Q_FUNC_INFO;
 
     OFXImporter* pofx = reinterpret_cast<OFXImporter*>(pv);
+    OFXImporter::Private* d = pofx->d;
     MyMoneyStatement& s = pofx->back();
 
     MyMoneyStatement::Transaction t;
 
     if (data.date_posted_valid) {
         QDateTime dt;
-        dt.setTime_t(data.date_posted - pofx->d->m_timestampOffset * 60);
+        dt.setSecsSinceEpoch(data.date_posted - d->m_timestampOffset * 60);
         t.m_datePosted = dt.date();
     } else if (data.date_initiated_valid) {
         QDateTime dt;
-        dt.setTime_t(data.date_initiated - pofx->d->m_timestampOffset * 60);
+        dt.setSecsSinceEpoch(data.date_initiated - d->m_timestampOffset * 60);
         t.m_datePosted = dt.date();
     }
     if (t.m_datePosted.isValid()) {
         // verify the transaction date is one we want
-        if (t.m_datePosted < pofx->d->m_updateStartDate) {
+        if (t.m_datePosted < d->m_updateStartDate) {
             //kDebug(0) << "discarding transaction dated" << qPrintable(t.m_datePosted.toString(Qt::ISODate));
             return 0;
         }
@@ -435,14 +437,20 @@ int OFXImporter::ofxTransactionCallback(struct OfxTransactionData data, void * p
 
     t.m_shares = MyMoneyMoney();
     if (data.units_valid) {
-        t.m_shares = MyMoneyMoney(data.units, 100000).reduce();
+        auto shares = MyMoneyMoney(data.units, 1000000000).reduce();
+        const auto denominator = MyMoneyMoney(shares.toString().remove(QRegularExpression("^[+-]?[\\d]+/")));
+        t.m_shareDenominator = MyMoneyMoney::ONE;
+        while (t.m_shareDenominator < denominator) {
+            t.m_shareDenominator *= MyMoneyMoney(10, 1);
+        }
+        t.m_shares = shares;
     }
 
     t.m_amount = MyMoneyMoney();
     if (data.amount_valid) {
         t.m_amount = MyMoneyMoney(data.amount, 1000);
 
-        if (pofx->d->m_fixBuySellSignage) {
+        if (d->m_fixBuySellSignage) {
             if (t.m_eAction == eMyMoney::Transaction::Action::Buy
                     || t.m_eAction == eMyMoney::Transaction::Action::ReinvestDividend) {
                 t.m_amount = -t.m_amount.abs();
@@ -454,20 +462,20 @@ int OFXImporter::ofxTransactionCallback(struct OfxTransactionData data, void * p
             }
         }
 
-        if (pofx->d->m_invertAmount) {
+        if (d->m_invertAmount) {
             t.m_amount = -t.m_amount;
         }
     }
 
     if (data.check_number_valid) {
-        t.m_strNumber = QString::fromUtf8(data.check_number);
+        t.m_strNumber = d->sanitizedString(data.check_number);
     }
 
     unsigned long h;
     QString tmpString;
     // in case the unique transaction id source is yet unknown we
     // use the global preset
-    UniqueTransactionIdSource idSource = pofx->d->m_uniqueIdSource;
+    UniqueTransactionIdSource idSource = d->m_uniqueIdSource;
     if (idSource == UniqueIdUnknown) {
         idSource = defaultIdSource();
     }
@@ -475,19 +483,19 @@ int OFXImporter::ofxTransactionCallback(struct OfxTransactionData data, void * p
     default:
     case UniqueIdOfx:
         if (data.fi_id_valid) {
-            t.m_strBankID = QStringLiteral("ID ") + QString::fromUtf8(data.fi_id);
+            t.m_strBankID = QStringLiteral("ID ") + d->sanitizedString(data.fi_id);
         } else if (data.reference_number_valid) {
-            t.m_strBankID = QStringLiteral("REF ") + QString::fromUtf8(data.reference_number);
+            t.m_strBankID = QStringLiteral("REF ") + d->sanitizedString(data.reference_number);
         }
         break;
 
     case UniqueIdKMyMoney:
         if (data.payee_id_valid) {
-            tmpString = QString::fromUtf8(data.payee_id);
+            tmpString = d->sanitizedString(data.payee_id);
         } else if (data.name_valid) {
-            tmpString = QString::fromUtf8(data.name);
+            tmpString = d->sanitizedString(data.name);
         } else if (data.memo_valid) {
-            tmpString = QString::fromUtf8(data.memo);
+            tmpString = d->sanitizedString(data.memo);
         }
         h = MyMoneyTransaction::hash(tmpString.trimmed());
         if (data.memo_valid)
@@ -496,14 +504,13 @@ int OFXImporter::ofxTransactionCallback(struct OfxTransactionData data, void * p
         h = MyMoneyTransaction::hash(t.m_amount.toString(), h);
         // make hash value unique in case we don't have one already
 
-        QString hashBase;
-        hashBase.sprintf("%s-%07lx", qPrintable(t.m_datePosted.toString(Qt::ISODate)), h);
+        const auto hashBase(QStringLiteral("%1-%2").arg(t.m_datePosted.toString(Qt::ISODate)).arg(h, 7, 16, QLatin1Char('0')));
         int idx = 1;
         QString hash;
         for (;;) {
             hash = QString("%1-%2").arg(hashBase).arg(idx);
-            if (!pofx->d->m_hashes.contains(hash)) {
-                pofx->d->m_hashes += hash;
+            if (!d->m_hashes.contains(hash)) {
+                d->m_hashes.insert(hash);
                 break;
             }
             ++idx;
@@ -515,33 +522,33 @@ int OFXImporter::ofxTransactionCallback(struct OfxTransactionData data, void * p
     // Decide whether to use NAME, PAYEEID or MEMO to construct the payee
     bool validity[3] = {false, false, false};
     QStringList values;
-    switch (pofx->d->m_preferName) {
+    switch (d->m_preferName) {
     case OFXImporter::Private::PreferId:  // PAYEEID
     default:
         validity[0] = data.payee_id_valid;
         validity[1] = data.name_valid;
         validity[2] = data.memo_valid;
-        values += QString::fromUtf8(data.payee_id);
-        values += QString::fromUtf8(data.name);
-        values += QString::fromUtf8(data.memo);
+        values += d->sanitizedString(data.payee_id);
+        values += d->sanitizedString(data.name);
+        values += d->sanitizedString(data.memo);
         break;
 
     case OFXImporter::Private::PreferName:  // NAME
         validity[0] = data.name_valid;
         validity[1] = data.payee_id_valid;
         validity[2] = data.memo_valid;
-        values += QString::fromUtf8(data.name);
-        values += QString::fromUtf8(data.payee_id);
-        values += QString::fromUtf8(data.memo);
+        values += d->sanitizedString(data.name);
+        values += d->sanitizedString(data.payee_id);
+        values += d->sanitizedString(data.memo);
         break;
 
     case OFXImporter::Private::PreferMemo:  // MEMO
         validity[0] = data.memo_valid;
         validity[1] = data.payee_id_valid;
         validity[2] = data.name_valid;
-        values += QString::fromUtf8(data.memo);
-        values += QString::fromUtf8(data.payee_id);
-        values += QString::fromUtf8(data.name);
+        values += d->sanitizedString(data.memo);
+        values += d->sanitizedString(data.payee_id);
+        values += d->sanitizedString(data.name);
         break;
     }
 
@@ -551,8 +558,8 @@ int OFXImporter::ofxTransactionCallback(struct OfxTransactionData data, void * p
         validity[0] = data.payee_id_valid;
         validity[1] = data.name_valid;
         validity[2] = false;
-        values += QString::fromUtf8(data.payee_id);
-        values += QString::fromUtf8(data.name);
+        values += d->sanitizedString(data.payee_id);
+        values += d->sanitizedString(data.name);
     }
 
     for (int idx = 0; idx < 3; ++idx) {
@@ -563,9 +570,8 @@ int OFXImporter::ofxTransactionCallback(struct OfxTransactionData data, void * p
     }
 
     // extract memo field if we haven't used it as payee
-    if ((data.memo_valid)
-            && (pofx->d->m_preferName != OFXImporter::Private::PreferMemo)) {
-        t.m_strMemo = QString::fromUtf8(data.memo);
+    if ((data.memo_valid) && (d->m_preferName != OFXImporter::Private::PreferMemo)) {
+        t.m_strMemo = d->sanitizedString(data.memo);
     }
 
     // If the payee or memo fields are blank, set them to
@@ -582,12 +588,26 @@ int OFXImporter::ofxTransactionCallback(struct OfxTransactionData data, void * p
     if (data.security_data_valid) {
         struct OfxSecurityData* secdata = data.security_data_ptr;
 
+        if (secdata->unique_id_valid) {
+            t.m_strSecurityId = d->sanitizedString(secdata->unique_id);
+        }
         if (secdata->ticker_valid) {
-            t.m_strSymbol = QString::fromUtf8(secdata->ticker);
+            t.m_strSymbol = d->sanitizedString(secdata->ticker);
         }
 
         if (secdata->secname_valid) {
-            t.m_strSecurity = QString::fromUtf8(secdata->secname);
+            t.m_strSecurity = d->sanitizedString(secdata->secname);
+        }
+
+        // scan over securities and check if security is there
+        // and possibly adjust the smallestFraction setting
+        for (auto& security : s.m_listSecurities) {
+            if ((t.m_strSecurityId == security.m_strId) || (t.m_strSecurity == security.m_strName)) {
+                if (t.m_shareDenominator > security.m_smallestFraction) {
+                    security.m_smallestFraction = t.m_shareDenominator;
+                    break;
+                }
+            }
         }
     }
 
@@ -605,7 +625,7 @@ int OFXImporter::ofxTransactionCallback(struct OfxTransactionData data, void * p
         t.m_fees += MyMoneyMoney(data.commission, 1000).reduce();
     }
 
-    // In the case of investment transactions, the 'total' is supposed to the total amount
+    // In the case of investment transactions, the 'total' is supposed to be the total amount
     // of the transaction.  units * unitprice +/- commission.  Easy, right?  Sadly, it seems
     // some ofx creators do not follow this in all circumstances.  Therefore, we have to double-
     // check the total here and adjust it if it's wrong.
@@ -639,17 +659,19 @@ int OFXImporter::ofxStatementCallback(struct OfxStatementData data, void* pv)
 //   kDebug(2) << Q_FUNC_INFO;
 
     OFXImporter* pofx = reinterpret_cast<OFXImporter*>(pv);
+    OFXImporter::Private* d = pofx->d;
+
     MyMoneyStatement& s = pofx->back();
 
     pofx->setValid();
 
     if (data.currency_valid) {
-        s.m_strCurrency = QString::fromUtf8(data.currency);
+        s.m_strCurrency = d->sanitizedString(data.currency);
     }
     if (data.account_id_valid) {
         // only use the account_id if it is filled with non-blank data
         // see https://bugs.kde.org/show_bug.cgi?id=428156
-        const auto account_id = QString::fromUtf8(data.account_id).trimmed();
+        const auto account_id = d->sanitizedString(data.account_id).trimmed();
         if (!account_id.isEmpty()) {
             s.m_strAccountNumber = account_id;
         }
@@ -657,20 +679,20 @@ int OFXImporter::ofxStatementCallback(struct OfxStatementData data, void* pv)
 
     if (data.date_start_valid) {
         QDateTime dt;
-        dt.setTime_t(data.date_start - pofx->d->m_timestampOffset * 60);
+        dt.setSecsSinceEpoch(data.date_start - d->m_timestampOffset * 60);
         s.m_dateBegin = dt.date();
     }
 
     if (data.date_end_valid) {
         QDateTime dt;
-        dt.setTime_t(data.date_end - pofx->d->m_timestampOffset * 60);
+        dt.setSecsSinceEpoch(data.date_end - d->m_timestampOffset * 60);
         s.m_dateEnd = dt.date();
     }
 
     if (data.ledger_balance_valid && data.ledger_balance_date_valid) {
         s.m_closingBalance = MyMoneyMoney(data.ledger_balance);
         QDateTime dt;
-        dt.setTime_t(data.ledger_balance_date);
+        dt.setSecsSinceEpoch(data.ledger_balance_date);
         s.m_dateEnd = dt.date();
     }
 
@@ -684,16 +706,18 @@ int OFXImporter::ofxAccountCallback(struct OfxAccountData data, void * pv)
 //   kDebug(2) << Q_FUNC_INFO;
 
     OFXImporter* pofx = reinterpret_cast<OFXImporter*>(pv);
+    OFXImporter::Private* d = pofx->d;
+
     pofx->addnew();
     MyMoneyStatement& s = pofx->back();
 
     // Having any account at all makes an ofx statement valid
-    pofx->d->m_valid = true;
+    d->m_valid = true;
     // new account means new hashes
-    pofx->d->m_hashes.clear();
+    d->m_hashes.clear();
 
     if (data.account_id_valid) {
-        const auto account_name = QString::fromUtf8(data.account_name);
+        const auto account_name = d->sanitizedString(data.account_name);
         // in case libofx does not extract any value, it returns
         // one of the following fixed strings in this member.
         // See OfxAccountContainer::gen_account_id in
@@ -709,19 +733,19 @@ int OFXImporter::ofxAccountCallback(struct OfxAccountData data, void * pv)
 
         // only use the account_id if it is filled with non-blank data
         // see https://bugs.kde.org/show_bug.cgi?id=428156
-        const auto account_id = QString::fromUtf8(data.account_id).trimmed();
+        const auto account_id = d->sanitizedString(data.account_id).trimmed();
         if (!account_id.isEmpty()) {
             s.m_strAccountNumber = account_id;
         }
     }
     if (data.bank_id_valid) {
-        s.m_strRoutingNumber = QString::fromUtf8(data.bank_id);
+        s.m_strBankCode = d->sanitizedString(data.bank_id);
     }
     if (data.broker_id_valid) {
-        s.m_strRoutingNumber = QString::fromUtf8(data.broker_id);
+        s.m_strBankCode = d->sanitizedString(data.broker_id);
     }
     if (data.currency_valid) {
-        s.m_strCurrency = QString::fromUtf8(data.currency);
+        s.m_strCurrency = d->sanitizedString(data.currency);
     }
 
     if (data.account_type_valid) {
@@ -747,19 +771,22 @@ int OFXImporter::ofxAccountCallback(struct OfxAccountData data, void * pv)
         case OfxAccountData::OFX_INVESTMENT :
             s.m_eType = eMyMoney::Statement::Type::Investment;
             break;
+        case OfxAccountData::OFX_401K:
+            s.m_eType = eMyMoney::Statement::Type::Investment;
+            break;
         }
     }
 
     // ask KMyMoney for an account id
     // but only if we have any information to ask for
-    if (!s.m_strAccountNumber.isEmpty() || !s.m_strRoutingNumber.isEmpty()) {
-        s.m_accountId = pofx->account(QStringLiteral("kmmofx-acc-ref"), QString("%1-%2").arg(s.m_strRoutingNumber, s.m_strAccountNumber)).id();
+    if (!s.m_strAccountNumber.isEmpty() || !s.m_strBankCode.isEmpty()) {
+        s.m_accountId = pofx->account(QStringLiteral("kmmofx-acc-ref"), QString("%1-%2").arg(s.m_strBankCode, s.m_strAccountNumber)).id();
     }
 
     // copy over the securities
-    s.m_listSecurities = pofx->d->m_securitylist;
+    s.m_listSecurities = d->m_securitylist;
 
-//   kDebug(2) << Q_FUNC_INFO << " return 0";
+    //   kDebug(2) << Q_FUNC_INFO << " return 0";
 
     return 0;
 }
@@ -769,19 +796,20 @@ int OFXImporter::ofxSecurityCallback(struct OfxSecurityData data, void* pv)
     //   kDebug(2) << Q_FUNC_INFO;
 
     OFXImporter* pofx = reinterpret_cast<OFXImporter*>(pv);
+    OFXImporter::Private* d = pofx->d;
     MyMoneyStatement::Security sec;
 
     if (data.unique_id_valid) {
-        sec.m_strId = QString::fromUtf8(data.unique_id);
+        sec.m_strId = d->sanitizedString(data.unique_id);
     }
     if (data.secname_valid) {
-        sec.m_strName = QString::fromUtf8(data.secname);
+        sec.m_strName = d->sanitizedString(data.secname);
     }
     if (data.ticker_valid) {
-        sec.m_strSymbol = QString::fromUtf8(data.ticker);
+        sec.m_strSymbol = d->sanitizedString(data.ticker);
     }
 
-    pofx->d->m_securitylist += sec;
+    d->m_securitylist += sec;
 
     return 0;
 }
@@ -791,6 +819,7 @@ int OFXImporter::ofxStatusCallback(struct OfxStatusData data, void * pv)
 //   kDebug(2) << Q_FUNC_INFO;
 
     OFXImporter* pofx = reinterpret_cast<OFXImporter*>(pv);
+    OFXImporter::Private* d = pofx->d;
     QString message;
 
     // if we got this far, we know we were able to parse the file.
@@ -799,13 +828,13 @@ int OFXImporter::ofxStatusCallback(struct OfxStatusData data, void * pv)
     pofx->d->m_fatalerror = i18n("No accounts found.");
 
     if (data.ofx_element_name_valid)
-        message.prepend(QString("%1: ").arg(QString::fromUtf8(data.ofx_element_name)));
+        message.prepend(QString("%1: ").arg(d->sanitizedString(data.ofx_element_name)));
 
     if (data.code_valid)
-        message += QString("%1 (Code %2): %3").arg(QString::fromUtf8(data.name)).arg(data.code).arg(QString::fromUtf8(data.description));
+        message += QString("%1 (Code %2): %3").arg(d->sanitizedString(data.name)).arg(data.code).arg(d->sanitizedString(data.description));
 
     if (data.server_message_valid)
-        message += QString(" (%1)").arg(QString::fromUtf8(data.server_message));
+        message += QString(" (%1)").arg(d->sanitizedString(data.server_message));
 
     if (data.severity_valid) {
         switch (data.severity) {
@@ -864,22 +893,12 @@ MyMoneyKeyValueContainer OFXImporter::onlineBankingSettings(const MyMoneyKeyValu
         kvp.deletePair(QStringLiteral("kmmofx-headerVersion"));
         kvp.deletePair(QStringLiteral("password"));
 
-        d->m_wallet = openSynchronousWallet();
-        if (d->m_wallet && (d->m_wallet->hasFolder(KWallet::Wallet::PasswordFolder()) ||
-                            d->m_wallet->createFolder(KWallet::Wallet::PasswordFolder())) &&
-                d->m_wallet->setFolder(KWallet::Wallet::PasswordFolder())) {
-            QString key = OFX_PASSWORD_KEY(kvp.value(QStringLiteral("url")), kvp.value(QStringLiteral("uniqueId")));
-            if (d->m_statusDlg->m_storePassword->isChecked()) {
-                d->m_wallet->writePassword(key, d->m_statusDlg->m_password->text());
-            } else {
-                if (d->m_wallet->hasEntry(key)) {
-                    d->m_wallet->removeEntry(key);
-                }
-            }
+        const QString key = OFX_PASSWORD_KEY(kvp.value(QStringLiteral("url")), kvp.value(QStringLiteral("uniqueId")));
+        auto keyChain = new KMMKeychain();
+        if (d->m_statusDlg->m_storePassword->isChecked()) {
+            keyChain->writeKey(key, d->m_statusDlg->m_password->password());
         } else {
-            if (d->m_statusDlg->m_storePassword->isChecked()) {
-                kvp.setValue(QStringLiteral("password"), d->m_statusDlg->m_password->text());
-            }
+            keyChain->deleteKey(key);
         }
 
         if (!d->m_statusDlg->appId().isEmpty())
@@ -999,7 +1018,7 @@ bool OFXImporter::updateAccount(const MyMoneyAccount& acc, bool moreAccounts)
 
 void OFXImporter::slotImportFile(const QString& url)
 {
-    qDebug("OfxImporterPlugin::slotImportFile");
+    qDebug() << "OfxImporterPlugin::slotImportFile";
     if (!import(url)) {
         KMessageBox::error(0, QString("<qt>%1</qt>").arg(i18n("<p>Unable to import <b>'%1'</b> using the OFX importer plugin.  The plugin returned the following error:</p><p>%2</p>", url, lastError())), i18n("Importing error"));
     }
@@ -1025,7 +1044,7 @@ bool OFXImporter::storeStatements(const QList<MyMoneyStatement> &statements)
         abort = true;
     }*/
 
-    qDebug("OfxImporterPlugin::storeStatements() with %d statements called", statements.count());
+    qDebug("OfxImporterPlugin::storeStatements() with " QLIST_COUNT_FORMAT " statements called", statements.count());
 
     for (const auto& statement : statements) {
         if (abort)
@@ -1081,6 +1100,6 @@ const QStringList& OFXImporter::errors() const
     return d->m_errors;
 }
 
-K_PLUGIN_FACTORY_WITH_JSON(OFXImporterFactory, "ofximporter.json", registerPlugin<OFXImporter>();)
+K_PLUGIN_CLASS_WITH_JSON(OFXImporter, "ofximporter.json")
 
 #include "ofximporter.moc"
