@@ -11,18 +11,17 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
-#include <QDateTime>
-#include <QEventLoop>
-#include <QFileInfo>
 #include <QApplication>
-#include <QRegExp>
-#include <QDir>
-#include <QFile>
-#include <QTextStream>
-#include <QDomDocument>
+#include <QDateTime>
 #include <QDebug>
+#include <QDir>
+#include <QDomDocument>
+#include <QEventLoop>
+#include <QFile>
+#include <QFileInfo>
 #include <QRegularExpression>
 #include <QRegularExpressionMatch>
+#include <QTextStream>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
@@ -49,9 +48,7 @@ namespace OfxPartner
 bool post(const QString& request, const QMap<QString, QString>& attr, const QUrl &url, const QUrl& filename);
 bool get(const QString& request, const QMap<QString, QString>& attr, const QUrl &url, const QUrl& filename);
 
-const QString kBankFilename = "ofx-bank-index.xml";
-const QString kCcFilename = "ofx-cc-index.xml";
-const QString kInvFilename = "ofx-inv-index.xml";
+Q_GLOBAL_STATIC_WITH_ARGS(QString, kBankFilename, ("ofx-bank-index.xml"));
 
 #define VER "9"
 
@@ -77,7 +74,7 @@ void ValidateIndexCache()
 
     QMap<QString, QString> attr;
 
-    fname = QUrl("file://" + directory + kBankFilename);
+    fname = QUrl(QStringLiteral("file://") + directory + *kBankFilename);
     QDir dir;
     dir.mkpath(directory);
 
@@ -91,7 +88,9 @@ static void ParseFile(QMap<QString, QString>& result, const QString& fileName, c
     QFile f(fileName);
     if (f.open(QIODevice::ReadOnly)) {
         QTextStream stream(&f);
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
         stream.setCodec("UTF-8");
+#endif
         QString msg;
         int errl, errc;
         QDomDocument doc;
@@ -123,7 +122,7 @@ QStringList BankNames()
     // Make sure the index files are up to date
     ValidateIndexCache();
 
-    ParseFile(result, directory + kBankFilename, QString());
+    ParseFile(result, directory + *kBankFilename, QString());
 
     // Add Innovision
     result["Innovision"].clear();
@@ -135,7 +134,7 @@ QStringList FipidForBank(const QString& bank)
 {
     QMap<QString, QString> result;
 
-    ParseFile(result, directory + kBankFilename, bank);
+    ParseFile(result, directory + *kBankFilename, bank);
 
     // the fipid for Innovision is 1.
     if (bank == "Innovision")
@@ -147,17 +146,18 @@ QStringList FipidForBank(const QString& bank)
 QString extractNodeText(QDomElement& node, const QString& name)
 {
     QString res;
-    QRegExp exp("([^/]+)/?([^/].*)?");
-    if (exp.indexIn(name) != -1) {
-        QDomNodeList olist = node.elementsByTagName(exp.cap(1));
+    static const QRegularExpression exp(QLatin1String("([^/]+)/?([^/].*)?"));
+    const auto match(exp.match(name));
+    if (match.hasMatch()) {
+        QDomNodeList olist = node.elementsByTagName(match.captured(1));
         if (olist.count()) {
             QDomNode onode = olist.item(0);
             if (onode.isElement()) {
                 QDomElement elo = onode.toElement();
-                if (exp.cap(2).isEmpty()) {
+                if (match.captured(2).isEmpty()) {
                     res = elo.text();
                 } else {
-                    res = extractNodeText(elo, exp.cap(2));
+                    res = extractNodeText(elo, match.captured(2));
                 }
             }
         }
@@ -168,17 +168,18 @@ QString extractNodeText(QDomElement& node, const QString& name)
 QString extractNodeText(QDomDocument& doc, const QString& name)
 {
     QString res;
-    QRegExp exp("([^/]+)/?([^/].*)?");
-    if (exp.indexIn(name) != -1) {
-        QDomNodeList olist = doc.elementsByTagName(exp.cap(1));
+    static const QRegularExpression exp(QLatin1String("([^/]+)/?([^/].*)?"));
+    const auto match(exp.match(name));
+    if (match.hasMatch()) {
+        QDomNodeList olist = doc.elementsByTagName(match.captured(1));
         if (olist.count()) {
             QDomNode onode = olist.item(0);
             if (onode.isElement()) {
                 QDomElement elo = onode.toElement();
-                if (exp.cap(2).isEmpty()) {
+                if (match.captured(2).isEmpty()) {
                     res = elo.text();
                 } else {
-                    res = extractNodeText(elo, exp.cap(2));
+                    res = extractNodeText(elo, match.captured(2));
                 }
             }
         }
@@ -210,7 +211,7 @@ OfxHomeServiceInfo ServiceInfo(const QString& fipid)
 
     QMap<QString, QString> attr;
 
-    QUrl guidFile(QString("file://%1fipid-%2.xml").arg(directory).arg(fipid));
+    QUrl guidFile(QString("file://%1fipid-%2.xml").arg(directory, fipid));
 
     QFileInfo i(guidFile.toLocalFile());
 
@@ -220,7 +221,9 @@ OfxHomeServiceInfo ServiceInfo(const QString& fipid)
     QFile f(guidFile.toLocalFile());
     if (f.open(QIODevice::ReadOnly)) {
         QTextStream stream(&f);
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
         stream.setCodec("UTF-8");
+#endif
         QString msg;
         int errl, errc;
         QDomDocument doc;
@@ -280,7 +283,12 @@ public:
     QFile  m_fpTrace;
 };
 
-OfxHttpRequest::OfxHttpRequest(const QString& type, const QUrl &url, const QByteArray &postData, const QMap<QString, QString>& metaData, const QUrl& dst, bool showProgressInfo)
+OfxHttpRequest::OfxHttpRequest(const QString& method,
+                               const QUrl& url,
+                               const QByteArray& postData,
+                               const QMap<QString, QString>& metaData,
+                               const QUrl& dst,
+                               bool showProgressInfo)
     : d(new Private)
     , m_dst(dst.toLocalFile())
     , m_error(-1)
@@ -318,7 +326,7 @@ OfxHttpRequest::OfxHttpRequest(const QString& type, const QUrl &url, const QByte
         jobFlags = KIO::HideProgressInfo | KIO::Overwrite;
 
     KIO::Job* job;
-    if(type.toLower() == QStringLiteral("get")) {
+    if (method.toLower() == QStringLiteral("get")) {
         job = m_getJob = KIO::copy(url, QUrl(QString("file://%1").arg(m_dst)), jobFlags);
     } else {
         job = m_postJob = KIO::http_post(url, postData, jobFlags);
@@ -337,7 +345,7 @@ OfxHttpRequest::OfxHttpRequest(const QString& type, const QUrl &url, const QByte
         ts << "request:\n" << QString(postData) << "\n" << "response:\n";
     }
 
-    connect(job, SIGNAL(result(KJob*)), this, SLOT(slotOfxFinished(KJob*)));
+    connect(job, &KIO::Job::result, this, &OfxHttpRequest::slotOfxFinished);
 
     job->start();
 
@@ -389,7 +397,7 @@ void OfxHttpRequest::slotOfxFinished(KJob* /* e */)
         if (m_error) {
             m_postJob->uiDelegate()->showErrorMessage();
             QFile::remove(m_dst);
-
+#if QT_VERSION < QT_VERSION_CHECK(6, 3, 0)
         } else if (m_postJob->isErrorPage()) {
             QString details;
             QFile f(m_dst);
@@ -400,10 +408,10 @@ void OfxHttpRequest::slotOfxFinished(KJob* /* e */)
                 }
                 f.close();
             }
-            KMessageBox::detailedSorry(0, i18n("The HTTP request failed."), details, i18nc("The HTTP request failed", "Failed"));
+            KMessageBox::detailedError(0, i18n("The HTTP request failed."), details, i18nc("The HTTP request failed", "Failed"));
             QFile::remove(m_dst);
+#endif
         }
-
     } else if(m_getJob) {
         m_error = m_getJob->error();
         if (m_error) {
