@@ -31,19 +31,17 @@
 // ----------------------------------------------------------------------------
 // Project Headers
 
-#include "mymoneycategory.h"
-#include "mymoneyfile.h"
-#include "mymoneyaccount.h"
-#include "mymoneytransaction.h"
-#include "mymoneytransactionfilter.h"
+#include "accountsmodel.h"
+#include "icons.h"
 #include "kmymoneyaccountcombo.h"
 #include "kmymoneyutils.h"
-#include "models.h"
-#include "accountsmodel.h"
-#include <icons/icons.h>
+#include "mymoneyaccount.h"
+#include "mymoneycategory.h"
 #include "mymoneyenums.h"
-#include "modelenums.h"
-#include "../config/mymoneyqifprofile.h"
+#include "mymoneyfile.h"
+#include "mymoneyqifprofile.h"
+#include "mymoneytransaction.h"
+#include "mymoneytransactionfilter.h"
 
 using namespace Icons;
 
@@ -77,13 +75,25 @@ KExportDlg::KExportDlg(QWidget *parent)
     connect(m_qbuttonCancel, &QAbstractButton::clicked, this, &QDialog::reject);
 
     // connect the change signals to the check slot and perform initial check
-    connect(m_qlineeditFile, SIGNAL(editingFinished()), this, SLOT(checkData()));
-    connect(m_qcheckboxAccount, SIGNAL(toggled(bool)), this, SLOT(checkData()));
-    connect(m_qcheckboxCategories, SIGNAL(toggled(bool)), this, SLOT(checkData()));
-    connect(m_accountComboBox, SIGNAL(accountSelected(QString)), this, SLOT(checkData(QString)));
-    connect(m_profileComboBox, SIGNAL(activated(int)), this, SLOT(checkData()));
-    connect(m_kmymoneydateStart, SIGNAL(dateChanged(QDate)), this, SLOT(checkData()));
-    connect(m_kmymoneydateEnd, SIGNAL(dateChanged(QDate)), this, SLOT(checkData()));
+    connect(m_qlineeditFile, &QLineEdit::editingFinished, this, [&]() {
+        checkData();
+    });
+    connect(m_qcheckboxAccount, &QCheckBox::toggled, this, [&]() {
+        checkData();
+    });
+    connect(m_qcheckboxCategories, &QCheckBox::toggled, this, [&]() {
+        checkData();
+    });
+    connect(m_accountComboBox, &KMyMoneyAccountCombo::accountSelected, this, &KExportDlg::checkData);
+    connect(m_profileComboBox, QOverload<int>::of(&KComboBox::activated), this, [&]() {
+        checkData();
+    });
+    connect(m_kmymoneydateStart, &KMyMoneyDateEdit::dateChanged, this, [&]() {
+        checkData();
+    });
+    connect(m_kmymoneydateEnd, &KMyMoneyDateEdit::dateChanged, this, [&]() {
+        checkData();
+    });
 
     checkData(QString());
 }
@@ -109,26 +119,26 @@ void KExportDlg::loadProfiles(const bool selectLast)
 
     QStringList list;
     KSharedConfigPtr config = KSharedConfig::openConfig();
-    KConfigGroup grp = config->group("Profiles");
+    KConfigGroup grp = config->group(QLatin1String("Profiles"));
 
-    list = grp.readEntry("profiles", QStringList());
+    list = grp.readEntry(QLatin1String("profiles"), QStringList());
     list.sort();
     if (list.isEmpty()) {
         // in case the list is empty, we need to create the default profile
         MyMoneyQifProfile defaultProfile;
         defaultProfile.setProfileDescription(i18n("The default QIF profile"));
-        defaultProfile.setProfileName("Profile-Default");
+        defaultProfile.setProfileName(QLatin1String("Default"));
 
-        list += "Default";
-        grp.writeEntry("profiles", list);
+        list += QLatin1String("Default");
+        grp.writeEntry(QLatin1String("profiles"), list);
 
         defaultProfile.saveProfile();
     }
     m_profileComboBox->insertItems(0, list);
 
     if (selectLast == true) {
-        grp = config->group("Last Use Settings");
-        current = grp.readEntry("KExportDlg_LastProfile");
+        grp = config->group(QLatin1String("Last Use Settings"));
+        current = grp.readEntry(QLatin1String("KExportDlg_LastProfile"));
     }
 
     m_profileComboBox->setCurrentItem(0);
@@ -163,8 +173,8 @@ void KExportDlg::writeConfig()
     grp.writeEntry("KExportDlg_LastFile", m_qlineeditFile->text());
     grp.writeEntry("KExportDlg_AccountOpt", m_qcheckboxAccount->isChecked());
     grp.writeEntry("KExportDlg_CatOpt", m_qcheckboxCategories->isChecked());
-    grp.writeEntry("KExportDlg_StartDate", QDateTime(m_kmymoneydateStart->date()));
-    grp.writeEntry("KExportDlg_EndDate", QDateTime(m_kmymoneydateEnd->date()));
+    grp.writeEntry("KExportDlg_StartDate", m_kmymoneydateStart->date().startOfDay());
+    grp.writeEntry("KExportDlg_EndDate", m_kmymoneydateEnd->date().startOfDay());
     grp.writeEntry("KExportDlg_LastProfile", m_profileComboBox->currentText());
     kconfig->sync();
 }
@@ -186,15 +196,16 @@ void KExportDlg::checkData(const QString& accountId)
         account = file->account(accountId);
         if (m_lastAccount != accountId) {
             MyMoneyTransactionFilter filter(accountId);
-            QList<MyMoneyTransaction> list = file->transactionList(filter);
+            QList<MyMoneyTransaction> list;
+            file->transactionList(list, filter);
             QList<MyMoneyTransaction>::Iterator it;
 
             if (!list.isEmpty()) {
                 it = list.begin();
-                m_kmymoneydateStart->loadDate((*it).postDate());
+                m_kmymoneydateStart->setDate((*it).postDate());
                 it = list.end();
                 --it;
-                m_kmymoneydateEnd->loadDate((*it).postDate());
+                m_kmymoneydateEnd->setDate((*it).postDate());
             }
             m_lastAccount = accountId;
             m_accountComboBox->setSelected(account.id());
@@ -215,10 +226,8 @@ void KExportDlg::loadAccounts()
 {
     auto filterProxyModel = new AccountNamesFilterProxyModel(this);
     filterProxyModel->addAccountGroup(QVector<eMyMoney::Account::Type> {eMyMoney::Account::Type::Asset, eMyMoney::Account::Type::Liability});
-    auto const model = Models::instance()->accountsModel();
-    filterProxyModel->setSourceColumns(model->getColumns());
-    filterProxyModel->setSourceModel(model);
-    filterProxyModel->sort((int)eAccountsModel::Column::Account);
+    filterProxyModel->setSourceModel(MyMoneyFile::instance()->accountsModel());
+    filterProxyModel->sort(AccountsModel::Column::AccountName);
     m_accountComboBox->setModel(filterProxyModel);
 }
 
