@@ -37,8 +37,8 @@
 #include "mymoneyfile.h"
 #include "mymoneyaccount.h"
 
-#include "../config/mymoneyqifprofile.h"
-#include <icons/icons.h>
+#include "icons.h"
+#include "mymoneyqifprofile.h"
 
 using namespace Icons;
 
@@ -89,7 +89,7 @@ void KImportDlg::slotBrowse()
     // determine what the browse prefix should be from the current profile
 
     MyMoneyQifProfile tmpprofile;
-    tmpprofile.loadProfile("Profile-" + profile());
+    tmpprofile.loadProfile(profile());
     QUrl file = QFileDialog::getOpenFileUrl(this,
                                             i18n("Import File..."),
                                             QUrl::fromLocalFile(m_qlineeditFile->text()),
@@ -129,9 +129,8 @@ void KImportDlg::slotFileTextChanged(const QString& text)
 {
     bool fileExists = false;
     if (file().isValid()) {
-#if KIO_VERSION < QT_VERSION_CHECK(5, 70, 0)
-        Q_CONSTEXPR short int detailLevel = 0; // Lowest level: file/dir/symlink/none
-        KIO::StatJob* statjob = KIO::stat(file(), KIO::StatJob::SourceSide, detailLevel);
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+        auto statjob = KIO::stat(file(), KIO::StatJob::SourceSide, KIO::StatNoDetails);
 #else
         auto statjob = KIO::statDetails(file(), KIO::StatJob::SourceSide, KIO::StatNoDetails);
 #endif
@@ -162,26 +161,26 @@ void KImportDlg::loadProfiles(const bool selectLast)
 
     QStringList list;
     KSharedConfigPtr config = KSharedConfig::openConfig();
-    KConfigGroup grp = config->group("Profiles");
+    KConfigGroup grp = config->group(QLatin1String("Profiles"));
 
-    list = grp.readEntry("profiles", QStringList());
+    list = grp.readEntry(QLatin1String("profiles"), QStringList());
     list.sort();
     if (list.isEmpty()) {
         // in case the list is empty, we need to create the default profile
         MyMoneyQifProfile defaultProfile;
         defaultProfile.setProfileDescription(i18n("The default QIF profile"));
-        defaultProfile.setProfileName("Profile-Default");
+        defaultProfile.setProfileName(QLatin1String("Default"));
 
-        list += "Default";
-        grp.writeEntry("profiles", list);
+        list += QLatin1String("Default");
+        grp.writeEntry(QLatin1String("profiles"), list);
 
         defaultProfile.saveProfile();
     }
     m_profileComboBox->addItems(list);
 
     if (selectLast == true) {
-        config->group("Last Use Settings");
-        current = grp.readEntry("KImportDlg_LastProfile");
+        config->group(QLatin1String("Last Use Settings"));
+        current = grp.readEntry(QLatin1String("KImportDlg_LastProfile"));
     }
 
     int index = m_profileComboBox->findText(current, Qt::MatchExactly);
@@ -199,9 +198,9 @@ void KImportDlg::addCategories(QStringList& strList, const QString& id, const QS
     MyMoneyAccount account = file->account(id);
 
     QStringList accList = account.accountList();
-    QStringList::ConstIterator it_a;
+    QStringList::const_iterator it_a;
 
-    for (it_a = accList.constBegin(); it_a != accList.constEnd(); ++it_a) {
+    for (it_a = accList.cbegin(); it_a != accList.cend(); ++it_a) {
         account = file->account(*it_a);
         strList << leadIn + account.name();
         addCategories(strList, *it_a, leadIn + account.name() + MyMoneyFile::AccountSeparator);
