@@ -14,37 +14,41 @@
 // ----------------------------------------------------------------------------
 // QT Headers
 
-#include <QFile>
-#include <QStringList>
-#include <QTimer>
-#include <QRegExp>
 #include <QBuffer>
 #include <QByteArray>
-#include <QInputDialog>
 #include <QDir>
+#include <QFile>
+#include <QInputDialog>
+#include <QRegularExpression>
+#include <QStringList>
+#include <QTimer>
 
 // ----------------------------------------------------------------------------
 // KDE Headers
 
-#include <kmessagebox.h>
-#include <kconfig.h>
-#include <KConfigGroup>
-#include <KSharedConfig>
-#include <KLocalizedString>
-#include "kjobwidgets.h"
 #include "kio/job.h"
+#include "kjobwidgets.h"
+#include <KConfigGroup>
+#include <KIO/FileCopyJob>
+#include <KLocalizedString>
+#include <KSharedConfig>
+#include <kconfig.h>
+#include <kmessagebox.h>
 
 // ----------------------------------------------------------------------------
 // Project Headers
 
+#include "config-kmymoney.h"
+#include "kmymoneysettings.h"
+#include "mymoneyexception.h"
 #include "mymoneyfile.h"
 #include "mymoneysecurity.h"
 #include "mymoneysplit.h"
-#include "mymoneyexception.h"
-#include "kmymoneysettings.h"
 
 #include "mymoneystatement.h"
 
+#include "kmmyesno.h"
+
 // define this to debug the code. Using external filters
 // while debugging did not work too good for me, so I added
 // this code.
@@ -123,8 +127,11 @@ void MyMoneyQifReader::Private::finishStatement()
     // in case we have collected any data in the statement, we keep it
     if ((st.m_listTransactions.count() + st.m_listPrices.count() + st.m_listSecurities.count()) > 0) {
         statements += st;
-        qDebug("Statement with %d transactions, %d prices and %d securities added to the statement list",
-               st.m_listTransactions.count(), st.m_listPrices.count(), st.m_listSecurities.count());
+        qDebug("Statement with " QLIST_COUNT_FORMAT " transactions, " QLIST_COUNT_FORMAT " prices and " QLIST_COUNT_FORMAT
+               " securities added to the statement list",
+               st.m_listTransactions.count(),
+               st.m_listPrices.count(),
+               st.m_listSecurities.count());
     }
     eMyMoney::Statement::Type type = st.m_eType; //stash type and...
     // start with a fresh statement
@@ -211,14 +218,14 @@ bool MyMoneyQifReader::Private::isTransfer(QString& tmp, const QString& leftDeli
     // S[Mehrwertsteuer]/_VATCode_N_I          (The '/' is the Quicken class symbol)
     //
     // so extracting is a bit more complex and we use a regexp for it
-    QRegExp exp(QString("\\%1(.*)\\%2(.*)").arg(leftDelim, rightDelim));
-
-    bool rc;
-    if ((rc = (exp.indexIn(tmp) != -1)) == true) {
-        tmp = exp.cap(1) + exp.cap(2);
+    const QRegularExpression transferExp(QStringLiteral("\\%1(.*)\\%2(.*)").arg(leftDelim, rightDelim));
+    const auto transfer(transferExp.match(tmp));
+    if (transfer.hasMatch()) {
+        tmp = transfer.captured(1) + transfer.captured(2);
         tmp = tmp.trimmed();
+        return true;
     }
-    return rc;
+    return false;
 }
 
 eMyMoney::Split::State MyMoneyQifReader::Private::reconcileState(const QString& state) const
@@ -254,10 +261,10 @@ MyMoneyQifReader::MyMoneyQifReader() :
     m_warnedSecurity = false;
     m_warnedPrice = false;
 
-    connect(&m_filter, SIGNAL(bytesWritten(qint64)), this, SLOT(slotSendDataToFilter()));
-    connect(&m_filter, SIGNAL(readyReadStandardOutput()), this, SLOT(slotReceivedDataFromFilter()));
-    connect(&m_filter, SIGNAL(finished(int,QProcess::ExitStatus)), this, SLOT(slotImportFinished()));
-    connect(&m_filter, SIGNAL(readyReadStandardError()), this, SLOT(slotReceivedErrorFromFilter()));
+    connect(&m_filter, &QProcess::bytesWritten, this, &MyMoneyQifReader::slotSendDataToFilter);
+    connect(&m_filter, &QProcess::readyReadStandardOutput, this, &MyMoneyQifReader::slotReceivedDataFromFilter);
+    connect(&m_filter, QOverload<int,QProcess::ExitStatus>::of(&QProcess::finished), this, &MyMoneyQifReader::slotImportFinished);
+    connect(&m_filter, &QProcess::readyReadStandardError, this, &MyMoneyQifReader::slotReceivedErrorFromFilter);
 }
 
 MyMoneyQifReader::~MyMoneyQifReader()
@@ -278,7 +285,7 @@ void MyMoneyQifReader::setURL(const QUrl &url)
 
 void MyMoneyQifReader::setProfile(const QString& profile)
 {
-    m_qifProfile.loadProfile("Profile-" + profile);
+    m_qifProfile.loadProfile(profile);
 }
 
 void MyMoneyQifReader::slotSendDataToFilter()
@@ -397,7 +404,7 @@ void MyMoneyQifReader::slotProcessData()
     qDebug("%d lines processed", m_linenumber);
     signalProgress(-1, -1);
 
-    emit statementsReady(d->statements);
+    Q_EMIT statementsReady(d->statements);
 }
 
 bool MyMoneyQifReader::startImport()
@@ -468,7 +475,7 @@ bool MyMoneyQifReader::startImport()
             arguments << "-";
 #endif
         } else {
-            arguments << m_qifProfile.filterScriptImport().split(' ', QString::KeepEmptyParts);
+            arguments << m_qifProfile.filterScriptImport().split(' ');
             program = arguments.takeFirst();
         }
         m_entryType = EntryUnknown;
@@ -479,7 +486,7 @@ bool MyMoneyQifReader::startImport()
             signalProgress(0, m_file->size(), i18n("Reading QIF..."));
             slotSendDataToFilter();
             rc = true;
-//      emit statementsReady(d->statements);
+//      Q_EMIT statementsReady(d->statements);
         } else {
             KMessageBox::detailedError(0, i18n("Error while running the filter '%1'.", m_filter.program()),
                                        m_filter.errorString(),
@@ -646,10 +653,10 @@ void MyMoneyQifReader::processQifEntry()
 
 const QString MyMoneyQifReader::extractLine(const QChar& id, int cnt)
 {
-    QStringList::ConstIterator it;
+    QStringList::const_iterator it;
 
     m_extractedLine = -1;
-    for (it = m_qifEntry.constBegin(); it != m_qifEntry.constEnd(); ++it) {
+    for (it = m_qifEntry.cbegin(); it != m_qifEntry.cend(); ++it) {
         ++m_extractedLine;
         if ((*it)[0] == id) {
             if (cnt-- == 1) {
@@ -663,34 +670,35 @@ const QString MyMoneyQifReader::extractLine(const QChar& id, int cnt)
 
 bool MyMoneyQifReader::extractSplits(QList<qSplit>& listqSplits) const
 {
-//     *** With apologies to QString MyMoneyQifReader::extractLine ***
-
-    QStringList::ConstIterator it;
+    QStringList::const_iterator it;
     bool ret = false;
     bool memoPresent = false;
     int neededCount = 0;
     qSplit q;
 
-    for (it = m_qifEntry.constBegin(); it != m_qifEntry.constEnd(); ++it) {
+    for (it = m_qifEntry.cbegin(); it != m_qifEntry.cend(); ++it) {
         if (((*it)[0] == 'S') || ((*it)[0] == '$') || ((*it)[0] == 'E')) {
-            memoPresent = false;  //                      in case no memo line in this split
+            memoPresent = false; // in case no memo line in this split
             if ((*it)[0] == 'E') {
-                q.m_strMemo = (*it).mid(1);  //             'E' = Memo
+                q.m_strMemo = (*it).mid(1); // 'E' = Memo
                 d->fixMultiLineMemo(q.m_strMemo);
-                memoPresent = true;  //                     This transaction contains memo
+                memoPresent = true; // This transaction contains memo
+
             } else if ((*it)[0] == 'S') {
-                q.m_strCategoryName = (*it).mid(1);  //   'S' = CategoryName
+                q.m_strCategoryName = (*it).mid(1); // 'S' = CategoryName
                 neededCount ++;
+
             } else if ((*it)[0] == '$') {
-                q.m_amount = (*it).mid(1);  //            '$' = Amount
+                q.m_amount = (*it).mid(1); // '$' = Amount
                 neededCount ++;
             }
-            if (neededCount > 1) {  //                         CategoryName & Amount essential
-                listqSplits += q;  //                       Add valid split
-                if (!memoPresent) {  //                     If no memo, clear previous
+
+            if (neededCount > 1) { // CategoryName & Amount essential
+                listqSplits += q; // Add valid split
+                if (!memoPresent) { // If no memo, clear previous
                     q.m_strMemo.clear();
                 }
-                q = qSplit();  //                               Start new split
+                q = qSplit(); // Start new split
                 neededCount = 0;
                 ret = true;
             }
@@ -729,14 +737,14 @@ void MyMoneyQifReader::processMSAccountEntry(const eMyMoney::Account::Type accou
 
                 if (split.shares() != balance) {
                     const MyMoneySecurity& sec = file->security(m_account.currencyId());
-                    if (KMessageBox::questionYesNo(
+                    if (KMessageBox::questionTwoActions(
                                 KMyMoneyUtils::mainWindow(),
                                 i18n("The %1 account currently has an opening balance of %2. This QIF file reports an opening balance of %3. Would you like to overwrite the current balance with the one from the QIF file?", m_account.name(), split.shares().formatMoney(m_account, sec), balance.formatMoney(m_account, sec)),
                                 i18n("Overwrite opening balance"),
-                                KStandardGuiItem::yes(),
-                                KStandardGuiItem::no(),
+                                KMMYesNo::yes(),
+                                KMMYesNo::no(),
                                 "OverwriteOpeningBalance")
-                            == KMessageBox::Yes) {
+                            == KMessageBox::PrimaryAction) {
                         file->removeTransaction(openingtx);
                         m_account.setOpeningDate(date);
                         file->createOpeningBalanceTransaction(m_account, balance);
@@ -868,7 +876,7 @@ MyMoneyAccount MyMoneyQifReader::findAccount(const MyMoneyAccount& acc, const My
             parents << parent;
         }
         QList<MyMoneyAccount>::const_iterator it_p;
-        for (it_p = parents.constBegin(); it_p != parents.constEnd(); ++it_p) {
+        for (it_p = parents.cbegin(); it_p != parents.cend(); ++it_p) {
             MyMoneyAccount parentAccount = *it_p;
             // search by name (allow hierarchy)
             int pos;
@@ -942,7 +950,7 @@ void MyMoneyQifReader::createOpeningBalance(eMyMoney::Account::Type accType)
         if (name.isEmpty()) {
             name = i18n("QIF imported, no account name supplied");
         }
-        auto b = d->isTransfer(name, m_qifProfile.accountDelimiter().left(1), m_qifProfile.accountDelimiter().mid(1, 1));
+        auto b = d->isTransfer(name, m_qifProfile.accountDelimiter().at(0), m_qifProfile.accountDelimiter().mid(1, 1));
         Q_UNUSED(b)
         QStringList entry = m_qifEntry;   // keep a temp copy
         m_qifEntry.clear();               // and construct a temp entry to create/search the account
@@ -974,7 +982,7 @@ void MyMoneyQifReader::createOpeningBalance(eMyMoney::Account::Type accType)
             }
             if (needCreate) {
                 // in case we create it anyway, we issue a warning to the user to check it manually
-                KMessageBox::sorry(0, QString("<qt>%1</qt>").arg(i18n("KMyMoney has imported a second opening balance transaction into account <b>%1</b> which differs from the one found already on file. Please correct this manually once the import is done.", acc.name())), i18n("Opening balance problem"));
+                KMessageBox::error(0, QString("<qt>%1</qt>").arg(i18n("KMyMoney has imported a second opening balance transaction into account <b>%1</b> which differs from the one found already on file. Please correct this manually once the import is done.", acc.name())), i18n("Opening balance problem"));
             }
         }
 
@@ -1025,15 +1033,14 @@ void MyMoneyQifReader::processTransactionEntry()
 
     h = MyMoneyTransaction::hash(m_qifEntry.join(";"));
 
-    QString hashBase;
-    hashBase.sprintf("%s-%07lx", qPrintable(m_qifProfile.date(extractLine('D')).toString(Qt::ISODate)), h);
+    const auto hashBase(QStringLiteral("%1-%2").arg(m_qifProfile.date(extractLine('D')).toString(Qt::ISODate)).arg(h, 7, 16, QLatin1Char('0')));
     int idx = 1;
     QString hash;
     for (;;) {
         hash = QString("%1-%2").arg(hashBase).arg(idx);
         QMap<QString, bool>::const_iterator it;
         it = d->m_hashMap.constFind(hash);
-        if (it == d->m_hashMap.constEnd()) {
+        if (it == d->m_hashMap.cend()) {
             d->m_hashMap[hash] = true;
             break;
         }
@@ -1096,13 +1103,13 @@ void MyMoneyQifReader::processTransactionEntry()
 
     tmp = extractLine('L');
     pos = tmp.lastIndexOf("--");
-    if (tmp.left(1) == m_qifProfile.accountDelimiter().left(1)) {
+    if (tmp.at(0) == m_qifProfile.accountDelimiter().at(0)) {
         // it's a transfer, so we wipe the memo
 //   tmp = "";         why??
 //    st.m_strAccountName = tmp;
     } else if (pos != -1) {
-//    what's this?
-//    t.setValue("Dialog", tmp.mid(pos+2));
+        //    what's this?
+        //    t.setValue("Dialog", tmp.mid(pos+2));
         tmp = tmp.left(pos);
     }
 //  t.setMemo(tmp);
@@ -1166,7 +1173,7 @@ void MyMoneyQifReader::processTransactionEntry()
 
         // standard transaction
         tmp = extractLine('L');
-        if (d->isTransfer(tmp, m_qifProfile.accountDelimiter().left(1), m_qifProfile.accountDelimiter().mid(1, 1))) {
+        if (d->isTransfer(tmp, m_qifProfile.accountDelimiter().at(0), m_qifProfile.accountDelimiter().mid(1, 1))) {
             accountId = transferAccount(tmp, false);
 
         } else {
@@ -1215,7 +1222,7 @@ void MyMoneyQifReader::processTransactionEntry()
             s2.m_strMemo = listqSplits[count-1].m_strMemo;                             // Memo in split
             tmp = listqSplits[count-1].m_strCategoryName;                              // Category in split
 
-            if (d->isTransfer(tmp, m_qifProfile.accountDelimiter().left(1), m_qifProfile.accountDelimiter().mid(1, 1))) {
+            if (d->isTransfer(tmp, m_qifProfile.accountDelimiter().at(0), m_qifProfile.accountDelimiter().mid(1, 1))) {
                 accountId = transferAccount(tmp, false);
 
             } else {
@@ -1330,15 +1337,14 @@ void MyMoneyQifReader::processInvestmentTransactionEntry()
 
     h = MyMoneyTransaction::hash(m_qifEntry.join(";"));
 
-    QString hashBase;
-    hashBase.sprintf("%s-%07lx", qPrintable(m_qifProfile.date(extractLine('D')).toString(Qt::ISODate)), h);
+    const auto hashBase(QStringLiteral("%1-%2").arg(m_qifProfile.date(extractLine('D')).toString(Qt::ISODate)).arg(h, 7, 16, QLatin1Char('0')));
     int idx = 1;
     QString hash;
     for (;;) {
         hash = QString("%1-%2").arg(hashBase).arg(idx);
         QMap<QString, bool>::const_iterator it;
         it = d->m_hashMap.constFind(hash);
-        if (it == d->m_hashMap.constEnd()) {
+        if (it == d->m_hashMap.cend()) {
             d->m_hashMap[hash] = true;
             break;
         }
@@ -1461,13 +1467,13 @@ void MyMoneyQifReader::processInvestmentTransactionEntry()
     }
 
     tmp = extractLine('L');
+    static const auto bracketRegex = QRegularExpression(QLatin1String("[\\[\\]]"));
+    tmp = tmp.remove(bracketRegex); //  xAction != true so ignore any'[ and ]'
     // if the action ends in an X, the L-Record contains the asset account
     // to which the dividend should be transferred. In the other cases, it
     // may contain a category that identifies the income category for the
     // dividend payment
-    if ((xAction == true)
-            || (d->isTransfer(tmp, m_qifProfile.accountDelimiter().left(1), m_qifProfile.accountDelimiter().mid(1, 1)) == true)) {
-        tmp = tmp.remove(QRegExp("[\\[\\]]"));                 //  xAction != true so ignore any'[ and ]'
+    if ((xAction == true) || (d->isTransfer(tmp, m_qifProfile.accountDelimiter().at(0), m_qifProfile.accountDelimiter().mid(1, 1)) == true)) {
         if (!tmp.isEmpty()) {                                  // use 'L' record name
             tr.m_strBrokerageAccount = tmp;
             transferAccount(tmp);                                // make sure the account exists
@@ -1476,7 +1482,6 @@ void MyMoneyQifReader::processInvestmentTransactionEntry()
             transferAccount(m_account.brokerageName());          // make sure the account exists
         }
     } else {
-        tmp = tmp.remove(QRegExp("[\\[\\]]"));                 //  xAction != true so ignore any'[ and ]'
         tr.m_strInterestCategory = tmp;
         tr.m_strBrokerageAccount = m_account.brokerageName();
     }
@@ -1536,7 +1541,7 @@ void MyMoneyQifReader::processInvestmentTransactionEntry()
                 tr.m_amount = -(amount - tr.m_fees);
 
         if (tr.m_strMemo.isEmpty())
-            tr.m_strMemo = (QString("%1 %2").arg(extractLine('Y')).arg(d->typeToAccountName(action))).trimmed();
+            tr.m_strMemo = (QString("%1 %2").arg(extractLine('Y'), d->typeToAccountName(action))).trimmed();
     } else if (action == "xin" || action == "xout") {
         QString payee = extractLine('P');
         if (!payee.isEmpty() && ((payee.toLower() == "opening balance") || KMyMoneySettings::qifOpeningBalance().toLower().contains(payee.toLower()))) {
@@ -1547,7 +1552,7 @@ void MyMoneyQifReader::processInvestmentTransactionEntry()
         tr.m_eAction = (eMyMoney::Transaction::Action::None);
         MyMoneyStatement::Split s2;
         tmp = extractLine('L');
-        if (d->isTransfer(tmp, m_qifProfile.accountDelimiter().left(1), m_qifProfile.accountDelimiter().mid(1, 1))) {
+        if (d->isTransfer(tmp, m_qifProfile.accountDelimiter().at(0), m_qifProfile.accountDelimiter().mid(1, 1))) {
             s2.m_accountId = transferAccount(tmp);
             s2.m_strCategoryName = tmp;
         } else {
@@ -1828,8 +1833,8 @@ const QString MyMoneyQifReader::findOrCreateIncomeAccount(const QString& searchn
     // First, try to find this account as an income account
     MyMoneyAccount acc = file->income();
     QStringList list = acc.accountList();
-    QStringList::ConstIterator it_accid = list.constBegin();
-    while (it_accid != list.constEnd()) {
+    QStringList::const_iterator it_accid = list.cbegin();
+    while (it_accid != list.cend()) {
         acc = file->account(*it_accid);
         if (acc.name() == searchname) {
             result = *it_accid;
@@ -1864,8 +1869,8 @@ const QString MyMoneyQifReader::findOrCreateExpenseAccount(const QString& search
     // First, try to find this account as an income account
     MyMoneyAccount acc = file->expense();
     QStringList list = acc.accountList();
-    QStringList::ConstIterator it_accid = list.constBegin();
-    while (it_accid != list.constEnd()) {
+    QStringList::const_iterator it_accid = list.cbegin();
+    while (it_accid != list.cend()) {
         acc = file->account(*it_accid);
         if (acc.name() == searchname) {
             result = *it_accid;
@@ -1910,8 +1915,9 @@ const QString MyMoneyQifReader::processAccountEntry(bool resetAccountId)
         account.setLastReconciliationDate(m_qifProfile.date(tmp));
 
     QifEntryTypeE transactionType = EntryTransaction;
-    QString type = extractLine('T').toLower().remove(QRegExp("\\s+"));
-    if (type == m_qifProfile.profileType().toLower().remove(QRegExp("\\s+"))) {
+    static const auto whiteSpaceRegex = QRegularExpression(QLatin1String("\\s+"));
+    QString type = extractLine('T').toLower().remove(whiteSpaceRegex);
+    if (type == m_qifProfile.profileType().toLower().remove(whiteSpaceRegex)) {
         account.setAccountType(eMyMoney::Account::Type::Checkings);
     } else if (type == "ccard" || type == "creditcard") {
         account.setAccountType(eMyMoney::Account::Type::CreditCard);
@@ -2026,16 +2032,17 @@ void MyMoneyQifReader::processPriceEntry()
 
     */
 
-    QStringList::const_iterator it_line = m_qifEntry.constBegin();
+    QStringList::const_iterator it_line = m_qifEntry.cbegin();
 
     // Make a price for each line
-    QRegExp priceExp("\"(.*)\",(.*),\"(.*)\"");
-    while (it_line != m_qifEntry.constEnd()) {
-        if (priceExp.indexIn(*it_line) != -1) {
+    static const QRegularExpression priceExp(QLatin1String("\"(.*)\",(.*),\"(.*)\""));
+    while (it_line != m_qifEntry.cend()) {
+        const auto priceMatch(priceExp.match(*it_line));
+        if (priceMatch.hasMatch()) {
             MyMoneyStatement::Price price;
-            price.m_strSecurity = priceExp.cap(1);
-            QString pricestr = priceExp.cap(2);
-            QString datestr = priceExp.cap(3);
+            price.m_strSecurity = priceMatch.captured(1);
+            QString pricestr = priceMatch.captured(2);
+            QString datestr = priceMatch.captured(3);
             qDebug() << "Price:" << price.m_strSecurity << " / " << pricestr << " / " << datestr;
 
             // Only add the price if the date is valid.  If invalid, fail silently.  See note above.
