@@ -1,6 +1,7 @@
 /*
     SPDX-FileCopyrightText: 2017 Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
     SPDX-FileCopyrightText: 2019 Thomas Baumgart <tbaumgart@kde.org>
+    SPDX-FileCopyrightText: 2021 Dawid Wróbel <me@dawidwrobel.com>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
@@ -9,10 +10,6 @@
 
 // ----------------------------------------------------------------------------
 // QT Includes
-#ifdef IS_APPIMAGE
-#include <QCoreApplication>
-#include <QStandardPaths>
-#endif
 
 // ----------------------------------------------------------------------------
 // KDE Includes
@@ -32,24 +29,14 @@
 
 class MyMoneyStatement;
 
-QIFImporter::QIFImporter(QObject *parent, const QVariantList &args) :
-    KMyMoneyPlugin::Plugin(parent, "qifimporter"/*must be the same as X-KDE-PluginInfo-Name*/),
+QIFImporter::QIFImporter(QObject *parent, const KPluginMetaData &metaData, const QVariantList &args) :
+    KMyMoneyPlugin::Plugin(parent, metaData, args),
     m_qifReader(nullptr)
 {
-    Q_UNUSED(args);
-    const auto componentName = QLatin1String("qifimporter");
+    Q_INIT_RESOURCE(qifimporter);
+
     const auto rcFileName = QLatin1String("qifimporter.rc");
-    setComponentName(componentName, i18n("QIF importer"));
-
-#ifdef IS_APPIMAGE
-    const QString rcFilePath = QString("%1/../share/kxmlgui5/%2/%3").arg(QCoreApplication::applicationDirPath(), componentName, rcFileName);
-    setXMLFile(rcFilePath);
-
-    const QString localRcFilePath = QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation).first() + QLatin1Char('/') + rcFileName;
-    setLocalXMLFile(localRcFilePath);
-#else
     setXMLFile(rcFileName);
-#endif
 
     createActions();
     // For information, announce that we have been loaded.
@@ -111,6 +98,6 @@ bool QIFImporter::slotGetStatements(const QList<MyMoneyStatement> &statements)
     return ret;
 }
 
-K_PLUGIN_FACTORY_WITH_JSON(QIFImporterFactory, "qifimporter.json", registerPlugin<QIFImporter>();)
+K_PLUGIN_CLASS_WITH_JSON(QIFImporter, "qifimporter.json")
 
 #include "qifimporter.moc"
