@@ -19,7 +19,7 @@
 #include "mymoneyfile.h"
 #include "mymoneyaccount.h"
 #include "mymoneyexception.h"
-#include "mymoneystoragemgr.h"
+#include "parametersmodel.h"
 
 #include <alkimia/alkvalue.h>
 
@@ -142,6 +142,8 @@ void MyMoneyDbDef::Payees()
     appendField(MyMoneyDbIntColumn("matchData", MyMoneyDbIntColumn::TINY, UNSIGNED, false, false, 5));
     appendField(MyMoneyDbColumn("matchIgnoreCase", "char(1)", false, false, 5));
     appendField(MyMoneyDbTextColumn("matchKeys", MyMoneyDbTextColumn::MEDIUM, false, false, 5));
+    appendField(MyMoneyDbTextColumn("idPattern"));
+    appendField(MyMoneyDbTextColumn("urlTemplate"));
     MyMoneyDbTable t("kmmPayees", fields);
     t.buildSQLStrings();
     m_tables[t.name()] = t;
@@ -492,9 +494,10 @@ const QString MyMoneyDbDef::generateSQL(const QExplicitlySharedDataPointer<MyMon
         QString replace = "NULL";
         if ((*fit)->name() == "version")
             replace = QString::number(m_currentVersion);
-        if ((*fit)->name() == "fixLevel")
-            replace =  QString::number
-                       (MyMoneyFile::instance()->storage()->currentFixVersion());
+        if ((*fit)->name() == "fixLevel") {
+            const auto file = MyMoneyFile::instance();
+            replace = file->parametersModel()->itemById(file->fixedKey(MyMoneyFile::FileFixVersion)).value();
+        }
         if ((*fit)->name() == "created")
             replace = QLatin1Char('\'')
                       + QDate::currentDate().toString(Qt::ISODate)
@@ -506,7 +509,7 @@ const QString MyMoneyDbDef::generateSQL(const QExplicitlySharedDataPointer<MyMon
         if ((*fit)->name() == "updateInProgress")
             replace = enclose("N");
 
-        qs.replace(QRegExp(toReplace + "(?=[,\\s\\)])"), replace);
+        qs.replace(QRegularExpression(toReplace + QLatin1String("(?=[,\\s\\)])")), replace);
         // only replace parameters followed by comma, whitespace, closing parenthesis - otherwise
         // conflicts may occur if one parameter starts with the name of another one.
     }
@@ -565,8 +568,8 @@ void MyMoneyDbTable::buildSQLStrings()
     // build the insert string with placeholders for each field
     QString qs = QString("INSERT INTO %1 (").arg(name());
     QString ws = ") VALUES (";
-    field_iterator ft = m_fields.constBegin();
-    while (ft != m_fields.constEnd()) {
+    field_iterator ft = m_fields.cbegin();
+    while (ft != m_fields.cend()) {
         qs += QString("%1, ").arg((*ft)->name());
         ws += QString(":%1, ").arg((*ft)->name());
         ++ft;
@@ -581,13 +584,13 @@ void MyMoneyDbTable::buildSQLStrings()
     // build an update string; key fields go in the where clause
     qs = "UPDATE " + name() + " SET ";
     ws.clear();
-    ft = m_fields.constBegin();
-    while (ft != m_fields.constEnd()) {
+    ft = m_fields.cbegin();
+    while (ft != m_fields.cend()) {
         if ((*ft)->isPrimaryKey()) {
             if (!ws.isEmpty()) ws += " AND ";
-            ws += QString("%1 = :%2").arg((*ft)->name()).arg((*ft)->name());
+            ws += QString("%1 = :%2").arg((*ft)->name(), (*ft)->name());
         } else {
-            qs += QString("%1 = :%2, ").arg((*ft)->name()).arg((*ft)->name());
+            qs += QString("%1 = :%2, ").arg((*ft)->name(), (*ft)->name());
         }
         ++ft;
     }
@@ -600,21 +603,20 @@ void MyMoneyDbTable::buildSQLStrings()
     m_deleteString = qs + ';';
 
     // Setup the column name hash
-    ft = m_fields.constBegin();
+    ft = m_fields.cbegin();
     m_fieldOrder.reserve(m_fields.size());
     int i = 0;
-    while (ft != m_fields.constEnd()) {
+    while (ft != m_fields.cend()) {
         m_fieldOrder[(*ft)->name()] = i;
         ++i;
         ++ft;
     }
 }
 
-const QString MyMoneyDbTable::columnList(const int version, bool useNewNames) const
+QString MyMoneyDbTable::columnList(const int version, bool useNewNames) const
 {
     field_iterator ft = m_fields.begin();
     QString qs;
-    ft = m_fields.begin();
     while (ft != m_fields.end()) {
         if ((*ft)->initVersion() <= version && (*ft)->lastVersion() >= version) {
             QString fieldName = (*ft)->name();
@@ -665,8 +667,8 @@ const QString MyMoneyDbTable::dropPrimaryKeyString(const QExplicitlySharedDataPo
 
 bool MyMoneyDbTable::hasPrimaryKey(int version) const
 {
-    field_iterator ft = m_fields.constBegin();
-    while (ft != m_fields.constEnd()) {
+    field_iterator ft = m_fields.cbegin();
+    while (ft != m_fields.cend()) {
         if ((*ft)->initVersion() <= version && (*ft)->lastVersion() >= version) {
             if ((*ft)->isPrimaryKey())
                 return (true);
@@ -683,9 +685,9 @@ const QString MyMoneyDbTable::modifyColumnString(const QExplicitlySharedDataPoin
 
 int MyMoneyDbTable::fieldNumber(const QString& name) const
 {
-    QHash<QString, int>::ConstIterator i = m_fieldOrder.find(name);
-    if (m_fieldOrder.constEnd() == i) {
-        throw MYMONEYEXCEPTION(QString::fromLatin1("Unknown field %1 in table %2").arg(name).arg(m_name));
+    QHash<QString, int>::const_iterator i = m_fieldOrder.find(name);
+    if (m_fieldOrder.cend() == i) {
+        throw MYMONEYEXCEPTION(QString::fromLatin1("Unknown field %1 in table %2").arg(name, m_name));
     }
     return i.value();
 }
@@ -714,7 +716,7 @@ const QString MyMoneyDbIndex::generateDDL(const QExplicitlySharedDataPointer<MyM
     // the result of an SQL function, but not a partial column.  There should be
     // a way to merge these, and support other DBMSs like SQLite at the same time.
     // For now, if we just use plain columns, this will work fine.
-    for (QStringList::ConstIterator it = m_columns.constBegin(); it != m_columns.constEnd(); ++it) {
+    for (QStringList::const_iterator it = m_columns.cbegin(); it != m_columns.cend(); ++it) {
         qs += *it + ',';
     }
 
