@@ -1,6 +1,7 @@
 /*
     SPDX-FileCopyrightText: 2018 Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
     SPDX-FileCopyrightText: 2019 Thomas Baumgart <tbaumgart@kde.org>
+    SPDX-FileCopyrightText: 2021 Dawid Wróbel <me@dawidwrobel.com>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
@@ -11,13 +12,12 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
-#include <QUrlQuery>
+#include <QFile>
+#include <QPointer>
+#include <QSqlDriver>
 #include <QSqlQuery>
 #include <QTimer>
-#include <QFile>
-#include <QSqlDriver>
-#include <QCoreApplication>
-#include <QStandardPaths>
+#include <QUrlQuery>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
@@ -30,19 +30,19 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include "appinterface.h"
-#include "icons.h"
-#include "kgeneratesqldlg.h"
-#include "kmymoneyenums.h"
-#include "kmymoneysettings.h"
-#include "kselectdatabasedlg.h"
-#include "mymoneyexception.h"
-#include "mymoneyfile.h"
-#include "mymoneystoragemgr.h"
-#include "mymoneystoragesql.h"
-#include "mymoneyutils.h"
 #include "sqlstorage.h"
+#include "appinterface.h"
 #include "viewinterface.h"
+#include "kselectdatabasedlg.h"
+#include "kgeneratesqldlg.h"
+#include "mymoneyfile.h"
+#include "mymoneystoragesql.h"
+#include "mymoneyexception.h"
+#include "icons.h"
+#include "kmymoneysettings.h"
+#include "kmymoneyenums.h"
+
+#include "kmmyesno.h"
 
 using namespace Icons;
 
@@ -71,24 +71,14 @@ QUrlQuery SQLStorage::convertOldUrl(const QUrl& url)
 }
 
 
-SQLStorage::SQLStorage(QObject *parent, const QVariantList &args) :
-    KMyMoneyPlugin::Plugin(parent, "sqlstorage"/*must be the same as X-KDE-PluginInfo-Name*/)
+SQLStorage::SQLStorage(QObject *parent, const KPluginMetaData &metaData, const QVariantList &args) :
+    KMyMoneyPlugin::Plugin(parent, metaData, args)
 {
-    Q_UNUSED(args)
-    const auto componentName = QLatin1String("sqlstorage");
+    Q_INIT_RESOURCE(sqlstorage);
+
     const auto rcFileName = QLatin1String("sqlstorage.rc");
-    setComponentName(componentName, i18n("SQL storage"));
 
-    if (MyMoneyUtils::isRunningAsAppImage()) {
-        const QString rcFilePath = QString("%1/../share/kxmlgui5/%2/%3").arg(QCoreApplication::applicationDirPath(), componentName, rcFileName);
-        setXMLFile(rcFilePath);
-
-        const QString localRcFilePath =
-            QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation).first() + QLatin1Char('/') + componentName + QLatin1Char('/') + rcFileName;
-        setLocalXMLFile(localRcFilePath);
-    } else {
-        setXMLFile(rcFileName);
-    }
+    setXMLFile(rcFileName);
 
     createActions();
     // For information, announce that we have been loaded.
@@ -103,13 +93,12 @@ SQLStorage::~SQLStorage()
     qDebug("Plugins: sqlstorage unloaded");
 }
 
-MyMoneyStorageMgr *SQLStorage::open(const QUrl &url)
+bool SQLStorage::open(const QUrl &url)
 {
     if (url.scheme() != QLatin1String("sql"))
-        return nullptr;
+        return false;
 
-    auto storage = new MyMoneyStorageMgr;
-    auto reader = std::make_unique<MyMoneyStorageSql>(storage, url);
+    auto reader = std::make_unique<MyMoneyStorageSql>(MyMoneyFile::instance(), url);
 
     dbUrl = url;
     if (dbUrl.password().isEmpty()) {
@@ -117,7 +106,7 @@ MyMoneyStorageMgr *SQLStorage::open(const QUrl &url)
         QPointer<KSelectDatabaseDlg> dialog = new KSelectDatabaseDlg(QIODevice::ReadWrite, dbUrl);
         if (!dialog->checkDrivers()) {
             delete dialog;
-            return nullptr;
+            return false;
         }
         QUrlQuery query = convertOldUrl(dbUrl);
         // if we need to supply a password, then show the dialog
@@ -127,7 +116,7 @@ MyMoneyStorageMgr *SQLStorage::open(const QUrl &url)
                 dbUrl = dialog->selectedURL();
             } else {
                 delete dialog;
-                return nullptr;
+                return false;
             }
         }
         delete dialog;
@@ -147,12 +136,11 @@ MyMoneyStorageMgr *SQLStorage::open(const QUrl &url)
             KMessageBox::detailedError(nullptr,
                                        i18n("Cannot open database %1\n", dbURL.toDisplayString()),
                                        reader->lastError());
-            delete storage;
-            return nullptr;
+            return false;
         case -1: // retryable error
-            if (KMessageBox::warningYesNo(nullptr, reader->lastError(), PACKAGE) == KMessageBox::No) {
-                delete storage;
-                return nullptr;
+            if (KMessageBox::warningTwoActions(nullptr, reader->lastError(), PACKAGE, KMMYesNo::yes(), KMMYesNo::no()) == KMessageBox::SecondaryAction) {
+                return false;
+
             } else {
                 QUrlQuery query(dbURL);
                 const QString optionKey = QLatin1String("options");
@@ -169,8 +157,7 @@ MyMoneyStorageMgr *SQLStorage::open(const QUrl &url)
             break;
         case 2: // bad password
         case 3: // unsupported operation
-            delete storage;
-            return nullptr;
+            return false;
         }
     }
     // single user mode; read some of the data into memory
@@ -183,11 +170,11 @@ MyMoneyStorageMgr *SQLStorage::open(const QUrl &url)
                                    i18n("An unrecoverable error occurred while reading the database"),
                                    reader->lastError().toLatin1(),
                                    i18n("Database malfunction"));
-        delete storage;
-        return nullptr;
+        return false;
     }
 //  reader->setProgressCallback(0);
-    return storage;
+    /// @todo port to new model code
+    return true;
 }
 
 QUrl SQLStorage::openUrl() const
@@ -202,7 +189,7 @@ bool SQLStorage::save(const QUrl &url)
         KMessageBox::error(nullptr, i18n("Tried to access a file when it has not been opened"));
         return (rc);
     }
-    auto writer = new MyMoneyStorageSql(MyMoneyFile::instance()->storage(), url);
+    auto writer = new MyMoneyStorageSql(MyMoneyFile::instance(), url);
     writer->open(url, QIODevice::ReadWrite);
 //  writer->setProgressCallback(&KMyMoneyView::progressCallback);
     if (!writer->writeFile()) {
@@ -211,7 +198,6 @@ bool SQLStorage::save(const QUrl &url)
                                         "It may well be corrupt."),
                                    writer->lastError().toLatin1(),
                                    i18n("Database malfunction"));
-        rc =  false;
     } else {
         rc = true;
     }
@@ -243,12 +229,12 @@ bool SQLStorage::saveAs()
                 && oldUrl.host() == url.host()
                 && QUrlQuery(oldUrl).queryItemValue("driver") == QUrlQuery(url).queryItemValue("driver")
                 && oldUrl.path().right(oldUrl.path().length() - 1) == url.path().right(url.path().length() - 1)) {
-            KMessageBox::sorry(nullptr, i18n("Cannot save to current database."));
+            KMessageBox::error(nullptr, i18n("Cannot save to current database."));
         } else {
             try {
                 rc = saveAsDatabase(url);
             } catch (const MyMoneyException &e) {
-                KMessageBox::sorry(nullptr, i18n("Cannot save to current database: %1", QString::fromLatin1(e.what())));
+                KMessageBox::error(nullptr, i18n("Cannot save to current database: %1", QString::fromLatin1(e.what())));
             }
         }
     }
@@ -317,9 +303,7 @@ void SQLStorage::slotOpenDatabase()
                     return;
                 }
             }
-            delete dialog;
         }
-
         appInterface()->slotFileOpenRecent(newurl);
     }
     delete dialog;
@@ -335,7 +319,7 @@ void SQLStorage::slotGenerateSql()
 
 bool SQLStorage::saveAsDatabase(const QUrl &url)
 {
-    auto writer = new MyMoneyStorageSql(MyMoneyFile::instance()->storage(), url);
+    auto writer = new MyMoneyStorageSql(MyMoneyFile::instance(), url);
     auto canWrite = false;
     switch (writer->open(url, QIODevice::WriteOnly)) {
     case 0:
@@ -371,6 +355,6 @@ bool SQLStorage::saveAsDatabase(const QUrl &url)
     }
 }
 
-K_PLUGIN_FACTORY_WITH_JSON(SQLStorageFactory, "sqlstorage.json", registerPlugin<SQLStorage>();)
+K_PLUGIN_CLASS_WITH_JSON(SQLStorage, "sqlstorage.json")
 
 #include "sqlstorage.moc"
