@@ -1,6 +1,7 @@
 /*
     SPDX-FileCopyrightText: 2007 Alvaro Soliverez <asoliverez@gmail.com>
     SPDX-FileCopyrightText: 2017 Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
+    SPDX-FileCopyrightText: 2020 Thomas Baumgart <tbaumgart@kde.org>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
@@ -76,18 +77,17 @@ class KForecastViewPrivate : public KMyMoneyViewBasePrivate
     Q_DECLARE_PUBLIC(KForecastView)
 
 public:
-    explicit KForecastViewPrivate(KForecastView *qq) :
-        KMyMoneyViewBasePrivate(),
-        q_ptr(qq),
-        ui(new Ui::KForecastView),
-        m_needLoad(true),
-        m_totalItem(0),
-        m_assetItem(0),
-        m_liabilityItem(0),
-        m_incomeItem(0),
-        m_expenseItem(0),
-        m_chartLayout(0),
-        m_forecastChart(nullptr)
+    explicit KForecastViewPrivate(KForecastView *qq)
+        : KMyMoneyViewBasePrivate(qq)
+        , ui(new Ui::KForecastView)
+        , m_needLoad(true)
+        , m_totalItem(nullptr)
+        , m_assetItem(nullptr)
+        , m_liabilityItem(nullptr)
+        , m_incomeItem(nullptr)
+        , m_expenseItem(nullptr)
+        , m_chartLayout(nullptr)
+        , m_forecastChart(nullptr)
     {
     }
 
@@ -102,6 +102,8 @@ public:
         m_needLoad = false;
         ui->setupUi(q);
 
+        ui->m_budgetHistoryRange->hide();
+
         for (int i = 0; i < MaxViewTabs; ++i)
             m_needReload[i] = false;
 
@@ -109,7 +111,7 @@ public:
         KConfigGroup grp = config->group("Last Use Settings");
         ui->m_tab->setCurrentIndex(grp.readEntry("KForecastView_LastType", 0));
 
-        ui->m_forecastButton->setIcon(Icons::get(Icon::Forecast));
+        ui->m_forecastButton->setIcon(Icons::get(Icon::OfficeChartLineForecast));
 
         q->connect(ui->m_tab, &QTabWidget::currentChanged, q, &KForecastView::slotTabChanged);
 
@@ -134,6 +136,8 @@ public:
         m_chartLayout->setSpacing(6);
 
         loadForecastSettings();
+
+        m_focusWidget = ui->m_forecastButton;
     }
 
     void loadForecast(ForecastViewTab tab)
@@ -189,7 +193,7 @@ public:
 
         for (int i = 1; i <= forecast.forecastDays(); ++i) {
             QDate forecastDate = QDate::currentDate().addDays(i);
-            headerLabels << QLocale().toString(forecastDate, QLocale::LongFormat);
+            headerLabels << MyMoneyUtils::formatDate(forecastDate, QLocale::LongFormat);
         }
 
         //add variation columns
@@ -225,7 +229,7 @@ public:
         if (accList.isEmpty())
             return;
 
-        foreach (const auto sAccount, accList) {
+        for (const auto& sAccount : accList) {
             auto subAccount = file->account(sAccount);
             //only add the account if it is a forecast account or the parent of a forecast account
             if (includeAccount(forecast, subAccount)) {
@@ -233,9 +237,8 @@ public:
             }
         }
 
-        QMap<QString, QString>::ConstIterator it_nc;
-        for (it_nc = nameIdx.constBegin(); it_nc != nameIdx.constEnd(); ++it_nc) {
-
+        QMap<QString, QString>::const_iterator it_nc;
+        for (it_nc = nameIdx.cbegin(); it_nc != nameIdx.cend(); ++it_nc) {
             const MyMoneyAccount subAccount = file->account(*it_nc);
             MyMoneySecurity currency;
             if (subAccount.isInvest()) {
@@ -247,7 +250,7 @@ public:
 
             forecastItem = new QTreeWidgetItem(parentItem);
             forecastItem->setText(0, subAccount.name());
-            forecastItem->setIcon(0, subAccount.accountPixmap());
+            forecastItem->setIcon(0, subAccount.accountIcon());
             forecastItem->setData(0, ForecastRole, QVariant::fromValue(forecast));
             forecastItem->setData(0, AccountRole, QVariant::fromValue(subAccount));
             forecastItem->setExpanded(true);
@@ -329,17 +332,16 @@ public:
         //Get all accounts of the right type to calculate forecast
         m_nameIdx.clear();
         accList = forecast.accountList();
-        QList<MyMoneyAccount>::const_iterator accList_t = accList.constBegin();
-        for (; accList_t != accList.constEnd(); ++accList_t) {
+        QList<MyMoneyAccount>::const_iterator accList_t = accList.cbegin();
+        for (; accList_t != accList.cend(); ++accList_t) {
             MyMoneyAccount acc = *accList_t;
             if (m_nameIdx[acc.id()] != acc.id()) { //Check if the account is there
                 m_nameIdx[acc.id()] = acc.id();
             }
         }
 
-        QMap<QString, QString>::ConstIterator it_nc;
-        for (it_nc = m_nameIdx.constBegin(); it_nc != m_nameIdx.constEnd(); ++it_nc) {
-
+        QMap<QString, QString>::const_iterator it_nc;
+        for (it_nc = m_nameIdx.cbegin(); it_nc != m_nameIdx.cend(); ++it_nc) {
             const MyMoneyAccount& acc = file->account(*it_nc);
             MyMoneySecurity currency;
 
@@ -462,8 +464,8 @@ public:
         //Get all accounts of the right type to calculate forecast
         m_nameIdx.clear();
         accList = forecast.accountList();
-        QList<MyMoneyAccount>::const_iterator accList_t = accList.constBegin();
-        for (; accList_t != accList.constEnd(); ++accList_t) {
+        QList<MyMoneyAccount>::const_iterator accList_t = accList.cbegin();
+        for (; accList_t != accList.cend(); ++accList_t) {
             MyMoneyAccount acc = *accList_t;
             if (m_nameIdx[acc.id()] != acc.id()) { //Check if the account is there
                 m_nameIdx[acc.id()] = acc.id();
@@ -501,8 +503,8 @@ public:
 
         QTreeWidgetItem *advancedItem = 0;
 
-        QMap<QString, QString>::ConstIterator it_nc;
-        for (it_nc = m_nameIdx.constBegin(); it_nc != m_nameIdx.constEnd(); ++it_nc) {
+        QMap<QString, QString>::const_iterator it_nc;
+        for (it_nc = m_nameIdx.cbegin(); it_nc != m_nameIdx.cend(); ++it_nc) {
             const MyMoneyAccount& acc = file->account(*it_nc);
             QString amount;
             MyMoneyMoney amountMM;
@@ -519,7 +521,7 @@ public:
 
             advancedItem = new QTreeWidgetItem(ui->m_advancedList, advancedItem, false);
             advancedItem->setText(0, acc.name());
-            advancedItem->setIcon(0, acc.accountPixmap());
+            advancedItem->setIcon(0, acc.accountIcon());
             auto it_c = 1; // iterator for the columns of the listview
 
             //get minimum balance list
@@ -537,7 +539,7 @@ public:
                 }
                 it_c++;
 
-                QString dateString = QLocale().toString(minDate, QLocale::ShortFormat);
+                QString dateString = MyMoneyUtils::formatDate(minDate);
                 advancedItem->setText(it_c, dateString);
                 advancedItem->setTextAlignment(it_c, Qt::AlignRight | Qt::AlignVCenter);
                 if (amountMM.isNegative()) {
@@ -561,7 +563,7 @@ public:
                 }
                 it_c++;
 
-                QString dateString = QLocale().toString(maxDate, QLocale::ShortFormat);
+                QString dateString = MyMoneyUtils::formatDate(maxDate);
                 advancedItem->setText(it_c, dateString);
                 advancedItem->setTextAlignment(it_c, Qt::AlignRight | Qt::AlignVCenter);
                 if (amountMM.isNegative()) {
@@ -617,7 +619,7 @@ public:
             //add cycle interval columns
             QDate f_date = forecastStartDate;
             for (; f_date <= forecastEndDate; f_date = f_date.addMonths(1)) {
-                headerLabels << QDate::longMonthName(f_date.month());
+                headerLabels << QLocale().standaloneMonthName(f_date.month(), QLocale::LongFormat);
             }
         }
         //add total column
@@ -636,6 +638,12 @@ public:
         loadAccounts(forecast, file->expense(), m_expenseItem, EForecastViewType::eBudget);
 
         adjustHeadersAndResizeToContents(ui->m_budgetList);
+
+        // Show the range which historic transactions are used to calculate the forecast
+        ui->m_budgetHistoryRange->setText(
+            i18nc("@info Hint about the base of the budget forecast", "Budget forecast is based on historic transactions found between %1 and %2.")
+                .arg(MyMoneyUtils::formatDate(historyStartDate), MyMoneyUtils::formatDate(historyEndDate)));
+        ui->m_budgetHistoryRange->setVisible(forecast.forecastMethod() == MyMoneyForecast::eForecastMethod::Historic);
     }
 
     void loadChartView()
@@ -671,13 +679,19 @@ public:
         ui->m_historyMethod->setId(ui->radioButton12, 1); // weighted moving avg
         ui->m_historyMethod->setId(ui->radioButton13, 2); // linear regression
         ui->m_historyMethod->button(KMyMoneySettings::historyMethod())->setChecked(true);
-        switch (KMyMoneySettings::forecastMethod()) {
-        case 0:
+
+        updateForecastMethod();
+    }
+
+    void updateForecastMethod()
+    {
+        switch (static_cast<MyMoneyForecast::eForecastMethod>(KMyMoneySettings::forecastMethod())) {
+        case MyMoneyForecast::eForecastMethod::Scheduled:
             ui->m_forecastMethod->setText(i18nc("Scheduled method", "Scheduled"));
             ui->m_forecastCycles->setDisabled(true);
             ui->m_historyMethodGroupBox->setDisabled(true);
             break;
-        case 1:
+        case MyMoneyForecast::eForecastMethod::Historic:
             ui->m_forecastMethod->setText(i18nc("History-based method", "History"));
             ui->m_forecastCycles->setEnabled(true);
             ui->m_historyMethodGroupBox->setEnabled(true);
@@ -694,14 +708,14 @@ public:
 
         m_assetItem = new QTreeWidgetItem(m_totalItem);
         m_assetItem->setText(0, file->asset().name());
-        m_assetItem->setIcon(0, file->asset().accountPixmap());
+        m_assetItem->setIcon(0, file->asset().accountIcon());
         m_assetItem->setData(0, ForecastRole, QVariant::fromValue(forecast));
         m_assetItem->setData(0, AccountRole, QVariant::fromValue(file->asset()));
         m_assetItem->setExpanded(true);
 
         m_liabilityItem = new QTreeWidgetItem(m_totalItem);
         m_liabilityItem->setText(0, file->liability().name());
-        m_liabilityItem->setIcon(0, file->liability().accountPixmap());
+        m_liabilityItem->setIcon(0, file->liability().accountIcon());
         m_liabilityItem->setData(0, ForecastRole, QVariant::fromValue(forecast));
         m_liabilityItem->setData(0, AccountRole, QVariant::fromValue(file->liability()));
         m_liabilityItem->setExpanded(true);
@@ -713,14 +727,14 @@ public:
 
         m_incomeItem = new QTreeWidgetItem(m_totalItem);
         m_incomeItem->setText(0, file->income().name());
-        m_incomeItem->setIcon(0, file->income().accountPixmap());
+        m_incomeItem->setIcon(0, file->income().accountIcon());
         m_incomeItem->setData(0, ForecastRole, QVariant::fromValue(forecast));
         m_incomeItem->setData(0, AccountRole, QVariant::fromValue(file->income()));
         m_incomeItem->setExpanded(true);
 
         m_expenseItem = new QTreeWidgetItem(m_totalItem);
         m_expenseItem->setText(0, file->expense().name());
-        m_expenseItem->setIcon(0, file->expense().accountPixmap());
+        m_expenseItem->setIcon(0, file->expense().accountIcon());
         m_expenseItem->setData(0, ForecastRole, QVariant::fromValue(forecast));
         m_expenseItem->setData(0, AccountRole, QVariant::fromValue(file->expense()));
         m_expenseItem->setExpanded(true);
@@ -735,7 +749,7 @@ public:
         font.setBold(true);
         m_totalItem->setFont(0, font);
         m_totalItem->setText(0, i18nc("Total balance", "Total"));
-        m_totalItem->setIcon(0, file->asset().accountPixmap());
+        m_totalItem->setIcon(0, file->asset().accountIcon());
         m_totalItem->setData(0, ForecastRole, QVariant::fromValue(forecast));
         m_totalItem->setData(0, AccountRole, QVariant::fromValue(file->asset()));
         m_totalItem->setExpanded(true);
@@ -748,7 +762,8 @@ public:
         if (forecast.isForecastAccount(acc))
             return true;
 
-        foreach (const auto sAccount, acc.accountList()) {
+        const auto subAccountList = acc.accountList();
+        for (const auto& sAccount : qAsConst(subAccountList)) {
             auto account = file->account(sAccount);
             if (includeAccount(forecast, account))
                 return true;
@@ -788,6 +803,8 @@ public:
         item->setFont(column, item->font(0));
         if (amount.isNegative()) {
             item->setForeground(column, KMyMoneySettings::schemeColor(SchemeColor::Negative));
+        } else {
+            item->setForeground(column, QBrush());
         }
     }
 
@@ -993,7 +1010,6 @@ public:
 //    return prices;
 //  }
 
-    KForecastView     *q_ptr;
     Ui::KForecastView *ui;
 
     bool m_needReload[MaxViewTabs];
