@@ -6,8 +6,19 @@
 #ifndef KONLINEJOBOUTBOXVIEW_H
 #define KONLINEJOBOUTBOXVIEW_H
 
-#include "kmymoneyviewbase.h"
+// ----------------------------------------------------------------------------
+// Qt Headers
 
+// ----------------------------------------------------------------------------
+// KDE Headers
+
+class KXMLGUIClient;
+class KXMLGUIFactory;
+
+// ----------------------------------------------------------------------------
+// Project Headers
+
+#include "kmymoneyviewbase.h"
 #include "onlinejob.h"
 
 class QModelIndex;
@@ -16,7 +27,22 @@ namespace KMyMoneyPlugin {
 class OnlinePlugin;
 }
 
+namespace eMenu {
+enum class OnlineAction {
+    LogOnlineJob,
+    AccountCreditTransfer,
+    DeleteOnlineJob,
+    EditOnlineJob,
+    SendOnlineJobs,
+};
+inline qHashSeedType qHash(const OnlineAction key, qHashSeedType seed)
+{
+    return ::qHash(static_cast<uint>(key), seed);
+}
+};
+
 class KOnlineJobOutboxViewPrivate;
+class SelectedObjects;
 class KOnlineJobOutboxView : public KMyMoneyViewBase
 {
     Q_OBJECT
@@ -25,15 +51,17 @@ public:
     explicit KOnlineJobOutboxView(QWidget *parent = 0);
     ~KOnlineJobOutboxView() override;
 
-    void executeCustomAction(eView::Action action) override;
+    void executeAction(eMenu::Action action, const SelectedObjects& selections) override;
 
-    void updateActions(const MyMoneyObject& obj);
+    void createActions(KXMLGUIFactory* guiFactory, KXMLGUIClient* guiClient);
+    void removeActions();
 
     QStringList selectedOnlineJobs() const;
 
 public Q_SLOTS:
-    void slotSelectByObject(const MyMoneyObject& obj, eView::Intent intent) override;
-    void slotSelectByVariant(const QVariantList& variant, eView::Intent intent) override;
+    void updateActions(const SelectedObjects& selections) override;
+
+    virtual void setOnlinePlugins(QMap<QString, KMyMoneyPlugin::OnlinePlugin*>* plugins) override;
 
 Q_SIGNALS:
     void sendJobs(QList<onlineJob>);
@@ -48,8 +76,7 @@ private:
     Q_DECLARE_PRIVATE(KOnlineJobOutboxView)
 
 private Q_SLOTS:
-    void updateNewCreditTransferButton();
-    void updateButtonState() const;
+    void updateSelection();
 
     void slotRemoveJob();
 
@@ -65,7 +92,10 @@ private Q_SLOTS:
     void slotEditJob();
     void slotEditJob(const QModelIndex&);
 
-    void slotOnlineJobSave(onlineJob job);
+    /**
+     * Saves the @a job to the engine and returns the id of the job
+     */
+    QString slotOnlineJobSave(onlineJob job);
     void slotOnlineJobSend(onlineJob job);
     void slotOnlineJobSend(QList<onlineJob> jobs);
 
