@@ -19,18 +19,17 @@
 #include <KLocalizedString>
 #include <KPluginFactory>
 
-#include "kguiutils.h"
-#include "onlinetasks/interfaces/ui/ionlinejobedit.h"
-
-#include "mymoneyfile.h"
-#include "mymoney/onlinejobadministration.h"
-#include "onlinejob.h"
-#include "tasks/onlinetask.h"
 #include "accountsmodel.h"
+#include "icons.h"
+#include "kguiutils.h"
+#include "mymoney/onlinejobadministration.h"
 #include "mymoneyexception.h"
+#include "mymoneyfile.h"
+#include "mymoneyutils.h"
+#include "onlinejob.h"
+#include "onlinetasks/interfaces/ui/ionlinejobedit.h"
+#include "tasks/onlinetask.h"
 
-#include "models/models.h"
-#include "icons/icons.h"
 using namespace Icons;
 
 kOnlineTransferForm::kOnlineTransferForm(QWidget *parent)
@@ -46,14 +45,14 @@ kOnlineTransferForm::kOnlineTransferForm(QWidget *parent)
     delete ui->creditTransferEdit->takeWidget();
 
     OnlineBankingAccountNamesFilterProxyModel* accountsModel = new OnlineBankingAccountNamesFilterProxyModel(this);
-    accountsModel->setSourceModel(Models::instance()->accountsModel());
+    accountsModel->setSourceModel(MyMoneyFile::instance()->accountsModel());
     ui->originAccount->setModel(accountsModel);
 
     ui->convertMessage->hide();
     ui->convertMessage->setWordWrap(true);
 
     auto edits = onlineJobAdministration::instance()->onlineJobEdits();
-    std::for_each(edits.constBegin(), edits.constEnd(), [this](onlineJobAdministration::onlineJobEditOffer in) {
+    std::for_each(edits.cbegin(), edits.cend(), [this](onlineJobAdministration::onlineJobEditOffer in) {
         this->loadOnlineJobEditPlugin(in);
     });
 
@@ -99,7 +98,7 @@ void kOnlineTransferForm::loadOnlineJobEditPlugin(const onlineJobAdministration:
             return;
         }
 
-        IonlineJobEdit* widget = pluginFactory->create<IonlineJobEdit>(pluginDesc.pluginKeyword, this);
+        IonlineJobEdit* widget = pluginFactory->create<IonlineJobEdit>(this);
         if (!widget) {
             qWarning() << "Could not create online job editor in file \"" << pluginDesc.fileName << "\".";
             return;
@@ -171,13 +170,13 @@ void kOnlineTransferForm::duplicateCurrentJob()
 
 void kOnlineTransferForm::accept()
 {
-    emit acceptedForSave(activeOnlineJob());
+    Q_EMIT acceptedForSave(activeOnlineJob());
     QDialog::accept();
 }
 
 void kOnlineTransferForm::sendJob()
 {
-    emit acceptedForSend(activeOnlineJob());
+    Q_EMIT acceptedForSend(activeOnlineJob());
     QDialog::accept();
 }
 
@@ -217,7 +216,7 @@ void kOnlineTransferForm::accountChanged()
         ui->orderAccountBalance->setText("");
     }
 
-    foreach (IonlineJobEdit* widget, m_onlineJobEditWidgets)
+    for (IonlineJobEdit* widget : qAsConst(m_onlineJobEditWidgets))
         widget->setOriginAccount(accountId);
 
     checkNotSupportedWidget();
@@ -280,7 +279,9 @@ void kOnlineTransferForm::setJobReadOnly(const bool& readOnly)
     if (readOnly) {
         ui->headMessage->setMessageType(KMessageWidget::Information);
         if (activeOnlineJob().sendDate().isValid())
-            ui->headMessage->setText(i18n("This credit-transfer was sent to your bank at %1 therefore cannot be edited anymore. You may create a copy for editing.", activeOnlineJob().sendDate().toString(Qt::DefaultLocaleShortDate)));
+            ui->headMessage->setText(
+                i18n("This credit-transfer was sent to your bank at %1 therefore cannot be edited anymore. You may create a copy for editing.",
+                     MyMoneyUtils::formatDateTime(activeOnlineJob().sendDate())));
         else
             ui->headMessage->setText(i18n("This credit-transfer is not editable. You may create a copy for editing."));
 
@@ -296,7 +297,7 @@ void kOnlineTransferForm::setJobReadOnly(const bool& readOnly)
 bool kOnlineTransferForm::showEditWidget(const QString& onlineTaskName)
 {
     int index = 0;
-    foreach (IonlineJobEdit* widget, m_onlineJobEditWidgets) {
+    for (IonlineJobEdit* widget : qAsConst(m_onlineJobEditWidgets)) {
         if (widget->supportedOnlineTasks().contains(onlineTaskName)) {
             ui->transferTypeSelection->setCurrentIndex(index);
             showEditWidget(widget);
