@@ -75,17 +75,17 @@ void KReportChartView::drawPivotChart(const PivotGrid &grid, const MyMoneyReport
 
     //remove existing headers
     const HeaderFooterList hfList = headerFooters();
-    foreach (const auto hf, hfList)
+    for (const auto& hf : hfList)
         delete hf;
 
     //remove existing legends
     const LegendList lgList = legends();
-    foreach (const auto lg, lgList)
+    for (const auto& lg : lgList)
         delete lg;
 
     //make sure the model is clear
     m_model.clear();
-    const bool blocked = m_model.blockSignals(true); // don't emit dataChanged() signal during each drawPivotRowSet
+    const bool blocked = m_model.blockSignals(true); // don't Q_EMIT dataChanged() signal during each drawPivotRowSet
 
     //set the new header
     HeaderFooter* header = new HeaderFooter;
@@ -283,15 +283,17 @@ void KReportChartView::drawPivotChart(const PivotGrid &grid, const MyMoneyReport
 
     int eBudgetDiffIdx = rowTypeList.indexOf(eBudgetDiff);
     QList<ERowType> myRowTypeList = rowTypeList;
-    myRowTypeList.removeAt(eBudgetDiffIdx);
+    if (eBudgetDiffIdx != -1)
+        myRowTypeList.removeAt(eBudgetDiffIdx);
     QStringList myColumnTypeHeaderList = columnTypeHeaderList;
-    myColumnTypeHeaderList.removeAt(eBudgetDiffIdx);
+    if (eBudgetDiffIdx != -1)
+        myColumnTypeHeaderList.removeAt(eBudgetDiffIdx);
     int myRowTypeListSize = myRowTypeList.size();
     MyMoneyFile* file = MyMoneyFile::instance();
     int precision = MyMoneyMoney::denomToPrec(file->baseCurrency().smallestAccountFraction());
     int rowNum = 0;
     QStringList legendNames;
-    QMap<MyMoneyMoney, int> legendTotal;
+    QMultiMap<MyMoneyMoney, int> legendTotal;
 
     switch (config.detailLevel()) {
     case eMyMoney::Report::DetailLevel::None:
@@ -329,13 +331,13 @@ void KReportChartView::drawPivotChart(const PivotGrid &grid, const MyMoneyReport
 
                             //only show the column type in the header if there is more than one type
                             if (myRowTypeListSize > 1)
-                                legendText = QString(myColumnTypeHeaderList.at(i) + QLatin1Literal(" - ") + it_row.key().name());
+                                legendText = QString(myColumnTypeHeaderList.at(i) + QLatin1String(" - ") + it_row.key().name());
                             else
                                 legendText = it_row.key().name();
 
                             //set the legend text
                             legendNames.append(legendText);
-                            legendTotal.insertMulti(it_row.value().value(myRowTypeList.at(i)).m_total.abs(), rowNum);
+                            legendTotal.insert(it_row.value().value(myRowTypeList.at(i)).m_total.abs(), rowNum);
 
                             precision = myRowTypeList.at(i) == ePrice ? securityPrecision : currencyPrecision;
 
@@ -369,13 +371,13 @@ void KReportChartView::drawPivotChart(const PivotGrid &grid, const MyMoneyReport
 
                     //only show the column type in the header if there is more than one type
                     if (myRowTypeListSize > 1)
-                        legendText = QString(myColumnTypeHeaderList.at(i) + QLatin1Literal(" - ") + it_innergroup.key());
+                        legendText = QString(myColumnTypeHeaderList.at(i) + QLatin1String(" - ") + it_innergroup.key());
                     else
                         legendText = it_innergroup.key();
 
                     //set the legend text
                     legendNames.append(legendText);
-                    legendTotal.insertMulti((*it_innergroup).m_total.value(myRowTypeList.at(i)).m_total.abs(), rowNum);
+                    legendTotal.insert((*it_innergroup).m_total.value(myRowTypeList.at(i)).m_total.abs(), rowNum);
 
                     //set the cell value and tooltip
                     rowNum = drawPivotGridRow(rowNum, (*it_innergroup).m_total.value(myRowTypeList.at(i)),
@@ -401,13 +403,13 @@ void KReportChartView::drawPivotChart(const PivotGrid &grid, const MyMoneyReport
 
                 //only show the column type in the header if there is more than one type
                 if (myRowTypeListSize > 1)
-                    legendText = QString(myColumnTypeHeaderList.at(i) + QLatin1Literal(" - ") + it_outergroup.key());
+                    legendText = QString(myColumnTypeHeaderList.at(i) + QLatin1String(" - ") + it_outergroup.key());
                 else
                     legendText = it_outergroup.key();
 
                 //set the legend text
                 legendNames.append(legendText);
-                legendTotal.insertMulti((*it_outergroup).m_total.value(myRowTypeList.at(i)).m_total.abs(), rowNum);
+                legendTotal.insert((*it_outergroup).m_total.value(myRowTypeList.at(i)).m_total.abs(), rowNum);
 
                 //set the cell value and tooltip
                 rowNum = drawPivotGridRow(rowNum, (*it_outergroup).m_total.value(myRowTypeList.at(i)),
@@ -425,13 +427,13 @@ void KReportChartView::drawPivotChart(const PivotGrid &grid, const MyMoneyReport
 
                 //only show the column type in the header if there is more than one type
                 if (myRowTypeListSize > 1)
-                    legendText = QString(myColumnTypeHeaderList.at(i) + QLatin1Literal(" - ") + i18nc("Total balance", "Total"));
+                    legendText = QString(myColumnTypeHeaderList.at(i) + QLatin1String(" - ") + i18nc("Total balance", "Total"));
                 else
                     legendText = QString(i18nc("Total balance", "Total"));
 
                 //set the legend text
                 legendNames.append(legendText);
-                legendTotal.insertMulti(grid.m_total.value(myRowTypeList.at(i)).m_total.abs(), rowNum);
+                legendTotal.insert(grid.m_total.value(myRowTypeList.at(i)).m_total.abs(), rowNum);
 
                 //set the cell value and tooltip
                 rowNum = drawPivotGridRow(rowNum, grid.m_total.value(myRowTypeList.at(i)),
@@ -449,33 +451,49 @@ void KReportChartView::drawPivotChart(const PivotGrid &grid, const MyMoneyReport
 
             //only show the column type in the header if there is more than one type
             if (myRowTypeListSize > 1)
-                legendText = QString(myColumnTypeHeaderList.at(i) + QLatin1Literal(" - ") + i18nc("Total balance", "Total"));
+                legendText = QString(myColumnTypeHeaderList.at(i) + QLatin1String(" - ") + i18nc("Total balance", "Total"));
             else
                 legendText = QString(i18nc("Total balance", "Total"));
 
             //set the legend text
             legendNames.append(legendText);
-            legendTotal.insertMulti(grid.m_total.value(myRowTypeList.at(i)).m_total.abs(), rowNum);
+            legendTotal.insert(grid.m_total.value(myRowTypeList.at(i)).m_total.abs(), rowNum);
 
             //set the cell value and tooltip
             if (config.isMixedTime()) {
                 if (myRowTypeList.at(i) == eActual)
-                    rowNum = drawPivotGridRow(rowNum, grid.m_total.value(myRowTypeList.at(i)),
+                    rowNum = drawPivotGridRow(rowNum,
+                                              grid.m_total.value(myRowTypeList.at(i)),
                                               config.isChartDataLabels() ? legendText : QString(),
-                                              0, config.currentDateColumn(), precision, false);
+                                              0,
+                                              config.currentDateColumn(),
+                                              precision,
+                                              config.isNegExpenses());
                 else if (myRowTypeList.at(i)== eForecast) {
-                    rowNum = drawPivotGridRow(rowNum, grid.m_total.value(myRowTypeList.at(i)),
+                    rowNum = drawPivotGridRow(rowNum,
+                                              grid.m_total.value(myRowTypeList.at(i)),
                                               config.isChartDataLabels() ? legendText : QString(),
-                                              config.currentDateColumn(), numberColumns - config.currentDateColumn(), precision, false);
+                                              config.currentDateColumn(),
+                                              numberColumns - config.currentDateColumn(),
+                                              precision,
+                                              config.isNegExpenses());
 
                 } else
-                    rowNum = drawPivotGridRow(rowNum, grid.m_total.value(myRowTypeList.at(i)),
+                    rowNum = drawPivotGridRow(rowNum,
+                                              grid.m_total.value(myRowTypeList.at(i)),
                                               config.isChartDataLabels() ? legendText : QString(),
-                                              0, numberColumns, precision, false);
+                                              0,
+                                              numberColumns,
+                                              precision,
+                                              config.isNegExpenses());
             } else
-                rowNum = drawPivotGridRow(rowNum, grid.m_total.value(myRowTypeList.at(i)),
+                rowNum = drawPivotGridRow(rowNum,
+                                          grid.m_total.value(myRowTypeList.at(i)),
                                           config.isChartDataLabels() ? legendText : QString(),
-                                          0, numberColumns, precision, false);
+                                          0,
+                                          numberColumns,
+                                          precision,
+                                          config.isNegExpenses());
         }
 
     }
@@ -508,7 +526,7 @@ void KReportChartView::drawPivotChart(const PivotGrid &grid, const MyMoneyReport
     // Set up X axis labels (ie "abscissa" to use the technical term)
     if (accountSeries()) { // if not, we will set these up while putting in the chart values.
         QStringList xLabels;
-        foreach (const auto colHeading, columnHeadings)
+        for (const auto& colHeading : columnHeadings)
             xLabels.append(QString(colHeading).replace(QLatin1String("&nbsp;"), QLatin1String(" ")));
         m_model.setVerticalHeaderLabels(xLabels);
     }
@@ -521,7 +539,7 @@ void KReportChartView::drawPivotChart(const PivotGrid &grid, const MyMoneyReport
         for (int i = 0 ; i < rowNum ; ++i) {
             QPen pen = diagramAttributes->headerData(i, Qt::Horizontal, DatasetPenRole).value< QPen >();
             pen.setWidth(penWidth);
-            m_model.setHeaderData(i, Qt::Horizontal, qVariantFromValue(pen), DatasetPenRole);
+            m_model.setHeaderData(i, Qt::Horizontal, QVariant::fromValue(pen), DatasetPenRole);
         }
     }
 
@@ -757,7 +775,7 @@ void KReportChartView::drawLimitLine(const double limit)
 {
     if (coordinatePlane()->diagram()->datasetDimension() != 1)
         return;
-    // temporarily disconnect the view from the model to aovid update of view on
+    // temporarily disconnect the view from the model to avoid update of view on
     // emission of the dataChanged() signal for each call of setDataCell().
     // This speeds up the runtime of drawLimitLine() by a factor of
     // approx. 60 on my box (1831ms vs. 31ms).
