@@ -2,6 +2,7 @@
     SPDX-FileCopyrightText: 2004-2005 Ace Jones <acejones@users.sourceforge.net>
     SPDX-FileCopyrightText: 2008-2011 Alvaro Soliverez <asoliverez@gmail.com>
     SPDX-FileCopyrightText: 2017-2018 Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
+    SPDX-FileCopyrightText: 2021 Dawid Wróbel <me@dawidwrobel.com>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
@@ -20,18 +21,22 @@
 // This is just needed for i18n().  Once I figure out how to handle i18n
 // without using this macro directly, I'll be freed of KDE dependency.
 
+#include <alkimia/alkdom.h>
+
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include "mymoneymoney.h"
-#include "mymoneyfile.h"
+#include "kmymoneysettings.h"
 #include "mymoneyaccount.h"
+#include "mymoneyenums.h"
+#include "mymoneyfile.h"
+#include "mymoneymoney.h"
+#include "mymoneyreport.h"
 #include "mymoneysecurity.h"
 #include "mymoneysplit.h"
 #include "mymoneytransaction.h"
-#include "mymoneyreport.h"
-#include "kmymoneysettings.h"
-#include "mymoneyenums.h"
+#include "mymoneyutils.h"
+#include "qmetaobject.h"
 
 namespace reports
 {
@@ -47,7 +52,7 @@ QVector<ListTable::cellTypeE> ListTable::TableRow::m_sortCriteria;
 bool ListTable::TableRow::operator< (const TableRow& _compare) const
 {
     bool result = false;
-    foreach (const auto criterion, m_sortCriteria) {
+    for (const auto& criterion : qAsConst(m_sortCriteria)) {
         if (this->operator[](criterion) < _compare[criterion]) {
             result = true;
             break;
@@ -107,12 +112,12 @@ void ListTable::render(QString& result, QString& csv) const
     if (!m_postcolumns.isEmpty()) // prevent creation of empty column
         columns.append(postcolumns);
 
-    result.append(QLatin1String("<table class=\"report\">\n<thead><tr class=\"itemheader\">"));
+    result.append(QLatin1String("<table align=\"center\" class=\"report\">\n<thead><tr class=\"itemheader\">"));
 
     //
     // Table header
     //
-    foreach (const auto cellType, columns) {
+    for (const auto& cellType : qAsConst(columns)) {
         result.append(QString::fromLatin1("<th>%1</th>").arg(tableHeader(cellType)));
         csv.append(tableHeader(cellType) + QLatin1Char(','));
     }
@@ -134,12 +139,11 @@ void ListTable::render(QString& result, QString& csv) const
     bool row_odd = true;
     bool isLowestGroupTotal = true;  // hack to inform whether to put separator line or not
 
+    result.append(QLatin1String("<tbody>\n"));
     // ***DV***
     MyMoneyMoney startingBalance;
     MyMoneyMoney balanceChange = MyMoneyMoney();
-    for (QList<TableRow>::ConstIterator it_row = m_rows.constBegin();
-            it_row != m_rows.constEnd();
-            ++it_row) {
+    for (QList<TableRow>::const_iterator it_row = m_rows.cbegin(); it_row != m_rows.cend(); ++it_row) {
         /* rank can be:
          * 0 - opening balance
          * 1 - major split of transaction
@@ -192,6 +196,7 @@ void ListTable::render(QString& result, QString& csv) const
                     tlink = QString::fromLatin1("id=%1&tid=%2").arg((*it_row).value(ctAccountID), (*it_row).value(ctID));
                 }
                 result.append(QString::fromLatin1("<tr class=\"row-%1\">").arg(row_odd ? QLatin1String("odd") : QLatin1String("even")));
+
             } else if (rowRank == 2) {
                 result.append(QString::fromLatin1("<tr class=\"item%1\">").arg(row_odd ? QLatin1Char('1') : QLatin1Char('0')));
             } else if (rowRank == 4 || rowRank == 5) {
@@ -232,10 +237,26 @@ void ListTable::render(QString& result, QString& csv) const
         // Columns
         //
 
-        QList<cellTypeE>::ConstIterator it_column = columns.constBegin();
-        while (it_column != columns.constEnd()) {
+        QList<cellTypeE>::const_iterator it_column = columns.cbegin();
+        int skippedColumn = 0;
+        QString tempResult;
+        while (it_column != columns.cend()) {
             QString data = (*it_row).value(*it_column);
 
+            // include a colspan information if we skipped some
+            // columns while processing the total and subtotal rows
+            auto flushTempResult = [&skippedColumn, &tempResult]() {
+                QString finishedResult(tempResult);
+                if (!tempResult.isEmpty()) {
+                    if (skippedColumn > 0) {
+                        finishedResult.replace(QLatin1String("<td"), QString::fromLatin1("<td colspan=\"%1\"").arg(skippedColumn + 1));
+                        skippedColumn = 0;
+                    }
+                    tempResult.clear();
+                }
+                return finishedResult;
+            };
+
             // ***DV***
             if (rowRank == 2) {
                 if (*it_column == ctValue)
@@ -288,24 +309,33 @@ void ListTable::render(QString& result, QString& csv) const
                 data = (balanceChange + startingBalance).toString();
             } else if ((rowRank == 4 || rowRank == 5)) {
                 // display total title but only if first column doesn't contain any data
-                if (it_column == columns.constBegin() && data.isEmpty()) {
-                    result.append(QString::fromLatin1("<td class=\"left%1\">").arg((*it_row).value(ctDepth)));
+                if (it_column == columns.cbegin() && data.isEmpty()) {
+                    tempResult.append(QString::fromLatin1("<td class=\"left%1\">").arg((*it_row).value(ctDepth)));
                     if (rowRank == 4) {
-                        if (!(*it_row).value(ctDepth).isEmpty())
-                            result += i18nc("Total balance", "Total") + QLatin1Char(' ') + prevGrpNames.at((*it_row).value(ctDepth).toInt());
-                        else
-                            result += i18n("Grand Total");
+                        if (!(*it_row).value(ctDepth).isEmpty()) {
+                            tempResult += i18nc("Total balance", "Total") + QLatin1Char(' ') + prevGrpNames.at((*it_row).value(ctDepth).toInt());
+                            csv.append(i18nc("Total balance", "Total") + QLatin1Char(' ') + prevGrpNames.at((*it_row).value(ctDepth).toInt())
+                                       + QLatin1Char(','));
+                        } else {
+                            tempResult += i18n("Grand Total");
+                            csv.append(i18n("Grand Total") + QLatin1Char(','));
+                        }
                     }
-                    result.append(QLatin1String("</td>"));
+                    tempResult.append(QLatin1String("</td>"));
                     ++it_column;
                     continue;
-                } else if (!m_subtotal.contains(*it_column)) {  // don't display e.g. account in totals row
-                    result.append(QLatin1String("<td></td>"));
+
+                } else if (!m_subtotal.contains(*it_column)) { // don't display e.g. account in totals row
+                    ++skippedColumn;
+                    csv.append(QLatin1Char(','));
                     ++it_column;
                     continue;
                 }
             }
 
+            // append any pending total or subtotal header
+            result += flushTempResult();
+
             // Figure out how to render the value in this column, depending on
             // what its properties are.
             //
@@ -314,7 +344,7 @@ void ListTable::render(QString& result, QString& csv) const
             // vector of a properties class.
             QString tlinkBegin, tlinkEnd;
             if (!tlink.isEmpty()) {
-                tlinkBegin = QString::fromLatin1("<a href=ledger?%1>").arg(tlink);
+                tlinkBegin = QString::fromLatin1("<a href=\"/ledger?%1\">").arg(tlink);
                 tlinkEnd = QLatin1String("</a>");
             }
 
@@ -366,7 +396,7 @@ void ListTable::render(QString& result, QString& csv) const
                 } else {
                     auto value = MyMoneyMoney(data) * MyMoneyMoney(100, 1);
                     auto valueStr = value.formatMoney(fraction);
-                    csv.append(QString::fromLatin1("%1%,").arg(valueStr));
+                    csv.append(QString::fromLatin1("\"%1 %\",").arg(valueStr));
 
                     QString colorBegin;
                     QString colorEnd;
@@ -378,7 +408,7 @@ void ListTable::render(QString& result, QString& csv) const
                     if ((rowRank == 4 || rowRank == 5) && value.isNegative())
                         valueStr = QString::fromLatin1("<font color=%1>%2</font>")
                                    .arg(KMyMoneySettings::schemeColor(SchemeColor::Negative).name(), valueStr);
-                    result.append(QString::fromLatin1("<td>%2%4%1%%5%3</td>").arg(valueStr, tlinkBegin, tlinkEnd, colorBegin, colorEnd));
+                    result.append(QString::fromLatin1("<td>%2%4%1 %%5%3</td>").arg(valueStr, tlinkBegin, tlinkEnd, colorBegin, colorEnd));
                 }
                 break;
             case cgPrice:
@@ -409,12 +439,12 @@ void ListTable::render(QString& result, QString& csv) const
                 // if we have a locale() then use its date formatter
                 if (!data.isEmpty()) {
                     QDate qd = QDate::fromString(data, Qt::ISODate);
-                    data = QLocale().toString(qd, QLocale::ShortFormat);
+                    data = MyMoneyUtils::formatDate(qd);
                 }
                 result.append(QString::fromLatin1("<td class=\"left%4\">%2%1%3</td>").arg(data, tlinkBegin, tlinkEnd, QString::number(prevGrpNames.count() - 1)));
                 break;
             default:
-                result.append(QString::fromLatin1("<td class=\"left%4\">%2%1%3</td>").arg(data, tlinkBegin, tlinkEnd, QString::number(prevGrpNames.count() - 1)));
+                result.append(QString::fromLatin1("<td class=\"left0\">%2%1%3</td>").arg(data, tlinkBegin, tlinkEnd));
                 csv.append(QString::fromLatin1("\"%1\",").arg(data));
                 break;
             }
@@ -426,7 +456,7 @@ void ListTable::render(QString& result, QString& csv) const
         csv.chop(1);  // remove final comma
         csv.append(QLatin1Char('\n'));
     }
-    result.append(QLatin1String("</table>\n"));
+    result.append(QLatin1String("</tbody>\n</table>\n"));
 }
 
 QString ListTable::renderHTML() const
@@ -455,6 +485,37 @@ void ListTable::dump(const QString& file, const QString& context) const
     g.close();
 }
 
+bool ListTable::saveToXml(const QString& file)
+{
+    QFile out(file);
+    if (!out.open(QIODevice::WriteOnly))
+        return false;
+    QTextStream stream(&out);
+    stream << toXml();
+    return true;
+}
+
+QString ListTable::toXml() const
+{
+    AlkDomDocument doc;
+    AlkDomElement el = doc.createElement("ListTable");
+    QString name = m_config.name();
+    el.setAttribute("name", name);
+
+    for (const ListTable::TableRow& row : m_rows) {
+        AlkDomElement r = doc.createElement("TableRow");
+        QList<cellTypeE> keys = row.keys();
+        for (const cellTypeE& key : keys) {
+            QMetaEnum metaEnum = QMetaEnum::fromType<cellTypeE>();
+            QString attrName = metaEnum.valueToKey(key);
+            r.setAttribute(attrName, row[key]);
+        }
+        el.appendChild(r);
+    }
+    doc.appendChild(el);
+    return doc.toString();
+}
+
 void ListTable::includeInvestmentSubAccounts()
 {
     // if we're not in expert mode, we need to make sure
@@ -478,17 +539,18 @@ void ListTable::includeInvestmentSubAccounts()
         QList<MyMoneyAccount> accountList;
         file->accountList(accountList);
         QList<MyMoneyAccount>::const_iterator it_ma;
-        for (it_ma = accountList.constBegin(); it_ma != accountList.constEnd(); ++it_ma) {
+        for (it_ma = accountList.cbegin(); it_ma != accountList.cend(); ++it_ma) {
             if ((*it_ma).accountType() == eMyMoney::Account::Type::Investment) {
                 accountIdList.append((*it_ma).id());
             }
         }
     }
 
-    foreach (const auto sAccount, accountIdList) {
-        auto acc = file->account(sAccount);
+    for (const auto& sAccount : qAsConst(accountIdList)) {
+        const auto acc = file->account(sAccount);
         if (acc.accountType() == eMyMoney::Account::Type::Investment) {
-            foreach (const auto sSubAccount, acc.accountList()) {
+            const auto subAccountList = acc.accountList();
+            for (const auto& sSubAccount : qAsConst(subAccountList)) {
                 if (!accountIdList.contains(sSubAccount)) {
                     subAccountsList.append(sSubAccount);
                 }
@@ -516,13 +578,14 @@ void ListTable::includeInvestmentSubAccounts()
         filter.addAccount(subAccountsList);
         filter.setReportAllSplits(false);
 
-        QList<MyMoneyTransaction> transactions = file->transactionList(filter);
-        QList<MyMoneyTransaction>::const_iterator it_t = transactions.constBegin();
+        QList<MyMoneyTransaction> transactions;
+        file->transactionList(transactions, filter);
+        QList<MyMoneyTransaction>::const_iterator it_t = transactions.cbegin();
 
         //Check each split for a matching account
-        for (; it_t != transactions.constEnd(); ++it_t) {
+        for (; it_t != transactions.cend(); ++it_t) {
             const QList<MyMoneySplit>& splits = (*it_t).splits();
-            foreach (const auto split, splits) {
+            for (const auto& split : splits) {
                 const QString& accountId = split.accountId();
                 if (!split.shares().isZero()
                         && subAccountsList.contains(accountId)) {
@@ -580,7 +643,7 @@ ListTable::cellGroupE ListTable::cellGroup(const cellTypeE cellType)
     case ctShares:
         return cgShares;
     // the list of columns which represent a percentage, so we can display them correctly
-    case ctReturn:
+    case ctExtendedInternalRateOfReturn:
     case ctReturnInvestment:
     case ctInterestRate:
     case ctPercentageGain:
@@ -661,8 +724,8 @@ QString ListTable::tableHeader(const cellTypeE cellType)
         return i18n("Ending Balance");
     case ctMarketValue:
         return i18n("Market Value");
-    case ctReturn:
-        return i18n("Annualized Return");
+    case ctExtendedInternalRateOfReturn:
+        return i18n("Extended internal rate of return");
     case ctReturnInvestment:
         return i18n("Return On Investment");
     case ctFees:
