@@ -43,42 +43,103 @@ class ReportAccount;
 
 class ListTable : public ReportTable
 {
+    Q_OBJECT
 public:
     explicit ListTable(const MyMoneyReport&);
     QString renderHTML() const final override;
     QString renderCSV() const final override;
     void drawChart(KReportChartView&) const final override {}
     void dump(const QString& file, const QString& context = QString()) const final override;
+    bool saveToXml(const QString& file) override;
+    QString toXml() const override;
     void init();
 
 public:
     enum cellTypeE /*{*/ /*Money*/
-    {   ctValue, ctNetInvValue, ctMarketValue,
-        ctPrice, ctLastPrice, ctBuyPrice,
-        ctBuys, ctSells, ctBuysST, ctSellsST, ctBuysLT, ctSellsLT,
-        ctCapitalGain, ctCapitalGainST,ctCapitalGainLT,
-        ctCashIncome, ctReinvestIncome,
-        ctFees, ctInterest,
-        ctStartingBalance, ctEndingBalance, ctBalance, ctCurrentBalance,
-        ctBalanceWarning, ctMaxBalanceLimit, ctOpeningBalance,
-        ctCreditWarning, ctMaxCreditLimit,
-        ctLoanAmount, ctPeriodicPayment, ctFinalPayment, ctPayment,
+    {
+        ctValue,
+        ctNetInvValue,
+        ctMarketValue,
+        ctPrice,
+        ctLastPrice,
+        ctBuyPrice,
+        ctBuys,
+        ctSells,
+        ctBuysST,
+        ctSellsST,
+        ctBuysLT,
+        ctSellsLT,
+        ctCapitalGain,
+        ctCapitalGainST,
+        ctCapitalGainLT,
+        ctCashIncome,
+        ctReinvestIncome,
+        ctFees,
+        ctInterest,
+        ctStartingBalance,
+        ctEndingBalance,
+        ctBalance,
+        ctCurrentBalance,
+        ctBalanceWarning,
+        ctMaxBalanceLimit,
+        ctOpeningBalance,
+        ctCreditWarning,
+        ctMaxCreditLimit,
+        ctLoanAmount,
+        ctPeriodicPayment,
+        ctFinalPayment,
+        ctPayment,
         /*Shares*/
         ctShares,
         /*Percent*/
-        ctReturn, ctReturnInvestment, ctInterestRate, ctPercentageGain,
+        ctExtendedInternalRateOfReturn,
+        ctReturnInvestment,
+        ctInterestRate,
+        ctPercentageGain,
         /*Date*/
-        ctPostDate, ctEntryDate, ctNextDueDate, ctOpeningDate, ctNextInterestChange,
-        ctMonth, ctWeek, ctReconcileDate,
+        ctPostDate,
+        ctEntryDate,
+        ctNextDueDate,
+        ctOpeningDate,
+        ctNextInterestChange,
+        ctMonth,
+        ctWeek,
+        ctReconcileDate,
         /*Misc*/
-        ctCurrency, ctCurrencyName, ctCommodity, ctID,
-        ctRank, ctSplit, ctMemo,
-        ctAccount, ctAccountID, ctTopAccount, ctInvestAccount, ctInstitution,
-        ctCategory, ctTopCategory, ctCategoryType,
-        ctNumber, ctReconcileFlag,
-        ctAction, ctTag, ctPayee, ctEquityType, ctType, ctName,
-        ctDepth, ctRowsCount, ctTax, ctFavorite, ctDescription, ctOccurrence, ctPaymentType,
+        ctCurrency,
+        ctCurrencyName,
+        ctCommodity,
+        ctID,
+        ctRank,
+        ctSplit,
+        ctMemo,
+        ctAccount,
+        ctAccountID,
+        ctTopAccount,
+        ctInvestAccount,
+        ctInstitution,
+        ctCategory,
+        ctTopCategory,
+        ctCategoryType,
+        ctNumber,
+        ctReconcileFlag,
+        ctAction,
+        ctTag,
+        ctPayee,
+        ctEquityType,
+        ctType,
+        ctName,
+        ctDepth,
+        ctRowsCount,
+        ctTax,
+        ctFavorite,
+        ctDescription,
+        ctOccurrence,
+        ctPaymentType,
+        ctValueSourceLine
     };
+    Q_ENUM(cellTypeE)
+
     /**
       * Contains a single row in the table.
       *
