@@ -6,6 +6,8 @@
 
 #include "pivotgrid-test.h"
 
+#include <KLocalizedString>
+
 #include <QTest>
 
 #include "tests/testutilities.h"
@@ -21,11 +23,14 @@ using namespace test;
 
 QTEST_GUILESS_MAIN(PivotGridTest)
 
+void PivotGridTest::initTestCase()
+{
+    KLocalizedString::setApplicationDomain(QByteArrayLiteral("kmymoney"));
+}
+
 void PivotGridTest::init()
 {
-    storage = new MyMoneyStorageMgr;
     file = MyMoneyFile::instance();
-    file->attachStorage(storage);
 
     MyMoneyFileTransaction ft;
     file->addCurrency(MyMoneySecurity("CAD", "Canadian Dollar",        "C$"));
@@ -64,8 +69,6 @@ void PivotGridTest::init()
 
 void PivotGridTest::cleanup()
 {
-    file->detachStorage(storage);
-    delete storage;
 }
 
 void PivotGridTest::testCellAddValue()
