@@ -17,7 +17,6 @@ class PivotGridTest;
 #define KMM_MYMONEY_UNIT_TESTABLE friend class reports::PivotGridTest;
 
 #include "mymoneyfile.h"
-#include "mymoneystoragemgr.h"
 
 namespace reports
 {
@@ -25,11 +24,12 @@ namespace reports
 class PivotGridTest : public QObject
 {
     Q_OBJECT
+
 private:
-    MyMoneyStorageMgr* storage;
     MyMoneyFile* file;
 
 private Q_SLOTS:
+    void initTestCase();
     void init();
     void cleanup();
     void testCellAddValue();
