@@ -6,8 +6,11 @@
 
 #include "pivottable-test.h"
 
-#include <QList>
+#include <KLocalizedString>
+
 #include <QFile>
+#include <QList>
+#include <QRegularExpression>
 #include <QTest>
 #include <QTextCodec>
 
@@ -33,29 +36,32 @@ using namespace test;
 
 QTEST_GUILESS_MAIN(PivotTableTest)
 
-void writeTabletoHTML(const PivotTable& table, const QString& _filename = QString())
+void writeTabletoHTML(const PivotTable& table, const QString& basename = QString())
 {
     static unsigned filenumber = 1;
-    QString filename = _filename;
-    if (filename.isEmpty()) {
-        filename = QString::fromLatin1("report-%1.html").arg(filenumber, 2, 10,QLatin1Char('0'));
+    QString filename;
+    if (basename.isEmpty()) {
+        filename = QString::fromLatin1("%1/report-%2.html").arg(CMAKE_CURRENT_BINARY_DIR).arg(filenumber, 2, 10, QLatin1Char('0'));
         ++filenumber;
+    } else {
+        filename = QString::fromLatin1("%1/%2.html").arg(CMAKE_CURRENT_BINARY_DIR).arg(basename);
     }
 
     QFile g(filename);
     g.open(QIODevice::WriteOnly);
     QTextStream(&g) << table.renderHTML();
     g.close();
-
 }
 
-void writeTabletoCSV(const PivotTable& table, const QString& _filename = QString())
+void writeTabletoCSV(const PivotTable& table, const QString& basename = QString())
 {
     static unsigned filenumber = 1;
-    QString filename = _filename;
-    if (filename.isEmpty()) {
-        filename = QString::fromLatin1("report-%1.csv").arg(filenumber, 2, 10,QLatin1Char('0'));
+    QString filename;
+    if (basename.isEmpty()) {
+        filename = QString::fromLatin1("%1/report-%2.csv").arg(CMAKE_CURRENT_BINARY_DIR).arg(filenumber, 2, 10, QLatin1Char('0'));
         ++filenumber;
+    } else {
+        filename = QString::fromLatin1("%1/%2.csv").arg(CMAKE_CURRENT_BINARY_DIR).arg(basename);
     }
 
     QFile g(filename);
@@ -64,15 +70,39 @@ void writeTabletoCSV(const PivotTable& table, const QString& _filename = QString
     g.close();
 }
 
-void PivotTableTest::setup()
+void writeTabletoXML(const PivotTable& table, const QString& basename = QString())
 {
+    static unsigned filenumber = 1;
+    QString filename;
+    if (basename.isEmpty()) {
+        filename = QString::fromLatin1("%1/report-%2.xml").arg(CMAKE_CURRENT_BINARY_DIR).arg(filenumber, 2, 10, QLatin1Char('0'));
+        ++filenumber;
+    } else {
+        filename = QString::fromLatin1("%1/%2.xml").arg(CMAKE_CURRENT_BINARY_DIR).arg(basename);
+    }
+
+    QFile g(filename);
+    g.open(QIODevice::WriteOnly);
+    QTextStream(&g) << table.toXml();
+    g.close();
+}
+
+void writeTable(const PivotTable& table, const QString& basename = QString())
+{
+    writeTabletoCSV(table, basename);
+    writeTabletoHTML(table, basename);
+    writeTabletoXML(table, basename);
+}
+
+void PivotTableTest::initTestCase()
+{
+    KLocalizedString::setApplicationDomain(QByteArrayLiteral("kmymoney"));
 }
 
 void PivotTableTest::init()
 {
-    storage = new MyMoneyStorageMgr;
     file = MyMoneyFile::instance();
-    file->attachStorage(storage);
+    file->unload();
 
     MyMoneyFileTransaction ft;
     file->addCurrency(MyMoneySecurity("CAD", "Canadian Dollar",        "C$"));
@@ -111,19 +141,18 @@ void PivotTableTest::init()
 
 void PivotTableTest::cleanup()
 {
-    file->detachStorage(storage);
-    delete storage;
 }
 
 void PivotTableTest::testNetWorthSingle()
 {
     try {
-        MyMoneyReport filter;
+        MyMoneyReport filter(QLatin1String("fake-id"));
         filter.setRowType(eMyMoney::Report::RowType::AssetLiability);
         filter.setDateFilter(QDate(2004, 1, 1), QDate(2004, 7, 1).addDays(-1));
+        filter.setName("Net worth single");
         XMLandback(filter);
         PivotTable networth_f(filter);
-        writeTabletoCSV(networth_f);
+        writeTable(networth_f, filter.name());
 
         QVERIFY(networth_f.m_grid["Asset"]["Checking Account"][ReportAccount(acChecking)][eActual][5] == moCheckingOpen);
         QVERIFY(networth_f.m_grid["Asset"]["Checking Account"][ReportAccount(acChecking)][eActual][6] == moCheckingOpen);
@@ -143,7 +172,7 @@ void PivotTableTest::testNetWorthOfsetting()
     // accounts opened during the period of the report, one asset & one liability.  Test
     // that it calculates the totals correctly.
 
-    MyMoneyReport filter;
+    MyMoneyReport filter(QLatin1String("fake-id"));
     filter.setRowType(eMyMoney::Report::RowType::AssetLiability);
     filter.setDateFilter(QDate(2004, 1, 1), QDate(2005, 1, 1).addDays(-1));
     XMLandback(filter);
@@ -159,13 +188,13 @@ void PivotTableTest::testNetWorthOpeningPrior()
     // Test the net worth report to make sure it's picking up opening balances PRIOR to
     // the period of the report.
 
-    MyMoneyReport filter;
+    MyMoneyReport filter(QLatin1String("fake-id"));
     filter.setRowType(eMyMoney::Report::RowType::AssetLiability);
     filter.setDateFilter(QDate(2005, 8, 1), QDate(2005, 12, 31));
     filter.setName("Net Worth Opening Prior 1");
     XMLandback(filter);
     PivotTable networth_f(filter);
-    writeTabletoCSV(networth_f);
+    writeTable(networth_f, filter.name());
 
     QVERIFY(networth_f.m_grid["Liability"]["Credit Card"].m_total[eActual][0] == -moCreditOpen);
     QVERIFY(networth_f.m_grid["Asset"]["Checking Account"].m_total[eActual][0] == moCheckingOpen);
@@ -181,7 +210,7 @@ void PivotTableTest::testNetWorthOpeningPrior()
 
     filter.setName("Net Worth Opening Prior 2");
     PivotTable networth_f2(filter);
-    writeTabletoCSV(networth_f2);
+    writeTable(networth_f2, filter.name());
 
     MyMoneyMoney m1 = (networth_f2.m_grid["Liability"]["Credit Card"].m_total[eActual][1]);
     MyMoneyMoney m2 = (-moCreditOpen + moParent);
@@ -196,7 +225,7 @@ void PivotTableTest::testNetWorthDateFilter()
     // Test a net worth report whose period is prior to the time any accounts are open,
     // so the report should be zero.
 
-    MyMoneyReport filter;
+    MyMoneyReport filter(QLatin1String("fake-id"));
     filter.setRowType(eMyMoney::Report::RowType::AssetLiability);
     filter.setDateFilter(QDate(2004, 1, 1), QDate(2004, 2, 1).addDays(-1));
     XMLandback(filter);
@@ -226,34 +255,40 @@ void PivotTableTest::testNetWorthOpening()
     TransactionHelper t12(QDate(2017, 6, 1), MyMoneySplit::actionName(eMyMoney::Split::Action::Deposit), MyMoneyMoney(-100000), acBasicAccount, ctBasicIncome);
     TransactionHelper t13(QDate(2017, 7, 1), MyMoneySplit::actionName(eMyMoney::Split::Action::Deposit), MyMoneyMoney(-100000), acBasicAccount, ctBasicIncome);
 
-    MyMoneyReport filter;
+    MyMoneyReport filter(QLatin1String("fake-id"));
     filter.setRowType(eMyMoney::Report::RowType::AssetLiability);
     filter.setDateFilter(QDate(2016, 1, 1), QDate(2017, 12, 31));
     filter.addAccount(acBasicAccount);
+    filter.setName("Networth opening 1");
     XMLandback(filter);
     PivotTable nt_opening1(filter);
-    writeTabletoCSV(nt_opening1, "networth-opening-1.csv");
+    writeTable(nt_opening1, filter.name());
 
-    QVERIFY(nt_opening1.m_grid["Asset"]["Basic Account"][ReportAccount(acBasicAccount)][eActual][0] == openingBalance); // opening value on 1st Jan 2016 is 12000000, but before that i.e. 31st Dec 2015 opening value is 0
+    QCOMPARE(nt_opening1.m_grid["Asset"]["Basic Account"][ReportAccount(acBasicAccount)][eActual][0].toDouble(),
+             openingBalance.toDouble()); // opening value on 1st Jan 2016 is 12000000, but before that i.e. 31st Dec 2015 opening value is 0
     for (auto i = 1; i <= 6; ++i)
-        QVERIFY(nt_opening1.m_grid["Asset"]["Basic Account"][ReportAccount(acBasicAccount)][eActual][i] == openingBalance);
-    QVERIFY(nt_opening1.m_grid["Asset"]["Basic Account"][ReportAccount(acBasicAccount)][eActual][7] == openingBalance + MyMoneyMoney(6200000));
-    QVERIFY(nt_opening1.m_grid["Asset"]["Basic Account"][ReportAccount(acBasicAccount)][eActual][12] == MyMoneyMoney(18700000)); // value after t6 transaction
+        QCOMPARE(nt_opening1.m_grid["Asset"]["Basic Account"][ReportAccount(acBasicAccount)][eActual][i].toDouble(), openingBalance.toDouble());
+    QCOMPARE(nt_opening1.m_grid["Asset"]["Basic Account"][ReportAccount(acBasicAccount)][eActual][7].toDouble(),
+             (openingBalance + MyMoneyMoney(6200000)).toDouble());
+    QCOMPARE(nt_opening1.m_grid["Asset"]["Basic Account"][ReportAccount(acBasicAccount)][eActual][12].toDouble(),
+             MyMoneyMoney(18700000).toDouble()); // value after t6 transaction
 
     filter.setDateFilter(QDate(2017, 1, 1), QDate(2017, 12, 31));
+    filter.setName("Networth opening 2");
     XMLandback(filter);
     PivotTable nt_opening2(filter);
-    writeTabletoCSV(nt_opening2, "networth-opening-2.csv");
+    writeTable(nt_opening2, filter.name());
 
-    QVERIFY(nt_opening2.m_grid["Asset"]["Basic Account"][ReportAccount(acBasicAccount)][eActual][0] == MyMoneyMoney(18700000)); // opening value is equal to the value after t6 transaction
-    QVERIFY(nt_opening2.m_grid["Asset"]["Basic Account"][ReportAccount(acBasicAccount)][eActual][12] == MyMoneyMoney(14800000));
+    QCOMPARE(nt_opening2.m_grid["Asset"]["Basic Account"][ReportAccount(acBasicAccount)][eActual][0].toDouble(),
+             MyMoneyMoney(18700000).toDouble()); // opening value is equal to the value after t6 transaction
+    QCOMPARE(nt_opening2.m_grid["Asset"]["Basic Account"][ReportAccount(acBasicAccount)][eActual][12].toDouble(), MyMoneyMoney(14800000).toDouble());
 }
 
 void PivotTableTest::testSpendingEmpty()
 {
     // test a spending report with no entries
 
-    MyMoneyReport filter;
+    MyMoneyReport filter(QLatin1String("fake-id"));
     filter.setRowType(eMyMoney::Report::RowType::ExpenseIncome);
     XMLandback(filter);
     PivotTable spending_f1(filter);
@@ -269,13 +304,13 @@ void PivotTableTest::testSingleTransaction()
     // Test a single transaction
     TransactionHelper t(QDate(2004, 10, 31), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moSolo, acChecking, acSolo);
 
-    MyMoneyReport filter;
+    MyMoneyReport filter(QLatin1String("fake-id"));
     filter.setRowType(eMyMoney::Report::RowType::ExpenseIncome);
     filter.setDateFilter(QDate(2004, 9, 1), QDate(2005, 1, 1).addDays(-1));
-    filter.setName("Spending with Single Transaction.html");
+    filter.setName("Spending with Single Transaction");
     XMLandback(filter);
     PivotTable spending_f(filter);
-    writeTabletoHTML(spending_f, "Spending with Single Transaction.html");
+    writeTable(spending_f, filter.name());
 
     QVERIFY(spending_f.m_grid["Expense"]["Solo"][ReportAccount(acSolo)][eActual][1] == moSolo);
     QVERIFY(spending_f.m_grid["Expense"]["Solo"].m_total[eActual][1] == moSolo);
@@ -299,14 +334,14 @@ void PivotTableTest::testSubAccount()
     TransactionHelper t2(QDate(2004, 11, 7), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moParent2, acCredit, acParent);
     TransactionHelper t3(QDate(2004, 11, 7), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moChild, acCredit, acChild);
 
-    MyMoneyReport filter;
+    MyMoneyReport filter(QLatin1String("fake-id"));
     filter.setRowType(eMyMoney::Report::RowType::ExpenseIncome);
     filter.setDateFilter(QDate(2004, 9, 1), QDate(2005, 1, 1).addDays(-1));
     filter.setDetailLevel(eMyMoney::Report::DetailLevel::All);
     filter.setName("Spending with Sub-Account");
     XMLandback(filter);
     PivotTable spending_f(filter);
-    writeTabletoHTML(spending_f, "Spending with Sub-Account.html");
+    writeTable(spending_f, filter.name());
 
     QVERIFY(spending_f.m_grid["Expense"]["Parent"][ReportAccount(acParent)][eActual][2] == moParent);
     QVERIFY(spending_f.m_grid["Expense"]["Parent"][ReportAccount(acChild)][eActual][2] == moChild);
@@ -322,7 +357,8 @@ void PivotTableTest::testSubAccount()
     filter.setName("Net Worth with Sub-Account");
     XMLandback(filter);
     PivotTable networth_f(filter);
-    writeTabletoHTML(networth_f, "Net Worth with Sub-Account.html");
+    writeTable(networth_f, filter.name());
+
     QVERIFY(networth_f.m_grid["Liability"]["Credit Card"].m_total[eActual][3] == moParent + moChild - moCreditOpen);
     QVERIFY(networth_f.m_grid.m_total[eActual][4] == -moParent - moChild + moCreditOpen + moCheckingOpen);
 
@@ -335,7 +371,7 @@ void PivotTableTest::testFilterIEvsIE()
     TransactionHelper t2(QDate(2004, 11, 7), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moParent1, acCredit, acParent);
     TransactionHelper t3(QDate(2004, 11, 7), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moChild, acCredit, acChild);
 
-    MyMoneyReport filter;
+    MyMoneyReport filter(QLatin1String("fake-id"));
     filter.setRowType(eMyMoney::Report::RowType::ExpenseIncome);
     filter.setDateFilter(QDate(2004, 9, 1), QDate(2005, 1, 1).addDays(-1));
     filter.addCategory(acChild);
@@ -356,7 +392,7 @@ void PivotTableTest::testFilterALvsAL()
     TransactionHelper t2(QDate(2004, 11, 7), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moParent1, acCredit, acParent);
     TransactionHelper t3(QDate(2004, 11, 7), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moChild, acCredit, acChild);
 
-    MyMoneyReport filter;
+    MyMoneyReport filter(QLatin1String("fake-id"));
     filter.setRowType(eMyMoney::Report::RowType::AssetLiability);
     filter.setDateFilter(QDate(2004, 9, 1), QDate(2005, 1, 1).addDays(-1));
     filter.addAccount(acChecking);
@@ -374,11 +410,13 @@ void PivotTableTest::testFilterALvsIE()
     TransactionHelper t2(QDate(2004, 11, 7), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moParent1, acCredit, acParent);
     TransactionHelper t3(QDate(2004, 11, 7), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moChild, acCredit, acChild);
 
-    MyMoneyReport filter;
+    MyMoneyReport filter(QLatin1String("fake-id"));
+    QList<MyMoneyTransaction> list;
     filter.setRowType(eMyMoney::Report::RowType::ExpenseIncome);
     filter.setDateFilter(QDate(2004, 9, 1), QDate(2005, 1, 1).addDays(-1));
     filter.addAccount(acChecking);
-    QVERIFY(file->transactionList(filter).count() == 1);
+    file->transactionList(list, filter);
+    QVERIFY(list.count() == 1);
 
     XMLandback(filter);
     PivotTable spending_f(filter);
@@ -396,7 +434,7 @@ void PivotTableTest::testFilterAllvsIE()
     TransactionHelper t2(QDate(2004, 11, 7), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moParent1, acCredit, acParent);
     TransactionHelper t3(QDate(2004, 11, 7), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moChild, acCredit, acChild);
 
-    MyMoneyReport filter;
+    MyMoneyReport filter(QLatin1String("fake-id"));
     filter.setRowType(eMyMoney::Report::RowType::ExpenseIncome);
     filter.setDateFilter(QDate(2004, 9, 1), QDate(2005, 1, 1).addDays(-1));
     filter.addAccount(acCredit);
@@ -416,6 +454,7 @@ void PivotTableTest::testFilterBasics()
     TransactionHelper t3(QDate(2004, 11, 7), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moParent2, acCredit, acParent);
     TransactionHelper t4(QDate(2004, 11, 7), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moChild, acCredit, acChild);
 
+    QList<MyMoneyTransaction> list;
     MyMoneyTransactionFilter filter;
     filter.clear();
     filter.setDateFilter(QDate(2004, 9, 1), QDate(2005, 1, 1).addDays(-1));
@@ -423,15 +462,18 @@ void PivotTableTest::testFilterBasics()
     filter.setReportAllSplits(false);
     filter.setConsiderCategory(true);
 
-    QVERIFY(file->transactionList(filter).count() == 1);
+    file->transactionList(list, filter);
+    QCOMPARE(list.count(), 1);
 
     filter.addCategory(acParent);
 
-    QVERIFY(file->transactionList(filter).count() == 3);
+    file->transactionList(list, filter);
+    QCOMPARE(list.count(), 3);
 
     filter.addAccount(acChecking);
 
-    QVERIFY(file->transactionList(filter).count() == 1);
+    file->transactionList(list, filter);
+    QCOMPARE(list.count(), 1);
 
     filter.clear();
     filter.setDateFilter(QDate(2004, 9, 1), QDate(2005, 1, 1).addDays(-1));
@@ -440,7 +482,8 @@ void PivotTableTest::testFilterBasics()
     filter.setReportAllSplits(false);
     filter.setConsiderCategory(true);
 
-    QVERIFY(file->transactionList(filter).count() == 2);
+    file->transactionList(list, filter);
+    QCOMPARE(list.count(), 2);
 }
 
 void PivotTableTest::testMultipleCurrencies()
@@ -455,10 +498,12 @@ void PivotTableTest::testMultipleCurrencies()
     MyMoneyMoney moCanTransaction(100.0, 10);
     MyMoneyMoney moJpyTransaction(100.0, 10);
 
-    QString acCanChecking = makeAccount(QString("Canadian Checking"), eMyMoney::Account::Type::Checkings, moCanOpening, QDate(2003, 11, 15), acAsset, "CAD");
-    QString acJpyChecking = makeAccount(QString("Japanese Checking"), eMyMoney::Account::Type::Checkings, moJpyOpening, QDate(2003, 11, 15), acAsset, "JPY");
-    QString acCanCash = makeAccount(QString("Canadian"), eMyMoney::Account::Type::Expense, MyMoneyMoney(), QDate(2004, 2, 11), acForeign, "CAD");
-    QString acJpyCash = makeAccount(QString("Japanese"), eMyMoney::Account::Type::Expense, MyMoneyMoney(), QDate(2004, 2, 11), acForeign, "JPY");
+    const QString canChecking =
+        makeAccount(QString("Canadian Checking"), eMyMoney::Account::Type::Checkings, moCanOpening, QDate(2003, 11, 15), acAsset, "CAD");
+    const QString jpyChecking =
+        makeAccount(QString("Japanese Checking"), eMyMoney::Account::Type::Checkings, moJpyOpening, QDate(2003, 11, 15), acAsset, "JPY");
+    const QString canCash = makeAccount(QString("Canadian"), eMyMoney::Account::Type::Expense, MyMoneyMoney(), QDate(2004, 2, 11), acForeign, "CAD");
+    const QString jpyCash = makeAccount(QString("Japanese"), eMyMoney::Account::Type::Expense, MyMoneyMoney(), QDate(2004, 2, 11), acForeign, "JPY");
 
     makePrice("CAD", QDate(2004, 1, 1), MyMoneyMoney(moCanPrice));
     makePrice("JPY", QDate(2004, 1, 1), MyMoneyMoney(moJpyPrice));
@@ -466,12 +511,42 @@ void PivotTableTest::testMultipleCurrencies()
     makePrice("JPY", QDate(2004, 6, 30), MyMoneyMoney(moJpyPrice3));
     makePrice("JPY", QDate(2004, 7, 15), MyMoneyMoney(moJpyPrice4));
 
-    TransactionHelper t1(QDate(2004, 2, 20), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), MyMoneyMoney(moJpyTransaction), acJpyChecking, acJpyCash, "JPY");
-    TransactionHelper t2(QDate(2004, 3, 20), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), MyMoneyMoney(moJpyTransaction), acJpyChecking, acJpyCash, "JPY");
-    TransactionHelper t3(QDate(2004, 4, 20), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), MyMoneyMoney(moJpyTransaction), acJpyChecking, acJpyCash, "JPY");
-    TransactionHelper t4(QDate(2004, 2, 20), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), MyMoneyMoney(moCanTransaction), acCanChecking, acCanCash, "CAD");
-    TransactionHelper t5(QDate(2004, 3, 20), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), MyMoneyMoney(moCanTransaction), acCanChecking, acCanCash, "CAD");
-    TransactionHelper t6(QDate(2004, 4, 20), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), MyMoneyMoney(moCanTransaction), acCanChecking, acCanCash, "CAD");
+    TransactionHelper t1(QDate(2004, 2, 20),
+                         MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal),
+                         MyMoneyMoney(moJpyTransaction),
+                         jpyChecking,
+                         jpyCash,
+                         "JPY");
+    TransactionHelper t2(QDate(2004, 3, 20),
+                         MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal),
+                         MyMoneyMoney(moJpyTransaction),
+                         jpyChecking,
+                         jpyCash,
+                         "JPY");
+    TransactionHelper t3(QDate(2004, 4, 20),
+                         MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal),
+                         MyMoneyMoney(moJpyTransaction),
+                         jpyChecking,
+                         jpyCash,
+                         "JPY");
+    TransactionHelper t4(QDate(2004, 2, 20),
+                         MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal),
+                         MyMoneyMoney(moCanTransaction),
+                         canChecking,
+                         canCash,
+                         "CAD");
+    TransactionHelper t5(QDate(2004, 3, 20),
+                         MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal),
+                         MyMoneyMoney(moCanTransaction),
+                         canChecking,
+                         canCash,
+                         "CAD");
+    TransactionHelper t6(QDate(2004, 4, 20),
+                         MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal),
+                         MyMoneyMoney(moCanTransaction),
+                         canChecking,
+                         canCash,
+                         "CAD");
 
 #if 0
     QFile g("multicurrencykmy.xml");
@@ -482,27 +557,25 @@ void PivotTableTest::testMultipleCurrencies()
     g.close();
 #endif
 
-    MyMoneyReport filter;
+    MyMoneyReport filter(QLatin1String("fake-id"));
     filter.setRowType(eMyMoney::Report::RowType::ExpenseIncome);
     filter.setDateFilter(QDate(2004, 1, 1), QDate(2005, 1, 1).addDays(-1));
     filter.setDetailLevel(eMyMoney::Report::DetailLevel::All);
     filter.setConvertCurrency(true);
     filter.setName("Multiple Currency Spending Rerport (with currency conversion)");
     XMLandback(filter);
-
     PivotTable spending_f(filter);
-
-    writeTabletoCSV(spending_f);
+    writeTable(spending_f, filter.name());
 
     // test single foreign currency
-    QVERIFY(spending_f.m_grid["Expense"]["Foreign"][ReportAccount(acCanCash)][eActual][1] == (moCanTransaction*moCanPrice));
-    QVERIFY(spending_f.m_grid["Expense"]["Foreign"][ReportAccount(acCanCash)][eActual][2] == (moCanTransaction*moCanPrice));
-    QVERIFY(spending_f.m_grid["Expense"]["Foreign"][ReportAccount(acCanCash)][eActual][3] == (moCanTransaction*moCanPrice));
+    QVERIFY(spending_f.m_grid["Expense"]["Foreign"][ReportAccount(canCash)][eActual][1] == (moCanTransaction * moCanPrice));
+    QVERIFY(spending_f.m_grid["Expense"]["Foreign"][ReportAccount(canCash)][eActual][2] == (moCanTransaction * moCanPrice));
+    QVERIFY(spending_f.m_grid["Expense"]["Foreign"][ReportAccount(canCash)][eActual][3] == (moCanTransaction * moCanPrice));
 
     // test multiple foreign currencies under a common parent
-    QVERIFY(spending_f.m_grid["Expense"]["Foreign"][ReportAccount(acJpyCash)][eActual][1] == (moJpyTransaction*moJpyPrice));
-    QVERIFY(spending_f.m_grid["Expense"]["Foreign"][ReportAccount(acJpyCash)][eActual][2] == (moJpyTransaction*moJpyPrice));
-    QVERIFY(spending_f.m_grid["Expense"]["Foreign"][ReportAccount(acJpyCash)][eActual][3] == (moJpyTransaction*moJpyPrice));
+    QVERIFY(spending_f.m_grid["Expense"]["Foreign"][ReportAccount(jpyCash)][eActual][1] == (moJpyTransaction * moJpyPrice));
+    QVERIFY(spending_f.m_grid["Expense"]["Foreign"][ReportAccount(jpyCash)][eActual][2] == (moJpyTransaction * moJpyPrice));
+    QVERIFY(spending_f.m_grid["Expense"]["Foreign"][ReportAccount(jpyCash)][eActual][3] == (moJpyTransaction * moJpyPrice));
     QVERIFY(spending_f.m_grid["Expense"]["Foreign"].m_total[eActual][1] == (moJpyTransaction*moJpyPrice + moCanTransaction*moCanPrice));
     QVERIFY(spending_f.m_grid["Expense"]["Foreign"].m_total[eActual].m_total == (moJpyTransaction*moJpyPrice + moCanTransaction*moCanPrice + moJpyTransaction*moJpyPrice + moCanTransaction*moCanPrice + moJpyTransaction*moJpyPrice + moCanTransaction*moCanPrice));
 
@@ -512,14 +585,14 @@ void PivotTableTest::testMultipleCurrencies()
     filter.setName("Multiple Currency Spending Report (WITHOUT currency conversion)");
     XMLandback(filter);
     PivotTable spending_fnc(filter);
-    writeTabletoCSV(spending_fnc);
+    writeTable(spending_fnc, filter.name());
 
-    QVERIFY(spending_fnc.m_grid["Expense"]["Foreign"][ReportAccount(acCanCash)][eActual][1] == (moCanTransaction));
-    QVERIFY(spending_fnc.m_grid["Expense"]["Foreign"][ReportAccount(acCanCash)][eActual][2] == (moCanTransaction));
-    QVERIFY(spending_fnc.m_grid["Expense"]["Foreign"][ReportAccount(acCanCash)][eActual][3] == (moCanTransaction));
-    QVERIFY(spending_fnc.m_grid["Expense"]["Foreign"][ReportAccount(acJpyCash)][eActual][1] == (moJpyTransaction));
-    QVERIFY(spending_fnc.m_grid["Expense"]["Foreign"][ReportAccount(acJpyCash)][eActual][2] == (moJpyTransaction));
-    QVERIFY(spending_fnc.m_grid["Expense"]["Foreign"][ReportAccount(acJpyCash)][eActual][3] == (moJpyTransaction));
+    QVERIFY(spending_fnc.m_grid["Expense"]["Foreign"][ReportAccount(canCash)][eActual][1] == (moCanTransaction));
+    QVERIFY(spending_fnc.m_grid["Expense"]["Foreign"][ReportAccount(canCash)][eActual][2] == (moCanTransaction));
+    QVERIFY(spending_fnc.m_grid["Expense"]["Foreign"][ReportAccount(canCash)][eActual][3] == (moCanTransaction));
+    QVERIFY(spending_fnc.m_grid["Expense"]["Foreign"][ReportAccount(jpyCash)][eActual][1] == (moJpyTransaction));
+    QVERIFY(spending_fnc.m_grid["Expense"]["Foreign"][ReportAccount(jpyCash)][eActual][2] == (moJpyTransaction));
+    QVERIFY(spending_fnc.m_grid["Expense"]["Foreign"][ReportAccount(jpyCash)][eActual][3] == (moJpyTransaction));
 
     filter.setConvertCurrency(true);
     filter.clearTransactionFilter();
@@ -528,25 +601,33 @@ void PivotTableTest::testMultipleCurrencies()
     filter.setDateFilter(QDate(2004, 1, 1), QDate(2005, 1, 1).addDays(-1));
     XMLandback(filter);
     PivotTable networth_f(filter);
-    writeTabletoCSV(networth_f);
+    writeTable(networth_f, filter.name());
 
     // test single foreign currency
-    QVERIFY(networth_f.m_grid["Asset"]["Canadian Checking"][ReportAccount(acCanChecking)][eActual][1] == (moCanOpening*moCanPrice));
-    QVERIFY(networth_f.m_grid["Asset"]["Canadian Checking"][ReportAccount(acCanChecking)][eActual][2] == ((moCanOpening - moCanTransaction)*moCanPrice));
-    QVERIFY(networth_f.m_grid["Asset"]["Canadian Checking"][ReportAccount(acCanChecking)][eActual][3] == ((moCanOpening - moCanTransaction - moCanTransaction)*moCanPrice));
-    QVERIFY(networth_f.m_grid["Asset"]["Canadian Checking"][ReportAccount(acCanChecking)][eActual][4] == ((moCanOpening - moCanTransaction - moCanTransaction - moCanTransaction)*moCanPrice));
-    QVERIFY(networth_f.m_grid["Asset"]["Canadian Checking"][ReportAccount(acCanChecking)][eActual][12] == ((moCanOpening - moCanTransaction - moCanTransaction - moCanTransaction)*moCanPrice));
+    QVERIFY(networth_f.m_grid["Asset"]["Canadian Checking"][ReportAccount(canChecking)][eActual][1] == (moCanOpening * moCanPrice));
+    QVERIFY(networth_f.m_grid["Asset"]["Canadian Checking"][ReportAccount(canChecking)][eActual][2] == ((moCanOpening - moCanTransaction) * moCanPrice));
+    QVERIFY(networth_f.m_grid["Asset"]["Canadian Checking"][ReportAccount(canChecking)][eActual][3]
+            == ((moCanOpening - moCanTransaction - moCanTransaction) * moCanPrice));
+    QVERIFY(networth_f.m_grid["Asset"]["Canadian Checking"][ReportAccount(canChecking)][eActual][4]
+            == ((moCanOpening - moCanTransaction - moCanTransaction - moCanTransaction) * moCanPrice));
+    QVERIFY(networth_f.m_grid["Asset"]["Canadian Checking"][ReportAccount(canChecking)][eActual][12]
+            == ((moCanOpening - moCanTransaction - moCanTransaction - moCanTransaction) * moCanPrice));
 
     // test Stable currency price, fluctuating account balance
-    QVERIFY(networth_f.m_grid["Asset"]["Japanese Checking"][ReportAccount(acJpyChecking)][eActual][1] == (moJpyOpening*moJpyPrice));
-    QVERIFY(networth_f.m_grid["Asset"]["Japanese Checking"][ReportAccount(acJpyChecking)][eActual][2] == ((moJpyOpening - moJpyTransaction)*moJpyPrice));
-    QVERIFY(networth_f.m_grid["Asset"]["Japanese Checking"][ReportAccount(acJpyChecking)][eActual][3] == ((moJpyOpening - moJpyTransaction - moJpyTransaction)*moJpyPrice));
-    QVERIFY(networth_f.m_grid["Asset"]["Japanese Checking"][ReportAccount(acJpyChecking)][eActual][4] == ((moJpyOpening - moJpyTransaction - moJpyTransaction - moJpyTransaction)*moJpyPrice));
+    QVERIFY(networth_f.m_grid["Asset"]["Japanese Checking"][ReportAccount(jpyChecking)][eActual][1] == (moJpyOpening * moJpyPrice));
+    QVERIFY(networth_f.m_grid["Asset"]["Japanese Checking"][ReportAccount(jpyChecking)][eActual][2] == ((moJpyOpening - moJpyTransaction) * moJpyPrice));
+    QVERIFY(networth_f.m_grid["Asset"]["Japanese Checking"][ReportAccount(jpyChecking)][eActual][3]
+            == ((moJpyOpening - moJpyTransaction - moJpyTransaction) * moJpyPrice));
+    QVERIFY(networth_f.m_grid["Asset"]["Japanese Checking"][ReportAccount(jpyChecking)][eActual][4]
+            == ((moJpyOpening - moJpyTransaction - moJpyTransaction - moJpyTransaction) * moJpyPrice));
 
     // test Fluctuating currency price, stable account balance
-    QVERIFY(networth_f.m_grid["Asset"]["Japanese Checking"][ReportAccount(acJpyChecking)][eActual][5] == ((moJpyOpening - moJpyTransaction - moJpyTransaction - moJpyTransaction)*moJpyPrice2));
-    QVERIFY(networth_f.m_grid["Asset"]["Japanese Checking"][ReportAccount(acJpyChecking)][eActual][6] == ((moJpyOpening - moJpyTransaction - moJpyTransaction - moJpyTransaction)*moJpyPrice3));
-    QVERIFY(networth_f.m_grid["Asset"]["Japanese Checking"][ReportAccount(acJpyChecking)][eActual][7] == ((moJpyOpening - moJpyTransaction - moJpyTransaction - moJpyTransaction)*moJpyPrice4));
+    QVERIFY(networth_f.m_grid["Asset"]["Japanese Checking"][ReportAccount(jpyChecking)][eActual][5]
+            == ((moJpyOpening - moJpyTransaction - moJpyTransaction - moJpyTransaction) * moJpyPrice2));
+    QVERIFY(networth_f.m_grid["Asset"]["Japanese Checking"][ReportAccount(jpyChecking)][eActual][6]
+            == ((moJpyOpening - moJpyTransaction - moJpyTransaction - moJpyTransaction) * moJpyPrice3));
+    QVERIFY(networth_f.m_grid["Asset"]["Japanese Checking"][ReportAccount(jpyChecking)][eActual][7]
+            == ((moJpyOpening - moJpyTransaction - moJpyTransaction - moJpyTransaction) * moJpyPrice4));
 
     // test multiple currencies totalled up
     QVERIFY(networth_f.m_grid["Asset"].m_total[eActual][4] == ((moCanOpening - moCanTransaction - moCanTransaction - moCanTransaction)*moCanPrice) + ((moJpyOpening - moJpyTransaction - moJpyTransaction - moJpyTransaction)*moJpyPrice));
@@ -563,7 +644,7 @@ void PivotTableTest::testAdvancedFilter()
         TransactionHelper t1(QDate(2004, 11, 7), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moParent1, acCredit, acParent);
         TransactionHelper t2(QDate(2004, 11, 7), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moChild, acCredit, acChild);
 
-        MyMoneyReport filter;
+        MyMoneyReport filter(QLatin1String("fake-id"));
         filter.setRowType(eMyMoney::Report::RowType::ExpenseIncome);
         filter.setDateFilter(QDate(2004, 1, 1), QDate(2005, 1, 1).addDays(-1));
         filter.setAmountFilter(moChild, moChild);
@@ -579,14 +660,14 @@ void PivotTableTest::testAdvancedFilter()
         TransactionHelper t3(QDate(2004, 11, 7), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moChild, acCredit, acChild);
         TransactionHelper t4(QDate(2004, 11, 7), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moThomas, acCredit, acParent, QString(), "Thomas Baumgart");
 
-        MyMoneyReport filter;
+        MyMoneyReport filter(QLatin1String("fake-id"));
         filter.setRowType(eMyMoney::Report::RowType::ExpenseIncome);
         filter.setDateFilter(QDate(2004, 1, 1), QDate(2005, 1, 1).addDays(-1));
         filter.addPayee(MyMoneyFile::instance()->payeeByName("Thomas Baumgart").id());
         filter.setName("Spending with Payee Filter");
         XMLandback(filter);
         PivotTable spending_f(filter);
-        writeTabletoHTML(spending_f, "Spending with Payee Filter.html");
+        writeTable(spending_f, filter.name());
 
         QVERIFY(spending_f.m_grid["Expense"]["Parent"][ReportAccount(acParent)][eActual][10] == moThomas);
         QVERIFY(spending_f.m_grid.m_total[eActual].m_total == -moThomas);
@@ -598,7 +679,7 @@ void PivotTableTest::testAdvancedFilter()
         TransactionHelper t3(QDate(2004, 11, 7), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moChild, acCredit, acChild);
         TransactionHelper t4(QDate(2004, 11, 7), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moNoPayee, acCredit, acParent, QString(), QString());
 
-        MyMoneyReport filter;
+        MyMoneyReport filter(QLatin1String("fake-id"));
         filter.setRowType(eMyMoney::Report::RowType::ExpenseIncome);
         filter.setDateFilter(QDate(2004, 1, 1), QDate(2005, 1, 1).addDays(-1));
         filter.addPayee(QString());
@@ -615,10 +696,10 @@ void PivotTableTest::testAdvancedFilter()
         TransactionHelper t3(QDate(2004, 11, 7), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moChild, acCredit, acChild);
         TransactionHelper t4(QDate(2004, 11, 7), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moThomas, acCredit, acParent, QString(), "Thomas Baumgart");
 
-        MyMoneyReport filter;
+        MyMoneyReport filter(QLatin1String("fake-id"));
         filter.setRowType(eMyMoney::Report::RowType::ExpenseIncome);
         filter.setDateFilter(QDate(2004, 1, 1), QDate(2005, 1, 1).addDays(-1));
-        filter.setTextFilter(QRegExp("Thomas"));
+        filter.setTextFilter(QRegularExpression("Thomas"), false, false);
         XMLandback(filter);
         PivotTable spending_f(filter);
     }
@@ -629,7 +710,7 @@ void PivotTableTest::testAdvancedFilter()
         TransactionHelper t2(QDate(2004, 2, 1), MyMoneySplit::actionName(eMyMoney::Split::Action::Deposit), -moParent1, acCredit, acParent);
         TransactionHelper t3(QDate(2004, 11, 1), MyMoneySplit::actionName(eMyMoney::Split::Action::Transfer), moChild, acCredit, acChecking);
 
-        MyMoneyReport filter;
+        MyMoneyReport filter(QLatin1String("fake-id"));
         filter.setRowType(eMyMoney::Report::RowType::ExpenseIncome);
         filter.addType((int)eMyMoney::TransactionFilter::Type::Payments);
         XMLandback(filter);
@@ -684,7 +765,7 @@ void PivotTableTest::testAdvancedFilter()
         t2.modifySplit(splits[1]);
         t2.update();
 
-        MyMoneyReport filter;
+        MyMoneyReport filter(QLatin1String("fake-id"));
         filter.setRowType(eMyMoney::Report::RowType::ExpenseIncome);
         filter.setDateFilter(QDate(2004, 1, 1), QDate(2005, 1, 1).addDays(-1));
         filter.addState((int)eMyMoney::TransactionFilter::State::Cleared);
@@ -745,7 +826,7 @@ void PivotTableTest::testAdvancedFilter()
         t4.modifySplit(splits[1]);
         t4.update();
 
-        MyMoneyReport filter;
+        MyMoneyReport filter(QLatin1String("fake-id"));
         filter.setRowType(eMyMoney::Report::RowType::ExpenseIncome);
         filter.setDateFilter(QDate(2004, 1, 1), QDate(2005, 1, 1).addDays(-1));
         filter.setNumberFilter("1", "3");
@@ -768,7 +849,7 @@ void PivotTableTest::testAdvancedFilter()
         TransactionHelper t2y3(QDate(2005, 5, 1), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moParent1, acCredit, acParent);
         TransactionHelper t3y3(QDate(2005, 9, 1), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moParent2, acCredit, acParent);
 
-        MyMoneyReport filter;
+        MyMoneyReport filter(QLatin1String("fake-id"));
         filter.setRowType(eMyMoney::Report::RowType::ExpenseIncome);
         filter.setDateFilter(QDate(), QDate(2004, 7, 1));
         XMLandback(filter);
@@ -800,7 +881,7 @@ void PivotTableTest::testColumnType()
     TransactionHelper t2y2(QDate(2005, 5, 1), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moParent1, acCredit, acParent);
     TransactionHelper t3y2(QDate(2005, 9, 1), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moParent2, acCredit, acParent);
 
-    MyMoneyReport filter;
+    MyMoneyReport filter(QLatin1String("fake-id"));
     filter.setRowType(eMyMoney::Report::RowType::ExpenseIncome);
     filter.setDateFilter(QDate(2003, 12, 31), QDate(2005, 12, 31));
     filter.setRowType(eMyMoney::Report::RowType::ExpenseIncome);
@@ -840,7 +921,7 @@ void PivotTableTest::testColumnType()
     filter.setName("Net Worth by Quarter");
     XMLandback(filter);
     PivotTable networth_q(filter);
-    writeTabletoHTML(networth_q, "Net Worth by Quarter.html");
+    writeTable(networth_q, filter.name());
 
     QVERIFY(networth_q.m_grid.m_total[eActual][1] == moZero);
     QVERIFY(networth_q.m_grid.m_total[eActual][2] == -moSolo - moParent);
@@ -889,10 +970,9 @@ void PivotTableTest::testColumnType()
     filter.setColumnType(eMyMoney::Report::ColumnType::Months);
     filter.setColumnsAreDays(true);
     filter.setName("Spending by Days");
-
     XMLandback(filter);
     PivotTable spending_days(filter);
-    writeTabletoHTML(spending_days, "Spending by Days.html");
+    writeTable(spending_days, filter.name());
 
     QVERIFY(spending_days.m_grid.m_total[eActual][2] == -moParent2);
     QVERIFY(spending_days.m_grid.m_total[eActual][12] == -moSolo);
@@ -906,10 +986,9 @@ void PivotTableTest::testColumnType()
     filter.setColumnType(static_cast<eMyMoney::Report::ColumnType>(7));
     filter.setColumnsAreDays(true);
     filter.setName("Spending by Weeks");
-
     XMLandback(filter);
     PivotTable spending_weeks(filter);
-    writeTabletoHTML(spending_weeks, "Spending by Weeks.html");
+    writeTable(spending_weeks, filter.name());
 
     // restore the locale
     QLocale::setDefault(QLocale::system());
@@ -953,13 +1032,13 @@ void PivotTableTest::testInvestment()
         // Net Worth Report (with investments)
         //
 
-        MyMoneyReport networth_r;
+        MyMoneyReport networth_r(QLatin1String("fake-id"));
         networth_r.setRowType(eMyMoney::Report::RowType::AssetLiability);
         networth_r.setDateFilter(QDate(2004, 1, 1), QDate(2004, 12, 31).addDays(-1));
+        networth_r.setName("Networth with investment");
         XMLandback(networth_r);
         PivotTable networth(networth_r);
-
-        networth.dump("networth_i.html");
+        writeTable(networth, networth_r.name());
 
         QVERIFY(networth.m_grid["Asset"]["Investment"].m_total[eActual][1] == moZero);
         // 1000 shares @ $100.00
@@ -1080,7 +1159,7 @@ void PivotTableTest::testBudget()
 
 void PivotTableTest::testHtmlEncoding()
 {
-    MyMoneyReport filter;
+    MyMoneyReport filter(QLatin1String("fake-id"));
     filter.setRowType(eMyMoney::Report::RowType::AssetLiability);
     filter.setDateFilter(QDate(2004, 1, 1), QDate(2005, 1, 1).addDays(-1));
     XMLandback(filter);
@@ -1088,11 +1167,9 @@ void PivotTableTest::testHtmlEncoding()
 
     QByteArray encoding = QTextCodec::codecForLocale()->name();
 
-    QString html = networth_f.renderReport(QLatin1String("html"), encoding,
-                                           filter.name(), false);
+    QString html = networth_f.renderReport(QLatin1String("html"), encoding, filter.name());
 
-    QRegExp rx(QString::fromLatin1("*<meta * charset=" + encoding + "*>*"));
-    rx.setPatternSyntax(QRegExp::Wildcard);
-    rx.setCaseSensitivity(Qt::CaseInsensitive);
-    QVERIFY(rx.exactMatch(html));
+    static const QRegularExpression rx(QLatin1String("^.*<meta .* charset=" + encoding + ".*>.*"),
+                                       QRegularExpression::CaseInsensitiveOption | QRegularExpression::DotMatchesEverythingOption);
+    QVERIFY(rx.match(html).hasMatch());
 }
