@@ -17,7 +17,6 @@ class PivotTableTest;
 #define KMM_MYMONEY_UNIT_TESTABLE friend class reports::PivotTableTest;
 
 #include "mymoneyfile.h"
-#include "mymoneystoragemgr.h"
 #include "reporttable.h"
 
 namespace reports
@@ -27,11 +26,10 @@ class PivotTableTest : public QObject
 {
     Q_OBJECT
 private:
-    MyMoneyStorageMgr* storage;
     MyMoneyFile* file;
 
 private Q_SLOTS:
-    void setup();
+    void initTestCase();
     void init();
     void cleanup();
     void testNetWorthSingle();
