@@ -6,38 +6,41 @@
 
 #include "querytable-test.h"
 
+#include <KLocalizedString>
+
 #include <QFile>
 #include <QTest>
 
-#include <KLocalizedString>
-
-#include "tests/testutilities.h"
 #include "cashflowlist.h"
-#include "querytable.h"
-#include "mymoneyinstitution.h"
-#include "mymoneyaccount.h"
-#include "mymoneysecurity.h"
-#include "mymoneyprice.h"
-#include "mymoneystoragedump.h"
-#include "mymoneyreport.h"
-#include "mymoneysplit.h"
-#include "mymoneypayee.h"
-#include "mymoneystatement.h"
-#include "mymoneyexception.h"
 #include "kmymoneysettings.h"
+#include "mymoneyaccount.h"
+#include "mymoneyexception.h"
+#include "mymoneyinstitution.h"
+#include "mymoneypayee.h"
+#include "mymoneyprice.h"
+#include "mymoneyreport.h"
+#include "mymoneysecurity.h"
+#include "mymoneysplit.h"
+#include "mymoneystatement.h"
+#include "mymoneystoragedump.h"
+#include "mymoneyutils.h"
+#include "querytable.h"
+#include "tests/testutilities.h"
 
 using namespace reports;
 using namespace test;
 
 QTEST_GUILESS_MAIN(QueryTableTest)
 
-void writeTabletoHTML(const QueryTable& table, const QString& _filename = QString())
+void writeTabletoHTML(const QueryTable& table, const QString& basename = QString())
 {
     static unsigned filenumber = 1;
-    QString filename = _filename;
-    if (filename.isEmpty()) {
-        filename = QString::fromLatin1("report-%1.html").arg(filenumber, 2, 10,QLatin1Char('0'));
+    QString filename;
+    if (basename.isEmpty()) {
+        filename = QString::fromLatin1("%1/report-%2.html").arg(CMAKE_CURRENT_BINARY_DIR).arg(filenumber, 2, 10, QLatin1Char('0'));
         ++filenumber;
+    } else {
+        filename = QString::fromLatin1("%1/%2.html").arg(CMAKE_CURRENT_BINARY_DIR).arg(basename);
     }
 
     QFile g(filename);
@@ -46,13 +49,15 @@ void writeTabletoHTML(const QueryTable& table, const QString& _filename = QStrin
     g.close();
 }
 
-void writeTabletoCSV(const QueryTable& table, const QString& _filename = QString())
+void writeTabletoCSV(const QueryTable& table, const QString& basename = QString())
 {
     static unsigned filenumber = 1;
-    QString filename = _filename;
-    if (filename.isEmpty()) {
-        filename = QString::fromLatin1("report-%1.csv").arg(filenumber, 2, 10,QLatin1Char('0'));
+    QString filename;
+    if (basename.isEmpty()) {
+        filename = QString::fromLatin1("%1/report-%2.csv").arg(CMAKE_CURRENT_BINARY_DIR).arg(filenumber, 2, 10, QLatin1Char('0'));
         ++filenumber;
+    } else {
+        filename = QString::fromLatin1("%1/%2.csv").arg(CMAKE_CURRENT_BINARY_DIR).arg(basename);
     }
 
     QFile g(filename);
@@ -61,15 +66,39 @@ void writeTabletoCSV(const QueryTable& table, const QString& _filename = QString
     g.close();
 }
 
-void QueryTableTest::setup()
+void writeTabletoXML(const QueryTable& table, const QString& basename = QString())
 {
+    static unsigned filenumber = 1;
+    QString filename;
+    if (basename.isEmpty()) {
+        filename = QString::fromLatin1("%1/report-%2.xml").arg(CMAKE_CURRENT_BINARY_DIR).arg(filenumber, 2, 10, QLatin1Char('0'));
+        ++filenumber;
+    } else {
+        filename = QString::fromLatin1("%1/%2.xml").arg(CMAKE_CURRENT_BINARY_DIR).arg(basename);
+    }
+
+    QFile g(filename);
+    g.open(QIODevice::WriteOnly);
+    QTextStream(&g) << table.toXml();
+    g.close();
+}
+
+void writeTable(const QueryTable& table, const QString& basename = QString())
+{
+    writeTabletoCSV(table, basename);
+    writeTabletoHTML(table, basename);
+    writeTabletoXML(table, basename);
+}
+
+void QueryTableTest::initTestCase()
+{
+    KLocalizedString::setApplicationDomain(QByteArrayLiteral("kmymoney"));
 }
 
 void QueryTableTest::init()
 {
-    storage = new MyMoneyStorageMgr;
     file = MyMoneyFile::instance();
-    file->attachStorage(storage);
+    file->unload();
     MyMoneyFileTransaction ft;
     file->addCurrency(MyMoneySecurity("CAD", "Canadian Dollar",        "C$"));
     file->addCurrency(MyMoneySecurity("USD", "US Dollar",              "$"));
@@ -104,8 +133,6 @@ void QueryTableTest::init()
 
 void QueryTableTest::cleanup()
 {
-    file->detachStorage(storage);
-    delete storage;
 }
 
 void QueryTableTest::testQueryBasics()
@@ -128,15 +155,14 @@ void QueryTableTest::testQueryBasics()
 
         unsigned cols;
 
-        MyMoneyReport filter;
+        MyMoneyReport filter(QLatin1String("fake-id"));
         filter.setRowType(eMyMoney::Report::RowType::Category);
         cols = eMyMoney::Report::QueryColumn::Number | eMyMoney::Report::QueryColumn::Payee | eMyMoney::Report::QueryColumn::Account;
         filter.setQueryColumns(static_cast<eMyMoney::Report::QueryColumn>(cols));   //
         filter.setName("Transactions by Category");
         XMLandback(filter);
         QueryTable qtbl_1(filter);
-
-        writeTabletoHTML(qtbl_1, "Transactions by Category.html");
+        writeTable(qtbl_1, filter.name());
 
         QList<ListTable::TableRow> rows = qtbl_1.rows();
 
@@ -160,8 +186,7 @@ void QueryTableTest::testQueryBasics()
         filter.setName("Transactions by Top Category");
         XMLandback(filter);
         QueryTable qtbl_2(filter);
-
-        writeTabletoHTML(qtbl_2, "Transactions by Top Category.html");
+        writeTable(qtbl_2, filter.name());
 
         rows = qtbl_2.rows();
 
@@ -188,7 +213,7 @@ void QueryTableTest::testQueryBasics()
         XMLandback(filter);
         QueryTable qtbl_3(filter);
 
-        writeTabletoHTML(qtbl_3, "Transactions by Account.html");
+        writeTable(qtbl_3, filter.name());
 
         rows = qtbl_3.rows();
 
@@ -220,8 +245,7 @@ void QueryTableTest::testQueryBasics()
         filter.setQueryColumns(static_cast<eMyMoney::Report::QueryColumn>(cols));   //
         XMLandback(filter);
         QueryTable qtbl_4(filter);
-
-        writeTabletoHTML(qtbl_4, "Transactions by Payee.html");
+        writeTable(qtbl_4, filter.name());
 
         rows = qtbl_4.rows();
 
@@ -245,8 +269,7 @@ void QueryTableTest::testQueryBasics()
         filter.setQueryColumns(static_cast<eMyMoney::Report::QueryColumn>(cols));   //
         XMLandback(filter);
         QueryTable qtbl_5(filter);
-
-        writeTabletoHTML(qtbl_5, "Transactions by Month.html");
+        writeTable(qtbl_5, filter.name());
 
         rows = qtbl_5.rows();
 
@@ -272,8 +295,7 @@ void QueryTableTest::testQueryBasics()
         filter.setQueryColumns(static_cast<eMyMoney::Report::QueryColumn>(cols));   //
         XMLandback(filter);
         QueryTable qtbl_6(filter);
-
-        writeTabletoHTML(qtbl_6, "Transactions by Week.html");
+        writeTable(qtbl_6, filter.name());
 
         rows = qtbl_6.rows();
 
@@ -378,7 +400,7 @@ void QueryTableTest::testCashFlowAnalysis()
     bool result = false;
     try {
         XIRR = MyMoneyMoney(list.XIRR(), 1000);
-    } catch (MyMoneyException &e) {
+    } catch (MyMoneyException&) {
         result  = true;
     }
     QVERIFY(result);
@@ -387,19 +409,16 @@ void QueryTableTest::testCashFlowAnalysis()
 void QueryTableTest::testAccountQuery()
 {
     try {
-        QString htmlcontext = QString("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\">\n<html><head><link rel=\"stylesheet\" type=\"text/css\" href=\"html/kmymoney.css\"></head><body>\n%1\n</body></html>\n");
-
         //
         // No transactions, opening balances only
         //
 
-        MyMoneyReport filter;
+        MyMoneyReport filter(QLatin1String("fake-id"));
         filter.setRowType(eMyMoney::Report::RowType::Institution);
         filter.setName("Accounts by Institution (No transactions)");
         XMLandback(filter);
         QueryTable qtbl_1(filter);
-
-        writeTabletoHTML(qtbl_1, "Accounts by Institution (No transactions).html");
+        writeTable(qtbl_1, filter.name());
 
         QList<ListTable::TableRow> rows = qtbl_1.rows();
 
@@ -468,7 +487,7 @@ void QueryTableTest::testAccountQuery()
         QVERIFY(MyMoneyMoney(rows[1][ListTable::ctValue]) == moCheckingOpen - moSolo * 3);
         QVERIFY(MyMoneyMoney(rows[3][ListTable::ctValue]) == moCreditOpen - (moParent1 + moParent2 + moChild) * 3);
         QVERIFY(MyMoneyMoney(rows[4][ListTable::ctValue]) == moCheckingOpen + moCreditOpen - (moParent1 + moParent2 + moSolo + moChild) * 3);
-    } catch (const MyMoneyException &e) {
+    } catch (const MyMoneyException& e) {
         QFAIL(e.what());
     }
 }
@@ -517,20 +536,19 @@ void QueryTableTest::testInvestment()
         //
 
         MyMoneyReport invtran_r(
-            eMyMoney::Report::RowType::TopAccount,
-            eMyMoney::Report::QueryColumn::Action | eMyMoney::Report::QueryColumn::Shares | eMyMoney::Report::QueryColumn::Price,
-            eMyMoney::TransactionFilter::Date::UserDefined,
-            eMyMoney::Report::DetailLevel::All,
-            i18n("Investment Transactions"),
-            i18n("Test Report")
-        );
+            QLatin1String("fake-id"),
+            MyMoneyReport(eMyMoney::Report::RowType::TopAccount,
+                          eMyMoney::Report::QueryColumn::Action | eMyMoney::Report::QueryColumn::Shares | eMyMoney::Report::QueryColumn::Price,
+                          eMyMoney::TransactionFilter::Date::UserDefined,
+                          eMyMoney::Report::DetailLevel::All,
+                          "Investment Transactions",
+                          "Test Report"));
         invtran_r.setDateFilter(QDate(2004, 1, 1), QDate(2004, 12, 31));
         invtran_r.setInvestmentsOnly(true);
         XMLandback(invtran_r);
         QueryTable invtran(invtran_r);
-
 #if 1
-        writeTabletoHTML(invtran, "investment_transactions_test.html");
+        writeTable(invtran, invtran_r.name());
 
         QList<ListTable::TableRow> rows = invtran.rows();
 
@@ -616,36 +634,34 @@ void QueryTableTest::testInvestment()
         // Investment Performance Report
         //
 
-        MyMoneyReport invhold_r(
-            eMyMoney::Report::RowType::AccountByTopAccount,
-            eMyMoney::Report::QueryColumn::Performance,
-            eMyMoney::TransactionFilter::Date::UserDefined,
-            eMyMoney::Report::DetailLevel::All,
-            i18n("Investment Performance by Account"),
-            i18n("Test Report")
-        );
+        MyMoneyReport invhold_r(QLatin1String("fake-id"),
+                                MyMoneyReport(eMyMoney::Report::RowType::AccountByTopAccount,
+                                              eMyMoney::Report::QueryColumn::Performance,
+                                              eMyMoney::TransactionFilter::Date::UserDefined,
+                                              eMyMoney::Report::DetailLevel::All,
+                                              "Investment Performance by Account",
+                                              "Test Report"));
         invhold_r.setDateFilter(QDate(2004, 1, 1), QDate(2004, 10, 1));
         invhold_r.setInvestmentsOnly(true);
         XMLandback(invhold_r);
         QueryTable invhold(invhold_r);
-
-        writeTabletoHTML(invhold, "Investment Performance by Account.html");
+        writeTable(invhold, invhold_r.name());
 
         rows = invhold.rows();
 
         QVERIFY(rows.count() == 5);
-        QVERIFY(MyMoneyMoney(rows[0][ListTable::ctReturn]) == MyMoneyMoney("669/10000"));
+        QVERIFY(MyMoneyMoney(rows[0][ListTable::ctExtendedInternalRateOfReturn]) == MyMoneyMoney("669/10000"));
         QVERIFY(MyMoneyMoney(rows[0][ListTable::ctReturnInvestment]) == MyMoneyMoney("-39/5000"));
         QVERIFY(MyMoneyMoney(rows[0][ListTable::ctBuys]) == MyMoneyMoney(-210000.00));
         QVERIFY(MyMoneyMoney(rows[0][ListTable::ctSells]) == MyMoneyMoney(44000.00));
         QVERIFY(MyMoneyMoney(rows[0][ListTable::ctReinvestIncome]) == MyMoneyMoney(9000.00));
         QVERIFY(MyMoneyMoney(rows[0][ListTable::ctCashIncome]) == MyMoneyMoney(3300.00));
 
-        QVERIFY(MyMoneyMoney(rows[1][ListTable::ctReturn]) == MyMoneyMoney("1349/10000"));
+        QVERIFY(MyMoneyMoney(rows[1][ListTable::ctExtendedInternalRateOfReturn]) == MyMoneyMoney("1349/10000"));
         QVERIFY(MyMoneyMoney(rows[1][ListTable::ctReturnInvestment]) == MyMoneyMoney("1/10"));
         QVERIFY(MyMoneyMoney(rows[1][ListTable::ctStartingBalance]) == MyMoneyMoney(100000.00)); // this should stay non-zero to check if investment performance is calculated at non-zero starting balance
 
-        QVERIFY(MyMoneyMoney(rows[2][ListTable::ctReturn]) == MyMoneyMoney("2501/2500"));
+        QVERIFY(MyMoneyMoney(rows[2][ListTable::ctExtendedInternalRateOfReturn]) == MyMoneyMoney("2501/2500"));
         QVERIFY(MyMoneyMoney(rows[2][ListTable::ctReturnInvestment]) == MyMoneyMoney("323/1250"));
         QVERIFY(MyMoneyMoney(rows[2][ListTable::ctBuys]) == MyMoneyMoney(-95200.00));
         QVERIFY(MyMoneyMoney(rows[2][ListTable::ctSells]) == MyMoneyMoney(119800.00));
@@ -699,20 +715,18 @@ void QueryTableTest::testSplitShares()
         // Investment Performance Report
         //
 
-        MyMoneyReport invhold_r(
-            eMyMoney::Report::RowType::AccountByTopAccount,
-            eMyMoney::Report::QueryColumn::Performance,
-            eMyMoney::TransactionFilter::Date::UserDefined,
-            eMyMoney::Report::DetailLevel::All,
-            i18n("Investment Performance by Account"),
-            i18n("Test Report")
-        );
+        MyMoneyReport invhold_r(QLatin1String("fake-id"),
+                                MyMoneyReport(eMyMoney::Report::RowType::AccountByTopAccount,
+                                              eMyMoney::Report::QueryColumn::Performance,
+                                              eMyMoney::TransactionFilter::Date::UserDefined,
+                                              eMyMoney::Report::DetailLevel::All,
+                                              "Investment Performance by Account (with stock split)",
+                                              "Test Report"));
         invhold_r.setDateFilter(QDate(2017, 8, 1), QDate(2017, 8, 3));
         invhold_r.setInvestmentsOnly(true);
         XMLandback(invhold_r);
         QueryTable invhold(invhold_r);
-
-        writeTabletoHTML(invhold, "Investment Performance by Account (with stock split).html");
+        writeTable(invhold, invhold_r.name());
 
         const auto rows = invhold.rows();
 
@@ -739,16 +753,15 @@ void QueryTableTest::testConversionRate()
 
         TransactionHelper t1(QDate(2017, 8, 1), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), amountWithdrawn, acCadChecking, acSolo, "CAD");
 
-        MyMoneyReport filter;
+        MyMoneyReport filter(QLatin1String("fake-id"));
         filter.setRowType(eMyMoney::Report::RowType::Account);
         filter.setDateFilter(QDate(2017, 8, 1), QDate(2017, 8, 2));
-        filter.setName("Transactions by Account");
+        filter.setName("Transactions by Account (conversion rate)");
         auto cols = eMyMoney::Report::QueryColumn::Number | eMyMoney::Report::QueryColumn::Payee | eMyMoney::Report::QueryColumn::Category | eMyMoney::Report::QueryColumn::Balance;
         filter.setQueryColumns(static_cast<eMyMoney::Report::QueryColumn>(cols));   //
         XMLandback(filter);
         QueryTable qtbl(filter);
-
-        writeTabletoHTML(qtbl, "Transactions by Account (conversion rate).html");
+        writeTable(qtbl, filter.name());
 
         const auto rows = qtbl.rows();
 
@@ -786,16 +799,15 @@ void QueryTableTest::testBalanceColumn()
 
         unsigned cols;
 
-        MyMoneyReport filter;
+        MyMoneyReport filter(QLatin1String("fake-id"));
 
         filter.setRowType(eMyMoney::Report::RowType::Account);
-        filter.setName("Transactions by Account");
+        filter.setName("Transactions by Account (balance column)");
         cols = eMyMoney::Report::QueryColumn::Number | eMyMoney::Report::QueryColumn::Payee | eMyMoney::Report::QueryColumn::Category | eMyMoney::Report::QueryColumn::Balance;
         filter.setQueryColumns(static_cast<eMyMoney::Report::QueryColumn>(cols));   //
         XMLandback(filter);
         QueryTable qtbl_3(filter);
-
-        writeTabletoHTML(qtbl_3, "Transactions by Account.html");
+        writeTable(qtbl_3, filter.name());
 
         QString html = qtbl_3.renderHTML();
 
@@ -804,8 +816,8 @@ void QueryTableTest::testBalanceColumn()
         QVERIFY(rows.count() == 19);
 
         //this is to make sure that the dates of closing and opening balances and the balance numbers are ok
-        QString openingDate = QLocale().toString(QDate(2004, 1, 1), QLocale::ShortFormat);
-        QString closingDate = QLocale().toString(QDate(2005, 9, 1), QLocale::ShortFormat);
+        QString openingDate = MyMoneyUtils::formatDate(QDate(2004, 1, 1));
+        QString closingDate = MyMoneyUtils::formatDate(QDate(2005, 9, 1));
         QVERIFY(html.indexOf(openingDate + "</td><td class=\"left0\"></td><td class=\"left0\">" + i18n("Opening Balance")) > 0);
         QVERIFY(html.indexOf(closingDate + "</td><td class=\"left0\"></td><td class=\"left0\">" + i18n("Closing Balance") + "</td><td class=\"left0\"></td><td class=\"value\"></td><td>&nbsp;-702.36</td></tr>") > 0);
         QVERIFY(html.indexOf(closingDate + "</td><td class=\"left0\"></td><td class=\"left0\">" + i18n("Closing Balance") + "</td><td class=\"left0\"></td><td class=\"value\"></td><td>&nbsp;-705.69</td></tr>") > 0);
@@ -827,7 +839,8 @@ void QueryTableTest::testBalanceColumnWithMultipleCurrencies()
         MyMoneyMoney moTransaction(100, 1);
         MyMoneyMoney moJpyTransaction(100, 1);
 
-        QString acJpyChecking = makeAccount(QString("Japanese Checking"), eMyMoney::Account::Type::Checkings, moJpyOpening, QDate(2003, 11, 15), acAsset, "JPY");
+        const QString jpyChecking =
+            makeAccount(QLatin1String("Japanese Checking"), eMyMoney::Account::Type::Checkings, moJpyOpening, QDate(2003, 11, 15), acAsset, "JPY");
 
         makePrice("JPY", QDate(2004, 1, 1), MyMoneyMoney(moJpyPrice));
         makePrice("JPY", QDate(2004, 5, 1), MyMoneyMoney(moJpyPrice2));
@@ -837,29 +850,48 @@ void QueryTableTest::testBalanceColumnWithMultipleCurrencies()
         QDate intermediateDate(2004, 5, 20);
         QDate closingDate(2004, 7, 20);
 
-        TransactionHelper t1(openingDate,      MyMoneySplit::actionName(eMyMoney::Split::Action::Transfer),   MyMoneyMoney(moJpyTransaction), acJpyChecking, acChecking, "JPY");
-        TransactionHelper t4(openingDate,      MyMoneySplit::actionName(eMyMoney::Split::Action::Deposit),    MyMoneyMoney(moTransaction),    acCredit,      acChecking);
-        TransactionHelper t2(intermediateDate, MyMoneySplit::actionName(eMyMoney::Split::Action::Transfer),   MyMoneyMoney(moJpyTransaction), acJpyChecking, acChecking, "JPY");
-        TransactionHelper t5(intermediateDate, MyMoneySplit::actionName(eMyMoney::Split::Action::Deposit),    MyMoneyMoney(moTransaction),    acCredit,      acChecking);
-        TransactionHelper t3(closingDate,      MyMoneySplit::actionName(eMyMoney::Split::Action::Transfer),   MyMoneyMoney(moJpyTransaction), acJpyChecking, acChecking, "JPY");
-        TransactionHelper t6(closingDate,      MyMoneySplit::actionName(eMyMoney::Split::Action::Deposit),    MyMoneyMoney(moTransaction),    acCredit,      acChecking);
+        TransactionHelper t1(openingDate,
+                             MyMoneySplit::actionName(eMyMoney::Split::Action::Transfer),
+                             MyMoneyMoney(moJpyTransaction),
+                             jpyChecking,
+                             acChecking,
+                             "JPY");
+        TransactionHelper t4(openingDate, MyMoneySplit::actionName(eMyMoney::Split::Action::Deposit), MyMoneyMoney(moTransaction), acCredit, acChecking);
+        TransactionHelper t2(intermediateDate,
+                             MyMoneySplit::actionName(eMyMoney::Split::Action::Transfer),
+                             MyMoneyMoney(moJpyTransaction),
+                             jpyChecking,
+                             acChecking,
+                             "JPY");
+        TransactionHelper t5(intermediateDate, MyMoneySplit::actionName(eMyMoney::Split::Action::Deposit), MyMoneyMoney(moTransaction), acCredit, acChecking);
+        TransactionHelper t3(closingDate,
+                             MyMoneySplit::actionName(eMyMoney::Split::Action::Transfer),
+                             MyMoneyMoney(moJpyTransaction),
+                             jpyChecking,
+                             acChecking,
+                             "JPY");
+        TransactionHelper t6(closingDate, MyMoneySplit::actionName(eMyMoney::Split::Action::Deposit), MyMoneyMoney(moTransaction), acCredit, acChecking);
         // test that an income/expense transaction that involves a currency exchange is properly reported
-        TransactionHelper t7(intermediateDate, MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), MyMoneyMoney(moJpyTransaction), acJpyChecking, acSolo, "JPY");
+        TransactionHelper t7(intermediateDate,
+                             MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal),
+                             MyMoneyMoney(moJpyTransaction),
+                             jpyChecking,
+                             acSolo,
+                             "JPY");
 
         unsigned cols;
 
-        MyMoneyReport filter;
+        MyMoneyReport filter(QLatin1String("fake-id"));
 
         filter.setRowType(eMyMoney::Report::RowType::Account);
-        filter.setName("Transactions by Account");
+        filter.setName("Transactions by Account (multiple currencies)");
         cols = eMyMoney::Report::QueryColumn::Number | eMyMoney::Report::QueryColumn::Payee | eMyMoney::Report::QueryColumn::Category | eMyMoney::Report::QueryColumn::Balance;
         filter.setQueryColumns(static_cast<eMyMoney::Report::QueryColumn>(cols));
         // don't convert values to the default currency
         filter.setConvertCurrency(false);
         XMLandback(filter);
         QueryTable qtbl_3(filter);
-
-        writeTabletoHTML(qtbl_3, "Transactions by Account (multiple currencies).html");
+        writeTable(qtbl_3, filter.name());
 
         QString html = qtbl_3.renderHTML();
 
@@ -868,9 +900,9 @@ void QueryTableTest::testBalanceColumnWithMultipleCurrencies()
         QVERIFY(rows.count() == 24);
 
         //this is to make sure that the dates of closing and opening balances and the balance numbers are ok
-        QString openingDateString = QLocale().toString(openingDate, QLocale::ShortFormat);
-        QString intermediateDateString = QLocale().toString(intermediateDate, QLocale::ShortFormat);
-        QString closingDateString = QLocale().toString(closingDate, QLocale::ShortFormat);
+        QString openingDateString = MyMoneyUtils::formatDate(openingDate);
+        QString intermediateDateString = MyMoneyUtils::formatDate(intermediateDate);
+        QString closingDateString = MyMoneyUtils::formatDate(closingDate);
         // check the opening and closing balances
 
         QVERIFY(html.indexOf(openingDateString + "</td><td class=\"left0\"></td><td class=\"left0\">" + i18n("Opening Balance") + "</td><td class=\"left0\"></td><td class=\"value\"></td><td>USD&nbsp;0.00</td></tr>") > 0);
@@ -879,32 +911,53 @@ void QueryTableTest::testBalanceColumnWithMultipleCurrencies()
         QVERIFY(html.indexOf(closingDateString + "</td><td class=\"left0\"></td><td class=\"left0\">" + i18n("Closing Balance") + "</td><td class=\"left0\"></td><td class=\"value\"></td><td>JPY&nbsp;-400</td></tr>") > 0);
 
         // after a transfer of 100 JPY the balance should be 1.00 - price is 0.010 (precision of 2)
-        QVERIFY(html.indexOf("<a href=ledger?id=A000001&tid=T000000000000000001>" + openingDateString + "</a></td><td class=\"left0\"></td><td class=\"left0\">Test Payee</td><td class=\"left0\">Transfer from Japanese Checking</td><td class=\"value\">USD&nbsp;1.00</td><td>USD&nbsp;1.00</td></tr>") > 0);
+        QVERIFY(html.indexOf("<a href=\"/ledger?id=A000001&tid=T000000000000000001\">" + openingDateString
+                             + "</a></td><td class=\"left0\"></td><td class=\"left0\">Test Payee</td><td class=\"left0\">Transfer from Japanese "
+                               "Checking</td><td class=\"value\">USD&nbsp;1.00</td><td>USD&nbsp;1.00</td></tr>")
+                > 0);
 
         // after a transfer of 100 the balance should be 101.00
-        QVERIFY(html.indexOf("<a href=ledger?id=A000001&tid=T000000000000000002>" + openingDateString + "</a></td><td class=\"left0\"></td><td class=\"left0\">Test Payee</td><td class=\"left0\">Transfer from Credit Card</td><td class=\"value\">USD&nbsp;100.00</td><td>USD&nbsp;101.00</td></tr>") > 0);
+        QVERIFY(html.indexOf("<a href=\"/ledger?id=A000001&tid=T000000000000000002\">" + openingDateString
+                             + "</a></td><td class=\"left0\"></td><td class=\"left0\">Test Payee</td><td class=\"left0\">Transfer from Credit Card</td><td "
+                               "class=\"value\">USD&nbsp;100.00</td><td>USD&nbsp;101.00</td></tr>")
+                > 0);
 
         // after a transfer of 100 JPY the balance should be 102.00 - price is 0.011 (precision of 2)
-        QVERIFY(html.indexOf("<a href=ledger?id=A000001&tid=T000000000000000003>" + intermediateDateString + "</a></td><td class=\"left0\"></td><td class=\"left0\">Test Payee</td><td class=\"left0\">Transfer from Japanese Checking</td><td class=\"value\">USD&nbsp;1.00</td><td>USD&nbsp;102.00</td></tr>") > 0);
+        QVERIFY(html.indexOf("<a href=\"/ledger?id=A000001&tid=T000000000000000003\">" + intermediateDateString
+                             + "</a></td><td class=\"left0\"></td><td class=\"left0\">Test Payee</td><td class=\"left0\">Transfer from Japanese "
+                               "Checking</td><td class=\"value\">USD&nbsp;1.00</td><td>USD&nbsp;102.00</td></tr>")
+                > 0);
 
         // after a transfer of 100 the balance should be 202.00
-        QVERIFY(html.indexOf("<a href=ledger?id=A000001&tid=T000000000000000004>" + intermediateDateString + "</a></td><td class=\"left0\"></td><td class=\"left0\">Test Payee</td><td class=\"left0\">Transfer from Credit Card</td><td class=\"value\">USD&nbsp;100.00</td><td>USD&nbsp;202.00</td></tr>") > 0);
+        QVERIFY(html.indexOf("<a href=\"/ledger?id=A000001&tid=T000000000000000004\">" + intermediateDateString
+                             + "</a></td><td class=\"left0\"></td><td class=\"left0\">Test Payee</td><td class=\"left0\">Transfer from Credit Card</td><td "
+                               "class=\"value\">USD&nbsp;100.00</td><td>USD&nbsp;202.00</td></tr>")
+                > 0);
 
         // after a transfer of 100 JPY the balance should be 204.00 - price is 0.024 (precision of 2)
-        QVERIFY(html.indexOf("<a href=ledger?id=A000001&tid=T000000000000000005>" + closingDateString + "</a></td><td class=\"left0\"></td><td class=\"left0\">Test Payee</td><td class=\"left0\">Transfer from Japanese Checking</td><td class=\"value\">USD&nbsp;2.00</td><td>USD&nbsp;204.00</td></tr>") > 0);
+        QVERIFY(html.indexOf("<a href=\"/ledger?id=A000001&tid=T000000000000000005\">" + closingDateString
+                             + "</a></td><td class=\"left0\"></td><td class=\"left0\">Test Payee</td><td class=\"left0\">Transfer from Japanese "
+                               "Checking</td><td class=\"value\">USD&nbsp;2.00</td><td>USD&nbsp;204.00</td></tr>")
+                > 0);
 
         // after a transfer of 100 the balance should be 304.00
-        QVERIFY(html.indexOf("<a href=ledger?id=A000001&tid=T000000000000000006>" + closingDateString + "</a></td><td class=\"left0\"></td><td class=\"left0\">Test Payee</td><td class=\"left0\">Transfer from Credit Card</td><td class=\"value\">USD&nbsp;100.00</td><td>USD&nbsp;304.00</td></tr>") > 0);
+        QVERIFY(html.indexOf("<a href=\"/ledger?id=A000001&tid=T000000000000000006\">" + closingDateString
+                             + "</a></td><td class=\"left0\"></td><td class=\"left0\">Test Payee</td><td class=\"left0\">Transfer from Credit Card</td><td "
+                               "class=\"value\">USD&nbsp;100.00</td><td>USD&nbsp;304.00</td></tr>")
+                > 0);
 
         // a 100.00 JPY withdrawal should be displayed as such even if the expense account uses another currency
-        QVERIFY(html.indexOf("<a href=ledger?id=A000008&tid=T000000000000000007>" + intermediateDateString + "</a></td><td class=\"left0\"></td><td class=\"left0\">Test Payee</td><td class=\"left0\">Solo</td><td class=\"value\">JPY&nbsp;-100</td><td>JPY&nbsp;-300</td></tr>") > 0);
+        QVERIFY(html.indexOf("<a href=\"/ledger?id=A000008&tid=T000000000000000007\">" + intermediateDateString
+                             + "</a></td><td class=\"left0\"></td><td class=\"left0\">Test Payee</td><td class=\"left0\">Solo</td><td "
+                               "class=\"value\">JPY&nbsp;-100</td><td>JPY&nbsp;-300</td></tr>")
+                > 0);
 
         // now run the same report again but this time convert all values to the base currency and make sure the values are correct
         filter.setConvertCurrency(true);
+        filter.setName("Transactions by Account (multiple currencies converted to base)");
         XMLandback(filter);
         QueryTable qtbl_4(filter);
-
-        writeTabletoHTML(qtbl_4, "Transactions by Account (multiple currencies converted to base).html");
+        writeTable(qtbl_4, filter.name());
 
         html = qtbl_4.renderHTML();
 
@@ -920,16 +973,28 @@ void QueryTableTest::testBalanceColumnWithMultipleCurrencies()
         QVERIFY(html.indexOf(closingDateString + "</td><td class=\"left0\"></td><td class=\"left0\">" + i18n("Closing Balance") + "</td><td class=\"left0\"></td><td class=\"value\"></td><td>&nbsp;-8.00</td></tr>") > 0);
 
         // a 100.00 JPY transfer should be displayed as -1.00 when converted to the base currency using the opening date price
-        QVERIFY(html.indexOf("<a href=ledger?id=A000008&tid=T000000000000000001>" + openingDateString + "</a></td><td class=\"left0\"></td><td class=\"left0\">Test Payee</td><td class=\"left0\">Transfer to Checking Account</td><td class=\"value\">&nbsp;-1.00</td><td>&nbsp;-1.00</td></tr>") > 0);
+        QVERIFY(html.indexOf("<a href=\"/ledger?id=A000008&tid=T000000000000000001\">" + openingDateString
+                             + "</a></td><td class=\"left0\"></td><td class=\"left0\">Test Payee</td><td class=\"left0\">Transfer to Checking Account</td><td "
+                               "class=\"value\">&nbsp;-1.00</td><td>&nbsp;-1.00</td></tr>")
+                > 0);
 
         // a 100.00 JPY transfer should be displayed as -1.00 when converted to the base currency using the intermediate date price
-        QVERIFY(html.indexOf("<a href=ledger?id=A000008&tid=T000000000000000003>" + intermediateDateString + "</a></td><td class=\"left0\"></td><td class=\"left0\">Test Payee</td><td class=\"left0\">Transfer to Checking Account</td><td class=\"value\">&nbsp;-1.00</td><td>&nbsp;-2.00</td></tr>") > 0);
+        QVERIFY(html.indexOf("<a href=\"/ledger?id=A000008&tid=T000000000000000003\">" + intermediateDateString
+                             + "</a></td><td class=\"left0\"></td><td class=\"left0\">Test Payee</td><td class=\"left0\">Transfer to Checking Account</td><td "
+                               "class=\"value\">&nbsp;-1.00</td><td>&nbsp;-2.00</td></tr>")
+                > 0);
 
         // a 100.00 JPY transfer should be displayed as -2.00 when converted to the base currency using the closing date price (notice the balance is -5.00)
-        QVERIFY(html.indexOf("<a href=ledger?id=A000008&tid=T000000000000000005>" + closingDateString + "</a></td><td class=\"left0\"></td><td class=\"left0\">Test Payee</td><td class=\"left0\">Transfer to Checking Account</td><td class=\"value\">&nbsp;-2.00</td><td>&nbsp;-5.00</td></tr>") > 0);
+        QVERIFY(html.indexOf("<a href=\"/ledger?id=A000008&tid=T000000000000000005\">" + closingDateString
+                             + "</a></td><td class=\"left0\"></td><td class=\"left0\">Test Payee</td><td class=\"left0\">Transfer to Checking Account</td><td "
+                               "class=\"value\">&nbsp;-2.00</td><td>&nbsp;-5.00</td></tr>")
+                > 0);
 
         // a 100.00 JPY withdrawal should be displayed as -1.00 when converted to the base currency using the intermediate date price
-        QVERIFY(html.indexOf("<a href=ledger?id=A000008&tid=T000000000000000007>" + intermediateDateString + "</a></td><td class=\"left0\"></td><td class=\"left0\">Test Payee</td><td class=\"left0\">Solo</td><td class=\"value\">&nbsp;-1.00</td><td>&nbsp;-3.00</td></tr>") > 0);
+        QVERIFY(html.indexOf("<a href=\"/ledger?id=A000008&tid=T000000000000000007\">" + intermediateDateString
+                             + "</a></td><td class=\"left0\"></td><td class=\"left0\">Test Payee</td><td class=\"left0\">Solo</td><td "
+                               "class=\"value\">&nbsp;-1.00</td><td>&nbsp;-3.00</td></tr>")
+                > 0);
 
     } catch (const MyMoneyException &e) {
         QFAIL(e.what());
@@ -943,7 +1008,7 @@ void QueryTableTest::testTaxReport()
         TransactionHelper t2q1(QDate(2004, 2, 1), MyMoneySplit::actionName(eMyMoney::Split::Action::Withdrawal), moParent1, acChecking, acTax);
 
         unsigned cols;
-        MyMoneyReport filter;
+        MyMoneyReport filter(QLatin1String("fake-id"));
 
         filter.setRowType(eMyMoney::Report::RowType::Category);
         filter.setName("Tax Transactions");
@@ -953,12 +1018,11 @@ void QueryTableTest::testTaxReport()
 
         XMLandback(filter);
         QueryTable qtbl_3(filter);
-
-        writeTabletoHTML(qtbl_3, "Tax Transactions.html");
+        writeTable(qtbl_3, filter.name());
 
         QList<ListTable::TableRow> rows = qtbl_3.rows();
 
-        QString html = qtbl_3.renderHTML();
+        qtbl_3.renderHTML();
         QVERIFY(rows.count() == 5);
     } catch (const MyMoneyException &e) {
         QFAIL(e.what());
