@@ -9,17 +9,15 @@
 
 #include <QObject>
 #include "mymoneyfile.h"
-#include "mymoneystoragemgr.h"
 
 class QueryTableTest : public QObject
 {
     Q_OBJECT
 private:
-    MyMoneyStorageMgr* storage;
     MyMoneyFile* file;
 
 private Q_SLOTS:
-    void setup();
+    void initTestCase();
     void init();
     void cleanup();
     void testQueryBasics();
