@@ -89,13 +89,17 @@ KReportChartView* KBalanceChartDlg::drawChart(const MyMoneyAccount& account)
     reportCfg.setIncludingForecast(true);
     reportCfg.setIncludingBudgetActuals(true);
     if (account.accountType() == eMyMoney::Account::Type::Investment) {
-        foreach (const auto accountID, account.accountList())
+        const auto subAccountList = account.accountList();
+        for (const auto& accountID : qAsConst(subAccountList))
             reportCfg.addAccount(accountID);
-    } else
+    } else {
         reportCfg.addAccount(account.id());
+    }
     reportCfg.setColumnsAreDays(true);
     reportCfg.setConvertCurrency(false);
     reportCfg.setMixedTime(true);
+    reportCfg.setNegExpenses(MyMoneyAccount::balanceFactor(account.accountType()).isNegative());
+
     reports::PivotTable table(reportCfg);
 
     reports::KReportChartView* chartWidget = new reports::KReportChartView(this);
