@@ -1,5 +1,6 @@
 /*
     SPDX-FileCopyrightText: 2018 Łukasz Wojniłowicz lukasz.wojnilowicz @gmail.com
+    SPDX-FileCopyrightText: 2021 Dawid Wróbel <me@dawidwrobel.com>
     SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 */
 
@@ -15,17 +16,18 @@
 // ----------------------------------------------------------------------------
 // KDE Includes
 
-#include <KPluginFactory>
 #include <KAboutData>
+#include <KLineEdit>
 #include <KLocalizedString>
 #include <KMessageBox>
-#include <KLineEdit>
+#include <KPluginFactory>
+#include <kio_version.h>
 
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include "ui_reportsviewsettings.h"
 #include "kmymoneysettings.h"
+#include "ui_reportsviewsettings.h"
 
 class ReportsViewSettingsWidgetPrivate
 {
@@ -70,7 +72,7 @@ public:
         QFileInfo* info = new QFileInfo(css);
 
         if (!info->exists()) {
-            KMessageBox::sorry(0, i18n("File %1 does not exist", css));
+            KMessageBox::error(0, i18n("File %1 does not exist", css));
             return;
         }
 
@@ -96,11 +98,11 @@ public:
         QString out = i18np("There is a problem with file %1", "There are problems with file %1", css);
 
         QList<QString>::const_iterator i;
-        for (i = warnings.constBegin(); i != warnings.constEnd(); ++i) {
+        for (i = warnings.cbegin(); i != warnings.cend(); ++i) {
             out += '\n' + *i;
         }
 
-        KMessageBox::sorry(0, out);
+        KMessageBox::error(0, out);
     }
 
     Ui::ReportsViewSettings *ui;
@@ -125,6 +127,12 @@ ReportsViewSettingsWidget::ReportsViewSettingsWidget(QWidget* parent) :
     Q_D(ReportsViewSettingsWidget);
     d->ui->setupUi(this);
 
+#if KIO_VERSION >= QT_VERSION_CHECK(5, 108, 0)
+    d->ui->kcfg_CssFileDefault->setNameFilters({i18n("CSS files") + QLatin1String(" (*.css)"), i18n("All files") + QLatin1String(" (*)")});
+#else
+    d->ui->kcfg_CssFileDefault->setFilter(QLatin1String("*.css|") + i18n("CSS files") + QLatin1String("\n*|") + i18n("All files"));
+#endif
+
     // keep initial (default) css file in mind
     d->m_cssFileOld = KMyMoneySettings::cssFileDefault();
 
@@ -179,14 +187,14 @@ void ReportsViewSettingsWidget::slotEditingFinished()
     d->checkCssFile(txt);
 }
 
-KCMReportsView::KCMReportsView(QWidget *parent, const QVariantList& args)
-    : KCModule(parent, args)
+KCMReportsView::KCMReportsView(QObject* parent, const QVariantList& args)
+    : KMMKCModule(parent, args)
 {
-    ReportsViewSettingsWidget* w = new ReportsViewSettingsWidget(this);
+    ReportsViewSettingsWidget* w = new ReportsViewSettingsWidget(widget());
     // addConfig(ReportsViewSettings::self(), w);
     addConfig(KMyMoneySettings::self(), w);
     QVBoxLayout *layout = new QVBoxLayout;
-    setLayout(layout);
+    widget()->setLayout(layout);
     layout->addWidget(w);
     setButtons(NoAdditionalButton);
     load();
@@ -196,6 +204,6 @@ KCMReportsView::~KCMReportsView()
 {
 }
 
-K_PLUGIN_FACTORY_WITH_JSON(KCMReportsViewFactory, "kcm_reportsview.json", registerPlugin<KCMReportsView>();)
+K_PLUGIN_CLASS(KCMReportsView)
 
 #include "kcm_reportsview.moc"
