@@ -119,7 +119,7 @@ KReportConfigurationFilterDlg::KReportConfigurationFilterDlg(MyMoneyReport repor
     //
     if (d->m_initialState.reportType() == eMyMoney::Report::ReportType::PivotTable) {
         // we will use date range together with data range
-        d->m_tabFilters = new KTransactionFilter(this, (report.rowType() == eMyMoney::Report::RowType::Account), report.isInvestmentsOnly(), false);
+        d->m_tabFilters = new KTransactionFilter(this, (report.rowType() == eMyMoney::Report::RowType::Account), true, false);
     } else {
         d->m_tabFilters = new KTransactionFilter(this, (report.rowType() == eMyMoney::Report::RowType::Account), report.isInvestmentsOnly());
         d->m_dateRange = d->m_tabFilters->dateRange();
@@ -145,7 +145,7 @@ KReportConfigurationFilterDlg::KReportConfigurationFilterDlg(MyMoneyReport repor
         d->ui->m_criteriaTab->insertTab(tabNr++, d->m_tabChart, i18n("Chart"));
 
         d->m_tabRange = new ReportTabRange(d->ui->m_criteriaTab);
-        d->ui->m_criteriaTab->insertTab(tabNr++, d->m_tabRange, i18n("Range"));
+        d->ui->m_criteriaTab->insertTab(tabNr, d->m_tabRange, i18n("Range"));
 
         d->m_dateRange = d->m_tabRange->m_dateRange;
 
@@ -177,7 +177,7 @@ KReportConfigurationFilterDlg::KReportConfigurationFilterDlg(MyMoneyReport repor
 
     QList<MyMoneyBudget> list = MyMoneyFile::instance()->budgetList();
     QList<MyMoneyBudget>::const_iterator it_b;
-    for (it_b = list.constBegin(); it_b != list.constEnd(); ++it_b) {
+    for (it_b = list.cbegin(); it_b != list.cend(); ++it_b) {
         d->m_budgets.push_back(*it_b);
     }
 
@@ -230,12 +230,12 @@ void KReportConfigurationFilterDlg::slotSearch()
 
         d->m_currentState.setShowingColumnTotals(d->m_tabRowColPivot->ui->m_checkTotalRow->isChecked());
         d->m_currentState.setIncludingSchedules(d->m_tabRowColPivot->ui->m_checkScheduled->isChecked());
-
+        d->m_currentState.setPropagateBudgetDifference(d->m_tabRowColPivot->ui->m_propagateRemainder->isChecked());
         d->m_currentState.setIncludingTransfers(d->m_tabRowColPivot->ui->m_checkTransfers->isChecked());
 
         d->m_currentState.setIncludingUnusedAccounts(d->m_tabRowColPivot->ui->m_checkUnused->isChecked());
 
-        if (d->m_tabRowColPivot->ui->m_comboBudget->isEnabled()) {
+        if (d->m_tabRowColPivot->ui->m_comboBudget->isEnabled() && (d->m_budgets.count() > 0)) {
             d->m_currentState.setBudget(d->m_budgets[d->m_tabRowColPivot->ui->m_comboBudget->currentItem()].id(), d->m_initialState.rowType() == eMyMoney::Report::RowType::BudgetActual);
         } else {
             d->m_currentState.setBudget(QString(), false);
@@ -443,17 +443,35 @@ void KReportConfigurationFilterDlg::slotReset()
         }
         d->m_tabRowColPivot->ui->m_checkTotalColumn->setChecked(d->m_initialState.isShowingRowTotals());
         d->m_tabRowColPivot->ui->m_checkTotalRow->setChecked(d->m_initialState.isShowingColumnTotals());
+        d->m_tabRowColPivot->ui->m_propagateRemainder->setEnabled(d->m_initialState.rowType() == eMyMoney::Report::RowType::BudgetActual);
+        d->m_tabRowColPivot->ui->m_propagateRemainder->setChecked(d->m_initialState.isPropagateBudgetDifference());
+        d->m_tabRowColPivot->ui->m_checkTotalRow->setDisabled(d->m_initialState.isPropagateBudgetDifference());
+
+        connect(d->m_tabRowColPivot->ui->m_propagateRemainder, &QCheckBox::stateChanged, this, [&](int _state) {
+            Q_D(KReportConfigurationFilterDlg);
+            const auto state = static_cast<Qt::CheckState>(_state);
+            d->m_tabRowColPivot->ui->m_checkTotalColumn->setDisabled(state == Qt::Checked);
+            switch (state) {
+            case Qt::Checked:
+                d->m_tabRowColPivot->ui->m_checkTotalColumn->setChecked(false);
+                break;
+            default:
+                break;
+            }
+        });
 
         slotRowTypeChanged(combo->currentIndex());
 
         //load budgets combo
+        d->m_tabRowColPivot->ui->m_comboBudget->setDisabled(true);
         if (d->m_initialState.rowType() == eMyMoney::Report::RowType::Budget
                 || d->m_initialState.rowType() == eMyMoney::Report::RowType::BudgetActual) {
+            d->m_tabRowColPivot->ui->m_comboBudget->setEnabled(true);
             d->m_tabRowColPivot->ui->m_comboRows->setEnabled(false);
             d->m_tabRowColPivot->ui->m_rowsLabel->setEnabled(false);
             d->m_tabRowColPivot->ui->m_budgetFrame->setEnabled(!d->m_budgets.empty());
             auto i = 0;
-            for (QVector<MyMoneyBudget>::const_iterator it_b = d->m_budgets.constBegin(); it_b != d->m_budgets.constEnd(); ++it_b) {
+            for (QVector<MyMoneyBudget>::const_iterator it_b = d->m_budgets.cbegin(); it_b != d->m_budgets.cend(); ++it_b) {
                 d->m_tabRowColPivot->ui->m_comboBudget->insertItem((*it_b).name(), i);
                 //set the current selected item
                 if ((d->m_initialState.budget() == "Any" && (*it_b).budgetStart().year() == QDate::currentDate().year())
