@@ -7,6 +7,7 @@
     SPDX-FileCopyrightText: 2000-2004 Kevin Tambascio <ktambascio@users.sourceforge.net>
     SPDX-FileCopyrightText: 2000-2004 Ace Jones <ace.j@hotpop.com>
     SPDX-FileCopyrightText: 2017 Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
+    SPDX-FileCopyrightText: 2021 Dawid Wróbel <me@dawidwrobel.com>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
@@ -17,26 +18,11 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
-#include <QTreeWidget>
-#include <QTreeWidgetItem>
-#include <QFile>
-#include <QTimer>
-#include <QClipboard>
-#include <QList>
-#include <QVBoxLayout>
-#include <QMimeData>
-#include <QIcon>
-#include <QUrlQuery>
-#include <QFileInfo>
 #include <QFileDialog>
 #include <QLocale>
-#include <QTextCodec>
 #include <QMenu>
-#ifdef ENABLE_WEBENGINE
-#include <QWebEngineView>
-#else
-#include <KWebView>
-#endif
+#include <QTreeWidget>
+#include <QUrlQuery>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
@@ -50,25 +36,25 @@
 
 #include "ui_reportcontrol.h"
 
+#include "icons.h"
+#include "journalmodel.h"
+#include "kbalancechartdlg.h"
+#include "kmymoneysettings.h"
+#include "kreportchartview.h"
+#include "kreportconfigurationfilterdlg.h"
+#include "menuenums.h"
+#include "mymoneyenums.h"
+#include "mymoneyexception.h"
 #include "mymoneyfile.h"
 #include "mymoneyreport.h"
-#include "mymoneyexception.h"
-#include "kmymoneysettings.h"
-#include "querytable.h"
 #include "objectinfotable.h"
-#include "kreportconfigurationfilterdlg.h"
-#include "icons/icons.h"
-#include "kbalancechartdlg.h"
-#include <kmymoneywebpage.h>
+#include "pivottable.h"
+#include "querytable.h"
+#include "reportcontrolimpl.h"
+#include "reporttable.h"
 #include "tocitem.h"
 #include "tocitemgroup.h"
 #include "tocitemreport.h"
-#include "kreportchartview.h"
-#include "pivottable.h"
-#include "reporttable.h"
-#include "reportcontrolimpl.h"
-#include "mymoneyenums.h"
-#include "menuenums.h"
 
 using namespace reports;
 using namespace eMyMoney;
@@ -80,6 +66,29 @@ using namespace Icons;
 #define VIEW_HOME           "home"
 #define VIEW_REPORTS        "reports"
 
+/**
+ * Creates a unique file name in a specific directory
+ * @param rootDir Directory used to search for a unique file name
+ * @param reportName Name of the report to create a file name
+ * @param extension Extension to add to the file name
+ * @returns the absolute file path or an empty string if no free name was found after more than 100 attempts
+ */
+static QString createSaveFileName(const QString& rootDir, const QString& reportName, const QString& extension)
+{
+    static QRegularExpression rx(QLatin1String("\\s+"));
+    QString name(reportName);
+    name.replace(rx, "-");
+    QString suffix;
+    QFileInfo fi;
+    int index = 0;
+    do {
+        fi.setFile(rootDir + QDir::separator() + name + suffix + extension);
+        suffix = QString(QLatin1String("-%1")).arg(++index);
+    } while (fi.exists() && index < 100);
+
+    return index < 100 ? fi.absoluteFilePath() : QString();
+}
+
 /**
   * KReportsView Implementation
   */
@@ -87,40 +96,58 @@ using namespace Icons;
 KReportsView::KReportsView(QWidget *parent) :
     KMyMoneyViewBase(*new KReportsViewPrivate(this), parent)
 {
-    connect(pActions[eMenu::Action::ReportAccountTransactions], &QAction::triggered, this, &KReportsView::slotReportAccountTransactions);
+    connect(pActions[eMenu::Action::ReportNew], &QAction::triggered, this, &KReportsView::slotDuplicate);
+    connect(pActions[eMenu::Action::ReportConfigure], &QAction::triggered, this, &KReportsView::slotConfigure);
+    connect(pActions[eMenu::Action::ReportExport], &QAction::triggered, this, &KReportsView::slotExportView);
+    connect(pActions[eMenu::Action::ReportDelete], &QAction::triggered, this, &KReportsView::slotDelete);
+    connect(pActions[eMenu::Action::ReportClose], &QAction::triggered, this, &KReportsView::slotCloseCurrent);
+
+    pActions[eMenu::Action::ReportNew]->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_Plus));
+    pActions[eMenu::Action::ReportConfigure]->setShortcut(QKeySequence(Qt::SHIFT | Qt::Key_P));
+    pActions[eMenu::Action::ReportDelete]->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_Delete));
+    pActions[eMenu::Action::ReportClose]->setShortcut(QKeySequence(Qt::SHIFT | Qt::Key_W));
 }
 
 KReportsView::~KReportsView()
 {
 }
 
-void KReportsView::executeCustomAction(eView::Action action)
+void KReportsView::executeAction(eMenu::Action action, const SelectedObjects& selections)
 {
     Q_D(KReportsView);
-    switch(action) {
-    case eView::Action::Refresh:
-        refresh();
+    switch (action) {
+    case eMenu::Action::ReportOpen:
+        slotOpenReport(selections.firstSelection(SelectedObjects::Report));
         break;
 
-    case eView::Action::SetDefaultFocus:
-        QTimer::singleShot(0, d->m_tocTreeWidget, SLOT(setFocus()));
+    case eMenu::Action::Print:
+        if (d->isActiveView()) {
+            slotPrintView();
+        }
         break;
 
-    case eView::Action::Print:
-        slotPrintView();
+    case eMenu::Action::PrintPreview:
+        if (d->isActiveView()) {
+            slotPrintPreviewView();
+        }
         break;
 
-    case eView::Action::CleanupBeforeFileClose:
+    case eMenu::Action::ChartAccountBalance: {
+        const auto account = MyMoneyFile::instance()->accountsModel()->itemById(selections.firstSelection(SelectedObjects::Account));
+        if (!account.id().isEmpty()) {
+            QPointer<KBalanceChartDlg> dlg = new KBalanceChartDlg(account, this);
+            dlg->exec();
+            delete dlg;
+        }
+    } break;
+
+    case eMenu::Action::FileClose:
         slotCloseAll();
         break;
 
-    case eView::Action::ShowBalanceChart:
-    {
-        QPointer<KBalanceChartDlg> dlg = new KBalanceChartDlg(d->m_currentAccount, this);
-        dlg->exec();
-        delete dlg;
-    }
-    break;
+    case eMenu::Action::ReportAccountTransactions:
+        d->showTransactionReport();
+        break;
 
     default:
         break;
@@ -140,53 +167,114 @@ void KReportsView::refresh()
 
 void KReportsView::showEvent(QShowEvent * event)
 {
-    if (MyMoneyFile::instance()->storageAttached()) {
-        Q_D(KReportsView);
-        if (d->m_needLoad)
-            d->init();
+    Q_D(KReportsView);
+    if (d->m_needLoad) {
+        d->init();
+        connect(d->ui.m_closeButton, &QToolButton::clicked, this, [&]() {
+            Q_D(KReportsView);
+            d->ui.m_filterContainer->hide();
+            d->ui.m_searchWidget->clear();
+            d->ui.m_reportTabWidget->setFocus();
+        });
+        connect(pActions[eMenu::Action::ShowFilterWidget], &QAction::triggered, this, [&]() {
+            if (isVisible()) {
+                Q_D(KReportsView);
+                d->ui.m_filterContainer->show();
+                d->ui.m_searchWidget->setFocus();
+            }
+        });
+        connect(d->ui.m_searchWidget, &QLineEdit::textChanged, this, [&](const QString& text) {
+            Q_D(KReportsView);
+            d->setFilter(text);
+        });
 
-        emit customActionRequested(View::Reports, eView::Action::AboutToShow);
-
-        if (d->m_needsRefresh)
-            refresh();
-
-        if (auto tab = dynamic_cast<KReportTab*>(d->m_reportTabWidget->currentWidget()))
-            emit reportSelected(tab->report());
-        else
-            emit reportSelected(MyMoneyReport());
+        // remove close button from list tab
+        QTabBar* bar = d->ui.m_reportTabWidget->findChild<QTabBar*>();
+        if (bar) {
+            QTabBar::ButtonPosition closeSide =
+                (QTabBar::ButtonPosition)style()->styleHint(QStyle::SH_TabBar_CloseButtonPosition, 0, d->ui.m_reportTabWidget->widget(0));
+            QWidget* w = bar->tabButton(0, closeSide);
+            bar->setTabButton(0, closeSide, nullptr);
+            w->deleteLater();
+        }
     }
+    if (d->m_needsRefresh) {
+        refresh();
+        d->setFilter(d->ui.m_searchWidget->text());
+    }
+
+    if (auto tab = dynamic_cast<KReportTab*>(d->ui.m_reportTabWidget->currentWidget()))
+        Q_EMIT reportSelected(tab->report());
+    else
+        Q_EMIT reportSelected(MyMoneyReport());
+
     // don't forget base class implementation
     QWidget::showEvent(event);
 }
 
-void KReportsView::updateActions(const MyMoneyObject& obj)
+bool KReportsView::eventFilter(QObject* watched, QEvent* event)
 {
     Q_D(KReportsView);
-    if (typeid(obj) != typeid(MyMoneyAccount) &&
-            (obj.id().isEmpty() && d->m_currentAccount.id().isEmpty())) // do not disable actions that were already disabled)))
-        return;
 
-    const auto& acc = static_cast<const MyMoneyAccount&>(obj);
+    if (event->type() == QEvent::KeyPress) {
+        if (watched == d->ui.m_searchWidget || watched == d->ui.m_tocTreeWidget) {
+            const auto kev = static_cast<QKeyEvent*>(event);
+            if (kev->modifiers() == Qt::NoModifier && kev->key() == Qt::Key_Escape) {
+                d->ui.m_closeButton->animateClick();
+                return true;
+            }
+        }
 
-    bool b;
-    if (MyMoneyFile::instance()->isStandardAccount(acc.id())) {
-        b = false;
-    } else {
-        switch (acc.accountType()) {
-        case eMyMoney::Account::Type::Asset:
-        case eMyMoney::Account::Type::Liability:
-        case eMyMoney::Account::Type::Equity:
-        case eMyMoney::Account::Type::Checkings:
-            b = true;
-            break;
-        default:
-            b = false;
-            break;
+    } else if (event->type() == QEvent::ShortcutOverride) {
+        const auto kev = static_cast<QKeyEvent*>(event);
+        const auto keySeq = QKeySequence(kev->modifiers() | kev->key());
+        const auto buttons = findChildren<QToolButton*>();
+        for (const auto& button : qAsConst(buttons)) {
+            const auto action = button->defaultAction();
+            if (action) {
+                const auto actionSeq = action->shortcut();
+                if (keySeq == actionSeq) {
+                    action->trigger();
+                    event->accept();
+                    return true;
+                }
+            }
         }
     }
-    pActions[eMenu::Action::ReportAccountTransactions]->setEnabled(b);
 
-    d->m_currentAccount = acc;
+    return QWidget::eventFilter(watched, event);
+}
+
+void KReportsView::updateActions(const SelectedObjects& selections)
+{
+    Q_D(KReportsView);
+    bool enable = false;
+
+    if (!selections.selection(SelectedObjects::Account).isEmpty()) {
+        const auto file = MyMoneyFile::instance();
+        const auto accId = selections.selection(SelectedObjects::Account).at(0);
+        if (!file->isStandardAccount(accId)) {
+            d->m_currentAccount = file->accountsModel()->itemById(accId);
+            switch (d->m_currentAccount.accountType()) {
+            case eMyMoney::Account::Type::Asset:
+            case eMyMoney::Account::Type::Liability:
+            case eMyMoney::Account::Type::Equity:
+            case eMyMoney::Account::Type::Checkings:
+                enable = true;
+                break;
+            default:
+                break;
+            }
+        }
+    }
+    pActions[eMenu::Action::ReportAccountTransactions]->setEnabled(enable);
+
+    // only access the widgets if they are initialized
+    if (!d->m_needLoad) {
+        if (auto tab = dynamic_cast<KReportTab*>(d->ui.m_reportTabWidget->currentWidget())) {
+            tab->enableAllReportActions();
+        }
+    }
 }
 
 void KReportsView::slotOpenUrl(const QUrl &url)
@@ -204,10 +292,8 @@ void KReportsView::slotOpenUrl(const QUrl &url)
             // slotRefreshView();
         } else if (command == QLatin1String("print"))
             slotPrintView();
-        else if (command == QLatin1String("copy"))
-            slotCopyView();
         else if (command == QLatin1String("save"))
-            slotSaveView();
+            slotExportView();
         else if (command == QLatin1String("configure"))
             slotConfigure();
         else if (command == QLatin1String("duplicate"))
@@ -220,7 +306,22 @@ void KReportsView::slotOpenUrl(const QUrl &url)
             qWarning() << i18n("Unknown command '%1' in KReportsView::slotOpenUrl()", qPrintable(command));
 
     } else if (view == VIEW_LEDGER) {
-        emit selectByVariant(QVariantList {QVariant(id), QVariant(tid)}, eView::Intent::ShowTransaction);
+        const auto gotoAccount = pActions[eMenu::Action::GoToAccount];
+        gotoAccount->setData(id);
+
+        // convert transaction id to journalEntryId and make it the current selection
+        SelectedObjects selection;
+        const auto indexes = MyMoneyFile::instance()->journalModel()->indexesByTransactionId(tid);
+        for (const auto& idx : indexes) {
+            if (idx.data(eMyMoney::Model::JournalSplitAccountIdRole).toString() == id) {
+                tid = idx.data(eMyMoney::Model::IdRole).toString();
+                break;
+            }
+        }
+        selection.setSelection(SelectedObjects::JournalEntry, tid);
+        Q_EMIT requestSelectionChange(selection);
+
+        gotoAccount->trigger();
     } else {
         qWarning() << i18n("Unknown view '%1' in KReportsView::slotOpenUrl()", qPrintable(view));
     }
@@ -229,29 +330,42 @@ void KReportsView::slotOpenUrl(const QUrl &url)
 void KReportsView::slotPrintView()
 {
     Q_D(KReportsView);
-    if (auto tab = dynamic_cast<KReportTab*>(d->m_reportTabWidget->currentWidget()))
+    if (auto tab = dynamic_cast<KReportTab*>(d->ui.m_reportTabWidget->currentWidget()))
         tab->print();
 }
 
-void KReportsView::slotCopyView()
+void KReportsView::slotPrintPreviewView()
 {
     Q_D(KReportsView);
-    if (auto tab = dynamic_cast<KReportTab*>(d->m_reportTabWidget->currentWidget()))
-        tab->copyToClipboard();
+    if (auto tab = dynamic_cast<KReportTab*>(d->ui.m_reportTabWidget->currentWidget()))
+        tab->printPreview();
 }
 
-void KReportsView::slotSaveView()
+void KReportsView::slotExportView()
 {
     Q_D(KReportsView);
-    if (auto tab = dynamic_cast<KReportTab*>(d->m_reportTabWidget->currentWidget())) {
-        QString filterList = i18nc("CSV (Filefilter)", "CSV files") + QLatin1String(" (*.csv);;") + i18nc("HTML (Filefilter)", "HTML files") + QLatin1String(" (*.html)");
-        QUrl newURL = QFileDialog::getSaveFileUrl(this, i18n("Export as"), QUrl::fromLocalFile(KRecentDirs::dir(":kmymoney-export")), filterList, &d->m_selectedExportFilter);
+    if (auto tab = dynamic_cast<KReportTab*>(d->ui.m_reportTabWidget->currentWidget())) {
+        QPointer<QFileDialog> dialog = new QFileDialog(this, i18n("Export as"), KRecentDirs::dir(":kmymoney-export"));
+        dialog->setMimeTypeFilters(
+            {QStringLiteral("text/csv"), QStringLiteral("text/html"), QStringLiteral("application/xml"), QStringLiteral("application/pdf")});
+        dialog->setFileMode(QFileDialog::AnyFile);
+        dialog->setAcceptMode(QFileDialog::AcceptSave);
+        dialog->selectFile(tab->report().name());
+
+        QUrl newURL;
+        QString selectedMimeType;
+        if (dialog->exec() == QDialog::Accepted) {
+            newURL = dialog->selectedUrls().first();
+            selectedMimeType = dialog->selectedMimeTypeFilter();
+        }
+        delete dialog;
+
         if (!newURL.isEmpty()) {
             KRecentDirs::add(":kmymoney-export", newURL.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash).path());
             QString newName = newURL.toDisplayString(QUrl::PreferLocalFile);
 
             try {
-                tab->saveAs(newName, true);
+                tab->saveAs(newName, selectedMimeType);
             } catch (const MyMoneyException &e) {
                 KMessageBox::error(this, i18n("Failed to save: %1", QString::fromLatin1(e.what())));
             }
@@ -260,15 +374,20 @@ void KReportsView::slotSaveView()
 }
 
 void KReportsView::slotConfigure()
+{
+    doConfigure(NoConfigureOption);
+}
+
+void KReportsView::doConfigure(ConfigureOption configureOption)
 {
     Q_D(KReportsView);
     QString cm = "KReportsView::slotConfigure";
 
-    auto tab = dynamic_cast<KReportTab*>(d->m_reportTabWidget->currentWidget());
+    auto tab = dynamic_cast<KReportTab*>(d->ui.m_reportTabWidget->currentWidget());
 
     if (!tab) // nothing to do
         return;
-    int tabNr = d->m_reportTabWidget->currentIndex();
+    int tabNr = d->ui.m_reportTabWidget->currentIndex();
 
     tab->updateDataRange(); // range will be needed during configuration, but cannot be obtained earlier
 
@@ -291,8 +410,8 @@ void KReportsView::slotConfigure()
                 ft.commit();
                 tab->modifyReport(newreport);
 
-                d->m_reportTabWidget->setTabText(tabNr, newreport.name());
-                d->m_reportTabWidget->setCurrentIndex(tabNr) ;
+                d->ui.m_reportTabWidget->setTabText(tabNr, newreport.name());
+                d->ui.m_reportTabWidget->setCurrentIndex(tabNr);
             } else {
                 MyMoneyFile::instance()->addReport(newreport);
                 ft.commit();
@@ -308,7 +427,7 @@ void KReportsView::slotConfigure()
                     qWarning() << cm << error;
 
                     // also inform user
-                    KMessageBox::error(d->m_reportTabWidget, error, i18n("Critical Error"));
+                    KMessageBox::error(d->ui.m_reportTabWidget, error, i18n("Critical Error"));
 
                     // cleanup
                     delete dlg;
@@ -319,11 +438,13 @@ void KReportsView::slotConfigure()
                 // do not add TocItemReport to TocItemGroup here,
                 // this is done in loadView
 
-                d->addReportTab(newreport);
+                d->addReportTab(newreport, OpenImmediately);
             }
         } catch (const MyMoneyException &e) {
             KMessageBox::error(this, i18n("Failed to configure report: %1", QString::fromLatin1(e.what())));
         }
+    } else if (configureOption == LoadReportOnCancel) {
+        tab->loadTab();
     }
     delete dlg;
 }
@@ -333,7 +454,7 @@ void KReportsView::slotDuplicate()
     Q_D(KReportsView);
     QString cm = "KReportsView::slotDuplicate";
 
-    auto tab = dynamic_cast<KReportTab*>(d->m_reportTabWidget->currentWidget());
+    auto tab = dynamic_cast<KReportTab*>(d->ui.m_reportTabWidget->currentWidget());
 
     if (!tab) {
         // nothing to do
@@ -365,7 +486,7 @@ void KReportsView::slotDuplicate()
                 qWarning() << cm << error;
 
                 // also inform user
-                KMessageBox::error(d->m_reportTabWidget, error, i18n("Critical Error"));
+                KMessageBox::error(d->ui.m_reportTabWidget, error, i18n("Critical Error"));
 
                 // cleanup
                 delete dlg;
@@ -376,7 +497,7 @@ void KReportsView::slotDuplicate()
             // do not add TocItemReport to TocItemGroup here,
             // this is done in loadView
 
-            d->addReportTab(newReport);
+            d->addReportTab(newReport, OpenImmediately);
         } catch (const MyMoneyException &e) {
             QString error = i18n("Cannot add report, reason: \"%1\"", e.what());
 
@@ -384,7 +505,7 @@ void KReportsView::slotDuplicate()
             qWarning() << cm << error;
 
             // also inform user
-            KMessageBox::error(d->m_reportTabWidget, error, i18n("Critical Error"));
+            KMessageBox::error(d->ui.m_reportTabWidget, error, i18n("Critical Error"));
         }
     }
     delete dlg;
@@ -393,7 +514,7 @@ void KReportsView::slotDuplicate()
 void KReportsView::slotDelete()
 {
     Q_D(KReportsView);
-    auto tab = dynamic_cast<KReportTab*>(d->m_reportTabWidget->currentWidget());
+    auto tab = dynamic_cast<KReportTab*>(d->ui.m_reportTabWidget->currentWidget());
     if (!tab) {
         // nothing to do
         return;
@@ -404,7 +525,7 @@ void KReportsView::slotDelete()
         if (KMessageBox::Continue == d->deleteReportDialog(report.name())) {
             // close the tab and then remove the report so that it is not
             // generated again during the following loadView() call
-            slotClose(d->m_reportTabWidget->currentIndex());
+            slotClose(d->ui.m_reportTabWidget->currentIndex());
 
             MyMoneyFileTransaction ft;
             MyMoneyFile::instance()->removeReport(report);
@@ -418,11 +539,12 @@ void KReportsView::slotDelete()
     }
 }
 
+
 void KReportsView::slotOpenReport(const QString& id)
 {
     Q_D(KReportsView);
     if (id.isEmpty()) {
-        // nothing to  do
+        // nothing to do
         return;
     }
 
@@ -430,8 +552,8 @@ void KReportsView::slotOpenReport(const QString& id)
 
     // Find the tab which contains the report
     int index = 1;
-    while (index < d->m_reportTabWidget->count()) {
-        auto current = dynamic_cast<KReportTab*>(d->m_reportTabWidget->widget(index));
+    while (index < d->ui.m_reportTabWidget->count()) {
+        auto current = dynamic_cast<KReportTab*>(d->ui.m_reportTabWidget->widget(index));
 
         if (current && current->report().id() == id) {
             page = current;
@@ -443,9 +565,9 @@ void KReportsView::slotOpenReport(const QString& id)
 
     // Show the tab, or create a new one, as needed
     if (page)
-        d->m_reportTabWidget->setCurrentIndex(index);
+        d->ui.m_reportTabWidget->setCurrentIndex(index);
     else
-        d->addReportTab(MyMoneyFile::instance()->report(id));
+        d->addReportTab(MyMoneyFile::instance()->report(id), OpenImmediately);
 }
 
 void KReportsView::slotOpenReport(const MyMoneyReport& report)
@@ -453,13 +575,12 @@ void KReportsView::slotOpenReport(const MyMoneyReport& report)
     Q_D(KReportsView);
     if (d->m_needLoad)
         d->init();
-    qDebug() << Q_FUNC_INFO << " " << report.name();
     KReportTab* page = 0;
 
     // Find the tab which contains the report indicated by this list item
     int index = 1;
-    while (index < d->m_reportTabWidget->count()) {
-        auto current = dynamic_cast<KReportTab*>(d->m_reportTabWidget->widget(index));
+    while (index < d->ui.m_reportTabWidget->count()) {
+        auto current = dynamic_cast<KReportTab*>(d->ui.m_reportTabWidget->widget(index));
 
         if (current && current->report().name() == report.name()) {
             page = current;
@@ -471,16 +592,23 @@ void KReportsView::slotOpenReport(const MyMoneyReport& report)
 
     // Show the tab, or create a new one, as needed
     if (page)
-        d->m_reportTabWidget->setCurrentIndex(index);
+        d->ui.m_reportTabWidget->setCurrentIndex(index);
     else
-        d->addReportTab(report);
+        d->addReportTab(report, OpenImmediately);
 
     if (!isVisible())
-        emit switchViewRequested(View::Reports);
+        Q_EMIT switchViewRequested(View::Reports);
 }
 
-void KReportsView::slotItemDoubleClicked(QTreeWidgetItem* item, int)
+void KReportsView::slotItemDoubleClicked(QTreeWidgetItem* item, int column)
 {
+    doItemDoubleClicked(item, column, OpenImmediately);
+}
+
+void KReportsView::doItemDoubleClicked(QTreeWidgetItem* item, int column, OpenOption openOption)
+{
+    Q_UNUSED(column)
+
     Q_D(KReportsView);
     auto tocItem = dynamic_cast<TocItem*>(item);
     if (tocItem && !tocItem->isReport()) {
@@ -499,8 +627,8 @@ void KReportsView::slotItemDoubleClicked(QTreeWidgetItem* item, int)
 
     // Find the tab which contains the report indicated by this list item
     int index = 1;
-    while (index < d->m_reportTabWidget->count()) {
-        auto current = dynamic_cast<KReportTab*>(d->m_reportTabWidget->widget(index));
+    while (index < d->ui.m_reportTabWidget->count()) {
+        auto current = dynamic_cast<KReportTab*>(d->ui.m_reportTabWidget->widget(index));
         if (current) {
             // If this report has an ID, we'll use the ID to match
             if (! report.id().isEmpty()) {
@@ -524,29 +652,44 @@ void KReportsView::slotItemDoubleClicked(QTreeWidgetItem* item, int)
 
     // Show the tab, or create a new one, as needed
     if (page)
-        d->m_reportTabWidget->setCurrentIndex(index);
+        d->ui.m_reportTabWidget->setCurrentIndex(index);
     else
-        d->addReportTab(report);
+        d->addReportTab(report, openOption);
 }
 
 void KReportsView::slotToggleChart()
 {
     Q_D(KReportsView);
-    if (auto tab = dynamic_cast<KReportTab*>(d->m_reportTabWidget->currentWidget()))
+    if (auto tab = dynamic_cast<KReportTab*>(d->ui.m_reportTabWidget->currentWidget()))
         tab->toggleChart();
 }
 
+bool KReportsView::hasClosableView() const
+{
+    Q_D(const KReportsView);
+    return d->ui.m_reportTabWidget->count() > 1;
+}
+
+void KReportsView::closeCurrentView()
+{
+    Q_D(KReportsView);
+    const auto idx = d->ui.m_reportTabWidget->currentIndex();
+    if (idx > 0) {
+        slotClose(idx);
+    }
+}
+
 void KReportsView::slotCloseCurrent()
 {
     Q_D(KReportsView);
-    slotClose(d->m_reportTabWidget->currentIndex());
+    slotClose(d->ui.m_reportTabWidget->currentIndex());
 }
 
 void KReportsView::slotClose(int index)
 {
     Q_D(KReportsView);
-    if (auto tab = dynamic_cast<KReportTab*>(d->m_reportTabWidget->widget(index))) {
-        d->m_reportTabWidget->removeTab(index);
+    if (auto tab = dynamic_cast<KReportTab*>(d->ui.m_reportTabWidget->widget(index))) {
+        d->ui.m_reportTabWidget->removeTab(index);
         tab->setReadyToDelete(true);
     }
 }
@@ -556,8 +699,8 @@ void KReportsView::slotCloseAll()
     Q_D(KReportsView);
     if(!d->m_needLoad) {
         while (true) {
-            if (auto tab = dynamic_cast<KReportTab*>(d->m_reportTabWidget->widget(1))) {
-                d->m_reportTabWidget->removeTab(1);
+            if (auto tab = dynamic_cast<KReportTab*>(d->ui.m_reportTabWidget->widget(1))) {
+                d->ui.m_reportTabWidget->removeTab(1);
                 tab->setReadyToDelete(true);
             } else
                 break;
@@ -569,14 +712,14 @@ void KReportsView::slotCloseAll()
 void KReportsView::slotListContextMenu(const QPoint & p)
 {
     Q_D(KReportsView);
-    auto items = d->m_tocTreeWidget->selectedItems();
+    const auto items = d->ui.m_tocTreeWidget->selectedItems();
 
     if (items.isEmpty()) {
         return;
     }
 
     QList<TocItem*> tocItems;
-    foreach(auto item, items) {
+    for (const auto& item : items) {
         auto tocItem = dynamic_cast<TocItem*>(item);
         if (tocItem && tocItem->isReport()) {
             tocItems.append(tocItem);
@@ -589,6 +732,7 @@ void KReportsView::slotListContextMenu(const QPoint & p)
 
     auto contextmenu = new QMenu(this);
 
+    contextmenu->addSection(i18nc("@title:menu Report context menu", "Report options"));
     contextmenu->addAction(i18nc("To open a new report", "&Open"),
                            this, SLOT(slotOpenFromList()));
 
@@ -596,6 +740,8 @@ void KReportsView::slotListContextMenu(const QPoint & p)
                            this, SLOT(slotPrintFromList()));
 
     if (tocItems.count() == 1) {
+        contextmenu->addAction(i18nc("To export a report", "&Export to PDF file"), this, SLOT(slotExportFromList()));
+
         contextmenu->addAction(i18nc("Configure a report", "&Configure"),
                                this, SLOT(slotConfigureFromList()));
 
@@ -613,22 +759,24 @@ void KReportsView::slotListContextMenu(const QPoint & p)
                                        this, SLOT(slotDeleteFromList()));
             }
         }
+    } else {
+        contextmenu->addAction(i18nc("To export reports", "&Export to PDF files"), this, SLOT(slotExportFromList()));
     }
 
-    contextmenu->popup(d->m_tocTreeWidget->mapToGlobal(p));
+    contextmenu->popup(d->ui.m_tocTreeWidget->viewport()->mapToGlobal(p));
 }
 
 void KReportsView::slotOpenFromList()
 {
     Q_D(KReportsView);
 
-    auto items = d->m_tocTreeWidget->selectedItems();
+    const auto items = d->ui.m_tocTreeWidget->selectedItems();
 
     if (items.isEmpty()) {
         return;
     }
 
-    foreach(auto item, items) {
+    for (const auto& item : items) {
         auto tocItem = dynamic_cast<TocItem*>(item);
         if (tocItem && tocItem->isReport()) {
             slotItemDoubleClicked(tocItem, 0);
@@ -640,13 +788,13 @@ void KReportsView::slotPrintFromList()
 {
     Q_D(KReportsView);
 
-    auto items = d->m_tocTreeWidget->selectedItems();
+    const auto items = d->ui.m_tocTreeWidget->selectedItems();
 
     if (items.isEmpty()) {
         return;
     }
 
-    foreach(auto item, items) {
+    for (const auto& item : items) {
         auto tocItem = dynamic_cast<TocItem*>(item);
         if (tocItem && tocItem->isReport()) {
             slotItemDoubleClicked(tocItem, 0);
@@ -655,19 +803,50 @@ void KReportsView::slotPrintFromList()
     }
 }
 
+void KReportsView::slotExportFromList()
+{
+    Q_D(KReportsView);
+
+    const auto items = d->ui.m_tocTreeWidget->selectedItems();
+
+    if (items.isEmpty()) {
+        return;
+    }
+
+    QString rootDir = QFileDialog::getExistingDirectory(this, i18n("Export into directory"), KRecentDirs::dir(":kmymoney-export"), QFileDialog::ShowDirsOnly);
+    if (rootDir.isEmpty())
+        return;
+
+    for (const auto item : qAsConst(items)) {
+        const auto tocItem = dynamic_cast<TocItem*>(item);
+        if (!tocItem || !tocItem->isReport())
+            continue;
+
+        slotItemDoubleClicked(tocItem, 0);
+        const auto tab = dynamic_cast<KReportTab*>(d->ui.m_reportTabWidget->currentWidget());
+        if (!tab)
+            continue;
+
+        const auto reportTocItem = dynamic_cast<TocItemReport*>(tocItem);
+        auto report = reportTocItem->getReport();
+        auto name = createSaveFileName(rootDir, report.name(), QLatin1String(".pdf"));
+        tab->saveAs(name, "application/pdf");
+    }
+}
+
 void KReportsView::slotConfigureFromList()
 {
     Q_D(KReportsView);
-    if (auto tocItem = dynamic_cast<TocItem*>(d->m_tocTreeWidget->currentItem())) {
-        slotItemDoubleClicked(tocItem, 0);
-        slotConfigure();
+    if (auto tocItem = dynamic_cast<TocItem*>(d->ui.m_tocTreeWidget->currentItem())) {
+        doItemDoubleClicked(tocItem, 0, OpenAfterConfiguration);
+        doConfigure(LoadReportOnCancel);
     }
 }
 
 void KReportsView::slotNewFromList()
 {
     Q_D(KReportsView);
-    if (auto tocItem = dynamic_cast<TocItem*>(d->m_tocTreeWidget->currentItem())) {
+    if (auto tocItem = dynamic_cast<TocItem*>(d->ui.m_tocTreeWidget->currentItem())) {
         slotItemDoubleClicked(tocItem, 0);
         slotDuplicate();
     }
@@ -676,7 +855,7 @@ void KReportsView::slotNewFromList()
 void KReportsView::slotDeleteFromList()
 {
     Q_D(KReportsView);
-    if (auto tocItem = dynamic_cast<TocItem*>(d->m_tocTreeWidget->currentItem())) {
+    if (auto tocItem = dynamic_cast<TocItem*>(d->ui.m_tocTreeWidget->currentItem())) {
         if (auto reportTocItem = dynamic_cast<TocItemReport*>(tocItem)) {
             MyMoneyReport& report = reportTocItem->getReport();
 
@@ -684,8 +863,8 @@ void KReportsView::slotDeleteFromList()
             if (! report.id().isEmpty() &&
                     KMessageBox::Continue == d->deleteReportDialog(report.name())) {
                 // check if report's tab is open; start from 1 because 0 is toc tab
-                for (int i = 1; i < d->m_reportTabWidget->count(); ++i) {
-                    auto tab = dynamic_cast<KReportTab*>(d->m_reportTabWidget->widget(i));
+                for (int i = 1; i < d->ui.m_reportTabWidget->count(); ++i) {
+                    auto tab = dynamic_cast<KReportTab*>(d->ui.m_reportTabWidget->widget(i));
                     if (tab && tab->report().id() == report.id()) {
                         slotClose(i); // if open, close it, so no crash when switching to it
                         break;
@@ -699,41 +878,6 @@ void KReportsView::slotDeleteFromList()
     }
 }
 
-void KReportsView::slotSelectByObject(const MyMoneyObject& obj, eView::Intent intent)
-{
-    switch(intent) {
-    case eView::Intent::UpdateActions:
-        updateActions(obj);
-        break;
-
-    case eView::Intent::OpenObject:
-        slotOpenReport(static_cast<const MyMoneyReport&>(obj));
-    default:
-        break;
-    }
-}
-
-void KReportsView::slotReportAccountTransactions()
-{
-    Q_D(KReportsView);
-    // Generate a transaction report that contains transactions for only the
-    // currently selected account.
-    if (!d->m_currentAccount.id().isEmpty()) {
-        MyMoneyReport report(
-            eMyMoney::Report::RowType::Account,
-            eMyMoney::Report::QueryColumn::Number | eMyMoney::Report::QueryColumn::Payee | eMyMoney::Report::QueryColumn::Category,
-            eMyMoney::TransactionFilter::Date::YearToDate,
-            eMyMoney::Report::DetailLevel::All,
-            i18n("%1 YTD Account Transactions", d->m_currentAccount.name()),
-            i18n("Generated Report")
-        );
-        report.setGroup(i18n("Transactions"));
-        report.addAccount(d->m_currentAccount.id());
-        emit customActionRequested(View::Reports, eView::Action::SwitchView);
-        slotOpenReport(report);
-    }
-}
-
 // Make sure, that these definitions are only used within this file
 // this does not seem to be necessary, but when building RPMs the
 // build option 'final' is used and all CPP files are concatenated.
