@@ -8,6 +8,7 @@
     SPDX-FileCopyrightText: 2000-2004 Ace Jones <ace.j@hotpop.com>
     SPDX-FileCopyrightText: 2017 Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
     SPDX-FileCopyrightText: 2018 Michael Kiefer <Michael-Kiefer@web.de>
+    SPDX-FileCopyrightText: 2021 Dawid Wróbel <me@dawidwrobel.com>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
@@ -19,60 +20,59 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
+#include <QClipboard>
+#include <QFile>
+#include <QFileDialog>
+#include <QFileInfo>
+#include <QIcon>
+#include <QList>
+#include <QLocale>
+#include <QMenu>
+#include <QMimeData>
+#include <QPointer>
+#include <QPrintPreviewDialog>
+#include <QTextCodec>
+#include <QTimer>
 #include <QTreeWidget>
 #include <QTreeWidgetItem>
-#include <QFile>
-#include <QTimer>
-#include <QClipboard>
-#include <QList>
-#include <QVBoxLayout>
-#include <QMimeData>
-#include <QIcon>
 #include <QUrlQuery>
-#include <QFileInfo>
-#include <QFileDialog>
-#include <QLocale>
-#include <QTextCodec>
-#include <QMenu>
-#include <QPointer>
+#include <QVBoxLayout>
 #include <QWheelEvent>
-#ifdef ENABLE_WEBENGINE
-#include <QWebEngineView>
-#else
-#include <KWebView>
-#endif
 
 // ----------------------------------------------------------------------------
 // KDE Includes
 
-#include <KMessageBox>
-#include <KLocalizedString>
 #include <KChartAbstractCoordinatePlane>
+#include <KLazyLocalizedString>
+#include <KLocalizedString>
+#include <KMessageBox>
+#include <QPainter>
 
 // ----------------------------------------------------------------------------
 // Project Includes
 
+#include "ui_kreportsview.h"
 #include "ui_reportcontrol.h"
 
+#include "icons.h"
+#include "kmm_printer.h"
+#include "kmmtextbrowser.h"
+#include "kmymoneysettings.h"
 #include "kmymoneyviewbase_p.h"
+#include "kreportchartview.h"
 #include "kreportconfigurationfilterdlg.h"
+#include "mymoneyenums.h"
+#include "mymoneyexception.h"
 #include "mymoneyfile.h"
 #include "mymoneyreport.h"
-#include "mymoneyexception.h"
-#include "kmymoneysettings.h"
-#include "querytable.h"
 #include "objectinfotable.h"
-#include "icons/icons.h"
-#include <kmymoneywebpage.h>
+#include "pivottable.h"
+#include "querytable.h"
+#include "reportcontrolimpl.h"
+#include "reporttable.h"
 #include "tocitem.h"
 #include "tocitemgroup.h"
 #include "tocitemreport.h"
-#include "kreportchartview.h"
-#include "pivottable.h"
-#include "reporttable.h"
-#include "reportcontrolimpl.h"
-#include "mymoneyenums.h"
-#include "kmm_printer.h"
 
 using namespace reports;
 using namespace eMyMoney;
@@ -95,11 +95,7 @@ using namespace Icons;
 class KReportTab: public QWidget
 {
 private:
-#ifdef ENABLE_WEBENGINE
-    QWebEngineView            *m_tableView;
-#else
-    KWebView                  *m_tableView;
-#endif
+    KMMTextBrowser* m_tableView;
     reports::KReportChartView *m_chartView;
     ReportControl             *m_control;
     QVBoxLayout               *m_layout;
@@ -118,33 +114,52 @@ private:
     QByteArray m_encoding;
 
 public:
-    KReportTab(QTabWidget* parent, const MyMoneyReport& report, const KReportsView *eventHandler);
+    KReportTab(QTabWidget* parent, const MyMoneyReport& report, const KReportsView* eventHandler, KReportsView::OpenOption openOption);
     ~KReportTab();
     const MyMoneyReport& report() const {
         return m_report;
     }
     void print();
+    void printPreview();
     void toggleChart();
     /**
      * Updates information about plotted chart in report's data
      */
     void updateDataRange();
     void copyToClipboard();
-    void saveAs(const QString& filename, const QString& selectedMimeType, bool includeCSS);
+    void saveAs(const QString& filename, const QString& selectedMimeType);
     void updateReport();
     QString createTable(const QString& links = QString());
-    const ReportControl* control() const {
+    const ReportControl* control() const
+    {
         return m_control;
     }
-    bool isReadyToDelete() const {
+
+    bool isReadyToDelete() const
+    {
         return m_deleteMe;
     }
-    void setReadyToDelete(bool f) {
+
+    void setReadyToDelete(bool f)
+    {
         m_deleteMe = f;
     }
-    void modifyReport(const MyMoneyReport& report) {
+
+    void modifyReport(const MyMoneyReport& report)
+    {
         m_report = report;
+        updateReport();
     }
+
+    void enableAllReportActions()
+    {
+        pActions[eMenu::Action::ReportNew]->setEnabled(true);
+        pActions[eMenu::Action::ReportConfigure]->setEnabled(true);
+        pActions[eMenu::Action::ReportExport]->setEnabled(true);
+        pActions[eMenu::Action::ReportDelete]->setEnabled(true);
+        pActions[eMenu::Action::ReportClose]->setEnabled(true);
+    }
+
     void showEvent(QShowEvent * event) final override;
     void loadTab();
 
@@ -180,41 +195,43 @@ public:
 /**
   * KReportTab Implementation
   */
-KReportTab::KReportTab(QTabWidget* parent, const MyMoneyReport& report, const KReportsView* eventHandler):
-    QWidget(parent),
-#ifdef ENABLE_WEBENGINE
-    m_tableView(new QWebEngineView(this)),
-#else
-    m_tableView(new KWebView(this)),
-#endif
-    m_chartView(new KReportChartView(this)),
-    m_control(new ReportControl(this)),
-    m_layout(new QVBoxLayout(this)),
-    m_report(report),
-    m_deleteMe(false),
-    m_chartEnabled(false),
-    m_showingChart(report.isChartByDefault()),
-    m_needReload(true),
-    m_isChartViewValid(false),
-    m_isTableViewValid(false),
-    m_table(0)
+KReportTab::KReportTab(QTabWidget* parent, const MyMoneyReport& report, const KReportsView* eventHandler, KReportsView::OpenOption openOption)
+    : QWidget(parent)
+    , m_tableView(new KMMTextBrowser(this))
+    , m_chartView(new KReportChartView(this))
+    , m_control(new ReportControl(this))
+    , m_layout(new QVBoxLayout(this))
+    , m_report(report)
+    , m_deleteMe(false)
+    , m_chartEnabled(false)
+    , m_showingChart(report.isChartByDefault())
+    , m_needReload(openOption == KReportsView::OpenImmediately)
+    , m_isChartViewValid(false)
+    , m_isTableViewValid(false)
+    , m_table(0)
 {
     m_layout->setSpacing(6);
-    m_tableView->setPage(new MyQWebEnginePage(m_tableView));
-    m_tableView->setZoomFactor(KMyMoneySettings::zoomFactor());
+    // TODO
+    //    m_tableView->setZoomFactor(KMyMoneySettings::zoomFactor());
 
-    //set button icons
-    m_control->ui->buttonChart->setIcon(Icons::get(Icon::OfficeChartLine));
-    m_control->ui->buttonClose->setIcon(Icons::get(Icon::DocumentClose));
+    // set button icons
+    m_control->ui->buttonChart->setIcon(Icons::get(Icon::OfficeCharBar));
     m_control->ui->buttonConfigure->setIcon(Icons::get(Icon::Configure));
-    m_control->ui->buttonCopy->setIcon(Icons::get(Icon::EditCopy));
-    m_control->ui->buttonDelete->setIcon(Icons::get(Icon::EditDelete));
+    m_control->ui->buttonDelete->setIcon(Icons::get(Icon::EditRemove));
     m_control->ui->buttonExport->setIcon(Icons::get(Icon::DocumentExport));
     m_control->ui->buttonNew->setIcon(Icons::get(Icon::DocumentNew));
 
+    // and actions
+    m_control->ui->buttonConfigure->setDefaultAction(pActions[eMenu::Action::ReportConfigure]);
+    m_control->ui->buttonDelete->setDefaultAction(pActions[eMenu::Action::ReportDelete]);
+    m_control->ui->buttonExport->setDefaultAction(pActions[eMenu::Action::ReportExport]);
+    m_control->ui->buttonNew->setDefaultAction(pActions[eMenu::Action::ReportNew]);
+
     m_chartView->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
     m_chartView->hide();
     m_tableView->hide();
+    m_tableView->setOpenLinks(false);
+
     m_layout->addWidget(m_control);
     m_layout->addWidget(m_tableView);
     m_layout->addWidget(m_chartView);
@@ -222,21 +239,15 @@ KReportTab::KReportTab(QTabWidget* parent, const MyMoneyReport& report, const KR
     m_layout->setStretch(2, 10);
 
     connect(m_control->ui->buttonChart, &QAbstractButton::clicked, eventHandler, &KReportsView::slotToggleChart);
-    connect(m_control->ui->buttonConfigure, &QAbstractButton::clicked, eventHandler, &KReportsView::slotConfigure);
-    connect(m_control->ui->buttonNew, &QAbstractButton::clicked, eventHandler, &KReportsView::slotDuplicate);
-    connect(m_control->ui->buttonCopy, &QAbstractButton::clicked, eventHandler, &KReportsView::slotCopyView);
-    connect(m_control->ui->buttonExport, &QAbstractButton::clicked, eventHandler, &KReportsView::slotExportView);
-    connect(m_control->ui->buttonDelete, &QAbstractButton::clicked, eventHandler, &KReportsView::slotDelete);
-    connect(m_control->ui->buttonClose, &QAbstractButton::clicked, eventHandler, &KReportsView::slotCloseCurrent);
 
-#ifdef ENABLE_WEBENGINE
-    connect(m_tableView->page(), &QWebEnginePage::urlChanged,
-            eventHandler, &KReportsView::slotOpenUrl);
-#else
-    m_tableView->page()->setLinkDelegationPolicy(QWebPage::DelegateAllLinks);
-    connect(m_tableView->page(), &KWebPage::linkClicked,
-            eventHandler, &KReportsView::slotOpenUrl);
-#endif
+    // setup style for all buttons to contain icon and text
+    const auto buttons = m_control->findChildren<QToolButton*>();
+    for (auto* button : qAsConst(buttons)) {
+        button->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
+    }
+    enableAllReportActions();
+
+    connect(m_tableView, &KMMTextBrowser::anchorClicked, eventHandler, &KReportsView::slotOpenUrl);
 
     // if this is a default report, then you can't delete it!
     if (report.id().isEmpty())
@@ -263,13 +274,14 @@ void KReportTab::wheelEvent(QWheelEvent* event)
     // Zoom text on Ctrl + Scroll
     if (event->modifiers() & Qt::CTRL) {
         if (!m_showingChart) {
-            qreal factor = m_tableView->zoomFactor();
-            if (event->delta() > 0)
-                factor += 0.1;
-            else if (event->delta() < 0)
-                factor -= 0.1;
-            m_tableView->setZoomFactor(factor);
-            event->accept();
+            // TODO
+            //            qreal factor = m_tableView->zoomFactor();
+            if (event->angleDelta().y() > 0)
+                //                factor += 0.1;
+                //            else if (event->delta() < 0)
+                //                factor -= 0.1;
+                //            m_tableView->setZoomFactor(factor);
+                event->accept();
             return;
         }
     }
@@ -287,8 +299,7 @@ void KReportTab::print()
                 QFont font = painter.font();
                 font.setPointSizeF(font.pointSizeF() * 0.8);
                 painter.setFont(font);
-                QLocale locale;
-                painter.drawText(0, 0, QDate::currentDate().toString(locale.dateFormat(QLocale::ShortFormat)));
+                painter.drawText(0, 0, MyMoneyUtils::formatDate(QDate::currentDate()));
 
                 /// @todo extract url from KMyMoneyApp
                 QUrl file;
@@ -296,35 +307,55 @@ void KReportTab::print()
                     painter.drawText(0, painter.window().height(), file.toLocalFile());
                 }
             } else {
-#ifdef ENABLE_WEBENGINE
-                m_tableView->page()->print(printer, [=] (bool) {});
-#else
                 m_tableView->print(printer);
-#endif
             }
         }
     }
 }
 
-void KReportTab::copyToClipboard()
+void KReportTab::printPreview()
 {
-    QMimeData* pMimeData =  new QMimeData();
-    pMimeData->setHtml(m_table->renderReport(QLatin1String("html"), m_encoding, m_report.name(), true));
-    QApplication::clipboard()->setMimeData(pMimeData);
+    if (m_showingChart) {
+        if (m_chartView) {
+            QPrintPreviewDialog dlg(KMyMoneyPrinter::instance(), m_chartView);
+            connect(&dlg, &QPrintPreviewDialog::paintRequested, m_tableView, [&](QPrinter* printer) {
+                QPainter painter(printer);
+                m_chartView->paint(&painter, painter.window());
+            });
+            dlg.exec();
+        }
+    } else {
+        if (m_tableView) {
+            QPrintPreviewDialog dlg(KMyMoneyPrinter::instance(), m_tableView);
+            connect(&dlg, &QPrintPreviewDialog::paintRequested, m_tableView, [&](QPrinter* printer) {
+                m_tableView->print(printer);
+            });
+            dlg.exec();
+        }
+    }
 }
 
-void KReportTab::saveAs(const QString& filename, const QString& selectedMimeType, bool includeCSS)
+void KReportTab::saveAs(const QString& filename, const QString& selectedMimeType)
 {
-    QFile file(filename);
+    if (selectedMimeType == QStringLiteral("application/pdf")) {
+        auto printer = KMyMoneyPDFPrinter::startPrint(m_report.name());
+        if (printer != nullptr) {
+            printer->setOutputFileName(filename);
+            m_tableView->print(printer);
+        }
+        return;
+    }
 
+    QFile file(filename);
     if (file.open(QIODevice::WriteOnly)) {
         if (selectedMimeType == QStringLiteral("text/csv")) {
             QTextStream(&file) << m_table->renderReport(QLatin1String("csv"), m_encoding, QString());
-        } else {
-            QString table =
-                m_table->renderReport(QLatin1String("html"), m_encoding, m_report.name(), includeCSS);
+        } else if (selectedMimeType == QStringLiteral("text/html")) {
+            QString table = m_table->renderReport(QLatin1String("html"), m_encoding, m_report.name());
             QTextStream stream(&file);
             stream << table;
+        } else if (selectedMimeType == QStringLiteral("application/xml")) {
+            QTextStream(&file) << m_table->toXml();
         }
         file.close();
     }
@@ -363,7 +394,7 @@ void KReportTab::updateReport()
     }
 
     delete m_table;
-    m_table = 0;
+    m_table = nullptr;
 
     if (m_report.reportType() == eMyMoney::Report::ReportType::PivotTable) {
         m_table = new PivotTable(m_report);
@@ -388,8 +419,7 @@ void KReportTab::toggleChart()
 
     if (m_showingChart) {
         if (!m_isTableViewValid) {
-            m_tableView->setHtml(m_table->renderReport(QLatin1String("html"), m_encoding, m_report.name()),
-                                 QUrl("file://")); // workaround for access permission to css file
+            m_tableView->setHtml(m_table->renderReport(QLatin1String("html"), m_encoding, m_report.name()));
         }
         m_isTableViewValid = true;
         m_tableView->show();
@@ -397,7 +427,7 @@ void KReportTab::toggleChart()
 
         m_control->ui->buttonChart->setText(i18n("Chart"));
         m_control->ui->buttonChart->setToolTip(i18n("Show the chart version of this report"));
-        m_control->ui->buttonChart->setIcon(Icons::get(Icon::OfficeChartLine));
+        m_control->ui->buttonChart->setIcon(Icons::get(Icon::OfficeCharBar));
     } else {
         if (!m_isChartViewValid)
             m_table->drawChart(*m_chartView);
@@ -414,11 +444,13 @@ void KReportTab::toggleChart()
 
 void KReportTab::updateDataRange()
 {
+    static const QRegularExpression trailingsZeroesRegEx("0+$");
+
     QList<DataDimension> grids = m_chartView->coordinatePlane()->gridDimensionsList();    // get dimensions of plotted graph
     if (grids.isEmpty())
         return;
-    QChar separator = locale().groupSeparator();
-    QChar decimalPoint = locale().decimalPoint();
+    auto separator = locale().groupSeparator(); // QChar in Qt5, QString in Qt6
+    auto decimalPoint = locale().decimalPoint(); // QChar in Qt5, QString in Qt6
     int precision = m_report.yLabelsPrecision();
     QList<QPair<QString, qreal>> dims;  // create list of dimension values in string and qreal
 
@@ -435,7 +467,11 @@ void KReportTab::updateDataRange()
         if (precision == 0)
             dims[i].first = locale().toString(qRound(dims.at(i).second));
         else
-            dims[i].first = locale().toString(dims.at(i).second, 'f', precision).remove(separator).remove(QRegularExpression("0+$")).remove(QRegularExpression("\\" + decimalPoint + "$"));
+            dims[i].first = locale()
+                                .toString(dims.at(i).second, 'f', precision)
+                                .remove(separator)
+                                .remove(trailingsZeroesRegEx)
+                                .remove(QRegularExpression("\\" + decimalPoint + "$"));
     }
 
     // save string variables in report's data
@@ -450,15 +486,11 @@ class KReportsViewPrivate : public KMyMoneyViewBasePrivate
     Q_DECLARE_PUBLIC(KReportsView)
 
 public:
-    explicit KReportsViewPrivate(KReportsView *qq):
-        q_ptr(qq),
-        m_needLoad(true),
-        m_reportListView(nullptr),
-        m_reportTabWidget(nullptr),
-        m_listTab(nullptr),
-        m_listTabLayout(nullptr),
-        m_tocTreeWidget(nullptr),
-        m_columnsAlreadyAdjusted(false)
+    explicit KReportsViewPrivate(KReportsView *qq)
+        : KMyMoneyViewBasePrivate(qq)
+        , m_needLoad(true)
+        , m_reportListView(nullptr)
+        , m_columnsAlreadyAdjusted(false)
     {
     }
 
@@ -470,66 +502,50 @@ public:
     {
         Q_Q(KReportsView);
         m_needLoad = false;
-        auto vbox = new QVBoxLayout(q);
-        q->setLayout(vbox);
-        vbox->setSpacing(6);
-        vbox->setMargin(0);
-
+        m_needsRefresh = true;
 
         // build reports toc
 
         setColumnsAlreadyAdjusted(false);
+        ui.setupUi(q);
+        ui.m_tocTreeWidget->sortByColumn(0, Qt::AscendingOrder);
+        ui.m_tocTreeWidget->setSelectionMode(QAbstractItemView::ExtendedSelection);
+        ui.m_closeButton->setIcon(Icons::get(Icon::DialogClose));
+        ui.m_filterContainer->hide();
+        ui.m_searchWidget->installEventFilter(q);
+        ui.m_tocTreeWidget->installEventFilter(q);
 
-        m_reportTabWidget = new QTabWidget(q);
-        vbox->addWidget(m_reportTabWidget);
-        m_reportTabWidget->setTabsClosable(true);
-
-        m_listTab = new QWidget(m_reportTabWidget);
-        m_listTabLayout = new QVBoxLayout(m_listTab);
-        m_listTabLayout->setSpacing(6);
-
-        m_tocTreeWidget = new QTreeWidget(m_listTab);
-
-        // report-group items have only 1 column (name of group),
-        // report items have 2 columns (report name and comment)
-        m_tocTreeWidget->setColumnCount(2);
-
-        // headers
-        QStringList headers;
-        headers << i18n("Reports") << i18n("Comment");
-        m_tocTreeWidget->setHeaderLabels(headers);
-
-        m_tocTreeWidget->setAlternatingRowColors(true);
-        m_tocTreeWidget->setSortingEnabled(true);
-        m_tocTreeWidget->sortByColumn(0, Qt::AscendingOrder);
-
-        // for report group items:
-        // doubleclick toggles the expand-state,
-        m_tocTreeWidget->setExpandsOnDoubleClick(false);
-
-        m_tocTreeWidget->setContextMenuPolicy(Qt::CustomContextMenu);
-
-        m_tocTreeWidget->setSelectionMode(QAbstractItemView::ExtendedSelection);
-
-        m_listTabLayout->addWidget(m_tocTreeWidget);
-        m_reportTabWidget->addTab(m_listTab, i18n("Reports"));
-
-        q->connect(m_reportTabWidget, &QTabWidget::tabCloseRequested,
-                   q, &KReportsView::slotClose);
-
-        q->connect(m_tocTreeWidget, &QTreeWidget::itemDoubleClicked,
-                   q, &KReportsView::slotItemDoubleClicked);
-
-        q->connect(m_tocTreeWidget, &QWidget::customContextMenuRequested,
-                   q, &KReportsView::slotListContextMenu);
-
+        q->connect(ui.m_reportTabWidget, &QTabWidget::tabCloseRequested, q, &KReportsView::slotClose);
+        q->connect(ui.m_tocTreeWidget, &QTreeWidget::itemDoubleClicked, q, &KReportsView::slotItemDoubleClicked);
+        q->connect(ui.m_tocTreeWidget, &QWidget::customContextMenuRequested, q, &KReportsView::slotListContextMenu);
         q->connect(MyMoneyFile::instance(), &MyMoneyFile::dataChanged, q, &KReportsView::refresh);
+
+        m_focusWidget = ui.m_tocTreeWidget;
+    }
+
+    QMap<QString, bool> saveTocExpandState()
+    {
+        QMap<QString, bool> expandStates;
+        for (int i = 0; i < ui.m_tocTreeWidget->topLevelItemCount(); ++i) {
+            const auto item = ui.m_tocTreeWidget->topLevelItem(i);
+
+            if (item) {
+                QString itemLabel = item->text(0);
+
+                if (item->isExpanded()) {
+                    expandStates.insert(itemLabel, true);
+                } else {
+                    expandStates.insert(itemLabel, false);
+                }
+            }
+        }
+        return expandStates;
     }
 
     void restoreTocExpandState(QMap<QString, bool>& expandStates)
     {
-        for (auto i = 0; i < m_tocTreeWidget->topLevelItemCount(); ++i) {
-            QTreeWidgetItem* item = m_tocTreeWidget->topLevelItem(i);
+        for (auto i = 0; i < ui.m_tocTreeWidget->topLevelItemCount(); ++i) {
+            QTreeWidgetItem* item = ui.m_tocTreeWidget->topLevelItem(i);
 
             if (item) {
                 QString itemLabel = item->text(0);
@@ -554,36 +570,24 @@ public:
                      reportName), i18n("Delete Report?"));
     }
 
-    void addReportTab(const MyMoneyReport& report)
+    void addReportTab(const MyMoneyReport& report, KReportsView::OpenOption openOption)
     {
         Q_Q(KReportsView);
-        new KReportTab(m_reportTabWidget, report, q);
+        auto reportTab = new KReportTab(ui.m_reportTabWidget, report, q, openOption);
+        reportTab->installEventFilter(q);
     }
 
     void loadView()
     {
         // remember the id of the current selected item
-        QTreeWidgetItem* item = m_tocTreeWidget->currentItem();
+        QTreeWidgetItem* item = ui.m_tocTreeWidget->currentItem();
         QString selectedItem = (item) ? item->text(0) : QString();
 
         // save expand states of all top-level items
-        QMap<QString, bool> expandStates;
-        for (int i = 0; i < m_tocTreeWidget->topLevelItemCount(); ++i) {
-            item = m_tocTreeWidget->topLevelItem(i);
-
-            if (item) {
-                QString itemLabel = item->text(0);
-
-                if (item->isExpanded()) {
-                    expandStates.insert(itemLabel, true);
-                } else {
-                    expandStates.insert(itemLabel, false);
-                }
-            }
-        }
+        QMap<QString, bool> expandStates = saveTocExpandState();
 
         // find the item visible on top
-        QTreeWidgetItem* visibleTopItem = m_tocTreeWidget->itemAt(0, 0);
+        QTreeWidgetItem* visibleTopItem = ui.m_tocTreeWidget->itemAt(0, 0);
 
         // text of column 0 identifies the item visible on top
         QString visibleTopItemText;
@@ -603,13 +607,13 @@ public:
         //
         // Rebuild the list page
         //
-        m_tocTreeWidget->clear();
+        ui.m_tocTreeWidget->clear();
 
         // Default Reports
         QList<ReportGroup> defaultreports;
         defaultReports(defaultreports);
 
-        QList<ReportGroup>::const_iterator it_group = defaultreports.constBegin();
+        QList<ReportGroup>::const_iterator it_group = defaultreports.cbegin();
 
         // the item to be set as current item
         QTreeWidgetItem* currentItem = 0L;
@@ -624,20 +628,16 @@ public:
         int chartGroupNo = defaultreports.size() + 1;
 
         // group for diagrams
-        QString groupName = I18N_NOOP("Charts");
+        QString groupName = kli18n("Charts").untranslatedText();
 
-        TocItemGroup* chartTocItemGroup =
-            new TocItemGroup(m_tocTreeWidget, chartGroupNo,
-                             i18n(groupName.toLatin1().data()));
+        TocItemGroup* chartTocItemGroup = new TocItemGroup(ui.m_tocTreeWidget, chartGroupNo, i18n(groupName.toLatin1().data()));
 
         m_allTocItemGroups.insert(groupName, chartTocItemGroup);
 
-        while (it_group != defaultreports.constEnd()) {
+        while (it_group != defaultreports.cend()) {
             groupName = (*it_group).name();
 
-            TocItemGroup* defaultTocItemGroup =
-                new TocItemGroup(m_tocTreeWidget, defaultGroupNo++,
-                                 i18n(groupName.toLatin1().data()));
+            TocItemGroup* defaultTocItemGroup = new TocItemGroup(ui.m_tocTreeWidget, defaultGroupNo++, (*it_group).title());
 
             m_allTocItemGroups.insert(groupName, defaultTocItemGroup);
 
@@ -671,20 +671,17 @@ public:
         // group for custom (favorite) reports
         int favoriteGroupNo = chartGroupNo + 1;
 
-        groupName = I18N_NOOP("Favorite Reports");
+        groupName = kli18n("Favorite Reports").untranslatedText();
 
-        TocItemGroup* favoriteTocItemGroup =
-            new TocItemGroup(m_tocTreeWidget, favoriteGroupNo,
-                             i18n(groupName.toLatin1().data()));
+        TocItemGroup* favoriteTocItemGroup = new TocItemGroup(ui.m_tocTreeWidget, favoriteGroupNo, i18n(groupName.toLatin1().data()));
 
         m_allTocItemGroups.insert(groupName, favoriteTocItemGroup);
 
         TocItemGroup* orphanTocItemGroup = 0;
 
         QList<MyMoneyReport> customreports = MyMoneyFile::instance()->reportList();
-        QList<MyMoneyReport>::const_iterator it_report = customreports.constBegin();
-        while (it_report != customreports.constEnd()) {
-
+        QList<MyMoneyReport>::const_iterator it_report = customreports.cbegin();
+        while (it_report != customreports.cend()) {
             MyMoneyReport report = *it_report;
 
             groupName = (*it_report).group();
@@ -702,11 +699,9 @@ public:
                     // group for orphaned reports
                     int orphanGroupNo = favoriteGroupNo + 1;
 
-                    groupName = I18N_NOOP("Old Customized Reports");
+                    groupName = kli18n("Old Customized Reports").untranslatedText();
 
-                    orphanTocItemGroup =
-                        new TocItemGroup(m_tocTreeWidget, orphanGroupNo,
-                                         i18n(groupName.toLatin1().data()));
+                    orphanTocItemGroup = new TocItemGroup(ui.m_tocTreeWidget, orphanGroupNo, i18n(groupName.toLatin1().data()));
                     m_allTocItemGroups.insert(groupName, orphanTocItemGroup);
                 }
                 new TocItemReport(orphanTocItemGroup, report);
@@ -730,9 +725,9 @@ public:
         //
 
         int index = 1;
-        while (index < m_reportTabWidget->count()) {
+        while (index < ui.m_reportTabWidget->count()) {
             // TODO: Find some way of detecting the file is closed and kill these tabs!!
-            if (auto tab = dynamic_cast<KReportTab*>(m_reportTabWidget->widget(index))) {
+            if (auto tab = dynamic_cast<KReportTab*>(ui.m_reportTabWidget->widget(index))) {
                 if (tab->isReadyToDelete() /* || ! reports.count() */) {
                     delete tab;
                     --index;
@@ -748,10 +743,7 @@ public:
 
             // intentionally not using 'Qt::MatchCaseSensitive' here
             // to avoid 'item not found' if someone corrected a typo only
-            QList<QTreeWidgetItem*> visibleTopItemList =
-                m_tocTreeWidget->findItems(visibleTopItemText,
-                                           Qt::MatchFixedString |
-                                           Qt::MatchRecursive);
+            QList<QTreeWidgetItem*> visibleTopItemList = ui.m_tocTreeWidget->findItems(visibleTopItemText, Qt::MatchFixedString | Qt::MatchRecursive);
 
             if (visibleTopItemList.isEmpty()) {
                 // the item could not be found, it was deleted or renamed
@@ -774,45 +766,45 @@ public:
             restoreTocExpandState(expandStates);
 
             // restore current item
-            m_tocTreeWidget->setCurrentItem(currentItem);
+            ui.m_tocTreeWidget->setCurrentItem(currentItem);
 
             // try to scroll to the item visible on top
             // when this method started
             if (visibleTopItemFound) {
-                m_tocTreeWidget->scrollToItem(visibleTopItem);
+                ui.m_tocTreeWidget->scrollToItem(visibleTopItem);
             } else {
-                m_tocTreeWidget->scrollToTop();
+                ui.m_tocTreeWidget->scrollToTop();
             }
             return;
         }
 
         // avoid flickering
-        m_tocTreeWidget->setUpdatesEnabled(false);
+        ui.m_tocTreeWidget->setUpdatesEnabled(false);
 
         // expand all top-level items
-        m_tocTreeWidget->expandAll();
+        ui.m_tocTreeWidget->expandAll();
 
         // resize columns
-        m_tocTreeWidget->resizeColumnToContents(0);
-        m_tocTreeWidget->resizeColumnToContents(1);
+        ui.m_tocTreeWidget->resizeColumnToContents(0);
+        ui.m_tocTreeWidget->resizeColumnToContents(1);
 
         // restore expand states of all top-level items
         restoreTocExpandState(expandStates);
 
         // restore current item
-        m_tocTreeWidget->setCurrentItem(currentItem);
+        ui.m_tocTreeWidget->setCurrentItem(currentItem);
 
         // try to scroll to the item visible on top
         // when this method started
         if (visibleTopItemFound) {
-            m_tocTreeWidget->scrollToItem(visibleTopItem);
+            ui.m_tocTreeWidget->scrollToItem(visibleTopItem);
         } else {
-            m_tocTreeWidget->scrollToTop();
+            ui.m_tocTreeWidget->scrollToTop();
         }
 
         setColumnsAlreadyAdjusted(true);
 
-        m_tocTreeWidget->setUpdatesEnabled(true);
+        ui.m_tocTreeWidget->setUpdatesEnabled(true);
     }
 
     void defaultReports(QList<ReportGroup>& groups)
@@ -1298,14 +1290,12 @@ public:
                            ));
             list.back().setBudget("Any", true);
 
-            list.push_back(MyMoneyReport(
-                               eMyMoney::Report::RowType::BudgetActual,
-                               static_cast<unsigned>(eMyMoney::Report::ColumnType::Months),
-                               TransactionFilter::Date::CurrentYear,
-                               eMyMoney::Report::DetailLevel::All,
-                               i18n("Yearly Budgeted vs. Actual"),
-                               i18n("Default Report")
-                           ));
+            list.push_back(MyMoneyReport(eMyMoney::Report::RowType::BudgetActual,
+                                         static_cast<unsigned>(eMyMoney::Report::ColumnType::Months),
+                                         TransactionFilter::Date::CurrentFiscalYear,
+                                         eMyMoney::Report::DetailLevel::All,
+                                         i18n("Yearly Budgeted vs. Actual"),
+                                         i18n("Default Report")));
             list.back().setBudget("Any", true);
             list.back().setShowingRowTotals(true);
 
@@ -1319,24 +1309,20 @@ public:
                            ));
             list.back().setBudget("Any", false);
 
-            list.push_back(MyMoneyReport(
-                               eMyMoney::Report::RowType::Budget,
-                               static_cast<unsigned>(eMyMoney::Report::ColumnType::Months),
-                               TransactionFilter::Date::CurrentYear,
-                               eMyMoney::Report::DetailLevel::All,
-                               i18n("Yearly Budget"),
-                               i18n("Default Report")
-                           ));
+            list.push_back(MyMoneyReport(eMyMoney::Report::RowType::Budget,
+                                         static_cast<unsigned>(eMyMoney::Report::ColumnType::Months),
+                                         TransactionFilter::Date::CurrentFiscalYear,
+                                         eMyMoney::Report::DetailLevel::All,
+                                         i18n("Yearly Budget"),
+                                         i18n("Default Report")));
             list.back().setBudget("Any", false);
             list.back().setShowingRowTotals(true);
-            list.push_back(MyMoneyReport(
-                               eMyMoney::Report::RowType::BudgetActual,
-                               static_cast<unsigned>(eMyMoney::Report::ColumnType::Months),
-                               TransactionFilter::Date::CurrentYear,
-                               eMyMoney::Report::DetailLevel::Group,
-                               i18n("Yearly Budgeted vs Actual Graph"),
-                               i18n("Default Report")
-                           ));
+            list.push_back(MyMoneyReport(eMyMoney::Report::RowType::BudgetActual,
+                                         static_cast<unsigned>(eMyMoney::Report::ColumnType::Months),
+                                         TransactionFilter::Date::CurrentFiscalYear,
+                                         eMyMoney::Report::DetailLevel::Group,
+                                         i18n("Yearly Budgeted vs Actual Graph"),
+                                         i18n("Default Report")));
             list.back().setChartByDefault(true);
             list.back().setChartCHGridLines(false);
             list.back().setChartSVGridLines(false);
@@ -1448,23 +1434,79 @@ public:
         m_columnsAlreadyAdjusted = adjusted;
     }
 
-    KReportsView       *q_ptr;
+    void setFilter(const QString& text)
+    {
+        const auto columns = ui.m_tocTreeWidget->columnCount();
+        for (auto i = 0; i < ui.m_tocTreeWidget->topLevelItemCount(); ++i) {
+            const auto toplevelItem = ui.m_tocTreeWidget->topLevelItem(i);
+            bool hideTopLevel = true;
+            for (auto j = 0; j < toplevelItem->childCount(); ++j) {
+                const auto reportItem = toplevelItem->child(j);
+                if (text.isEmpty()) {
+                    reportItem->setHidden(false);
+                    hideTopLevel = false;
+                } else {
+                    reportItem->setHidden(true);
+                    for (auto column = 0; column < columns; ++column) {
+                        if (reportItem->text(column).contains(text, Qt::CaseInsensitive)) {
+                            reportItem->setHidden(false);
+                            hideTopLevel = false;
+                            break;
+                        }
+                    }
+                }
+            }
+            toplevelItem->setHidden(hideTopLevel);
+        }
+
+        if (text.isEmpty()) {
+            if (!expandStatesBeforeSearch.isEmpty()) {
+                restoreTocExpandState(expandStatesBeforeSearch);
+                expandStatesBeforeSearch.clear();
+            }
+        } else {
+            if (expandStatesBeforeSearch.isEmpty()) {
+                expandStatesBeforeSearch = saveTocExpandState();
+            }
+            // show groups with matching reports as expanded
+            for (auto i = 0; i < ui.m_tocTreeWidget->topLevelItemCount(); ++i) {
+                const auto toplevelItem = ui.m_tocTreeWidget->topLevelItem(i);
+                toplevelItem->setExpanded(!toplevelItem->isHidden());
+            }
+        }
+    }
+
+    // Generate a transaction report that contains transactions for only the
+    // currently selected account.
+    void showTransactionReport()
+    {
+        Q_Q(KReportsView);
+        if (!m_currentAccount.id().isEmpty()) {
+            MyMoneyReport report(eMyMoney::Report::RowType::Account,
+                                 eMyMoney::Report::QueryColumn::Number | eMyMoney::Report::QueryColumn::Payee | eMyMoney::Report::QueryColumn::Category,
+                                 eMyMoney::TransactionFilter::Date::YearToDate,
+                                 eMyMoney::Report::DetailLevel::All,
+                                 i18n("%1 YTD Account Transactions", m_currentAccount.name()),
+                                 i18n("Generated Report"));
+            report.setGroup(i18n("Transactions"));
+            report.addAccount(m_currentAccount.id());
+            q->slotOpenReport(report);
+        }
+    }
 
     /**
       * This member holds the load state of page
       */
     bool m_needLoad;
 
+    Ui::KReportsView ui;
     QListWidget* m_reportListView;
-    QTabWidget* m_reportTabWidget;
-    QWidget* m_listTab;
-    QVBoxLayout* m_listTabLayout;
-    QTreeWidget* m_tocTreeWidget;
     QMap<QString, TocItemGroup*> m_allTocItemGroups;
     QString m_selectedExportFilter;
 
     bool m_columnsAlreadyAdjusted;
     MyMoneyAccount m_currentAccount;
+    QMap<QString, bool> expandStatesBeforeSearch;
 };
 
 #endif
