@@ -1,6 +1,7 @@
 /*
 
     SPDX-FileCopyrightText: 2018 Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
+    SPDX-FileCopyrightText: 2021 Dawid Wróbel <me@dawidwrobel.com>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
@@ -31,12 +32,10 @@
 
 #define VIEW_LEDGER         "ledger"
 
-ReportsView::ReportsView(QObject *parent, const QVariantList &args) :
-    KMyMoneyPlugin::Plugin(parent, "reportsview"/*must be the same as X-KDE-PluginInfo-Name*/),
+ReportsView::ReportsView(QObject *parent, const KPluginMetaData &metaData, const QVariantList &args) :
+    KMyMoneyPlugin::Plugin(parent, metaData, args),
     m_view(nullptr)
 {
-    Q_UNUSED(args)
-    setComponentName("reportsview", i18n("Reports view"));
     // For information, announce that we have been loaded.
     qDebug("Plugins: reportsview loaded");
 }
@@ -46,8 +45,9 @@ ReportsView::~ReportsView()
     qDebug("Plugins: reportsview unloaded");
 }
 
-void ReportsView::plug()
+void ReportsView::plug(KXMLGUIFactory* guiFactory)
 {
+    Q_UNUSED(guiFactory)
     m_view = new KReportsView;
     viewInterface()->addView(m_view, i18n("Reports"), View::Reports, Icons::Icon::Reports);
 }
@@ -146,15 +146,20 @@ QString ReportsView::budget() const
     const auto file = MyMoneyFile::instance();
 
     QString html;
-    //div header
-    html += "<div class=\"shadow\"><div class=\"displayblock\"><div class=\"summaryheader\">" + i18n("Budget") + "</div>\n<div class=\"gap\">&nbsp;</div>\n";
+
+    // header
+    html +=
+        "<table width=\"97%\" align=\"center\" class=\"displayblock\" >"
+        "<tr><td class=\"summaryheader\">"
+        + i18n("Budget") + "</td></tr>";
 
     if (file->countBudgets() == 0) {
-        html += "<table width=\"100%\" cellspacing=\"0\" cellpadding=\"2\" class=\"summarytable\" >";
-        html += QString("<tr>");
+        html += "<tr><td><table width=\"100%\" cellspacing=\"0\" cellpadding=\"2\" class=\"summarytable\" >";
+        html += "<tr>";
         html += QString("<td><center>%1</center></td>").arg(i18n("You have no budgets to display."));
-        html += QString("</tr>");
-        html += "</table></div></div>";
+        html += "</tr>";
+        html += "</table></td></tr>";
+        html += "</table>";
         return html;
     }
 
@@ -178,13 +183,13 @@ QString ReportsView::budget() const
     reports::PivotGrid grid = table.grid();
 
     //display budget summary
-    html += "<table width=\"100%\" cellspacing=\"0\" cellpadding=\"2\" class=\"summarytable\" >";
+    html += "<tr><td><table width=\"100%\" cellspacing=\"0\" cellpadding=\"2\" class=\"summarytable\" >";
     html += "<tr class=\"itemtitle\">";
     html += "<td class=\"left\" colspan=\"3\">";
     html += i18n("Current Month Summary");
     html += "</td></tr>";
     html += "<tr class=\"item\">";
-    html += "<td class=\"right\" width=\"50%\">";
+    html += "<td class=\"right\" width=\"60%\">";
     html += i18n("Budgeted");
     html += "</td>";
     html += "<td class=\"right\" width=\"20%\">";
@@ -204,21 +209,21 @@ QString ReportsView::budget() const
     QString totalActualAmount = totalActualValue.formatMoney(file->baseCurrency().tradingSymbol(), prec);
     QString totalBudgetDiffAmount = totalBudgetDiffValue.formatMoney(file->baseCurrency().tradingSymbol(), prec);
 
-    html += QString("<td align=\"right\">%1</td>").arg(showColoredAmount(totalBudgetAmount, totalBudgetValue.isNegative()));
-    html += QString("<td align=\"right\">%1</td>").arg(showColoredAmount(totalActualAmount, totalActualValue.isNegative()));
-    html += QString("<td align=\"right\">%1</td>").arg(showColoredAmount(totalBudgetDiffAmount, totalBudgetDiffValue.isNegative()));
+    html += QString("<td class=\"right nowrap\">%1</td>").arg(showColoredAmount(totalBudgetAmount, totalBudgetValue.isNegative()));
+    html += QString("<td class=\"right nowrap\">%1</td>").arg(showColoredAmount(totalActualAmount, totalActualValue.isNegative()));
+    html += QString("<td class=\"right nowrap\">%1</td>").arg(showColoredAmount(totalBudgetDiffAmount, totalBudgetDiffValue.isNegative()));
     html += "</tr>";
-    html += "</table>";
+    html += "</table></td></tr>";
 
     //budget overrun
-    html += "<div class=\"gap\">&nbsp;</div>\n";
-    html += "<table width=\"100%\" cellspacing=\"0\" cellpadding=\"2\" class=\"summarytable\" >";
+    html += "<tr class=\"gap\"><td>&nbsp;\n</td></tr>";
+    html += "<tr><td><table width=\"100%\" cellspacing=\"0\" cellpadding=\"2\" class=\"summarytable\" >";
     html += "<tr class=\"itemtitle\">";
     html += "<td class=\"left\" colspan=\"4\">";
     html += i18n("Budget Overruns");
     html += "</td></tr>";
     html += "<tr class=\"item\">";
-    html += "<td class=\"left\" width=\"30%\">";
+    html += "<td class=\"left\" width=\"40%\">";
     html += i18n("Account");
     html += "</td>";
     html += "<td class=\"right\" width=\"20%\">";
@@ -266,9 +271,9 @@ QString ReportsView::budget() const
                     html += QString("<td>") + link(VIEW_LEDGER, QString("?id=%1").arg(rowname.id()), QString()) + rowname.name() + linkend() + "</td>";
 
                     //show amounts
-                    html += QString("<td align=\"right\">%1</td>").arg(showColoredAmount(budgetAmount, budgetValue.isNegative()));
-                    html += QString("<td align=\"right\">%1</td>").arg(showColoredAmount(actualAmount, actualValue.isNegative()));
-                    html += QString("<td align=\"right\">%1</td>").arg(showColoredAmount(budgetDiffAmount, budgetDiffValue.isNegative()));
+                    html += QString("<td class=\"right nowrap\">%1</td>").arg(showColoredAmount(budgetAmount, budgetValue.isNegative()));
+                    html += QString("<td class=\"right nowrap\">%1</td>").arg(showColoredAmount(actualAmount, actualValue.isNegative()));
+                    html += QString("<td class=\"right nowrap\">%1</td>").arg(showColoredAmount(budgetDiffAmount, budgetDiffValue.isNegative()));
                     html += "</tr>";
 
                     //set the flag that there are overruns
@@ -287,7 +292,8 @@ QString ReportsView::budget() const
         html += QString::fromLatin1("<td class=\"center\" colspan=\"4\">%1</td>").arg(i18n("No Budget Categories have been overrun"));
         html += "</tr>";
     }
-    html += "</table></div></div>";
+    html += "</table></td></tr>";
+    html += "</table>";
     return html;
 }
 
@@ -317,6 +323,6 @@ QString ReportsView::linkend() const
     return QStringLiteral("</a>");
 }
 
-K_PLUGIN_FACTORY_WITH_JSON(ReportsViewFactory, "reportsview.json", registerPlugin<ReportsView>();)
+K_PLUGIN_CLASS_WITH_JSON(ReportsView, "reportsview.json")
 
 #include "reportsview.moc"
