@@ -1,5 +1,7 @@
 /*
     SPDX-FileCopyrightText: 2018 Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
+    SPDX-FileCopyrightText: 2021 Dawid Wróbel <me@dawidwrobel.com>
+    SPDX-FileCopyrightText: 2022 Thomas Baumgart <tbaumgart@kde.org>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
@@ -11,12 +13,13 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
-#include <QTimer>
 #include <QFile>
-#include <QTemporaryFile>
 #include <QFileDialog>
+#include <QPointer>
 #include <QRegularExpression>
 #include <QRegularExpressionMatch>
+#include <QTemporaryFile>
+#include <QTimer>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
@@ -33,25 +36,26 @@
 // Project Includes
 
 #include "appinterface.h"
-#include "viewinterface.h"
-#include "mymoneyfile.h"
-#include "mymoneystoragemgr.h"
-#include "mymoneyexception.h"
-#include "mymoneystoragebin.h"
-#include "mymoneystoragexml.h"
-#include "mymoneystorageanon.h"
+#include "gpg-recover-key.h"
 #include "icons.h"
-#include "kmymoneysettings.h"
-#include "kmymoneyutils.h"
 #include "kgpgfile.h"
 #include "kgpgkeyselectiondlg.h"
 #include "kmymoneyenums.h"
+#include "kmymoneysettings.h"
+#include "kmymoneyutils.h"
+#include "mymoneyanonwriter.h"
+#include "mymoneyexception.h"
+#include "mymoneyfile.h"
+#include "mymoneystoragebin.h"
+#include "mymoneyxmlreader.h"
+#include "mymoneyxmlwriter.h"
+#include "viewinterface.h"
+
+#include "kmmyesno.h"
 
 using namespace Icons;
 
 static constexpr KCompressionDevice::CompressionType const& COMPRESSION_TYPE = KCompressionDevice::GZip;
-// static constexpr char recoveryKeyId[] = "0xD2B08440";
-static constexpr char recoveryKeyId[] = "59B0F826D2B08440";
 
 // define the default period to warn about an expiring recoverkey to 30 days
 // but allows to override this setting during build time
@@ -59,11 +63,9 @@ static constexpr char recoveryKeyId[] = "59B0F826D2B08440";
 #define RECOVER_KEY_EXPIRATION_WARNING 30
 #endif
 
-XMLStorage::XMLStorage(QObject *parent, const QVariantList &args) :
-    KMyMoneyPlugin::Plugin(parent, "xmlstorage"/*must be the same as X-KDE-PluginInfo-Name*/)
+XMLStorage::XMLStorage(QObject *parent, const KPluginMetaData &metaData, const QVariantList &args) :
+    KMyMoneyPlugin::Plugin(parent, metaData, args)
 {
-    Q_UNUSED(args)
-    setComponentName("xmlstorage", i18n("XML storage"));
     // For information, announce that we have been loaded.
     qDebug("Plugins: xmlstorage loaded");
     checkRecoveryKeyValidity();
@@ -74,12 +76,12 @@ XMLStorage::~XMLStorage()
     qDebug("Plugins: xmlstorage unloaded");
 }
 
-MyMoneyStorageMgr *XMLStorage::open(const QUrl &url)
+bool XMLStorage::open(const QUrl &url)
 {
     fileUrl.clear();
 
     if (url.scheme() == QLatin1String("sql"))
-        return nullptr;
+        return false;
 
     QString fileName;
     auto downloadedFile = false;
@@ -123,11 +125,14 @@ MyMoneyStorageMgr *XMLStorage::open(const QUrl &url)
     } else if (sFileHeader == QString("--") ||        // PGP ASCII armored?
                sFileHeader == QString("\205\001") ||  // PGP binary?
                sFileHeader == QString("\205\002")) {  // PGP binary?
+#ifdef ENABLE_GPG
         if (KGPGFile::GPGAvailable()) {
             qfile = new KGPGFile(fileName);
             haveAt = false;
             isEncrypted = true;
-        } else {
+        } else
+#endif
+        {
             throw MYMONEYEXCEPTION(QString::fromLatin1("GPG is not available for decryption of file <b>%1</b>").arg(fileName));
         }
     } else {
@@ -150,7 +155,7 @@ MyMoneyStorageMgr *XMLStorage::open(const QUrl &url)
         ungetString(qfile, qbaFileHeader.data(), 8);
 
     // Ok, we got the first block of 8 bytes. Read in the two
-    // unsigned long int's by preserving endianess. This is
+    // unsigned long int's by preserving endianness. This is
     // achieved by reading them through a QDataStream object
     qint32 magic0, magic1;
     QDataStream s(&qbaFileHeader, QIODevice::ReadOnly);
@@ -186,20 +191,15 @@ MyMoneyStorageMgr *XMLStorage::open(const QUrl &url)
     else
         ungetString(qfile, qbaFileHeader.data(), 70);
 
-    QRegExp kmyexp("<!DOCTYPE KMYMONEY-FILE>");
+    static const QRegularExpression kmyexp(QLatin1String("<!DOCTYPE KMYMONEY-FILE>"));
     QByteArray txt(qbaFileHeader, 70);
-    if (kmyexp.indexIn(txt) == -1)
-        return nullptr;
+    const auto docType(kmyexp.match(txt));
+    if (!docType.hasMatch())
+        return false;
 
-    // attach the storage before reading the file, since the online
-    // onlineJobAdministration object queries the engine during
-    // loading.
-
-    auto storage = new MyMoneyStorageMgr;
-    MyMoneyStorageXML pReader;
-    pReader.setProgressCallback(appInterface()->progressCallback());
-    pReader.readFile(qfile, storage);
-    pReader.setProgressCallback(0);
+    MyMoneyXmlReader reader;
+    reader.setFile(MyMoneyFile::instance());
+    reader.read(qfile);
 
     qfile->close();
     delete qfile;
@@ -212,21 +212,19 @@ MyMoneyStorageMgr *XMLStorage::open(const QUrl &url)
 
     // make sure we setup the encryption key correctly
     if (isEncrypted) {
-        MyMoneyFile::instance()->attachStorage(storage);
         if (MyMoneyFile::instance()->value("kmm-encryption-key").isEmpty()) {
             // encapsulate transactions to the engine to be able to commit/rollback
             MyMoneyFileTransaction ft;
             MyMoneyFile::instance()->setValue("kmm-encryption-key", KMyMoneySettings::gpgRecipientList().join(","));
             ft.commit();
         }
-        MyMoneyFile::instance()->detachStorage();
     }
 
     fileUrl = url;
     //write the directory used for this file as the default one for next time.
     appInterface()->writeLastUsedDir(url.toDisplayString(QUrl::RemoveFilename | QUrl::PreferLocalFile | QUrl::StripTrailingSlash));
 
-    return storage;
+    return true;
 }
 
 QUrl XMLStorage::openUrl() const
@@ -236,22 +234,22 @@ QUrl XMLStorage::openUrl() const
 
 bool XMLStorage::save(const QUrl &url)
 {
-    QString filename = url.toLocalFile();
+    QString filename = url.path();
 
     if (!appInterface()->fileOpen()) {
         KMessageBox::error(nullptr, i18n("Tried to access a file when it has not been opened"));
         return false;
     }
 
-    std::unique_ptr<IMyMoneyOperationsFormat> storageWriter;
+    std::unique_ptr<MyMoneyXmlWriter> storageWriter;
 
     // If this file ends in ".ANON.XML" then this should be written using the
     // anonymous writer.
     bool plaintext = filename.right(4).toLower() == ".xml";
     if (filename.right(9).toLower() == ".anon.xml")
-        storageWriter = std::make_unique<MyMoneyStorageANON>();
+        storageWriter = std::make_unique<MyMoneyAnonWriter>();
     else
-        storageWriter = std::make_unique<MyMoneyStorageXML>();
+        storageWriter = std::make_unique<MyMoneyXmlWriter>();
 
     QString keyList;
     if (!appInterface()->filenameURL().isEmpty())
@@ -303,6 +301,8 @@ bool XMLStorage::save(const QUrl &url)
                 throw MYMONEYEXCEPTION(QString::fromLatin1("Unable to upload to '%1'.<br />%2").arg(url.toDisplayString(), putjob->errorString()));
             }
             file.close();
+            // remove the temporary file from the local medium
+            file.remove();
         }
     } catch (const MyMoneyException &e) {
         KMessageBox::error(nullptr, QString::fromLatin1(e.what()));
@@ -319,6 +319,7 @@ bool XMLStorage::saveAs()
     m_encryptionKeys.clear();
 
     QString selectedKeyName;
+#ifdef ENABLE_GPG
     if (KGPGFile::GPGAvailable() && KMyMoneySettings::writeDataEncrypted()) {
         // fill the secret key list and combo box
         QStringList keyList;
@@ -328,7 +329,7 @@ bool XMLStorage::saveAs()
         dlg->setSecretKeys(keyList, KMyMoneySettings::gpgRecipient());
         dlg->setAdditionalKeys(KMyMoneySettings::gpgRecipientList());
         rc = dlg->exec();
-        if ((rc == QDialog::Accepted) && (dlg != 0)) {
+        if ((rc == QDialog::Accepted) && (dlg != nullptr)) {
             m_additionalGpgKeys = dlg->additionalKeys();
             selectedKeyName = dlg->secretKey();
         }
@@ -337,17 +338,20 @@ bool XMLStorage::saveAs()
             return rc;
         }
     }
+#endif
 
     QString prevDir; // don't prompt file name if not a native file
     if (appInterface()->isNativeFile())
         prevDir = appInterface()->readLastUsedDir();
 
     QPointer<QFileDialog> dlg =
-        new QFileDialog(nullptr, i18n("Save As"), prevDir,
-                        QString(QLatin1String("%2 (%1);;")).arg(QStringLiteral("*.kmy")).arg(i18nc("KMyMoney (Filefilter)", "KMyMoney files")) +
-                        QString(QLatin1String("%2 (%1);;")).arg(QStringLiteral("*.anon.xml")).arg(i18nc("Anonymous (Filefilter)", "Anonymous files")) +
-                        QString(QLatin1String("%2 (%1);;")).arg(QStringLiteral("*.xml")).arg(i18nc("XML (Filefilter)", "XML files")) +
-                        QString(QLatin1String("%2 (%1);;")).arg(QStringLiteral("*")).arg(i18nc("All files (Filefilter)", "All files")));
+        new QFileDialog(nullptr,
+                        i18n("Save As"),
+                        prevDir,
+                        QString(QLatin1String("%2 (%1);;")).arg(QStringLiteral("*.kmy"), i18nc("KMyMoney (Filefilter)", "KMyMoney files"))
+                            + QString(QLatin1String("%2 (%1);;")).arg(QStringLiteral("*.anon.xml"), i18nc("Anonymous (Filefilter)", "Anonymous files"))
+                            + QString(QLatin1String("%2 (%1);;")).arg(QStringLiteral("*.xml"), i18nc("XML (Filefilter)", "XML files"))
+                            + QString(QLatin1String("%2 (%1);;")).arg(QStringLiteral("*"), i18nc("All files (Filefilter)", "All files")));
     dlg->setAcceptMode(QFileDialog::AcceptSave);
     connect(dlg, &QFileDialog::filterSelected, this, [&](const QString txt) {
         // for some reason, txt sometimes contains the filter expression only
@@ -382,9 +386,10 @@ bool XMLStorage::saveAs()
                 rc = save(newURL);
             else {
                 appInterface()->writeFilenameURL(newURL);
-                QRegExp keyExp(".* \\((.*)\\)");
-                if (keyExp.indexIn(selectedKeyName) != -1) {
-                    m_encryptionKeys = keyExp.cap(1);
+                static const QRegularExpression keyExp(QLatin1String(".* \\((.*)\\)"));
+                const auto key(keyExp.match(selectedKeyName));
+                if (key.hasMatch()) {
+                    m_encryptionKeys = key.captured(1);
                     if (!m_additionalGpgKeys.isEmpty()) {
                         if (!m_encryptionKeys.isEmpty())
                             m_encryptionKeys.append(QLatin1Char(','));
@@ -396,7 +401,7 @@ bool XMLStorage::saveAs()
                 try {
                     MyMoneyFile::instance()->deletePair("kmm-encryption-key");
                     ft.commit();
-                } catch(MyMoneyException& e) {
+                } catch (MyMoneyException&) {
                     ; // do nothing
                 }
                 rc = save(newURL);
@@ -431,27 +436,37 @@ void XMLStorage::ungetString(QIODevice *qfile, char *buf, int len)
     }
 }
 
-void XMLStorage::saveToLocalFile(const QString& localFile, IMyMoneyOperationsFormat* pWriter, bool plaintext, const QString& keyList)
+void XMLStorage::saveToLocalFile(const QString& localFile, MyMoneyXmlWriter* pWriter, bool plaintext, const QString& keyList)
 {
+#ifndef ENABLE_GPG
+    Q_UNUSED(keyList)
+#else
     // Check GPG encryption
     bool encryptFile = true;
     bool encryptRecover = false;
     if (!keyList.isEmpty()) {
         if (!KGPGFile::GPGAvailable()) {
-            KMessageBox::sorry(nullptr, i18n("GPG does not seem to be installed on your system. Please make sure that GPG can be found using the standard search path. This time, encryption is disabled."), i18n("GPG not found"));
+            KMessageBox::error(nullptr, i18n("GPG does not seem to be installed on your system. Please make sure that GPG can be found using the standard search path. This time, encryption is disabled."), i18n("GPG not found"));
             encryptFile = false;
         } else {
             if (KMyMoneySettings::encryptRecover()) {
                 encryptRecover = true;
-                if (!KGPGFile::keyAvailable(QString(recoveryKeyId))) {
-                    KMessageBox::sorry(nullptr, i18n("<p>You have selected to encrypt your data also with the KMyMoney recover key, but the key with id</p><p><center><b>%1</b></center></p><p>has not been found in your keyring at this time. Please make sure to import this key into your keyring. You can find it on the <a href=\"https://kmymoney.org/\">KMyMoney web-site</a>. This time your data will not be encrypted with the KMyMoney recover key.</p>", QString(recoveryKeyId)), i18n("GPG Key not found"));
+                if (!KGPGFile::keyAvailable(QLatin1String(RECOVER_KEY_ID))) {
+                    KMessageBox::error(nullptr,
+                                       i18n("<p>You have selected to encrypt your data also with the KMyMoney recover key, but the key with "
+                                            "id</p><p><center><b>%1</b></center></p><p>has not been found in your keyring at this time. Please make sure to "
+                                            "import this key into your keyring. You can find it on the <a href=\"https://kmymoney.org/\">KMyMoney "
+                                            "web-site</a>. This time your data will not be encrypted with the KMyMoney recover key.</p>",
+                                            QLatin1String(RECOVER_KEY_ID)),
+                                       i18n("GPG Key not found"));
                     encryptRecover = false;
                 }
             }
 
-            for(const QString& key: keyList.split(',', QString::SkipEmptyParts)) {
+            const auto keys = keyList.split(',', Qt::SkipEmptyParts);
+            for (const QString& key : qAsConst(keys)) {
                 if (!KGPGFile::keyAvailable(key)) {
-                    KMessageBox::sorry(nullptr, i18n("<p>You have specified to encrypt your data for the user-id</p><p><center><b>%1</b>.</center></p><p>Unfortunately, a valid key for this user-id was not found in your keyring. Please make sure to import a valid key for this user-id. This time, encryption is disabled.</p>", key), i18n("GPG Key not found"));
+                    KMessageBox::error(nullptr, i18n("<p>You have specified to encrypt your data for the user-id</p><p><center><b>%1</b>.</center></p><p>Unfortunately, a valid key for this user-id was not found in your keyring. Please make sure to import a valid key for this user-id. This time, encryption is disabled.</p>", key), i18n("GPG Key not found"));
                     encryptFile = false;
                     break;
                 }
@@ -459,13 +474,14 @@ void XMLStorage::saveToLocalFile(const QString& localFile, IMyMoneyOperationsFor
 
             if (encryptFile == true) {
                 QString msg = i18n("<p>You have configured to save your data in encrypted form using GPG. Make sure you understand that you might lose all your data if you encrypt it, but cannot decrypt it later on. If unsure, answer <b>No</b>.</p>");
-                if (KMessageBox::questionYesNo(nullptr, msg, i18n("Store GPG encrypted"), KStandardGuiItem::yes(), KStandardGuiItem::no(), "StoreEncrypted") == KMessageBox::No) {
+                if (KMessageBox::questionTwoActions(nullptr, msg, i18n("Store GPG encrypted"), KMMYesNo::yes(), KMMYesNo::no(), "StoreEncrypted")
+                    == KMessageBox::SecondaryAction) {
                     encryptFile = false;
                 }
             }
         }
     }
-
+#endif
     // Permissions to apply to new file
     QFileDevice::Permissions fmode = QFileDevice::ReadUser | QFileDevice::WriteUser;
 
@@ -485,20 +501,24 @@ void XMLStorage::saveToLocalFile(const QString& localFile, IMyMoneyOperationsFor
     MyMoneyFile::instance()->deletePair("kmm-encryption-key");
     std::unique_ptr<QIODevice> device;
 
+#ifdef ENABLE_GPG
     if (!keyList.isEmpty() && encryptFile && !plaintext) {
         std::unique_ptr<KGPGFile> kgpg = std::unique_ptr<KGPGFile>(new KGPGFile{writeFile});
         if (kgpg) {
-            for(const QString& key: keyList.split(',', QString::SkipEmptyParts)) {
+            const auto keys = keyList.split(',', Qt::SkipEmptyParts);
+            for (const QString& key : qAsConst(keys)) {
                 kgpg->addRecipient(key.toLatin1());
             }
 
             if (encryptRecover) {
-                kgpg->addRecipient(recoveryKeyId);
+                kgpg->addRecipient(QLatin1String(RECOVER_KEY_ID));
             }
             MyMoneyFile::instance()->setValue("kmm-encryption-key", keyList);
             device = std::unique_ptr<decltype(device)::element_type>(kgpg.release());
         }
-    } else {
+    } else
+#endif
+    {
         QFile *file = new QFile(writeFile);
         // The second parameter of KCompressionDevice means that KCompressionDevice will delete the QFile object
         device = std::unique_ptr<decltype(device)::element_type>(new KCompressionDevice{file, true, (plaintext) ? KCompressionDevice::None : COMPRESSION_TYPE});
@@ -507,19 +527,16 @@ void XMLStorage::saveToLocalFile(const QString& localFile, IMyMoneyOperationsFor
     ft.commit();
 
     if (!device || !device->open(QIODevice::WriteOnly)) {
-        throw MYMONEYEXCEPTION(QString::fromLatin1("Unable to open file '%1' for writing.").arg(localFile));
+        throw MYMONEYEXCEPTION(i18n("Unable to open file '%1' for writing").arg(localFile).append(QString::fromLatin1(": ") + device->errorString()));
     }
 
-    pWriter->setProgressCallback(appInterface()->progressCallback());
-    pWriter->writeFile(device.get(), MyMoneyFile::instance()->storage());
+    pWriter->setFile(MyMoneyFile::instance());
+    const auto xmlWrittenOk = pWriter->write(device.get());
     device->close();
-    // The next statement is a try to resolve bug #425752
-    // In case of GPG encryption it will delete the context
-    // before the QFile::copy() operation takes place.
-    // I suspect the GPG encryption to somehow keep the file
-    // open on MS-WIN systems in the background, something
-    // which does not harm on other operating systems.
-    delete device.release();
+
+    if (!xmlWrittenOk) {
+        throw MYMONEYEXCEPTION(QString::fromLatin1("XML write failure while writing to '%1'").arg(localFile));
+    }
 
     // Check for errors if possible, only possible for KGPGFile
     QFileDevice *fileDevice = qobject_cast<QFileDevice*>(device.get());
@@ -544,17 +561,17 @@ void XMLStorage::saveToLocalFile(const QString& localFile, IMyMoneyOperationsFor
         }
     }
     QFile::setPermissions(localFile, fmode);
-    pWriter->setProgressCallback(0);
 }
 
 void XMLStorage::checkRecoveryKeyValidity()
 {
-// check if the recovery key is still valid or expires soon
+#ifdef ENABLE_GPG
+    // check if the recovery key is still valid or expires soon
 
     if (KMyMoneySettings::writeDataEncrypted() && KMyMoneySettings::encryptRecover()) {
         if (KGPGFile::GPGAvailable()) {
             KGPGFile file;
-            QDateTime expirationDate = file.keyExpires(QLatin1String(recoveryKeyId));
+            QDateTime expirationDate = file.keyExpires(QLatin1String(RECOVER_KEY_ID));
             if (expirationDate.isValid() && QDateTime::currentDateTime().daysTo(expirationDate) <= RECOVER_KEY_EXPIRATION_WARNING) {
                 bool skipMessage = false;
 
@@ -578,8 +595,9 @@ void XMLStorage::checkRecoveryKeyValidity()
             }
         }
     }
+#endif
 }
 
-K_PLUGIN_FACTORY_WITH_JSON(XMLStorageFactory, "xmlstorage.json", registerPlugin<XMLStorage>();)
+K_PLUGIN_CLASS_WITH_JSON(XMLStorage, "xmlstorage.json")
 
 #include "xmlstorage.moc"
