@@ -23,7 +23,8 @@ private Q_SLOTS:
     void initTestCase();
     void init();
     void cleanup();
-    void testNextCheckNumber();
+    void testGetAdjacentNumber_data();
+    void testGetAdjacentNumber();
 };
 
 #endif
