@@ -1,6 +1,7 @@
 /*
-    SPDX-FileCopyrightText: 2007 Thomas Baumgart <ipwizard@users.sourceforge.net>
+    SPDX-FileCopyrightText: 2007-2019 Thomas Baumgart <ipwizard@users.sourceforge.net>
     SPDX-FileCopyrightText: 2017, 2018 Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
+    SPDX-FileCopyrightText: 2020 Robert Szczesiak <dev.rszczesiak@gmail.com>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
@@ -11,7 +12,6 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
-#include <QTimer>
 #include <QMenu>
 #include <QAction>
 #include <QBitArray>
@@ -30,46 +30,59 @@
 #include "kmymoneysettings.h"
 #include "storageenums.h"
 #include "menuenums.h"
+#include "accountdelegate.h"
+#include "accountsmodel.h"
 
 using namespace Icons;
 
 KAccountsView::KAccountsView(QWidget *parent) :
-    KMyMoneyAccountsViewBase(*new KAccountsViewPrivate(this), parent)
+    KMyMoneyViewBase(*new KAccountsViewPrivate(this), parent)
 {
     Q_D(KAccountsView);
-    d->ui->setupUi(this);
-
+    d->init();
     connect(pActions[eMenu::Action::NewAccount],          &QAction::triggered, this, &KAccountsView::slotNewAccount);
     connect(pActions[eMenu::Action::EditAccount],         &QAction::triggered, this, &KAccountsView::slotEditAccount);
     connect(pActions[eMenu::Action::DeleteAccount],       &QAction::triggered, this, &KAccountsView::slotDeleteAccount);
-    connect(pActions[eMenu::Action::CloseAccount],        &QAction::triggered, this, &KAccountsView::slotCloseAccount);
-    connect(pActions[eMenu::Action::ReopenAccount],       &QAction::triggered, this, &KAccountsView::slotReopenAccount);
-    connect(pActions[eMenu::Action::ChartAccountBalance], &QAction::triggered, this, &KAccountsView::slotChartAccountBalance);
     connect(pActions[eMenu::Action::MapOnlineAccount],    &QAction::triggered, this, &KAccountsView::slotAccountMapOnline);
     connect(pActions[eMenu::Action::UnmapOnlineAccount],  &QAction::triggered, this, &KAccountsView::slotAccountUnmapOnline);
     connect(pActions[eMenu::Action::UpdateAccount],       &QAction::triggered, this, &KAccountsView::slotAccountUpdateOnline);
     connect(pActions[eMenu::Action::UpdateAllAccounts],   &QAction::triggered, this, &KAccountsView::slotAccountUpdateOnlineAll);
+
+    d->ui->m_accountTree->setItemDelegate(new AccountDelegate(d->ui->m_accountTree));
+    connect(MyMoneyFile::instance()->accountsModel(), &AccountsModel::netWorthChanged, this, &KAccountsView::slotNetWorthChanged);
+
+    d->m_sharedToolbarActions.insert(eMenu::Action::FileNew, pActions[eMenu::Action::NewAccount]);
+
+    d->ui->m_accountTree->setSelectionMode(QAbstractItemView::SingleSelection);
+    d->ui->m_accountTree->setDragEnabled(true);
+    d->ui->m_accountTree->setAcceptDrops(true);
+    d->ui->m_accountTree->setDropIndicatorShown(true);
+    d->ui->m_accountTree->setDragDropMode(QAbstractItemView::InternalMove);
 }
 
 KAccountsView::~KAccountsView()
 {
 }
 
-void KAccountsView::executeCustomAction(eView::Action action)
+void KAccountsView::slotSettingsChanged()
 {
     Q_D(KAccountsView);
-    switch(action) {
-    case eView::Action::Refresh:
-        refresh();
-        break;
+    d->m_proxyModel->setHideClosedAccounts(!KMyMoneySettings::showAllAccounts());
+    d->m_proxyModel->setHideEquityAccounts(!KMyMoneySettings::expertMode());
+    d->m_proxyModel->setHideZeroBalancedEquityAccounts(KMyMoneySettings::hideZeroBalanceEquities());
+    d->m_proxyModel->setHideZeroBalancedAccounts(KMyMoneySettings::hideZeroBalanceAccounts());
+    d->m_proxyModel->setShowAllEntries(KMyMoneySettings::showAllAccounts());
+    d->m_proxyModel->setHideFavoriteAccounts(false);
 
-    case eView::Action::SetDefaultFocus:
-        QTimer::singleShot(0, d->ui->m_accountTree, SLOT(setFocus()));
-        break;
-
-    default:
-        break;
+    if (KMyMoneySettings::showCategoriesInAccountsView()) {
+        d->m_proxyModel->addAccountGroup(QVector<eMyMoney::Account::Type> {eMyMoney::Account::Type::Income, eMyMoney::Account::Type::Expense});
+    } else {
+        d->m_proxyModel->removeAccountType(eMyMoney::Account::Type::Income);
+        d->m_proxyModel->removeAccountType(eMyMoney::Account::Type::Expense);
     }
+
+    MyMoneyFile::instance()->accountsModel()->setColorScheme(AccountsModel::Positive, KMyMoneySettings::schemeColor(SchemeColor::Positive));
+    MyMoneyFile::instance()->accountsModel()->setColorScheme(AccountsModel::Negative, KMyMoneySettings::schemeColor(SchemeColor::Negative));
 }
 
 void KAccountsView::refresh()
@@ -80,10 +93,10 @@ void KAccountsView::refresh()
         return;
     }
     d->m_needsRefresh = false;
-    // TODO: check why the invalidate is needed here
-    d->m_proxyModel->invalidate();
+
     d->m_proxyModel->setHideClosedAccounts(!KMyMoneySettings::showAllAccounts());
     d->m_proxyModel->setHideEquityAccounts(!KMyMoneySettings::expertMode());
+    d->m_proxyModel->setHideFavoriteAccounts(false);
     if (KMyMoneySettings::showCategoriesInAccountsView()) {
         d->m_proxyModel->addAccountGroup(QVector<eMyMoney::Account::Type> {eMyMoney::Account::Type::Income, eMyMoney::Account::Type::Expense});
     } else {
@@ -95,37 +108,23 @@ void KAccountsView::refresh()
     d->m_haveUnusedCategories = false;
     d->ui->m_hiddenCategories->hide();  // hides label
     d->m_proxyModel->setHideUnusedIncomeExpenseAccounts(KMyMoneySettings::hideUnusedCategory());
+    d->m_proxyModel->invalidate();
 }
 
-void KAccountsView::showEvent(QShowEvent * event)
-{
-    Q_D(KAccountsView);
-    if (!d->m_proxyModel)
-        d->init();
-
-    emit customActionRequested(View::Accounts, eView::Action::AboutToShow);
-
-    if (d->m_needsRefresh)
-        refresh();
-
-    // don't forget base class implementation
-    QWidget::showEvent(event);
-}
-
-void KAccountsView::updateActions(const MyMoneyObject& obj)
+void KAccountsView::updateActions(const SelectedObjects& selections)
 {
     Q_D(KAccountsView);
 
     const auto file = MyMoneyFile::instance();
 
-    if (typeid(obj) != typeid(MyMoneyAccount) &&
-            (obj.id().isEmpty() && d->m_currentAccount.id().isEmpty())) // do not disable actions that were already disabled)
+    // check if there is anything todo and quit if not
+    if (selections.selection(SelectedObjects::Account).count() < 1
+            && d->m_currentAccount.id().isEmpty() ) {
         return;
-
-    const auto& acc = static_cast<const MyMoneyAccount&>(obj);
+    }
 
     const QVector<eMenu::Action> actionsToBeDisabled {
-        eMenu::Action::NewAccount, eMenu::Action::EditAccount, eMenu::Action::DeleteAccount,
+        eMenu::Action::EditAccount, eMenu::Action::DeleteAccount,
         eMenu::Action::CloseAccount, eMenu::Action::ReopenAccount,
         eMenu::Action::ChartAccountBalance,
         eMenu::Action::UnmapOnlineAccount, eMenu::Action::MapOnlineAccount,
@@ -138,81 +137,19 @@ void KAccountsView::updateActions(const MyMoneyObject& obj)
     pActions[eMenu::Action::NewAccount]->setEnabled(true);
     pActions[eMenu::Action::UpdateAllAccounts]->setEnabled(KMyMoneyUtils::canUpdateAllAccounts());
 
-    if (acc.id().isEmpty()) {
+    const auto accountId = selections.firstSelection(SelectedObjects::Account);
+    if (accountId.isEmpty()) {
         d->m_currentAccount = MyMoneyAccount();
         return;
-    } else if (file->isStandardAccount(acc.id())) {
-        d->m_currentAccount = acc;
-        return;
     }
+    const auto acc = file->accountsModel()->itemById(accountId);
     d->m_currentAccount = acc;
 
-    switch (acc.accountGroup()) {
-    case eMyMoney::Account::Type::Asset:
-    case eMyMoney::Account::Type::Liability:
-    case eMyMoney::Account::Type::Equity:
-    {
-        pActions[eMenu::Action::EditAccount]->setEnabled(true);
-        pActions[eMenu::Action::DeleteAccount]->setEnabled(!file->isReferenced(acc));
-
-        auto b = acc.isClosed() ? true : false;
-        pActions[eMenu::Action::ReopenAccount]->setEnabled(b);
-        pActions[eMenu::Action::CloseAccount]->setEnabled(!b);
-
-        if (!acc.isClosed()) {
-            b = (d->canCloseAccount(acc) == KAccountsViewPrivate::AccountCanClose) ? true : false;
-            pActions[eMenu::Action::CloseAccount]->setEnabled(b);
-            d->hintCloseAccountAction(acc, pActions[eMenu::Action::CloseAccount]);
-        }
-
-        pActions[eMenu::Action::ChartAccountBalance]->setEnabled(true);
-
-        if (d->m_currentAccount.hasOnlineMapping()) {
-            pActions[eMenu::Action::UnmapOnlineAccount]->setEnabled(true);
-
-            if (d->m_onlinePlugins) {
-                // check if provider is available
-                QMap<QString, KMyMoneyPlugin::OnlinePlugin*>::const_iterator it_p;
-                it_p = d->m_onlinePlugins->constFind(d->m_currentAccount.onlineBankingSettings().value(QLatin1String("provider")).toLower());
-                if (it_p != d->m_onlinePlugins->constEnd()) {
-                    QStringList protocols;
-                    (*it_p)->protocols(protocols);
-                    if (protocols.count() > 0) {
-                        pActions[eMenu::Action::UpdateAccount]->setEnabled(true);
-                    }
-                }
-            }
-
-        } else {
-            pActions[eMenu::Action::MapOnlineAccount]->setEnabled(!acc.isClosed() && d->m_onlinePlugins && !d->m_onlinePlugins->isEmpty());
-        }
-
-        break;
-    }
-    default:
-        break;
+    if (file->isStandardAccount(acc.id())) {
+        return;
     }
 
-#if 0
-    // It is unclear to me, what the following code should do
-    // as it just does nothing
-    QBitArray skip((int)eStorage::Reference::Count);
-    if (!d->m_currentAccount.id().isEmpty()) {
-        if (!file->isStandardAccount(d->m_currentAccount.id())) {
-            switch (d->m_currentAccount.accountGroup()) {
-            case eMyMoney::Account::Type::Asset:
-            case eMyMoney::Account::Type::Liability:
-            case eMyMoney::Account::Type::Equity:
-
-                break;
-
-            default:
-                break;
-            }
-        }
-    }
-#endif
-
+    d->updateActions(acc);
 }
 
 /**
@@ -225,51 +162,19 @@ void KAccountsView::slotUnusedIncomeExpenseAccountHidden()
     d->ui->m_hiddenCategories->setVisible(d->m_haveUnusedCategories);
 }
 
-void KAccountsView::slotNetWorthChanged(const MyMoneyMoney &netWorth)
+void KAccountsView::slotNetWorthChanged(const MyMoneyMoney &netWorth, bool isApproximate)
 {
     Q_D(KAccountsView);
-    d->netBalProChanged(netWorth, d->ui->m_totalProfitsLabel, View::Accounts);
+    const auto formattedValue = d->formatViewLabelValue(netWorth);
+    d->updateViewLabel(d->ui->m_totalProfitsLabel,
+                       isApproximate ? i18nc("Approximate net worth", "Net Worth: ~%1", formattedValue)
+                       : i18n("Net Worth: %1", formattedValue));
 }
 
-void KAccountsView::slotShowAccountMenu(const MyMoneyAccount& acc)
-{
-    Q_UNUSED(acc);
-    pMenus[eMenu::Menu::Account]->exec(QCursor::pos());
-}
-
-void KAccountsView::slotSelectByObject(const MyMoneyObject& obj, eView::Intent intent)
-{
-    switch(intent) {
-    case eView::Intent::UpdateActions:
-        updateActions(obj);
-        break;
-
-    case eView::Intent::OpenContextMenu:
-        slotShowAccountMenu(static_cast<const MyMoneyAccount&>(obj));
-        break;
-
-    default:
-        break;
-    }
-}
-
-void KAccountsView::slotSelectByVariant(const QVariantList& variant, eView::Intent intent)
+void KAccountsView::setOnlinePlugins(QMap<QString, KMyMoneyPlugin::OnlinePlugin*>* plugins)
 {
     Q_D(KAccountsView);
-    switch (intent) {
-    case eView::Intent::UpdateNetWorth:
-        if (variant.count() == 1)
-            slotNetWorthChanged(variant.first().value<MyMoneyMoney>());
-        break;
-
-    case eView::Intent::SetOnlinePlugins:
-        if (variant.count() == 1)
-            d->m_onlinePlugins = static_cast<QMap<QString, KMyMoneyPlugin::OnlinePlugin*>*>(variant.first().value<void*>());
-        break;
-
-    default:
-        break;
-    }
+    d->m_onlinePlugins = plugins;
 }
 
 void KAccountsView::slotNewAccount()
@@ -277,8 +182,11 @@ void KAccountsView::slotNewAccount()
     Q_D(KAccountsView);
     MyMoneyAccount account;
     account.setOpeningDate(KMyMoneySettings::firstFiscalDate());
-    account.setParentAccountId(d->m_currentAccount.id());
+
     account.setAccountType(d->m_currentAccount.accountType());
+    account.setInstitutionId(d->m_currentAccount.institutionId());
+    account.setParentAccountId(d->m_currentAccount.id());
+
     NewAccountWizard::Wizard::newAccount(account);
 }
 
@@ -295,7 +203,6 @@ void KAccountsView::slotEditAccount()
         d->editAccount();
         break;
     }
-    emit selectByObject(d->m_currentAccount, eView::Intent::None);
 }
 
 void KAccountsView::slotDeleteAccount()
@@ -332,66 +239,12 @@ void KAccountsView::slotDeleteAccount()
     try {
         file->removeAccount(d->m_currentAccount);
         d->m_currentAccount.clearId();
-        emit selectByObject(MyMoneyAccount(), eView::Intent::None);
         ft.commit();
     } catch (const MyMoneyException &e) {
         KMessageBox::error(this, i18n("Unable to delete account '%1'. Cause: %2", selectedAccountName, QString::fromLatin1(e.what())));
     }
 }
 
-void KAccountsView::slotCloseAccount()
-{
-    Q_D(KAccountsView);
-    MyMoneyFileTransaction ft;
-    MyMoneyFile* file = MyMoneyFile::instance();
-
-    try {
-        // in case of investment, try to close the sub-accounts first
-        if (d->m_currentAccount.accountType() == eMyMoney::Account::Type::Investment) {
-            d->closeSubAccounts(d->m_currentAccount);
-        }
-        d->m_currentAccount.setClosed(true);
-        file->modifyAccount(d->m_currentAccount);
-        emit selectByObject(d->m_currentAccount, eView::Intent::None);
-        ft.commit();
-        if (!KMyMoneySettings::showAllAccounts())
-            KMessageBox::information(this,
-                                     i18n("<qt>You have closed this account. It remains in the system because you have transactions which still refer to it, "
-                                          "but it is not shown in the views. You can make it visible again by going to the View menu and selecting <b>Show all "
-                                          "accounts</b> or by deselecting the <b>Do not show closed accounts</b> setting.</qt>"),
-                                     i18n("Information"),
-                                     "CloseAccountInfo");
-    } catch (const MyMoneyException& e) {
-        qDebug() << e.what();
-    }
-}
-
-void KAccountsView::slotReopenAccount()
-{
-    Q_D(KAccountsView);
-    const auto file = MyMoneyFile::instance();
-    MyMoneyFileTransaction ft;
-    try {
-        auto& acc = d->m_currentAccount;
-        while (acc.isClosed()) {
-            acc.setClosed(false);
-            file->modifyAccount(acc);
-            acc = file->account(acc.parentAccountId());
-        }
-        emit selectByObject(d->m_currentAccount, eView::Intent::None);
-        ft.commit();
-    } catch (const MyMoneyException &) {
-    }
-}
-
-void KAccountsView::slotChartAccountBalance()
-{
-    Q_D(KAccountsView);
-    if (!d->m_currentAccount.id().isEmpty()) {
-        emit customActionRequested(View::Accounts, eView::Action::ShowBalanceChart);
-    }
-}
-
 void KAccountsView::slotNewCategory()
 {
     Q_D(KAccountsView);
@@ -400,7 +253,8 @@ void KAccountsView::slotNewCategory()
 
 void KAccountsView::slotNewPayee(const QString& nameBase, QString& id)
 {
-    KMyMoneyUtils::newPayee(nameBase, id);
+    bool ok;
+    std::tie(ok, id) = KMyMoneyUtils::newPayee(nameBase);
 }
 
 void KAccountsView::slotAccountUnmapOnline()
@@ -414,7 +268,15 @@ void KAccountsView::slotAccountUnmapOnline()
     if (!d->m_currentAccount.hasOnlineMapping())
         return;
 
-    if (KMessageBox::warningYesNo(this, QString("<qt>%1</qt>").arg(i18n("Do you really want to remove the mapping of account <b>%1</b> to an online account? Depending on the details of the online banking method used, this action cannot be reverted.", d->m_currentAccount.name())), i18n("Remove mapping to online account")) == KMessageBox::Yes) {
+    if (KMessageBox::warningTwoActions(this,
+                                       QString("<qt>%1</qt>")
+                                           .arg(i18n("Do you really want to remove the mapping of account <b>%1</b> to an online account? Depending on the "
+                                                     "details of the online banking method used, this action cannot be reverted.",
+                                                     d->m_currentAccount.name())),
+                                       i18n("Remove mapping to online account"),
+                                       KMMYesNo::yes(),
+                                       KMMYesNo::no())
+        == KMessageBox::PrimaryAction) {
         MyMoneyFileTransaction ft;
         try {
             d->m_currentAccount.setOnlineBankingSettings(MyMoneyKeyValueContainer());
@@ -432,7 +294,7 @@ void KAccountsView::slotAccountUnmapOnline()
             KMessageBox::error(this, i18n("Unable to unmap account from online account: %1", QString::fromLatin1(e.what())));
         }
     }
-    updateActions(d->m_currentAccount);
+    d->updateActions(d->m_currentAccount);
 }
 
 void KAccountsView::slotAccountMapOnline()
@@ -466,14 +328,14 @@ void KAccountsView::slotAccountMapOnline()
         break;
     default: {
         QMenu popup(this);
-        popup.setTitle(i18n("Select online banking plugin"));
+        popup.addSection(i18nc("@title:menu Online provider selection", "Online provider"));
 
         // Populate the pick list with all the provider
-        for (it_p = d->m_onlinePlugins->constBegin(); it_p != d->m_onlinePlugins->constEnd(); ++it_p) {
+        for (it_p = d->m_onlinePlugins->cbegin(); it_p != d->m_onlinePlugins->cend(); ++it_p) {
             popup.addAction(it_p.key())->setData(it_p.key());
         }
 
-        QAction *item = popup.actions()[0];
+        QAction* item = popup.actions().at(0);
         if (item) {
             popup.setActiveAction(item);
         }
@@ -493,7 +355,7 @@ void KAccountsView::slotAccountMapOnline()
 
     // find the provider
     it_p = d->m_onlinePlugins->constFind(provider.toLower());
-    if (it_p != d->m_onlinePlugins->constEnd()) {
+    if (it_p != d->m_onlinePlugins->cend()) {
         // plugin found, call it
         MyMoneyKeyValueContainer settings;
         if ((*it_p)->mapAccount(d->m_currentAccount, settings)) {
@@ -505,13 +367,14 @@ void KAccountsView::slotAccountMapOnline()
                 MyMoneyFile::instance()->modifyAccount(acc);
                 ft.commit();
                 // The mapping could enable the online task system
+                d->m_currentAccount = acc;
                 onlineJobAdministration::instance()->updateOnlineTaskProperties();
             } catch (const MyMoneyException &e) {
                 KMessageBox::error(this, i18n("Unable to map account to online account: %1", QString::fromLatin1(e.what())));
             }
         }
     }
-    updateActions(d->m_currentAccount);
+    d->updateActions(d->m_currentAccount);
 }
 
 void KAccountsView::slotAccountUpdateOnlineAll()
@@ -522,7 +385,7 @@ void KAccountsView::slotAccountUpdateOnlineAll()
     MyMoneyFile::instance()->accountList(accList);
 
     QList<MyMoneyAccount> mappedAccList;
-    Q_FOREACH(auto account, accList) {
+    for (const auto& account : qAsConst(accList)) {
         if (account.hasOnlineMapping())
             mappedAccList += account;
     }
