@@ -1,6 +1,6 @@
 /*
-    SPDX-FileCopyrightText: 2007 Thomas Baumgart <ipwizard@users.sourceforge.net>
-    SPDX-FileCopyrightText: 2017 Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
+    SPDX-FileCopyrightText: 2007-2019 Thomas Baumgart <tbaumgart@kde.org>
+    SPDX-FileCopyrightText: 2017-2018 Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
@@ -24,7 +24,9 @@
 // Project Includes
 
 #include "ui_kaccountsview.h"
-#include "kmymoneyaccountsviewbase_p.h"
+#include "kmymoneyviewbase_p.h"
+#include "mymoneyfile.h"
+#include "accountsmodel.h"
 
 #include "mymoneyexception.h"
 #include "mymoneysplit.h"
@@ -34,27 +36,28 @@
 #include "keditloanwizard.h"
 #include "mymoneyaccount.h"
 #include "mymoneymoney.h"
-#include "mymoneyfile.h"
-#include "accountsviewproxymodel.h"
+#include "accountsproxymodel.h"
 #include "kmymoneyplugin.h"
 #include "icons.h"
 #include "mymoneyenums.h"
 #include "menuenums.h"
 #include "mymoneystatementreader.h"
 #include "kmymoneyutils.h"
+#include "columnselector.h"
 
 using namespace Icons;
 
-class KAccountsViewPrivate : public KMyMoneyAccountsViewBasePrivate
+class KAccountsViewPrivate : public KMyMoneyViewBasePrivate
 {
     Q_DECLARE_PUBLIC(KAccountsView)
 
 public:
-    explicit KAccountsViewPrivate(KAccountsView *qq) :
-        q_ptr(qq),
-        ui(new Ui::KAccountsView),
-        m_haveUnusedCategories(false),
-        m_onlinePlugins(nullptr)
+    explicit KAccountsViewPrivate(KAccountsView *qq)
+        : KMyMoneyViewBasePrivate(qq)
+        , ui(new Ui::KAccountsView)
+        , m_haveUnusedCategories(false)
+        , m_onlinePlugins(nullptr)
+        , m_proxyModel(nullptr)
     {
     }
 
@@ -66,19 +69,56 @@ public:
     void init()
     {
         Q_Q(KAccountsView);
-        m_accountTree = &ui->m_accountTree;
+
+        ui->setupUi(q);
 
         // setup icons for collapse and expand button
         ui->m_collapseButton->setIcon(Icons::get(Icon::ListCollapse));
         ui->m_expandButton->setIcon(Icons::get(Icon::ListExpand));
 
-        m_proxyModel = ui->m_accountTree->init(View::Accounts);
-        q->connect(m_proxyModel, &AccountsProxyModel::unusedIncomeExpenseAccountHidden, q, &KAccountsView::slotUnusedIncomeExpenseAccountHidden);
+        // setup filter
+        m_proxyModel = ui->m_accountTree->proxyModel();
         q->connect(ui->m_searchWidget, &QLineEdit::textChanged, m_proxyModel, &QSortFilterProxyModel::setFilterFixedString);
-        q->connect(ui->m_accountTree, &KMyMoneyAccountTreeView::selectByObject, q, &KAccountsView::selectByObject);
-        q->connect(ui->m_accountTree, &KMyMoneyAccountTreeView::selectByVariant, q, &KAccountsView::selectByVariant);
 
-        q->connect(MyMoneyFile::instance(), &MyMoneyFile::dataChanged, q, &KAccountsView::refresh);
+        auto columnSelector = new ColumnSelector(ui->m_accountTree, q->metaObject()->className());
+        columnSelector->setAlwaysVisible(QVector<int>({ AccountsModel::Column::AccountName }));
+        columnSelector->setAlwaysHidden(QVector<int>({
+            AccountsModel::Column::PostedValue,
+            AccountsModel::Column::BankCode,
+            AccountsModel::Column::Bic,
+            AccountsModel::Column::CostCenter,
+        }));
+
+        ui->m_accountTree->setModel(MyMoneyFile::instance()->accountsModel());
+        m_proxyModel->addAccountGroup(AccountsProxyModel::assetLiabilityEquity());
+        m_proxyModel->setFilterComboBox(ui->m_filterBox);
+        m_proxyModel->setClosedSelectable(true);
+
+        columnSelector->setModel(m_proxyModel);
+        q->slotSettingsChanged();
+
+        // forward the widget requests
+        q->connect(ui->m_accountTree, &KMyMoneyAccountTreeView::requestCustomContextMenu, q, &KAccountsView::requestCustomContextMenu);
+        q->connect(ui->m_accountTree, &KMyMoneyAccountTreeView::requestSelectionChange, q, &KAccountsView::requestSelectionChange);
+        q->connect(ui->m_accountTree, &KMyMoneyAccountTreeView::requestActionTrigger, q, &KAccountsView::requestActionTrigger);
+
+        m_focusWidget = ui->m_accountTree;
+
+        // make sure to update our local copy if the account data changes
+        q->connect(ui->m_accountTree->model(), &QAbstractItemModel::dataChanged, q, [&](const QModelIndex& topLeft, const QModelIndex& bottomRight) {
+            // it should be only one row, but we never know and so we loop over all of them
+            for (int row = topLeft.row(); row <= bottomRight.row(); ++row) {
+                const auto idx = ui->m_accountTree->model()->index(row, 0, topLeft.parent());
+                const auto id = idx.data(eMyMoney::Model::IdRole).toString();
+                if (id == m_currentAccount.id()) {
+                    m_currentAccount = MyMoneyFile::instance()->accountsModel()->itemById(id);
+                    break;
+                }
+            }
+        });
+
+        // ignore entries in the root level, they are the account groups and not accounts
+        ui->m_accountTree->setSkipRootLevelEntries(true);
     }
 
     void editLoan()
@@ -169,14 +209,14 @@ public:
         // check for online modules
         QMap<QString, KMyMoneyPlugin::OnlinePlugin *>::const_iterator it_plugin;
         if (m_onlinePlugins) {
-            it_plugin = m_onlinePlugins->constEnd();
+            it_plugin = m_onlinePlugins->cend();
             const auto& kvp = m_currentAccount.onlineBankingSettings();
             if (!kvp["provider"].isEmpty()) {
                 // if we have an online provider for this account, we need to check
                 // that we have the corresponding plugin. If that exists, we ask it
                 // to provide an additional tab for the account editor.
                 it_plugin = m_onlinePlugins->constFind(kvp["provider"].toLower());
-                if (it_plugin != m_onlinePlugins->constEnd()) {
+                if (it_plugin != m_onlinePlugins->cend()) {
                     QString name;
                     auto w = (*it_plugin)->accountConfigTab(m_currentAccount, name);
                     dlg->addTab(w, name);
@@ -184,58 +224,66 @@ public:
             }
         }
 
-        if (dlg != 0 && dlg->exec() == QDialog::Accepted) {
-            try {
-                MyMoneyFileTransaction ft;
-
-                auto account = dlg->account();
-                auto parent = dlg->parentAccount();
-                if (m_onlinePlugins && it_plugin != m_onlinePlugins->constEnd()) {
-                    account.setOnlineBankingSettings((*it_plugin)->onlineBankingSettings(account.onlineBankingSettings()));
-                }
-                auto bal = dlg->openingBalance();
-                if (m_currentAccount.accountGroup() == eMyMoney::Account::Type::Liability) {
-                    bal = -bal;
-                }
-
-                // we need to modify first, as reparent would override all other changes
-                file->modifyAccount(account);
-                if (account.parentAccountId() != parent.id()) {
-                    file->reparentAccount(account, parent);
-                }
-                if (!tid.isEmpty() && dlg->openingBalance().isZero()) {
-                    file->removeTransaction(t);
-
-                } else if (!tid.isEmpty() && !dlg->openingBalance().isZero()) {
-                    // update transaction only if changed
-                    if ((s0.shares() != bal) || (t.postDate() != account.openingDate())) {
-                        s0.setShares(bal);
-                        s0.setValue(bal);
-                        t.modifySplit(s0);
-                        s1.setShares(-bal);
-                        s1.setValue(-bal);
-                        t.modifySplit(s1);
-                        t.setPostDate(account.openingDate());
-                        file->modifyTransaction(t);
+        if (dlg->exec() == QDialog::Accepted) {
+            if (dlg != nullptr) {
+                try {
+                    auto account = dlg->account();
+                    auto parent = dlg->parentAccount();
+                    auto bal = dlg->openingBalance();
+                    if (account.accountGroup() == eMyMoney::Account::Type::Liability) {
+                        bal = -bal;
                     }
-                } else if (tid.isEmpty() && !dlg->openingBalance().isZero()) {
-                    file->createOpeningBalanceTransaction(m_currentAccount, bal);
+
+                    // determine if the opening balance transaction will change
+
+                    // do we remove the transaction?
+                    auto balanceTransactionChanges = (!tid.isEmpty() && bal.isZero());
+                    // do we modify the transaction?
+                    balanceTransactionChanges |= (!tid.isEmpty() && !bal.isZero() && ((s0.shares() != bal) || (t.postDate() != account.openingDate())));
+                    // do we create the transaction?
+                    balanceTransactionChanges |= (tid.isEmpty() && !bal.isZero());
+                    // or do we reparent the account
+                    balanceTransactionChanges |= (account.parentAccountId() != parent.id());
+
+                    MyMoneyFileTransaction ft(i18nc("Undo action description", "Edit account"), balanceTransactionChanges);
+
+                    if (m_onlinePlugins && it_plugin != m_onlinePlugins->cend()) {
+                        account.setOnlineBankingSettings((*it_plugin)->onlineBankingSettings(account.onlineBankingSettings()));
+                    }
+                    // we need to modify first, as reparent would override all other changes
+                    file->modifyAccount(account);
+                    if (account.parentAccountId() != parent.id()) {
+                        file->reparentAccount(account, parent);
+                    }
+                    if (!tid.isEmpty()) {
+                        if (bal.isZero()) {
+                            file->removeTransaction(t);
+
+                        } else if ((s0.shares() != bal) || (t.postDate() != account.openingDate())) {
+                            s0.setShares(bal);
+                            s0.setValue(bal);
+                            t.modifySplit(s0);
+                            s1.setShares(-bal);
+                            s1.setValue(-bal);
+                            t.modifySplit(s1);
+                            t.setPostDate(account.openingDate());
+                            file->modifyTransaction(t);
+                        }
+
+                    } else if (!bal.isZero()) {
+                        file->createOpeningBalanceTransaction(account, bal);
+                    }
+
+                    ft.commit();
+                    m_currentAccount = account;
+
+                } catch (const MyMoneyException& e) {
+                    Q_Q(KAccountsView);
+                    KMessageBox::error(q, i18n("Unable to modify account '%1'. Cause: %2", m_currentAccount.name(), e.what()));
                 }
-
-                ft.commit();
-
-                // reload the account object as it might have changed in the meantime
-                //            slotSelectAccount(file->account(account.id()));
-
-            } catch (const MyMoneyException &e) {
-                Q_Q(KAccountsView);
-                KMessageBox::error(q, i18n("Unable to modify account '%1'. Cause: %2", m_currentAccount.name(), e.what()));
             }
         }
-
         delete dlg;
-        //        ready();
-
     }
 
     enum CanCloseAccountCodeE {
@@ -284,7 +332,7 @@ public:
         // there must be no unfinished schedule referencing the account
         QList<MyMoneySchedule> list = MyMoneyFile::instance()->scheduleList();
         QList<MyMoneySchedule>::const_iterator it_l;
-        for (it_l = list.constBegin(); it_l != list.constEnd(); ++it_l) {
+        for (it_l = list.cbegin(); it_l != list.cend(); ++it_l) {
             if ((*it_l).isFinished())
                 continue;
             if ((*it_l).hasReferenceTo(acc.id()))
@@ -324,7 +372,6 @@ public:
     void accountsUpdateOnline(const QList<MyMoneyAccount>& accList)
     {
         Q_Q(KAccountsView);
-
         // block the update account actions for now so that we don't get here twice
         const QVector<eMenu::Action> disabledActions {eMenu::Action::UpdateAccount, eMenu::Action::UpdateAllAccounts};
         for (const auto& a : disabledActions)
@@ -338,7 +385,8 @@ public:
         // 'moreAccounts' parameter in the call to updateAccount() to false
         auto processedAccounts = 0;
 
-        for (auto it_provider = m_onlinePlugins->constBegin(); it_provider != m_onlinePlugins->constEnd(); ++it_provider) {
+        Q_EMIT q->beginImportingStatements();
+        for (auto it_provider = m_onlinePlugins->cbegin(); it_provider != m_onlinePlugins->cend(); ++it_provider) {
             auto nextAccount = accList.cend();
             for (auto it_a = accList.cbegin(); it_a != accList.cend(); ++it_a) {
                 if ((*it_a).hasOnlineMapping()
@@ -355,31 +403,71 @@ public:
                 (*it_provider)->updateAccount(*nextAccount, false);
             }
         }
+        Q_EMIT q->endImportingStatements();
 
         // re-enable the disabled actions
-        q->updateActions(m_currentAccount);
+        updateActions(m_currentAccount);
 
         KMyMoneyUtils::showStatementImportResult(MyMoneyStatementReader::resultMessages(), processedAccounts);
     }
 
-    void closeSubAccounts(const MyMoneyAccount& account)
+    void updateActions(const MyMoneyAccount& acc)
     {
-        MyMoneyFile* file = MyMoneyFile::instance();
-        const auto accountList = account.accountList();
+        const auto file = MyMoneyFile::instance();
+        switch (acc.accountGroup()) {
+        case eMyMoney::Account::Type::Asset:
+        case eMyMoney::Account::Type::Liability:
+        case eMyMoney::Account::Type::Equity:
+        {
+            auto isClosed = acc.isClosed() ? true : false;
+            pActions[eMenu::Action::EditAccount]->setEnabled(!isClosed);
+            pActions[eMenu::Action::DeleteAccount]->setEnabled(!file->isReferenced(acc));
 
-        for (const auto& subAccountId : accountList) {
-            auto subAccount = file->account(subAccountId);
-            closeSubAccounts(subAccount);
-            subAccount.setClosed(true);
-            file->modifyAccount(subAccount);
+            pActions[eMenu::Action::ReopenAccount]->setEnabled(isClosed);
+            pActions[eMenu::Action::CloseAccount]->setEnabled(!isClosed);
+
+            if (!isClosed) {
+                const auto canClose = (canCloseAccount(acc) == KAccountsViewPrivate::AccountCanClose) ? true : false;
+                pActions[eMenu::Action::CloseAccount]->setEnabled(canClose);
+                hintCloseAccountAction(acc, pActions[eMenu::Action::CloseAccount]);
+            }
+
+            pActions[eMenu::Action::ChartAccountBalance]->setEnabled(true);
+
+            if (acc.hasOnlineMapping()) {
+                pActions[eMenu::Action::MapOnlineAccount]->setEnabled(false);
+                pActions[eMenu::Action::UnmapOnlineAccount]->setEnabled(true);
+
+                if (m_onlinePlugins) {
+                    // check if provider is available
+                    QMap<QString, KMyMoneyPlugin::OnlinePlugin*>::const_iterator it_p;
+                    it_p = m_onlinePlugins->constFind(acc.onlineBankingSettings().value(QLatin1String("provider")).toLower());
+                    if (it_p != m_onlinePlugins->cend()) {
+                        QStringList protocols;
+                        (*it_p)->protocols(protocols);
+                        if (protocols.count() > 0) {
+                            pActions[eMenu::Action::UpdateAccount]->setEnabled(true);
+                        }
+                    }
+                }
+
+            } else {
+                pActions[eMenu::Action::MapOnlineAccount]->setEnabled(!acc.isClosed() && m_onlinePlugins && !m_onlinePlugins->isEmpty());
+            }
+
+            break;
         }
-    };
+        default:
+            break;
+        }
+        pActions[eMenu::Action::UpdateAllAccounts]->setEnabled(KMyMoneyUtils::canUpdateAllAccounts());
+    }
 
-    KAccountsView* q_ptr;
-    Ui::KAccountsView* ui;
-    bool m_haveUnusedCategories;
-    MyMoneyAccount m_currentAccount;
+    Ui::KAccountsView   *ui;
+    bool                m_haveUnusedCategories;
+    MyMoneyAccount      m_currentAccount;
     QMap<QString, KMyMoneyPlugin::OnlinePlugin*>* m_onlinePlugins;
+    AccountsProxyModel* m_proxyModel;
 };
 
 #endif
