@@ -6,6 +6,7 @@
     SPDX-FileCopyrightText: 2000-2002 Thomas Baumgart <ipwizard@users.sourceforge.net>
     SPDX-FileCopyrightText: 2000-2002 Kevin Tambascio <ktambascio@users.sourceforge.net>
     SPDX-FileCopyrightText: 2017 Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
+    SPDX-FileCopyrightText: 2020 Robert Szczesiak <dev.rszczesiak@gmail.com>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
@@ -16,7 +17,6 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
-#include <QTimer>
 #include <QBitArray>
 #include <QMenu>
 
@@ -28,6 +28,7 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
+#include "mymoneyfile.h"
 #include "mymoneyexception.h"
 #include "kmymoneysettings.h"
 #include "knewaccountdlg.h"
@@ -39,111 +40,94 @@
 #include "mymoneyenums.h"
 #include "storageenums.h"
 #include "menuenums.h"
+#include "mymoneymoney.h"
+#include "accountdelegate.h"
 
 using namespace Icons;
 
 KCategoriesView::KCategoriesView(QWidget *parent) :
-    KMyMoneyAccountsViewBase(*new KCategoriesViewPrivate(this), parent)
+    KMyMoneyViewBase(*new KCategoriesViewPrivate(this), parent)
 {
     Q_D(KCategoriesView);
-    d->ui->setupUi(this);
+    d->init();
 
     connect(pActions[eMenu::Action::NewCategory],    &QAction::triggered, this, &KCategoriesView::slotNewCategory);
     connect(pActions[eMenu::Action::EditCategory],   &QAction::triggered, this, &KCategoriesView::slotEditCategory);
     connect(pActions[eMenu::Action::DeleteCategory], &QAction::triggered, this, &KCategoriesView::slotDeleteCategory);
+
+    d->ui->m_accountTree->setItemDelegate(new AccountDelegate(d->ui->m_accountTree));
+    connect(MyMoneyFile::instance()->accountsModel(), &AccountsModel::profitLossChanged, this, &KCategoriesView::slotProfitLossChanged);
+
+    d->m_sharedToolbarActions.insert(eMenu::Action::FileNew, pActions[eMenu::Action::NewCategory]);
+
+    d->ui->m_accountTree->setSelectionMode(QAbstractItemView::SingleSelection);
+    d->ui->m_accountTree->setDragEnabled(true);
+    d->ui->m_accountTree->setAcceptDrops(true);
+    d->ui->m_accountTree->setDropIndicatorShown(true);
+    d->ui->m_accountTree->setDragDropMode(QAbstractItemView::InternalMove);
 }
 
 KCategoriesView::~KCategoriesView()
 {
 }
 
-void KCategoriesView::executeCustomAction(eView::Action action)
+void KCategoriesView::slotSettingsChanged()
 {
     Q_D(KCategoriesView);
-    switch(action) {
-    case eView::Action::Refresh:
-        refresh();
-        break;
-
-    case eView::Action::SetDefaultFocus:
-        QTimer::singleShot(0, d->ui->m_accountTree, SLOT(setFocus()));
-        break;
-
-    default:
-        break;
-    }
-}
-
-void KCategoriesView::refresh()
-{
-    Q_D(KCategoriesView);
-    if (!isVisible()) {
-        d->m_needsRefresh = true;
-        return;
-    }
-    d->m_needsRefresh = false;
-
-    d->m_proxyModel->invalidate();
     d->m_proxyModel->setHideClosedAccounts(!KMyMoneySettings::showAllAccounts());
+    d->m_proxyModel->setHideEquityAccounts(true);
+    d->m_proxyModel->setHideZeroBalancedEquityAccounts(true);
+    d->m_proxyModel->setShowAllEntries(KMyMoneySettings::showAllAccounts());
+    d->m_proxyModel->setHideFavoriteAccounts(true);
 
-    // reinitialize the default state of the hidden categories label
-    d->m_haveUnusedCategories = false;
-    d->ui->m_hiddenCategories->hide();
-    d->m_proxyModel->setHideUnusedIncomeExpenseAccounts(KMyMoneySettings::hideUnusedCategory());
+    MyMoneyFile::instance()->accountsModel()->setColorScheme(AccountsModel::Positive, KMyMoneySettings::schemeColor(SchemeColor::Positive));
+    MyMoneyFile::instance()->accountsModel()->setColorScheme(AccountsModel::Negative, KMyMoneySettings::schemeColor(SchemeColor::Negative));
 }
 
-void KCategoriesView::showEvent(QShowEvent * event)
+
+void KCategoriesView::updateActions(const SelectedObjects& selections)
 {
     Q_D(KCategoriesView);
-    if (!d->m_proxyModel)
-        d->init();
+    const auto file = MyMoneyFile::instance();
 
-    emit customActionRequested(View::Categories, eView::Action::AboutToShow);
-
-    if (d->m_needsRefresh)
-        refresh();
-
-    // don't forget base class implementation
-    QWidget::showEvent(event);
-}
-
-void KCategoriesView::updateActions(const MyMoneyObject& obj)
-{
-    Q_D(KCategoriesView);
-    if (typeid(obj) != typeid(MyMoneyAccount) &&
-            (obj.id().isEmpty() && d->m_currentCategory.id().isEmpty())) // do not disable actions that were already disabled))
+    // check if there is anything todo and quit if not
+    if (selections.selection(SelectedObjects::Account).count() < 1
+            && d->m_currentCategory.id().isEmpty() ) {
         return;
+    }
 
-    const auto& acc = static_cast<const MyMoneyAccount&>(obj);
+    pActions[eMenu::Action::NewCategory]->setEnabled(true);
+    pActions[eMenu::Action::EditCategory]->setEnabled(false);
+    pActions[eMenu::Action::DeleteCategory]->setEnabled(false);
 
-    if (d->m_currentCategory.id().isEmpty() && acc.id().isEmpty())
+    const auto accountIds = selections.selection(SelectedObjects::Account);
+    if (accountIds.isEmpty()) {
+        d->m_currentCategory = MyMoneyAccount();
         return;
+    }
+    const auto acc = file->accountsModel()->itemById(accountIds.at(0));
+    auto b = file->isStandardAccount(acc.id()) ? false : true;
 
+    QBitArray skip((int)eStorage::Reference::Count);
     switch (acc.accountType()) {
     case eMyMoney::Account::Type::Income:
     case eMyMoney::Account::Type::Expense:
-    {
-        const auto file = MyMoneyFile::instance();
-        auto b = file->isStandardAccount(acc.id()) ? false : true;
+        d->m_currentCategory = acc;
         pActions[eMenu::Action::EditCategory]->setEnabled(b);
+
         // enable delete action, if category/account itself is not referenced
         // by any object except accounts, because we want to allow
         // deleting of sub-categories. Also, we allow transactions, schedules and budgets
         // to be present because we can re-assign them during the delete process
-        QBitArray skip((int)eStorage::Reference::Count);
         skip.fill(false);
         skip.setBit((int)eStorage::Reference::Transaction);
         skip.setBit((int)eStorage::Reference::Account);
         skip.setBit((int)eStorage::Reference::Schedule);
         skip.setBit((int)eStorage::Reference::Budget);
-
         pActions[eMenu::Action::DeleteCategory]->setEnabled(b && !file->isReferenced(acc, skip));
-        d->m_currentCategory = acc;
         break;
-    }
+
     default:
-        pActions[eMenu::Action::EditCategory]->setEnabled(false);
-        pActions[eMenu::Action::DeleteCategory]->setEnabled(false);
         d->m_currentCategory = MyMoneyAccount();
         break;
     }
@@ -159,44 +143,19 @@ void KCategoriesView::slotUnusedIncomeExpenseAccountHidden()
     d->ui->m_hiddenCategories->setVisible(d->m_haveUnusedCategories);
 }
 
-void KCategoriesView::slotProfitChanged(const MyMoneyMoney &profit)
+void KCategoriesView::slotProfitLossChanged(const MyMoneyMoney &profit, bool isApproximate)
 {
     Q_D(KCategoriesView);
-    d->netBalProChanged(profit, d->ui->m_totalProfitsLabel, View::Categories);
-}
-
-void KCategoriesView::slotShowCategoriesMenu(const MyMoneyAccount& acc)
-{
-    Q_UNUSED(acc);
-    pMenus[eMenu::Menu::Category]->exec(QCursor::pos());
-}
-
-void KCategoriesView::slotSelectByObject(const MyMoneyObject& obj, eView::Intent intent)
-{
-    switch(intent) {
-    case eView::Intent::UpdateActions:
-        updateActions(obj);
-        break;
-
-    case eView::Intent::OpenContextMenu:
-        slotShowCategoriesMenu(static_cast<const MyMoneyAccount&>(obj));
-        break;
-
-    default:
-        break;
-    }
-}
-
-void KCategoriesView::slotSelectByVariant(const QVariantList& variant, eView::Intent intent)
-{
-    switch (intent) {
-    case eView::Intent::UpdateProfit:
-        if (variant.count() == 1)
-            slotProfitChanged(variant.first().value<MyMoneyMoney>());
-        break;
-    default:
-        break;
-    }
+    const auto formattedValue = profit.isNegative() ? d->formatViewLabelValue(-profit, KMyMoneySettings::schemeColor(SchemeColor::Negative))
+                                : d->formatViewLabelValue(profit, KMyMoneySettings::schemeColor(SchemeColor::Positive));
+    if (profit.isNegative())
+        d->updateViewLabel(d->ui->m_totalProfitsLabel,
+                           isApproximate ? i18nc("Approximate loss", "Loss: ~%1", formattedValue)
+                           : i18n("Loss: %1", formattedValue));
+    else
+        d->updateViewLabel(d->ui->m_totalProfitsLabel,
+                           isApproximate ? i18nc("Approximate profit", "Profit: ~%1", formattedValue)
+                           : i18n("Profit: %1", formattedValue));
 }
 
 void KCategoriesView::slotNewCategory()
@@ -235,39 +194,35 @@ void KCategoriesView::slotEditCategory()
         return;
     }
 
-    // set a status message so that the application can't be closed until the editing is done
-    //        slotStatusMsg(caption);
-
     QPointer<KNewAccountDlg> dlg =
         new KNewAccountDlg(d->m_currentCategory, true, true, 0, i18n("Edit category '%1'", d->m_currentCategory.name()));
 
     dlg->setOpeningBalanceShown(false);
     dlg->setOpeningDateShown(false);
 
-    if (dlg && dlg->exec() == QDialog::Accepted) {
-        try {
-            MyMoneyFileTransaction ft;
+    if (dlg->exec() == QDialog::Accepted) {
+        if (dlg != nullptr) {
+            try {
+                MyMoneyFileTransaction ft(i18nc("Undo action description", "Edit category"), false);
 
-            auto account = dlg->account();
-            auto parent = dlg->parentAccount();
+                auto account = dlg->account();
+                auto parent = dlg->parentAccount();
 
-            // we need to modify first, as reparent would override all other changes
-            file->modifyAccount(account);
-            if (account.parentAccountId() != parent.id())
-                file->reparentAccount(account, parent);
+                // we need to modify first, as reparent would override all other changes
+                file->modifyAccount(account);
+                if (account.parentAccountId() != parent.id())
+                    file->reparentAccount(account, parent);
 
-            ft.commit();
-
-            // reload the account object as it might have changed in the meantime
-            emit selectByObject(account, eView::Intent::None);
-
-        } catch (const MyMoneyException &e) {
-            KMessageBox::error(this, i18n("Unable to modify category '%1'. Cause: %2", d->m_currentCategory.name(), QString::fromLatin1(e.what())));
+                ft.commit();
+                // update our local copy of the category data
+                d->m_currentCategory = account;
+            } catch (const MyMoneyException& e) {
+                KMessageBox::error(this, i18n("Unable to modify category '%1'. Cause: %2", d->m_currentCategory.name(), QString::fromLatin1(e.what())));
+            }
         }
     }
 
     delete dlg;
-    //        ready();
 }
 
 void KCategoriesView::slotDeleteCategory()
@@ -312,7 +267,6 @@ void KCategoriesView::slotDeleteCategory()
         auto newCategory = file->account(categoryId);
         try {
             {
-                //        KMSTATUS(i18n("Adjusting transactions..."));
                 /*
                   d->m_currentCategory.id() is the old id, categoryId the new one
                   Now search all transactions and schedules that reference d->m_currentCategory.id()
@@ -325,39 +279,29 @@ void KCategoriesView::slotDeleteCategory()
                 QList<MyMoneyTransaction>::iterator it_t;
                 file->transactionList(tlist, filter);
 
-                //        slotStatusProgressBar(0, tlist.count());
-//          int cnt = 0;
                 for (it_t = tlist.begin(); it_t != tlist.end(); ++it_t) {
-                    //          slotStatusProgressBar(++cnt, 0);
                     MyMoneyTransaction t = (*it_t);
                     if (t.replaceId(categoryId, d->m_currentCategory.id()))
                         file->modifyTransaction(t);
                 }
-                //        slotStatusProgressBar(tlist.count(), 0);
             }
             // now fix all schedules
             {
-                //        KMSTATUS(i18n("Adjusting scheduled transactions..."));
                 QList<MyMoneySchedule> slist = file->scheduleList(d->m_currentCategory.id());
                 QList<MyMoneySchedule>::iterator it_s;
 
-//          int cnt = 0;
-                //        slotStatusProgressBar(0, slist.count());
                 for (it_s = slist.begin(); it_s != slist.end(); ++it_s) {
-                    //          slotStatusProgressBar(++cnt, 0);
                     MyMoneySchedule sch = (*it_s);
                     if (sch.replaceId(categoryId, d->m_currentCategory.id())) {
                         file->modifySchedule(sch);
                     }
                 }
-                //        slotStatusProgressBar(slist.count(), 0);
             }
             // now fix all budgets
             {
-                //        KMSTATUS(i18n("Adjusting budgets..."));
                 QList<MyMoneyBudget> blist = file->budgetList();
                 QList<MyMoneyBudget>::const_iterator it_b;
-                for (it_b = blist.constBegin(); it_b != blist.constEnd(); ++it_b) {
+                for (it_b = blist.cbegin(); it_b != blist.cend(); ++it_b) {
                     if ((*it_b).hasReferenceTo(d->m_currentCategory.id())) {
                         MyMoneyBudget b = (*it_b);
                         MyMoneyBudget::AccountGroup fromBudget = b.account(d->m_currentCategory.id());
@@ -366,17 +310,13 @@ void KCategoriesView::slotDeleteCategory()
                         b.setAccount(toBudget, categoryId);
                         b.removeReference(d->m_currentCategory.id());
                         file->modifyBudget(b);
-
                     }
                 }
-                //        slotStatusProgressBar(blist.count(), 0);
             }
         } catch (MyMoneyException &e) {
             KMessageBox::error(this, i18n("Unable to exchange category <b>%1</b> with category <b>%2</b>. Reason: %3", d->m_currentCategory.name(), newCategory.name(), QString::fromLatin1(e.what())));
-            //    slotStatusProgressBar(-1, -1);
             return;
         }
-        //    slotStatusProgressBar(-1, -1);
     }
 
     // retain the account name for a possible later usage in the error message box
@@ -393,11 +333,16 @@ void KCategoriesView::slotDeleteCategory()
 
         // case A - only a single, unused category without subcats selected
         if (d->m_currentCategory.accountList().isEmpty()) {
-            if (!needAskUser || (KMessageBox::questionYesNo(this, i18n("<qt>Do you really want to delete category <b>%1</b>?</qt>", selectedAccountName)) == KMessageBox::Yes)) {
+            if (!needAskUser
+                || (KMessageBox::questionTwoActions(this,
+                                                    i18n("<qt>Do you really want to delete category <b>%1</b>?</qt>", selectedAccountName),
+                                                    i18nc("@title:window", "Delete category"),
+                                                    KMMYesNo::yes(),
+                                                    KMMYesNo::no())
+                    == KMessageBox::PrimaryAction)) {
                 try {
                     file->removeAccount(d->m_currentCategory);
                     d->m_currentCategory.clearId();
-                    emit selectByObject(d->m_currentCategory, eView::Intent::None);
                     ft.commit();
                 } catch (const MyMoneyException &e) {
                     KMessageBox::error(this, i18n("<qt>Unable to delete category <b>%1</b>. Cause: %2</qt>", selectedAccountName, QString::fromLatin1(e.what())));
@@ -410,25 +355,28 @@ void KCategoriesView::slotDeleteCategory()
         auto parentAccount = file->account(d->m_currentCategory.parentAccountId());
 
         QStringList accountsToReparent;
-        int result = KMessageBox::questionYesNoCancel(this,
-                     i18n("<qt>Do you want to delete category <b>%1</b> with all its sub-categories or only "
-                          "the category itself? If you only delete the category itself, all its sub-categories "
-                          "will be made sub-categories of <b>%2</b>.</qt>", selectedAccountName, parentAccount.name()),
-                     QString(),
-                     KGuiItem(i18n("Delete all")),
-                     KGuiItem(i18n("Just the category")));
+        int result = KMessageBox::questionTwoActionsCancel(this,
+                                                           i18n("<qt>Do you want to delete category <b>%1</b> with all its sub-categories or only "
+                                                                "the category itself? If you only delete the category itself, all its sub-categories "
+                                                                "will be made sub-categories of <b>%2</b>.</qt>",
+                                                                selectedAccountName,
+                                                                parentAccount.name()),
+                                                           QString(),
+                                                           KGuiItem(i18n("Delete all")),
+                                                           KGuiItem(i18n("Just the category")));
         if (result == KMessageBox::Cancel)
             return; // cancel pressed? ok, no delete then...
         // "No" means "Just the category" and that means we need to reparent all subaccounts
         bool need_confirmation = false;
         // case C - User only wants to delete the category itself
-        if (result == KMessageBox::No)
+        if (result == KMessageBox::SecondaryAction)
             accountsToReparent = d->m_currentCategory.accountList();
         else {
             // case D - User wants to delete all subcategories, now check all subcats of
             //          d->m_currentCategory and remember all that cannot be deleted and
             //          must be "reparented"
-            foreach (const auto accountID, d->m_currentCategory.accountList()) {
+            const auto subAccountList = d->m_currentCategory.accountList();
+            for (const auto& accountID : qAsConst(subAccountList)) {
                 // reparent account if a transaction is assigned
                 if (file->transactionCount(accountID) != 0)
                     accountsToReparent.push_back(accountID);
@@ -444,15 +392,22 @@ void KCategoriesView::slotDeleteCategory()
                 need_confirmation = true;
         }
         if (!accountsToReparent.isEmpty() && need_confirmation) {
-            if (KMessageBox::questionYesNo(this, i18n("<p>Some sub-categories of category <b>%1</b> cannot "
-                                           "be deleted, because they are still used. They will be made sub-categories of <b>%2</b>. Proceed?</p>", selectedAccountName, parentAccount.name())) != KMessageBox::Yes) {
+            if (KMessageBox::questionTwoActions(this,
+                                                i18n("<p>Some sub-categories of category <b>%1</b> cannot "
+                                                     "be deleted, because they are still used. They will be made sub-categories of <b>%2</b>. Proceed?</p>",
+                                                     selectedAccountName,
+                                                     parentAccount.name()),
+                                                i18nc("@title:window", "Delete category"),
+                                                KMMYesNo::yes(),
+                                                KMMYesNo::no())
+                != KMessageBox::PrimaryAction) {
                 return; // user gets wet feet...
             }
         }
         // all good, now first reparent selected sub-categories
         try {
             auto parent = file->account(d->m_currentCategory.parentAccountId());
-            for (QStringList::const_iterator it = accountsToReparent.constBegin(); it != accountsToReparent.constEnd(); ++it) {
+            for (QStringList::const_iterator it = accountsToReparent.cbegin(); it != accountsToReparent.cend(); ++it) {
                 auto child = file->account(*it);
                 file->reparentAccount(child, parent);
             }
@@ -469,15 +424,11 @@ void KCategoriesView::slotDeleteCategory()
         }
     }
     // the category/account is deleted after the switch
-
-
     try {
         file->removeAccount(d->m_currentCategory);
         d->m_currentCategory.clearId();
-        emit selectByObject(MyMoneyAccount(), eView::Intent::None);
         ft.commit();
     } catch (const MyMoneyException &e) {
         KMessageBox::error(this, i18n("Unable to delete category '%1'. Cause: %2", selectedAccountName, QString::fromLatin1(e.what())));
     }
 }
-
