@@ -6,6 +6,7 @@
     SPDX-FileCopyrightText: 2000-2002 Kevin Tambascio <ktambascio@users.sourceforge.net>
     SPDX-FileCopyrightText: 2017 Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
     SPDX-FileCopyrightText: 2002-2019 Thomas Baumgart <tbaumgart@kde.org>
+    SPDX-FileCopyrightText: 2021 Dawid Wróbel <me@dawidwrobel.com>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
@@ -13,6 +14,8 @@
 
 // ----------------------------------------------------------------------------
 // QT Includes
+#include <QPrintPreviewDialog>
+#include <QTimer>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
@@ -33,12 +36,10 @@ KHomeView::~KHomeView()
 
 void KHomeView::slotAdjustScrollPos()
 {
-#ifndef ENABLE_WEBENGINE
     Q_D(KHomeView);
-    if (d && d->m_view && d->m_view->page() && d->m_view->page()->mainFrame()) {
-        d->m_view->page()->mainFrame()->setScrollBarValue(Qt::Vertical, d->m_scrollBarPos);
-    }
-#endif
+
+    if (d && d->m_view)
+        d->m_view->verticalScrollBar()->setValue(d->m_scrollBarPos);
 }
 
 bool KHomeView::eventFilter(QObject* o, QEvent* e)
@@ -54,38 +55,43 @@ bool KHomeView::eventFilter(QObject* o, QEvent* e)
     return KMyMoneyViewBase::eventFilter(o, e);
 }
 
-void KHomeView::wheelEvent(QWheelEvent* event)
+void KHomeView::executeAction(eMenu::Action action, const SelectedObjects& selections)
 {
+    Q_UNUSED(selections)
+
     Q_D(KHomeView);
-    // Zoom text on Ctrl + Scroll
-    if (event->modifiers() & Qt::CTRL) {
-        qreal factor = d->m_view->zoomFactor();
-        if (event->delta() > 0)
-            factor += 0.1;
-        else if (event->delta() < 0)
-            factor -= 0.1;
-        d->m_view->setZoomFactor(factor);
-        event->accept();
-        return;
+    switch (action) {
+    case eMenu::Action::FileNew:
+        d->m_fileOpen = true;
+        break;
+    case eMenu::Action::Print:
+        if (d->isActiveView()) {
+            slotPrintView();
+        }
+        break;
+    case eMenu::Action::PrintPreview:
+        if (d->isActiveView()) {
+            slotPrintPreviewView();
+        }
+        break;
+    case eMenu::Action::FileClose:
+        d->m_fileOpen = false;
+        d->loadView();
+        break;
+    default:
+        break;
     }
 }
 
 void KHomeView::executeCustomAction(eView::Action action)
 {
-    Q_D(KHomeView);
     switch(action) {
-    case eView::Action::Refresh:
-        refresh();
+    case eView::Action::BlockViewDuringFileOpen:
+        slotDisableRefresh();
         break;
-
-    case eView::Action::Print:
-        slotPrintView();
+    case eView::Action::UnblockViewAfterFileOpen:
+        slotEnableRefresh();
         break;
-
-    case eView::Action::CleanupBeforeFileClose:
-        d->m_view->setHtml(KWelcomePage::welcomePage(), QUrl("file://"));
-        break;
-
     default:
         break;
     }
@@ -94,7 +100,7 @@ void KHomeView::executeCustomAction(eView::Action action)
 void KHomeView::refresh()
 {
     Q_D(KHomeView);
-    if (isVisible()) {
+    if (isVisible() && !d->m_skipRefresh && !d->m_resizeRefreshTimer.isActive()) {
         d->loadView();
         d->m_needsRefresh = false;
     } else {
@@ -102,14 +108,20 @@ void KHomeView::refresh()
     }
 }
 
+void KHomeView::resizeEvent(QResizeEvent* event)
+{
+    Q_D(KHomeView);
+    d->repaintAfterResize(event->oldSize(), event->size());
+    KMyMoneyViewBase::resizeEvent(event);
+}
+
 void KHomeView::showEvent(QShowEvent* event)
 {
     Q_D(KHomeView);
+
     if (d->m_needLoad)
         d->init();
 
-    emit customActionRequested(View::Home, eView::Action::AboutToShow);
-
     if (d->m_needsRefresh)
         refresh();
 
@@ -122,52 +134,65 @@ void KHomeView::slotPrintView()
     if (d->m_view) {
         auto printer = KMyMoneyPrinter::startPrint();
         if (printer != nullptr) {
-#ifdef ENABLE_WEBENGINE
-            d->m_view->page()->print(printer, [=] (bool) {});
-#else
             d->m_view->print(printer);
-#endif
         }
     }
 }
 
+void KHomeView::slotPrintPreviewView()
+{
+    Q_D(KHomeView);
+    if (d->m_view) {
+        QPrintPreviewDialog dlg(KMyMoneyPrinter::instance(), d->m_view);
+        connect(&dlg, &QPrintPreviewDialog::paintRequested, d->m_view, [&](QPrinter* printer) {
+            Q_D(KHomeView);
+            d->m_view->print(printer);
+        });
+        dlg.exec();
+    }
+}
+
 void KHomeView::slotOpenUrl(const QUrl &url)
 {
     Q_D(KHomeView);
 
+    auto triggerAction = [&](eMenu::Action action, const QString& id) {
+        pActions[action]->setData(id);
+        Q_EMIT requestActionTrigger(action);
+    };
+
     QString protocol = url.scheme();
     QString view = url.fileName();
 
-    // empty view -> bail out
-    if (view.isEmpty())
-        return;
-
     QUrlQuery query(url);
     QString id = query.queryItemValue("id");
     QString mode = query.queryItemValue("mode");
 
-    const auto file = MyMoneyFile::instance();
-
     if (protocol == QLatin1String("https")) {
         QDesktopServices::openUrl(url);
     } else if (protocol == QLatin1String("mailto")) {
         QDesktopServices::openUrl(url);
     } else {
+        // empty view -> bail out
+        if (view.isEmpty())
+            return;
+
         KXmlGuiWindow* mw = KMyMoneyUtils::mainWindow();
         Q_CHECK_PTR(mw);
         if (view == VIEW_LEDGER) {
-            emit selectByVariant(QVariantList {QVariant(id), QVariant(QString())}, eView::Intent::ShowTransaction);
+            pActions[eMenu::Action::GoToAccount]->setData(id);
+            Q_EMIT requestActionTrigger(eMenu::Action::GoToAccount);
 
         } else if (view == VIEW_SCHEDULE) {
             if (mode == QLatin1String("enter")) {
-                emit selectByObject(file->schedule(id), eView::Intent::None);
-                QTimer::singleShot(0, pActions[eMenu::Action::EnterSchedule], SLOT(trigger()));
+                triggerAction(eMenu::Action::EnterSchedule, id);
+
             } else if (mode == QLatin1String("edit")) {
-                emit selectByObject(file->schedule(id), eView::Intent::None);
-                QTimer::singleShot(0, pActions[eMenu::Action::EditSchedule], SLOT(trigger()));
+                triggerAction(eMenu::Action::EditSchedule, id);
+
             } else if (mode == QLatin1String("skip")) {
-                emit selectByObject(file->schedule(id), eView::Intent::None);
-                QTimer::singleShot(0, pActions[eMenu::Action::SkipSchedule], SLOT(trigger()));
+                triggerAction(eMenu::Action::SkipSchedule, id);
+
             } else if (mode == QLatin1String("full")) {
                 d->m_showAllSchedules = true;
                 d->loadView();
@@ -178,17 +203,14 @@ void KHomeView::slotOpenUrl(const QUrl &url)
             }
 
         } else if (view == VIEW_REPORTS) {
-            emit selectByObject(file->report(id), eView::Intent::OpenObject);
-//      emit openObjectRequested(file->report(id));
+            triggerAction(eMenu::Action::ReportOpen, id);
 
         } else if (view == VIEW_WELCOME) {
-            if (mode == QLatin1String("whatsnew"))
-                d->m_view->setHtml(KWelcomePage::whatsNewPage(), QUrl("file://"));
-            else
-                d->m_view->setHtml(KWelcomePage::welcomePage(), QUrl("file://"));
+            d->m_view->setHtml(KWelcomePage::welcomePage());
 
         } else if (view == QLatin1String("action")) {
-            QTimer::singleShot(0, mw->actionCollection()->action(id), SLOT(trigger()));
+            QMetaObject::invokeMethod(mw->actionCollection()->action(id), "trigger");
+
         } else if (view == VIEW_HOME) {
             QList<MyMoneyAccount> list;
             // it could be, that we don't even have a storage object attached.
@@ -196,7 +218,7 @@ void KHomeView::slotOpenUrl(const QUrl &url)
             // which we catch here and treat it as 'no accounts found'.
             try {
                 MyMoneyFile::instance()->accountList(list);
-            } catch(const MyMoneyException& e) {
+            } catch (const MyMoneyException&) {
             }
             if (list.count() == 0) {
                 KMessageBox::information(this, i18n("Before KMyMoney can give you detailed information about your financial status, you need to create at least one account. Until then, KMyMoney shows the welcome page instead."));
@@ -208,6 +230,27 @@ void KHomeView::slotOpenUrl(const QUrl &url)
         }
     }
 }
+
+void KHomeView::slotSettingsChanged()
+{
+    refresh();
+}
+
+void KHomeView::slotDisableRefresh()
+{
+    Q_D(KHomeView);
+    d->m_skipRefresh = true;
+}
+
+void KHomeView::slotEnableRefresh()
+{
+    Q_D(KHomeView);
+    d->m_skipRefresh = false;
+    // prevent stopping the timer in case we delayed the refresh
+    d->m_endSkipWithTimerRunning = d->m_resizeRefreshTimer.isActive();
+    refresh();
+}
+
 // Make sure, that these definitions are only used within this file
 // this does not seem to be necessary, but when building RPMs the
 // build option 'final' is used and all CPP files are concatenated.
