@@ -22,7 +22,6 @@
 
 // ----------------------------------------------------------------------------
 // Project Includes
-#include "selectedtransactions.h"
 
 #ifdef ENABLE_ACTIVITIES
 namespace KActivities
@@ -36,6 +35,7 @@ enum class Column;
 }
 namespace eMenu {
 enum class Action;
+enum class Menu;
 }
 namespace KMyMoneyPlugin {
 class OnlinePlugin;
@@ -44,9 +44,6 @@ namespace eDialogs {
 enum class ScheduleResultCode;
 }
 namespace eView {
-enum class Intent;
-}
-namespace eView {
 enum class Action;
 }
 namespace Icons {
@@ -73,12 +70,12 @@ class MyMoneySecurity;
 class MyMoneyReport;
 class TransactionEditor;
 class KOnlineJobOutbox;
-class KMyMoneyTitleLabel;
 class MyMoneyAccount;
 class MyMoneyMoney;
 class MyMoneyObject;
 class QLabel;
 class KMyMoneyViewBase;
+class SelectedObjects;
 
 /**
   * This class represents the view of the MyMoneyFile which contains
@@ -91,21 +88,15 @@ class KMyMoneyViewBase;
   * @short Handles the view of the MyMoneyFile.
   */
 enum class View;
+class KMyMoneyViewPrivate;
 class KMyMoneyView : public KPageWidget
 {
     Q_OBJECT
+    Q_DECLARE_PRIVATE(KMyMoneyView)
+
 private:
-
-    KPageWidgetModel* m_model;
-
-    QHash<View, KPageWidgetItem*> viewFrames;
-    QHash<View, KMyMoneyViewBase*> viewBases;
-
-    KMyMoneyTitleLabel* m_header;
-
-    void viewAccountList(const QString& selectAccount); // Show the accounts view
-
-    void createSchedule(MyMoneySchedule s, MyMoneyAccount& a);
+    const QScopedPointer<KMyMoneyViewPrivate> d_ptr;
+    Q_PRIVATE_SLOT(d_func(), void _q_selectionChangeRequested(const SelectedObjects& selection));
 
 public:
     /**
@@ -129,28 +120,14 @@ public:
     void addWidget(QWidget* w);
 
     void showPageAndFocus(View idView);
-    void showPage(View idView);
-
-    /**
-      * check if the current view allows to print something
-      *
-      * @retval true Yes, view allows to print
-      * @retval false No, view cannot print
-      */
-    bool canPrint();
-
-    void finishReconciliation(const MyMoneyAccount& account);
-
-    void showTitleBar(bool show);
+    void showPage(View idView, bool forceShow = false);
 
     /**
       * This method changes the view type according to the settings.
       */
     void updateViewType();
 
-    void slotAccountTreeViewChanged(const eAccountsModel::Column column, const bool show);
-
-    void setOnlinePlugins(QMap<QString, KMyMoneyPlugin::OnlinePlugin*>& plugins);
+    void setOnlinePlugins(QMap<QString, KMyMoneyPlugin::OnlinePlugin*>* plugins);
 
     // TODO: remove that function
     /**
@@ -168,6 +145,28 @@ public:
      */
     QHash<eMenu::Action, QAction *> actionsToBeConnected();
 
+    /**
+     * Execute the @a action using the selected objects of @a selections
+     */
+    void executeAction(eMenu::Action action, const SelectedObjects& selections);
+
+    /**
+     * Send the @a action to all views
+     */
+    void executeCustomAction(eView::Action action);
+
+    void setupSharedActions();
+
+    /**
+     * Returns @c true if the current view has a closable tab/sub-view.
+     */
+    bool hasClosableView() const;
+
+    /**
+     * Closes the current selected closable tab/sub-view in the current view.
+     */
+    void closeCurrentView();
+
 protected:
     /**
       * Overwritten because KMyMoney has it's custom header.
@@ -176,112 +175,26 @@ protected:
 
 
 public Q_SLOTS:
-    /**
-      * This slot writes information about the page passed as argument @a current
-      * in the kmymoney.rc file so that it can be selected automatically when
-      * the application is started again.
-      *
-      * @param current QModelIndex of the current page item
-      * @param previous QModelIndex of the previous page item
-      */
-    void slotCurrentPageChanged(const QModelIndex current, const QModelIndex previous);
+    void slotSettingsChanged();
 
-    /**
-      * Brings up a dialog to change the list(s) settings and saves them into the
-      * class KMyMoneySettings (a singleton).
-      *
-      * @see KListSettingsDlg
-      * Refreshes all views. Used e.g. after settings have been changed or
-      * data has been loaded from external sources (QIF import).
-      **/
-    void slotRefreshViews();
-
-    /**
-      * Called, whenever the payees view should pop up and a specific
-      * transaction in an account should be shown.
-      *
-      * @param payeeId The ID of the payee to be shown
-      * @param accountId The ID of the account to be shown
-      * @param transactionId The ID of the transaction to be selected
-      */
-    void slotPayeeSelected(const QString& payeeId, const QString& accountId, const QString& transactionId);
-
-    /**
-      * Called, whenever the tags view should pop up and a specific
-      * transaction in an account should be shown.
-      *
-      * @param tagId The ID of the tag to be shown
-      * @param accountId The ID of the account to be shown
-      * @param transactionId The ID of the transaction to be selected
-      */
-    void slotTagSelected(const QString& tagId, const QString& accountId, const QString& transactionId);
-
-    /**
-      * This slot prints the current view.
-      */
-    void slotPrintView();
-
-    /**
-      * Called when the user changes the detail
-      * setting of the transaction register
-      *
-      * @param detailed if true, the register is shown with all details
-      */
-    void slotShowTransactionDetail(bool detailed);
-
-    /**
-     * Informs respective views about selected object, so they can
-     * update action states and current object.
-     * @param obj Account, Category, Investment, Stock, Institution
-     */
-    void slotObjectSelected(const MyMoneyObject& obj);
-
-    void slotSelectByObject(const MyMoneyObject& obj, eView::Intent intent);
-    void slotSelectByVariant(const QVariantList& variant, eView::Intent intent);
-    void slotCustomActionRequested(View view, eView::Action action);
-
-    void slotFileOpened();
-    void slotFileClosed();
+    void updateActions(const SelectedObjects& selections);
 
 private Q_SLOTS:
     /**
-      * This slots switches the view to the specific page
-      */
-    void slotShowHomePage();
-    void slotShowInstitutionsPage();
-    void slotShowAccountsPage();
-    void slotShowSchedulesPage();
-    void slotShowCategoriesPage();
-    void slotShowTagsPage();
-    void slotShowPayeesPage();
-    void slotShowLedgersPage();
-    void slotShowInvestmentsPage();
-    void slotShowReportsPage();
-    void slotShowBudgetPage();
-    void slotShowForecastPage();
-    void slotShowOutboxPage();
+     * This slot writes information about the page passed as argument @a current
+     * in the kmymoney.rc file so that it can be selected automatically when
+     * the application is started again.
+     *
+     * @param current KPageWidgetItem pointer to the current page item
+     * @param previous KPageWidgetItem pointer to the previous page item
+     */
+    void slotSwitchView(KPageWidgetItem* current, KPageWidgetItem* previous);
+
     void switchToDefaultView();
 
-    /**
-     * Opens object in ledgers or edits in case of institution
-     * @param obj Account, Category, Investment, Stock, Institution
-     */
-    void slotOpenObjectRequested(const MyMoneyObject& obj);
+    void slotRememberLastView(View view);
 
-    /**
-     * Opens context menu based on objects's type
-     * @param obj Account, Category, Investment, Stock, Institution
-     */
-    void slotContextMenuRequested(const MyMoneyObject& obj);
-
-private:
-
-    /**
-      * Internal method used by slotAccountNew() and slotAccountCategory().
-      */
-    void accountNew(const bool createCategory);
-
-    void resetViewSelection();
+    void switchView(QWidget* viewWidget, const QString& accountId, const QString& journalEntryId);
 
 Q_SIGNALS:
     /**
@@ -309,15 +222,11 @@ Q_SIGNALS:
      */
     void statusProgress(int cnt, int base);
 
-    void accountReconciled(const MyMoneyAccount& account, const QDate& date, const MyMoneyMoney& startingBalance, const MyMoneyMoney& endingBalance, const QList<QPair<MyMoneyTransaction, MyMoneySplit> >& transactionList);
-
-    /**
-      * This signal is emitted when a transaction/list of transactions has been selected by
-      * the GUI. If no transaction is selected or the selection is removed,
-      * @p transactions is identical to an empty QList. This signal is used
-      * by plugins to get information about changes.
-      */
-    void transactionsSelected(const KMyMoneyRegister::SelectedTransactions& transactions);
+    void accountReconciled(const MyMoneyAccount& account,
+                           const QDate& date,
+                           const MyMoneyMoney& startingBalance,
+                           const MyMoneyMoney& endingBalance,
+                           const QStringList& transactionList);
 
     /**
       * This signal is emitted when a new account has been selected by
@@ -326,6 +235,18 @@ Q_SIGNALS:
       * by plugins to get information about changes.
       */
     void accountSelected(const MyMoneyAccount& account);
+
+    // these signals request a change by the application
+    void requestSelectionChange (const SelectedObjects& selection);
+    void requestCustomContextMenu(eMenu::Menu type, const QPoint& pos);
+    void requestActionTrigger(eMenu::Action action);
+
+    void settingsChanged();
+
+    void addSharedActionButton(eMenu::Action action, QAction* defaultAction);
+    void selectSharedActionButton(eMenu::Action action, QAction* defaultAction);
+
+    void onlinePluginsChanged(QMap<QString, KMyMoneyPlugin::OnlinePlugin*>* plugins);
 };
 
 #endif
