@@ -22,10 +22,10 @@
 
 #include "kmymoneyviewbase.h"
 
-class QTreeWidgetItem;
-class KTreeWidgetSearchLineWidget;
+class QItemSelection;
 class MyMoneySchedule;
 class MyMoneyAccount;
+class SelectedObjects;
 
 namespace eDialogs {
 enum class ScheduleResultCode;
@@ -63,24 +63,19 @@ public:
       */
     ~KScheduledView() override;
 
-    void executeCustomAction(eView::Action action) override;
-    void refresh();
-    void updateActions(const MyMoneyObject& obj);
-
     // TODO: remove that function
     /**
      * ugly proxy function
      */
     eDialogs::ScheduleResultCode enterSchedule(MyMoneySchedule& schedule, bool autoEnter, bool extendedKeys);
 
+    void slotSettingsChanged() override;
+
 public Q_SLOTS:
-    void slotSelectSchedule(const QString& schedule);
-    void slotShowScheduleMenu(const MyMoneySchedule& sch);
     void slotEditSchedule();
+    void slotEditLoanSchedule();
 
-    void slotEnterOverdueSchedules(const MyMoneyAccount& acc);
-
-    void slotSelectByObject(const MyMoneyObject& obj, eView::Intent intent) override;
+    void updateActions(const SelectedObjects& selections) override;
 
 Q_SIGNALS:
     void enterOverdueSchedulesFinished(eView::Schedules::Requester req);
@@ -92,20 +87,10 @@ private:
     Q_DECLARE_PRIVATE(KScheduledView)
 
 private Q_SLOTS:
+    void slotListViewCollapsed(const QModelIndex& idx);
+    void slotListViewExpanded(const QModelIndex& idx);
 
-    void customContextMenuRequested(const QPoint);
-    void slotListItemExecuted(QTreeWidgetItem*, int);
-
-    void slotAccountActivated();
-
-    void slotListViewCollapsed(QTreeWidgetItem* item);
-    void slotListViewExpanded(QTreeWidgetItem* item);
-
-    void slotTimerDone();
-
-    void slotSetSelectedItem();
-
-    void slotRearrange();
+    void slotSetSelectedItem(const QItemSelection& selected, const QItemSelection& deselected);
 
     void slotNewSchedule();
     void slotDeleteSchedule();
