@@ -1,5 +1,5 @@
 /*
-    SPDX-FileCopyrightText: 2016 Thomas Baumgart <tbaumgart@kde.org>
+    SPDX-FileCopyrightText: 2016-2019 Thomas Baumgart <tbaumgart@kde.org>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
@@ -17,6 +17,8 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
+class MyMoneySecurity;
+
 class SplitDelegate : public QStyledItemDelegate
 {
     Q_OBJECT
@@ -41,6 +43,10 @@ public:
     void setShowValuesInverted(bool inverse);
     bool showValuesInverted();
 
+    void setCommodity(const MyMoneySecurity& commodity);
+    void setTransactionPayeeId(const QString& id);
+    void setReadOnlyMode(bool readOnly);
+
     static void setErroneousColor(const QColor& color);
     static void setImportedColor(const QColor& color);
 
@@ -52,9 +58,6 @@ protected:
 protected Q_SLOTS:
     void endEdit();
 
-Q_SIGNALS:
-    void sizeHintChanged(const QModelIndex&) const;
-
 private:
     class Private;
     Private * const d;
