@@ -123,8 +123,8 @@ void KMandatoryFieldGroup::add(QWidget *widget)
 
         else if (qobject_cast<IonlineJobEdit*>(widget)) {
             connect(qobject_cast<IonlineJobEdit*>(widget),
-                    SIGNAL(validityChanged(bool)),
-                    this, SLOT(changed()));
+                    &IonlineJobEdit::validityChanged,
+                    this, &KMandatoryFieldGroup::changed);
 
             // Do not set palette for IonlineJobEdits as they contain subwidgets
             d->m_widgets.append(widget);
@@ -183,8 +183,8 @@ void KMandatoryFieldGroup::changed()
 {
     Q_D(KMandatoryFieldGroup);
     bool enable = d->m_externalMandatoryState;
-    QList<QWidget *>::ConstIterator i;
-    for (i = d->m_widgets.constBegin(); i != d->m_widgets.constEnd(); ++i) {
+    QList<QWidget*>::const_iterator i;
+    for (i = d->m_widgets.cbegin(); i != d->m_widgets.cend(); ++i) {
         QWidget *widget = *i;
         // disabled widgets don't count
         if (!(widget->isEnabled())) {
@@ -261,8 +261,8 @@ void KMandatoryFieldGroup::changed()
         d->m_okButton->setEnabled(enable);
     d->m_enabled = enable;
 
-    emit stateChanged();
-    emit stateChanged(enable);
+    Q_EMIT stateChanged();
+    Q_EMIT stateChanged(enable);
 }
 
 bool KMandatoryFieldGroup::isEnabled() const
