@@ -10,11 +10,11 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
+#include <QIcon>
 #include <QList>
-#include <QVBoxLayout>
 #include <QPixmapCache>
 #include <QPushButton>
-#include <QIcon>
+#include <QVBoxLayout>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
@@ -24,11 +24,11 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include "mymoneyfile.h"
-#include "mymoneyaccount.h"
-#include "icons/icons.h"
-#include "mymoneyenums.h"
 #include "dialogenums.h"
+#include "icons.h"
+#include "mymoneyaccount.h"
+#include "mymoneyenums.h"
+#include "mymoneyfile.h"
 #include "widgetenums.h"
 
 using namespace Icons;
@@ -129,7 +129,7 @@ void KMyMoneyAccountSelector::selectCategories(const bool income, const bool exp
         else if ((*it_v)->text(0) == i18n("Expense categories"))
             selectAllSubItems(*it_v, expense);
     }
-    emit stateChanged();
+    Q_EMIT stateChanged();
 }
 
 void KMyMoneyAccountSelector::slotSelectIncomeCategories()
@@ -171,11 +171,12 @@ QStringList KMyMoneyAccountSelector::accountList() const
     return accountList(QList<eMyMoney::Account::Type>());
 }
 
-bool KMyMoneyAccountSelector::match(const QRegExp& exp, QTreeWidgetItem* item) const
+bool KMyMoneyAccountSelector::match(const QRegularExpression& exp, QTreeWidgetItem* item) const
 {
     if (!item->flags().testFlag(Qt::ItemIsSelectable))
         return false;
-    return exp.indexIn(item->data(0, (int)eWidgets::Selector::Role::Key).toString().mid(1)) != -1;
+    const auto matchingItem(exp.match(item->data(0, static_cast<int>(eWidgets::Selector::Role::Key)).toString().mid(1)));
+    return matchingItem.hasMatch();
 }
 
 bool KMyMoneyAccountSelector::contains(const QString& txt) const
@@ -192,8 +193,9 @@ bool KMyMoneyAccountSelector::contains(const QString& txt) const
                        i18n("Security");
 
     while ((it_v = *it) != 0) {
-        QRegExp exp(QString("^(?:%1):%2$").arg(baseName).arg(QRegExp::escape(txt)));
-        if (exp.indexIn(it_v->data(0, (int)eWidgets::Selector::Role::Key).toString().mid(1)) != -1) {
+        const QRegularExpression exp(QString("^(?:%1):%2$").arg(baseName).arg(QRegularExpression::escape(txt)));
+        const auto matchingItem(exp.match(it_v->data(0, static_cast<int>(eWidgets::Selector::Role::Key)).toString().mid(1)));
+        if (matchingItem.hasMatch()) {
             return true;
         }
         it++;
@@ -201,6 +203,13 @@ bool KMyMoneyAccountSelector::contains(const QString& txt) const
     return false;
 }
 
+QSize KMyMoneyAccountSelector::sizeHint() const
+{
+    auto size = KMyMoneySelector::sizeHint();
+    size.setWidth(400);
+    return size;
+}
+
 class AccountSetPrivate
 {
     Q_DISABLE_COPY(AccountSetPrivate)
@@ -296,7 +305,7 @@ int AccountSet::load(KMyMoneyAccountSelector* selector)
 {
     Q_D(AccountSet);
     QStringList list;
-    QStringList::ConstIterator it_l;
+    QStringList::const_iterator it_l;
     int count = 0;
     int typeMask = 0;
     QString currentId;
@@ -346,7 +355,7 @@ int AccountSet::load(KMyMoneyAccountSelector* selector)
 
     //get the account icon from cache or insert it if it is not there
     QPixmap accountPixmap;
-    if (!QPixmapCache::find("account", accountPixmap)) {
+    if (!QPixmapCache::find("account", &accountPixmap)) {
         QIcon icon = Icons::get(Icon::BankAccount);
         if (!icon.availableSizes().isEmpty())
             accountPixmap = icon.pixmap(icon.availableSizes().first());
@@ -360,7 +369,7 @@ int AccountSet::load(KMyMoneyAccountSelector* selector)
             ++d->m_count;
             key = QString("B%1").arg(i18n("Asset"));
             item = selector->newItem(i18n("Asset accounts"), key);
-            item->setIcon(0, d->m_file->asset().accountPixmap());
+            item->setIcon(0, d->m_file->asset().accountIcon());
             list = d->m_file->asset().accountList();
         }
 
@@ -368,7 +377,7 @@ int AccountSet::load(KMyMoneyAccountSelector* selector)
             ++d->m_count;
             key = QString("C%1").arg(i18n("Liability"));
             item = selector->newItem(i18n("Liability accounts"), key);
-            item->setIcon(0, d->m_file->liability().accountPixmap());
+            item->setIcon(0, d->m_file->liability().accountIcon());
             list = d->m_file->liability().accountList();
         }
 
@@ -376,7 +385,7 @@ int AccountSet::load(KMyMoneyAccountSelector* selector)
             ++d->m_count;
             key = QString("D%1").arg(i18n("Income"));
             item = selector->newItem(i18n("Income categories"), key);
-            item->setIcon(0, d->m_file->income().accountPixmap());
+            item->setIcon(0, d->m_file->income().accountIcon());
             list = d->m_file->income().accountList();
             if (selector->selectionMode() == QTreeWidget::MultiSelection) {
                 selector->d_func()->m_incomeCategoriesButton->show();
@@ -387,7 +396,7 @@ int AccountSet::load(KMyMoneyAccountSelector* selector)
             ++d->m_count;
             key = QString("E%1").arg(i18n("Expense"));
             item = selector->newItem(i18n("Expense categories"), key);
-            item->setIcon(0, d->m_file->expense().accountPixmap());
+            item->setIcon(0, d->m_file->expense().accountIcon());
             list = d->m_file->expense().accountList();
             if (selector->selectionMode() == QTreeWidget::MultiSelection) {
                 selector->d_func()->m_expenseCategoriesButton->show();
@@ -398,7 +407,7 @@ int AccountSet::load(KMyMoneyAccountSelector* selector)
             ++d->m_count;
             key = QString("F%1").arg(i18n("Equity"));
             item = selector->newItem(i18n("Equity accounts"), key);
-            item->setIcon(0, d->m_file->equity().accountPixmap());
+            item->setIcon(0, d->m_file->equity().accountIcon());
             list = d->m_file->equity().accountList();
         }
 
@@ -407,7 +416,7 @@ int AccountSet::load(KMyMoneyAccountSelector* selector)
 
         if (item != 0) {
             // scan all matching accounts found in the engine
-            for (it_l = list.constBegin(); it_l != list.constEnd(); ++it_l) {
+            for (it_l = list.cbegin(); it_l != list.cend(); ++it_l) {
                 const MyMoneyAccount& acc = d->m_file->account(*it_l);
                 ++d->m_count;
                 ++count;
@@ -418,10 +427,9 @@ int AccountSet::load(KMyMoneyAccountSelector* selector)
                     QString tmpKey;
                     tmpKey = key + MyMoneyFile::AccountSeparator + acc.name();
                     QTreeWidgetItem* subItem = selector->newItem(item, acc.name(), tmpKey, acc.id());
-                    subItem->setIcon(0, acc.accountPixmap());
-                    if (acc.value("PreferredAccount") == "Yes"
-                            && d->m_typeList.contains(acc.accountType())) {
-                        selector->newItem(d->m_favorites, acc.name(), tmpKey, acc.id())->setIcon(0, acc.accountPixmap());;
+                    subItem->setIcon(0, acc.accountIcon());
+                    if (acc.value("PreferredAccount", false) && d->m_typeList.contains(acc.accountType())) {
+                        selector->newItem(d->m_favorites, acc.name(), tmpKey, acc.id())->setIcon(0, acc.accountIcon());
                     }
                     if (acc.accountList().count() > 0) {
                         subItem->setExpanded(true);
@@ -475,15 +483,15 @@ int AccountSet::load(KMyMoneyAccountSelector* selector, const QString& baseName,
     item = selector->newItem(baseName);
     ++d->m_count;
 
-    QList<QString>::ConstIterator it;
-    for (it = accountIdList.constBegin(); it != accountIdList.constEnd(); ++it)   {
+    QList<QString>::const_iterator it;
+    for (it = accountIdList.cbegin(); it != accountIdList.cend(); ++it) {
         const MyMoneyAccount& acc = d->m_file->account(*it);
         if (acc.isClosed())
             continue;
         QString tmpKey;
         // the first character must be preset. Since we don't know any sort order here, we just use A
         tmpKey = QString("A%1%2%3").arg(baseName, MyMoneyFile::AccountSeparator, acc.name());
-        selector->newItem(item, acc.name(), tmpKey, acc.id())->setIcon(0, acc.accountPixmap());
+        selector->newItem(item, acc.name(), tmpKey, acc.id())->setIcon(0, acc.accountIcon());
         ++d->m_count;
         ++count;
     }
@@ -518,10 +526,10 @@ bool AccountSet::isHidingClosedAccounts() const
 int AccountSet::loadSubAccounts(KMyMoneyAccountSelector* selector, QTreeWidgetItem* parent, const QString& key, const QStringList& list)
 {
     Q_D(AccountSet);
-    QStringList::ConstIterator it_l;
+    QStringList::const_iterator it_l;
     int count = 0;
 
-    for (it_l = list.constBegin(); it_l != list.constEnd(); ++it_l) {
+    for (it_l = list.cbegin(); it_l != list.cend(); ++it_l) {
         const MyMoneyAccount& acc = d->m_file->account(*it_l);
         // don't include stock accounts if not in expert mode
         if (acc.isInvest() && !d->m_showInvestments)
@@ -536,10 +544,9 @@ int AccountSet::loadSubAccounts(KMyMoneyAccountSelector* selector, QTreeWidgetIt
             ++count;
             ++d->m_count;
             QTreeWidgetItem* item = selector->newItem(parent, acc.name(), tmpKey, acc.id());
-            item->setIcon(0, acc.accountPixmap());
-            if (acc.value("PreferredAccount") == "Yes"
-                    && d->m_typeList.contains(acc.accountType())) {
-                selector->newItem(d->m_favorites, acc.name(), tmpKey, acc.id())->setIcon(0, acc.accountPixmap());
+            item->setIcon(0, acc.accountIcon());
+            if (acc.value("PreferredAccount", false) && d->m_typeList.contains(acc.accountType())) {
+                selector->newItem(d->m_favorites, acc.name(), tmpKey, acc.id())->setIcon(0, acc.accountIcon());
             }
             if (acc.accountList().count() > 0) {
                 item->setExpanded(true);
@@ -562,7 +569,7 @@ bool AccountSet::includeAccount(const MyMoneyAccount& acc)
     if (d->m_typeList.contains(acc.accountType()))
         return true;
 
-    foreach (const auto sAccount, acc.accountList())
+    for (const auto& sAccount : acc.accountList())
         if (includeAccount(d->m_file->account(sAccount)))
             return true;
 
