@@ -11,7 +11,6 @@
 // QT Includes
 
 #include <QPalette>
-#include <QTimer>
 #include <QHBoxLayout>
 #include <QFrame>
 #include <QPushButton>
@@ -153,7 +152,7 @@ void KMyMoneyCategory::slotItemSelected(const QString& id)
 
     if (d->m_id != id) {
         d->m_id = id;
-        emit itemSelected(id);
+        Q_EMIT itemSelected(id);
     }
 }
 
@@ -173,7 +172,7 @@ void KMyMoneyCategory::focusInEvent(QFocusEvent *ev)
     // make sure, we get a clean state before we automagically move the focus to
     // some other widget (like for 'split transaction'). We do this by delaying
     // the emission of the focusIn signal until the next run of the event loop.
-    QTimer::singleShot(0, this, SIGNAL(focusIn()));
+    QMetaObject::invokeMethod(this, "focusIn", Qt::QueuedConnection);
 }
 
 void KMyMoneyCategory::setSplitTransaction()
