@@ -8,6 +8,8 @@
 #include "kmymoneycombo.h"
 #include "kmymoneycombo_p.h"
 
+#include "config-kmymoney.h"
+
 // ----------------------------------------------------------------------------
 // QT Includes
 
@@ -93,7 +95,7 @@ void KMyMoneyCombo::slotItemSelected(const QString& id)
 
     if (d->m_id != id) {
         d->m_id = id;
-        emit itemSelected(id);
+        Q_EMIT itemSelected(id);
     }
 }
 
@@ -161,7 +163,8 @@ void KMyMoneyCombo::mousePressEvent(QMouseEvent *e)
     if (e->button() != Qt::LeftButton)
         return;
 
-    if (((!isEditable() || isInArrowArea(e->globalPos())) && selector()->itemList().count()) && !d->m_completion->isVisible()) {
+    if (((!isEditable() || isInArrowArea(QT6_IF(e->globalPosition(), e->screenPos()).toPoint())) && selector()->itemList().count())
+        && !d->m_completion->isVisible()) {
         d->m_completion->setVisible(true);
     }
 
@@ -269,12 +272,12 @@ void KMyMoneyCombo::focusOutEvent(QFocusEvent* e)
                 QString id;
                 // announce that we go into a possible dialog to create an object
                 // This can be used by upstream widgets to disable filters etc.
-                emit objectCreation(true);
+                Q_EMIT objectCreation(true);
 
-                emit createItem(currentText(), id);
+                Q_EMIT createItem(currentText(), id);
 
                 // Announce that we return from object creation
-                emit objectCreation(false);
+                Q_EMIT objectCreation(false);
 
                 // update the field to a possibly created object
                 d->m_id = id;
@@ -298,7 +301,7 @@ void KMyMoneyCombo::focusOutEvent(QFocusEvent* e)
         QString id = d->m_id;
         d->m_id.clear();
         if (!id.isEmpty())
-            emit itemSelected(d->m_id);
+            Q_EMIT itemSelected(d->m_id);
         update();
     }
     d->m_inFocusOutEvent = false;
@@ -355,7 +358,7 @@ QSize KMyMoneyCombo::sizeHint() const
     int i, w;
     QFontMetrics fm = fontMetrics();
 
-    int maxW = count() ? 18 : 7 * fm.width(QChar('x')) + 18;
+    int maxW = count() ? 18 : 7 * fm.horizontalAdvance(QChar('x')) + 18;
     int maxH = qMax(fm.lineSpacing(), 14) + 2;
 
     w = selector()->optimizedWidth();
