@@ -9,6 +9,8 @@
 #ifndef KMYMONEYRECONCILECOMBO_H
 #define KMYMONEYRECONCILECOMBO_H
 
+#include "kmm_base_widgets_export.h"
+
 // ----------------------------------------------------------------------------
 // QT Includes
 
@@ -32,7 +34,7 @@ enum class State;
   * reconciliation.
   */
 
-class KMM_WIDGETS_EXPORT KMyMoneyReconcileCombo : public KMyMoneyMVCCombo
+class KMM_BASE_WIDGETS_EXPORT KMyMoneyReconcileCombo : public KMyMoneyMVCCombo
 {
     Q_OBJECT
     Q_DISABLE_COPY(KMyMoneyReconcileCombo)
