@@ -9,9 +9,9 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
-#include <QStyle>
-#include <QRegExp>
 #include <QApplication>
+#include <QRegularExpression>
+#include <QStyle>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
@@ -20,7 +20,6 @@
 // Project Includes
 
 #include "kmymoneysettings.h"
-#include "widgetenums.h"
 
 using namespace eWidgets;
 
@@ -104,7 +103,7 @@ void KMyMoneySelector::slotItemSelected(QTreeWidgetItem *item)
     Q_D(KMyMoneySelector);
     if (d->m_selMode == QTreeWidget::SingleSelection) {
         if (item && item->flags().testFlag(Qt::ItemIsSelectable)) {
-            emit itemSelected(item->data(0, (int)Selector::Role::Id).toString());
+            Q_EMIT itemSelected(item->data(0, (int)Selector::Role::Id).toString());
         }
     }
 }
@@ -159,17 +158,7 @@ QTreeWidgetItem* KMyMoneySelector::newTopItem(const QString& name, const QString
 {
     Q_D(KMyMoneySelector);
     QTreeWidgetItem* item = new QTreeWidgetItem(d->m_treeWidget);
-
-    item->setText(0, name);
-    item->setData(0, (int)Selector::Role::Key, key);
-    item->setData(0, (int)Selector::Role::Id, id);
-    item->setText(1, key); // hidden, but used for sorting
-    item->setFlags(item->flags() & ~Qt::ItemIsUserCheckable);
-
-    if (d->m_selMode == QTreeWidget::MultiSelection) {
-        item->setFlags(item->flags() | Qt::ItemIsUserCheckable);
-        item->setCheckState(0, Qt::Checked);
-    }
+    d->setupItemData(item, name, key, id);
     return item;
 }
 
@@ -177,17 +166,7 @@ QTreeWidgetItem* KMyMoneySelector::newItem(QTreeWidgetItem* parent, const QStrin
 {
     Q_D(KMyMoneySelector);
     QTreeWidgetItem* item = new QTreeWidgetItem(parent);
-
-    item->setText(0, name);
-    item->setData(0, (int)Selector::Role::Key, key);
-    item->setData(0, (int)Selector::Role::Id, id);
-    item->setText(1, key); // hidden, but used for sorting
-    item->setFlags(item->flags() & ~Qt::ItemIsUserCheckable);
-
-    if (d->m_selMode == QTreeWidget::MultiSelection) {
-        item->setFlags(item->flags() | Qt::ItemIsUserCheckable);
-        item->setCheckState(0, Qt::Checked);
-    }
+    d->setupItemData(item, name, key, id);
     return item;
 }
 
@@ -285,14 +264,14 @@ void KMyMoneySelector::selectAllItems(const bool state)
 {
     Q_D(KMyMoneySelector);
     selectAllSubItems(d->m_treeWidget->invisibleRootItem(), state);
-    emit stateChanged();
+    Q_EMIT stateChanged();
 }
 
 void KMyMoneySelector::selectItems(const QStringList& itemList, const bool state)
 {
     Q_D(KMyMoneySelector);
     selectSubItems(d->m_treeWidget->invisibleRootItem(), itemList, state);
-    emit stateChanged();
+    Q_EMIT stateChanged();
 }
 
 void KMyMoneySelector::selectSubItems(QTreeWidgetItem* item, const QStringList& itemList, const bool state)
@@ -304,7 +283,7 @@ void KMyMoneySelector::selectSubItems(QTreeWidgetItem* item, const QStringList&
         }
         selectSubItems(child, itemList, state);
     }
-    emit stateChanged();
+    Q_EMIT stateChanged();
 }
 
 void KMyMoneySelector::selectAllSubItems(QTreeWidgetItem* item, const bool state)
@@ -316,7 +295,7 @@ void KMyMoneySelector::selectAllSubItems(QTreeWidgetItem* item, const bool state
         }
         selectAllSubItems(child, state);
     }
-    emit stateChanged();
+    Q_EMIT stateChanged();
 }
 
 void KMyMoneySelector::selectedItems(QStringList& list) const
@@ -396,30 +375,29 @@ QTreeWidget* KMyMoneySelector::listView() const
 
 int KMyMoneySelector::slotMakeCompletion(const QString& _txt)
 {
-    QString txt(QRegExp::escape(_txt));
+    QString txt(QRegularExpression::escape(_txt));
     if (KMyMoneySettings::stringMatchFromStart() && QLatin1String(this->metaObject()->className()) == QLatin1String("KMyMoneySelector"))
         txt.prepend('^');
-    return slotMakeCompletion(QRegExp(txt, Qt::CaseInsensitive));
+    return slotMakeCompletion(QRegularExpression(txt, QRegularExpression::CaseInsensitiveOption));
 }
 
-bool KMyMoneySelector::match(const QRegExp& exp, QTreeWidgetItem* item) const
+bool KMyMoneySelector::match(const QRegularExpression& exp, QTreeWidgetItem* item) const
 {
-    return exp.indexIn(item->text(0)) != -1;
+    return exp.match(item->text(0)).hasMatch();
 }
 
-int KMyMoneySelector::slotMakeCompletion(const QRegExp& _exp)
+int KMyMoneySelector::slotMakeCompletion(const QRegularExpression& _exp)
 {
     Q_D(KMyMoneySelector);
     auto exp(_exp);
-    QString pattern = exp.pattern();
-    if (exp.patternSyntax() == QRegExp::RegExp) {
-        auto replacement = QStringLiteral(".*:");
-        if (!KMyMoneySettings::stringMatchFromStart() || QLatin1String(this->metaObject()->className()) != QLatin1String("KMyMoneySelector")) {
-            replacement.append(QLatin1String(".*"));
-        }
-        pattern.replace(QLatin1String(":"), replacement);
-        exp.setPattern(pattern);
+    auto pattern = exp.pattern();
+    auto replacement = QStringLiteral(".*:");
+    if (!KMyMoneySettings::stringMatchFromStart() || QLatin1String(this->metaObject()->className()) != QLatin1String("KMyMoneySelector")) {
+        replacement.append(QLatin1String(".*"));
     }
+    pattern.replace(QLatin1String(":"), replacement);
+    exp.setPattern(pattern);
+
     QTreeWidgetItemIterator it(d->m_treeWidget, QTreeWidgetItemIterator::Selectable);
 
     QTreeWidgetItem* it_v;
@@ -514,7 +492,7 @@ int KMyMoneySelector::slotMakeCompletion(const QRegExp& _exp)
     auto cnt = 0;
 
     it = QTreeWidgetItemIterator(d->m_treeWidget, QTreeWidgetItemIterator::Selectable | QTreeWidgetItemIterator::NotHidden);
-    while ((it_v = *it) != 0) {
+    while ((*it) != 0) {
         cnt++;
         it++;
     }
@@ -544,7 +522,7 @@ void KMyMoneySelector::slotItemPressed(QTreeWidgetItem* item, int /* col */)
     if (item->flags().testFlag(Qt::ItemIsUserCheckable)) {
         QStyleOptionButton opt;
         opt.rect = d->m_treeWidget->visualItemRect(item);
-        QRect rect = d->m_treeWidget->style()->subElementRect(QStyle::SE_ViewItemCheckIndicator, &opt, d->m_treeWidget);
+        QRect rect = d->m_treeWidget->style()->subElementRect(QStyle::SE_ItemViewItemCheckIndicator, &opt, d->m_treeWidget);
         if (rect.contains(d->m_treeWidget->mapFromGlobal(QCursor::pos()))) {
             // we get down here, if we have a right click onto the checkbox
             item->setCheckState(0, item->checkState(0) == Qt::Checked ? Qt::Unchecked : Qt::Checked);
