@@ -19,15 +19,17 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include "ui_ktransactionfilter.h"
-#include "mymoneyreport.h"
-#include "mymoneysplit.h"
+#include "daterangedlg.h"
+#include "kmymoneysettings.h"
 #include "mymoneyexception.h"
 #include "mymoneyfile.h"
+#include "mymoneyreport.h"
+#include "mymoneysplit.h"
 #include "mymoneytransaction.h"
 #include "mymoneytransactionfilter.h"
-#include "kmymoneysettings.h"
-#include "daterangedlg.h"
+#include "mymoneyutils.h"
+
+#include "ui_ktransactionfilter.h"
 
 KTransactionFilter::KTransactionFilter(QWidget *parent, bool withEquityAccounts, bool withInvestments, bool withDataTab) :
     QWidget(parent),
@@ -35,6 +37,7 @@ KTransactionFilter::KTransactionFilter(QWidget *parent, bool withEquityAccounts,
 {
     Q_D(KTransactionFilter);
     d->init(withEquityAccounts, withInvestments, withDataTab);
+    QMetaObject::invokeMethod(d->ui->m_textEdit, "setFocus", Qt::QueuedConnection);
 }
 
 KTransactionFilter::~KTransactionFilter()
@@ -49,16 +52,9 @@ void KTransactionFilter::slotReset()
     d->ui->m_textEdit->setText(QString());
     d->ui->m_regExp->setChecked(false);
     d->ui->m_caseSensitive->setChecked(false);
-    d->ui->m_textNegate->setCurrentIndex(0);
+    d->ui->m_textNegate->setCurrentItem(0);
 
-    d->ui->m_amountEdit->setEnabled(true);
-    d->ui->m_amountFromEdit->setEnabled(false);
-    d->ui->m_amountToEdit->setEnabled(false);
-    d->ui->m_amountEdit->setText(QString());
-    d->ui->m_amountFromEdit->setText(QString());
-    d->ui->m_amountToEdit->setText(QString());
-    d->ui->m_amountButton->setChecked(true);
-    d->ui->m_amountRangeButton->setChecked(false);
+    d->clearAmountWidgets();
 
     d->ui->m_emptyPayeesButton->setChecked(false);
     d->selectAllItems(d->ui->m_payeesView, true);
@@ -70,14 +66,7 @@ void KTransactionFilter::slotReset()
     d->ui->m_stateBox->setCurrentIndex((int)eMyMoney::TransactionFilter::State::All);
     d->ui->m_validityBox->setCurrentIndex((int)eMyMoney::TransactionFilter::Validity::Any);
 
-    d->ui->m_nrEdit->setEnabled(true);
-    d->ui->m_nrFromEdit->setEnabled(false);
-    d->ui->m_nrToEdit->setEnabled(false);
-    d->ui->m_nrEdit->setText(QString());
-    d->ui->m_nrFromEdit->setText(QString());
-    d->ui->m_nrToEdit->setText(QString());
-    d->ui->m_nrButton->setChecked(true);
-    d->ui->m_nrRangeButton->setChecked(false);
+    d->clearNumberWidgets();
 
     // the following call implies a call to slotUpdateSelections,
     // that's why we call it last
@@ -99,7 +88,7 @@ void KTransactionFilter::slotUpdateSelections()
         if (!txt.isEmpty())
             txt += separator;
         txt += i18n("Text");
-        d->ui->m_regExp->setEnabled(QRegExp(d->ui->m_textEdit->text()).isValid());
+        d->ui->m_regExp->setEnabled(QRegularExpression(d->ui->m_textEdit->text()).isValid());
     } else
         d->ui->m_regExp->setEnabled(false);
 
@@ -173,7 +162,7 @@ void KTransactionFilter::slotUpdateSelections()
     }
 
     // disable the search button if no selection is made
-    emit selectionNotEmpty(!txt.isEmpty());
+    Q_EMIT selectionNotEmpty(!txt.isEmpty());
 
     if (txt.isEmpty()) {
         txt = i18nc("No selection", "(None)");
@@ -258,8 +247,14 @@ MyMoneyTransactionFilter KTransactionFilter::setupFilter()
 
     // Text tab
     if (!d->ui->m_textEdit->text().isEmpty()) {
-        QRegExp exp(d->ui->m_textEdit->text(), d->ui->m_caseSensitive->isChecked() ? Qt::CaseSensitive : Qt::CaseInsensitive, !d->ui->m_regExp->isChecked() ? QRegExp::Wildcard : QRegExp::RegExp);
-        d->m_filter.setTextFilter(exp, d->ui->m_textNegate->currentIndex() != 0);
+        QRegularExpression exp;
+        auto pattern(d->ui->m_textEdit->text());
+        if (!d->ui->m_regExp->isChecked()) {
+            pattern = MyMoneyUtils::convertWildcardToRegularExpression(pattern);
+        }
+        exp.setPattern(pattern);
+        exp.setPatternOptions(d->ui->m_caseSensitive->isChecked() ? QRegularExpression::NoPatternOption : QRegularExpression::CaseInsensitiveOption);
+        d->m_filter.setTextFilter(exp, d->ui->m_regExp->isChecked(), d->ui->m_textNegate->currentIndex() != 0);
     }
 
     // Account tab
@@ -273,10 +268,10 @@ MyMoneyTransactionFilter KTransactionFilter::setupFilter()
         // account are also selected
         if (!KMyMoneySettings::expertMode()) {
             QStringList missing;
-            foreach (const auto selection, list) {
-                auto acc = MyMoneyFile::instance()->account(selection);
+            for (const auto& selection : qAsConst(list)) {
+                const auto acc = MyMoneyFile::instance()->account(selection);
                 if (acc.accountType() == eMyMoney::Account::Type::Investment) {
-                    foreach (const auto sAccount, acc.accountList()) {
+                    for (const auto& sAccount : acc.accountList()) {
                         if (!list.contains(sAccount)) {
                             missing.append(sAccount);
                         }
@@ -358,11 +353,16 @@ void KTransactionFilter::resetFilter(MyMoneyReport& rep)
     // Text Filter
     //
 
-    QRegExp textfilter;
-    if (rep.textFilter(textfilter)) {
-        d->ui->m_textEdit->setText(textfilter.pattern());
-        d->ui->m_caseSensitive->setChecked(Qt::CaseSensitive == textfilter.caseSensitivity());
-        d->ui->m_regExp->setChecked(QRegExp::RegExp == textfilter.patternSyntax());
+    QRegularExpression textfilter;
+    bool isRegExp(false);
+    if (rep.textFilter(textfilter, isRegExp)) {
+        auto pattern(textfilter.pattern());
+        if (!isRegExp) {
+            pattern = MyMoneyUtils::convertRegularExpressionToWildcard(pattern);
+        }
+        d->ui->m_textEdit->setText(pattern);
+        d->ui->m_caseSensitive->setChecked(!(textfilter.patternOptions() & QRegularExpression::CaseInsensitiveOption));
+        d->ui->m_regExp->setChecked(isRegExp);
         d->ui->m_textNegate->setCurrentIndex(rep.isInvertingText());
     }
 
@@ -408,14 +408,7 @@ void KTransactionFilter::resetFilter(MyMoneyReport& rep)
             d->ui->m_nrRangeButton->setChecked(true);
         }
     } else {
-        d->ui->m_nrEdit->setEnabled(true);
-        d->ui->m_nrFromEdit->setEnabled(false);
-        d->ui->m_nrToEdit->setEnabled(false);
-        d->ui->m_nrEdit->setText(QString());
-        d->ui->m_nrFromEdit->setText(QString());
-        d->ui->m_nrToEdit->setText(QString());
-        d->ui->m_nrButton->setChecked(true);
-        d->ui->m_nrRangeButton->setChecked(false);
+        d->clearNumberWidgets();
     }
 
     //
@@ -444,14 +437,7 @@ void KTransactionFilter::resetFilter(MyMoneyReport& rep)
             d->ui->m_amountRangeButton->setChecked(true);
         }
     } else {
-        d->ui->m_amountEdit->setEnabled(true);
-        d->ui->m_amountFromEdit->setEnabled(false);
-        d->ui->m_amountToEdit->setEnabled(false);
-        d->ui->m_amountEdit->setText(QString());
-        d->ui->m_amountFromEdit->setText(QString());
-        d->ui->m_amountToEdit->setText(QString());
-        d->ui->m_amountButton->setChecked(true);
-        d->ui->m_amountRangeButton->setChecked(false);
+        d->clearAmountWidgets();
     }
 
     //
@@ -497,7 +483,7 @@ void KTransactionFilter::resetFilter(MyMoneyReport& rep)
             // ... we need to turn them on again in case our own
             // configuration references a closed account
             const MyMoneyFile* file = MyMoneyFile::instance();
-            foreach(const auto accId, accounts) {
+            for (const auto& accId : qAsConst(accounts)) {
                 try {
                     if (file->account(accId).isClosed()) {
                         d->accountSet.setHideClosedAccounts(false);
@@ -517,6 +503,15 @@ void KTransactionFilter::resetFilter(MyMoneyReport& rep)
     // Categories Filter
     //
 
+    d->ui->m_categoriesView->setEnabled(true);
+    d->ui->m_categoriesHint->show();
+
+    if (rep.isNetWortReport()) {
+        rep.clearCategoryFilter();
+        d->ui->m_categoriesView->setDisabled(true);
+        d->ui->m_categoriesHint->show();
+        d->ui->m_categoriesHint->setText(i18nc("@info Info why selection is disabled", "Category selection is not possible for net worth reports."));
+    }
     if (rep.categories(accounts)) {
         d->ui->m_categoriesView->selectAllItems(false);
         d->ui->m_categoriesView->selectItems(accounts, true);
