@@ -7,7 +7,7 @@
 #define BICVALIDATOR_H
 
 #include <QValidator>
-#include "kmm_widgets_export.h"
+#include "kmm_base_widgets_export.h"
 #include "kmymoneyvalidationfeedback.h"
 
 namespace eWidgets {
@@ -16,7 +16,7 @@ enum class MessageType;
 }
 }
 
-class KMM_WIDGETS_EXPORT bicValidator : public QValidator
+class KMM_BASE_WIDGETS_EXPORT bicValidator : public QValidator
 {
     Q_OBJECT
 
