@@ -6,7 +6,7 @@
 #ifndef IBANVALIDATOR_H
 #define IBANVALIDATOR_H
 
-#include "kmm_widgets_export.h"
+#include "kmm_base_widgets_export.h"
 
 #include <QValidator>
 
@@ -18,15 +18,14 @@ enum class MessageType;
 }
 }
 
-class KMM_WIDGETS_EXPORT ibanValidator : public QValidator
+class KMM_BASE_WIDGETS_EXPORT ibanValidator : public QValidator
 {
     Q_OBJECT
 
 public:
     explicit ibanValidator(QObject* parent = 0);
-    State validate(QString&, int&) const final override;
-    State validate(const QString&) const;
-    void fixup(QString&) const final override;
+    State validate(QString& input, int& pos) const final override;
+    void fixup(QString& input) const final override;
 
     static QPair<eWidgets::ValidationFeedback::MessageType, QString> validateWithMessage(const QString&);
 };
