@@ -1,5 +1,5 @@
 /*
-    SPDX-FileCopyrightText: 2007 Thomas Baumgart <Thomas Baumgart <ipwizard@users.sourceforge.net>>
+    SPDX-FileCopyrightText: 2007 Thomas Baumgart <ipwizard@users.sourceforge.net>
     SPDX-FileCopyrightText: 2017 Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
     SPDX-License-Identifier: GPL-2.0-or-later
 */
@@ -26,28 +26,26 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include "ui_kaccounttypepage.h"
-
+#include "accountsmodel.h"
+#include "kbrokeragepage.h"
+#include "kcreditcardschedulepage.h"
 #include "kequitypriceupdatedlg.h"
+#include "kgeneralloaninfopage.h"
+#include "kguiutils.h"
+#include "khierarchypage.h"
 #include "kmymoneycurrencyselector.h"
-#include "kmymoneydateinput.h"
 #include "kmymoneygeneralcombo.h"
 #include "kmymoneysettings.h"
 #include "kmymoneywizardpage.h"
 #include "knewaccountwizard.h"
 #include "knewaccountwizard_p.h"
-#include "kbrokeragepage.h"
-#include "kcreditcardschedulepage.h"
-#include "kgeneralloaninfopage.h"
-#include "khierarchypage.h"
 #include "mymoneyaccount.h"
+#include "mymoneyenums.h"
 #include "mymoneyfile.h"
 #include "mymoneymoney.h"
 #include "mymoneyprice.h"
 #include "mymoneysecurity.h"
 #include "wizardpage.h"
-#include "kguiutils.h"
-#include "mymoneyenums.h"
 
 using namespace eMyMoney;
 
@@ -60,16 +58,25 @@ AccountTypePage::AccountTypePage(Wizard* wizard) :
     Q_D(AccountTypePage);
     d->ui->setupUi(this);
     d->m_showPriceWarning = true;
-    d->ui->m_typeSelection->insertItem(i18n("Checking"), (int)Account::Type::Checkings);
-    d->ui->m_typeSelection->insertItem(i18n("Savings"), (int)Account::Type::Savings);
-    d->ui->m_typeSelection->insertItem(i18n("Credit Card"), (int)Account::Type::CreditCard);
-    d->ui->m_typeSelection->insertItem(i18n("Cash"), (int)Account::Type::Cash);
-    d->ui->m_typeSelection->insertItem(i18n("Loan"), (int)Account::Type::Loan);
-    d->ui->m_typeSelection->insertItem(i18n("Investment"), (int)Account::Type::Investment);
-    d->ui->m_typeSelection->insertItem(i18n("Asset"), (int)Account::Type::Asset);
-    d->ui->m_typeSelection->insertItem(i18n("Liability"), (int)Account::Type::Liability);
+    d->ui->m_typeSelection->insertItem(i18n("Checking"), (int)Account::Type::Checkings, 0);
+    d->ui->m_typeSelection->setItemIcon(0, Icons::get(Icon::Checking));
+    d->ui->m_typeSelection->insertItem(i18n("Savings"), (int)Account::Type::Savings, 1);
+    d->ui->m_typeSelection->setItemIcon(1, Icons::get(Icon::Savings));
+    d->ui->m_typeSelection->insertItem(i18n("Credit Card"), (int)Account::Type::CreditCard, 2);
+    d->ui->m_typeSelection->setItemIcon(2, Icons::get(Icon::CreditCard));
+    d->ui->m_typeSelection->insertItem(i18n("Cash"), (int)Account::Type::Cash, 3);
+    d->ui->m_typeSelection->setItemIcon(3, Icons::get(Icon::Cash));
+    d->ui->m_typeSelection->insertItem(i18n("Loan"), (int)Account::Type::Loan, 4);
+    d->ui->m_typeSelection->setItemIcon(4, Icons::get(Icon::Loan));
+    d->ui->m_typeSelection->insertItem(i18n("Investment"), (int)Account::Type::Investment, 5);
+    d->ui->m_typeSelection->setItemIcon(5, Icons::get(Icon::Investment));
+    d->ui->m_typeSelection->insertItem(i18n("Asset"), (int)Account::Type::Asset, 6);
+    d->ui->m_typeSelection->setItemIcon(6, Icons::get(Icon::Asset));
+    d->ui->m_typeSelection->insertItem(i18n("Liability"), (int)Account::Type::Liability, 7);
+    d->ui->m_typeSelection->setItemIcon(7, Icons::get(Icon::Liability));
     if (KMyMoneySettings::expertMode()) {
-        d->ui->m_typeSelection->insertItem(i18n("Equity"), (int)Account::Type::Equity);
+        d->ui->m_typeSelection->insertItem(i18n("Equity"), (int)Account::Type::Equity, 8);
+        d->ui->m_typeSelection->setItemIcon(8, Icons::get(Icon::Equity));
     }
 
     d->ui->m_typeSelection->setCurrentItem((int)Account::Type::Checkings);
@@ -84,8 +91,9 @@ AccountTypePage::AccountTypePage(Wizard* wizard) :
 
     connect(d->ui->m_typeSelection, &KMyMoneyGeneralCombo::itemSelected, this, &AccountTypePage::slotUpdateType);
     connect(d->ui->m_currencyComboBox, static_cast<void (QComboBox::*)(int)>(&QComboBox::activated), this, &AccountTypePage::slotUpdateCurrency);
+    /// @todo fix this logic with new amount edit logic (at least check)
     connect(d->ui->m_conversionRate, &AmountEdit::textChanged, this, &AccountTypePage::slotUpdateConversionRate);
-    connect(d->ui->m_conversionRate, &AmountEdit::valueChanged, this, &AccountTypePage::slotPriceWarning);
+    connect(d->ui->m_conversionRate, &AmountEdit::amountChanged, this, &AccountTypePage::slotPriceWarning);
     connect(d->ui->m_onlineQuote, &QAbstractButton::clicked, this, &AccountTypePage::slotGetOnlineQuote);
 }
 
@@ -238,7 +246,7 @@ const MyMoneySecurity& AccountTypePage::currency() const
 
 void AccountTypePage::setAccount(const MyMoneyAccount& acc)
 {
-    Q_D(const AccountTypePage);
+    Q_D(AccountTypePage);
     if (acc.accountType() != Account::Type::Unknown) {
         if (acc.accountType() == Account::Type::AssetLoan) {
             d->ui->m_typeSelection->setCurrentItem((int)Account::Type::Loan);
@@ -248,22 +256,32 @@ void AccountTypePage::setAccount(const MyMoneyAccount& acc)
     }
     d->ui->m_openingDate->setDate(acc.openingDate());
     d->ui->m_accountName->setText(acc.name());
+    d->m_parentAccountId = acc.parentAccountId();
 }
 
 MyMoneyAccount AccountTypePage::parentAccount()
 {
-    switch (accountType()) {
-    case Account::Type::CreditCard:
-    case Account::Type::Liability:
-    case Account::Type::Loan: // Can be either but we return liability here
-        return MyMoneyFile::instance()->liability();
-        break;
-    case Account::Type::Equity:
-        return MyMoneyFile::instance()->equity();
-    default:
-        break;
+    Q_D(AccountTypePage);
+    const auto file = MyMoneyFile::instance();
+    const auto acc = file->accountsModel()->itemById(d->m_parentAccountId);
+    // in case we don't have a parent account or its group does
+    // not match the selected account type, we use the corresponding
+    // standard top level account
+    if (acc.id().isEmpty() || (acc.accountGroup() != acc.accountGroup(accountType()))) {
+        switch (accountType()) {
+        case Account::Type::CreditCard:
+        case Account::Type::Liability:
+        case Account::Type::Loan: // Can be either but we return liability here
+            return file->liability();
+            break;
+        case Account::Type::Equity:
+            return file->equity();
+        default:
+            break;
+        }
+        return file->asset();
     }
-    return MyMoneyFile::instance()->asset();
+    return acc;
 }
 
 bool AccountTypePage::allowsParentAccount() const
