@@ -21,6 +21,7 @@
 
 #include "knewloanwizard.h"
 #include "mymoneyaccountloan.h"
+#include "mymoneyutils.h"
 
 EffectiveDateWizardPage::EffectiveDateWizardPage(QWidget *parent)
     : QWizardPage(parent),
@@ -30,7 +31,7 @@ EffectiveDateWizardPage::EffectiveDateWizardPage(QWidget *parent)
     // Register the fields with the QWizard and connect the
     // appropriate signals to update the "Next" button correctly
     registerField("effectiveChangeDateEdit", ui->m_effectiveChangeDateEdit, "date", SIGNAL(dateChanged(QDate)));
-    connect(ui->m_effectiveChangeDateEdit, &KMyMoneyDateInput::dateChanged, this, &QWizardPage::completeChanged);
+    connect(ui->m_effectiveChangeDateEdit, &KMyMoneyDateEdit::dateChanged, this, &QWizardPage::completeChanged);
 }
 
 EffectiveDateWizardPage::~EffectiveDateWizardPage()
@@ -40,10 +41,11 @@ EffectiveDateWizardPage::~EffectiveDateWizardPage()
 
 void EffectiveDateWizardPage::initializePage()
 {
-    ui->m_effectiveDateLabel->setText(QString("\n") + i18n(
-                                          "Please enter the date from which on the following changes will be effective. "
-                                          "The date entered must be later than the opening date of this account (%1), but must "
-                                          "not be in the future. The default will be today.", QLocale().toString(qobject_cast<KNewLoanWizard*>(wizard())->account().openingDate())));
+    ui->m_effectiveDateLabel->setText(QString("\n")
+                                      + i18n("Please enter the date from which on the following changes will be effective. "
+                                             "The date entered must be later than the opening date of this account (%1), but must "
+                                             "not be in the future. The default will be today.",
+                                             MyMoneyUtils::formatDate(qobject_cast<KNewLoanWizard*>(wizard())->account().openingDate(), QLocale::LongFormat)));
 }
 
 /**
@@ -51,6 +53,8 @@ void EffectiveDateWizardPage::initializePage()
  */
 bool EffectiveDateWizardPage::isComplete() const
 {
+    if (!ui->m_effectiveChangeDateEdit->isValid())
+        return false;
     return !(ui->m_effectiveChangeDateEdit->date() < qobject_cast<KNewLoanWizard*>(wizard())->account().openingDate()
              || ui->m_effectiveChangeDateEdit->date() > QDate::currentDate());
     return true;
