@@ -38,17 +38,18 @@
 #include "ui_editselectionwizardpage.h"
 #include "ui_finalpaymentwizardpage.h"
 
+#include "kmmyesno.h"
+#include "kmymoneylineedit.h"
 #include "knewloanwizard.h"
 #include "knewloanwizard_p.h"
-#include "kmymoneylineedit.h"
-#include "kmymoneyaccountselector.h"
-#include "mymoneyfile.h"
-#include "mymoneyinstitution.h"
 #include "mymoneyaccount.h"
 #include "mymoneyaccountloan.h"
+#include "mymoneyfile.h"
+#include "mymoneyinstitution.h"
 #include "mymoneypayee.h"
 #include "mymoneyschedule.h"
 #include "mymoneytransactionfilter.h"
+#include "mymoneyutils.h"
 
 class KEditLoanWizardPrivate : public KNewLoanWizardPrivate
 {
@@ -87,9 +88,11 @@ KEditLoanWizard::KEditLoanWizard(const MyMoneyAccount& account, QWidget *parent)
 
     if (d->m_account.openingDate() > QDate::currentDate()) {
         //FIXME: port
-        d->ui->m_effectiveDatePage->ui->m_effectiveDateNoteLabel->setText(QString("\n") + i18n(
-                    "Note: you will not be able to modify this account today, because the opening date \"%1\" is in the future. "
-                    "Please revisit this dialog when the time has come.", QLocale().toString(d->m_account.openingDate())));
+        d->ui->m_effectiveDatePage->ui->m_effectiveDateNoteLabel->setText(
+            QString("\n")
+            + i18n("Note: you will not be able to modify this account today, because the opening date \"%1\" is in the future. "
+                   "Please revisit this dialog when the time has come.",
+                   MyMoneyUtils::formatDate(d->m_account.openingDate(), QLocale::LongFormat)));
     } else {
         d->ui->m_effectiveDatePage->ui->m_effectiveDateNoteLabel->hide();
     }
@@ -171,9 +174,11 @@ void KEditLoanWizard::loadWidgets(const MyMoneyAccount& /* account */)
     MyMoneyMoney basePayment;
     MyMoneyMoney addPayment;
 
-    d->m_transaction = d->m_schedule.transaction();
-
-    foreach (const MyMoneySplit& it_s, d->m_schedule.transaction().splits()) {
+    d->m_feeSplitModel.unload();
+    d->m_additionalFeesTransaction = d->m_schedule.transaction();
+    const auto splits = d->m_additionalFeesTransaction.splits();
+    for (const auto& it_s : splits) {
+        bool isAdditionalCostSplit = true;
         MyMoneyAccount acc = file->account(it_s.accountId());
         // if it's the split that references the source/dest
         // of the money, we check if we borrow or loan money
@@ -203,9 +208,10 @@ void KEditLoanWizard::loadWidgets(const MyMoneyAccount& /* account */)
 
             // remove this split with one that will be replaced
             // later and has a phony id
-            d->m_transaction.removeSplit(it_s);
-            d->m_split.clearId();
-            d->m_transaction.addSplit(d->m_split);
+            d->m_additionalFeesTransaction.removeSplit(it_s);
+            d->m_phonySplit.clearId();
+            d->m_additionalFeesTransaction.addSplit(d->m_phonySplit);
+            isAdditionalCostSplit = false;
         }
 
         if (it_s.action() == MyMoneySplit::actionName(eMyMoney::Split::Action::Interest)) {
@@ -217,10 +223,15 @@ void KEditLoanWizard::loadWidgets(const MyMoneyAccount& /* account */)
         } else {
             // remove the splits which should not show up
             // for additional fees
-            d->m_transaction.removeSplit(it_s);
+            d->m_additionalFeesTransaction.removeSplit(it_s);
+            isAdditionalCostSplit = false;
         }
 
+        if (isAdditionalCostSplit) {
+            d->m_feeSplitModel.appendSplit(it_s);
+        }
     }
+
     if (field("borrowButton").toBool()) {
         basePayment = -basePayment;
         addPayment = -addPayment;
@@ -250,7 +261,7 @@ void KEditLoanWizard::loadWidgets(const MyMoneyAccount& /* account */)
         setField("interestFrequencyUnitEdit", changeFrequencyUnit);
     }
 
-    // keep track, if the loan should be fully repayed
+    // keep track, if the loan should be fully repaid
     d->m_fullyRepayLoan = d->m_account.finalPayment() < basePayment;
 
     d->updateLoanInfo();
@@ -273,7 +284,8 @@ bool KEditLoanWizard::validateCurrentPage()
                                  "KMyMoney will dismiss the changes you have just entered. "
                                  "Do you wish to proceed?", button->text());
 
-            if (KMessageBox::questionYesNo(this, errMsg) == KMessageBox::No) {
+            if (KMessageBox::questionTwoActions(this, errMsg, i18nc("@title:window", "Confirmation"), KMMYesNo::yes(), KMMYesNo::no())
+                == KMessageBox::SecondaryAction) {
                 dontLeavePage = true;
             } else {
                 loadWidgets(d->m_account);
@@ -376,7 +388,7 @@ bool KEditLoanWizard::validateCurrentPage()
 
         // if interest rate and payment amount is given, then force
         // the term to be recalculated. The final payment is adjusted to
-        // 0 if the loan was meant to be fully repayed
+        // 0 if the loan was meant to be fully repaid
         d->ui->m_durationPage->updateTermWidgets(d->m_account.term());
         if (field("interestRateEditValid").toBool()
                 && field("paymentEditValid").toBool()) {
@@ -402,7 +414,7 @@ bool KEditLoanWizard::validateCurrentPage()
             MyMoneyMoney balance = m_account.openingBalance();
 
             QList<MyMoneyTransaction> list;
-            QList<MyMoneyTransaction>::ConstIterator it;
+            QList<MyMoneyTransaction>::const_iterator it;
             MyMoneySplit split;
             MyMoneyTransactionFilter filter(m_account.id());
 
@@ -459,11 +471,11 @@ void KEditLoanWizard::updateEditSummary()
 
     int count = 0;
     QList<MyMoneyTransaction> list;
-    list = MyMoneyFile::instance()->transactionList(filter);
+    MyMoneyFile::instance()->transactionList(list, filter);
 
-    foreach (const MyMoneyTransaction& it, list) {
+    for (const auto& it : qAsConst(list)) {
         int match = 0;
-        foreach (const MyMoneySplit& it_s, it.splits()) {
+        for (const auto& it_s : it.splits()) {
             // we only count those transactions that have an interest
             // and amortization part
             if (it_s.action() == MyMoneySplit::actionName(eMyMoney::Split::Action::Interest))
@@ -475,7 +487,7 @@ void KEditLoanWizard::updateEditSummary()
             ++count;
     }
 
-    setField("affectedPayments", QString().sprintf("%d", count));
+    setField("affectedPayments", QString::number(count));
 }
 
 const MyMoneySchedule KEditLoanWizard::schedule() const
@@ -530,7 +542,7 @@ const MyMoneyAccount KEditLoanWizard::account() const
                                        field("interestFrequencyUnitEdit").toInt());
     }
 
-    return std::move(acc);
+    return acc;
 }
 
 const MyMoneyTransaction KEditLoanWizard::transaction() const
