@@ -35,18 +35,18 @@
 #include "ui_schedulewizardpage.h"
 #include "ui_paymentwizardpage.h"
 
-#include "kmymoneyutils.h"
 #include "kmymoneysettings.h"
-
-#include "mymoneyfinancialcalculator.h"
-#include "mymoneyfile.h"
-#include "mymoneyexception.h"
-#include "mymoneysecurity.h"
+#include "kmymoneyutils.h"
 #include "mymoneyaccountloan.h"
+#include "mymoneyenums.h"
+#include "mymoneyexception.h"
+#include "mymoneyfile.h"
+#include "mymoneyfinancialcalculator.h"
 #include "mymoneyschedule.h"
+#include "mymoneysecurity.h"
 #include "mymoneysplit.h"
 #include "mymoneytransaction.h"
-#include "mymoneyenums.h"
+#include "splitmodel.h"
 
 namespace Ui {
 class KNewLoanWizard;
@@ -98,7 +98,7 @@ public:
         // should be recorded and none have been made so far.
 
         //FIXME: port
-        ui->m_firstPaymentPage->ui->m_firstDueDateEdit->loadDate(QDate(QDate::currentDate().year(), QDate::currentDate().month(), 30));
+        ui->m_firstPaymentPage->ui->m_firstDueDateEdit->setDate(QDate(QDate::currentDate().year(), QDate::currentDate().month(), 30));
 
         // FIXME: we currently only support interest calculation on reception
         m_pages.clearBit(KNewLoanWizard::Page_InterestCalculation);
@@ -116,9 +116,9 @@ public:
 
         // setup a phony transaction for additional fee processing
         m_account = MyMoneyAccount("Phony-ID", MyMoneyAccount());
-        m_split.setAccountId(m_account.id());
-        m_split.setValue(MyMoneyMoney());
-        m_transaction.addSplit(m_split);
+        m_phonySplit.setAccountId(m_account.id());
+        m_phonySplit.setValue(MyMoneyMoney());
+        m_additionalFeesTransaction.addSplit(m_phonySplit);
 
         KMyMoneyUtils::updateWizardButtons(q);
     }
@@ -177,8 +177,7 @@ public:
         if (q->field("durationValueEdit").toInt() == 0) {
             txt = QString("<") + i18n("calculate") + QString(">");
         } else {
-            txt = QString().sprintf("%d ", q->field("durationValueEdit").toInt())
-                  + q->field("durationUnitEdit").toString();
+            txt = QStringLiteral("%1 %2").arg(q->field("durationValueEdit").toInt()).arg(q->field("durationUnitEdit").toString());
         }
         q->setField("duration1", txt);
         q->setField("duration2", txt);
@@ -237,8 +236,7 @@ public:
         int fraction = m_account.fraction(MyMoneyFile::instance()->security(m_account.currencyId()));
         q->setField("loanAmount6", q->field("loanAmountEdit").value<MyMoneyMoney>().formatMoney(fraction));
         q->setField("interestRate6", QString(q->field("interestRateEdit").value<MyMoneyMoney>().formatMoney("", 3) + QString("%")));
-        txt = QString().sprintf("%d ", q->field("durationValueEdit").toInt())
-              + q->field("durationUnitEdit").toString();
+        txt = QStringLiteral("%1 %2").arg(q->field("durationValueEdit").toInt()).arg(q->field("durationUnitEdit").toString());
         q->setField("duration6", txt);
         q->setField("payment6", q->field("paymentEdit").value<MyMoneyMoney>().formatMoney(fraction));
         q->setField("balloon6", q->field("finalPaymentEdit").value<MyMoneyMoney>().formatMoney(fraction));
@@ -373,8 +371,8 @@ public:
                 // a) means, we have paid more than we owed. This can't be
                 // b) means, we paid more than we owed but the last payment is
                 //    less in value than regular payments. That means, that the
-                //    future value is to be treated as  (fully payed back)
-                // c) the loan is not payed back yet
+                //    future value is to be treated as  (fully paid back)
+                // c) the loan is not paid back yet
 
                 if ((q->field("borrowButton").toBool() && val < 0 && qAbs(val) > qAbs(calc.payment()))
                         || (q->field("lendButton").toBool() && val > 0 && qAbs(val) > qAbs(calc.payment()))) {
@@ -482,16 +480,12 @@ public:
             }
 
             // copy the splits from the other costs and update the payment split
-            foreach (const MyMoneySplit& it, m_transaction.splits()) {
-                if (it.accountId() != QStringLiteral("Phony-ID")) {
-                    MyMoneySplit sp = it;
-                    sp.clearId();
-                    t.addSplit(sp);
-                    sPayment.setValue(sPayment.value() - sp.value());
-                    sPayment.setShares(sPayment.value());
-                    t.modifySplit(sPayment);
-                }
-            }
+            m_feeSplitModel.addSplitsToTransaction(t);
+
+            // and subtract the sum from the payment value
+            sPayment.setValue(sPayment.value() - m_feeSplitModel.valueSum());
+            sPayment.setShares(sPayment.value());
+            t.modifySplit(sPayment);
         }
         return t;
     }
@@ -523,12 +517,13 @@ public:
             assetSet.load(ui->m_schedulePage->ui->m_paymentAccountEdit);
     }
 
-    KNewLoanWizard       *q_ptr;
-    Ui::KNewLoanWizard   *ui;
-    MyMoneyAccountLoan    m_account;
-    MyMoneyTransaction    m_transaction;
-    MyMoneySplit          m_split;
-    QBitArray             m_pages;
+    KNewLoanWizard* q_ptr;
+    Ui::KNewLoanWizard* ui;
+    SplitModel m_feeSplitModel;
+    MyMoneyAccountLoan m_account;
+    MyMoneyTransaction m_additionalFeesTransaction;
+    MyMoneySplit m_phonySplit;
+    QBitArray m_pages;
 };
 
 #endif
