@@ -22,7 +22,7 @@ loanattributeswizardpage  -  description
 
 #include "ui_loanattributeswizardpage.h"
 
-#include "knewbankdlg.h"
+#include "knewinstitutiondlg.h"
 #include "mymoneyfile.h"
 #include "mymoneyinstitution.h"
 #include "mymoneyexception.h"
@@ -47,9 +47,9 @@ LoanAttributesWizardPage::LoanAttributesWizardPage(QWidget *parent)
         MyMoneyFile *file = MyMoneyFile::instance();
 
         auto list = file->institutionList();
-        qSort(list);
+        std::sort(list.begin(), list.end());
 
-        Q_FOREACH(const MyMoneyInstitution &institution, list) {
+        for (const auto& institution : qAsConst(list)) {
             ui->m_qcomboboxInstitutions->addItem(institution.name());
         }
     } catch (const MyMoneyException &e) {
@@ -67,7 +67,7 @@ LoanAttributesWizardPage::~LoanAttributesWizardPage()
  */
 bool LoanAttributesWizardPage::isComplete() const
 {
-    return true;
+    return !ui->m_accountNameEdit->text().isEmpty();
 }
 
 void LoanAttributesWizardPage::initializePage()
@@ -87,7 +87,7 @@ void LoanAttributesWizardPage::slotNewClicked()
 {
     MyMoneyInstitution institution;
 
-    QPointer<KNewBankDlg> dlg = new KNewBankDlg(institution, this);
+    QPointer<KNewInstitutionDlg> dlg = new KNewInstitutionDlg(institution, this);
     if (dlg->exec() && dlg != 0) {
         MyMoneyFileTransaction ft;
         try {
