@@ -5,13 +5,14 @@
 
 #include "../kmymoney/mymoney/mymoneyaccount.h"
 
+#include <QDebug>
 #include <QDir>
 #include <QFile>
-#include <QStringList>
 #include <QMap>
+#include <QRegularExpression>
+#include <QStringList>
 #include <QTextStream>
 #include <QXmlStreamReader>
-#include <QDebug>
 
 #include "mymoneyenums.h"
 
@@ -112,16 +113,6 @@ public:
     {
     }
 
-    TemplateAccount(const TemplateAccount &b)
-        : id(b.id),
-          m_type(b.m_type),
-          m_name(b.m_name),
-          code(b.code),
-          parent(b.parent),
-          slotList(b.slotList)
-    {
-    }
-
     void clear()
     {
         id.clear();
@@ -137,22 +128,22 @@ public:
         while (!xml.atEnd()) {
             QXmlStreamReader::TokenType type = xml.readNext();
             if (type == QXmlStreamReader::StartElement) {
-                QStringRef _name = xml.name();
-                if (_name == "slot") {
+                QStringView _name = xml.name();
+                if (_name == QLatin1String("slot")) {
                     type = xml.readNext();
                     if (type == QXmlStreamReader::Characters)
                         type = xml.readNext();
                     if (type == QXmlStreamReader::StartElement) {
-                        QStringRef name = xml.name();
+                        QStringView name = xml.name();
                         QString key, value;
-                        if (name == "key")
+                        if (name == QLatin1String("key"))
                             key = xml.readElementText(QXmlStreamReader::SkipChildElements).trimmed();
                         type = xml.readNext();
                         if (type == QXmlStreamReader::Characters)
                             type = xml.readNext();
                         if (type == QXmlStreamReader::StartElement) {
                             name = xml.name();
-                            if (name == "value")
+                            if (name == QLatin1String("value"))
                                 value = xml.readElementText(QXmlStreamReader::SkipChildElements).trimmed();
                         }
                         if (!key.isEmpty() && !value.isEmpty())
@@ -160,8 +151,8 @@ public:
                     }
                 }
             } else if (type == QXmlStreamReader::EndElement) {
-                QStringRef _name = xml.name();
-                if (_name  == "slots")
+                QStringView _name = xml.name();
+                if (_name == QLatin1String("slots"))
                     return true;
             }
         }
@@ -172,25 +163,25 @@ public:
     {
         while (!xml.atEnd()) {
             xml.readNext();
-            QStringRef _name = xml.name();
-            if (xml.isEndElement() && _name == "account") {
+            QStringView _name = xml.name();
+            if (xml.isEndElement() && _name == QLatin1String("account")) {
                 if (prefixNameWithCode && !code.isEmpty() && !m_name.startsWith(code))
                     m_name = code + ' ' + m_name;
                 return true;
             }
             if (xml.isStartElement())
             {
-                if (_name == "name")
+                if (_name == QLatin1String("name"))
                     m_name = xml.readElementText(QXmlStreamReader::SkipChildElements).trimmed();
-                else if (_name == "id")
+                else if (_name == QLatin1String("id"))
                     id = xml.readElementText(QXmlStreamReader::SkipChildElements).trimmed();
-                else if (_name == "type")
+                else if (_name == QLatin1String("type"))
                     m_type = xml.readElementText(QXmlStreamReader::SkipChildElements).trimmed();
-                else if (_name == "code")
+                else if (_name == QLatin1String("code"))
                     code = xml.readElementText(QXmlStreamReader::SkipChildElements).trimmed();
-                else if (_name == "parent")
+                else if (_name == QLatin1String("parent"))
                     parent = xml.readElementText(QXmlStreamReader::SkipChildElements).trimmed();
-                else if (_name == "slots")
+                else if (_name == QLatin1String("slots"))
                     readSlots(xml);
                 else
                 {
@@ -220,7 +211,7 @@ QDebug operator <<(QDebug out, const TemplateAccount &a)
 QDebug operator <<(QDebug out, const TemplateAccount::PointerList &a)
 {
     out << "TemplateAccount::List(";
-    foreach(const TemplateAccount *account, a)
+    for (const auto* account : a)
         out << *account;
     out << ")";
     return out;
@@ -237,24 +228,21 @@ public:
 
     bool read(QXmlStreamReader &xml)
     {
-        Q_ASSERT(xml.isStartElement() && xml.name() == "gnc-account-example");
+        Q_ASSERT(xml.isStartElement() && xml.name().toString() == "gnc-account-example");
 
         while (xml.readNextStartElement()) {
-            QStringRef name = xml.name();
-            if (xml.name() == "title")
+            QStringView name = xml.name();
+            if (name == QLatin1String("title"))
                 title = xml.readElementText().trimmed();
-            else if (xml.name() == "short-description")
+            else if (name == QLatin1String("short-description"))
                 shortDescription = xml.readElementText().trimmed().replace("  ", " ");
-            else if (xml.name() == "long-description")
+            else if (name == QLatin1String("long-description"))
                 longDescription = xml.readElementText().trimmed().replace("  ", " ");
-            else if (xml.name() == "account")
-            {
+            else if (name == QLatin1String("account")) {
                 TemplateAccount account;
                 if (account.read(xml))
                     accounts.append(account);
-            }
-            else
-            {
+            } else {
                 if (debug)
                     qDebug() << "skipping" << name.toString();
                 xml.skipCurrentElement();
@@ -289,8 +277,7 @@ public:
         else
             list = accountsByParentID(id);
 
-        foreach(TemplateAccount *account, list)
-        {
+        for (const auto account : qAsConst(list)) {
             if (account->m_type != "ROOT")
             {
                 xml.writeStartElement("","account");
@@ -368,7 +355,7 @@ public:
             if (account.parent == parentID)
                 list.append(&account);
         }
-        qSort(list.begin(), list.end(), nameLessThan);
+        std::sort(list.begin(), list.end(), nameLessThan);
         return list;
     }
 
@@ -381,8 +368,7 @@ public:
         else
             list = accountsByParentID(id);
 
-        foreach(TemplateAccount *account, list)
-        {
+        for (const auto account : qAsConst(list)) {
             QString a;
             a.fill(' ', index);
             qDebug() << a << account->m_name << toKMyMoneyAccountType(account->m_type, index);
@@ -401,7 +387,7 @@ QDebug operator <<(QDebug out, const TemplateFile &a)
         << "short description:" << a.shortDescription
         << "long description:" << a.longDescription
         << "accounts:";
-    foreach(const TemplateAccount &account, a.accounts)
+    for (const auto& account : a.accounts)
         out << account;
     out << ")";
     return out;
@@ -417,7 +403,9 @@ public:
     {
         QFile file(filename);
         QTextStream in(&file);
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
         in.setCodec("utf-8");
+#endif
 
         if(!file.open(QIODevice::ReadOnly))
             return false;
@@ -475,12 +463,10 @@ protected:
             m_xml.readNext();
             if (m_xml.isStartElement())
             {
-                if (m_xml.name() == "gnc-account-example")
-                {
+                if (m_xml.name() == QLatin1String("gnc-account-example")) {
                     checkAndUpdateAvailableNamespaces(m_xml);
                     _template.read(m_xml);
-                }
-                else
+                } else
                     m_xml.raiseError(QObject::tr("The file is not an gnucash account template file."));
             }
         }
@@ -491,14 +477,17 @@ protected:
 
     bool writeAsXml(QIODevice *device)
     {
+        static const QRegularExpression regExp(".*/accounts");
         QXmlStreamWriter xml(device);
         xml.setAutoFormatting(true);
         xml.setAutoFormattingIndent(1);
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
         xml.setCodec("utf-8");
+#endif
         xml.writeStartDocument();
 
         QString fileName = inFileName;
-        fileName.replace(QRegExp(".*/accounts"),"accounts");
+        fileName.replace(regExp, "accounts");
         xml.writeComment(QString("\n"
                                  "     Converted using xea2kmt from GnuCash sources\n"
                                  "\n"
@@ -534,14 +523,14 @@ void scanDir(QDir dir, QStringList &files)
     {
         if (debug)
             qDebug() << "Found file: " << fileList[i];
-        files.append(QString("%1/%2").arg(dir.absolutePath()).arg(fileList[i]));
+        files.append(QString("%1/%2").arg(dir.absolutePath(), fileList[i]));
     }
 
     dir.setFilter(QDir::AllDirs | QDir::NoDotAndDotDot | QDir::NoSymLinks);
     QStringList dirList = dir.entryList();
     for (int i=0; i<dirList.size(); ++i)
     {
-        QString newPath = QString("%1/%2").arg(dir.absolutePath()).arg(dirList.at(i));
+        QString newPath = QString("%1/%2").arg(dir.absolutePath(), dirList.at(i));
         scanDir(QDir(newPath), files);
     }
 }
@@ -562,15 +551,14 @@ int convertFileStructure(const QString &indir, const QString &outdir)
     QStringList files;
     scanDir(d, files);
 
-    QString inPath = d.absolutePath();
-    QDir outDir(outdir);
-    QString outPath = outDir.absolutePath();
-    QStringList mapKeys = dirNameMap.keys();
+    const QString inPath = d.absolutePath();
+    const QDir outDir(outdir);
+    const QString outPath = outDir.absolutePath();
+    const QStringList mapKeys = dirNameMap.keys();
     int result = 0;
 
     // process templates
-    foreach (const QString &file, files)
-    {
+    for (const auto& file : qAsConst(files)) {
         if (debug || verbose)
             qDebug() << "processing" << file;
 
@@ -580,14 +568,13 @@ int convertFileStructure(const QString &indir, const QString &outdir)
         outFileName.replace(inPath, outPath);
         outFileName.remove("acctchrt_");
         outFileName.replace(".gnucash-xea", ".kmt");
-        foreach(const QString &key, mapKeys)
-        {
+        for (const auto& key : mapKeys) {
             if (outFileName.contains('/' + key + '/'))
                 outFileName = outFileName.replace('/' + key + '/', '/' + dirNameMap[key] + '/');
         }
         fi.setFile(outFileName);
 
-        d = fi.absolutePath();
+        d.setPath(fi.absolutePath());
         if (!d.exists())
         {
             if  (debug)
